library ACR;

uses
     PlugAPI,
     PlugSIDE,

     TossAPI,

     Wizard,

     jtMHLcmn,

     annAPI,

     acrAPI,
     acrCmn,
     acrACR,
     acrAnn;

const
 AutoCreatorName                                = 'AutoCreator';
 AutoCreatorVersion                             = 1;

 AutoCreatorPlugin              : Pointer       = nil;

{ CheckLinkValidity }

procedure CheckLinkValidity(var Stuff);
 var
  Block: packed record
   Link, Linker: Pointer;
  end absolute Stuff;
  st: Pointer;
 begin
  st:=jt.link.st(Block.Link);

  if LongBool(jt.st.Get(st, acrAutoCreate)) then
   if jt.st.Get(st, acrAutoCreateFormat) = nil then
    jt.linker.Error(Block.Linker, 'AUTOCREATEFORMAT is not defined');
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, AutoCreatorName, AutoCreatorVersion);

  AutoCreatorPlugin:=Plugin;

  PluginSetup:=True;

  jt.linker.RegisterLinkSectionCallback(AutoCreatorPlugin);

  CreateLongFNamesMap:=New(PMap, Init($10, $10));
 end;

{ PluginStartup }

procedure PluginStartup; export;
 begin
  RegisterTosserCallback(AutoCreatorPlugin, taAutoCreateLast);

  InitAnnouncerAPI;

  RegisterAnnounceType(AutoCreatorPlugin, 'AreaCreated');
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
  Dispose(CreateLongFNamesMap, Done);

  DisposeTosserCallbacks;
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 var
  Block: PacrPrepareAreaFormatBlock absolute Param;
 begin
  if Command = tcAutoCreateLast then AutoCreateLast(PTosserAutoCreateBlock(Param)^) else
  if Command = jtpcLinkSectionEnd then CheckLinkValidity(Pointer(Param)^) else
  if Command = AnnouncerProcessFirst then ProcessFirstAnnounce(PProcessAnnounceFirstBlock(Param)^) else
  if Command = AnnouncerProcess then ProcessAnnounce(PProcessAnnounceBlock(Param)^) else
  if Command = AnnouncerProcessLast then ProcessLastAnnounce(PProcessAnnounceLastBlock(Param)^) else
  if Command = acrcPrepareAreaFormat then PrepareAreaFormat(Block^.Format^, Block^.AreaName^, Block^.LinkAddress);
 end;

{ PluginHandleConfigSwitch }

procedure PluginHandleConfigSwitch(const Keyword: String; Value: String; const Linker: Pointer); export;
 begin
  if Keyword = 'CREATELONGFNAMES' then CreateLongFNames:=jt.linker.ParseBoolean(Linker, Value) else
  if Keyword = 'CREATELONGFNAMESCASE' then
   begin
    Value:=jt.linker.ParseWord(Linker, Value);

    TrimEx(Value);

    StUpcaseEx(Value);

    if Value = 'LO' then CreateLongFNamesCase:=ccLo else
    if Value = 'UP' then CreateLongFNamesCase:=ccUp else
     jt.linker.Error(Linker, 'LO or UP expected');
   end else
  if Keyword = 'CREATELONGFNAMESSET' then CreateLongFNamesSet:=StUpcase(jt.linker.ParseString(Linker, Value)) else
  if Keyword = 'CREATELONGFNAMESMAP' then CreateLongFNamesMap^.AddMapItem(jt.linker.ParseStringToken(Linker, Value),
   jt.linker.ParseStringToken(Linker, Value))
  else
   jt.linker.Error(Linker, 'syntax error');
 end;

{ PluginHandleLinkSwitch }

function PluginHandleLinkSwitch(const Link: Pointer; const Keyword: String; Value: String; const Linker: Pointer): Boolean;
 export;
 var
  st: Pointer;
 procedure ParseAutoCreate;
  begin
   jt.st.Add(st, acrAutoCreate, Pointer(jt.linker.ParseBoolean(Linker, Value)), nil);
  end;
 procedure ParseAutoCreateFormat;
  var
   Format: PAreaFormat;
  begin
   Format:=jt.NewAreaFormat;

   Format^.Style:=asEchomail;

   jt.linker.ParseAreaFormat(Linker, ExtractWord(1, Value, [' ']), Format);

   TrimEx(Value);

   Value:=GetAllAfterSpace(Value, 1);

   jt.linker.ParseAreaPath(Linker, Value, Format);

   jt.st.Add(st, acrAutoCreateFormat, Format, jt.svc.stAreaFormat);
  end;
 procedure ParseAutoCreateMask;
  var
   List: Pointer;
  begin
   List:=jt.st.Get(st, acrAutoCreateMasks);

   if List = nil then
    begin
     List:=jt.list.ex.Create(lefUpcase);

     jt.st.Add(st, acrAutoCreateMasks, List, jt.svc.stListEx);
    end;

   if not jt.list.ex.Add(List, jt.linker.ParseString(Linker, Value)) then
    jt.linker.Error(Linker, jt.list.ex.GetError(List));
  end;
 procedure ParseAutoCreateSubscribe;
  var
   AutoCreateSubscribes: PAutoCreateSubscribes;
   AutoCreateSubscribe: PAutoCreateSubscribe;
   Address, AAddress: TAddress;
   S: String;
  begin
   S:=jt.linker.ParseWord(Linker, Value);

   ClearAddress(Address);

   if not StrToAddress(S, Address) then
    jt.linker.Error(Linker, 'unable to parse address "' + S + '"');

   jt.link.GetAddress(Link, AAddress);

   if AddressEqu(Address, AAddress) then
    jt.linker.Error(Linker, 'ACR can do this stuff by itself ;))');

   jt.linker.CreateLinkForward(Address, nil, jt.linker.FName(Linker), jt.linker.Line(Linker));

   AutoCreateSubscribes:=jt.st.Get(st, acrAutoCreateSubscribes);

   if AutoCreateSubscribes = nil then
    begin
     AutoCreateSubscribes:=New(PAutoCreateSubscribes, Init($10, $10));

     jt.st.Add(st, acrAutoCreateSubscribes, AutoCreateSubscribes, stAutoCreateSubscribes);
    end;

   AutoCreateSubscribe:=AutoCreateSubscribes^.Seek(Address);

   if not jt.list.ex.Add(AutoCreateSubscribe^.List, jt.linker.ParseString(Linker, Value)) then
    jt.linker.Error(Linker, jt.list.ex.GetError(AutoCreateSubscribe^.List));
  end;
 begin
  st:=jt.link.st(Link);

  PluginHandleLinkSwitch:=True;

  if Keyword = 'AUTOCREATE' then ParseAutoCreate else
  if Keyword = 'AUTOCREATEFORMAT' then ParseAutoCreateFormat else
  if Keyword = 'AUTOCREATEMASK' then ParseAutoCreateMask else
  if Keyword = 'AUTOCREATESUBSCRIBE' then ParseAutoCreateSubscribe else
   PluginHandleLinkSwitch:=False;
 end;

{ PluginExportLinkSwitches }

procedure PluginExportLinkSwitches(const Link, Linker: Pointer); export;
 var
  AutoCreateSubscribes: PAutoCreateSubscribes;
  Format: TAreaFormat;
  List, st: Pointer;
  K: Longint;
 procedure ExportAutoCreateSubscribe(const Subscribe: PAutoCreateSubscribe); far;
  var
   K: Longint;
  begin
   for K:=0 to jt.list.ex.Count(Subscribe^.List) - 1 do
    jt.linker.ExportLine(Linker, '   AutoCreateSubscribe ' + AddressToStrEx(Subscribe^.Address) + ' ' +
     jt.linker.UnparseString(Linker, jt.list.ex.At(Subscribe^.List, K)^));
  end;
 begin
  st:=jt.link.st(Link);

  if LongBool(jt.st.Get(st, acrAutoCreate)) then
   begin
    jt.linker.ExportLine(Linker, ' AutoCreate Yes');

    Format:=PAreaFormat(jt.st.Get(st, acrAutoCreateFormat))^;

    jt.linker.ExportLine(Linker, '   AutoCreateFormat ' +
         jt.linker.UnparseAreaFormat(Linker, @Format) + ' ' + jt.linker.UnparseAreaPath(Linker, @Format));

    List:=jt.st.Get(st, acrAutoCreateMasks);

    if List <> nil then
     for K:=0 to jt.list.ex.Count(List) - 1 do
      jt.linker.ExportLine(Linker, '   AutoCreateMask ' + jt.linker.UnparseString(Linker, jt.list.ex.At(List, K)^));

    AutoCreateSubscribes:=jt.st.Get(st, acrAutoCreateSubscribes);

    if AutoCreateSubscribes <> nil then
     AutoCreateSubscribes^.ForEach(@ExportAutoCreateSubscribe);
   end;
 end;

{ PluginHandleCmdlineSwitch }

function PluginHandleCmdlineSwitch(Switch: String): Boolean; export;
 begin
  StUpcaseEx(Switch);

  PluginHandleCmdlineSwitch:=True;

  if Switch = 'DEBUGACR' then Debugger.Enabled:=True else
   PluginHandleCmdlineSwitch:=False;
 end;

{ PluginExportLinksHeader }

procedure PluginExportLinksHeader(const Linker: Pointer); export;
 begin
  jt.linker.ExportLine(Linker, '; AutoCreator switches:');
  jt.linker.ExportLine(Linker, ';   AutoCreate Yes|No');
  jt.linker.ExportLine(Linker, ';   AutoCreateFormat Squish "path"');
  jt.linker.ExportLine(Linker, ';   AutoCreateMask "regexp"');
  jt.linker.ExportLine(Linker, ';   AutoCreateSubscribe address "regexp"');
  jt.linker.ExportLine(Linker, ';');
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleConfigSwitch,
 PluginHandleLinkSwitch,
 PluginExportLinkSwitches,
 PluginHandleCmdlineSwitch,
 PluginExportLinksHeader;

begin
 Assign(Output, '');
 Rewrite(Output);
end.