unit acrCmn;

interface
uses
     Objects,

     Wizard,

     jtMHLcmn,

     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     acrAPI;

type
 PMapItem = ^TMapItem;
 TMapItem = object(TObject)
  Source, Destination: PString;

  constructor Init(const ASource, ADestination: String);
  procedure DoMapping(var S: String);
  destructor Done; virtual;
 end;

 PMap = ^TMap;
 TMap = object(TCollection)
  procedure AddMapItem(const Source, Destination: String);
  procedure DoMapping(var S: String);
 end;

type
 PAutoCreateSubscribe = ^TAutoCreateSubscribe;
 TAutoCreateSubscribe = object(TObject)
  Address: TAddress;
  List: Pointer;

  constructor Init(const AAddress: TAddress);
  constructor Clone(const Source: PAutoCreateSubscribe);
  destructor Done; virtual;
 end;

 PAutoCreateSubscribes = ^TAutoCreateSubscribes;
 TAutoCreateSubscribes = object(TCollection)
  constructor Clone(const Source: PAutoCreateSubscribes);
  function Seek(const Address: TAddress): PAutoCreateSubscribe;
 end;

const
 CreateLongFNames               : Boolean               = False;
 CreateLongFNamesCase           : (ccLo, ccUp)          = ccLo;
 CreateLongFNamesSet            : String                = '.ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
 CreateLongFNamesMap            : PMap                  = nil;

 acrAutoCreate                  = 'ACR.AutoCreate';
 acrAutoCreateFormat            = 'ACR.AutoCreateFormat';
 acrAutoCreateMasks             = 'ACR.AutoCreateMasks';
 acrAutoCreateSubscribes        = 'ACR.AutoCreateSubscribes';

 acrLetter                      = 'A';

 Debugger                       : TDebugger = (Enabled: False; Prefix: 'dbgACR');

function stAutoCreateSubscribes(const Action: TStorageAction; const Data: Pointer): Pointer; export;

procedure PrepareAreaFormat(var Format: TAreaFormat; const AreaName: String; const LinkAddress: PAddress);

implementation

{ TMapItem }

constructor TMapItem.Init(const ASource, ADestination: String);
 begin
  inherited Init;

  Source:=NewStr(ASource);
  Destination:=NewStr(ADestination);
 end;

procedure TMapItem.DoMapping(var S: String);
 begin
  ReplaceEx(S, GetPString(Source), GetPString(Destination));
 end;

destructor TMapItem.Done;
 begin
  DisposeStr(Source);
  DisposeStr(Destination);

  inherited Done;
 end;

{ TMap }

procedure TMap.AddMapItem(const Source, Destination: String);
 begin
  if Source <> '' then
   Insert(New(PMapItem, Init(Source, Destination)));
 end;

procedure TMap.DoMapping(var S: String);
 procedure PerformMapping(const Item: PMapItem); far;
  begin
   Item^.DoMapping(S);
  end;
 begin
  ForEach(@PerformMapping);
 end;

{ TAutoCreateSubscribe }

constructor TAutoCreateSubscribe.Init(const AAddress: TAddress);
 begin
  inherited Init;

  Address:=AAddress;

  List:=jt.list.ex.Create(lefUpcase);
 end;

constructor TAutoCreateSubscribe.Clone(const Source: PAutoCreateSubscribe);
 begin
  inherited Init;

  Address:=Source^.Address;

  List:=jt.list.ex.Clone(Source^.List);
 end;

destructor TAutoCreateSubscribe.Done;
 begin
  jt.list.ex.Destroy(List);

  inherited Done;
 end;

{ TAutoCreateSubscribes }

constructor TAutoCreateSubscribes.Clone(const Source: PAutoCreateSubscribes);
 procedure DoClone(const ACS: PAutoCreateSubscribe); far;
  begin
   Insert(New(PAutoCreateSubscribe, Clone(ACS)));
  end;
 begin
  inherited Init(Source^.Count, $10);

  Source^.ForEach(@DoClone);
 end;

function TAutoCreateSubscribes.Seek(const Address: TAddress): PAutoCreateSubscribe;
 function DoSeek(const AutoCreateSubscribe: PAutoCreateSubscribe): Boolean; far;
  begin
   DoSeek:=AddressCompare(AutoCreateSubscribe^.Address, Address) = 0;
  end;
 var
  AutoCreateSubscribe: PAutoCreateSubscribe;
 begin
  AutoCreateSubscribe:=FirstThat(@DoSeek);

  if AutoCreateSubscribe = nil then
   begin
    AutoCreateSubscribe:=New(PAutoCreateSubscribe, Init(Address));

    Insert(AutoCreateSubscribe);
   end;

  Seek:=AutoCreateSubscribe;
 end;

{ stAutoCreateSubscribes }

function stAutoCreateSubscribes(const Action: TStorageAction; const Data: Pointer): Pointer;
 var
  ACS: PAutoCreateSubscribes absolute Data;
 begin
  case Action of
   saDestroy: Dispose(ACS, Done);
   saClone: stAutoCreateSubscribes:=New(PAutoCreateSubscribes, Clone(Data));
   saInfo: stAutoCreateSubscribes:=jt.NewStr('auto create subscribes');
  end;
 end;

{ CreateBaseNameSet }

function CreateBaseNameSet(const AreaName: String): String;
 var
  Name: String;
  K: Byte;
 begin
  Name:='';

  for K:=1 to Length(AreaName) do
   if Pos(AreaName[K], CreateLongFNamesSet) > 0 then
    Name:=Name + AreaName[K];

  case CreateLongFNamesCase of
   ccLo: StLocaseEx(Name);
   ccUp: StUpcaseEx(Name);
  end;

  CreateBaseNameSet:=Name;
 end;

{ CreateBaseName }

function CreateBaseName(const AreaName: String): String;
{$IFDEF DPMI}
 begin
  CreateBaseName:=GenerateMSGID;
 end;
{$ELSE}
 var
  Name: String;
 begin
  if not CreateLongFNames then
   Name:=GenerateMSGID
  else
   begin
    Name:=CreateBaseNameSet(AreaName);

    CreateLongFNamesMap^.DoMapping(Name);

    if Name = '' then
     Name:=GenerateMSGID;
   end;

  case CreateLongFNamesCase of
   ccLo: StLocaseEx(Name);
   ccUp: StUpcaseEx(Name);
  end;

  CreateBaseName:=Name;
 end;
{$ENDIF}

{ PrepareAreaFormat }

procedure PrepareAreaFormat(var Format: TAreaFormat; const AreaName: String; const LinkAddress: PAddress);
 var
  Suffix, Path: String;
  K: Longint;
 begin
  Path:=Format.Path;

  {$IFDEF VIRTUALPASCAL}
  if CreateLongFNames then
   begin
    K:=0;

    repeat
     if K = 0 then
      Suffix:=''
     else
      Suffix:=Long2Str(K);

     Format.Path:=Path;

     if Pos('@AREA', StUpcase(Format.Path)) > 0 then
      ReplaceEx(Format.Path, '@area', CreateBaseName(AreaName) + Suffix)
     else
      Format.Path:=AddBackSlash(Format.Path) + CreateBaseName(AreaName) + Suffix;

     if Pos('@OAREA', StUpcase(Format.Path)) > 0 then
      ReplaceEx(Format.Path, '@oarea', CreateBaseNameSet(AreaName));

     Inc(K);

     if LinkAddress <> nil then
      with LinkAddress^ do
       begin
        ReplaceEx(Format.Path, '@zone', Long2Str(Zone));
        ReplaceEx(Format.Path, '@net', Long2Str(Net));
        ReplaceEx(Format.Path, '@node', Long2Str(Node));
        ReplaceEx(Format.Path, '@point', Long2Str(Point));
       end
     else
      with NullAddress do
       begin
        ReplaceEx(Format.Path, '@zone', Long2Str(Zone));
        ReplaceEx(Format.Path, '@net', Long2Str(Net));
        ReplaceEx(Format.Path, '@node', Long2Str(Node));
        ReplaceEx(Format.Path, '@point', Long2Str(Point));
       end;

     Format.Path:=RemoveBackSlash(Format.Path);
    until not jt.area.IsBaseExists(Format);
   end
  else
  {$ENDIF}
  repeat
   Format.Path:=AddBackSlash(Path) + CreateBaseName(AreaName)
  until not jt.area.IsBaseExists(Format);
 end;

end.
