library AE;

{
  dedicated to Andy Elkin, 2:5030/15@fidonet
}

uses
     PlugAPI,
     PlugSIDE,

     Objects,

     Wizard,

     jtMHLcmn,

     aeCmn,
     aeBBS,
     aeSquish,
     aeGolded,
     aeUni;

const
 AreaExporterName                               = 'AreaExporter';
 AreaExporterVersion                            = 3;

 AreaExporterPlugin             : Pointer       = nil;

 AreaExporterPerform                            = $1000;

 ParsingExporter                : PExporter     = nil;

{ aeGroupValueParser }

procedure aeGroupValueParser(const Keyword, Linker, st: Pointer; Value: String); export;
 begin
  if Length(Value) <> 1 then
   jt.linker.Error(Linker, 'aeGroup value must be single char');

  StUpcaseEx(Value);

  if Value[1] <= ' ' then
   jt.linker.Error(Linker, 'invalid aeGroup value');

  jt.st.Add(st, 'Group', Pointer(Value[1]), nil);
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, AreaExporterName, AreaExporterVersion);

  AreaExporterPlugin:=Plugin;

  PluginSetup:=True;

  Exporters:=New(PCollection, Init($3, $10));

  aeDefaultDesc:=jt.rests.Add('aeDefaultDesc', rsValue);
  aeGroup:=jt.rests.AddValueParser('aeGroup', rsValue or rsStorage, aeGroupValueParser);
 end;

{ PluginStartup }

procedure PluginStartup; export;
 begin
  jt.EnqueueCommand(AreaExporterPlugin, AreaExporterPerform, 0, cwExportAreasFull);
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
  Dispose(Exporters, Done);
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 procedure Perform(const Exporter: PExporter); far;
  begin
   Exporter^.Perform;
  end;
 begin
  if Command = AreaExporterPerform then
   begin
    jt.PushTitle;

    Exporters^.ForEach(@Perform);

    jt.ClearStatus;

    jt.PopTitle;
   end else
  if Command = jtpcPluginSectionEnd then
   if ParsingExporter <> nil then
    ParsingExporter^.Valid(Pointer(Param))
   else
  else
 end;

{ PluginHandleConfigSwitch }

procedure PluginHandleConfigSwitch(const Keyword: String; Value: String; const Linker: Pointer); export;
 procedure ParseExportTo;
  begin
   if ParsingExporter <> nil then
    ParsingExporter^.Valid(Linker);

   Value:=jt.linker.ParseWord(Linker, Value);

   StUpcaseEx(Value);

   if Value = 'AREASBBS' then ParsingExporter:=New(PAreasBbsExporter, Init) else
   if Value = 'SQUISH' then ParsingExporter:=New(PSquishExporter, Init) else
   if Value = 'GOLDED' then ParsingExporter:=New(PGoldedExporter, Init) else
   if Value = 'UNIFORM' then ParsingExporter:=New(PUniformExporter, Init) else
    jt.linker.Error(Linker, 'unknown exporter "' + Value + '"');

   Exporters^.Insert(ParsingExporter);
  end;
 begin
  if Keyword = 'EXPORTTO' then ParseExportTo else
  if ParsingExporter = nil then jt.linker.Error(Linker, 'EXPORTTO expected') else
   ParsingExporter^.Parse(Keyword, Value, Linker);
 end;

{ PluginHandleAreaSwitch }

function PluginHandleAreaSwitch(const Area: Pointer; const Keyword: String; Value: String; const Linker: Pointer): Boolean;
  export;
 begin
  if Keyword = 'AEGROUP' then
   begin
    Value:=jt.linker.ParseStringToken(Linker, Value);

    if Length(Value) <> 1 then
     jt.linker.Error(Linker, 'aeGroup value must be single char');

    StUpcaseEx(Value);

    if Value[1] <= ' ' then
     jt.linker.Error(Linker, 'invalid aeGroup value');

    jt.st.Add(jt.area.st(Area), 'aeGroup', Pointer(Value[1]), nil);

    PluginHandleAreaSwitch:=True;
   end
  else
   PluginHandleAreaSwitch:=False;
 end;

{ PluginExportAreaSwitches }

procedure PluginExportAreaSwitches(const Area, Linker: Pointer); export;
 var
  Ch: Char;
 begin
  Ch:=Chr(Longint(jt.st.Get(jt.area.st(Area), 'aeGroup')));

  if Ch <> #0 then
   jt.linker.ExportLine(Linker, ' aeGroup ' + jt.linker.UnparseString(Linker, Ch));
 end;

{ PluginExportAreasHeader }

procedure PluginExportAreasHeader(const Linker: Pointer); export;
 begin
  jt.linker.ExportLine(Linker, '; AreaExporter switches:');
  jt.linker.ExportLine(Linker, ';  aeGroup "g"');
  jt.linker.ExportLine(Linker, ';');
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleConfigSwitch,
 PluginHandleAreaSwitch,
 PluginExportAreaSwitches,
 PluginExportAreasHeader;

begin
 Assign(Output, '');
 Rewrite(Output);
end.