unit aeCmn;

interface
uses
     Objects,

     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     Wizard,

     jtMHLcmn;

const
 efNone                 = $0000;
 efAppend               = $0001;
 efNoHeader             = $0002;

type
 PExporter = ^TExporter;
 TExporter = object(TObject)
  reName, reUplink: Pointer;
  FileName: PString;
  Prefix: PString;
  Flags, Styles: Longint;
  Formats: PNumbers;
  Link: Text;

  constructor Init;
  procedure Valid(const Linker: Pointer); virtual;
  procedure Parse(const Keyword: String; Value: String; const Linker: Pointer); virtual;
  function ParseFlag(const Flag: String): Boolean; virtual;
 public
  function GetAreaName(const Area: Pointer): String; virtual;
  function GetAreaDesc(const Area: Pointer): String; virtual;
  function GetAreaGroup(const Area: Pointer): String; virtual;
 public
  procedure Perform; virtual;
  function Applicable(const Area: Pointer; const Format: TAreaFormat): Boolean; virtual;
  procedure PerformArea(const Area: Pointer; const Format: TAreaFormat); virtual;
  function Name: String; virtual;
  destructor Done; virtual;
 end;

const
 Exporters                      : PCollection   = nil;

 aeDefaultDesc                  : Pointer       = nil;
 aeGroup                        : Pointer       = nil;

implementation

{ TExporter }

constructor TExporter.Init;
 begin
  inherited Init;

  Formats:=New(PNumbers, Init($4, $4));
 end;

procedure TExporter.Valid(const Linker: Pointer);
 begin
  if reName = nil then jt.CompileRegExp('.*', reName, Linker, True);
  if reUplink = nil then jt.CompileRegExp('.*', reUplink, Linker, True);
  if Styles = 0 then Styles:=asNetmail or asLocal or asEchomail;
  if Formats^.Count = 0 then Formats^.SubmitAll:=True;
 end;

procedure TExporter.Parse(const Keyword: String; Value: String; const Linker: Pointer);
 var
  S: String;
 function GetToken: Boolean;
  begin
   if Value = '' then
    GetToken:=False
   else
    begin
     S:=jt.linker.ParseWord(Linker, Value);

     StUpcaseEx(S);

     GetToken:=S <> '';
    end;
  end;
 procedure ParseStyles;
  begin
   while GetToken do
    if S = 'NETMAIL' then Styles:=Styles or asNetmail else
    if S = 'ECHOMAIL' then Styles:=Styles or asEchomail else
    if S = 'LOCAL' then Styles:=Styles or asLocal else
     jt.linker.Error(Linker, 'unknown style "' + S + '"');
  end;
 procedure ParseFormats;
  var
   Index, id: Longint;
   FullName, ShortName: String;
   Found: Boolean;
  begin
   while GetToken do
    begin
     Index:=0;

     Found:=False;

     while jt.core.AtFormat(Index, id, FullName, ShortName) do
      begin
       StUpcaseEx(FullName);
       StUpcaseEx(ShortName);

       if (S = FullName) or (S = ShortName) then
        begin
         Formats^.Add(id);

         Found:=True;
        end;

       Inc(Index);
      end;

     if not Found then
      jt.linker.Error(Linker, 'unknown format "' + S + '"');
    end;
  end;
 procedure ParseFlags;
  begin
   while GetToken do
    if S = 'APPEND' then Flags:=Flags or efAppend else
    if S = 'NOHEADER' then Flags:=Flags or efNoHeader else
    if not ParseFlag(S) then
     jt.linker.Error(Linker, 'unknown format "' + S + '"');
  end;
 begin
  if Keyword = 'MASK' then
   if reName = nil then
    jt.CompileRegExp(jt.linker.ParseStringToken(Linker, Value), reName, Linker, True)
   else
    jt.linker.Error(Linker, 'redefinition of MASK') else
  if Keyword = 'UPLINK' then
   if reUplink = nil then
    jt.CompileRegExp(jt.linker.ParseStringToken(Linker, Value), reUplink, Linker, True)
   else
    jt.linker.Error(Linker, 'redefinition of UPLINK') else
  if Keyword = 'STYLES' then ParseStyles else
  if Keyword = 'FORMATS' then ParseFormats else
  if Keyword = 'FLAGS' then ParseFlags else
  if Keyword = 'FILENAME' then
   if FileName = nil then
    FileName:=NewStr(jt.linker.ParseStringToken(Linker, Value))
   else
    jt.linker.Error(Linker, 'redefinition of FILENAME') else
  if Keyword = 'PREFIX' then
   if Prefix = nil then
    Prefix:=NewStr(jt.linker.ParseStringToken(Linker, Value))
   else
    jt.linker.Error(Linker, 'redefinition of PREFIX')
  else
   jt.linker.Error(Linker, 'syntax error');
 end;

function TExporter.ParseFlag(const Flag: String): Boolean;
 begin
  ParseFlag:=False;
 end;

{ public methods }

function TExporter.GetAreaName(const Area: Pointer): String;
 var
  S: String;
 begin
  S:=jt.area.GetName(Area)^;

  if Prefix <> nil then
   S:=Prefix^ + S;

  GetAreaName:=S;
 end;

function TExporter.GetAreaDesc(const Area: Pointer): String;
 var
  Desc: String;
  Uplink: TAddress;
 begin
  GetPStringEx(jt.area.GetDesc(Area), Desc);

  if Desc = '' then
   jt.rests.GetValue(aeDefaultDesc, nil, jt.area.GetName(Area), Desc);

  ReplaceChar(Desc, '"', '''');

  if Pos('@', Desc) > 0 then
   begin
    jt.area.GetUplink(Area, Uplink);

    ReplaceEx(Desc, '@uplink', AddressToStrEx(Uplink));
   end;

  GetAreaDesc:=Desc;
 end;

function TExporter.GetAreaGroup(const Area: Pointer): String;
 var
  st: Pointer;
  Ch: Char;
 begin
  Ch:=Chr(Longint(jt.st.Get(jt.area.st(Area), 'aeGroup')));

  if Ch = #0 then
   begin
    jt.rests.Getst(aeGroup, nil, jt.area.GetName(Area), st);

    if st = nil then
     GetAreaGroup:='A'
    else
     GetAreaGroup:=Char(jt.st.Get(st, 'Group'));
   end
  else
   GetAreaGroup:=Ch;
 end;

{ public methods }

procedure TExporter.Perform;
 const
  BufferSize = {$IFDEF DPMI} 16 * 1024 {$ELSE} 64 * 1024 {$ENDIF};
 var
  Buffer: Pointer;
  Format: TAreaFormat;
  Area: Pointer;
  K: Longint;
 begin
  if IOResult <> 0 then;

  jt.CreateDirectory(JustPathName(FileName^));

  Assign(Link, FileName^);

  GetMem(Buffer, BufferSize);

  SetTextBuf(Link, Buffer^, BufferSize);

  jt.SetStatus('exporting ' + FileName^ + '...');

  jt.SetTitle(Name + ' - ' + FileName^ + ' (exporting)');

  if Flags and efAppend <> 0 then
   Append(Link);

  if (IOResult <> 0) or (Flags and efAppend = 0) then
   Rewrite(Link);

  if InOutRes = 0 then
   begin
    if Flags and efNoHeader = 0 then
     begin
      WriteLn(Link, '; exported file, created at ', GetBinkDateTime);
      WriteLn(Link);
     end;

    for K:=0 to jt.area.Count - 1 do
     begin
      Area:=jt.area.At(K);

      jt.area.GetFormat(Area, Format);

      if Applicable(Area, Format) then
       PerformArea(Area, Format);
     end;

    Close(Link);
   end
  else
   jt.Report('Unable to create ' + FileName^ + ', rc=' + Long2Str(IOResult), True);

  FreeMem(Buffer, BufferSize);
 end;

function TExporter.Applicable(const Area: Pointer; const Format: TAreaFormat): Boolean;
 var
  Uplink: TAddress;
 begin
  jt.area.GetUplink(Area, Uplink);

  Applicable:=jt.AreaMatchRegExp(reName, jt.area.GetName(Area)) and
              jt.AddressMatchRegExp(reUplink, @Uplink) and
              (Styles and Format.Style <> 0) and
              (Formats^.IsIn(Format.Format));
 end;

procedure TExporter.PerformArea(const Area: Pointer; const Format: TAreaFormat);
 begin
  Abstract;
 end;

function TExporter.Name: String;
 begin
  Abstract;
 end;

destructor TExporter.Done;
 begin
  DisposeStr(FileName);
  DisposeStr(Prefix);

  jt.DestroyRegExp(reName);
  jt.DestroyRegExp(reUplink);

  Dispose(Formats, Done);

  inherited Done;
 end;

end.