unit afxAPI;

interface
uses
     PlugAPI,
     PlugSIDE,

     jtMHLcmn,

     Objects;

type
 TAreafixCommandHandler = procedure(const Command: String; Value: String);

 PAreafixRegisterCommandHandlerBlock = ^TAreafixRegisterCommandHandlerBlock;
 TAreafixRegisterCommandHandlerBlock = packed record
  Command: PString;
  Handler: TAreafixCommandHandler;
 end;

 PAreafixCreateReplyBlock = ^TAreafixCreateReplyBlock;
 TAreafixCreateReplyBlock = packed record
  Name: PString;
  Created: Boolean;
  Reply: Pointer;
 end;

 PAreafixGetRequestInfoBlock = ^TAreafixGetRequestInfoBlock;
 TAreafixGetRequestInfoBlock = packed record
  ReplyToName: String;
  ReplyToAddress: TAddress;
  OurName: String;
  OurAddress: TAddress;
  Link: Pointer;
  Area: Pointer;
  No: Longint;
 end;

 TAreafixAction = (aaSubscribeBefore,
                   aaSubscribeResumed,
                   aaSubscribeSubscribed,
                   aaSubscribeAfter,

                   aaUnsubscribeBefore,
                   aaUnsubscribeUnsubscribed,
                   aaUnsubscribeAfter);

 { TRegisterAreafixCallbackBlock ᯮ afxAPI  Areafix p
   pp樨  }

 PRegisterAreafixCallbackBlock = ^TRegisterAreafixCallbackBlock;
 TRegisterAreafixCallbackBlock = packed record
  Plugin: Pointer;
  Action: TAreafixAction;
  Next: PRegisterAreafixCallbackBlock;
 end;

 PAreafixSubscribeBlock = ^TAreafixSubscribeBlock;
 TAreafixSubscribeBlock = packed record
  Mask: PString;
  Changes: Longint;
  Reply: Pointer;
  Area: Pointer; { ᯮ ⮫쪮 p Resumed/Subscribed }
 end;

 PAreafixUnsubscribeBlock = ^TAreafixUnsubscribeBlock;
 TAreafixUnsubscribeBlock = packed record
  Mask: PString;
  Changes: Longint;
  Reply: Pointer;
  Area: Pointer; { ᯮ ⮫쪮 p Unsubscribed }
 end;

 PafxIsAreaBlock = ^TafxIsAreaBlock;
 TafxIsAreaBlock = packed record
  Area: Pointer;
  Yes: Boolean;
 end;

 PafxIsLinkAreaBlock = ^TafxIsLinkAreaBlock;
 TafxIsLinkAreaBlock = packed record
  Link: Pointer;
  Area: Pointer;
  Yes: Boolean;
 end;

 PafxGetAreaDescriptionBlock = ^TafxGetAreaDescriptionBlock;
 TafxGetAreaDescriptionBlock = packed record
  Area: Pointer;
  Description: String;
 end;

const
 AreafixActionNames: array[TAreafixAction] of PChar = (
                   'aaSubscribeBefore',
                   'aaSubscribeResumed',
                   'aaSubscribeSubscribed',
                   'aaSubscribeAfter',

                   'aaUnsubscribeBefore',
                   'aaUnsubscribeUnsubscribed',
                   'aaUnsubscribeAfter');

 {  ᠬ p䨪 (뫠  Areafix) }

 afxcRegisterCommandHandler                     = $2000;
 afxcCreateReply                                = $2001;
 afxcPerformSubscribe                           = $2002;
 afxcPerformUnsubscribe                         = $2003;
 afxcFixEmptyReply                              = $2004;
 afxcGetRequestInfo                             = $2005;
 afxcRegisterCallback                           = $2006;
 afxcIsOperable                                 = $2007;
 afxcIsShowable                                 = $2008;
 afxcIsSubscribeable                            = $2009;
 afxcIsUnsubscribeable                          = $200A;
 afxcGetAreaDescription                         = $200B;
 afxcGetName                                    = $200C;
 afxcAddRecipient                               = $200D;

 {   p䨪 (뫠  , 襬 ᮮ⢥騩
   callback) }

 acSubscribeBefore                              = $6000;
 acSubscribeResumed                             = $6001;
 acSubscribeSubscribed                          = $6002;
 acSubscribeAfter                               = $6003;

 acUnsubscribeBefore                            = $6010;
 acUnsubscribeUnsubscribed                      = $6011;
 acUnsubscribeAfter                             = $6012;

procedure RegisterAreafixCallback(const Plugin: Pointer; const Action: TAreafixAction);

{ RegisterAreafixCallback 뢠  PluginSetup/PluginStartup  ⮣,
  ⮡ ppp callback Areafix' }

procedure DisposeAreafixCallbacks;

{ DisposeAreafixCallbacks 뢠  PluginShutdown, ᫨   
  맮 RegisterAreafixCallback }

procedure afxRegisterCommandHandler(const Command: String; const AHandler: TAreafixCommandHandler);

{ afxRegisterCommandHandler ppp p稪 %- p䨪 }

function afxCreateReply(const Name: String; var Created: Boolean): Pointer;

{ afxCreateReply ᮧ ⢥   Name, p頥  Created, 
    䠪᪨ ᮧ (True)   ᮧ p (False),
  p頥 msg (jt.msg...), . 室 Areafix }

procedure afxPerformSubscribe(const Mask: String);
procedure afxPerformUnsubscribe(const Mask: String);

procedure afxFixEmptyReply(const Reply: Pointer);

procedure afxGetRequestInfo(var Info: TAreafixGetRequestInfoBlock);

function afxIsOperable(const Area: Pointer): Boolean;
function afxIsShowable(const Link, Area: Pointer): Boolean;
function afxIsSubscribeable(const Link, Area: Pointer): Boolean;
function afxIsUnsubscribeable(const Link, Area: Pointer): Boolean;
function afxGetAreaDescription(const Area: Pointer): String;
function afxGetName: String;

procedure afxAddRecipient(const Address: PAddress);

implementation

const
 AreafixPluginHandleCommand: TPluginHandleCommand = nil;

 Blocks: PRegisterAreafixCallbackBlock = nil;

{ RegisterAreafixCallback }

procedure RegisterAreafixCallback(const Plugin: Pointer; const Action: TAreafixAction);
 var
  Block: PRegisterAreafixCallbackBlock;
 begin
  New(Block);

  Block^.Plugin:=Plugin;
  Block^.Action:=Action;
  Block^.Next:=Blocks;

  jt.EnqueueCommandEx('Areafix', afxcRegisterCallback, Longint(Block), cwAfterStartup);

  Blocks:=Block;
 end;

{ DisposeAreafixCallbacks }

procedure DisposeAreafixCallbacks;
 var
  Block, ABlock: PRegisterAreafixCallbackBlock;
 begin
  Block:=Blocks;

  while Block <> nil do
   begin
    ABlock:=Block^.Next;

    Dispose(Block);

    Block:=ABlock;
   end;
 end;

{ FixPlugin }

function FixPlugin: Boolean;
 var
  Plugin: Pointer;
 begin
  if @AreafixPluginHandleCommand <> nil then
   FixPlugin:=True
  else
   begin
    Plugin:=jt.plugs.Seek('Areafix');

    if Plugin = nil then
     FixPlugin:=False
    else
     begin
      jt.plugs.GetPluginHandleCommand(Plugin, AreafixPluginHandleCommand);

      FixPlugin:=@AreafixPluginHandleCommand <> nil;
     end;
   end;
 end;

{ afxRegisterCommandHandler }

procedure afxRegisterCommandHandler(const Command: String; const AHandler: TAreafixCommandHandler);
 var
  Block: TAreafixRegisterCommandHandlerBlock;
 begin
  if FixPlugin then
   begin
    Block.Command:=@Command;
    Block.Handler:=AHandler;

    AreafixPluginHandleCommand(afxcRegisterCommandHandler, Longint(@Block));
   end;
 end;

{ afxCreateReply }

function afxCreateReply(const Name: String; var Created: Boolean): Pointer;
 var
  Block: TAreafixCreateReplyBlock;
 begin
  if FixPlugin then
   begin
    Block.Name:=@Name;

    AreafixPluginHandleCommand(afxcCreateReply, Longint(@Block));

    Created:=Block.Created;

    afxCreateReply:=Block.Reply;
   end
  else
   afxCreateReply:=nil;
 end;

{ afxPerformSubscribe }

procedure afxPerformSubscribe(const Mask: String);
 begin
  if FixPlugin then
   AreafixPluginHandleCommand(afxcPerformSubscribe, Longint(@Mask));
 end;

{ afxPerformUnsubscribe }

procedure afxPerformUnsubscribe(const Mask: String);
 begin
  if FixPlugin then
   AreafixPluginHandleCommand(afxcPerformUnsubscribe, Longint(@Mask));
 end;

{ afxFixEmptyReply }

procedure afxFixEmptyReply(const Reply: Pointer);
 begin
  if FixPlugin then
   AreafixPluginHandleCommand(afxcFixEmptyReply, Longint(Reply));
 end;

{ afxGetRequestInfo }

procedure afxGetRequestInfo(var Info: TAreafixGetRequestInfoBlock);
 begin
  if FixPlugin then
   AreafixPluginHandleCommand(afxcGetRequestInfo, Longint(@Info));
 end;

{ afxIsOperable }

function afxIsOperable(const Area: Pointer): Boolean;
 var
  Block: TafxIsAreaBlock;
 begin
  if FixPlugin then
   begin
    Block.Area:=Area;

    AreafixPluginHandleCommand(afxcIsOperable, Longint(@Block));

    afxIsOperable:=Block.Yes;
   end
  else
   afxIsOperable:=False;
 end;

{ afxIsShowable }

function afxIsShowable(const Link, Area: Pointer): Boolean;
 var
  Block: TafxIsLinkAreaBlock;
 begin
  if FixPlugin then
   begin
    Block.Link:=Link;
    Block.Area:=Area;

    AreafixPluginHandleCommand(afxcIsShowable, Longint(@Block));

    afxIsShowable:=Block.Yes;
   end
  else
   afxIsShowable:=False;
 end;

{ afxIsSubscribeable }

function afxIsSubscribeable(const Link, Area: Pointer): Boolean;
 var
  Block: TafxIsLinkAreaBlock;
 begin
  if FixPlugin then
   begin
    Block.Link:=Link;
    Block.Area:=Area;

    AreafixPluginHandleCommand(afxcIsSubscribeable, Longint(@Block));

    afxIsSubscribeable:=Block.Yes;
   end
  else
   afxIsSubscribeable:=False;
 end;

{ afxIsUnsubscribeable }

function afxIsUnsubscribeable(const Link, Area: Pointer): Boolean;
 var
  Block: TafxIsLinkAreaBlock;
 begin
  if FixPlugin then
   begin
    Block.Link:=Link;
    Block.Area:=Area;

    AreafixPluginHandleCommand(afxcIsUnsubscribeable, Longint(@Block));

    afxIsUnsubscribeable:=Block.Yes;
   end
  else
   afxIsUnsubscribeable:=False;
 end;

{ afxGetAreaDescription }

function afxGetAreaDescription(const Area: Pointer): String;
 var
  Block: TafxGetAreaDescriptionBlock;
 begin
  if FixPlugin then
   begin
    Block.Area:=Area;

    AreafixPluginHandleCommand(afxcGetAreaDescription, Longint(@Block));

    afxGetAreaDescription:=Block.Description;
   end
  else
   afxGetAreaDescription:='';
 end;

{ afxGetName }

function afxGetName: String;
 var
  S: String;
 begin
  if FixPlugin then
   begin
    AreafixPluginHandleCommand(afxcGetName, Longint(@S));

    afxGetName:=S;
   end
  else
   afxGetName:='';
 end;

{ afxAddRecipient }

procedure afxAddRecipient(const Address: PAddress);
 begin
  if FixPlugin then
   AreafixPluginHandleCommand(afxcAddRecipient, Longint(Address));
 end;

end.