unit annAnn;

{.$DEFINE NORESREMOVE}

interface
uses
     Objects,

     PlugAPI,
     PlugSIDE,

     annCmn,
     annRule,
     annAPI,

     Wizard;

type
 PResTableItem = ^TResTableItem;
 TResTableItem = object(TStrings)
  LinkedType: PType;

  constructor Init(const ALinkedType: PType);
 public
  procedure Add(const ResourceName: String);
 end;

 PResTable = ^TResTable;
 TResTable = object(TStringCollection)
  procedure FreeItem(Item: Pointer); virtual;
  function KeyOf(Item: Pointer): Pointer; virtual;
 public
  function Lookup(const TypeName: String): PResTableItem;
 end;

procedure PerformAnnounce;

implementation

{ TResTableItem }

constructor TResTableItem.Init(const ALinkedType: PType);
 begin
  inherited Init($10, $10);

  LinkedType:=ALinkedType;
 end;

{ public methods }

procedure TResTableItem.Add(const ResourceName: String);
 begin
  Insert(NewStr(ResourceName));
 end;

{ TResTable }

procedure TResTable.FreeItem(Item: Pointer);
 begin
  Dispose(PObject(Item), Done);
 end;

function TResTable.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=PResTableItem(Item)^.LinkedType^.Name;
 end;

{ public methods }

function TResTable.Lookup(const TypeName: String): PResTableItem;
 var
  Index: xInteger;
  AType: PType;
  Item: PResTableItem;
 begin
  if Search(@TypeName, Index) then
   Lookup:=At(Index)
  else
   begin
    AType:=Types^.Lookup(TypeName);

    if AType = nil then
     Lookup:=nil
    else
     begin
      Item:=New(PResTableItem, Init(AType));

      Insert(Item);

      Lookup:=Item;
     end;
   end;
 end;

{ PerformAnnounce }

procedure PerformAnnounce;
 var
  ResTable: PResTable;
 procedure CreateAnnounceTable;
  var
   K, Size: Longint;
   Name: PString;
   Junk: Boolean;
   Item: PResTableItem;
   PluginInformation: TPluginInformation;
  begin
   for K:=0 to jt.rf.Count(rf) - 1 do
    begin
     Name:=jt.rf.At(rf, K, Junk, Size);

     if (Name = nil) or Junk then
      Continue;

     Item:=ResTable^.Lookup(ExtractWord(1, Name^, ['.']));

     if Item = nil then
      Continue;

     if Debugger.Enabled then
      begin
       jt.plugs.GetInformation(Item^.LinkedType^.Plugin, PluginInformation);

       Debugger.Log('found "' + Name^ + '" - type "' + Item^.LinkedType^.Name^ + '" (' + PluginInformation.Name + ')');
      end;

     Item^.Add(Name^);
    end;
  end;
 function Process(const Item: PResTableItem): Boolean; far;
  var
   PluginHandleCommand: TPluginHandleCommand;
   PluginInformation: TPluginInformation;
   FirstBlock: TProcessAnnounceFirstBlock;
   Block: TProcessAnnounceBlock;
   LastBlock: TProcessAnnounceLastBlock;
   msg: Pointer;
  procedure Perform(const ResourceName: PString); far;
   begin
    Block.ResourceName:=ResourceName;

    Inc(Block.This);

    if Debugger.Enabled then
     Debugger.Log('sending "' + ResourceName^ + '" to ' + PluginInformation.Name + '; msg=' + HexPtr(msg));

    PluginHandleCommand(AnnouncerProcess, Longint(@Block));

    {$IFNDEF NORESREMOVE}
    jt.rf.Remove(rf, ResourceName^);
    {$ENDIF}
   end;
  begin
   jt.SetStatus('announcing ' + Item^.LinkedType^.Name^ + '...');

   jt.plugs.GetPluginHandleCommand(Item^.LinkedType^.Plugin, PluginHandleCommand);

   if Debugger.Enabled then
    begin
     jt.plugs.GetInformation(Item^.LinkedType^.Plugin, PluginInformation);

     Debugger.Log('doing ' + Long2Str(Item^.Count) + ' ' + Item^.LinkedType^.Name^ + ' ann(s) - ' + PluginInformation.Name);
    end;

   msg:=jt.msg.Create;

   { // send first }

   if Debugger.Enabled then
    Debugger.Log('sending "first" to ' + PluginInformation.Name + '; msg=' + HexPtr(msg));

   FirstBlock.Name:=Item^.LinkedType^.Name;
   FirstBlock.msg:=msg;
   FirstBlock.Count:=Item^.Count;

   PluginHandleCommand(AnnouncerProcessFirst, Longint(@FirstBlock));

   { // send each resource }

   Block.Name:=Item^.LinkedType^.Name;
   Block.msg:=msg;
   Block.rf:=rf;
   Block.Count:=Item^.Count;
   Block.This:=0;

   Item^.ForEach(@Perform);

   { // send last }

   if Debugger.Enabled then
    Debugger.Log('sending "last" to ' + PluginInformation.Name + '; msg=' + HexPtr(msg));

   LastBlock.Name:=Item^.LinkedType^.Name;
   LastBlock.msg:=msg;
   LastBlock.Count:=Item^.Count;

   PluginHandleCommand(AnnouncerProcessLast, Longint(@LastBlock));

   { // process to rules }

   if not Rules^.Process(Item^.LinkedType^.Name^, msg) then
    jt.LogWrite('?', 'No one handled announce type "' + Item^.LinkedType^.Name^ + '" (' + Long2Str(Item^.Count) + ' actions)')
   else
    jt.LogWrite('A', Item^.LinkedType^.Name^ + ' announced');

   jt.msg.Destroy(msg);

   Process:=jt.Cancelled;
  end;
 begin
  if jt.Cancelled then
   Exit;

  ResTable:=New(PResTable, Init($10, $10));

  Debugger.Log('creating ResTable');

  CreateAnnounceTable;

  Debugger.Log('doing ResTable');

  ResTable^.FirstThat(@Process);

  Dispose(ResTable, Done);

  Debugger.Log('finished');

  jt.ClearStatus;
 end;

end.