library Announce;

{
  dedicated to Anna "Ann" Rogatina
}

uses
     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     annCmn,
     annAPI,
     annRule,
     annAnn,

     PrioAPI,

     Wizard,

     Objects;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, AnnouncerName, AnnouncerVersion);

  AnnouncerPlugin:=Plugin;

  PluginSetup:=True;

  Rules:=New(PRules, Init($10, $10));
  Types:=New(PTypes, Init($10, $10));

  RegisterPriorityAction('ANNOUNCING');
 end;

{ PluginStartup }

procedure PluginStartup; export;
 begin
  rf:=jt.rf.Create(AreaActions, rffNone);
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
  jt.rf.Destroy(rf);

  Dispose(Types, Done);
  Dispose(Rules, Done);
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 var
  RegisterAnnounceTypeBlock: PRegisterAnnounceTypeBlock absolute Param;
 begin
  if Command = jtpcPluginSectionEnd then
   begin
    if (Rules^.Count > 0) and (AreaActions = '') then
     jt.linker.Error(Pointer(Param), 'AREAACTIONS was not defined correctly');
   end else
  if Command = AnnouncerPerform then PerformAnnounce else
  if Command = AnnouncerGetRF then PPointer(Param)^:=rf else
  if Command = AnnouncerRegisterAnnounceType then with RegisterAnnounceTypeBlock^ do Types^.Register(Plugin, Name^) else
 end;

{ PluginHandleConfigSwitch }

procedure PluginHandleConfigSwitch(const Keyword: String; Value: String; const Linker: Pointer); export;
 procedure ParseRule;
  var
   Mode: String;
   Rule: PRule;
  begin
   Mode:=jt.linker.ParseWord(Linker, Value);

   StUpcaseEx(Mode);

   if Mode = 'NULL' then Rule:=New(PNullRule, Init(Value, Linker)) else
   if Mode = 'NET' then Rule:=New(PNetRule, Init(Value, Linker)) else
   if Mode = 'ECHO' then Rule:=New(PEchoRule, Init(Value, Linker)) else
    jt.linker.Error(Linker, 'unknown rule mode (NULL/NET/ECHO expected)');

   if Rule <> nil then
    Rules^.Insert(Rule);
  end;
 begin
  if Keyword = 'AREAACTIONS' then AreaActions:=jt.linker.ParseStringToken(Linker, Value) else
  if Keyword = 'RULE' then ParseRule else
   jt.linker.Error(Linker, 'syntax error');
 end;

{ PluginHandleCmdlineSwitch }

function PluginHandleCmdlineSwitch(Switch: String): Boolean; export;
 begin
  StUpcaseEx(Switch);

  PluginHandleCmdlineSwitch:=True;

  if Switch = 'ANNOUNCE' then jt.EnqueueCommand(AnnouncerPlugin, AnnouncerPerform, 0, cwLatest) else
  if Switch = 'DEBUGANNOUNCER' then Debugger.Enabled:=True else
   PluginHandleCmdlineSwitch:=False;
 end;

{ PluginShowHelp }

procedure PluginShowHelp; export;
 begin
  jt.PrintStr(' ANNOUNCE         post announces'#13#10);
  jt.PrintStr(' DEBUGANNOUNCER   debug announcer (use with ANNOUNCE)'#13#10);
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleConfigSwitch,
 PluginHandleCmdlineSwitch,
 PluginShowHelp;

begin
 Assign(Output, '');
 Rewrite(Output);
end.
