unit annRule;

interface
uses
     Objects,

     PlugAPI,
     PlugSIDE,

     PrioAPI,

     jtMHLcmn,

     Wizard,

     annCmn;

type
 PRule = ^TRule;
 TRule = object(TObject)
  Tokens: Pointer;

  constructor Init(var Value: String; const Linker: Pointer);
  destructor Done; virtual;
 public
  function Match(const Token: String): Boolean; virtual;
  procedure Perform(const Token: String; msg: Pointer); virtual;
 end;

 PNullRule = ^TNullRule;
 TNullRule = object(TRule)
  procedure Perform(const Token: String; msg: Pointer); virtual;
 end;

 PAreaRule = ^TAreaRule;
 TAreaRule = object(TRule)
  Area: Pointer;

  constructor Init(var Value: String; const Linker: Pointer);
 end;

 PEchoRule = ^TEchoRule;
 TEchoRule = object(TAreaRule)
  procedure Perform(const Token: String; msg: Pointer); virtual;
 end;

 PNetRule = ^TNetRule;
 TNetRule = object(TRule)
  ToName: PString;
  ToAddress: TAddress;

  constructor Init(var Value: String; const Linker: Pointer);
  destructor Done; virtual;
 public
  procedure Perform(const Token: String; msg: Pointer); virtual;
 end;

 PRules = ^TRules;
 TRules = object(TCollection)
  function Process(const Name: String; const msg: Pointer): Boolean;
 end;

 PType = ^TType;
 TType = record
  Plugin: Pointer;
  Name: PString;
 end;

 PTypes = ^TTypes;
 TTypes = object(TStringCollection)
  procedure FreeItem(Item: Pointer); virtual;
  function KeyOf(Item: Pointer): Pointer; virtual;
 public
  procedure Register(const Plugin: Pointer; const Name: String);
  function Lookup(const Name: String): PType;
 end;

const
 Rules                  : PRules                = nil;
 Types                  : PTypes                = nil;

 PriorityDid            : Boolean               = False;

implementation

{ TRule }

constructor TRule.Init(var Value: String; const Linker: Pointer);
 var
  S: String;
 begin
  inherited Init;

  jt.CompileRegExp(jt.linker.ParseStringToken(Linker, Value), Tokens, Linker, True);
 end;

destructor TRule.Done;
 begin
  jt.DestroyRegExp(Tokens);

  inherited Done;
 end;

{ public methods }

function TRule.Match(const Token: String): Boolean;
 begin
  Match:=jt.re.MatchStr(Tokens, Token, False);
 end;

procedure TRule.Perform(const Token: String; msg: Pointer);
 begin
  Abstract;
 end;

{ TNullRule }

procedure TNullRule.Perform(const Token: String; msg: Pointer);
 begin
 end;

{ TAreaRule }

constructor TAreaRule.Init(var Value: String; const Linker: Pointer);
 begin
  inherited Init(Value, Linker);

  jt.linker.CreateAreaForward(jt.linker.ParseStringToken(Linker, Value), @Area, jt.linker.FName(Linker),
   jt.linker.Line(Linker));
 end;

{ TEchoRule }

procedure TEchoRule.Perform(const Token: String; msg: Pointer);
 var
  Aka: TAddress;
  msgp: Pointer;
 begin
  Debugger.Log('posting echorule "' + Token + '" to ' + jt.area.GetName(Area)^ + '; msg=' + HexPtr(msg));

  msg:=jt.msg.Clone(msg, mcfCloneAll);

  jt.msg.SetArea(msg, jt.area.GetName(Area)^);
  jt.msg.SetFromName(msg, jt.core.GetFullTitle);
  jt.msg.SetToName(msg, 'All');

  jt.st.Add(jt.msg.st(msg), 'Source', Pointer(tsLocalInbound), nil);

  jt.area.GetAka(Area, Aka);

  msgp:=jt.msg.CreateParser(msg, False);

  jt.msg.p.SetFromAddress(msgp, Aka, True);

  jt.msg.p.SetTextPos(msgp, jt.msg.p.GetTextSize(msgp));

  jt.msg.p.PutTearline(msgp);

  jt.msg.p.PutOrigin(msgp, Aka);

  jt.msg.DestroyParser(msgp);

  jt.tossscan.TossMessage(msg);
 end;

{ TNetRule }

constructor TNetRule.Init(var Value: String; const Linker: Pointer);
 begin
  inherited Init(Value, Linker);

  ToName:=NewStr(jt.linker.ParseStringToken(Linker, Value));

  if not StrToAddress(jt.linker.ParseWord(Linker, Value), ToAddress) then
   jt.linker.Error(Linker, 'failed to parse destination address');
 end;

destructor TNetRule.Done;
 begin
  DisposeStr(ToName);

  inherited Done;
 end;

{ public methods }

procedure TNetRule.Perform(const Token: String; msg: Pointer);
 var
  FromAddress: TAddress;
  msgp: Pointer;
 begin
  Debugger.Log('posting netrule "' + Token + '" to ' + GetPString(ToName) + ' (' +
   AddressToStrEx(ToAddress) + '); msg=' + HexPtr(msg));

  msg:=jt.msg.Clone(msg, mcfCloneAll);

  jt.st.Add(jt.msg.st(msg), 'Source', Pointer(tsLocalInbound), nil);

  jt.msg.SetArea(msg, '');
  jt.msg.SetFromName(msg, jt.core.GetFullTitle);
  jt.msg.SetToName(msg, GetPString(ToName));

  msgp:=jt.msg.CreateParser(msg, True);

  jt.GetNetmailAka(ToAddress, FromAddress);

  jt.msg.p.SetFromAndToAddress(msgp, FromAddress, ToAddress, True);

  jt.msg.p.SetTextPos(msgp, jt.msg.p.GetTextSize(msgp));

  jt.msg.p.PutTearline(msgp);

  jt.msg.p.PutOrigin(msgp, FromAddress);

  jt.msg.DestroyParser(msgp);

  jt.tossscan.TossMessage(msg);
 end;

{ TRules }

function TRules.Process(const Name: String; const msg: Pointer): Boolean;
 var
  AnyoneHandled: Boolean;
 procedure Perform(const Rule: PRule); far;
  begin
   if Rule^.Match(Name) then
    begin
     if not PriorityDid then
      begin
       PerformPriorityAction('announcing');

       PriorityDid:=False;
      end;

     Rule^.Perform(Name, msg);

     AnyoneHandled:=True;
    end;
  end;
 begin
  AnyoneHandled:=False;

  ForEach(@Perform);

  Process:=AnyoneHandled;
 end;

{ TTypes }

procedure TTypes.FreeItem(Item: Pointer);
 begin
  DisposeStr(PType(Item)^.Name);

  Dispose(PType(Item));
 end;

function TTypes.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=PType(Item)^.Name;
 end;

procedure TTypes.Register(const Plugin: Pointer; const Name: String);
 var
  Index: xInteger;
  AType: PType;
 begin
  if not Search(@Name, Index) then
   begin
    New(AType);

    AType^.Plugin:=Plugin;
    AType^.Name:=NewStr(Name);

    Insert(AType);
   end;
 end;

function TTypes.Lookup(const Name: String): PType;
 var
  Index: xInteger;
 begin
  if Search(@Name, Index) then
   Lookup:=At(Index)
  else
   Lookup:=nil;
 end;

end.
