unit afxCore;

{$IFDEF DEVELOPING}
 {$DEFINE NORECEIVED}
{$ENDIF}

interface
uses
     PlugAPI,
     PlugSIDE,

     Objects,

     jtMHLcmn,

     ccAPI,

     afxCmn,
     afxDisp,
     afxRight,
     afxAPI,

     MParser,

     Wizard;

type
 TReplyMode = (rmNormal, rmHand, rmHandLog, rmHandQuiet);

 PAreafixParser = ^TAreafixParser;
 TAreafixParser = object(TMParser)
  constructor Init(const AArea: Pointer);
  procedure Status(const AStatus: Word; const Param: Pointer); virtual;
  procedure Error(const AError: Word; const Param: Pointer); virtual;
 public
  function Applicable(const msg: Pointer): Boolean; virtual;
  procedure ProcessMessage(const msg, msgp: Pointer); virtual;
  procedure ProcessLine(const msg, msgp: Pointer; var Line: String; var Finish: Boolean); virtual;
 public
  Empty: Boolean;
  Link: Pointer;
  LinkAddress: TAddress;
  st, Thismsg: Pointer;
  ReplyMode: TReplyMode;
  ReplyKludge: String;

  function CreateReplyEx(const Name: String; var Created: Boolean): Pointer; virtual;
  procedure CheckValidReplyToName(var ToName: String); virtual;
  procedure AfterCreateReply(const msg: Pointer); virtual;
  procedure PostReply(const msg: Pointer); virtual;
 public
  procedure PerformSubscribe(Mask: String);
  procedure PerformUnsubscribe(Mask: String);
  procedure AddRecipient(const Address: TAddress);
  procedure Command(const ALink: Pointer; const ACommand: String; const AReplyMode: TReplyMode);
 end;

const
 AreafixParser: PAreafixParser = nil;

 ToDoToNumber: array[TMParserToDo] of Longint = (0, 1, 2);
 ReplyModeToNumber: array[TReplyMode] of Longint = (0, 1, 2, 3);

procedure NumberToToDo(const Number: Longint; var ToDo: TMParserToDo);
procedure NumberToReplyMode(const Number: Longint; var ReplyMode: TReplyMode);

implementation

const
 First: Boolean = True;

{ TAreafixParser }

constructor TAreafixParser.Init(const AArea: Pointer);
 begin
  inherited Init(AArea);

  ReplyMode:=rmNormal;
 end;

procedure TAreafixParser.Status(const AStatus: Word; const Param: Pointer);
 begin
  case AStatus of
   mpsOpeningBase: jt.SetTitle('areafix - ' + AreaName + ' (opening)');
   mpsScanningBase: jt.SetTitle('areafix - ' + AreaName + ' (scanning)');
   mpsClosingBase: jt.SetTitle('areafix - ' + AreaName + ' (closing)');
  end;

  jt.SetStatus('areafix - ' + AreaName);
 end;

procedure TAreafixParser.Error(const AError: Word; const Param: Pointer);
 var
  S: PString absolute Param;
 begin
  case AError of
   mpeFailedToOpenBase: jt.LogWrite('?', 'Failed to open base ' + AreaName + ': ' + S^);
   mpeFailedToLock: jt.LogWrite('?', 'Failed to lock base ' + AreaName);
   mpeFailedToKillMessage: jt.LogWrite('?', 'Failed to kill msg ' + Long2Str(No) + ' in ' + AreaName + ': ' + S^);
   mpeFailedToLoadMessage: jt.LogWrite('?', 'Failed to load msg ' + Long2Str(No) + ' in ' + AreaName + ': ' + S^);
   mpeFailedToStoreMessage: jt.LogWrite('?', 'Failed to store msg ' + Long2Str(No) + ' in ' + AreaName + ': ' + S^);
  end;
 end;

{ public methods }

function TAreafixParser.Applicable(const msg: Pointer): Boolean;
 var
  msgp: Pointer;
  ToAddress: TAddress;
 begin
  msgp:=jt.msg.CreateParser(msg, True);

  ClearAddress(ToAddress);

  jt.msg.p.GetToAddress(msgp, ToAddress);

  jt.msg.DestroyParser(msgp);

  Applicable:=jt.core.IsOurAddress(ToAddress) and
              jt.re.MatchStr(afxRespondTo, GetPString(jt.msg.GetToName(msg)), False) and
              (jt.msg.GetAttributes(msg) and maReceived = 0);
 end;

procedure TAreafixParser.ProcessMessage(const msg, msgp: Pointer);
 procedure CheckTodo;
  begin
   if afxKillRequests then
    ToDo:=tdKill
   else
    begin
     {$IFNDEF NORECEIVED}
     jt.msg.SetAttributes(msg, jt.msg.GetAttributes(msg) or maReceived);
     {$ENDIF}

     ToDo:=tdStore;
    end;
  end;
 var
  Reply, Linker, linkst: Pointer;
  Subject, Password: String;
  LinkPassword: PString;
  Created: Boolean;
 begin
  ReplyKludge:='';

  Thismsg:=msg;

  if First then
   begin
    jt.LogWrite('A', 'Scanning ' + AreaName + ' for areafix requests');

    First:=False;
   end;

  case ReplyMode of
   rmNormal:
    jt.LogWrite('A', 'Message from ' + AddressToStrEx(ReplyToAddress) + ' (' + ReplyToName + '):');
   rmHand:
    jt.LogWrite('A', 'Fake message from ' + AddressToStrEx(ReplyToAddress) + ' (' + ReplyToName + '):');
   rmHandLog:
    jt.LogWrite('A', 'Fake message (replies to log) from ' + AddressToStrEx(ReplyToAddress) + ' (' + ReplyToName + '):');
   rmHandQuiet:
    jt.LogWrite('A', 'Fake message (no replies produced) from ' + AddressToStrEx(ReplyToAddress) + ' (' + ReplyToName + '):');
  end;

  Link:=nil;

  st:=jt.st.Create;

  Reply:=CreateReplyEx('Common', Created);

  jt.msg.SetSubject(Reply, 'answer on your request');

  FixEmptyReply(Reply);

  if ccAvailable then
   with ccafBeforeProcess^ do
    begin
     AddConstant('MSG', etNumber)^.SetNumber(Longint(msg));
     AddConstant('REPLYTOADDRESS', etNumber)^.SetNumber(Longint(@ReplyToAddress));
     AddVariable('REPLYTONAME', etString)^.SetString(ReplyToName);
     AddVariable('REPLYMODE', etNumber)^.SetNumber(ReplyModeToNumber[ReplyMode]);
     AddVariable('TODO', etNumber)^.SetNumber(ToDoToNumber[ToDo]);
     AddVariable('STOP', etBoolean)^.SetBoolean(False);

     Execute;

     ReplyToName:=AddVariable('REPLYTONAME', etString)^.GetString;

     NumberToReplyMode(AddVariable('REPLYMODE', etNumber)^.GetNumber, ReplyMode);
     NumberToToDo(AddVariable('TODO', etNumber)^.GetNumber, ToDo);

     if AddVariable('STOP', etBoolean)^.GetBoolean then
      begin
       jt.st.Destroy(st);

       Exit;
      end;
    end;

  { // check security and remote operating }

  GetPStringEx(jt.msg.GetSubject(msg), Subject);

  if Subject = '' then
   begin
    jt.msg.Add(Reply, 'invalid (empty) subject specified');

    jt.LogWrite('A', 'Invalid (empty) subject specified');

    jt.st.Destroy(st);

    CheckTodo;

    Exit;
   end;

  if Subject[1] = '"' then
   begin
    Linker:=jt.linker.v.Create;

    Password:=jt.linker.ParseStringToken(Linker, Subject);

    if jt.linker.v.WasError(Linker) then
     begin
      jt.msg.Add(Reply, jt.linker.v.GetError(Linker) + ' in password "' + jt.msg.GetSubject(msg)^ + '"');

      jt.LogWrite('A', jt.linker.v.GetError(Linker) + ' in password "' + jt.msg.GetSubject(msg)^ + '"');

      jt.linker.v.Destroy(Linker);

      jt.st.Destroy(st);

      CheckTodo;

      Exit;
     end;

    jt.linker.v.Destroy(Linker);
   end
  else
   begin
    Password:=ExtractWord(1, Subject, [' ']);

    Subject:=GetAllAfterSpace(Subject, 1);
   end;

  TrimEx(Subject);

  ClearAddress(LinkAddress);

  if Subject <> '' then
   if not StrToAddress(Subject, LinkAddress) then
    begin
     jt.msg.Add(Reply, 'unable to parse link address "' + Subject + '"');

     jt.LogWrite('A', 'Unable to parse link address "' + Subject + '" (password "' + Password + '")');

     jt.st.Destroy(st);

     CheckTodo;

     Exit;
    end
   else
  else
   LinkAddress:=ReplyToAddress;

  Link:=jt.link.Seek(LinkAddress);

  if Link = nil then
   begin
    jt.msg.Add(Reply, 'unknown link ' + AddressToStrEx(LinkAddress));

    jt.LogWrite('A', 'Unknown link ' + AddressToStrEx(LinkAddress));

    jt.st.Destroy(st);

    CheckTodo;

    Exit;
   end;

  if not AddressEqu(ReplyToAddress, LinkAddress) then
   jt.LogWrite('A', 'Remote operating for link ' + AddressToStrEx(LinkAddress));

  linkst:=jt.link.st(Link);

  LinkPassword:=jt.st.Get(linkst, 'Areafix.Password');

  if LinkPassword = nil then
   begin
    jt.LogWrite('A', 'No AreafixPassword defined for ' + AddressToStrEx(LinkAddress));

    jt.msg.Add(Reply, AddressToStrEx(LinkAddress) + ' have no rights to operate Areafix');

    jt.st.Destroy(st);

    CheckTodo;

    Exit;
   end;

  if not jt.ComparePasswords(Password, LinkPassword^, Link) then
   begin
    jt.LogWrite('A', 'Password mismatch for ' + AddressToStrEx(LinkAddress) + ': got "' + Password +
      '", expected "' + LinkPassword^ + '"');

    jt.msg.Add(Reply, 'invalid password specified for ' + AddressToStrEx(LinkAddress));

    jt.st.Destroy(st);

    CheckTodo;

    Exit;
   end;

  if not AddressEqu(ReplyToAddress, LinkAddress) then
   AddRecipient(LinkAddress);

  if (ReplyMode = rmNormal) and jt.msg.p.GetKludge(msgp, #1'MSGID: ', ReplyKludge) then
   ReplyKludge:=#1'REPLY: ' + Copy(ReplyKludge, 9, 255)
  else
   ReplyKludge:='';

  Empty:=True;

  inherited ProcessMessage(msg, msgp);

  if Empty then
   begin
    jt.msg.Add(Reply, 'empty request');

    jt.LogWrite('A', 'Empty request (what the hell?)');
   end;

  jt.st.Destroy(st);

  CheckTodo;
 end;

procedure TAreafixParser.ProcessLine(const msg, msgp: Pointer; var Line: String; var Finish: Boolean);
 var
  Linker: Pointer;
  S, Value: String;
  CommandHandler: PCommandHandler;
  Index: xInteger;
  Subscribe: Boolean;
 begin
  TrimEx(Line);

  if Line = '' then
   Exit;

  if ccAvailable then
   with ccafProcessLine^ do
    begin
     AddConstant('MSG', etNumber)^.SetNumber(Longint(msg));
     AddConstant('MSGP', etNumber)^.SetNumber(Longint(msgp));
     AddConstant('LINK', etNumber)^.SetNumber(Longint(Link));
     AddVariable('LINE', etString)^.SetString(Line);
     AddVariable('FINISH', etBoolean)^.SetBoolean(False);

     Execute;

     Line:=AddVariable('LINE', etString)^.GetString;
     Finish:=AddVariable('FINISH', etBoolean)^.GetBoolean;

     if Line = '' then
      Exit;
    end;

  if (Copy(Line, 1, 3) = '---') or (Copy(Line, 1, 3) = '...') or (Copy(Line, 1, 9) = '* Origin:') then
   begin
    Finish:=True;

    Exit;
   end;

  { // check "value" and explain quotes }

  if Line[1] = '"' then
   begin
    Linker:=jt.linker.v.Create;

    S:=jt.linker.ParseStringToken(Linker, Line);

    if jt.linker.v.WasError(Linker) then
     begin
      jt.msg.Add(jt.st.Get(st, 'Reply.Common'), jt.linker.v.GetError(Linker) + ' in expression "' + Line + '"');

      jt.linker.v.Destroy(Linker);

      Exit;
     end;

    jt.linker.v.Destroy(Linker);

    Line:=S;

    TrimEx(Line);

    if Line = '' then
     Exit;
   end;

  { // process command }

  Empty:=False;

  if Line[1] = '%' then
   begin
    S:=Copy(ExtractWord(1, Line, [' ']), 2, 255);

    Value:=GetAllAfterSpace(Line, 1);

    TrimEx(Value);

    StUpcaseEx(S);

    if CommandHandlers^.Search(@S, Index) then
     begin
      jt.LogWrite('A', Line);

      CommandHandler:=CommandHandlers^.At(Index);

      CommandHandler^.Handler(S, Value);
     end
    else
     begin
      jt.msg.Add(jt.st.Get(st, 'Reply.Common'), 'unknown command "' + Line + '"');

      jt.LogWrite('A', 'Unknown command "' + Line + '"');
     end;

    Exit;
   end;

  { // process subscribe/unsubscribe }

  if Line[1] in ['-', '+'] then
   begin
    Subscribe:=Line[1] = '+';

    Delete(Line, 1, 1);
   end
  else
   Subscribe:=True;

  if Subscribe then
   begin
    jt.LogWrite('A', '+' + Line);

    PerformSubscribe(Line);
   end
  else
   begin
    jt.LogWrite('A', '-' + Line);

    PerformUnsubscribe(Line);
   end;
 end;

{ public methods }

function TAreafixParser.CreateReplyEx(const Name: String; var Created: Boolean): Pointer;
 var
  Reply: Pointer;
 begin
  Reply:=jt.st.Get(st, 'Reply.' + Name);

  if Reply = nil then
   begin
    Reply:=CreateReply(afxName, afxReplyToArea);

    jt.st.Add(st, 'Reply.' + Name, Reply, stAreafixReply);

    Created:=True;
   end
  else
   Created:=False;

  CreateReplyEx:=Reply;
 end;

procedure TAreafixParser.CheckValidReplyToName(var ToName: String);
 begin
  if jt.re.MatchStr(afxRespondTo, ToName, False) then
   ToName:='Sysop';
 end;

procedure TAreafixParser.AfterCreateReply(const msg: Pointer);
 begin
  if afxKillReplies then
   jt.msg.SetAttributes(msg, jt.msg.GetAttributes(msg) or maKill);
 end;

procedure TAreafixParser.PostReply(const msg: Pointer);
 var
  msgp: Pointer;
 procedure Perform(const Address: PAddress); far;
  var
   Suffix: String;
   Amsg: Pointer;
   Aka: TAddress;
  begin
   Amsg:=jt.msg.Clone(msg, mcfCloneAll);

   jt.GetNetmailAka(Address^, Aka);

   msgp:=jt.msg.CreateParser(Amsg, True);

   jt.msg.p.SetFromAndToAddress(msgp, Aka, Address^, True);

   if not AddressEqu(LinkAddress, ReplyToAddress) then
    Suffix:=' (operating ' + AddressToStrEx(LinkAddress) + ')'
   else
    Suffix:='';

   jt.msg.p.AddKludge(msgp, '');
   jt.msg.p.AddKludge(msgp, '*** this is a copy of ' + afxName + ' replies to ' + AddressToStrEx(ReplyToAddress) + Suffix);

   jt.msg.DestroyParser(msgp);

   PostReply(Amsg);
  end;
 const
  AddressesBusy: Boolean = False;
 var
  Addresses: PAddresses;
  Address, FromAddress, ToAddress: TAddress;
  S: String;
 begin
  if IsEmptyReply(msg) then
   begin
    jt.msg.Destroy(msg);

    Exit;
   end;

  if not AddressesBusy then
   begin
    AddressesBusy:=True;

    Addresses:=jt.st.Get(jt.msg.st(Thismsg), 'Areafix.Recipients');

    if Addresses <> nil then
     Addresses^.ForEach(@Perform);

    AddressesBusy:=False;
   end;

  if ccAvailable then
   with ccafPostReply^ do
    begin
     AddVariable('MSG', etNumber)^.SetNumber(Longint(msg));

     Execute;

     if AddVariable('MSG', etNumber)^.GetNumber = 0 then
      Exit;
    end;

  msgp:=jt.msg.CreateParser(msg, True);

  if jt.msg.GetAttributes(msg) and maAttach = 0 then
   begin
    jt.msg.Add(msg, '');
    jt.msg.Add(msg, jt.core.GetTearline);

    jt.GetNetmailAka(ReplyToAddress, Address);

    jt.msg.p.PutOrigin(msgp, Address);
   end;

  if ReplyKludge <> '' then
   jt.msg.p.SetKludge(msgp, #1'REPLY:', ReplyKludge);

  jt.msg.p.SetKludge(msgp, #1'PID:', #1'PID: ' + jt.core.GetPID);

  jt.msg.DestroyParser(msgp);

  if jt.msg.GetAttributes(msg) and maAttach <> 0 then
   inherited PostReply(msg)
  else
   case ReplyMode of
    rmNormal, rmHand:
     inherited PostReply(msg);
    rmHandLog:
     begin
      msgp:=jt.msg.CreateParser(msg, True);

      jt.msg.p.GetFromAndToAddress(msgp, FromAddress, ToAddress);

      jt.LogWrite('A', 'Areafix reply message, ' + AddressToStrEx(FromAddress) + '  ' +
                                                   AddressToStrEx(ToAddress) + ', ' +
                                                   GetPString(jt.msg.GetSubject(msg)) + ':');

      jt.msg.p.SetTextPos(msgp, 0);

      while not jt.msg.p.EndOfMessage(msgp) do
       begin
        jt.msg.p.GetString(msgp, S);

        if ((S <> '') and (S[1] = #1)) or (Copy(S, 1, 3) = '---') or (Copy(S, 1, 10) = ' * Origin:') then
         Continue;

        if S <> '' then
         jt.LogWrite('A', S);
       end;

      jt.msg.DestroyParser(msgp);

      jt.msg.Destroy(msg);
     end;
    rmHandQuiet:
     jt.msg.Destroy(msg);
   end;

  Status(mpsScanningBase, nil);
 end;

{ public methods }

procedure TAreafixParser.PerformSubscribe(Mask: String);
 var
  Reply, AArea, LinkArea: Pointer;
  Name, Rights: String;
  Created: Boolean;
  Block: TAreafixSubscribeBlock;
  K: Longint;
 begin
  Mask:=ExtractWord(1, Mask, [#32, #9]);

  TrimEx(Mask);

  if Mask = '' then
   Exit;

  StUpcaseEx(Mask);

  Reply:=CreateReplyEx('SubUnsub', Created);

  if Created then
   begin
    jt.msg.SetSubject(Reply, 'subscribe/unsubscribe results');

    FixEmptyReply(Reply);
   end;

  Block.Mask:=@Mask;
  Block.Changes:=0;
  Block.Reply:=Reply;

  Callbacks[aaSubscribeBefore]^.ChainedPointer(acSubscribeBefore, @Block, False);

  if Block.Mask <> nil then
   begin
    for K:=0 to jt.area.Count - 1 do
     begin
      AArea:=jt.area.At(K);

      GetPStringEx(jt.area.GetName(AArea), Name);

      Block.Area:=AArea;

      if CheckWildcard(Name, Mask) and IsOperable(AArea) and IsSubscribeable(Link, AArea) then
       begin
        Inc(Block.Changes);

        LinkArea:=jt.link.SeekArea(Link, AArea);

        if LinkArea = nil then
         begin
          LinkArea:=jt.link.CreateLinkArea(Link, AArea);

          jt.link.area.SetReadable(LinkArea, True);
          jt.link.area.SetWriteable(LinkArea, True);
          jt.link.area.SetSubscribed(LinkArea, False);
         end;

        if not (jt.link.area.IsReadable(LinkArea) or jt.link.area.IsWriteable(LinkArea)) then
         jt.msg.Add(Reply, '+' + Name + ': no any rights')
        else
         begin
          Rights:=jt.link.area.GetACS(LinkArea);

          if jt.link.area.IsSubscribed(LinkArea) and not jt.link.area.IsPassive(LinkArea) then
           begin
            jt.msg.Add(Reply, '+' + Name + ': already subscribed (' + Rights + ')');

            jt.LogWrite('A', '+' + Name + ': already subscribed (' + Rights + ')');
           end
          else
           begin
            jt.link.area.SetSubscribed(LinkArea, True);

            if jt.link.area.IsPassive(LinkArea) then
             begin
              jt.link.area.SetPassive(LinkArea, False);

              Rights:=jt.link.area.GetACS(LinkArea);

              jt.msg.Add(Reply, '+' + Name + ': resumed (' + Rights + ')');

              jt.LogWrite('A', '+' + Name + ': resumed (' + Rights + ')');

              Block.Area:=Area;

              Callbacks[aaSubscribeResumed]^.ChainedPointer(acSubscribeResumed, @Block, False);

              if Block.Mask = nil then
               Break;
             end
            else
             begin
              Callbacks[aaSubscribeSubscribed]^.ChainedPointer(acSubscribeSubscribed, @Block, False);

              Rights:=jt.link.area.GetACS(LinkArea);

              jt.msg.Add(Reply, '+' + Name + ': subscribed (' + Rights + ')');

              jt.LogWrite('A', '+' + Name + ': subscribed (' + Rights + ')');

              if Block.Mask = nil then
               Break;
             end;
           end;
         end;
       end;
     end;

    if Block.Mask <> nil then
     begin
      Callbacks[aaSubscribeAfter]^.ChainedPointer(acSubscribeAfter, @Block, False);

      if Block.Mask <> nil then
       if Block.Changes = 0 then
        begin
         jt.msg.Add(Reply, '+' + Mask + ': nothing to subscribe');

         jt.LogWrite('A', '+' + Mask + ': nothing to subscribe');
        end
       else
        jt.link.SetChanged(lcfChanged, 'areafix sub');
     end;
   end;
 end;

procedure TAreafixParser.PerformUnsubscribe(Mask: String);
 var
  Reply, AArea, LinkArea: Pointer;
  Name, Rights: String;
  Created: Boolean;
  Block: TAreafixUnsubscribeBlock;
  K: Longint;
 begin
  Mask:=ExtractWord(1, Mask, [#32, #9]);

  TrimEx(Mask);

  if Mask = '' then
   Exit;

  StUpcaseEx(Mask);

  Reply:=CreateReplyEx('SubUnsub', Created);

  if Created then
   begin
    jt.msg.SetSubject(Reply, 'subscribe/unsubscribe results');

    FixEmptyReply(Reply);
   end;

  Block.Mask:=@Mask;
  Block.Changes:=0;
  Block.Reply:=Reply;

  Callbacks[aaUnsubscribeBefore]^.ChainedPointer(acUnsubscribeBefore, @Block, False);

  if Block.Mask <> nil then
   begin
    for K:=0 to jt.link.area.Count(Link) - 1 do
     begin
      LinkArea:=jt.link.area.At(Link, K);

      AArea:=jt.link.area.GetArea(LinkArea);

      Block.Area:=AArea;

      GetPStringEx(jt.area.GetName(AArea), Name);

      if CheckWildcard(Name, Mask) and jt.link.area.IsSubscribed(LinkArea) and IsUnsubscribeable(Link, AArea) then
       begin
        Inc(Block.Changes);

        jt.link.area.SetSubscribed(LinkArea, False);
        jt.link.area.SetPassive(LinkArea, False);

        Callbacks[aaUnsubscribeUnsubscribed]^.ChainedPointer(acUnsubscribeUnsubscribed, @Block, False);

        Rights:=jt.link.area.GetACS(LinkArea);

        jt.msg.Add(Reply, '-' + Name + ': unsubscribed (' + Rights + ')');

        jt.LogWrite('A', '-' + Name + ' (' + Rights + ')');

        if Block.Mask = nil then
         Break;
       end;
     end;

    if Block.Mask <> nil then
     begin
      Callbacks[aaUnsubscribeAfter]^.ChainedPointer(acUnsubscribeAfter, @Block, False);

      if Block.Mask <> nil then
       if Block.Changes = 0 then
        begin
         jt.msg.Add(Reply, '-' + Mask + ': nothing to unsubscribe');

         jt.LogWrite('A', '-' + Mask + ': nothing to unsubscribe');
        end
       else
        jt.link.SetChanged(lcfChanged, 'areafix unsub');
     end;
   end;
 end;

procedure TAreafixParser.AddRecipient(const Address: TAddress);
 var
  Addresses: PAddresses;
 begin
  Addresses:=jt.st.Get(jt.msg.st(Thismsg), 'Areafix.Recipients');

  if Addresses = nil then
   begin
    Addresses:=New(PAddresses, Init($2, $2));

    jt.st.Add(jt.msg.st(Thismsg), 'Areafix.Recipients', Addresses, stAddresses);
   end;

  Addresses^.Add(Address);
 end;

procedure TAreafixParser.Command(const ALink: Pointer; const ACommand: String; const AReplyMode: TReplyMode);
 var
  msg, msgp: Pointer;
  FromAddress, ToAddress: TAddress;
 begin
  msg:=jt.msg.Create;

  jt.msg.SetFromName(msg, 'Sysop');
  jt.msg.SetToName(msg, afxName);
  jt.msg.SetSubject(msg, jt.linker.UnparseString(jt.linker.StubSoft,
    GetPString(jt.st.Get(jt.link.st(ALink), 'Areafix.Password'))));

  jt.link.GetAddress(ALink, FromAddress);
  jt.link.GetAka(ALink, ToAddress);

  msgp:=jt.msg.CreateParser(msg, True);

  jt.msg.p.SetFromAndToAddress(msgp, FromAddress, ToAddress, True);

  jt.msg.DestroyParser(msgp);

  jt.msg.Add(msg, ACommand);

  ReplyMode:=AReplyMode;

  Process(msg);

  ReplyMode:=rmNormal;

  jt.msg.Destroy(msg);
 end;

{ NumberToToDo }

procedure NumberToToDo(const Number: Longint; var ToDo: TMParserToDo);
 begin
  if Number = 0 then ToDo:=tdNothing else
  if Number = 1 then ToDo:=tdKill else
  if Number = 2 then ToDo:=tdStore;
 end;

{ NumberToReplyMode }

procedure NumberToReplyMode(const Number: Longint; var ReplyMode: TReplyMode);
 begin
  if Number = 0 then ReplyMode:=rmNormal else
  if Number = 1 then ReplyMode:=rmHand else
  if Number = 2 then ReplyMode:=rmHandLog else
  if Number = 3 then ReplyMode:=rmHandQuiet;
 end;

end.