unit afxHelp;

interface
uses
     Objects,

     PlugAPI,
     PlugSIDE,

     jtMHLcmn,

     afxCmn,
     afxAPI,

     stpl,

     Wizard;

type
 PHelpTemplate = ^THelpTemplate;
 THelpTemplate = object(TSimpleTemplate)
  procedure Error(const Message: String); virtual;
  function IsOperableCommand(const Command: String): Boolean; virtual;
  function IsPendingCommand(const Command: String): Boolean; virtual;
  procedure PerformOperableCommand(const Command: String; Value: String); virtual;
 end;

procedure afxHelpHandler(const Command: String; Value: String); export;

implementation

{ THelpTemplate }

procedure THelpTemplate.Error(const Message: String);
 begin
  inherited Error(Message);

  jt.Report(afxHelpTemplate + ', line ' + Long2Str(CurrentLine) + ': ' + Message, True);
 end;

function THelpTemplate.IsOperableCommand(const Command: String): Boolean;
 begin
  IsOperableCommand:=(Command = 'IFLOADED') or
                     (Command = 'IFNOTLOADED');
 end;

function THelpTemplate.IsPendingCommand(const Command: String): Boolean;
 begin
  IsPendingCommand:=(Command = 'ENDIFLOADED') or
                    (Command = 'ENDIFNOTLOADED');
 end;

procedure THelpTemplate.PerformOperableCommand(const Command: String; Value: String);
 var
  Ok: Boolean;
 begin
  TrimEx(Value);

  if Command = 'IFLOADED' then
   begin
    Ok:=jt.plugs.Seek(Value) = nil;

    AddPendingCommand('ENDIFLOADED', Ok, Ok);
   end else
  if Command = 'IFNOTLOADED' then
   begin
    Ok:=jt.plugs.Seek(Value) <> nil;

    AddPendingCommand('ENDIFNOTLOADED', Ok, Ok);
   end;
 end;

{ afxHelpHandler }

procedure afxHelpHandler(const Command: String; Value: String);
 var
  Reply: Pointer;
  Created: Boolean;
  Info: TAreafixGetRequestInfoBlock;
  Source, Target: PStrings;
  Template: PHelpTemplate;
  F: Text;
  S: String;
 procedure Transform(const Line: PString); far;
  begin
   GetPStringEx(Line, S);

   if Pos('@', S) > 0 then
    begin
     ReplaceEx(S, '@sendername', Info.ReplyToName);
     ReplaceEx(S, '@senderaddress', AddressToStrEx(Info.ReplyToAddress));
     ReplaceEx(S, '@recipientname', Info.OurName);
     ReplaceEx(S, '@recipientaddress', AddressToStrEx(Info.OurAddress));
    end;

   jt.msg.Add(Reply, S);
  end;
 begin
  Reply:=afxCreateReply('Help.' + GenerateMSGID, Created);

  jt.msg.SetSubject(Reply, '%HELP response');

  afxFixEmptyReply(Reply);

  afxGetRequestInfo(Info);

  if IOResult <> 0 then;

  Assign(F, afxHelpTemplate);
  Reset(F);

  if IOResult <> 0 then
   begin
    jt.msg.Add(Reply, 'unable to send to you help cause of internal error');

    jt.Report('Unable to open ' + afxHelpTemplate, True);

    Exit;
   end;

  Source:=New(PStrings, Init($10, $10));

  while not Eof(F) do
   begin
    ReadLn(F, S);

    Source^.Insert(NewStr(S));
   end;

  Close(F);

  Target:=New(PStrings, Init($10, $10));

  Template:=New(PHelpTemplate, Init(Source, Target));

  Template^.Process;

  if Template^.Stop then
   begin
    jt.msg.Add(Reply, 'unable to send to you help cause of internal error');

    Dispose(Template, Done);

    Dispose(Target, Done);

    Dispose(Source, Done);

    Exit;
   end;

  Dispose(Template, Done);

  Dispose(Source, Done);

  Target^.ForEach(@Transform);

  Dispose(Target, Done);
 end;

end.