unit afxInfo;

interface

const
 MaxLowBound            = 64 * 1024;

procedure afxInfoHandler(const Command: String; Value: String); export;
procedure afxSetPasswordHandler(const Command: String; Value: String); export;
procedure afxSetMaxSize(const Command: String; Value: String); export;
procedure afxSetArchiver(const Command: String; Value: String); export;

implementation
uses
     PlugAPI,
     PlugSIDE,

     afxAPI,
     afxCmn,

     jtMHLcmn,

     Wizard;

{ afxInfoHandler }

procedure afxInfoHandler(const Command: String; Value: String);
 var
  Reply: Pointer;
  Created: Boolean;
  Block: TAreafixGetRequestInfoBlock;
  Address: TAddress;
  S: String;
  K: Longint;
 begin
  Reply:=afxCreateReply('Info.' + GenerateMSGID, Created);

  if Created then
   begin
    jt.msg.SetSubject(Reply, 'link information');

    afxFixEmptyReply(Reply);
   end;

  afxGetRequestInfo(Block);

  jt.link.GetAddress(Block.Link, Address);

  jt.msg.Add(Reply, 'Link address .................... ' + AddressToStrEx(Address));

  jt.link.GetAka(Block.Link, Address);

  jt.msg.Add(Reply, 'Our aka ......................... ' + AddressToStrEx(Address));

  jt.msg.Add(Reply, '');

  jt.link.GetInPassword(Block.Link, S);

  jt.msg.Add(Reply, 'In password ..................... ''' + S + '''');

  jt.link.GetOutPassword(Block.Link, S);

  jt.msg.Add(Reply, 'Out password .................... ''' + S + '''');

  GetPStringEx(jt.st.Get(jt.link.st(Block.Link), 'Areafix.Password'), S);

  jt.msg.Add(Reply, 'Areafix password ................ ''' + S + '''');

  jt.msg.Add(Reply, '');

  jt.msg.Add(Reply, 'Max packet size ................. ' + Size2KB(jt.link.GetMaxPacketSize(Block.Link)));
  jt.msg.Add(Reply, 'Max bundle size ................. ' + Size2KB(jt.link.GetMaxBundleSize(Block.Link)));

  jt.msg.Add(Reply, '');

  jt.msg.Add(Reply, 'Archiver ........................ ' + jt.arc.GetName(jt.link.GetArchiver(Block.Link)));

  jt.msg.Add(Reply, '');

  jt.msg.Add(Reply, 'Valid archivers are:');

  S:='';

  for K:=0 to jt.arc.Count - 1 do
   S:=Concat(S, jt.arc.GetName(jt.arc.At(K)), ' ');

  if S <> '' then
   Dec(S[0])
  else
   S:='no valid archivers';

  jt.msg.Add(Reply, S);
 end;

{ afxSetPasswordHandler }

procedure afxSetPasswordHandler(const Command: String; Value: String);
 var
  Linker, Restriction: Pointer;
  Password: String;
  Created: Boolean;
  Address: TAddress;
  Block: TAreafixGetRequestInfoBlock;
 begin
  afxGetRequestInfo(Block);

  if Command = 'SETINP' then Restriction:=restNoSETINP else
  if Command = 'SETOUTP' then Restriction:=restNoSETOUTP else
  if Command = 'SETAFXP' then Restriction:=restNoSETAFXP;

  jt.link.GetAddress(Block.Link, Address);

  if jt.rests.Check(Restriction, @Address, nil) then
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%' + Command + ': you have no rights to execute this command');

    jt.LogWrite('A', 'Prohibited');

    Exit;
   end;

  Linker:=jt.linker.v.Create;

  Password:=jt.linker.ParseStringToken(Linker, Value);

  if jt.linker.v.WasError(Linker) then
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%' + Command + ': ' + jt.linker.v.GetError(Linker));

    jt.LogWrite('A', '%' + Command + ': ' + jt.linker.v.GetError(Linker));

    jt.linker.v.Destroy(Linker);

    Exit;
   end;

  jt.linker.v.Destroy(Linker);

  if Command = 'SETINP' then
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%' + Command + ': password changed');

    jt.LogWrite('A', '%' + Trim(Command) + ': password changed to "' + Password + '"');

    jt.link.SetInPassword(Block.Link, Password);

    jt.link.SetChanged(lcfChanged, 'afx setinp');
   end else
  if Command = 'SETOUTP' then
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%' + Command + ': password changed');

    jt.LogWrite('A', '%' + Trim(Command) + ': password changed to "' + Password + '"');

    jt.link.SetOutPassword(Block.Link, Password);

    jt.link.SetChanged(lcfChanged, 'afx setoutp');
   end else
  if Command = 'SETAFXP' then
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%' + Command + ': password changed');

    jt.LogWrite('A', '%' + Trim(Command) + ': password changed to "' + Password + '"');

    jt.st.Add(jt.link.st(Block.Link), 'Areafix.Password', jt.NewStr(Password), jt.svc.stString);

    jt.link.SetChanged(lcfChanged, 'afx setafxp');
   end;
 end;

{ afxSetMaxSize }

procedure afxSetMaxSize(const Command: String; Value: String);
 var
  Created: Boolean;
  Linker, Restriction: Pointer;
  Size: Longint;
  Address: TAddress;
  Block: TAreafixGetRequestInfoBlock;
 begin
  afxGetRequestInfo(Block);

  Linker:=jt.linker.v.Create;

  Size:=jt.linker.ParseSize(Linker, Value);

  if jt.linker.v.WasError(Linker) then
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%' + Command + ': ' + jt.linker.v.GetError(Linker));

    jt.LogWrite('A', '%' + Command + ': ' + jt.linker.v.GetError(Linker));

    jt.linker.v.Destroy(Linker);

    Exit;
   end;

  jt.linker.v.Destroy(Linker);

  if Command = 'SETMPS' then Restriction:=restNoSETMPS else
  if Command = 'SETMBS' then Restriction:=restNoSETMBS;

  jt.link.GetAddress(Block.Link, Address);

  if jt.rests.Check(Restriction, @Address, nil) then
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%' + Command + ': you have no rights to execute this command');

    jt.LogWrite('A', 'Prohibited');

    Exit;
   end;

  if Size < MaxLowBound then
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%' + Command + ': too low value');

    jt.LogWrite('A', 'Too low value (' + Long2StrFmt(Size) + ')');

    Exit;
   end;

  if Command = 'SETMPS' then
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%' + Command + ': max packet size set to ' + Long2StrFmt(Size) + ' bytes');

    jt.LogWrite('A', '%' + Trim(Command) + ': max packet size set to ' + Long2StrFmt(Size) + ' bytes');

    jt.link.SetMaxPacketSize(Block.Link, Size);

    jt.link.SetChanged(lcfChanged, 'afx setmps');
   end else
  if Command = 'SETMBS' then
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%' + Command + ': max bundle size set to ' + Long2StrFmt(Size) + ' bytes');

    jt.LogWrite('A', '%' + Trim(Command) + ': max bundle size set to ' + Long2StrFmt(Size) + ' bytes');

    jt.link.SetMaxBundleSize(Block.Link, Size);

    jt.link.SetChanged(lcfChanged, 'afx setmbs');
   end;
 end;

{ afxSetArchiver }

procedure afxSetArchiver(const Command: String; Value: String);
 var
  Block: TAreafixGetRequestInfoBlock;
  Linker, Archiver: Pointer;
  ArchiverName: String;
  Created: Boolean;
 begin
  afxGetRequestInfo(Block);

  Linker:=jt.linker.v.Create;

  ArchiverName:=jt.linker.ParseStringToken(Linker, Value);

  if jt.linker.v.WasError(Linker) then
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%' + Command + ': ' + jt.linker.v.GetError(Linker));

    jt.LogWrite('A', '%' + Command + ': ' + jt.linker.v.GetError(Linker));

    jt.linker.v.Destroy(Linker);

    Exit;
   end;

  jt.linker.v.Destroy(Linker);

  Archiver:=jt.arc.Seek(ArchiverName);

  if Archiver = nil then
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%' + Command + ': unknown archiver "' + ArchiverName + '"');

    jt.LogWrite('A', 'Unknown archiver "' + ArchiverName + '"');
   end
  else
   begin
    jt.link.SetArchiver(Block.Link, Archiver);

    jt.msg.Add(afxCreateReply('Common', Created), '%' + Command + ': archiver changed to "' + ArchiverName + '"');

    jt.LogWrite('A', 'Archiver changed to "' + ArchiverName + '"');

    jt.link.SetChanged(lcfChanged, 'afx setarc');
   end;
 end;

end.