unit afxLists;

interface
uses
     PlugAPI,
     PlugSIDE,

     Objects,

     afxAPI,
     afxCmn,
     afxRight,

     jtMHLcmn,

     Columns,

     Wizard;

const
 colListAreaName            = 36;
 colListAreaNameWrapped     = 3;
 colListAccess              = 4;
 colListDescription         = 37;

 colMaxWidth                = 79;

 FlushLimit                 = 128;

type
 PAreasGroups = ^TAreasGroups;

 PFormatter = ^TFormatter;
 TFormatter = object(TObject)
  Target: PStrings;
  Groups: PAreasGroups;

  constructor Init(const ATarget: PStrings; const AGroups: PAreasGroups);
  procedure BeforeAll; virtual;
  procedure BeforeGroup(const Group: Char; const Count: Longint); virtual;
  procedure DoArea(const Area: Pointer); virtual;
  procedure AfterGroup; virtual;
  procedure AfterAll; virtual;
 end;

 PListFullFormatter = ^TListFullFormatter;
 TListFullFormatter = object(TFormatter)
  Rows: PColumnRows;
  Total: Longint;
  Action: PString;

  constructor Init(const ATarget: PStrings; const AGroups: PAreasGroups; const AAction: String);
  procedure BeforeAll; virtual;
  procedure BeforeGroup(const Group: Char; const Count: Longint); virtual;
  procedure DoArea(const Area: Pointer); virtual;
  procedure AfterAll; virtual;
  destructor Done; virtual;
 private
  procedure Flush;
 end;

 PListPureFormatter = ^TListPureFormatter;
 TListPureFormatter = object(TFormatter)
  WriteHeader: Boolean;
  Total: Longint;
  Action: PString;

  constructor Init(const ATarget: PStrings; const AGroups: PAreasGroups; const AWriteHeader: Boolean; const AAction: String);
  procedure BeforeAll; virtual;
  procedure BeforeGroup(const Group: Char; const Count: Longint); virtual;
  procedure DoArea(const Area: Pointer); virtual;
  procedure AfterAll; virtual;
  destructor Done; virtual;
 end;

 PAreasGroup = ^TAreasGroup;
 TAreasGroup = object(TStringCollection)
  Group: Char;

  constructor Init(const AGroup: Char);
  procedure FreeItem(Item: Pointer); virtual;
  function KeyOf(Item: Pointer): Pointer; virtual;
 end;

 TAreasGroups = object(TSortedCollection)
  Block: TAreafixGetRequestInfoBlock;
  DivideToGroups: Boolean;

  constructor Init(const ADivideToGroups: Boolean);
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  function KeyOf(Item: Pointer): Pointer; virtual;
  procedure AddArea(const Area: Pointer); virtual;
  procedure RemoveArea(const Area: Pointer); virtual;
  procedure FillAreas; virtual;
  procedure Format(const Formatter: PFormatter); virtual;
 end;

 PLinkedAreasGroups = ^TLinkedAreasGroups;
 TLinkedAreasGroups = object(TAreasGroups)
  procedure FillAreas; virtual;
 end;

 PUnlinkedAreasGroups = ^TUnlinkedAreasGroups;
 TUnlinkedAreasGroups = object(TAreasGroups)
  procedure FillAreas; virtual;
 end;

procedure afxListHandler(const Command: String; Value: String); export;
procedure afxFListHandler(const Command: String; Value: String); export;
procedure afxLinkedHandler(const Command: String; Value: String); export;
procedure afxUnlinkedHandler(const Command: String; Value: String); export;

implementation

const
 DefaultGroup           : Char          = #0;

{ TFormatter }

constructor TFormatter.Init(const ATarget: PStrings; const AGroups: PAreasGroups);
 begin
  inherited Init;

  Target:=ATarget;
  Groups:=AGroups;
 end;

procedure TFormatter.BeforeAll;
 begin
 end;

procedure TFormatter.BeforeGroup(const Group: Char; const Count: Longint);
 begin
 end;

procedure TFormatter.DoArea(const Area: Pointer);
 begin
 end;

procedure TFormatter.AfterGroup;
 begin
 end;

procedure TFormatter.AfterAll;
 begin
 end;

{ TListFullFormatter }

constructor TListFullFormatter.Init(const ATarget: PStrings; const AGroups: PAreasGroups; const AAction: String);
 begin
  inherited Init(ATarget, AGroups);

  Action:=NewStr(AAction);
 end;

procedure TListFullFormatter.BeforeAll;
 var
  Address: TAddress;
 begin
  Rows:=New(PColumnRows, Init($100, $100));

  Total:=0;

  jt.link.GetAddress(Groups^.Block.Link, Address);

  Rows^.Add^.Insert(New(PColumn, Init(0, 0, colMaxWidth, caLeft, 'The following areas ' + Action^ + ' to ' +
   AddressToStrEx(Address) + ' on ' + AddressToStrEx(Groups^.Block.OurAddress) + ':')));
 end;

procedure TListFullFormatter.BeforeGroup(const Group: Char; const Count: Longint);
 var
  Title: PAreasGroupTitle;
  Index: xInteger;
  S: String;
 begin
  if Count > 0 then
   begin
    Rows^.Add^.Insert(New(PColumn, Init(0, 0, colMaxWidth, caLeft, '')));

    if AreasGroupTitles^.Search(@Group, Index) then
     begin
      Title:=AreasGroupTitles^.At(Index);

      if Title^.Title <> nil then
       S:=Title^.Title^
      else
       S:='';
     end
    else
     S:='';

    if S <> '' then
     begin
      Rows^.Add^.Insert(New(PColumn, Init(0, 0, colMaxWidth, caLeft, Title^.Title^ + ' (' + Long2Str(Count) + ' area' +
       _s(Count) + ')')));

      Rows^.Add^.Insert(New(PColumn, Init(0, 0, colMaxWidth, caLeft, '')));
     end
    else
     if Groups^.Count > 1 then
      begin
       Rows^.Add^.Insert(New(PColumn, Init(0, 0, colMaxWidth, caLeft, Long2Str(Count) + ' area' + _s(Count))));

       Rows^.Add^.Insert(New(PColumn, Init(0, 0, colMaxWidth, caLeft, '')));
      end;

    Inc(Total, Count);
   end;
 end;

procedure TListFullFormatter.DoArea(const Area: Pointer);
 var
  Name, Rights: String;
  LinkArea: Pointer;
  Row: PColumnRow;
 begin
  Row:=Rows^.Add;

  GetPStringEx(jt.area.GetName(Area), Name);

  LinkArea:=jt.link.SeekArea(Groups^.Block.Link, Area);

  if (LinkArea = nil) or not jt.link.area.IsSubscribed(LinkArea) then
   begin
    if IsSubscribeable(Groups^.Block.Link, Area) then Rights:='S' else Rights:='-';
    if IsUnsubscribeable(Groups^.Block.Link, Area) then Rights:=Rights + 'U' else Rights:=Rights + '-';
   end
  else
   begin
    Rights:='*';

    if jt.link.area.IsReadable(LinkArea) then Rights:=Rights + 'R' else Rights:=Rights + '-';
    if jt.link.area.IsWriteable(LinkArea) then Rights:=Rights + 'W' else Rights:=Rights + '-';
    if jt.link.area.IsPassive(LinkArea) then Rights:=Rights + 'P';
   end;

  Row^.Insert(New(PColumn, Init(0, colListAreaNameWrapped, colListAreaName, caLeft, Name)));
  Row^.Insert(New(PColumn, Init(1, 0, colListAccess, caRight, Rights)));
  Row^.Insert(New(PColumn, Init(1, 0, colListDescription, caLeft, GetAreaDescription(Area))));

  if Rows^.Count > FlushLimit then
   Flush;
 end;

procedure TListFullFormatter.AfterAll;
 begin
  Rows^.Add^.Insert(New(PColumn, Init(0, 0, colMaxWidth, caLeft, '')));
  Rows^.Add^.Insert(New(PColumn, Init(0, 0, colMaxWidth, caLeft, 'Total: ' + Long2Str(Total) + ' area' + _s(Total))));

  Flush;

  Dispose(Rows, Done);
 end;

destructor TListFullFormatter.Done;
 begin
  DisposeStr(Action);

  inherited Done;
 end;

{ private methods }

procedure TListFullFormatter.Flush;
 begin
  Rows^.FormatRows(Target);

  Rows^.FreeAll;
 end;

{ TListPureFormatter }

constructor TListPureFormatter.Init(const ATarget: PStrings; const AGroups: PAreasGroups; const AWriteHeader: Boolean;
  const AAction: String);
 begin
  inherited Init(ATarget, AGroups);

  WriteHeader:=AWriteHeader;

  Action:=NewStr(AAction);

  Total:=0;
 end;

procedure TListPureFormatter.BeforeAll;
 var
  Address: TAddress;
 begin
  if WriteHeader then
   begin
    jt.link.GetAddress(Groups^.Block.Link, Address);

    Target^.Insert(NewStr('The following areas ' + Action^ + ' to ' + AddressToStrEx(Address) +
     ' on ' + AddressToStrEx(Groups^.Block.OurAddress) + ':'));

    Target^.Insert(nil);
   end;
 end;

procedure TListPureFormatter.BeforeGroup(const Group: Char; const Count: Longint);
 begin
  Inc(Total, Count);
 end;

procedure TListPureFormatter.DoArea(const Area: Pointer);
 begin
  Target^.Insert(NewStr(jt.area.GetName(Area)^));
 end;

procedure TListPureFormatter.AfterAll;
 begin
  if WriteHeader then
   begin
    if Total > 0 then
     Target^.Insert(nil);

    Target^.Insert(NewStr('Total: ' + Long2Str(Total) + ' area' + _s(Total)));
   end;
 end;

destructor TListPureFormatter.Done;
 begin
  DisposeStr(Action);

  inherited Done;
 end;

{ TAreasGroup }

constructor TAreasGroup.Init(const AGroup: Char);
 begin
  inherited Init($100, $100);

  Group:=AGroup;
 end;

procedure TAreasGroup.FreeItem(Item: Pointer);
 begin
 end;

function TAreasGroup.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=jt.area.GetName(Item);
 end;

{ TAreasGroups }

constructor TAreasGroups.Init(const ADivideToGroups: Boolean);
 begin
  inherited Init($10, $10);

  DivideToGroups:=ADivideToGroups;

  afxGetRequestInfo(Block);
 end;

function TAreasGroups.Compare(Key1, Key2: Pointer): xInteger;
 var
  K1: PChar absolute Key1;
  K2: PChar absolute Key2;
 begin
  if K1[0] < K2[0] then Compare:=-1 else
  if K1[0] > K2[0] then Compare:=1 else
   Compare:=0;
 end;

function TAreasGroups.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=@PAreasGroup(Item)^.Group;
 end;

procedure TAreasGroups.AddArea(const Area: Pointer);
 var
  AreasGroup: PAreasGroup;
  Index: xInteger;
  Group: Char;
  st: Pointer;
 begin
  if IsOperable(Area) and IsShowable(Block.Link, Area) then
   begin
    if DivideToGroups then
     begin
      jt.rests.Getst(restGroup, nil, jt.area.GetName(Area), st);

      if st = nil then
       Group:=DefaultGroup
      else
       Group:=Char(jt.st.Get(st, 'Group'));
     end
    else
     Group:=DefaultGroup;

    if Search(@Group, Index) then
     PAreasGroup(At(Index))^.Insert(Area)
    else
     begin
      AreasGroup:=New(PAreasGroup, Init(Group));

      AtInsert(Index, AreasGroup);

      AreasGroup^.Insert(Area);
     end;
   end;
 end;

procedure TAreasGroups.RemoveArea(const Area: Pointer);
 function CheckGroup(const Group: PAreasGroup): Boolean; far;
  var
   Index: xInteger;
  begin
   Index:=Group^.IndexOf(Area);

   if Index <> -1 then
    Group^.AtFree(Index);

   CheckGroup:=Index <> -1;
  end;
 begin
  FirstThat(@CheckGroup);
 end;

procedure TAreasGroups.FillAreas;
 var
  K: Longint;
 begin
  for K:=0 to jt.area.Count - 1 do
   AddArea(jt.area.At(K));
 end;

procedure TAreasGroups.Format(const Formatter: PFormatter);
 procedure FormatGroup(const Group: PAreasGroup); far;
  procedure FormatArea(const Area: Pointer); far;
   begin
    Formatter^.DoArea(Area);
   end;
  begin
   Formatter^.BeforeGroup(Group^.Group, Group^.Count);

   Group^.ForEach(@FormatArea);

   Formatter^.AfterGroup;
  end;
 begin
  Formatter^.BeforeAll;

  ForEach(@FormatGroup);

  Formatter^.AfterAll;

  Dispose(Formatter, Done);
 end;

{ TLinkedAreasGroups }

procedure TLinkedAreasGroups.FillAreas;
 var
  K: Longint;
  LinkArea: Pointer;
 begin
  for K:=0 to jt.link.area.Count(Block.Link) - 1 do
   begin
    LinkArea:=jt.link.area.At(Block.Link, K);

    if jt.link.area.IsSubscribed(LinkArea) then
     AddArea(jt.link.area.GetArea(LinkArea));
   end;
 end;

{ TUnlinkedAreasGroups }

procedure TUnlinkedAreasGroups.FillAreas;
 var
  K: Longint;
  LinkArea: Pointer;
 begin
  inherited FillAreas;

  for K:=0 to jt.link.area.Count(Block.Link) - 1 do
   begin
    LinkArea:=jt.link.area.At(Block.Link, K);

    if jt.link.area.IsSubscribed(LinkArea) then
     RemoveArea(jt.link.area.GetArea(LinkArea));
   end;
 end;

{ afxListHandler }

procedure afxListHandler(const Command: String; Value: String);
 var
  Groups: PAreasGroups;
  Reply: Pointer;
  Created: Boolean;
  List: PStrings;
  Mode: (lmPure, lmFull);
 procedure Transform(const Line: PString); far;
  begin
   if Line = nil then
    jt.msg.Add(Reply, '')
   else
    jt.msg.Add(Reply, RTrim(Line^));
  end;
 begin
  TrimEx(Value);

  StUpcaseEx(Value);

  if Value = 'PURE' then Mode:=lmPure else
  if Value = 'FULL' then Mode:=lmFull else
  if Value = '' then Mode:=lmFull else
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%LIST ' + Value + ': invalid switch; PURE, FULL or nothing expected');

    Exit;
   end;

  Groups:=New(PAreasGroups, Init(Mode = lmFull));

  if not AllowLIST(Groups^.Block.Link) then
   begin
    jt.LogWrite('A', 'Prohibited');

    jt.msg.Add(afxCreateReply('Common', Created), Trim('%LIST ' + Value) + ': you have no rights to execute this command');

    Dispose(Groups, Done);

    Exit;
   end;

  Groups^.FillAreas;

  Reply:=afxCreateReply('List.' + GenerateMSGID, Created);

  if Created then
   begin
    case Mode of
     lmPure: jt.msg.SetSubject(Reply, '%LIST PURE response');
     lmFull: jt.msg.SetSubject(Reply, '%LIST FULL response');
    end;

    afxFixEmptyReply(Reply);
   end;

  List:=New(PStrings, Init($100, $1000));

  case Mode of
   lmFull: Groups^.Format(New(PListFullFormatter, Init(List, Groups, 'available')));
   lmPure: Groups^.Format(New(PListPureFormatter, Init(List, Groups, True, 'available')));
  end;

  List^.ForEach(@Transform);

  Dispose(List, Done);

  Dispose(Groups, Done);
 end;

{ afxFListHandler }

procedure afxFListHandler(const Command: String; Value: String);
 var
  Groups: PAreasGroups;
  Reply, msgp: Pointer;
  Created: Boolean;
  List: PStrings;
  Mode: (lmPure, lmFull);
  FName: String;
  F: Text;
 procedure Transform(const S: PString); far;
  begin
   if S = nil then
    WriteLn(F)
   else
    WriteLn(F, RTrim(S^));
  end;
 begin
  TrimEx(Value);

  StUpcaseEx(Value);

  if Value = 'PURE' then Mode:=lmPure else
  if Value = 'FULL' then Mode:=lmFull else
  if Value = '' then Mode:=lmPure else
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%FLIST ' + Value + ': invalid switch; PURE, FULL or nothing expected');

    Exit;
   end;

  repeat
   FName:=jt.core.GetTempAttaches + GenerateMSGID + '.lst';
  until not ExistFile(FName);

  if IOResult <> 0 then;

  Assign(F, FName);
  Rewrite(F);

  if InOutRes <> 0 then
   begin
    jt.Report('Failed to create ' + FName + ', rc=' + Long2Str(IOResult), True);

    jt.msg.Add(afxCreateReply('Common', Created), 'Unable to execute %FLIST cause of internal error');

    Exit;
   end;

  Groups:=New(PAreasGroups, Init(Mode = lmFull));

  if not AllowFLIST(Groups^.Block.Link) then
   begin
    jt.LogWrite('A', 'Prohibited');

    jt.msg.Add(afxCreateReply('Common', Created), Trim('%FLIST ' + Value) + ': you have no rights to execute this command');

    Dispose(Groups, Done);

    Erase(F);

    if IOResult <> 0 then;

    Exit;
   end;

  Groups^.FillAreas;

  Reply:=afxCreateReply('FList.' + GenerateMSGID, Created);

  afxFixEmptyReply(Reply);

  jt.msg.SetAttributes(Reply, jt.msg.GetAttributes(Reply) or maAttach);

  jt.msg.SetSubject(Reply, FName);

  msgp:=jt.msg.CreateParser(Reply, True);

  jt.msg.p.AddKludge(msgp, #1'FLAGS KFS');

  jt.msg.DestroyParser(msgp);

  List:=New(PStrings, Init($100, $1000));

  case Mode of
   lmFull: Groups^.Format(New(PListFullFormatter, Init(List, Groups, 'available')));
   lmPure: Groups^.Format(New(PListPureFormatter, Init(List, Groups, False, 'available')));
  end;

  List^.ForEach(@Transform);

  Close(F);

  Dispose(List, Done);

  Dispose(Groups, Done);

  jt.msg.Add(afxCreateReply('Common', Created), Trim('%FLIST ' + Value) + ': ' + JustFileName(FName) + ' attached');

  jt.LogWrite('A', FName + ' attached');
 end;

{ afxLinkedHandler }

procedure afxLinkedHandler(const Command: String; Value: String);
 var
  Groups: PAreasGroups;
  Reply: Pointer;
  Created: Boolean;
  List: PStrings;
  Mode: (lmPure, lmFull);
 procedure Transform(const Line: PString); far;
  begin
   if Line = nil then
    jt.msg.Add(Reply, '')
   else
    jt.msg.Add(Reply, RTrim(Line^));
  end;
 begin
  TrimEx(Value);

  StUpcaseEx(Value);

  if Value = 'PURE' then Mode:=lmPure else
  if Value = 'FULL' then Mode:=lmFull else
  if Value = '' then Mode:=lmFull else
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%LINKED ' + Value + ': invalid switch; PURE, FULL or nothing expected');

    Exit;
   end;

  Groups:=New(PLinkedAreasGroups, Init(Mode = lmFull));

  if not AllowLINKED(Groups^.Block.Link) then
   begin
    jt.LogWrite('A', 'Prohibited');

    jt.msg.Add(afxCreateReply('Common', Created), Trim('%LINKED ' + Value) + ': you have no rights to execute this command');

    Dispose(Groups, Done);

    Exit;
   end;

  Groups^.FillAreas;

  Reply:=afxCreateReply('Linked.' + GenerateMSGID, Created);

  if Created then
   begin
    case Mode of
     lmPure: jt.msg.SetSubject(Reply, '%LINKED PURE response');
     lmFull: jt.msg.SetSubject(Reply, '%LINKED FULL response');
    end;

    afxFixEmptyReply(Reply);
   end;

  List:=New(PStrings, Init($100, $1000));

  case Mode of
   lmFull: Groups^.Format(New(PListFullFormatter, Init(List, Groups, 'linked')));
   lmPure: Groups^.Format(New(PListPureFormatter, Init(List, Groups, True, 'linked')));
  end;

  List^.ForEach(@Transform);

  Dispose(List, Done);

  Dispose(Groups, Done);
 end;

{ afxUnlinkedHandler }

procedure afxUnlinkedHandler(const Command: String; Value: String);
 var
  Groups: PAreasGroups;
  Reply: Pointer;
  Created: Boolean;
  List: PStrings;
  Mode: (lmPure, lmFull);
 procedure Transform(const Line: PString); far;
  begin
   if Line = nil then
    jt.msg.Add(Reply, '')
   else
    jt.msg.Add(Reply, RTrim(Line^));
  end;
 begin
  TrimEx(Value);

  StUpcaseEx(Value);

  if Value = 'PURE' then Mode:=lmPure else
  if Value = 'FULL' then Mode:=lmFull else
  if Value = '' then Mode:=lmFull else
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%UNLINKED ' + Value + ': invalid switch; PURE, FULL or nothing expected');

    Exit;
   end;

  Groups:=New(PUnlinkedAreasGroups, Init(Mode = lmFull));

  if not AllowUNLINKED(Groups^.Block.Link) then
   begin
    jt.LogWrite('A', 'Prohibited');

    jt.msg.Add(afxCreateReply('Common', Created), Trim('%UNLINKED ' + Value) + ': you have no rights to execute this command');

    Dispose(Groups, Done);

    Exit;
   end;

  Groups^.FillAreas;

  Reply:=afxCreateReply('Unlinked.' + GenerateMSGID, Created);

  if Created then
   begin
    case Mode of
     lmPure: jt.msg.SetSubject(Reply, '%UNLINKED PURE response');
     lmFull: jt.msg.SetSubject(Reply, '%UNLINKED FULL response');
    end;

    afxFixEmptyReply(Reply);
   end;

  List:=New(PStrings, Init($100, $1000));

  case Mode of
   lmFull: Groups^.Format(New(PListFullFormatter, Init(List, Groups, 'unlinked')));
   lmPure: Groups^.Format(New(PListPureFormatter, Init(List, Groups, True, 'unlinked')));
  end;

  List^.ForEach(@Transform);

  Dispose(List, Done);

  Dispose(Groups, Done);
 end;

end.