unit afxPause;

interface
uses
     PlugAPI,
     PlugSIDE,

     afxAPI,
     afxRight,

     Wizard;

procedure afxPauseHandler(const Command: String; Value: String); export;
procedure afxResumeHandler(const Command: String; Value: String); export;

implementation

{ afxPauseHandler }

procedure afxPauseHandler(const Command: String; Value: String);
 var
  Reply, LinkArea, Area: Pointer;
  K, Changes: Longint;
  Created: Boolean;
  Block: TAreafixGetRequestInfoBlock;
  Name, Rights: String;
 begin
  Value:=ExtractWord(1, Value, [#32, #9]);

  TrimEx(Value);

  if Value = '' then
   Value:='*'
  else
   StUpcaseEx(Value);

  afxGetRequestInfo(Block);

  Reply:=afxCreateReply('Pause/Resume', Created);

  if Created then
   begin
    jt.msg.SetSubject(Reply, 'pause/resume results');

    afxFixEmptyReply(Reply);
   end;

  Changes:=0;

  for K:=0 to jt.link.area.Count(Block.Link) - 1 do
   begin
    LinkArea:=jt.link.area.At(Block.Link, K);

    Area:=jt.link.area.GetArea(LinkArea);

    Name:=jt.area.GetName(Area)^;

    if CheckWildcard(Name, Value) and
       jt.link.area.IsSubscribed(LinkArea) and
       not jt.link.area.IsPassive(LinkArea) and
       AllowPAUSE(Block.Link, Area) then
     begin
      jt.link.area.SetPassive(LinkArea, True);

      Rights:=jt.link.area.GetACS(LinkArea);

      jt.msg.Add(Reply, '%PAUSE ' + Name + ': paused (' + Rights + ')');

      jt.LogWrite('A', '%PAUSE ' + Name + ': paused (' + Rights + ')');

      Inc(Changes);
     end;
   end;

  if Changes > 0 then
   jt.link.SetChanged(lcfChanged, 'afx %pause')
  else
   begin
    jt.msg.Add(Reply, '%PAUSE ' + Value + ': nothing to pause');

    jt.LogWrite('A', '%PAUSE ' + Value + ': nothing to pause');
   end;
 end;

{ afxResumeHandler }

procedure afxResumeHandler(const Command: String; Value: String);
 var
  Reply, LinkArea, Area: Pointer;
  K, Changes: Longint;
  Created: Boolean;
  Block: TAreafixGetRequestInfoBlock;
  Name, Rights: String;
 begin
  Value:=ExtractWord(1, Value, [#32, #9]);

  TrimEx(Value);

  if Value = '' then
   Value:='*'
  else
   StUpcaseEx(Value);

  afxGetRequestInfo(Block);

  Reply:=afxCreateReply('Pause/Resume', Created);

  if Created then
   begin
    jt.msg.SetSubject(Reply, 'pause/resume results');

    afxFixEmptyReply(Reply);
   end;

  Changes:=0;

  for K:=0 to jt.link.area.Count(Block.Link) - 1 do
   begin
    LinkArea:=jt.link.area.At(Block.Link, K);

    Area:=jt.link.area.GetArea(LinkArea);

    Name:=jt.area.GetName(Area)^;

    if CheckWildcard(Name, Value) and
       jt.link.area.IsSubscribed(LinkArea) and
       jt.link.area.IsPassive(LinkArea) and
       AllowRESUME(Block.Link, Area) then
     begin
      jt.link.area.SetPassive(LinkArea, False);

      Rights:=jt.link.area.GetACS(LinkArea);

      jt.msg.Add(Reply, '%RESUME ' + Name + ': resumed (' + Rights + ')');

      jt.LogWrite('A', '%RESUME ' + Name + ': resumed (' + Rights + ')');

      Inc(Changes);
     end;
   end;

  if Changes > 0 then
   jt.link.SetChanged(lcfChanged, 'afx %resume')
  else
   begin
    jt.msg.Add(Reply, '%RESUME ' + Value + ': nothing to resume');

    jt.LogWrite('A', '%RESUME ' + Value + ': nothing to resume');
   end;
 end;

end.