library Areafix;

uses
     PlugAPI,
     PlugSIDE,

     Objects,
     Strings,

     jtMHLcmn,

     TossAPI,

     MParser,

     afxAPI,
     afxCmn,
     afxDisp,
     afxCore,
     afxUtil,
     afxHelp,
     afxPause,
     afxLists,
     afxInfo,
     afxRight,

     ccAPI,

     Wizard;

const
 AreafixName                            = 'Areafix';
 AreafixVersion                         = 1;

 AreafixPerform                         = $1000;
 AreafixDown                            = $1001;

 AreafixPlugin          : Pointer       = nil;

 afNone                                 = $0000;
 afScan                                 = $0001;
 afHand                                 = $0002;

 AreafixFlags           : Longint       = afNone;

 ReplyMode              : TReplyMode    = rmNormal;

 HandreName             : String        = '';
 Handre                 : Pointer       = nil;
 HandCommand            : String        = '';

var
 HandAddress            : TAddress;
 HandLink               : Pointer;
 HandDid                : Longint;

{ CreateParser }

procedure CreateParser;
 begin
  if AreafixParser = nil then
   AreafixParser:=New(PAreafixParser, Init(afxScanArea));
 end;

{ DestroyParser }

procedure DestroyParser;
 begin
  if AreafixParser <> nil then
   begin
    Dispose(AreafixParser, Done);

    AreafixParser:=nil;
   end;
 end;

{ RegisterCallback }

procedure RegisterCallback(const Block: TRegisterAreafixCallbackBlock);
 begin
  Callbacks[Block.Action]^.Add(Block.Plugin);
 end;

{ NetmailMessageToss }

procedure NetmailMessageToss(var Block: TNetmailMessageTossBlock);
 begin
  CreateParser;

  if AreafixParser^.Applicable(Block.msg) then
   begin
    AreafixParser^.Todo:=tdNothing;

    AreafixParser^.Process(Block.msg);

    if AreafixParser^.ToDo = tdKill then
     Block.Done:=True;
   end;
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, AreafixName, AreafixVersion);

  AreafixPlugin:=Plugin;

  PluginSetup:=True;

  CommandHandlers:=New(PCommandHandlers, Init($10, $10));

  CommandHandlers^.Insert(New(PCommandHandler, Init('SUB', afxSubHandler)));
  CommandHandlers^.Insert(New(PCommandHandler, Init('UNSUB', afxUnsubHandler)));
  CommandHandlers^.Insert(New(PCommandHandler, Init('HELP', afxHelpHandler)));
  CommandHandlers^.Insert(New(PCommandHandler, Init('PAUSE', afxPauseHandler)));
  CommandHandlers^.Insert(New(PCommandHandler, Init('RESUME', afxResumeHandler)));
  CommandHandlers^.Insert(New(PCommandHandler, Init('LIST', afxListHandler)));
  CommandHandlers^.Insert(New(PCommandHandler, Init('LINKED', afxLinkedHandler)));
  CommandHandlers^.Insert(New(PCommandHandler, Init('QUERY', afxLinkedHandler)));
  CommandHandlers^.Insert(New(PCommandHandler, Init('UNLINKED', afxUnlinkedHandler)));
  CommandHandlers^.Insert(New(PCommandHandler, Init('FLIST', afxFListHandler)));
  CommandHandlers^.Insert(New(PCommandHandler, Init('INFO', afxInfoHandler)));
  CommandHandlers^.Insert(New(PCommandHandler, Init('SETINP', afxSetPasswordHandler)));
  CommandHandlers^.Insert(New(PCommandHandler, Init('SETOUTP', afxSetPasswordHandler)));
  CommandHandlers^.Insert(New(PCommandHandler, Init('SETAFXP', afxSetPasswordHandler)));
  CommandHandlers^.Insert(New(PCommandHandler, Init('SETMPS', afxSetMaxSize)));
  CommandHandlers^.Insert(New(PCommandHandler, Init('SETMBS', afxSetMaxSize)));
  CommandHandlers^.Insert(New(PCommandHandler, Init('SETARC', afxSetArchiver)));
  CommandHandlers^.Insert(New(PCommandHandler, Init('RECIPIENT', afxRecipient)));

  AreasGroupTitles:=New(PAreasGroupTitles, Init($10, $10));

  restGroup:=jt.rests.AddValueParser('afxGroup', rsValue or rsMinus or rsStorage, afxGroupValueParser);
  restDefaultDesc:=jt.rests.Add('afxDefaultdesc', rsValue or rsMinus);
  restNotOperable:=jt.rests.Add('afxNotOperable', rsMinus);
  restHide:=jt.rests.Add('afxHide', rsMinus);
  restNoSubscribe:=jt.rests.Add('afxNoSubscribe', rsMinus);
  restNoUnsubscribe:=jt.rests.Add('afxNoUnsubscribe', rsMinus);
  restNoPAUSE:=jt.rests.Add('afxNoPAUSE', rsMinus);
  restNoRESUME:=jt.rests.Add('afxNoRESUME', rsMinus);
  restNoLIST:=jt.rests.Add('afxNoLIST', rsMinus);
  restNoFLIST:=jt.rests.Add('afxNoFLIST', rsMinus);
  restNoLINKED:=jt.rests.Add('afxNoLINKED', rsMinus);
  restNoUNLINKED:=jt.rests.Add('afxNoUNLINKED', rsMinus);
  restNoSETINP:=jt.rests.Add('afxNoSETINP', rsMinus);
  restNoSETOUTP:=jt.rests.Add('afxNoSETOUTP', rsMinus);
  restNoSETAFXP:=jt.rests.Add('afxNoSETAFXP', rsMinus);
  restNoSETMPS:=jt.rests.Add('afxNoSETMPS', rsMinus);
  restNoSETMBS:=jt.rests.Add('afxNoSETMBS', rsMinus);
  restNoRECIPIENT:=jt.rests.Add('afxNoRECIPIENT', rsMinus);

  InitCCAPI;

  if ccAvailable then
   begin
    ccafBeforeProcess:=New(PccHook, Init('AFBEFOREPROCESS'));

    with ccafBeforeProcess^ do
     begin
      AddConstant('MSG', etNumber);
      AddConstant('REPLYTOADDRESS', etNumber);
      AddVariable('REPLYTONAME', etString);
      AddVariable('REPLYMODE', etNumber);
      AddVariable('TODO', etNumber);
      AddVariable('STOP', etBoolean);

      AddConstant('TDNOTHING', etNumber)^.SetNumber(ToDoToNumber[tdNothing]);
      AddConstant('TDKILL', etNumber)^.SetNumber(ToDoToNumber[tdKill]);
      AddConstant('TDSTORE', etNumber)^.SetNumber(ToDoToNumber[tdStore]);

      AddConstant('RMNORMAL', etNumber)^.SetNumber(ReplyModeToNumber[rmNormal]);
      AddConstant('RMHAND', etNumber)^.SetNumber(ReplyModeToNumber[rmHand]);
      AddConstant('RMHANDLOG', etNumber)^.SetNumber(ReplyModeToNumber[rmHandLog]);
      AddConstant('RMHANDQUIET', etNumber)^.SetNumber(ReplyModeToNumber[rmHandQuiet]);
     end;

    ccafProcessLine:=New(PccHook, Init('AFPROCESSLINE'));

    with ccafProcessLine^ do
     begin
      AddConstant('MSG', etNumber);
      AddConstant('MSGP', etNumber);
      AddConstant('LINK', etNumber);
      AddVariable('LINE', etString);
      AddVariable('FINISH', etBoolean);
     end;

    ccafPostReply:=New(PccHook, Init('AFPOSTREPLY'));

    with ccafPostReply^ do
     AddVariable('MSG', etNumber);
   end;
 end;

{ PluginStartup }

procedure PluginStartup; export;
 var
  Action: TAreafixAction;
 begin
  if (AreafixFlags <> afNone) and (afxScanArea <> nil) and (afxReplyToArea <> nil) then
   jt.EnqueueCommand(AreafixPlugin, AreafixPerform, 0, cwEarliest);

  jt.EnqueueCommand(AreafixPlugin, AreafixDown, 0, cwFlush);

  if afxCatchNetmail then
   RegisterTosserCallback(AreafixPlugin, taNetmailMessageTossLast);

  for Action:=Low(Action) to High(Action) do
   Callbacks[Action]:=New(PCallbacks, Init(StrPas(AreafixActionNames[Action])));
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 var
  Action: TAreafixAction;
 begin
  if ccAvailable then
   begin
    Dispose(ccafPostReply, Done);
    Dispose(ccafProcessLine, Done);
    Dispose(ccafBeforeProcess, Done);
   end;

  DisposeTosserCallbacks;

  if Handre <> nil then
   jt.re.Destroy(Handre);

  for Action:=Low(Action) to High(Action) do
   Dispose(Callbacks[Action], Done);

  Dispose(CommandHandlers, Done);
  Dispose(AreasGroupTitles, Done);

  jt.DestroyRegExp(afxRespondTo);
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 var
  Linker: Pointer absolute Param;
  AreaBlock: PafxIsAreaBlock absolute Param;
  LinkAreaBlock: PafxIsLinkAreaBlock absolute Param;
  GetAreaDescriptionBlock: PafxGetAreaDescriptionBlock absolute Param;
 procedure RegisterCommandHandler(const Block: TAreafixRegisterCommandHandlerBlock);
  var
   Index: xInteger;
  begin
   if (Block.Command^ <> '') and not CommandHandlers^.Search(Block.Command, Index) then
    CommandHandlers^.AtInsert(Index, New(PCommandHandler, Init(Block.Command^, Block.Handler)));
  end;
 procedure CreateReply(var Block: TAreafixCreateReplyBlock);
  begin
   Block.Reply:=AreafixParser^.CreateReplyEx(Block.Name^, Block.Created);
  end;
 procedure GetRequestInfo(var Block: TAreafixGetRequestInfoBlock);
  begin
   Block.ReplyToName:=AreafixParser^.ReplyToName;
   Block.ReplyToAddress:=AreafixParser^.ReplyToAddress;
   Block.OurName:=afxName;

   jt.GetNetmailAka(Block.ReplyToAddress, Block.OurAddress);

   Block.Link:=AreafixParser^.Link;
   Block.Area:=AreafixParser^.Area;
   Block.No:=AreafixParser^.No;
  end;
 var
  K: Longint;
 begin
  if Command = afxcIsOperable then AreaBlock^.Yes:=IsOperable(AreaBlock^.Area) else
  if Command = afxcIsShowable then LinkAreaBlock^.Yes:=IsShowable(LinkAreaBlock^.Link, LinkAreaBlock^.Area) else
  if Command = afxcIsSubscribeable then LinkAreaBlock^.Yes:=IsSubscribeable(LinkAreaBlock^.Link, LinkAreaBlock^.Area) else
  if Command = afxcIsUnsubscribeable then LinkAreaBlock^.Yes:=IsUnsubscribeable(LinkAreaBlock^.Link, LinkAreaBlock^.Area) else
  if Command = afxcGetAreaDescription then GetAreaDescriptionBlock^.Description:=GetAreaDescription(
    GetAreaDescriptionBlock^.Area) else
  if Command = afxcRegisterCommandHandler then RegisterCommandHandler(PAreafixRegisterCommandHandlerBlock(Param)^) else
  if Command = afxcCreateReply then CreateReply(PAreafixCreateReplyBlock(Param)^) else
  if Command = afxcPerformSubscribe then AreafixParser^.PerformSubscribe(PString(Param)^) else
  if Command = afxcPerformUnsubscribe then AreafixParser^.PerformUnsubscribe(PString(Param)^) else
  if Command = afxcFixEmptyReply then AreafixParser^.FixEmptyReply(Pointer(Param)) else
  if Command = afxcGetRequestInfo then GetRequestInfo(PAreafixGetRequestInfoBlock(Param)^) else
  if Command = afxcRegisterCallback then RegisterCallback(PRegisterAreafixCallbackBlock(Param)^) else
  if Command = afxcGetName then PString(Param)^:=GetAreafixName else
  if Command = afxcAddRecipient then AreafixParser^.AddRecipient(PAddress(Param)^) else
  if Command = tcNetmailMessageTossLast then NetmailMessageToss(PNetmailMessageTossBlock(Param)^) else
  if Command = AreafixPerform then
   if AreafixFlags <> 0 then
    begin
     jt.PushTitle;

     CreateParser;

     if AreafixFlags and afScan <> 0 then
      AreafixParser^.Scan;

     if AreafixFlags and afHand <> 0 then
      if Handre = nil then
       jt.LogWrite('?', 'HAND without links regexp specified')
      else
       if HandCommand = '' then
        jt.LogWrite('?', 'HAND without command specified')
       else
        begin
         if HandCommand[1] = '@' then
          HandCommand[1]:='%';

         case ReplyMode of
          rmHand: jt.LogWrite('A', 'HAND ' + HandreName + ' ' + HandCommand);
          rmHandLog: jt.LogWrite('A', 'HANDLOG ' + HandreName + ' ' + HandCommand);
          rmHandQuiet: jt.LogWrite('A', 'HANDQUIET ' + HandreName + ' ' + HandCommand);
         end;

         HandDid:=0;

         for K:=0 to jt.link.Count - 1 do
          begin
           HandLink:=jt.link.At(K);

           jt.link.GetAddress(HandLink, HandAddress);

           if jt.AddressMatchRegExp(Handre, @HandAddress) then
            begin
             AreafixParser^.Command(HandLink, HandCommand, ReplyMode);

             Inc(HandDid);
            end;
          end;

         if HandDid = 0 then
          jt.LogWrite('A', 'Nothing to do.');
        end;

     DestroyParser;

     jt.PopTitle;
    end
   else else
  if Command = jtpcPluginSectionEnd then
   begin
    if afxScanArea = nil then
     jt.linker.Error(Linker, 'ScanArea was not defined');

    if afxRespondTo = nil then
     jt.linker.Error(Linker, 'RespondTo was not defined');

    if afxName = '' then
     jt.linker.Error(Linker, 'AreafixName was not defined correctly');

    if afxHelpTemplate = '' then
     jt.linker.Error(Linker, 'HelpTemplate was not defined correctly');

    if afxReplyToArea = nil then
     afxReplyToArea:=afxScanArea;
   end else
  if Command = AreafixDown then DestroyParser;
 end;

{ PluginHandleConfigSwitch }

procedure PluginHandleConfigSwitch(const Keyword: String; Value: String; const Linker: Pointer); export;
 var
  Group, Title: String;
  Index: xInteger;
 begin
  if Keyword = 'SCANAREA' then
   begin
    Value:=jt.linker.ParseStringToken(Linker, Value);

    afxScanArea:=jt.area.Seek(Value);

    if afxScanArea = nil then
     jt.linker.Error(Linker, 'unknown area "' + Value + '"')
    else
     jt.area.Used(afxScanArea);
   end else
  if Keyword = 'REPLYTOAREA' then
   begin
    Value:=jt.linker.ParseStringToken(Linker, Value);

    afxReplyToArea:=jt.area.Seek(Value);

    if afxReplyToArea = nil then
     jt.linker.Error(Linker, 'unknown area "' + Value + '"')
    else
     jt.area.Used(afxReplyToArea);
   end else
  if Keyword = 'RESPONDTO' then
   begin
    if afxRespondTo <> nil then
     jt.linker.Error(Linker, 'redefinition of RespondTo');

    jt.CompileRegExp(jt.linker.ParseStringToken(Linker, Value), afxRespondTo, Linker, False);
   end else
  if Keyword = 'AREAFIXNAME' then afxName:=jt.linker.ParseStringToken(Linker, Value) else
  if Keyword = 'HELPTEMPLATE' then afxHelpTemplate:=jt.linker.ParseStringToken(Linker, Value) else
  if Keyword = 'AREAGROUPNAME' then
   begin
    Group:=jt.linker.ParseStringToken(Linker, Value);

    if Length(Group) <> 1 then
     jt.linker.Error(Linker, 'invalid group name got');

    Title:=jt.linker.ParseStringToken(Linker, Value);

    if AreasGroupTitles^.Search(@Group[1], Index) then
     jt.linker.Error(Linker, 'redefinition of title for areagroup "' + Group[1] + '"')
    else
     AreasGroupTitles^.AtInsert(Index, New(PAreasGroupTitle, Init(Group[1], Title)));
   end else
  if Keyword = 'KILLREQUESTS' then afxKillRequests:=jt.linker.ParseBoolean(Linker, Value) else
  if Keyword = 'KILLREPLIES' then afxKillReplies:=jt.linker.ParseBoolean(Linker, Value) else
  if Keyword = 'CATCHNETMAIL' then afxCatchNetmail:=jt.linker.ParseBoolean(Linker, Value) else
   jt.linker.Error(Linker, 'syntax error');
 end;

{ PluginShowHelp }

procedure PluginShowHelp; export;
 begin
  jt.PrintStr(' AREAFIX          scan and reply messages addressed to Areafix'#13#10);
  jt.PrintStr(' HAND re command  perform "command" to links matching "re"'#13#10);
  jt.PrintStr(' HANDLOG re cmd   same stuff, but all replies will be redirected to log'#13#10);
  jt.PrintStr(' HANDQUIET re cmd same stuff, but no replies produced'#13#10);
 end;

{ PluginHandleLinkSwitch }

function PluginHandleLinkSwitch(const Link: Pointer; const Keyword: String; Value: String; const Linker: Pointer): Boolean;
 export;
 begin
  PluginHandleLinkSwitch:=True;

  if Keyword = 'AREAFIXPASSWORD' then jt.st.Add(jt.link.st(Link), 'Areafix.Password',
   jt.NewStr(jt.linker.ParseStringToken(Linker, Value)), jt.svc.stString)
  else
   PluginHandleLinkSwitch:=False;
 end;

{ PluginExportLinkSwitches }

procedure PluginExportLinkSwitches(const Link, Linker: Pointer); export;
 var
  st: Pointer;
  Password: PString;
 begin
  st:=jt.link.st(Link);

  Password:=jt.st.Get(st, 'Areafix.Password');

  if Password <> nil then
   jt.linker.ExportLine(Linker, ' AreafixPassword ' + jt.linker.UnparseString(Linker, Password^));
 end;

{ PluginExportLinksHeader }

procedure PluginExportLinksHeader(const Linker: Pointer); export;
 begin
  jt.linker.ExportLine(Linker, '; Areafix switch:');
  jt.linker.ExportLine(Linker, ';   AreafixPassword "password"');
  jt.linker.ExportLine(Linker, ';');
 end;

{ PluginHandleCmdlineSwitch }

function PluginHandleCmdlineSwitch(Switch: String): Boolean; export;
 procedure Lock(const AReplyMode: TReplyMode);
  begin
   ReplyMode:=AReplyMode;

   jt.LockCommandLine(AreafixPlugin);

   AreafixFlags:=AreafixFlags or afHand;
  end;
 begin
  if ReplyMode <> rmNormal then
   if Handre = nil then
    begin
     Handre:=jt.re.CreateStr(Switch);

     if Handre = nil then
      begin
       jt.PrintStr('regexp compiler returned nil on "' + Switch + '"'#13#10);

       PluginHandleCmdlineSwitch:=False;

       Exit;
      end;

     if not jt.re.IsValid(Handre) then
      begin
       jt.PrintStr(jt.re.GetErrorMsg(Handre) + ' on "' + Switch + '"'#13#10);

       PluginHandleCmdlineSwitch:=False;

       Exit;
      end;

     HandreName:=Switch;

     PluginHandleCmdlineSwitch:=True;
    end
   else
    begin
     if HandCommand = '' then
      HandCommand:=Switch
     else
      HandCommand:=HandCommand + ' ' + Switch;

     PluginHandleCmdlineSwitch:=True;
    end
  else
   begin
    StUpcaseEx(Switch);

    PluginHandleCmdlineSwitch:=True;

    if Switch = 'AREAFIX' then AreafixFlags:=AreafixFlags or afScan else
    if Switch = 'HAND' then Lock(rmHand) else
    if Switch = 'HANDLOG' then Lock(rmHandLog) else
    if Switch = 'HANDQUIET' then Lock(rmHandQuiet) else
     PluginHandleCmdlineSwitch:=False;
   end;
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleConfigSwitch,
 PluginShowHelp,
 PluginHandleLinkSwitch,
 PluginExportLinkSwitches,
 PluginExportLinksHeader,
 PluginHandleCmdlineSwitch;

begin
 Assign(Output, '');
 Rewrite(Output);
end.