library BCL;

uses
     PlugAPI,
     PlugSIDE,

     Objects,

     jtMHLcmn,

     bclAPI,
     bclAcc,
     bclCmn,
     bclAfx,
     bclAuto,
     bclNotfy,
     bclToss,
     bclBCL,

     afxAPI,

     Wizard;

{ apiOpen }

procedure apiOpen(var Block: TBCLOpenBlock);
 begin
  Block.bcl:=LinksBCL^.Open(Block.Link);
 end;

{ apiQuery }

procedure apiQuery(var Block: TBCLQueryBlock);
 var
  Index: xInteger;
  Area: PAreaBCL;
 begin
  if PLinkBCL(Block.bcl)^.Search(Block.Name, Index) then
   begin
    Area:=PLinkBCL(Block.bcl)^.At(Index);

    GetPStringEx(Area^.Desc, Block.Desc^);

    Block.Did:=True;
   end
  else
   Block.Did:=False;
 end;

{ apiEnumerate }

procedure apiEnumerate(var Block: TBCLEnumerateBlock);
 function Enumerate(const AreaBCL: PAreaBCL): Boolean; far;
  begin
   Enumerate:=Block.Enumerator(AreaBCL^.Name, AreaBCL^.Desc, Block.Param);
  end;
 begin
  PLinkBCL(Block.bcl)^.FirstThat(@Enumerate);
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, 'BCL', 1);

  bclPlugin:=Plugin;

  restNoBLIST:=jt.rests.Add('afxNoBLIST', rsMinus);
  restNoSETAUTOBCL:=jt.rests.Add('afxNoSETAUTOBCL', rsMinus);
  restNoSETUPDATEBCL:=jt.rests.Add('afxNoSETUPDATEBCL', rsMinus);

  Notifies:=New(PCollection, Init($10, $10));

  PluginSetup:=True;

  Preprocessor.Plugin:=Plugin;

  jt.fmt.pp.Register(@Preprocessor);

  LinksBCL:=New(PLinksBCL, Init($10, $10));
 end;

{ PluginStartup }

procedure PluginStartup; export;
 begin
  afxRegisterCommandHandler('BLIST', afxBListHandler);
  afxRegisterCommandHandler('AUTOBCL', afxAutoBCLHandler);
  afxRegisterCommandHandler('GETAUTOBCL', afxGetAutoBCLHandler);
  afxRegisterCommandHandler('UPDATEBCL', afxUpdateBCLHandler);
  afxRegisterCommandHandler('GETUPDATEBCL', afxGetUpdateBCLHandler);

  if bclAutoUseful then
   jt.EnqueueCommand(bclPlugin, bclAutoBCL, 0, cwEarliest);

  if bclUpdateUseful then
   begin
    jt.area.RegisterNotifyAreaCreated(bclPlugin);
    jt.area.RegisterNotifyAreaChanged(bclPlugin);
    jt.area.RegisterNotifyAreaDestroyed(bclPlugin);

    jt.EnqueueCommand(bclPlugin, bclUpdateBCL, 0, cwLatest);
   end;

  jt.EnqueueCommand(bclPlugin, bclFlush, 0, cwFlush);
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
  Dispose(LinksBCL, Done);

  Dispose(Notifies, Done);
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 begin
  if Command = bclAutoBCL then SendAutoBCL else
  if Command = bclUpdateBCL then UpdateBCL else
  if Command = bclFlush then LinksBCL^.Flush else
  if Command = bclcOpen then apiOpen(PBCLOpenBlock(Param)^) else
  if Command = bclcQuery then apiQuery(PBCLQueryBlock(Param)^) else
  if Command = bclcEnumerate then apiEnumerate(PBCLEnumerateBlock(Param)^) else
  if Command = jtpcanCreated then AreaNotify(Pointer(Param), anmCreated) else
  if Command = jtpcanChanged then AreaNotify(Pointer(Param), anmChanged) else
  if Command = jtpcanDestroyed then AreaNotify(Pointer(Param), anmDestroyed) else
  if Command = jtpcPluginSectionEnd then
   begin
    if bclBCLs = '' then
     jt.linker.Error(Pointer(Param), 'BCLs was not defined correctly');
   end;
 end;

{ PluginHandleConfigSwitch }

procedure PluginHandleConfigSwitch(const Keyword: String; Value: String; const Linker: Pointer); export;
 begin
  if Keyword = 'BCLS' then
   begin
    bclBCLs:=AddBackSlash(jt.linker.ParseStringToken(Linker, Value));

    if not jt.CreateDirectory(bclBCLs) then
     jt.linker.Error(Linker, 'unable to create ' + bclBCLs);
   end
  else
   jt.linker.Error(Linker, 'syntax error');
 end;

{ PluginHandleLinkSwitch }

function PluginHandleLinkSwitch(const Link: Pointer; const Keyword: String; Value: String; const Linker: Pointer): Boolean;
 export;
 var
  Days, AutoBCLLast: Longint;
  BCL: String;
  DT: TDateTime;
 begin
  PluginHandleLinkSwitch:=True;

  if Keyword = 'BCL' then
   begin
    BCL:=jt.linker.ParseStringToken(Linker, Value);

    jt.st.Add(jt.link.st(Link), 'BCL.BCL', jt.NewStr(BCL), jt.svc.stString);
   end else
  if Keyword = 'AUTOBCL' then
   begin
    Days:=jt.linker.ParseNumber(Linker, Value);

    if Days < 0 then
     jt.linker.Error(Linker, 'invalid value');

    jt.st.Add(jt.link.st(Link), 'BCL.AutoBCL', Pointer(Days), nil);

    bclAutoUseful:=True;
   end else
  if Keyword = 'AUTOBCLLAST' then
   begin
    jt.linker.ParseDateTime(Linker, Value, DT);

    DateTimeToUnixDateTime(DT, AutoBCLLast);

    jt.st.Add(jt.link.st(Link), 'BCL.AutoBCLLast', Pointer(AutoBCLLast), nil);
   end else
  if Keyword = 'UPDATEBCL' then
   begin
    jt.st.Add(jt.link.st(Link), 'BCL.UpdateBCL', Pointer(jt.linker.ParseBoolean(Linker, Value)), nil);

    bclUpdateUseful:=True;
   end
  else
   PluginHandleLinkSwitch:=False;
 end;

{ PluginExportLinkSwitches }

procedure PluginExportLinkSwitches(const Link, Linker: Pointer); export;
 var
  Days: Longint;
  BCL: PString;
  DT: TDateTime;
  st: Pointer;
 begin
  st:=jt.link.st(Link);

  BCL:=jt.st.Get(st, 'BCL.BCL');

  if BCL <> nil then
   jt.linker.ExportLine(Linker, ' BCL ' + jt.linker.UnparseString(Linker, BCL^));

  Days:=Longint(jt.st.Get(st, 'BCL.AutoBCL'));

  if Days > 0 then
   begin
    jt.linker.ExportLine(Linker, ' AutoBCL ' + jt.linker.UnparseNumber(Linker, Days));

    UnixDateTimeToDateTime(Longint(jt.st.Get(st, 'BCL.AutoBCLLast')), DT);

    if IsValidDateTime(DT) then
     jt.linker.ExportLine(Linker, '   AutoBCLLast ' + jt.linker.UnparseDateTime(Linker, DT));
   end;

  if LongBool(jt.st.Get(st, 'BCL.UpdateBCL')) then
   jt.linker.ExportLine(Linker, ' UpdateBCL Yes');
 end;

{ PluginExportLinksHeader }

procedure PluginExportLinksHeader(const Linker: Pointer); export;
 begin
  jt.linker.ExportLine(Linker, '; BCL switches:');
  jt.linker.ExportLine(Linker, ';   AutoBCL Yes|No');
  jt.linker.ExportLine(Linker, ';   AutoBCLLast unixtime');
  jt.linker.ExportLine(Linker, ';   UpdateBCL Yes|No');
  jt.linker.ExportLine(Linker, ';');
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleConfigSwitch,
 PluginHandleLinkSwitch,
 PluginExportLinkSwitches,
 PluginExportLinksHeader;

begin
 Assign(Output, '');
 Rewrite(Output);
end.