unit bclAcc;

interface
uses
     Objects,
     Strings,

     jtMHLcmn,

     Wizard;

const
 bhfIsList              = $0001;
 bhfIsUpdate            = $0002;

 bifReadonly            = $0001;
 bifPrivate             = $0002;
 bifFilearea            = $0004;
 bifMailarea            = $0008;
 bifRemove              = $0010;

 BufferSize             = 256;

type
 TBCLMode = (bmList, bmUpdate);

 PBCLHeader = ^TBCLHeader;
 TBCLHeader = packed record
  id: array[0..3] of Char;
  ConfMgrName: array[0..30] of Char;
  Origin: array[0..50] of Char;
  CreationTime: Longint;
  Flags: Longint;
  Reserved: array[0..255] of Char;
 end;

 PBCLItem = ^TBCLItem;
 TBCLItem = packed record
  Size: System.Integer;
  Flags1, Flags2: Longint;
 end;

 PBCLWriter = ^TBCLWriter;
 TBCLWriter = object(TObject)
  Stream: PStream;

  constructor Init(const AStream: PStream; const ConfMgrName, Origin: String; const Mode: TBCLMode);
  procedure Add(const Flags: Longint; const Tag, Description, Administrator: String);
 end;

 PBCLReader = ^TBCLReader;
 TBCLReader = object(TObject)
  Stream: PStream;
  Buffer: PChar;
  ConfMgrName, Origin: String;
  Mode: TBCLMode;

  constructor Init(const AStream: PStream; var rc: String);
  function Get(var Flags: Longint; var Tag, Description, Administrator: String): Boolean;
  function GetString: String;
  procedure GetLinkAddress(var Address: TAddress);
  destructor Done; virtual;
 end;

implementation

{ TBCLWriter }

constructor TBCLWriter.Init(const AStream: PStream; const ConfMgrName, Origin: String; const Mode: TBCLMode);
 var
  Header: TBCLHeader;
  DT: TDateTime;
 begin
  inherited Init;

  Stream:=AStream;

  FillChar(Header, SizeOf(Header), 0);

  StrPCopy(Header.id, 'BCL');
  StrPCopy(Header.ConfMgrName, ConfMgrName);
  StrPCopy(Header.Origin, Origin);

  GetCurrentDateTime(DT);

  DateTimeToUnixDateTime(DT, Header.CreationTime);

  case Mode of
   bmList: Header.Flags:=bhfIsList;
   bmUpdate: Header.Flags:=bhfIsUpdate;
  end;

  Stream^.Write(Header, SizeOf(Header));
 end;

procedure TBCLWriter.Add(const Flags: Longint; const Tag, Description, Administrator: String);
 var
  Item: TBCLItem;
 const
  NULL: System.Byte = 0;
 begin
  Item.Size:=SizeOf(Item) + Length(Tag) + 1 + Length(Description) + 1 + Length(Administrator) + 1;

  Item.Flags1:=Flags;
  Item.Flags2:=0;

  Stream^.Write(Item, SizeOf(Item));

  Stream^.Write(PString(@Tag)^[1], Length(Tag));
  Stream^.Write(NULL, SizeOf(NULL));

  Stream^.Write(PString(@Description)^[1], Length(Description));
  Stream^.Write(NULL, SizeOf(NULL));

  Stream^.Write(PString(@Administrator)^[1], Length(Administrator));
  Stream^.Write(NULL, SizeOf(NULL));
 end;

{ TBCLReader }

constructor TBCLReader.Init(const AStream: PStream; var rc: String);
 var
  Header: TBCLHeader;
 begin
  inherited Init;

  Stream:=AStream;

  Stream^.Read(Header, SizeOf(Header));

  if StrComp(Header.id, 'BCL') <> 0 then
   begin
    rc:='invalid header';

    Fail;
   end;

  ConfMgrName:=StrPas(Header.ConfMgrName);
  Origin:=StrPas(Header.Origin);

  if Header.Flags and bhfIsList <> 0 then Mode:=bmList else
  if Header.Flags and bhfIsUpdate <> 0 then Mode:=bmUpdate else
   begin
    rc:='unable to determine type of list';

    Fail;
   end;

  GetMem(Buffer, BufferSize + 1);
 end;

function TBCLReader.Get(var Flags: Longint; var Tag, Description, Administrator: String): Boolean;
 var
  Item: TBCLItem;
 begin
  if (Stream^.GetPos <> Stream^.GetSize) and (Stream^.Status = stOk) then
   begin
    Stream^.Read(Item, SizeOf(Item));

    Flags:=Item.Flags1;

    Tag:=GetString;
    Description:=GetString;
    Administrator:=GetString;

    Get:=True;
   end
  else
   Get:=False;
 end;

function TBCLReader.GetString: String;
 var
  StreamSize, Anchor, AnchorSize, Pos, Size, TargetPos: Longint;
  Line, Finish: PChar;
  S: String;
 begin
  StreamSize:=Stream^.GetSize;

  Anchor:=Stream^.GetPos;
  AnchorSize:=0;

  repeat
   Pos:=Stream^.GetPos;

   Size:=BufferSize;

   if Pos + Size > StreamSize then
    Size:=StreamSize - Pos;

   if Size <= 0 then
    begin
     TargetPos:=StreamSize;

     Break;
    end;

   Stream^.Read(Buffer[0], Size);

   Buffer[Size]:=#0;

   Finish:=StrEnd(Buffer);

   if @Finish[0] <> @Buffer[Size] then
    begin
     Inc(AnchorSize, Finish - Buffer);

     TargetPos:=Pos + (Finish - Buffer) + 1;

     Break;
    end
   else
    Inc(AnchorSize, Size);
  until False;

  if AnchorSize = 0 then
   GetString:=''
  else
   begin
    if AnchorSize > 255 then
     AnchorSize:=255;

    SetLength(S, AnchorSize);

    Stream^.Seek(Anchor);

    Stream^.Read(S[1], AnchorSize);

    GetString:=S;
   end;

  Stream^.Seek(TargetPos);
 end;

procedure TBCLReader.GetLinkAddress(var Address: TAddress);
 begin
  ClearAddress(Address);

  StrToAddress(Origin, Address);
 end;

destructor TBCLReader.Done;
 begin
  FreeMem(Buffer, BufferSize + 1);
 end;

end.