unit bclAfx;

interface
uses
     PlugAPI,
     PlugSIDE,

     Objects,

     bclCmn,
     bclAcc,

     afxAPI,

     jtMHLcmn,

     Wizard;

procedure afxBListHandler(const Command: String; Value: String); export;
procedure afxAutoBCLHandler(const Command: String; Value: String); export;
procedure afxGetAutoBCLHandler(const Command: String; Value: String); export;
procedure afxUpdateBCLHandler(const Command: String; Value: String); export;
procedure afxGetUpdateBCLHandler(const Command: String; Value: String); export;

implementation

{ afxBListHandler }

procedure afxBListHandler(const Command: String; Value: String);
 var
  Block: TAreafixGetRequestInfoBlock;
  FName: String;
  Stream: PStream;
  Writer: PBCLWriter;
  Created: Boolean;
  Address: TAddress;
  Reply, Area, msgp, st: Pointer;
  DT: TDateTime;
  K, AutoBCLLast: Longint;
 begin
  afxGetRequestInfo(Block);

  jt.link.GetAddress(Block.Link, Address);

  if jt.rests.Check(restNoBLIST, @Address, nil) then
   begin
    jt.LogWrite('A', 'Prohibited');

    jt.msg.Add(afxCreateReply('Common', Created), '%BLIST: you have no rights to execute this command');

    Exit;
   end;

  repeat
   FName:=jt.core.GetTempAttaches + GenerateMSGID + '.bcl';
  until not ExistFile(FName);

  Stream:=New(PBufStream, Init(FName, stCreate, 32768));

  if Stream^.Status <> stOk then
   begin
    jt.Report('Unable to create ' + FName + ', rc=' + Long2Str(Stream^.Status), True);

    jt.msg.Add(afxCreateReply('Common', Created), 'Unable to execute %BLIST cause of internal error');

    Dispose(Stream, Done);

    Exit;
   end;

  Reply:=afxCreateReply('BLIST.' + GenerateMSGID, Created);

  msgp:=jt.msg.CreateParser(Reply, True);

  jt.msg.p.AddKludge(msgp, #1'FLAGS KFS');

  jt.msg.DestroyParser(msgp);

  jt.msg.SetAttributes(Reply, jt.msg.GetAttributes(Reply) or maAttach);

  jt.msg.SetSubject(Reply, FName);

  jt.link.GetAka(Block.Link, Address);

  Writer:=New(PBCLWriter, Init(Stream, afxGetName, AddressToStrEx(Address), bmList));

  for K:=0 to jt.area.Count - 1 do
   begin
    Area:=jt.area.At(K);

    if afxIsOperable(Area) and afxIsShowable(Block.Link, Area) then
     Writer^.Add(bifMailarea, jt.area.GetName(Area)^, GetPString(jt.area.GetDesc(Area)), '');
   end;

  Dispose(Writer, Done);

  Dispose(Stream, Done);

  jt.msg.Add(afxCreateReply('Common', Created), '%BLIST: ' + JustFileName(FName) + ' attached');

  jt.LogWrite('A', FName + ' attached');

  st:=jt.link.st(Block.Link);

  if LongBool(jt.st.Get(st, 'BCL.AutoBCL')) then
   begin
    GetCurrentDateTime(DT);

    DateTimeToUnixDateTime(DT, AutoBCLLast);

    jt.st.Add(st, 'BCL.AutoBCLLast', Pointer(AutoBCLLast), nil);

    jt.link.SetChanged(lcfChanged, 'afx blist/autobcl');
   end;
 end;

{ afxAutoBCLHandler }

procedure afxAutoBCLHandler(const Command: String; Value: String);
 var
  Address: TAddress;
  Created: Boolean;
  Block: TAreafixGetRequestInfoBlock;
  Days: Longint;
  st: Pointer;
 begin
  afxGetRequestInfo(Block);

  jt.link.GetAddress(Block.Link, Address);

  if jt.rests.Check(restNoSETAUTOBCL, @Address, nil) then
   begin
    jt.LogWrite('A', 'Prohibited');

    jt.msg.Add(afxCreateReply('Common', Created), '%AUTOBCL: you have no rights to execute this command');

    Exit;
   end;

  TrimEx(Value);

  if (Value = '') or not Str2Longint(Value, Days) or (Days < 0) then
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%AUTOBCL: invalid value "' + Value + '" specified');

    jt.LogWrite('A', 'Invalid value "' + Value + '" specified');

    Exit;
   end;

  st:=jt.link.st(Block.Link);

  jt.st.Add(st, 'BCL.AutoBCL', Pointer(Days), nil);

  jt.link.SetChanged(lcfChanged, 'afx setautobcl');

  if Days = 0 then
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%AUTOBCL: autosending of binary conference list disabled');

    jt.LogWrite('A', 'Autosending of binary conference list disabled');
   end
  else
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%AUTOBCL: autosending of binary conference list every ' +
     Long2Str(Days) + ' day' + _s(Days));

    jt.LogWrite('A', 'Autosending of binary conference list every ' +
     Long2Str(Days) + ' day' + _s(Days));
   end;
 end;

{ afxGetAutoBCLHandler }

procedure afxGetAutoBCLHandler(const Command: String; Value: String);
 var
  Created: Boolean;
  Block: TAreafixGetRequestInfoBlock;
  Days: Longint;
  st: Pointer;
 begin
  afxGetRequestInfo(Block);

  st:=jt.link.st(Block.Link);

  Days:=Longint(jt.st.Get(st, 'BCL.AutoBCL'));

  if Days = 0 then
   jt.msg.Add(afxCreateReply('Common', Created), '%GETAUTOBCL: autosending of binary conference list disabled')
  else
   jt.msg.Add(afxCreateReply('Common', Created), '%GETAUTOBCL: autosending of binary conference list every ' +
    Long2Str(Days) + ' day' + _s(Days));
 end;

{ afxUpdateBCLHandler }

procedure afxUpdateBCLHandler(const Command: String; Value: String);
 var
  Address: TAddress;
  Created: Boolean;
  Block: TAreafixGetRequestInfoBlock;
  st: Pointer;
 begin
  afxGetRequestInfo(Block);

  jt.link.GetAddress(Block.Link, Address);

  if jt.rests.Check(restNoSETUPDATEBCL, @Address, nil) then
   begin
    jt.LogWrite('A', 'Prohibited');

    jt.msg.Add(afxCreateReply('Common', Created), '%UPDATEBCL: you have no rights to execute this command');

    Exit;
   end;

  st:=jt.link.st(Block.Link);

  TrimEx(Value);
  StUpcaseEx(Value);

  if Value = 'YES' then
   begin
    jt.st.Add(st, 'BCL.UpdateBCL', Pointer(1), nil);

    jt.link.SetChanged(lcfChanged, 'afx setupdatebcl');

    jt.msg.Add(afxCreateReply('Common', Created), '%UPDATEBCL: updating of binary conference list enabled');

    jt.LogWrite('A', 'Updating of binary conference list enabled');
   end else
  if Value = 'NO' then
   begin
    jt.st.Add(st, 'BCL.UpdateBCL', nil, nil);

    jt.link.SetChanged(lcfChanged, 'afx setupdatebcl');

    jt.msg.Add(afxCreateReply('Common', Created), '%UPDATEBCL: updating of binary conference list disabled');

    jt.LogWrite('A', 'Updating of binary conference list disabled');
   end
  else
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%UPDATEBCL: invalid value (YES or NO expected)');

    jt.LogWrite('A', 'Invalid value');
   end;
 end;

{ afxGetUpdateBCLHandler }

procedure afxGetUpdateBCLHandler(const Command: String; Value: String);
 var
  Created: Boolean;
  Block: TAreafixGetRequestInfoBlock;
  st: Pointer;
 begin
  afxGetRequestInfo(Block);

  st:=jt.link.st(Block.Link);

  if LongBool(jt.st.Get(st, 'BCL.UpdateBCL')) then
   jt.msg.Add(afxCreateReply('Common', Created), '%GETUPDATEBCL: updating of binary conference list enabled')
  else
   jt.msg.Add(afxCreateReply('Common', Created), '%GETUPDATEBCL: updating of binary conference list disabled');
 end;

end.