unit bclBCL;

interface
uses
     PlugAPI,
     PlugSIDE,

     jtMHLcmn,

     bclCmn,
     bclAcc,

     Objects,
     Strings,

     Wizard;

type
 PAreaBCL = ^TAreaBCL;
 TAreaBCL = object(TObject)
  Name, Desc: PString;

  constructor Init(const AName, ADesc: String);
  destructor Done; virtual;
 end;

 PLinkBCL = ^TLinkBCL;
 TLinkBCL = object(TSortedCollection)
  Link, st: Pointer;
  Address: TAddress;
  Changed: Boolean;
  ConfMgrName, FName: String;

  constructor Init(const ALink: Pointer);
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  function KeyOf(Item: Pointer): Pointer; virtual;
  procedure Flush;
  destructor Done; virtual;
 public
  procedure LoadBCL(const FileName: String; const First, Progress: Boolean);
 end;

 PLinksBCL = ^TLinksBCL;
 TLinksBCL = object(TSortedCollection)
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  function KeyOf(Item: Pointer): Pointer; virtual;
 public
  function Open(Link: Pointer): PLinkBCL;
  procedure Flush;
 end;

const
 LinksBCL               : PLinksBCL             = nil;

implementation

{ TAreaBCL }

constructor TAreaBCL.Init(const AName, ADesc: String);
 begin
  inherited Init;

  Name:=NewStr(AName);
  Desc:=NewStr(ADesc);
 end;

destructor TAreaBCL.Done;
 begin
  DisposeStr(Desc);
  DisposeStr(Name);

  inherited Done;
 end;

{ TLinkBCL }

constructor TLinkBCL.Init(const ALink: Pointer);
 var
  AFName: PString;
 begin
  inherited Init($100, $100);

  Link:=ALink;

  jt.link.GetAddress(Link, Address);

  st:=jt.link.st(Link);

  ConfMgrName:='';

  AFName:=jt.st.Get(st, 'BCL.BCL');

  if AFName = nil then
   begin
    repeat
     FName:=bclBCLs + GenerateMSGID + '.bcl';
    until not ExistFile(FName);

    jt.st.Add(st, 'BCL.BCL', jt.NewStr(FName), jt.svc.stString);

    jt.link.SetChanged(lcfChanged, 'lbcl/init/bclfnamecreation');

    Changed:=True;
   end
  else
   begin
    FName:=AFName^;

    LoadBCL(FName, True, False);
   end;
 end;

function TLinkBCL.Compare(Key1, Key2: Pointer): xInteger;
 var
  K1: PString absolute Key1;
  K2: PString absolute Key2;
 begin
  if K1^ < K2^ then Compare:=-1 else
  if K1^ > K2^ then Compare:=1 else
   Compare:=0;
 end;

function TLinkBCL.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=PAreaBCL(Item)^.Name;
 end;

procedure TLinkBCL.Flush;
 var
  Stream: PStream;
  Writer: PBCLWriter;
 procedure Perform(const AreaBCL: PAreaBCL); far;
  begin
   Writer^.Add(bifMailarea, AreaBCL^.Name^, GetPString(AreaBCL^.Desc), '');
  end;
 begin
  if Changed then
   begin
    jt.SetTitle(AddressToStrEx(Address) + ' (flushing BCL)');
    jt.SetStatus('flusing BCL for ' + AddressToStrEx(Address));

    Stream:=New(PDosStream, Init(FName, stCreate));

    if Stream^.Status = stOk then
     begin
      Writer:=New(PBCLWriter, Init(Stream, ConfMgrName, AddressToStrEx(Address), bmList));

      ForEach(@Perform);

      if Stream^.Status <> stOk then
       jt.LogWrite('?', 'Unable to write ' + FName + ', rc=' + Long2Str(Stream^.Status));

      Dispose(Writer, Done);
     end
    else
     jt.LogWrite('?', 'Unable to create ' + FName + ', rc=' + Long2Str(Stream^.Status));

    Dispose(Stream, Done);

    Changed:=False;
   end;
 end;

destructor TLinkBCL.Done;
 begin
  Flush;

  inherited Done;
 end;

{ public methods }

procedure TLinkBCL.LoadBCL(const FileName: String; const First, Progress: Boolean);
 var
  Stream: PStream;
  Reader: PBCLReader;
  Flags, Added, Removed, Freshed: Longint;
  Index: xInteger;
  Area: PAreaBCL;
  rc, Name, Desc, S: String;
 begin
  Stream:=New(PDosStream, Init(FileName, stOpenRead));

  if Stream^.Status <> stOk then
   begin
    jt.LogWrite('?', 'Unable to open ' + FileName + ', rc=' + Long2Str(Stream^.Status));

    Dispose(Stream, Done);

    if First then
     Changed:=True;

    Exit;
   end;

  Added:=0;
  Removed:=0;
  Freshed:=0;

  Reader:=New(PBCLReader, Init(Stream, rc));

  if Reader = nil then
   jt.LogWrite('?', 'Unable to load ' + FileName + ': ' + rc)
  else
   begin
    if Reader^.Mode = bmList then
     begin
      FreeAll;

      if not First then
       Changed:=True;
     end;

    if ConfMgrName = '' then
     ConfMgrName:=Reader^.ConfMgrName;

    case Reader^.Mode of
     bmList: jt.SetStatus('loading ' + FileName + ' for ' + AddressToStrEx(Address));
     bmUpdate: jt.SetStatus('updating ' + FileName + ' for ' + AddressToStrEx(Address));
    end;

    while Reader^.Get(Flags, Name, Desc, rc) do
     begin
      StUpcaseEx(Name);

      if Flags and bifRemove <> 0 then
       if Search(@Name, Index) then
        begin
         AtFree(Index);

         if not First then
          Changed:=True;

         Inc(Removed);
        end
       else
      else
       if not Search(@Name, Index) then
        begin
         AtInsert(Index, New(PAreaBCL, Init(Name, Desc)));

         if not First then
          Changed:=True;

         Inc(Added);
        end
       else
        begin
         Area:=At(Index);

         if (Area^.Desc = nil) and (Desc <> '') then
          begin
           Area^.Desc:=NewStr(Desc);

           if not First then
            Changed:=True;

           Inc(Freshed);
          end;
        end;
     end;

    if Progress then
     begin
      S:='';

      if Added <> 0 then
       S:=S + Long2Str(Added) + ' added, ';

      if Freshed <> 0 then
       S:=S + Long2Str(Freshed) + ' freshed, ';

      if Removed <> 0 then
       S:=S + Long2Str(Removed) + ' removed, ';

      if S <> '' then
       begin
        S:=Copy(S, 1, Length(S) - 2);

        case Reader^.Mode of
         bmList: jt.LogWrite('B', 'BCL for ' + AddressToStrEx(Address) + ' affected: ' + S);
         bmUpdate: jt.LogWrite('B', 'BCL for ' + AddressToStrEx(Address) + ' updated: ' + S);
        end;
       end;
     end;

    Dispose(Reader, Done);
   end;

  Dispose(Stream, Done);
 end;

{ TLinksBCL }

function TLinksBCL.Compare(Key1, Key2: Pointer): xInteger;
 var
  K1: Longint absolute Key1;
  K2: Longint absolute Key2;
 begin
  Compare:=CompareLongint(K1, K2);
 end;

function TLinksBCL.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=PLinkBCL(Item)^.Link;
 end;

{ public methods }

function TLinksBCL.Open(Link: Pointer): PLinkBCL;
 var
  Index: xInteger;
  LinkBCL: PLinkBCL;
 begin
  if Search(Link, Index) then
   Open:=At(Index)
  else
   begin
    LinkBCL:=New(PLinkBCL, Init(Link));

    AtInsert(Index, LinkBCL);

    Open:=LinkBCL;
   end;
 end;

procedure TLinksBCL.Flush;
 procedure Perform(const LinkBCL: PLinkBCL); far;
  begin
   LinkBCL^.Flush;
  end;
 begin
  jt.PushTitle;

  ForEach(@Perform);

  jt.ClearStatus;
  jt.PopTitle;
 end;

end.