unit bclNotfy;

interface
uses
     PlugAPI,
     PlugSIDE,

     Objects,
     Strings,

     bclAcc,

     afxAPI,

     jtMHLcmn,

     Wizard;

type
 TAreaNotifyMode = (anmCreated, anmChanged, anmDestroyed);

 PAreaNotify = ^TAreaNotify;
 TAreaNotify = object(TObject)
  Name, Desc: PString;
  Mode: TAreaNotifyMode;

  constructor Init(const AName, ADesc: String; const AMode: TAreaNotifyMode);
  destructor Done; virtual;
 end;

const
 Notifies       : PCollection           = nil;

procedure AreaNotify(const Area: Pointer; const Mode: TAreaNotifyMode);

procedure UpdateBCL;

implementation

{ TAreaNotify }

constructor TAreaNotify.Init(const AName, ADesc: String; const AMode: TAreaNotifyMode);
 begin
  inherited Init;

  Name:=NewStr(AName);
  Desc:=NewStr(ADesc);

  Mode:=AMode;
 end;

destructor TAreaNotify.Done;
 begin
  DisposeStr(Desc);
  DisposeStr(Name);

  inherited Done;
 end;

{ AreaNotify }

procedure AreaNotify(const Area: Pointer; const Mode: TAreaNotifyMode);
 begin
  Notifies^.Insert(New(PAreaNotify, Init(jt.area.GetName(Area)^, GetPString(jt.area.GetDesc(Area)), Mode)));
 end;

{ UpdateBCL }

procedure UpdateBCL;
 var
  Link, st, msg, msgp: Pointer;
  Writer: PBCLWriter;
  Stream: PStream;
  FName: String;
  K: Longint;
  FromAddress, ToAddress: TAddress;
 procedure WriteNotify(const Notify: PAreaNotify); far;
  var
   Flags: Longint;
  begin
   Flags:=bifMailarea;

   if Notify^.Mode = anmDestroyed then
    Flags:=Flags or bifRemove;

   Writer^.Add(Flags, GetPString(Notify^.Name), GetPString(Notify^.Desc), '');
  end;
 begin
  if Notifies^.Count = 0 then
   Exit;

  jt.PushTitle;

  for K:=0 to jt.link.Count - 1 do
   begin
    Link:=jt.link.At(K);

    st:=jt.link.st(Link);

    if not LongBool(jt.st.Get(st, 'BCL.UpdateBCL')) then
     Continue;

    repeat
     FName:=jt.core.GetTempAttaches + GenerateMSGID + '.bcl';
    until not ExistFile(FName);

    Stream:=New(PBufStream, Init(FName, stCreate, 32768));

    if Stream^.Status <> stOk then
     begin
      jt.Report('Unable to create ' + FName + ', rc=' + Long2Str(Stream^.Status), True);

      Dispose(Stream, Done);

      Continue;
     end;

    msg:=jt.msg.Create;

    jt.st.Add(jt.msg.st(msg), 'Source', Pointer(tsLocalInbound), nil);

    jt.msg.SetFromName(msg, 'UpdateBCL');
    jt.msg.SetToName(msg, 'Sysop');
    jt.msg.SetSubject(msg, FName);
    jt.msg.SetAttributes(msg, maPrivate or maKill or maAttach);
    jt.msg.SetArea(msg, jt.area.GetName(jt.area.SeekNetmail)^);

    jt.link.GetAddress(Link, ToAddress);

    jt.GetNetmailAka(ToAddress, FromAddress);

    msgp:=jt.msg.CreateParser(msg, True);

    jt.msg.p.SetFromAndToAddress(msgp, FromAddress, ToAddress, True);
    jt.msg.p.AddKludge(msgp, #1'FLAGS KFS');

    jt.msg.DestroyParser(msgp);

    jt.SetStatus('creating UpdateBCL to ' + AddressToStrEx(ToAddress));
    jt.SetTitle(AddressToStrEx(ToAddress) + ' (creating UpdateBCL)');

    jt.LogWrite('B', 'Creating UpdateBCL to ' + AddressToStrEx(ToAddress) + ' (' + FName + ')');

    Writer:=New(PBCLWriter, Init(Stream, afxGetName, AddressToStrEx(FromAddress), bmUpdate));

    Notifies^.ForEach(@WriteNotify);

    Dispose(Writer, Done);

    Dispose(Stream, Done);

    jt.tossscan.TossMessage(msg);
   end;

  jt.ClearStatus;

  jt.PopTitle;
 end;

end.