library CC;

{ CarbonCopierEx "CCEx" scripting host supplement plugin for JustTosser
  (c) by sergey korowkin // [sk], 2001.

  contact information:
   - 2:6033/27@fidonet
   - sk@fido.irk.ru
   - +7(81153)50643

  ⥡     ⥭. p㬠,   ?

  dedicated to Svetlana L. Kiseleva // [slk], 14/03/2001-today.
}

uses
     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     Objects,
     Strings,

     Wizard,

     ccCmn,
     ccAPI,
     ccAtom,
     ccHost,
     ccRoot,
     ccIdent,
     ccProto,
     ccToken,
     ccAction,
     ccExpr,

     ccJTAPI,
     ccJTSupp,
     ccSupp;

type
 PccRoot = ^TccRoot;
 TccRoot = object(TRoot)
 end;

 PccCommandWhenHook = ^TccCommandWhenHook;
 TccCommandWhenHook = packed record
  Hook: PccHook;
  Name: PChar;
  When: TCommandWhen;
 end;

 PccScriptsList = ^TccScriptsList;
 TccScriptsList = object(TStrings)
  function Lookup(FName: String): PString;
 end;

const
 ccPlugin               : Pointer               = nil;

 ccCompileInfo          : Boolean               = False;

 ccScriptsFirst         : PccScriptsList        = nil;
 ccScriptsLast          : PccScriptsList        = nil;

 ccScriptsRoot          : PccRoot               = nil;

 ccStartupHook          : PccHook               = nil;
 ccHelpHook             : PccHook               = nil;

const
 AutoExecFName                                  = 'autoexec.cc';

 CommandWhenHooks: array[$A000..$A017] of TccCommandWhenHook = (
  (Hook: nil; Name: 'CWBEFOREPLUGINS';   When: cwBeforePlugins),
  (Hook: nil; Name: 'CWPLUGINSLOADED';   When: cwPluginsLoaded),
  (Hook: nil; Name: 'CWBEFORESTARTUP';   When: cwBeforeStartup),
  (Hook: nil; Name: 'CWSTARTUP';         When: cwStartup),
  (Hook: nil; Name: 'CWAFTERSTARTUP';    When: cwAfterStartup),
  (Hook: nil; Name: 'CWEARLIEST';        When: cwEarliest),
  (Hook: nil; Name: 'CWBEFORETOSS';      When: cwBeforeToss),
  (Hook: nil; Name: 'CWTOSS';            When: cwToss),
  (Hook: nil; Name: 'CWSCAN';            When: cwScan),
  (Hook: nil; Name: 'CWAFTERTOSS';       When: cwAfterToss),
  (Hook: nil; Name: 'CWPURGE';           When: cwPurge),
  (Hook: nil; Name: 'CWLINK';            When: cwLink),
  (Hook: nil; Name: 'CWLATEST';          When: cwLatest),
  (Hook: nil; Name: 'CWBEFOREFLUSH';     When: cwBeforeFlush),
  (Hook: nil; Name: 'CWFLUSH';           When: cwFlush),
  (Hook: nil; Name: 'CWBEFOREEXPORT';    When: cwBeforeExport),
  (Hook: nil; Name: 'CWEXPORTAREAS';     When: cwExportAreas),
  (Hook: nil; Name: 'CWEXPORTAREASFULL'; When: cwExportAreasFull),
  (Hook: nil; Name: 'CWEXPORTLINKS';     When: cwExportLinks),
  (Hook: nil; Name: 'CWEXPORTLINKSFULL'; When: cwExportLinksFull),
  (Hook: nil; Name: 'CWAFTEREXPORT';     When: cwAfterExport),
  (Hook: nil; Name: 'CWBEFORESHUTDOWN';  When: cwBeforeShutdown),
  (Hook: nil; Name: 'CWSHUTDOWN';        When: cwShutdown),
  (Hook: nil; Name: 'CWAFTERSHUTDOWN';   When: cwAfterShutdown));

type
 PCommandLineItem = ^TCommandLineItem;
 TCommandLineItem = object(TObject)
  SwitchName: PString;
  Specified: PBooleanExpression;
  Value: PStringExpression;
  Assigned: Boolean;

  constructor Init(const ASwitchName: String; const ASpecified: PBooleanExpression; const AValue: PStringExpression);
  procedure Check; virtual;
  destructor Done; virtual;
 end;

 PCommandLineLoadItem = ^TCommandLineLoadItem;
 TCommandLineLoadItem = object(TCommandLineItem)
  ScriptName: PString;

  constructor Init(const AScriptName, ASwitchName: String; const ASpecified: PBooleanExpression;
    const AValue: PStringExpression);
  procedure Check; virtual;
  destructor Done; virtual;
 end;

 PCommandLine = ^TCommandLine;
 TCommandLine = object(TStringCollection)
  procedure FreeItem(Item: Pointer); virtual;
  function KeyOf(Item: Pointer): Pointer; virtual;
 public
  function Check(const Value: String): Boolean;
 end;

 PCommandLineProto = ^TCommandLineProto;
 TCommandLineProto = object(TProto)
  function Create(const ATokenizer, AActions: Pointer): PAtom; virtual;
 end;

 PCommandLineLoadProto = ^TCommandLineLoadProto;
 TCommandLineLoadProto = object(TProto)
  function Create(const ATokenizer, AActions: Pointer): PAtom; virtual;
 end;

const
 ccCommandLine          : PCommandLine          = nil;

{ TccRoot }

{ TccScriptsList }

function TccScriptsList.Lookup(FName: String): PString;
 function Perform(const AFName: PString): Boolean;
  begin
   Perform:=FName = StUpcase(FExpand(AFName^));
  end;
 begin
  FName:=FExpand(FName);

  StUpcaseEx(FName);

  Lookup:=FirstThat(@Perform);
 end;

{ TCommandLineItem }

constructor TCommandLineItem.Init(const ASwitchName: String; const ASpecified: PBooleanExpression;
  const AValue: PStringExpression);
 begin
  inherited Init;

  SwitchName:=NewStr(ASwitchName);
  Specified:=ASpecified;
  Value:=AValue;
 end;

procedure TCommandLineItem.Check;
 begin
 end;

destructor TCommandLineItem.Done;
 begin
  DisposeStr(SwitchName);

  inherited Done;
 end;

{ TCommandLineLoadItem }

constructor TCommandLineLoadItem.Init(const AScriptName, ASwitchName: String; const ASpecified: PBooleanExpression;
  const AValue: PStringExpression);
 begin
  inherited Init(ASwitchName, ASpecified, AValue);

  ScriptName:=NewStr(AScriptName);
 end;

procedure TCommandLineLoadItem.Check;
 begin
  if ccScriptsLast^.Lookup(ScriptName^) = nil then
   ccScriptsLast^.Insert(NewStr(ScriptName^));
 end;

destructor TCommandLineLoadItem.Done;
 begin
  DisposeStr(ScriptName);

  inherited Done;
 end;

{ TCommandLine }

procedure TCommandLine.FreeItem(Item: Pointer);
 begin
  TCollection.FreeItem(Item);
 end;

function TCommandLine.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=PCommandLineItem(Item)^.SwitchName;
 end;

{ public methods }

function TCommandLine.Check(const Value: String): Boolean;
 var
  Name, Parameter: String;
  ExecuteData: Pointer;
  Index: xInteger;
  Item: PCommandLineItem;
  K: Byte;
 begin
  K:=Pos(':', Value);

  if K = 0 then
   begin
    Name:=Value;
    Parameter:='';
   end
  else
   begin
    Name:=Copy(Value, 1, K - 1);
    Parameter:=Copy(Value, K + 1, 255);
   end;

  StUpcaseEx(Name);

  if Search(@Name, Index) then
   begin
    Check:=True;

    Item:=At(Index);

    if Item^.Assigned then
     jt.linker.Error(jt.linker.Stub, 'duplicate switch "' + Name + '"');

    Item^.Assigned:=True;

    if (Parameter <> '') and (Item^.Value = nil) then
     jt.linker.Error(jt.linker.Stub, 'there is no need in parameter of "' + Name + '" switch');

    ExecuteData:=ccAPI.cc.CreateExecuteData;

    Item^.Specified^.Assign(ExecuteData, Item^.Assigned);

    ccAPI.cc.ResetExecuteData(ExecuteData);

    if Item^.Value <> nil then
     Item^.Value^.Assign(ExecuteData, Parameter);

    ccAPI.cc.DestroyExecuteData(ExecuteData);

    Item^.Check;
   end
  else
   Check:=False;
 end;

{ TCommandLineProto }

function TCommandLineProto.Create(const ATokenizer, AActions: Pointer): PAtom;
 var
  Tokenizer: PTokenizer absolute ATokenizer;
  Actions: PActions absolute AActions;
  SwitchName: String;
  SpecifiedExpression: PBooleanExpression;
  ValueExpression: PStringExpression;
  Index: xInteger;
 begin
  SwitchName:=Tokenizer^.Next([tmString])^.Value^;

  if SwitchName = '' then
   Tokenizer^.Error('non-empty string expected');

  StUpcaseEx(SwitchName);

  if ccCommandLine^.Search(@SwitchName, Index) then
   Tokenizer^.Error('duplicate command line switch name "' + SwitchName + '"');

  Tokenizer^.Next([tmComma]);

  SpecifiedExpression:=PBooleanExpression(PExpressionProto(Actions^.Identifiers^.AddIdentifier(imBooleanExpressionProto,
    New(PExpressionProto, Init(Tokenizer^.Next([tmToken])^.Value^,
    New(PConstantBooleanExpression, Init(False, Actions)))))^.Proto)^.Expression);

  if Tokenizer^.Next([tmComma, tmSemicolon])^.Mode = tmComma then
   begin
    ValueExpression:=PStringExpression(PExpressionProto(Actions^.Identifiers^.AddIdentifier(imStringExpressionProto,
      New(PExpressionProto, Init(Tokenizer^.Next([tmToken])^.Value^,
      New(PConstantStringExpression, Init('', Actions)))))^.Proto)^.Expression);

    Tokenizer^.Next([tmSemicolon]);
   end
  else
   ValueExpression:=nil;

  ccCommandLine^.AtInsert(Index, New(PCommandLineItem, Init(SwitchName, SpecifiedExpression, ValueExpression)));
 end;

{ TCommandLineLoadProto }

function TCommandLineLoadProto.Create(const ATokenizer, AActions: Pointer): PAtom;
 var
  Tokenizer: PTokenizer absolute ATokenizer;
  Actions: PActions absolute AActions;
  ScriptName, SwitchName: String;
  SpecifiedExpression: PBooleanExpression;
  ValueExpression: PStringExpression;
  Index: xInteger;
 begin
  ScriptName:=Tokenizer^.Next([tmString])^.Value^;

  if ScriptName = '' then
   Tokenizer^.Error('non-empty string expected');

  Tokenizer^.Next([tmComma]);

  SwitchName:=Tokenizer^.Next([tmString])^.Value^;

  if SwitchName = '' then
   Tokenizer^.Error('non-empty string expected');

  StUpcaseEx(SwitchName);

  if ccCommandLine^.Search(@SwitchName, Index) then
   Tokenizer^.Error('duplicate command line switch name "' + SwitchName + '"');

  Tokenizer^.Next([tmComma]);

  SpecifiedExpression:=PBooleanExpression(PExpressionProto(Actions^.Identifiers^.AddIdentifier(imBooleanExpressionProto,
    New(PExpressionProto, Init(Tokenizer^.Next([tmToken])^.Value^,
    New(PConstantBooleanExpression, Init(False, Actions)))))^.Proto)^.Expression);

  if Tokenizer^.Next([tmComma, tmSemicolon])^.Mode = tmComma then
   begin
    ValueExpression:=PStringExpression(PExpressionProto(Actions^.Identifiers^.AddIdentifier(imStringExpressionProto,
      New(PExpressionProto, Init(Tokenizer^.Next([tmToken])^.Value^,
      New(PConstantStringExpression, Init('', Actions)))))^.Proto)^.Expression);

    Tokenizer^.Next([tmSemicolon]);
   end
  else
   ValueExpression:=nil;

  ccCommandLine^.AtInsert(Index, New(PCommandLineLoadItem, Init(ScriptName, SwitchName, SpecifiedExpression,
    ValueExpression)));
 end;

{ CompileScript }

procedure CompileScript(const FName: String);
 var
  CompiledTime, CompiledLines, CompiledMemory: Longint;
 begin
  ccScriptsRoot^.Compile(FName, CompiledTime, CompiledLines, CompiledMemory);

  if ccCompileInfo then
   jt.LogWrite('C', FName + ': ' + jt.FormatTime(CompiledTime) + ', ' +
                                   Long2Str(CompiledLines) + ' line' + _s(CompiledLines) + ', ' +
                                   Long2StrFmt(CompiledMemory) + ' byte' + _s(CompiledMemory) + ' in memory');
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 var
  K: Longint;
  CommandLine, CommandLineLoad: PIdentifier;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, 'CarbonCopierEx', 1);

  ccPlugin:=Plugin;

  PluginSetup:=True;

  { // check all records that you still don't know }

  InitHost;

  ccScriptsFirst:=New(PccScriptsList, Init($10, $10));
  ccScriptsLast:=New(PccScriptsList, Init($10, $10));

  ccScriptsRoot:=New(PccRoot, Init);

  { // accross my mind }

  AddJTAPIProtos(ccScriptsRoot);
  AddVariables(ccScriptsRoot);
  AddSupportRoutines(ccScriptsRoot);

  { //  ⠪   }

  for K:=Low(CommandWhenHooks) to High(CommandWhenHooks) do
   begin
    CommandWhenHooks[K].Hook:=New(PccHook, Init(StrPas(CommandWhenHooks[K].Name)));

    jt.EnqueueCommand(Plugin, K, 0, CommandWhenHooks[K].When);
   end;

  { // he's a real Johnny }

  ccCommandLine:=New(PCommandLine, Init($10, $10));

  { // music so wonderful }

  ccHelpHook:=New(PccHook, Init('CCHELP'));

  if ExistFile(AutoExecFName) then
   begin
    CommandLine:=ccScriptsRoot^.Identifiers^.AddIdentifier(imExtendedProto, New(PCommandLineProto, Init('COMMANDLINE')));
    CommandLineLoad:=ccScriptsRoot^.Identifiers^.AddIdentifier(imExtendedProto, New(PCommandLineLoadProto,
      Init('COMMANDLINELOAD')));

    ccStartupHook:=New(PccHook, Init('CCSTARTUP'));

    CompileScript(AutoExecFName);

    ccScriptsRoot^.Identifiers^.Free(CommandLineLoad);
    ccScriptsRoot^.Identifiers^.Free(CommandLine);

    ccStartupHook^.Execute;
   end;
 end;

{ PluginStartup }

procedure PluginStartup; export;
 procedure Perform(const FName: PString); far;
  begin
   CompileScript(FName^);
  end;
 begin
  ccScriptsFirst^.ForEach(@Perform);
  ccScriptsLast^.ForEach(@Perform);
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 var
  K: Longint;
 begin
  if ccStartupHook <> nil then
   Dispose(ccStartupHook, Done);

  Dispose(ccHelpHook, Done);

  Dispose(ccCommandLine, Done);

  for K:=Low(CommandWhenHooks) to High(CommandWhenHooks) do
   Dispose(CommandWhenHooks[K].Hook, Done);

  Dispose(ccScriptsRoot, Done);

  Dispose(ccScriptsLast, Done);
  Dispose(ccScriptsFirst, Done);
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 var
  Pcc: PPccAPI absolute Param;
 begin
  if (Command >= Low(CommandWhenHooks)) and (Command <= High(CommandWhenHooks)) then
   CommandWhenHooks[Command].Hook^.Execute else
  if Command = cccQueryAPI then Pcc^:=@ccAPI.cc;
 end;

{ PluginHandleCmdlineSwitch }

function PluginHandleCmdlineSwitch(Switch: String): Boolean; export;
 begin
  PluginHandleCmdlineSwitch:=True;

  if not ccCommandLine^.Check(Switch) then
   begin
    StUpcaseEx(Switch);

    if Switch = 'CCINFO' then ccCompileInfo:=True else
    if Switch = 'CCDEBUGTOKENIZER' then dbgTokenizer.Enabled:=True else
    if Switch = 'CCDEBUGATOMCREATE' then dbgAtomCreate.Enabled:=True else
    if Switch = 'CCDEBUGCODEEXECUTE' then dbgCodeExecute.Enabled:=True else
    if Switch = 'CCDEBUGRECURSION' then dbgRecursion.Enabled:=True else
    if Switch = 'CCDEBUGAPI' then dbgAPI.Enabled:=True else
     PluginHandleCmdlineSwitch:=False;
   end;
 end;

{ PluginHandleConfigSwitch }

procedure PluginHandleConfigSwitch(const Keyword: String; Value: String; const Linker: Pointer); export;
 var
  S: String;
 begin
  if Keyword = 'SCRIPT' then
   begin
    S:=jt.linker.ParseStringToken(Linker, Value);

    if S = '' then
     jt.linker.Error(Linker, 'invalid script name');

    ccScriptsFirst^.Insert(NewStr(S));
   end
  else
   jt.linker.Error(Linker, 'syntax error');
 end;

{ PluginShowHelp }

procedure PluginShowHelp; export;
 begin
  ccHelpHook^.Execute;
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleCmdlineSwitch,
 PluginHandleConfigSwitch,
 PluginShowHelp;

begin
 Assign(Output, '');
 Rewrite(Output);
end.
