unit ccAPI;

{$I cc.inc}

{.$DEFINE DEBUGSETVARIABLE}

interface
uses
{$IFDEF JT}
     PlugAPI,
     PlugSIDE,
{$ENDIF}

     Objects,
     Strings,

     Wizard;

{ JT supply }

{$IFDEF JT}
const
 cccQueryAPI                            = $1000;
{$ENDIF}

{ general types and constants }

type
 TExecuteResult  = (erOk,               { well done }
                    erExit,             { EXIT operator got }
                    erBreak,            { BREAK operator got }
                    erContinue,         { CONTINUE opeartor got }
                    erStackOverflow);   { stack overflow }

 TTokenMode      = (tmToken,           (* END *)
                    tmComma,           (*  ,  *)
                    tmSemicolon,       (*  ;  *)
                    tmOpenBrace,       (*  {  *)
                    tmCloseBrace,      (*  }  *)
                    tmOpenBracket,     (*  (  *)
                    tmCloseBracket,    (*  )  *)
                    tmString,          (* 'a' *)
                    tmNumber,          (*  1  *)
                    tmColon,           (*  :  *)
                    tmEqual,           (*  =  *)
                    tmPlus,            (*  +  *)
                    tmMinus,           (*  -  *)
                    tmAsterik,         (*  *  *)
                    tmSlash,           (*  /  *)
                    tmSmaller,         (*  <  *)
                    tmLarger,          (*  >  *)
                    tmEnd);            (* EOF *)

 TTokenizerFlag  = (tfSemicolonNecessary,         { ';' after code need }
                    tfAllowBreak,                 { allow BREAK }
                    tfAllowContinue,              { allow CONTINUE }
                    tfRepeatUntil);               { REPEAT..UNTIL syntax }

 TIdentifierMode = (imActionProto,                { action prototype }
                    imStringExpressionProto,      { string expression prototype }
                    imNumberExpressionProto,      { number expression prototype }
                    imBooleanExpressionProto,     { boolean expression prototype }
                    imHookProto,                  { hook prototype }
                    imExtendedProto,              { extended prototype (STRING etc, PROCEDURE, FUNCTION) }
                    imStringOperatorProto,        { s:='a' operator ... }
                    imNumberOperatorProto,        { n:=1 operator ... }
                    imBooleanOperatorProto);      { b:=true operator ... }

 TExpressionType = (etString,                     { 'string' }
                    etNumber,                     { $1234 or 1234 }
                    etBoolean);                   { boolean stuff }

 TParseNumberExpressionFlag = (pnefInvertSign);

 TParseNumberExpressionFlags = set of TParseNumberExpressionFlag;

 TTokenModes     = set of TTokenMode;

 TTokenizerFlags = set of TTokenizerFlag;

 TAtomCreator = function(const ATokenizer, AActions: Pointer): Pointer;
 TOperatorCreator = function(const AExpression, ATokenizer, AActions: Pointer): Pointer;

 TActionExecute = function(const ccAction: Pointer; const Data: Pointer): TExecuteResult;
 TActionDone = procedure(const ccAction: Pointer);

const
 IdentifierModeName: array[TIdentifierMode] of PChar = ('action operator',
                                                        'string expression',
                                                        'number expression',
                                                        'boolean expression',
                                                        'hook name',
                                                        'extended operator',
                                                        'string operator',
                                                        'number operator',
                                                        'boolean operator');

 StringExpressionTokens     : TTokenModes = [tmPlus,
                                             tmToken];

 NumberExpressionTokens     : TTokenModes = [tmPlus,
                                             tmMinus,
                                             tmAsterik,
                                             tmSlash,
                                             tmToken];

 BooleanExpressionTokens    : TTokenModes = [tmToken];

 ExecuteResultNames         : array[TExecuteResult] of PChar =
                                            ('erOk',
                                             'erExit',
                                             'erBreak',
                                             'erContinue',
                                             'erStackOverflow');

 ExpressionTypeToIdentifier : array[TExpressionType] of TIdentifierMode =
                                            (imStringExpressionProto,
                                             imNumberExpressionProto,
                                             imBooleanExpressionProto);

 ExpressionTypeName         : array[TExpressionType] of PChar =
                                            ('string',
                                             'number',
                                             'boolean');

 ValueSizeString            = SizeOf(String);
 ValueSizeNumber            = SizeOf(Longint);
 ValueSizeBoolean           = SizeOf(Boolean);

 ValueSizeMax               = ValueSizeString;

{ API types, variables and constants }

type
 PPccAPI = ^PccAPI;
 PccAPI = ^TccAPI;
 TccAPI = packed record
  RegisterHook: function(const Name: String): Pointer;
  ExecuteHook: function(const ExecuteData, hp: Pointer): TExecuteResult;
  GetHooksCount: function(const hp: Pointer): Longint;
  AddVariable: function(const hp: Pointer; const Name: String; const VariableType: TExpressionType): Pointer;
  AddConstant: function(const hp: Pointer; const Name: String; const VariableType: TExpressionType): Pointer;
  AssignVariable: procedure(const Variable: Pointer; const ExecuteData: Pointer; var Value);
  GetVariable: procedure(const Variable: Pointer; const ExecuteData: Pointer; var Value);
  CreateExecuteData: function: Pointer;
  ResetExecuteData: procedure(const ExecuteData: Pointer);
  DestroyExecuteData: procedure(const ExecuteData: Pointer);
  AddAction: procedure(const hp: Pointer; const Name: String; const Creator: TAtomCreator);
  CreateActionThunk: function(const Name: String; const Owner, Parameter: Pointer; const AExecute: TActionExecute;
    const ADone: TActionDone): Pointer;
  DisposeObject: procedure(const Obj: Pointer);
  TokenizerError: procedure(const Tokenizer: Pointer; const Message: String);
  TokenizerNext: function(const Tokenizer: Pointer; const Modes: TTokenModes): Pointer;
  TokenizerNextSoft: function(const Tokenizer: Pointer; const Modes: TTokenModes): Pointer;
  TokenizerRewind: procedure(const Tokenizer: Pointer);
  ParseStringExpression: function(const Tokenizer, Actions: Pointer; const Continuable: Boolean): Pointer;
  ParseNumberExpression: function(const Tokenizer, Actions: Pointer; const Continuable: Boolean;
    const Flags: TParseNumberExpressionFlags): Pointer;
  ParseBooleanExpression: function(const Tokenizer, Actions: Pointer; const Continuable: Boolean): Pointer;
  ExpressionGet: procedure(const Expression, Data: Pointer; var Value);
  ExpressionAssign: procedure(const Expression, Data: Pointer; var Value);
  ExpressionAssignable: function(const Expression: Pointer): Boolean;
 end;

 PccHook = ^TccHook;

 PccVariable = ^TccVariable;
 TccVariable = object(TObject)
  Owner: PccHook;
  VariableType: TExpressionType;
  VariableName: PString;
  Variable: Pointer;

  constructor Init(const AOwner: PccHook; const AVariableType: TExpressionType; const AVariableName: String;
    const AVariable: Pointer);
  destructor Done; virtual;
 public
  procedure SetString(const S: String);
  procedure SetNumber(const N: Longint);
  procedure SetBoolean(const B: Boolean);
  function GetString: String;
  function GetNumber: Longint;
  function GetBoolean: Boolean;
 end;

 TccHook = object(TStringCollection)
  Name: PString;
  HookProto, ExecuteData: Pointer;

  constructor Init(const AName: String);
  function KeyOf(Item: Pointer): Pointer; virtual;
  procedure FreeItem(Item: Pointer); virtual;
  destructor Done; virtual;
 public
  procedure Execute;
  function HooksCount: Longint;
  function AddVariable(const AName: String; const VariableType: TExpressionType): PccVariable;
  function AddConstant(const AName: String; const VariableType: TExpressionType): PccVariable;
 end;

 PccAction = ^TccAction;
 TccAction = object(TObject)
  constructor Init(const Name: String; const AOwner: Pointer; var Atom: Pointer);
  function Execute(const Data: Pointer): TExecuteResult; virtual;
 end;

var
 cc                             : TccAPI;

const
 ccAvailable                    : Boolean               = False;

function ccAction_Execute(const ccAction: Pointer; const Data: Pointer): TExecuteResult; export;
procedure ccAction_Done(const ccAction: Pointer); export;

{$IFDEF JT}
procedure InitCCAPI;
{$ENDIF}

implementation
{$IFDEF DEBUGSETVARIABLE}
uses
     ccDepend;
{$ENDIF}

const
 BooleanTrue            : Boolean       = True;
 BooleanFalse           : Boolean       = False;

 BooleanNames           : array[Boolean] of String[5] = ('FALSE', 'TRUE');
 BooleanNumbers         : array[Boolean] of Longint = (0, 1);

{ StringToBoolean }

function StringToBoolean(const S: String): Boolean;
 begin
  if S = 'TRUE' then
   StringToBoolean:=True
  else
   StringToBoolean:=False;
 end;

{ TccVariable }

constructor TccVariable.Init(const AOwner: PccHook; const AVariableType: TExpressionType; const AVariableName: String;
  const AVariable: Pointer);
 begin
  inherited Init;

  Owner:=AOwner;

  VariableType:=AVariableType;
  VariableName:=NewStr(AVariableName);
  Variable:=AVariable;
 end;

destructor TccVariable.Done;
 begin
  DisposeStr(VariableName);

  inherited Done;
 end;

{ public methods }

procedure TccVariable.SetString(const S: String);
 var
  Number: Longint;
 begin
  {$IFDEF DEBUGSETVARIABLE}
  depWarning('setting ' + VariableName^ + ' (' + StrPas(ExpressionTypeName[VariableType]) + ') to "' + S + '"');
  {$ENDIF}

  case VariableType of
   etString: cc.AssignVariable(Variable, Owner^.ExecuteData, (@S)^);
   etNumber:
    if Str2Longint(S, Number) then
     cc.AssignVariable(Variable, Owner^.ExecuteData, Number);
   etBoolean:
    if StringToBoolean(S) then
     cc.AssignVariable(Variable, Owner^.ExecuteData, BooleanTrue)
    else
     cc.AssignVariable(Variable, Owner^.ExecuteData, BooleanFalse);
  end;
 end;

procedure TccVariable.SetNumber(const N: Longint);
 var
  TempString: PString;
 begin
  {$IFDEF DEBUGSETVARIABLE}
  depWarning('setting ' + VariableName^ + ' (' + StrPas(ExpressionTypeName[VariableType]) + ') to "' + Long2Str(N) + '"');
  {$ENDIF}

  case VariableType of
   etString:
    begin
     New(TempString);

     TempString^:=Long2Str(N);

     cc.AssignVariable(Variable, Owner^.ExecuteData, TempString^);

     Dispose(TempString);
    end;
   etNumber:
    cc.AssignVariable(Variable, Owner^.ExecuteData, (@N)^);
   etBoolean:
    if N = 0 then cc.AssignVariable(Variable, Owner^.ExecuteData, BooleanFalse) else
    if N = 1 then cc.AssignVariable(Variable, Owner^.ExecuteData, BooleanTrue);
  end;
 end;

procedure TccVariable.SetBoolean(const B: Boolean);
 begin
  {$IFDEF DEBUGSETVARIABLE}
  if B then
   depWarning('setting ' + VariableName^ + ' (' + StrPas(ExpressionTypeName[VariableType]) + ') to "TRUE"')
  else
   depWarning('setting ' + VariableName^ + ' (' + StrPas(ExpressionTypeName[VariableType]) + ') to "FALSE"');
  {$ENDIF}

  case VariableType of
   etString: cc.AssignVariable(Variable, Owner^.ExecuteData, BooleanNames[B]);
   etNumber: cc.AssignVariable(Variable, Owner^.ExecuteData, BooleanNumbers[B]);
   etBoolean: cc.AssignVariable(Variable, Owner^.ExecuteData, (@B)^);
  end;
 end;

function TccVariable.GetString: String;
 var
  S: String;
  N: Longint;
  B: Boolean;
 begin
  case VariableType of
   etString: cc.GetVariable(Variable, Owner^.ExecuteData, S);
   etNumber:
    begin
     cc.GetVariable(Variable, Owner^.ExecuteData, N);

     S:=Long2Str(N);
    end;
   etBoolean:
    begin
     cc.GetVariable(Variable, Owner^.ExecuteData, B);

     S:=BooleanNames[B];
    end;
  end;

  GetString:=S;
 end;

function TccVariable.GetNumber: Longint;
 var
  S: String;
  N: Longint;
  B: Boolean;
 begin
  case VariableType of
   etString:
    begin
     cc.GetVariable(Variable, Owner^.ExecuteData, S);

     if not Str2Longint(S, N) then
      N:=0;
    end;
   etNumber: cc.GetVariable(Variable, Owner^.ExecuteData, N);
   etBoolean:
    begin
     cc.GetVariable(Variable, Owner^.ExecuteData, B);

     N:=BooleanNumbers[B];
    end;
  end;

  GetNumber:=N;
 end;

function TccVariable.GetBoolean: Boolean;
 var
  S: String;
  N: Longint;
  B: Boolean;
 begin
  case VariableType of
   etString:
    begin
     cc.GetVariable(Variable, Owner^.ExecuteData, S);

     B:=StringToBoolean(S);
    end;
   etNumber:
    begin
     cc.GetVariable(Variable, Owner^.ExecuteData, N);

     B:=LongBool(N);
    end;
   etBoolean: cc.GetVariable(Variable, Owner^.ExecuteData, B);
  end;

  GetBoolean:=B;
 end;

{ TccHook }

constructor TccHook.Init(const AName: String);
 begin
  inherited Init($10, $10);

  Name:=NewStr(AName);

  HookProto:=cc.RegisterHook(AName);

  ExecuteData:=cc.CreateExecuteData;
 end;

function TccHook.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=PccVariable(Item)^.VariableName;
 end;

procedure TccHook.FreeItem(Item: Pointer);
 begin
  TCollection.FreeItem(Item);
 end;

destructor TccHook.Done;
 begin
  cc.DestroyExecuteData(ExecuteData);

  DisposeStr(Name);

  inherited Done;
 end;

{ public methods }

procedure TccHook.Execute;
 begin
  cc.ResetExecuteData(ExecuteData);

  cc.ExecuteHook(ExecuteData, HookProto);
 end;

function TccHook.HooksCount: Longint;
 begin
  HooksCount:=cc.GetHooksCount(HookProto);
 end;

function TccHook.AddVariable(const AName: String; const VariableType: TExpressionType): PccVariable;
 var
  ccVariable: PccVariable;
  Variable: Pointer;
  Index: xInteger;
 begin
  if Search(@AName, Index) then
   AddVariable:=At(Index)
  else
   begin
    Variable:=cc.AddVariable(HookProto, AName, VariableType);

    ccVariable:=New(PccVariable, Init(@Self, VariableType, AName, Variable));

    AtInsert(Index, ccVariable);

    AddVariable:=ccVariable;
   end;
 end;

function TccHook.AddConstant(const AName: String; const VariableType: TExpressionType): PccVariable;
 var
  ccVariable: PccVariable;
  Variable: Pointer;
  Index: xInteger;
 begin
  if Search(@AName, Index) then
   AddConstant:=At(Index)
  else
   begin
    Variable:=cc.AddConstant(HookProto, AName, VariableType);

    ccVariable:=New(PccVariable, Init(@Self, VariableType, AName, Variable));

    AtInsert(Index, ccVariable);

    AddConstant:=ccVariable;
   end;
 end;

{ TccAction }

constructor TccAction.Init(const Name: String; const AOwner: Pointer; var Atom: Pointer);
 begin
  inherited Init;

  Atom:=cc.CreateActionThunk(Name, AOwner, @Self, ccAction_Execute, ccAction_Done);
 end;

function TccAction.Execute(const Data: Pointer): TExecuteResult;
 begin
  Abstract;
 end;

{ ccAction thunks }

function ccAction_Execute(const ccAction: Pointer; const Data: Pointer): TExecuteResult;
 begin
  ccAction_Execute:=PccAction(ccAction)^.Execute(Data);
 end;

procedure ccAction_Done(const ccAction: Pointer);
 begin
  Dispose(PccAction(ccAction), Done);
 end;

{ InitCCAPI }

{$IFDEF JT}
procedure InitCCAPI;
 var
  Plugin: Pointer;
  PluginHandleCommand: TPluginHandleCommand;
  ccAPI: PccAPI;
 begin
  Plugin:=jt.plugs.Seek('CarbonCopierEx');

  if Plugin <> nil then
   begin
    jt.plugs.GetPluginHandleCommand(Plugin, PluginHandleCommand);

    PluginHandleCommand(cccQueryAPI, Longint(@ccAPI));

    cc:=ccAPI^;

    ccAvailable:=True;
   end;
 end;
{$ENDIF}

end.
