{$IFNDEF VIRTUALPASCAL}
{$D-}
{$ENDIF}
unit ccJTAPI4;

interface
uses
     PlugAPI,
     PlugSIDE,

     Objects,
     Strings,

     ccAPI,
     ccAction,
     ccCmn,
     ccCode,
     ccExpr,
     ccJTSupp,
     ccProto,
     ccRoot,
     ccSupp,
     ccToken,

     jtMHLcmn,

     Thunks,

     Wizard;

procedure AddJTAPIProtos(const Target: PActions);

implementation

{ GetError: String; }

type
 PGetError = ^TGetError;
 TGetError = object(TStringExpression)
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
 end;

procedure TGetError.Get(const ExecuteData: PExecuteData; var Value);
 begin
  String(Value):=jt.GetError;
 end;

{ LockCommandLine(const Plugin: Pointer); }

type
 PLockCommandLine = ^TLockCommandLine;
 TLockCommandLine = object(TAction)
  _Plugin: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TLockCommandLine.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('LockCommandLine', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Plugin:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TLockCommandLine.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __Plugin: Pointer;
 begin
  _Plugin^.Get(Data, __Plugin);

  jt.LockCommandLine(__Plugin);

  Execute:=erOk;
 end;

destructor TLockCommandLine.Done;
 begin
  Dispose(_Plugin, Done);

  inherited Done;
 end;

{ Execute(const CmdLine: String; var rc: Longint; const Hide: Boolean); }

type
 PExecute = ^TExecute;
 TExecute = object(TAction)
  _CmdLine: PStringExpression;
  _rc: PNumberExpression;
  _Hide: PBooleanExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TExecute.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('Execute', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _CmdLine:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _rc:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  if not _rc^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etNumber);

  Tokenizer^.Next([tmComma]);

  _Hide:=Actions^.ParseBooleanExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TExecute.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __CmdLine: String;
  __rc: Longint;
  __Hide: Boolean;
 begin
  _CmdLine^.Get(Data, __CmdLine);
  _rc^.Get(Data, __rc);
  _Hide^.Get(Data, __Hide);

  jt.Execute(__CmdLine, 
    __rc, 
    __Hide);

  _rc^.Assign(Data, __rc);

  Execute:=erOk;
 end;

destructor TExecute.Done;
 begin
  Dispose(_CmdLine, Done);
  Dispose(_rc, Done);
  Dispose(_Hide, Done);

  inherited Done;
 end;

{ MoveFile(const ASource, ADestination: String; var rc: Longint); }

type
 PMoveFile = ^TMoveFile;
 TMoveFile = object(TAction)
  _ASource: PStringExpression;
  _ADestination: PStringExpression;
  _rc: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TMoveFile.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('MoveFile', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _ASource:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _ADestination:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _rc:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  if not _rc^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etNumber);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TMoveFile.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __ASource: String;
  __ADestination: String;
  __rc: Longint;
 begin
  _ASource^.Get(Data, __ASource);
  _ADestination^.Get(Data, __ADestination);
  _rc^.Get(Data, __rc);

  jt.MoveFile(__ASource, 
    __ADestination, 
    __rc);

  _rc^.Assign(Data, __rc);

  Execute:=erOk;
 end;

destructor TMoveFile.Done;
 begin
  Dispose(_ASource, Done);
  Dispose(_ADestination, Done);
  Dispose(_rc, Done);

  inherited Done;
 end;

{ CreateDirectory(const ADirectory: String): Boolean; }

type
 PCreateDirectory = ^TCreateDirectory;
 TCreateDirectory = object(TBooleanExpression)
  _ADirectory: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TCreateDirectory.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('CreateDirectory', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _ADirectory:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TCreateDirectory.Get(const ExecuteData: PExecuteData; var Value);
 var
  __ADirectory: String;
 begin
  _ADirectory^.Get(ExecuteData, __ADirectory);

  Boolean(Value):=jt.CreateDirectory(__ADirectory);
 end;

destructor TCreateDirectory.Done;
 begin
  Dispose(_ADirectory, Done);

  inherited Done;
 end;

{ PushTitle; }

type
 PPushTitle = ^TPushTitle;
 TPushTitle = object(TAction)
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
 end;

function TPushTitle.Execute(const Data: PExecuteData): TExecuteResult;
 begin
  jt.PushTitle;

  Execute:=erOk;
 end;

{ SetTitle(const S: String); }

type
 PSetTitle = ^TSetTitle;
 TSetTitle = object(TAction)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TSetTitle.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('SetTitle', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TSetTitle.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __S: String;
 begin
  _S^.Get(Data, __S);

  jt.SetTitle(__S);

  Execute:=erOk;
 end;

destructor TSetTitle.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ PopTitle; }

type
 PPopTitle = ^TPopTitle;
 TPopTitle = object(TAction)
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
 end;

function TPopTitle.Execute(const Data: PExecuteData): TExecuteResult;
 begin
  jt.PopTitle;

  Execute:=erOk;
 end;

{ MakeShortName(const LongName: String): String; }

type
 PMakeShortName = ^TMakeShortName;
 TMakeShortName = object(TStringExpression)
  _LongName: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TMakeShortName.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('MakeShortName', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _LongName:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TMakeShortName.Get(const ExecuteData: PExecuteData; var Value);
 var
  __LongName: String;
 begin
  _LongName^.Get(ExecuteData, __LongName);

  String(Value):=jt.MakeShortName(__LongName);
 end;

destructor TMakeShortName.Done;
 begin
  Dispose(_LongName, Done);

  inherited Done;
 end;

{ LogWrite(const Symbol: Char; const Message: String); }

type
 PLogWrite = ^TLogWrite;
 TLogWrite = object(TAction)
  _Symbol: PStringExpression;
  _Message: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TLogWrite.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('LogWrite', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Symbol:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Message:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TLogWrite.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __Symbol: String;
  __Message: String;
 begin
  _Symbol^.Get(Data, __Symbol);
  _Message^.Get(Data, __Message);

  jt.LogWrite(String2Char(__Symbol), 
    __Message);

  Execute:=erOk;
 end;

destructor TLogWrite.Done;
 begin
  Dispose(_Symbol, Done);
  Dispose(_Message, Done);

  inherited Done;
 end;

{ SetStatus(const Status: String); }

type
 PSetStatus = ^TSetStatus;
 TSetStatus = object(TAction)
  _Status: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TSetStatus.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('SetStatus', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Status:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TSetStatus.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __Status: String;
 begin
  _Status^.Get(Data, __Status);

  jt.SetStatus(__Status);

  Execute:=erOk;
 end;

destructor TSetStatus.Done;
 begin
  Dispose(_Status, Done);

  inherited Done;
 end;

{ ClearStatus; }

type
 PClearStatus = ^TClearStatus;
 TClearStatus = object(TAction)
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
 end;

function TClearStatus.Execute(const Data: PExecuteData): TExecuteResult;
 begin
  jt.ClearStatus;

  Execute:=erOk;
 end;

{ EnqueueCommand(const Plugin: Pointer; const Command, Param: Longint;
   const When: TCommandWhen); }

type
 PEnqueueCommand = ^TEnqueueCommand;
 TEnqueueCommand = object(TAction)
  _Plugin: PNumberExpression;
  _Command: PNumberExpression;
  _Param: PNumberExpression;
  _When: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TEnqueueCommand.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('EnqueueCommand', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Plugin:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmComma]);

  _Command:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmComma]);

  _Param:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmComma]);

  _When:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TEnqueueCommand.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __Plugin: Pointer;
  __Command: Longint;
  __Param: Longint;
  __When: record case Byte of 1: (When: TCommandWhen); 2: (Junk: Longint) end;
 begin
  _Plugin^.Get(Data, __Plugin);
  _Command^.Get(Data, __Command);
  _Param^.Get(Data, __Param);
  _When^.Get(Data, __When);

  jt.EnqueueCommand(__Plugin, 
    __Command, 
    __Param, 
    __When.When);

  Execute:=erOk;
 end;

destructor TEnqueueCommand.Done;
 begin
  Dispose(_Plugin, Done);
  Dispose(_Command, Done);
  Dispose(_Param, Done);
  Dispose(_When, Done);

  inherited Done;
 end;

{ EnqueueCommandEx(const Plugin: String; const Command, Param: Longint;
   const When: TCommandWhen); }

type
 PEnqueueCommandEx = ^TEnqueueCommandEx;
 TEnqueueCommandEx = object(TAction)
  _Plugin: PStringExpression;
  _Command: PNumberExpression;
  _Param: PNumberExpression;
  _When: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TEnqueueCommandEx.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('EnqueueCommandEx', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Plugin:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Command:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmComma]);

  _Param:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmComma]);

  _When:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TEnqueueCommandEx.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __Plugin: String;
  __Command: Longint;
  __Param: Longint;
  __When: record case Byte of 1: (When: TCommandWhen); 2: (Junk: Longint) end;
 begin
  _Plugin^.Get(Data, __Plugin);
  _Command^.Get(Data, __Command);
  _Param^.Get(Data, __Param);
  _When^.Get(Data, __When);

  jt.EnqueueCommandEx(__Plugin, 
    __Command, 
    __Param, 
    __When.When);

  Execute:=erOk;
 end;

destructor TEnqueueCommandEx.Done;
 begin
  Dispose(_Plugin, Done);
  Dispose(_Command, Done);
  Dispose(_Param, Done);
  Dispose(_When, Done);

  inherited Done;
 end;

{ Report(const S: String; const Log: Boolean); }

type
 PReport = ^TReport;
 TReport = object(TAction)
  _S: PStringExpression;
  _Log: PBooleanExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TReport.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('Report', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Log:=Actions^.ParseBooleanExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TReport.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __S: String;
  __Log: Boolean;
 begin
  _S^.Get(Data, __S);
  _Log^.Get(Data, __Log);

  jt.Report(__S, 
    __Log);

  Execute:=erOk;
 end;

destructor TReport.Done;
 begin
  Dispose(_S, Done);
  Dispose(_Log, Done);

  inherited Done;
 end;

{ ReportAreaAction(const Name: String; const Stream: Pointer; const
   StreamLinker: PStreamLinker; const Plugin: Pointer); }

type
 PReportAreaAction = ^TReportAreaAction;
 TReportAreaAction = object(TAction)
  _Name: PStringExpression;
  _Stream: PNumberExpression;
  _StreamLinker: PNumberExpression;
  _Plugin: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TReportAreaAction.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('ReportAreaAction', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Name:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Stream:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmComma]);

  _StreamLinker:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmComma]);

  _Plugin:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TReportAreaAction.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __Name: String;
  __Stream: Pointer;
  __StreamLinker: PStreamLinker;
  __Plugin: Pointer;
 begin
  _Name^.Get(Data, __Name);
  _Stream^.Get(Data, __Stream);
  _StreamLinker^.Get(Data, __StreamLinker);
  _Plugin^.Get(Data, __Plugin);

  jt.ReportAreaAction(__Name, 
    __Stream, 
    __StreamLinker, 
    __Plugin);

  Execute:=erOk;
 end;

destructor TReportAreaAction.Done;
 begin
  Dispose(_Name, Done);
  Dispose(_Stream, Done);
  Dispose(_StreamLinker, Done);
  Dispose(_Plugin, Done);

  inherited Done;
 end;

{ DoAreaActions(const APlugin: Pointer); }

type
 PDoAreaActions = ^TDoAreaActions;
 TDoAreaActions = object(TAction)
  _APlugin: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TDoAreaActions.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('DoAreaActions', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _APlugin:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TDoAreaActions.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __APlugin: Pointer;
 begin
  _APlugin^.Get(Data, __APlugin);

  jt.DoAreaActions(__APlugin);

  Execute:=erOk;
 end;

destructor TDoAreaActions.Done;
 begin
  Dispose(_APlugin, Done);

  inherited Done;
 end;

{ GetCommaToken(var S: String): String; }

type
 PGetCommaToken = ^TGetCommaToken;
 TGetCommaToken = object(TStringExpression)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TGetCommaToken.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('GetCommaToken', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  if not _S^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etString);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TGetCommaToken.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
 begin
  _S^.Get(ExecuteData, __S);

  String(Value):=jt.GetCommaToken(__S);

  _S^.Assign(ExecuteData, __S);
 end;

destructor TGetCommaToken.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ NewStr(const S: String): PString; }

type
 PNewStr = ^TNewStr;
 TNewStr = object(TNumberExpression)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TNewStr.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('NewStr', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TNewStr.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
 begin
  _S^.Get(ExecuteData, __S);

  Longint(Value):=Longint(jt.NewStr(__S));
 end;

destructor TNewStr.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ DisposeStr(const S: PString); }

type
 PDisposeStr = ^TDisposeStr;
 TDisposeStr = object(TAction)
  _S: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TDisposeStr.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('DisposeStr', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TDisposeStr.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __S: PString;
 begin
  _S^.Get(Data, __S);

  jt.DisposeStr(__S);

  Execute:=erOk;
 end;

destructor TDisposeStr.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ StreamLinker: PStreamLinker; }

type
 PStreamLinker = ^TStreamLinker;
 TStreamLinker = object(TNumberExpression)
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
 end;

procedure TStreamLinker.Get(const ExecuteData: PExecuteData; var Value);
 begin
  Longint(Value):=Longint(jt.StreamLinker);
 end;

{ MinimizeHandles; }

type
 PMinimizeHandles = ^TMinimizeHandles;
 TMinimizeHandles = object(TAction)
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
 end;

function TMinimizeHandles.Execute(const Data: PExecuteData): TExecuteResult;
 begin
  jt.MinimizeHandles;

  Execute:=erOk;
 end;

{ FormatTime(const Time: Longint): String; }

type
 PFormatTime = ^TFormatTime;
 TFormatTime = object(TStringExpression)
  _Time: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TFormatTime.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('FormatTime', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Time:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TFormatTime.Get(const ExecuteData: PExecuteData; var Value);
 var
  __Time: Longint;
 begin
  _Time^.Get(ExecuteData, __Time);

  String(Value):=jt.FormatTime(__Time);
 end;

destructor TFormatTime.Done;
 begin
  Dispose(_Time, Done);

  inherited Done;
 end;

{ StrNew(const Str: PChar): PChar; }

type
 PStrNew = ^TStrNew;
 TStrNew = object(TNumberExpression)
  _Str: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TStrNew.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('StrNew', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Str:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TStrNew.Get(const ExecuteData: PExecuteData; var Value);
 var
  __Str: PChar;
 begin
  _Str^.Get(ExecuteData, __Str);

  Longint(Value):=Longint(jt.StrNew(__Str));
 end;

destructor TStrNew.Done;
 begin
  Dispose(_Str, Done);

  inherited Done;
 end;

{ StrAlloc(const S: String): PChar; }

type
 PStrAlloc = ^TStrAlloc;
 TStrAlloc = object(TNumberExpression)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TStrAlloc.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('StrAlloc', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TStrAlloc.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
 begin
  _S^.Get(ExecuteData, __S);

  Longint(Value):=Longint(jt.StrAlloc(__S));
 end;

destructor TStrAlloc.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ StrDispose(const Str: PChar); }

type
 PStrDispose = ^TStrDispose;
 TStrDispose = object(TAction)
  _Str: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TStrDispose.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('StrDispose', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Str:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TStrDispose.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __Str: PChar;
 begin
  _Str^.Get(Data, __Str);

  jt.StrDispose(__Str);

  Execute:=erOk;
 end;

destructor TStrDispose.Done;
 begin
  Dispose(_Str, Done);

  inherited Done;
 end;

{ COA: Longint; }

type
 PCOA = ^TCOA;
 TCOA = object(TNumberExpression)
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
 end;

procedure TCOA.Get(const ExecuteData: PExecuteData; var Value);
 begin
  Longint(Value):=Longint(jt.COA);
 end;

{ POA: Longint; }

type
 PPOA = ^TPOA;
 TPOA = object(TNumberExpression)
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
 end;

procedure TPOA.Get(const ExecuteData: PExecuteData; var Value);
 begin
  Longint(Value):=Longint(jt.POA);
 end;

{ NewAddressEx(const AZone, ANet, ANode, APoint: System.Integer):
   PAddress; }

type
 PNewAddressEx = ^TNewAddressEx;
 TNewAddressEx = object(TNumberExpression)
  _AZone: PNumberExpression;
  _ANet: PNumberExpression;
  _ANode: PNumberExpression;
  _APoint: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TNewAddressEx.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('NewAddressEx', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _AZone:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmComma]);

  _ANet:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmComma]);

  _ANode:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmComma]);

  _APoint:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TNewAddressEx.Get(const ExecuteData: PExecuteData; var Value);
 var
  __AZone: Longint;
  __ANet: Longint;
  __ANode: Longint;
  __APoint: Longint;
 begin
  _AZone^.Get(ExecuteData, __AZone);
  _ANet^.Get(ExecuteData, __ANet);
  _ANode^.Get(ExecuteData, __ANode);
  _APoint^.Get(ExecuteData, __APoint);

  Longint(Value):=Longint(jt.NewAddressEx(__AZone, 
    __ANet, 
    __ANode, 
    __APoint));
 end;

destructor TNewAddressEx.Done;
 begin
  Dispose(_AZone, Done);
  Dispose(_ANet, Done);
  Dispose(_ANode, Done);
  Dispose(_APoint, Done);

  inherited Done;
 end;

{ RegisterMessageBaseInterface(const mbi: Pointer); }

type
 PRegisterMessageBaseInterface = ^TRegisterMessageBaseInterface;
 TRegisterMessageBaseInterface = object(TAction)
  _mbi: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TRegisterMessageBaseInterface.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('RegisterMessageBaseInterface', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _mbi:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TRegisterMessageBaseInterface.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __mbi: Pointer;
 begin
  _mbi^.Get(Data, __mbi);

  jt.RegisterMessageBaseInterface(__mbi);

  Execute:=erOk;
 end;

destructor TRegisterMessageBaseInterface.Done;
 begin
  Dispose(_mbi, Done);

  inherited Done;
 end;

{ JokeSwitch(const Normal, Joke: String): String; }

type
 PJokeSwitch = ^TJokeSwitch;
 TJokeSwitch = object(TStringExpression)
  _Normal: PStringExpression;
  _Joke: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TJokeSwitch.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('JokeSwitch', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Normal:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Joke:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TJokeSwitch.Get(const ExecuteData: PExecuteData; var Value);
 var
  __Normal: String;
  __Joke: String;
 begin
  _Normal^.Get(ExecuteData, __Normal);
  _Joke^.Get(ExecuteData, __Joke);

  String(Value):=jt.JokeSwitch(__Normal, 
    __Joke);
 end;

destructor TJokeSwitch.Done;
 begin
  Dispose(_Normal, Done);
  Dispose(_Joke, Done);

  inherited Done;
 end;

{ ReplaceMacros(var S: String); }

type
 PReplaceMacros = ^TReplaceMacros;
 TReplaceMacros = object(TAction)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TReplaceMacros.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('ReplaceMacros', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  if not _S^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etString);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TReplaceMacros.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __S: String;
 begin
  _S^.Get(Data, __S);

  jt.ReplaceMacros(__S);

  _S^.Assign(Data, __S);

  Execute:=erOk;
 end;

destructor TReplaceMacros.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ StLoCase(S: String): String; }

type
 PStLoCase = ^TStLoCase;
 TStLoCase = object(TStringExpression)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TStLoCase.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('StLoCase', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TStLoCase.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
 begin
  _S^.Get(ExecuteData, __S);

  String(Value):=Wizard.StLoCase(__S);
 end;

destructor TStLoCase.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ StUpCase(S: String): String; }

type
 PStUpCase = ^TStUpCase;
 TStUpCase = object(TStringExpression)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TStUpCase.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('StUpCase', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TStUpCase.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
 begin
  _S^.Get(ExecuteData, __S);

  String(Value):=Wizard.StUpCase(__S);
 end;

destructor TStUpCase.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ StLocaseEx(var S: String); }

type
 PStLocaseEx = ^TStLocaseEx;
 TStLocaseEx = object(TAction)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TStLocaseEx.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('StLocaseEx', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  if not _S^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etString);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TStLocaseEx.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __S: String;
 begin
  _S^.Get(Data, __S);

  Wizard.StLocaseEx(__S);

  _S^.Assign(Data, __S);

  Execute:=erOk;
 end;

destructor TStLocaseEx.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ StUpcaseEx(var S: String); }

type
 PStUpcaseEx = ^TStUpcaseEx;
 TStUpcaseEx = object(TAction)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TStUpcaseEx.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('StUpcaseEx', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  if not _S^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etString);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TStUpcaseEx.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __S: String;
 begin
  _S^.Get(Data, __S);

  Wizard.StUpcaseEx(__S);

  _S^.Assign(Data, __S);

  Execute:=erOk;
 end;

destructor TStUpcaseEx.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ StrLocase(P: PChar); }

type
 PStrLocase = ^TStrLocase;
 TStrLocase = object(TAction)
  _P: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TStrLocase.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('StrLocase', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _P:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TStrLocase.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __P: PChar;
 begin
  _P^.Get(Data, __P);

  Wizard.StrLocase(__P);

  Execute:=erOk;
 end;

destructor TStrLocase.Done;
 begin
  Dispose(_P, Done);

  inherited Done;
 end;

{ StrUpcase(P: PChar); }

type
 PStrUpcase = ^TStrUpcase;
 TStrUpcase = object(TAction)
  _P: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TStrUpcase.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('StrUpcase', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _P:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TStrUpcase.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __P: PChar;
 begin
  _P^.Get(Data, __P);

  Wizard.StrUpcase(__P);

  Execute:=erOk;
 end;

destructor TStrUpcase.Done;
 begin
  Dispose(_P, Done);

  inherited Done;
 end;

{ LeftPad(const S: String; const Len: Byte): String; }

type
 PLeftPad = ^TLeftPad;
 TLeftPad = object(TStringExpression)
  _S: PStringExpression;
  _Len: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TLeftPad.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('LeftPad', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Len:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TLeftPad.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
  __Len: Longint;
 begin
  _S^.Get(ExecuteData, __S);
  _Len^.Get(ExecuteData, __Len);

  String(Value):=Wizard.LeftPad(__S, 
    __Len);
 end;

destructor TLeftPad.Done;
 begin
  Dispose(_S, Done);
  Dispose(_Len, Done);

  inherited Done;
 end;

{ LeftPadCh(const S: String; const Ch: Char; const Len: Byte): String; }

type
 PLeftPadCh = ^TLeftPadCh;
 TLeftPadCh = object(TStringExpression)
  _S: PStringExpression;
  _Ch: PStringExpression;
  _Len: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TLeftPadCh.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('LeftPadCh', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Ch:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Len:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TLeftPadCh.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
  __Ch: String;
  __Len: Longint;
 begin
  _S^.Get(ExecuteData, __S);
  _Ch^.Get(ExecuteData, __Ch);
  _Len^.Get(ExecuteData, __Len);

  String(Value):=Wizard.LeftPadCh(__S, 
    String2Char(__Ch), 
    __Len);
 end;

destructor TLeftPadCh.Done;
 begin
  Dispose(_S, Done);
  Dispose(_Ch, Done);
  Dispose(_Len, Done);

  inherited Done;
 end;

{ Center(const S: String; const Width: Byte): String; }

type
 PCenter = ^TCenter;
 TCenter = object(TStringExpression)
  _S: PStringExpression;
  _Width: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TCenter.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('Center', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Width:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TCenter.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
  __Width: Longint;
 begin
  _S^.Get(ExecuteData, __S);
  _Width^.Get(ExecuteData, __Width);

  String(Value):=Wizard.Center(__S, 
    __Width);
 end;

destructor TCenter.Done;
 begin
  Dispose(_S, Done);
  Dispose(_Width, Done);

  inherited Done;
 end;

{ CenterCh(const S: String; const Ch: Char; const Width: Byte):
   String; }

type
 PCenterCh = ^TCenterCh;
 TCenterCh = object(TStringExpression)
  _S: PStringExpression;
  _Ch: PStringExpression;
  _Width: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TCenterCh.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('CenterCh', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Ch:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Width:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TCenterCh.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
  __Ch: String;
  __Width: Longint;
 begin
  _S^.Get(ExecuteData, __S);
  _Ch^.Get(ExecuteData, __Ch);
  _Width^.Get(ExecuteData, __Width);

  String(Value):=Wizard.CenterCh(__S, 
    String2Char(__Ch), 
    __Width);
 end;

destructor TCenterCh.Done;
 begin
  Dispose(_S, Done);
  Dispose(_Ch, Done);
  Dispose(_Width, Done);

  inherited Done;
 end;

{ Pad(const S: String; const Len: Byte): String; }

type
 PPad = ^TPad;
 TPad = object(TStringExpression)
  _S: PStringExpression;
  _Len: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TPad.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('Pad', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Len:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TPad.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
  __Len: Longint;
 begin
  _S^.Get(ExecuteData, __S);
  _Len^.Get(ExecuteData, __Len);

  String(Value):=Wizard.Pad(__S, 
    __Len);
 end;

destructor TPad.Done;
 begin
  Dispose(_S, Done);
  Dispose(_Len, Done);

  inherited Done;
 end;

{ PadCh(const S: String; const Ch: Char; const Len: Byte): String; }

type
 PPadCh = ^TPadCh;
 TPadCh = object(TStringExpression)
  _S: PStringExpression;
  _Ch: PStringExpression;
  _Len: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TPadCh.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('PadCh', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Ch:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Len:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TPadCh.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
  __Ch: String;
  __Len: Longint;
 begin
  _S^.Get(ExecuteData, __S);
  _Ch^.Get(ExecuteData, __Ch);
  _Len^.Get(ExecuteData, __Len);

  String(Value):=Wizard.PadCh(__S, 
    String2Char(__Ch), 
    __Len);
 end;

destructor TPadCh.Done;
 begin
  Dispose(_S, Done);
  Dispose(_Ch, Done);
  Dispose(_Len, Done);

  inherited Done;
 end;

{ LTrim(S: String): String; }

type
 PLTrim = ^TLTrim;
 TLTrim = object(TStringExpression)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TLTrim.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('LTrim', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TLTrim.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
 begin
  _S^.Get(ExecuteData, __S);

  String(Value):=Wizard.LTrim(__S);
 end;

destructor TLTrim.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ Trim(S: String): String; }

type
 PTrim = ^TTrim;
 TTrim = object(TStringExpression)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TTrim.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('Trim', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TTrim.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
 begin
  _S^.Get(ExecuteData, __S);

  String(Value):=Wizard.Trim(__S);
 end;

destructor TTrim.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ TrimEx(var S: String); }

type
 PTrimEx = ^TTrimEx;
 TTrimEx = object(TAction)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TTrimEx.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('TrimEx', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  if not _S^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etString);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TTrimEx.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __S: String;
 begin
  _S^.Get(Data, __S);

  Wizard.TrimEx(__S);

  _S^.Assign(Data, __S);

  Execute:=erOk;
 end;

destructor TTrimEx.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ RTrim(S: String): String; }

type
 PRTrim = ^TRTrim;
 TRTrim = object(TStringExpression)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TRTrim.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('RTrim', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TRTrim.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
 begin
  _S^.Get(ExecuteData, __S);

  String(Value):=Wizard.RTrim(__S);
 end;

destructor TRTrim.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ ExtractWord(const N: Byte; const S: String; const WordDelims:
   TCharSet): String; }

type
 PExtractWord = ^TExtractWord;
 TExtractWord = object(TStringExpression)
  _N: PNumberExpression;
  _S: PStringExpression;
  _WordDelims: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TExtractWord.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('ExtractWord', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _N:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmComma]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _WordDelims:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TExtractWord.Get(const ExecuteData: PExecuteData; var Value);
 var
  __N: Longint;
  __S: String;
  __WordDelims: String;
 begin
  _N^.Get(ExecuteData, __N);
  _S^.Get(ExecuteData, __S);
  _WordDelims^.Get(ExecuteData, __WordDelims);

  String(Value):=Wizard.ExtractWord(__N, 
    __S, 
    String2CharSet(__WordDelims)^);
 end;

destructor TExtractWord.Done;
 begin
  Dispose(_N, Done);
  Dispose(_S, Done);
  Dispose(_WordDelims, Done);

  inherited Done;
 end;

{ WordCount(const S: String; const WordDelims: TCharSet): Byte; }

type
 PWordCount = ^TWordCount;
 TWordCount = object(TNumberExpression)
  _S: PStringExpression;
  _WordDelims: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TWordCount.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('WordCount', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _WordDelims:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TWordCount.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
  __WordDelims: String;
 begin
  _S^.Get(ExecuteData, __S);
  _WordDelims^.Get(ExecuteData, __WordDelims);

  Longint(Value):=Longint(Wizard.WordCount(__S, 
    String2CharSet(__WordDelims)^));
 end;

destructor TWordCount.Done;
 begin
  Dispose(_S, Done);
  Dispose(_WordDelims, Done);

  inherited Done;
 end;

{ GetAllAfterChar(const S: String; const Spc: Byte; const Ch: Char):
   String; }

type
 PGetAllAfterChar = ^TGetAllAfterChar;
 TGetAllAfterChar = object(TStringExpression)
  _S: PStringExpression;
  _Spc: PNumberExpression;
  _Ch: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TGetAllAfterChar.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('GetAllAfterChar', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Spc:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmComma]);

  _Ch:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TGetAllAfterChar.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
  __Spc: Longint;
  __Ch: String;
 begin
  _S^.Get(ExecuteData, __S);
  _Spc^.Get(ExecuteData, __Spc);
  _Ch^.Get(ExecuteData, __Ch);

  String(Value):=Wizard.GetAllAfterChar(__S, 
    __Spc, 
    String2Char(__Ch));
 end;

destructor TGetAllAfterChar.Done;
 begin
  Dispose(_S, Done);
  Dispose(_Spc, Done);
  Dispose(_Ch, Done);

  inherited Done;
 end;

{ GetAllAfterSpace(const S: String; const Spc: Byte): String; }

type
 PGetAllAfterSpace = ^TGetAllAfterSpace;
 TGetAllAfterSpace = object(TStringExpression)
  _S: PStringExpression;
  _Spc: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TGetAllAfterSpace.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('GetAllAfterSpace', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Spc:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TGetAllAfterSpace.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
  __Spc: Longint;
 begin
  _S^.Get(ExecuteData, __S);
  _Spc^.Get(ExecuteData, __Spc);

  String(Value):=Wizard.GetAllAfterSpace(__S, 
    __Spc);
 end;

destructor TGetAllAfterSpace.Done;
 begin
  Dispose(_S, Done);
  Dispose(_Spc, Done);

  inherited Done;
 end;

{ Replace(S: String; const A, B: String): String; }

type
 PReplace = ^TReplace;
 TReplace = object(TStringExpression)
  _S: PStringExpression;
  _A: PStringExpression;
  _B: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TReplace.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('Replace', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _A:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _B:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TReplace.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
  __A: String;
  __B: String;
 begin
  _S^.Get(ExecuteData, __S);
  _A^.Get(ExecuteData, __A);
  _B^.Get(ExecuteData, __B);

  String(Value):=Wizard.Replace(__S, 
    __A, 
    __B);
 end;

destructor TReplace.Done;
 begin
  Dispose(_S, Done);
  Dispose(_A, Done);
  Dispose(_B, Done);

  inherited Done;
 end;

{ ReplaceChar(var S: String; const Source, Dest: Char); }

type
 PReplaceChar = ^TReplaceChar;
 TReplaceChar = object(TAction)
  _S: PStringExpression;
  _Source: PStringExpression;
  _Dest: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TReplaceChar.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('ReplaceChar', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  if not _S^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etString);

  Tokenizer^.Next([tmComma]);

  _Source:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Dest:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TReplaceChar.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __S: String;
  __Source: String;
  __Dest: String;
 begin
  _S^.Get(Data, __S);
  _Source^.Get(Data, __Source);
  _Dest^.Get(Data, __Dest);

  Wizard.ReplaceChar(__S, 
    String2Char(__Source), 
    String2Char(__Dest));

  _S^.Assign(Data, __S);

  Execute:=erOk;
 end;

destructor TReplaceChar.Done;
 begin
  Dispose(_S, Done);
  Dispose(_Source, Done);
  Dispose(_Dest, Done);

  inherited Done;
 end;

{ ReplaceEx(var S: String; A: String; const B: String); }

type
 PReplaceEx = ^TReplaceEx;
 TReplaceEx = object(TAction)
  _S: PStringExpression;
  _A: PStringExpression;
  _B: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TReplaceEx.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('ReplaceEx', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  if not _S^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etString);

  Tokenizer^.Next([tmComma]);

  _A:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _B:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TReplaceEx.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __S: String;
  __A: String;
  __B: String;
 begin
  _S^.Get(Data, __S);
  _A^.Get(Data, __A);
  _B^.Get(Data, __B);

  Wizard.ReplaceEx(__S, 
    __A, 
    __B);

  _S^.Assign(Data, __S);

  Execute:=erOk;
 end;

destructor TReplaceEx.Done;
 begin
  Dispose(_S, Done);
  Dispose(_A, Done);
  Dispose(_B, Done);

  inherited Done;
 end;

{ FixTabs(var S: String); }

type
 PFixTabs = ^TFixTabs;
 TFixTabs = object(TAction)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TFixTabs.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('FixTabs', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  if not _S^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etString);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TFixTabs.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __S: String;
 begin
  _S^.Get(Data, __S);

  Wizard.FixTabs(__S);

  _S^.Assign(Data, __S);

  Execute:=erOk;
 end;

destructor TFixTabs.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ RepackString(var S: String; const Delims: TCharSet; const Divider:
   String); }

type
 PRepackString = ^TRepackString;
 TRepackString = object(TAction)
  _S: PStringExpression;
  _Delims: PStringExpression;
  _Divider: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TRepackString.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('RepackString', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  if not _S^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etString);

  Tokenizer^.Next([tmComma]);

  _Delims:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Divider:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TRepackString.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __S: String;
  __Delims: String;
  __Divider: String;
 begin
  _S^.Get(Data, __S);
  _Delims^.Get(Data, __Delims);
  _Divider^.Get(Data, __Divider);

  Wizard.RepackString(__S, 
    String2CharSet(__Delims)^, 
    __Divider);

  _S^.Assign(Data, __S);

  Execute:=erOk;
 end;

destructor TRepackString.Done;
 begin
  Dispose(_S, Done);
  Dispose(_Delims, Done);
  Dispose(_Divider, Done);

  inherited Done;
 end;

{ nlz(const S: String): String; }

type
 Pnlz = ^Tnlz;
 Tnlz = object(TStringExpression)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor Tnlz.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('nlz', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure Tnlz.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
 begin
  _S^.Get(ExecuteData, __S);

  String(Value):=Wizard.nlz(__S);
 end;

destructor Tnlz.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ GetPString(const S: Pointer): String; }

type
 PGetPString = ^TGetPString;
 TGetPString = object(TStringExpression)
  _S: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TGetPString.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('GetPString', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TGetPString.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: Pointer;
 begin
  _S^.Get(ExecuteData, __S);

  String(Value):=Wizard.GetPString(__S);
 end;

destructor TGetPString.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ GetPStringEx(const S: Pointer; var R: String); }

type
 PGetPStringEx = ^TGetPStringEx;
 TGetPStringEx = object(TAction)
  _S: PNumberExpression;
  _R: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TGetPStringEx.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('GetPStringEx', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmComma]);

  _R:=Actions^.ParseStringExpression(Tokenizer, True);

  if not _R^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etString);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TGetPStringEx.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __S: Pointer;
  __R: String;
 begin
  _S^.Get(Data, __S);
  _R^.Get(Data, __R);

  Wizard.GetPStringEx(__S, 
    __R);

  _R^.Assign(Data, __R);

  Execute:=erOk;
 end;

destructor TGetPStringEx.Done;
 begin
  Dispose(_S, Done);
  Dispose(_R, Done);

  inherited Done;
 end;

{ HexB(const B: Byte): String; }

type
 PHexB = ^THexB;
 THexB = object(TStringExpression)
  _B: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor THexB.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('HexB', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _B:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure THexB.Get(const ExecuteData: PExecuteData; var Value);
 var
  __B: Longint;
 begin
  _B^.Get(ExecuteData, __B);

  String(Value):=Wizard.HexB(__B);
 end;

destructor THexB.Done;
 begin
  Dispose(_B, Done);

  inherited Done;
 end;

{ HexT(const W: Word): String; }

type
 PHexT = ^THexT;
 THexT = object(TStringExpression)
  _W: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor THexT.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('HexT', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _W:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure THexT.Get(const ExecuteData: PExecuteData; var Value);
 var
  __W: Longint;
 begin
  _W^.Get(ExecuteData, __W);

  String(Value):=Wizard.HexT(Word(__W));
 end;

destructor THexT.Done;
 begin
  Dispose(_W, Done);

  inherited Done;
 end;

{ HexW(const W: Word): String; }

type
 PHexW = ^THexW;
 THexW = object(TStringExpression)
  _W: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor THexW.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('HexW', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _W:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure THexW.Get(const ExecuteData: PExecuteData; var Value);
 var
  __W: Longint;
 begin
  _W^.Get(ExecuteData, __W);

  String(Value):=Wizard.HexW(Word(__W));
 end;

destructor THexW.Done;
 begin
  Dispose(_W, Done);

  inherited Done;
 end;

{ HexL(const L: Longint): String; }

type
 PHexL = ^THexL;
 THexL = object(TStringExpression)
  _L: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor THexL.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('HexL', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _L:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure THexL.Get(const ExecuteData: PExecuteData; var Value);
 var
  __L: Longint;
 begin
  _L^.Get(ExecuteData, __L);

  String(Value):=Wizard.HexL(__L);
 end;

destructor THexL.Done;
 begin
  Dispose(_L, Done);

  inherited Done;
 end;

{ HexPtr(const P: Pointer): String; }

type
 PHexPtr = ^THexPtr;
 THexPtr = object(TStringExpression)
  _P: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor THexPtr.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('HexPtr', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _P:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure THexPtr.Get(const ExecuteData: PExecuteData; var Value);
 var
  __P: Pointer;
 begin
  _P^.Get(ExecuteData, __P);

  String(Value):=Wizard.HexPtr(__P);
 end;

destructor THexPtr.Done;
 begin
  Dispose(_P, Done);

  inherited Done;
 end;

{ Long2Str(const L: Longint): String; }

type
 PLong2Str = ^TLong2Str;
 TLong2Str = object(TStringExpression)
  _L: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TLong2Str.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('Long2Str', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _L:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TLong2Str.Get(const ExecuteData: PExecuteData; var Value);
 var
  __L: Longint;
 begin
  _L^.Get(ExecuteData, __L);

  String(Value):=Wizard.Long2Str(__L);
 end;

destructor TLong2Str.Done;
 begin
  Dispose(_L, Done);

  inherited Done;
 end;

{ Long2StrFmt(const L: Longint): String; }

type
 PLong2StrFmt = ^TLong2StrFmt;
 TLong2StrFmt = object(TStringExpression)
  _L: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TLong2StrFmt.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('Long2StrFmt', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _L:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TLong2StrFmt.Get(const ExecuteData: PExecuteData; var Value);
 var
  __L: Longint;
 begin
  _L^.Get(ExecuteData, __L);

  String(Value):=Wizard.Long2StrFmt(__L);
 end;

destructor TLong2StrFmt.Done;
 begin
  Dispose(_L, Done);

  inherited Done;
 end;

{ Lz(const Number: Longint): String; }

type
 PLz = ^TLz;
 TLz = object(TStringExpression)
  _Number: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TLz.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('Lz', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Number:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TLz.Get(const ExecuteData: PExecuteData; var Value);
 var
  __Number: Longint;
 begin
  _Number^.Get(ExecuteData, __Number);

  String(Value):=Wizard.Lz(__Number);
 end;

destructor TLz.Done;
 begin
  Dispose(_Number, Done);

  inherited Done;
 end;

{ Str2Number(const S: String): Boolean; }

type
 PStr2Number = ^TStr2Number;
 TStr2Number = object(TBooleanExpression)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TStr2Number.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('Str2Number', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TStr2Number.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
 begin
  _S^.Get(ExecuteData, __S);

  Boolean(Value):=Wizard.Str2Number(__S);
 end;

destructor TStr2Number.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ Str2Longint(const S: String; var A: Longint): Boolean; }

type
 PStr2Longint = ^TStr2Longint;
 TStr2Longint = object(TBooleanExpression)
  _S: PStringExpression;
  _A: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TStr2Longint.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('Str2Longint', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _A:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  if not _A^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etNumber);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TStr2Longint.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
  __A: Longint;
 begin
  _S^.Get(ExecuteData, __S);
  _A^.Get(ExecuteData, __A);

  Boolean(Value):=Wizard.Str2Longint(__S, 
    __A);

  _A^.Assign(ExecuteData, __A);
 end;

destructor TStr2Longint.Done;
 begin
  Dispose(_S, Done);
  Dispose(_A, Done);

  inherited Done;
 end;

{ Size2KB(const Size: Longint): String; }

type
 PSize2KB = ^TSize2KB;
 TSize2KB = object(TStringExpression)
  _Size: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TSize2KB.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('Size2KB', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Size:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TSize2KB.Get(const ExecuteData: PExecuteData; var Value);
 var
  __Size: Longint;
 begin
  _Size^.Get(ExecuteData, __Size);

  String(Value):=Wizard.Size2KB(__Size);
 end;

destructor TSize2KB.Done;
 begin
  Dispose(_Size, Done);

  inherited Done;
 end;

{ AddBackSlash(S: String): String; }

type
 PAddBackSlash = ^TAddBackSlash;
 TAddBackSlash = object(TStringExpression)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TAddBackSlash.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('AddBackSlash', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TAddBackSlash.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
 begin
  _S^.Get(ExecuteData, __S);

  String(Value):=Wizard.AddBackSlash(__S);
 end;

destructor TAddBackSlash.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ RemoveBackSlash(S: String): String; }

type
 PRemoveBackSlash = ^TRemoveBackSlash;
 TRemoveBackSlash = object(TStringExpression)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TRemoveBackSlash.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('RemoveBackSlash', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TRemoveBackSlash.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
 begin
  _S^.Get(ExecuteData, __S);

  String(Value):=Wizard.RemoveBackSlash(__S);
 end;

destructor TRemoveBackSlash.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ AddDot(const S: String): String; }

type
 PAddDot = ^TAddDot;
 TAddDot = object(TStringExpression)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TAddDot.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('AddDot', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TAddDot.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
 begin
  _S^.Get(ExecuteData, __S);

  String(Value):=Wizard.AddDot(__S);
 end;

destructor TAddDot.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ ExistDir(const S: String): Boolean; }

type
 PExistDir = ^TExistDir;
 TExistDir = object(TBooleanExpression)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TExistDir.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('ExistDir', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TExistDir.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
 begin
  _S^.Get(ExecuteData, __S);

  Boolean(Value):=Wizard.ExistDir(__S);
 end;

destructor TExistDir.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ ExistFile(const S: String): Boolean; }

type
 PExistFile = ^TExistFile;
 TExistFile = object(TBooleanExpression)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TExistFile.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('ExistFile', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TExistFile.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
 begin
  _S^.Get(ExecuteData, __S);

  Boolean(Value):=Wizard.ExistFile(__S);
 end;

destructor TExistFile.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ FExpand(const S: String): String; }

type
 PFExpand = ^TFExpand;
 TFExpand = object(TStringExpression)
  _S: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TFExpand.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('FExpand', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _S:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TFExpand.Get(const ExecuteData: PExecuteData; var Value);
 var
  __S: String;
 begin
  _S^.Get(ExecuteData, __S);

  String(Value):=Wizard.FExpand(__S);
 end;

destructor TFExpand.Done;
 begin
  Dispose(_S, Done);

  inherited Done;
 end;

{ HasExtension(const Name: String; var DotPos: Word): Boolean; }

type
 PHasExtension = ^THasExtension;
 THasExtension = object(TBooleanExpression)
  _Name: PStringExpression;
  _DotPos: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor THasExtension.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('HasExtension', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Name:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _DotPos:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  if not _DotPos^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etNumber);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure THasExtension.Get(const ExecuteData: PExecuteData; var Value);
 var
  __Name: String;
  __DotPos: Longint;
 begin
  _Name^.Get(ExecuteData, __Name);
  _DotPos^.Get(ExecuteData, __DotPos);

  Boolean(Value):=Wizard.HasExtension(__Name, 
    Word(__DotPos));

  _DotPos^.Assign(ExecuteData, __DotPos);
 end;

destructor THasExtension.Done;
 begin
  Dispose(_Name, Done);
  Dispose(_DotPos, Done);

  inherited Done;
 end;

{ ForceExtension(const Name, Ext: String): String; }

type
 PForceExtension = ^TForceExtension;
 TForceExtension = object(TStringExpression)
  _Name: PStringExpression;
  _Ext: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TForceExtension.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('ForceExtension', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Name:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Ext:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TForceExtension.Get(const ExecuteData: PExecuteData; var Value);
 var
  __Name: String;
  __Ext: String;
 begin
  _Name^.Get(ExecuteData, __Name);
  _Ext^.Get(ExecuteData, __Ext);

  String(Value):=Wizard.ForceExtension(__Name, 
    __Ext);
 end;

destructor TForceExtension.Done;
 begin
  Dispose(_Name, Done);
  Dispose(_Ext, Done);

  inherited Done;
 end;

{ JustExtension(const Name: String): String; }

type
 PJustExtension = ^TJustExtension;
 TJustExtension = object(TStringExpression)
  _Name: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TJustExtension.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('JustExtension', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Name:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TJustExtension.Get(const ExecuteData: PExecuteData; var Value);
 var
  __Name: String;
 begin
  _Name^.Get(ExecuteData, __Name);

  String(Value):=Wizard.JustExtension(__Name);
 end;

destructor TJustExtension.Done;
 begin
  Dispose(_Name, Done);

  inherited Done;
 end;

{ JustFilename(const PathName: String): String; }

type
 PJustFilename = ^TJustFilename;
 TJustFilename = object(TStringExpression)
  _PathName: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TJustFilename.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('JustFilename', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _PathName:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TJustFilename.Get(const ExecuteData: PExecuteData; var Value);
 var
  __PathName: String;
 begin
  _PathName^.Get(ExecuteData, __PathName);

  String(Value):=Wizard.JustFilename(__PathName);
 end;

destructor TJustFilename.Done;
 begin
  Dispose(_PathName, Done);

  inherited Done;
 end;

{ JustFilenameOnly(const PathName: String): String; }

type
 PJustFilenameOnly = ^TJustFilenameOnly;
 TJustFilenameOnly = object(TStringExpression)
  _PathName: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TJustFilenameOnly.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('JustFilenameOnly', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _PathName:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TJustFilenameOnly.Get(const ExecuteData: PExecuteData; var Value);
 var
  __PathName: String;
 begin
  _PathName^.Get(ExecuteData, __PathName);

  String(Value):=Wizard.JustFilenameOnly(__PathName);
 end;

destructor TJustFilenameOnly.Done;
 begin
  Dispose(_PathName, Done);

  inherited Done;
 end;

{ JustPathname(const PathName: String): String; }

type
 PJustPathname = ^TJustPathname;
 TJustPathname = object(TStringExpression)
  _PathName: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TJustPathname.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('JustPathname', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _PathName:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TJustPathname.Get(const ExecuteData: PExecuteData; var Value);
 var
  __PathName: String;
 begin
  _PathName^.Get(ExecuteData, __PathName);

  String(Value):=Wizard.JustPathname(__PathName);
 end;

destructor TJustPathname.Done;
 begin
  Dispose(_PathName, Done);

  inherited Done;
 end;

{ FileLocked(const FName: String): Boolean; }

type
 PFileLocked = ^TFileLocked;
 TFileLocked = object(TBooleanExpression)
  _FName: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TFileLocked.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('FileLocked', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _FName:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TFileLocked.Get(const ExecuteData: PExecuteData; var Value);
 var
  __FName: String;
 begin
  _FName^.Get(ExecuteData, __FName);

  Boolean(Value):=Wizard.FileLocked(__FName);
 end;

destructor TFileLocked.Done;
 begin
  Dispose(_FName, Done);

  inherited Done;
 end;

{ EraseFile(const FName: String): Boolean; }

type
 PEraseFile = ^TEraseFile;
 TEraseFile = object(TBooleanExpression)
  _FName: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TEraseFile.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('EraseFile', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _FName:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TEraseFile.Get(const ExecuteData: PExecuteData; var Value);
 var
  __FName: String;
 begin
  _FName^.Get(ExecuteData, __FName);

  Boolean(Value):=Wizard.EraseFile(__FName);
 end;

destructor TEraseFile.Done;
 begin
  Dispose(_FName, Done);

  inherited Done;
 end;

{ TouchFile(const FName: String): Boolean; }

type
 PTouchFile = ^TTouchFile;
 TTouchFile = object(TBooleanExpression)
  _FName: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TTouchFile.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('TouchFile', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _FName:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TTouchFile.Get(const ExecuteData: PExecuteData; var Value);
 var
  __FName: String;
 begin
  _FName^.Get(ExecuteData, __FName);

  Boolean(Value):=Wizard.TouchFile(__FName);
 end;

destructor TTouchFile.Done;
 begin
  Dispose(_FName, Done);

  inherited Done;
 end;

{ RenameFile(const AFName, BFName: String): Boolean; }

type
 PRenameFile = ^TRenameFile;
 TRenameFile = object(TBooleanExpression)
  _AFName: PStringExpression;
  _BFName: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TRenameFile.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('RenameFile', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _AFName:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _BFName:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TRenameFile.Get(const ExecuteData: PExecuteData; var Value);
 var
  __AFName: String;
  __BFName: String;
 begin
  _AFName^.Get(ExecuteData, __AFName);
  _BFName^.Get(ExecuteData, __BFName);

  Boolean(Value):=Wizard.RenameFile(__AFName, 
    __BFName);
 end;

destructor TRenameFile.Done;
 begin
  Dispose(_AFName, Done);
  Dispose(_BFName, Done);

  inherited Done;
 end;

{ RenameFileEx(const AFName, BFName: String; var rc: Longint): Boolean; }

type
 PRenameFileEx = ^TRenameFileEx;
 TRenameFileEx = object(TBooleanExpression)
  _AFName: PStringExpression;
  _BFName: PStringExpression;
  _rc: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TRenameFileEx.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('RenameFileEx', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _AFName:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _BFName:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _rc:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  if not _rc^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etNumber);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TRenameFileEx.Get(const ExecuteData: PExecuteData; var Value);
 var
  __AFName: String;
  __BFName: String;
  __rc: Longint;
 begin
  _AFName^.Get(ExecuteData, __AFName);
  _BFName^.Get(ExecuteData, __BFName);
  _rc^.Get(ExecuteData, __rc);

  Boolean(Value):=Wizard.RenameFileEx(__AFName, 
    __BFName, 
    __rc);

  _rc^.Assign(ExecuteData, __rc);
 end;

destructor TRenameFileEx.Done;
 begin
  Dispose(_AFName, Done);
  Dispose(_BFName, Done);
  Dispose(_rc, Done);

  inherited Done;
 end;

{ EmptyDirectory(const Path: String): Boolean; }

type
 PEmptyDirectory = ^TEmptyDirectory;
 TEmptyDirectory = object(TBooleanExpression)
  _Path: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TEmptyDirectory.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('EmptyDirectory', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Path:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TEmptyDirectory.Get(const ExecuteData: PExecuteData; var Value);
 var
  __Path: String;
 begin
  _Path^.Get(ExecuteData, __Path);

  Boolean(Value):=Wizard.EmptyDirectory(__Path);
 end;

destructor TEmptyDirectory.Done;
 begin
  Dispose(_Path, Done);

  inherited Done;
 end;

{ EmptyDirectoryEx(const Path: String; const OnlyFiles: Boolean):
   Boolean; }

type
 PEmptyDirectoryEx = ^TEmptyDirectoryEx;
 TEmptyDirectoryEx = object(TBooleanExpression)
  _Path: PStringExpression;
  _OnlyFiles: PBooleanExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TEmptyDirectoryEx.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('EmptyDirectoryEx', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Path:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _OnlyFiles:=Actions^.ParseBooleanExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TEmptyDirectoryEx.Get(const ExecuteData: PExecuteData; var Value);
 var
  __Path: String;
  __OnlyFiles: Boolean;
 begin
  _Path^.Get(ExecuteData, __Path);
  _OnlyFiles^.Get(ExecuteData, __OnlyFiles);

  Boolean(Value):=Wizard.EmptyDirectoryEx(__Path, 
    __OnlyFiles);
 end;

destructor TEmptyDirectoryEx.Done;
 begin
  Dispose(_Path, Done);
  Dispose(_OnlyFiles, Done);

  inherited Done;
 end;

{ DistortFileName(var FName: String; const DistortExtension: Boolean):
   Boolean; }

type
 PDistortFileName = ^TDistortFileName;
 TDistortFileName = object(TBooleanExpression)
  _FName: PStringExpression;
  _DistortExtension: PBooleanExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TDistortFileName.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('DistortFileName', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _FName:=Actions^.ParseStringExpression(Tokenizer, True);

  if not _FName^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etString);

  Tokenizer^.Next([tmComma]);

  _DistortExtension:=Actions^.ParseBooleanExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TDistortFileName.Get(const ExecuteData: PExecuteData; var Value);
 var
  __FName: String;
  __DistortExtension: Boolean;
 begin
  _FName^.Get(ExecuteData, __FName);
  _DistortExtension^.Get(ExecuteData, __DistortExtension);

  Boolean(Value):=Wizard.DistortFileName(__FName, 
    __DistortExtension);

  _FName^.Assign(ExecuteData, __FName);
 end;

destructor TDistortFileName.Done;
 begin
  Dispose(_FName, Done);
  Dispose(_DistortExtension, Done);

  inherited Done;
 end;

{ DistortFileExtension(var FName: String): Boolean; }

type
 PDistortFileExtension = ^TDistortFileExtension;
 TDistortFileExtension = object(TBooleanExpression)
  _FName: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TDistortFileExtension.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('DistortFileExtension', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _FName:=Actions^.ParseStringExpression(Tokenizer, True);

  if not _FName^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etString);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TDistortFileExtension.Get(const ExecuteData: PExecuteData; var Value);
 var
  __FName: String;
 begin
  _FName^.Get(ExecuteData, __FName);

  Boolean(Value):=Wizard.DistortFileExtension(__FName);

  _FName^.Assign(ExecuteData, __FName);
 end;

destructor TDistortFileExtension.Done;
 begin
  Dispose(_FName, Done);

  inherited Done;
 end;

{ DistortFileExtensionLastLetter(var FName: String; Force: Boolean):
   Boolean; }

type
 PDistortFileExtensionLastLetter = ^TDistortFileExtensionLastLetter;
 TDistortFileExtensionLastLetter = object(TBooleanExpression)
  _FName: PStringExpression;
  _Force: PBooleanExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TDistortFileExtensionLastLetter.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('DistortFileExtensionLastLetter', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _FName:=Actions^.ParseStringExpression(Tokenizer, True);

  if not _FName^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etString);

  Tokenizer^.Next([tmComma]);

  _Force:=Actions^.ParseBooleanExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TDistortFileExtensionLastLetter.Get(const ExecuteData: PExecuteData; var Value);
 var
  __FName: String;
  __Force: Boolean;
 begin
  _FName^.Get(ExecuteData, __FName);
  _Force^.Get(ExecuteData, __Force);

  Boolean(Value):=Wizard.DistortFileExtensionLastLetter(__FName, 
    __Force);

  _FName^.Assign(ExecuteData, __FName);
 end;

destructor TDistortFileExtensionLastLetter.Done;
 begin
  Dispose(_FName, Done);
  Dispose(_Force, Done);

  inherited Done;
 end;

{ Clock: Longint; }

type
 PClock = ^TClock;
 TClock = object(TNumberExpression)
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
 end;

procedure TClock.Get(const ExecuteData: PExecuteData; var Value);
 begin
  Longint(Value):=Longint(Wizard.Clock);
 end;

{ Wait(const ms: Longint); }

type
 PWait = ^TWait;
 TWait = object(TAction)
  _ms: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

constructor TWait.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('Wait', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _ms:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

function TWait.Execute(const Data: PExecuteData): TExecuteResult;
 var
  __ms: Longint;
 begin
  _ms^.Get(Data, __ms);

  Wizard.Wait(__ms);

  Execute:=erOk;
 end;

destructor TWait.Done;
 begin
  Dispose(_ms, Done);

  inherited Done;
 end;

{ DayOfWeek(const Year, Month, Day: Longint): Word; }

type
 PDayOfWeek = ^TDayOfWeek;
 TDayOfWeek = object(TNumberExpression)
  _Year: PNumberExpression;
  _Month: PNumberExpression;
  _Day: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TDayOfWeek.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('DayOfWeek', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Year:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmComma]);

  _Month:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmComma]);

  _Day:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TDayOfWeek.Get(const ExecuteData: PExecuteData; var Value);
 var
  __Year: Longint;
  __Month: Longint;
  __Day: Longint;
 begin
  _Year^.Get(ExecuteData, __Year);
  _Month^.Get(ExecuteData, __Month);
  _Day^.Get(ExecuteData, __Day);

  Longint(Value):=Longint(Wizard.DayOfWeek(__Year, 
    __Month, 
    __Day));
 end;

destructor TDayOfWeek.Done;
 begin
  Dispose(_Year, Done);
  Dispose(_Month, Done);
  Dispose(_Day, Done);

  inherited Done;
 end;

{ CheckWildcard(const Src, Mask: String): Boolean; }

type
 PCheckWildcard = ^TCheckWildcard;
 TCheckWildcard = object(TBooleanExpression)
  _Src: PStringExpression;
  _Mask: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TCheckWildcard.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('CheckWildcard', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Src:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  _Mask:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TCheckWildcard.Get(const ExecuteData: PExecuteData; var Value);
 var
  __Src: String;
  __Mask: String;
 begin
  _Src^.Get(ExecuteData, __Src);
  _Mask^.Get(ExecuteData, __Mask);

  Boolean(Value):=Wizard.CheckWildcard(__Src, 
    __Mask);
 end;

destructor TCheckWildcard.Done;
 begin
  Dispose(_Src, Done);
  Dispose(_Mask, Done);

  inherited Done;
 end;

{ IsWildcard(const Mask: String): Boolean; }

type
 PIsWildcard = ^TIsWildcard;
 TIsWildcard = object(TBooleanExpression)
  _Mask: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TIsWildcard.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('IsWildcard', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _Mask:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TIsWildcard.Get(const ExecuteData: PExecuteData; var Value);
 var
  __Mask: String;
 begin
  _Mask^.Get(ExecuteData, __Mask);

  Boolean(Value):=Wizard.IsWildcard(__Mask);
 end;

destructor TIsWildcard.Done;
 begin
  Dispose(_Mask, Done);

  inherited Done;
 end;

{ GetBinkDateTime: String; }

type
 PGetBinkDateTime = ^TGetBinkDateTime;
 TGetBinkDateTime = object(TStringExpression)
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
 end;

procedure TGetBinkDateTime.Get(const ExecuteData: PExecuteData; var Value);
 begin
  String(Value):=Wizard.GetBinkDateTime;
 end;

{ GetAttr(const FName: String): Longint; }

type
 PGetAttr = ^TGetAttr;
 TGetAttr = object(TNumberExpression)
  _FName: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TGetAttr.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('GetAttr', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _FName:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TGetAttr.Get(const ExecuteData: PExecuteData; var Value);
 var
  __FName: String;
 begin
  _FName^.Get(ExecuteData, __FName);

  Longint(Value):=Longint(Wizard.GetAttr(__FName));
 end;

destructor TGetAttr.Done;
 begin
  Dispose(_FName, Done);

  inherited Done;
 end;

{ GetFileDate(const FName: String): Longint; }

type
 PGetFileDate = ^TGetFileDate;
 TGetFileDate = object(TNumberExpression)
  _FName: PStringExpression;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

constructor TGetFileDate.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('GetFileDate', Actions);

  Tokenizer^.Next([tmOpenBracket]);

  _FName:=Actions^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure TGetFileDate.Get(const ExecuteData: PExecuteData; var Value);
 var
  __FName: String;
 begin
  _FName^.Get(ExecuteData, __FName);

  Longint(Value):=Longint(Wizard.GetFileDate(__FName));
 end;

destructor TGetFileDate.Done;
 begin
  Dispose(_FName, Done);

  inherited Done;
 end;

function CreatorGETERROR(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorGETERROR:=New(PGetError, Init('GetError', Actions));
 end;

function CreatorLOCKCOMMANDLINE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorLOCKCOMMANDLINE:=New(PLockCommandLine, Init(Tokenizer, Actions));
 end;

function CreatorEXECUTE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorEXECUTE:=New(PExecute, Init(Tokenizer, Actions));
 end;

function CreatorMOVEFILE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorMOVEFILE:=New(PMoveFile, Init(Tokenizer, Actions));
 end;

function CreatorCREATEDIRECTORY(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorCREATEDIRECTORY:=New(PCreateDirectory, Init(Tokenizer, Actions));
 end;

function CreatorPUSHTITLE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorPUSHTITLE:=New(PPushTitle, Init('PushTitle', Actions));
 end;

function CreatorSETTITLE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorSETTITLE:=New(PSetTitle, Init(Tokenizer, Actions));
 end;

function CreatorPOPTITLE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorPOPTITLE:=New(PPopTitle, Init('PopTitle', Actions));
 end;

function CreatorMAKESHORTNAME(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorMAKESHORTNAME:=New(PMakeShortName, Init(Tokenizer, Actions));
 end;

function CreatorLOGWRITE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorLOGWRITE:=New(PLogWrite, Init(Tokenizer, Actions));
 end;

function CreatorSETSTATUS(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorSETSTATUS:=New(PSetStatus, Init(Tokenizer, Actions));
 end;

function CreatorCLEARSTATUS(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorCLEARSTATUS:=New(PClearStatus, Init('ClearStatus', Actions));
 end;

function CreatorENQUEUECOMMAND(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorENQUEUECOMMAND:=New(PEnqueueCommand, Init(Tokenizer, Actions));
 end;

function CreatorENQUEUECOMMANDEX(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorENQUEUECOMMANDEX:=New(PEnqueueCommandEx, Init(Tokenizer, Actions));
 end;

function CreatorREPORT(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorREPORT:=New(PReport, Init(Tokenizer, Actions));
 end;

function CreatorREPORTAREAACTION(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorREPORTAREAACTION:=New(PReportAreaAction, Init(Tokenizer, Actions));
 end;

function CreatorDOAREAACTIONS(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorDOAREAACTIONS:=New(PDoAreaActions, Init(Tokenizer, Actions));
 end;

function CreatorGETCOMMATOKEN(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorGETCOMMATOKEN:=New(PGetCommaToken, Init(Tokenizer, Actions));
 end;

function CreatorNEWSTR(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorNEWSTR:=New(PNewStr, Init(Tokenizer, Actions));
 end;

function CreatorDISPOSESTR(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorDISPOSESTR:=New(PDisposeStr, Init(Tokenizer, Actions));
 end;

function CreatorSTREAMLINKER(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorSTREAMLINKER:=New(PStreamLinker, Init('StreamLinker', Actions));
 end;

function CreatorMINIMIZEHANDLES(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorMINIMIZEHANDLES:=New(PMinimizeHandles, Init('MinimizeHandles', Actions));
 end;

function CreatorFORMATTIME(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorFORMATTIME:=New(PFormatTime, Init(Tokenizer, Actions));
 end;

function CreatorSTRNEW(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorSTRNEW:=New(PStrNew, Init(Tokenizer, Actions));
 end;

function CreatorSTRALLOC(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorSTRALLOC:=New(PStrAlloc, Init(Tokenizer, Actions));
 end;

function CreatorSTRDISPOSE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorSTRDISPOSE:=New(PStrDispose, Init(Tokenizer, Actions));
 end;

function CreatorCOA(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorCOA:=New(PCOA, Init('COA', Actions));
 end;

function CreatorPOA(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorPOA:=New(PPOA, Init('POA', Actions));
 end;

function CreatorNEWADDRESSEX(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorNEWADDRESSEX:=New(PNewAddressEx, Init(Tokenizer, Actions));
 end;

function CreatorREGISTERMESSAGEBASEINTERFACE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorREGISTERMESSAGEBASEINTERFACE:=New(PRegisterMessageBaseInterface, Init(Tokenizer, Actions));
 end;

function CreatorJOKESWITCH(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorJOKESWITCH:=New(PJokeSwitch, Init(Tokenizer, Actions));
 end;

function CreatorREPLACEMACROS(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorREPLACEMACROS:=New(PReplaceMacros, Init(Tokenizer, Actions));
 end;

function CreatorSTLOCASE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorSTLOCASE:=New(PStLoCase, Init(Tokenizer, Actions));
 end;

function CreatorSTUPCASE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorSTUPCASE:=New(PStUpCase, Init(Tokenizer, Actions));
 end;

function CreatorSTLOCASEEX(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorSTLOCASEEX:=New(PStLocaseEx, Init(Tokenizer, Actions));
 end;

function CreatorSTUPCASEEX(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorSTUPCASEEX:=New(PStUpcaseEx, Init(Tokenizer, Actions));
 end;

function CreatorSTRLOCASE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorSTRLOCASE:=New(PStrLocase, Init(Tokenizer, Actions));
 end;

function CreatorSTRUPCASE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorSTRUPCASE:=New(PStrUpcase, Init(Tokenizer, Actions));
 end;

function CreatorLEFTPAD(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorLEFTPAD:=New(PLeftPad, Init(Tokenizer, Actions));
 end;

function CreatorLEFTPADCH(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorLEFTPADCH:=New(PLeftPadCh, Init(Tokenizer, Actions));
 end;

function CreatorCENTER(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorCENTER:=New(PCenter, Init(Tokenizer, Actions));
 end;

function CreatorCENTERCH(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorCENTERCH:=New(PCenterCh, Init(Tokenizer, Actions));
 end;

function CreatorPAD(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorPAD:=New(PPad, Init(Tokenizer, Actions));
 end;

function CreatorPADCH(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorPADCH:=New(PPadCh, Init(Tokenizer, Actions));
 end;

function CreatorLTRIM(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorLTRIM:=New(PLTrim, Init(Tokenizer, Actions));
 end;

function CreatorTRIM(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorTRIM:=New(PTrim, Init(Tokenizer, Actions));
 end;

function CreatorTRIMEX(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorTRIMEX:=New(PTrimEx, Init(Tokenizer, Actions));
 end;

function CreatorRTRIM(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorRTRIM:=New(PRTrim, Init(Tokenizer, Actions));
 end;

function CreatorEXTRACTWORD(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorEXTRACTWORD:=New(PExtractWord, Init(Tokenizer, Actions));
 end;

function CreatorWORDCOUNT(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorWORDCOUNT:=New(PWordCount, Init(Tokenizer, Actions));
 end;

function CreatorGETALLAFTERCHAR(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorGETALLAFTERCHAR:=New(PGetAllAfterChar, Init(Tokenizer, Actions));
 end;

function CreatorGETALLAFTERSPACE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorGETALLAFTERSPACE:=New(PGetAllAfterSpace, Init(Tokenizer, Actions));
 end;

function CreatorREPLACE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorREPLACE:=New(PReplace, Init(Tokenizer, Actions));
 end;

function CreatorREPLACECHAR(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorREPLACECHAR:=New(PReplaceChar, Init(Tokenizer, Actions));
 end;

function CreatorREPLACEEX(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorREPLACEEX:=New(PReplaceEx, Init(Tokenizer, Actions));
 end;

function CreatorFIXTABS(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorFIXTABS:=New(PFixTabs, Init(Tokenizer, Actions));
 end;

function CreatorREPACKSTRING(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorREPACKSTRING:=New(PRepackString, Init(Tokenizer, Actions));
 end;

function CreatorNLZ(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorNLZ:=New(Pnlz, Init(Tokenizer, Actions));
 end;

function CreatorGETPSTRING(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorGETPSTRING:=New(PGetPString, Init(Tokenizer, Actions));
 end;

function CreatorGETPSTRINGEX(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorGETPSTRINGEX:=New(PGetPStringEx, Init(Tokenizer, Actions));
 end;

function CreatorHEXB(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorHEXB:=New(PHexB, Init(Tokenizer, Actions));
 end;

function CreatorHEXT(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorHEXT:=New(PHexT, Init(Tokenizer, Actions));
 end;

function CreatorHEXW(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorHEXW:=New(PHexW, Init(Tokenizer, Actions));
 end;

function CreatorHEXL(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorHEXL:=New(PHexL, Init(Tokenizer, Actions));
 end;

function CreatorHEXPTR(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorHEXPTR:=New(PHexPtr, Init(Tokenizer, Actions));
 end;

function CreatorLONG2STR(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorLONG2STR:=New(PLong2Str, Init(Tokenizer, Actions));
 end;

function CreatorLONG2STRFMT(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorLONG2STRFMT:=New(PLong2StrFmt, Init(Tokenizer, Actions));
 end;

function CreatorLZ(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorLZ:=New(PLz, Init(Tokenizer, Actions));
 end;

function CreatorSTR2NUMBER(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorSTR2NUMBER:=New(PStr2Number, Init(Tokenizer, Actions));
 end;

function CreatorSTR2LONGINT(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorSTR2LONGINT:=New(PStr2Longint, Init(Tokenizer, Actions));
 end;

function CreatorSIZE2KB(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorSIZE2KB:=New(PSize2KB, Init(Tokenizer, Actions));
 end;

function CreatorADDBACKSLASH(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorADDBACKSLASH:=New(PAddBackSlash, Init(Tokenizer, Actions));
 end;

function CreatorREMOVEBACKSLASH(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorREMOVEBACKSLASH:=New(PRemoveBackSlash, Init(Tokenizer, Actions));
 end;

function CreatorADDDOT(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorADDDOT:=New(PAddDot, Init(Tokenizer, Actions));
 end;

function CreatorEXISTDIR(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorEXISTDIR:=New(PExistDir, Init(Tokenizer, Actions));
 end;

function CreatorEXISTFILE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorEXISTFILE:=New(PExistFile, Init(Tokenizer, Actions));
 end;

function CreatorFEXPAND(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorFEXPAND:=New(PFExpand, Init(Tokenizer, Actions));
 end;

function CreatorHASEXTENSION(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorHASEXTENSION:=New(PHasExtension, Init(Tokenizer, Actions));
 end;

function CreatorFORCEEXTENSION(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorFORCEEXTENSION:=New(PForceExtension, Init(Tokenizer, Actions));
 end;

function CreatorJUSTEXTENSION(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorJUSTEXTENSION:=New(PJustExtension, Init(Tokenizer, Actions));
 end;

function CreatorJUSTFILENAME(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorJUSTFILENAME:=New(PJustFilename, Init(Tokenizer, Actions));
 end;

function CreatorJUSTFILENAMEONLY(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorJUSTFILENAMEONLY:=New(PJustFilenameOnly, Init(Tokenizer, Actions));
 end;

function CreatorJUSTPATHNAME(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorJUSTPATHNAME:=New(PJustPathname, Init(Tokenizer, Actions));
 end;

function CreatorFILELOCKED(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorFILELOCKED:=New(PFileLocked, Init(Tokenizer, Actions));
 end;

function CreatorERASEFILE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorERASEFILE:=New(PEraseFile, Init(Tokenizer, Actions));
 end;

function CreatorTOUCHFILE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorTOUCHFILE:=New(PTouchFile, Init(Tokenizer, Actions));
 end;

function CreatorRENAMEFILE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorRENAMEFILE:=New(PRenameFile, Init(Tokenizer, Actions));
 end;

function CreatorRENAMEFILEEX(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorRENAMEFILEEX:=New(PRenameFileEx, Init(Tokenizer, Actions));
 end;

function CreatorEMPTYDIRECTORY(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorEMPTYDIRECTORY:=New(PEmptyDirectory, Init(Tokenizer, Actions));
 end;

function CreatorEMPTYDIRECTORYEX(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorEMPTYDIRECTORYEX:=New(PEmptyDirectoryEx, Init(Tokenizer, Actions));
 end;

function CreatorDISTORTFILENAME(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorDISTORTFILENAME:=New(PDistortFileName, Init(Tokenizer, Actions));
 end;

function CreatorDISTORTFILEEXTENSION(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorDISTORTFILEEXTENSION:=New(PDistortFileExtension, Init(Tokenizer, Actions));
 end;

function CreatorDISTORTFILEEXTENSIONLASTLETTER(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorDISTORTFILEEXTENSIONLASTLETTER:=New(PDistortFileExtensionLastLetter, Init(Tokenizer, Actions));
 end;

function CreatorCLOCK(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorCLOCK:=New(PClock, Init('Clock', Actions));
 end;

function CreatorWAIT(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorWAIT:=New(PWait, Init(Tokenizer, Actions));
 end;

function CreatorDAYOFWEEK(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorDAYOFWEEK:=New(PDayOfWeek, Init(Tokenizer, Actions));
 end;

function CreatorCHECKWILDCARD(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorCHECKWILDCARD:=New(PCheckWildcard, Init(Tokenizer, Actions));
 end;

function CreatorISWILDCARD(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorISWILDCARD:=New(PIsWildcard, Init(Tokenizer, Actions));
 end;

function CreatorGETBINKDATETIME(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorGETBINKDATETIME:=New(PGetBinkDateTime, Init('GetBinkDateTime', Actions));
 end;

function CreatorGETATTR(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorGETATTR:=New(PGetAttr, Init(Tokenizer, Actions));
 end;

function CreatorGETFILEDATE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorGETFILEDATE:=New(PGetFileDate, Init(Tokenizer, Actions));
 end;

{ AddJTAPIProtos }

procedure AddJTAPIProtos(const Target: PActions);
 begin
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('GETERROR', CreatorGETERROR)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('LOCKCOMMANDLINE', CreatorLOCKCOMMANDLINE)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('EXECUTE', CreatorEXECUTE)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('MOVEFILE', CreatorMOVEFILE)));
  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto, New(PProtoCreator, Init('CREATEDIRECTORY',
    CreatorCREATEDIRECTORY)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('PUSHTITLE', CreatorPUSHTITLE)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('SETTITLE', CreatorSETTITLE)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('POPTITLE', CreatorPOPTITLE)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('MAKESHORTNAME',
    CreatorMAKESHORTNAME)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('LOGWRITE', CreatorLOGWRITE)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('SETSTATUS', CreatorSETSTATUS)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('CLEARSTATUS', CreatorCLEARSTATUS)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('ENQUEUECOMMAND', CreatorENQUEUECOMMAND)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('ENQUEUECOMMANDEX', CreatorENQUEUECOMMANDEX)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('REPORT', CreatorREPORT)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('REPORTAREAACTION', CreatorREPORTAREAACTION)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('DOAREAACTIONS', CreatorDOAREAACTIONS)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('GETCOMMATOKEN',
    CreatorGETCOMMATOKEN)));
  Target^.Identifiers^.AddIdentifier(imNumberExpressionProto, New(PProtoCreator, Init('NEWSTR', CreatorNEWSTR)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('DISPOSESTR', CreatorDISPOSESTR)));
  Target^.Identifiers^.AddIdentifier(imNumberExpressionProto, New(PProtoCreator, Init('STREAMLINKER', CreatorSTREAMLINKER)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('MINIMIZEHANDLES', CreatorMINIMIZEHANDLES)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('FORMATTIME', CreatorFORMATTIME)));
  Target^.Identifiers^.AddIdentifier(imNumberExpressionProto, New(PProtoCreator, Init('STRNEW', CreatorSTRNEW)));
  Target^.Identifiers^.AddIdentifier(imNumberExpressionProto, New(PProtoCreator, Init('STRALLOC', CreatorSTRALLOC)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('STRDISPOSE', CreatorSTRDISPOSE)));
  Target^.Identifiers^.AddIdentifier(imNumberExpressionProto, New(PProtoCreator, Init('COA', CreatorCOA)));
  Target^.Identifiers^.AddIdentifier(imNumberExpressionProto, New(PProtoCreator, Init('POA', CreatorPOA)));
  Target^.Identifiers^.AddIdentifier(imNumberExpressionProto, New(PProtoCreator, Init('NEWADDRESSEX', CreatorNEWADDRESSEX)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('REGISTERMESSAGEBASEINTERFACE',
    CreatorREGISTERMESSAGEBASEINTERFACE)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('JOKESWITCH', CreatorJOKESWITCH)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('REPLACEMACROS', CreatorREPLACEMACROS)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('STLOCASE', CreatorSTLOCASE)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('STUPCASE', CreatorSTUPCASE)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('STLOCASEEX', CreatorSTLOCASEEX)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('STUPCASEEX', CreatorSTUPCASEEX)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('STRLOCASE', CreatorSTRLOCASE)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('STRUPCASE', CreatorSTRUPCASE)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('LEFTPAD', CreatorLEFTPAD)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('LEFTPADCH', CreatorLEFTPADCH)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('CENTER', CreatorCENTER)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('CENTERCH', CreatorCENTERCH)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('PAD', CreatorPAD)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('PADCH', CreatorPADCH)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('LTRIM', CreatorLTRIM)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('TRIM', CreatorTRIM)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('TRIMEX', CreatorTRIMEX)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('RTRIM', CreatorRTRIM)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('EXTRACTWORD', CreatorEXTRACTWORD)));
  Target^.Identifiers^.AddIdentifier(imNumberExpressionProto, New(PProtoCreator, Init('WORDCOUNT', CreatorWORDCOUNT)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('GETALLAFTERCHAR',
    CreatorGETALLAFTERCHAR)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('GETALLAFTERSPACE',
    CreatorGETALLAFTERSPACE)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('REPLACE', CreatorREPLACE)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('REPLACECHAR', CreatorREPLACECHAR)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('REPLACEEX', CreatorREPLACEEX)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('FIXTABS', CreatorFIXTABS)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('REPACKSTRING', CreatorREPACKSTRING)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('NLZ', CreatorNLZ)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('GETPSTRING', CreatorGETPSTRING)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('GETPSTRINGEX', CreatorGETPSTRINGEX)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('HEXB', CreatorHEXB)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('HEXT', CreatorHEXT)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('HEXW', CreatorHEXW)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('HEXL', CreatorHEXL)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('HEXPTR', CreatorHEXPTR)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('LONG2STR', CreatorLONG2STR)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('LONG2STRFMT', CreatorLONG2STRFMT)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('LZ', CreatorLZ)));
  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto, New(PProtoCreator, Init('STR2NUMBER', CreatorSTR2NUMBER)));
  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto, New(PProtoCreator, Init('STR2LONGINT', CreatorSTR2LONGINT)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('SIZE2KB', CreatorSIZE2KB)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('ADDBACKSLASH', CreatorADDBACKSLASH)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('REMOVEBACKSLASH',
    CreatorREMOVEBACKSLASH)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('ADDDOT', CreatorADDDOT)));
  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto, New(PProtoCreator, Init('EXISTDIR', CreatorEXISTDIR)));
  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto, New(PProtoCreator, Init('EXISTFILE', CreatorEXISTFILE)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('FEXPAND', CreatorFEXPAND)));
  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto, New(PProtoCreator, Init('HASEXTENSION', CreatorHASEXTENSION)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('FORCEEXTENSION',
    CreatorFORCEEXTENSION)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('JUSTEXTENSION',
    CreatorJUSTEXTENSION)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('JUSTFILENAME', CreatorJUSTFILENAME)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('JUSTFILENAMEONLY',
    CreatorJUSTFILENAMEONLY)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('JUSTPATHNAME', CreatorJUSTPATHNAME)));
  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto, New(PProtoCreator, Init('FILELOCKED', CreatorFILELOCKED)));
  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto, New(PProtoCreator, Init('ERASEFILE', CreatorERASEFILE)));
  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto, New(PProtoCreator, Init('TOUCHFILE', CreatorTOUCHFILE)));
  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto, New(PProtoCreator, Init('RENAMEFILE', CreatorRENAMEFILE)));
  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto, New(PProtoCreator, Init('RENAMEFILEEX', CreatorRENAMEFILEEX)));
  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto, New(PProtoCreator, Init('EMPTYDIRECTORY',
    CreatorEMPTYDIRECTORY)));
  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto, New(PProtoCreator, Init('EMPTYDIRECTORYEX',
    CreatorEMPTYDIRECTORYEX)));
  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto, New(PProtoCreator, Init('DISTORTFILENAME',
    CreatorDISTORTFILENAME)));
  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto, New(PProtoCreator, Init('DISTORTFILEEXTENSION',
    CreatorDISTORTFILEEXTENSION)));
  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto, New(PProtoCreator, Init('DISTORTFILEEXTENSIONLASTLETTER',
    CreatorDISTORTFILEEXTENSIONLASTLETTER)));
  Target^.Identifiers^.AddIdentifier(imNumberExpressionProto, New(PProtoCreator, Init('CLOCK', CreatorCLOCK)));
  Target^.Identifiers^.AddIdentifier(imActionProto, New(PProtoCreator, Init('WAIT', CreatorWAIT)));
  Target^.Identifiers^.AddIdentifier(imNumberExpressionProto, New(PProtoCreator, Init('DAYOFWEEK', CreatorDAYOFWEEK)));
  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto, New(PProtoCreator, Init('CHECKWILDCARD',
    CreatorCHECKWILDCARD)));
  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto, New(PProtoCreator, Init('ISWILDCARD', CreatorISWILDCARD)));
  Target^.Identifiers^.AddIdentifier(imStringExpressionProto, New(PProtoCreator, Init('GETBINKDATETIME',
    CreatorGETBINKDATETIME)));
  Target^.Identifiers^.AddIdentifier(imNumberExpressionProto, New(PProtoCreator, Init('GETATTR', CreatorGETATTR)));
  Target^.Identifiers^.AddIdentifier(imNumberExpressionProto, New(PProtoCreator, Init('GETFILEDATE', CreatorGETFILEDATE)));
 end;

end.
