unit ccSupp;

interface
uses
     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     ccAPI,
     ccAction,
     ccAtom,
     ccExpr,
     ccToken,
     ccCode,
     ccProto,

     Thunks,

     jtMHLcmn,

     Objects,

     Wizard;

type
 Pmatchre = ^Tmatchre;
 Tmatchre = object(TBooleanExpression)
  Expression: PStringExpression;
  re: Pointer;

  constructor Init(const AName: String; const Tokenizer: PTokenizer; const AOwner: PAtom);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

 Pmatchrecs = ^Tmatchrecs;
 Tmatchrecs = object(Tmatchre)
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
 end;

 PeachArea = ^TeachArea;
 TeachArea = object(TAction)
  re: PNumberExpression;
  Area: PNumberExpression;
  eachAreaActions: PActions;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

 PeachMsg = ^TeachMsg;
 TeachMsg = object(TAction)
  mb: PNumberExpression;
  msg: PNumberExpression;
  Flags: PNumberExpression;
  No: PNumberExpression;
  Killed: PBooleanExpression;
  eachMsgActions: PActions;

  constructor Init(const Tokenizer: PTokenizer; const Actions: PActions);
  function Execute(const Data: PExecuteData): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

 Pnaddr = ^Tnaddr;
 Tnaddr = object(TNumberExpression)
  Expression: PNumberExpression;

  constructor Init(const Tokenizer: PTokenizer; const AOwner: PAtom);
  procedure Get(const ExecuteData: PExecuteData; var Value); virtual;
  destructor Done; virtual;
 end;

procedure UnpackTechInfo(const TechInfo: TTechInfo; var Net, Node: Longint); {*}
procedure PackTechInfo(const Net, Node: Longint; var TechInfo: TTechInfo); {*}

procedure UnpackAreaFormat(const AreaFormat: TAreaFormat; var Style, Format: Longint; var Path: String); {*}
procedure PackAreaFormat(const Style, Format: Longint; const Path: String; var AreaFormat: TAreaFormat); {*}

procedure UnpackAddress(const Address: TAddress; var Zone, Net, Node, Point: Longint); {*}
procedure PackAddress(const Zone, Net, Node, Point: Longint; var Address: TAddress); {*}

procedure UnpackTime(const DateTime: TDateTime; var Hour, Min, Sec: Longint); {*}
procedure UnpackDate(const DateTime: TDateTime; var Year, Month, Day: Longint); {*}
procedure UnpackDateTime(const DateTime: TDateTime; var Hour, Min, Sec, Year, Month, Day: Longint); {*}
procedure PackTime(const Hour, Min, Sec: Longint; var DateTime: TDateTime); {*}
procedure PackDate(const Year, Month, Day: Longint; var DateTime: TDateTime); {*}
procedure PackDateTime(const Hour, Min, Sec, Year, Month, Day: Longint; var DateTime: TDateTime); {*}

procedure Touch(const FName: String); {*}
procedure TouchOnce(const FName: String); {*}

function Pstos(const S: PString): String; {*}

function Chr(const N: Longint): String; {*}

function Pos(const SubStr, Str: String): Longint; {*}
function Copy(const S: String; const Index, Count: Longint): String; {*}
procedure Delete(var S: String; const Index, Count: Longint); {*}

procedure Inc(var N: Longint); {*}
procedure Dec(var N: Longint); {*}

procedure Carbon(const smsg: Pointer; const AreaName: String); {*}

function Speed(A, B: Longint): String; {*}

procedure AddSupportRoutines(const Target: PActions);

implementation

{ Tmatchre }

constructor Tmatchre.Init(const AName: String; const Tokenizer: PTokenizer; const AOwner: PAtom);
 var
  S: String;
 begin
  inherited Init(AName, AOwner);

  Tokenizer^.Next([tmOpenBracket]);

  Expression:=PActions(AOwner)^.ParseStringExpression(Tokenizer, True);

  Tokenizer^.Next([tmComma]);

  S:=Tokenizer^.Next([tmString])^.Value^;

  re:=jt.re.CreateStr(S);

  if re = nil then
   Tokenizer^.Error('regexp compiler returned nil on expression "' + S + '"');

  if not jt.re.IsValid(re) then
   Tokenizer^.Error(jt.re.GetErrorMsg(re) + ' in expression "' + S + '"');

  Tokenizer^.Next([tmCloseBracket]);
 end;

procedure Tmatchre.Get(const ExecuteData: PExecuteData; var Value);
 var
  S: String;
 begin
  Expression^.Get(ExecuteData, S);

  Boolean(Value):=jt.re.MatchStr(re, S, False);
 end;

destructor Tmatchre.Done;
 begin
  jt.re.Destroy(re);

  Dispose(Expression, Done);

  inherited Done;
 end;

function CreatorMATCHRE(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorMATCHRE:=New(Pmatchre, Init('matchre', Tokenizer, Actions));
 end;

{ Tmatchrecs }

procedure Tmatchrecs.Get(const ExecuteData: PExecuteData; var Value);
 var
  S: String;
 begin
  Expression^.Get(ExecuteData, S);

  Boolean(Value):=jt.re.MatchStr(re, S, True);
 end;

function CreatorMATCHRECS(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorMATCHRECS:=New(Pmatchrecs, Init('matchrecs', Tokenizer, Actions));
 end;

{ TeachArea }

constructor TeachArea.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('eachArea', Actions);

  Tokenizer^.OpenBracket;

  re:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmComma]);

  Area:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  if not Area^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etNumber);

  Tokenizer^.CloseBracket;

  eachAreaActions:=New(PActions, Init('eachArea-actions', Tokenizer, Actions, [tfAllowBreak, tfAllowContinue]));
 end;

function TeachArea.Execute(const Data: PExecuteData): TExecuteResult;
 var
  ExecuteResult: TExecuteResult;
  Are, AArea: Pointer;
  K: Longint;
 begin
  re^.Get(Data, Are);

  for K:=0 to jt.area.Count - 1 do
   begin
    AArea:=jt.area.At(K);

    if (Are = nil) or jt.AreaMatchRegExp(Are, jt.area.GetName(jt.area.At(K))) then
     begin
      Area^.Assign(Data, AArea);

      ExecuteResult:=eachAreaActions^.Execute(Data);

      case ExecuteResult of
       erOk, erContinue:;
       erBreak:
        begin
         Execute:=erOk;

         Exit;
        end;
      else
       Execute:=ExecuteResult;

       Exit;
      end;
     end;
   end;

  Execute:=erOk;
 end;

destructor TeachArea.Done;
 begin
  Dispose(eachAreaActions, Done);
  Dispose(Area, Done);
  Dispose(re, Done);

  inherited Done;
 end;

function CreatorEACHAREA(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorEACHAREA:=New(PeachArea, Init(Tokenizer, Actions));
 end;

{ TeachMsg }

constructor TeachMsg.Init(const Tokenizer: PTokenizer; const Actions: PActions);
 begin
  inherited Init('eachMsg', Actions);

  Tokenizer^.OpenBracket;

  mb:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmComma]);

  msg:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  if not msg^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etNumber);

  Tokenizer^.Next([tmComma]);

  Flags:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  Tokenizer^.Next([tmComma]);

  No:=Actions^.ParseNumberExpression(Tokenizer, True, []);

  if not No^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etNumber);

  Tokenizer^.Next([tmComma]);

  Killed:=Actions^.ParseBooleanExpression(Tokenizer, True);

  if not Killed^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etBoolean);

  Tokenizer^.CloseBracket;

  eachMsgActions:=New(PActions, Init('eachMsg-actions', Tokenizer, Actions, [tfAllowBreak, tfAllowContinue]));
 end;

function TeachMsg.Execute(const Data: PExecuteData): TExecuteResult;
 var
  ExecuteResult: TExecuteResult;
  Stop, AKilled: Boolean;
  AFlags, ANo: Longint;
  Amb, Amsg: Pointer;
  rc: String;
 begin
  mb^.Get(Data, Amb);
  Flags^.Get(Data, AFlags);

  Execute:=erOk;

  ANo:=0;

  while ANo < jt.mb.Count(Amb) do
   if jt.mb.Load(Amb, ANo, Amsg, rc, AFlags) then
    begin
     AKilled:=False;

     msg^.Assign(Data, Amsg);
     No^.Assign(Data, ANo);
     Killed^.Assign(Data, AKilled);

     ExecuteResult:=eachMsgActions^.Execute(Data);

     case ExecuteResult of
      erOk, erContinue: Stop:=False;
      erBreak:
       begin
        Execute:=erOk;

        Stop:=True;
       end;
     else
      Execute:=ExecuteResult;

      Stop:=True;
     end;

     msg^.Get(Data, Amsg);

     if Amsg <> nil then
      jt.msg.Destroy(Amsg);

     Killed^.Get(Data, AKilled);

     if not AKilled then
      System.Inc(ANo);

     if Stop then
      Break;
    end
   else
    System.Inc(ANo);
 end;

destructor TeachMsg.Done;
 begin
  Dispose(eachMsgActions, Done);
  Dispose(Killed, Done);
  Dispose(No, Done);
  Dispose(Flags, Done);
  Dispose(msg, Done);
  Dispose(mb, Done);

  inherited Done;
 end;

function CreatorEACHMSG(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorEACHMSG:=New(PeachMsg, Init(Tokenizer, Actions));
 end;

{ Tnaddr }

constructor Tnaddr.Init(const Tokenizer: PTokenizer; const AOwner: PAtom);
 var
  AExpression: PExpression;
 begin
  Tokenizer^.OpenBracket;

  AExpression:=PActions(AOwner)^.ParseNumberExpression(Tokenizer, True, []);

  inherited Init('naddr(' + AExpression^.Name^ + ')', AOwner);

  Expression:=PNumberExpression(AExpression);

  if not Expression^.Assignable then
   Tokenizer^.AssignableExpressionExpected(etNumber);

  Tokenizer^.CloseBracket;
 end;

procedure Tnaddr.Get(const ExecuteData: PExecuteData; var Value);
 begin
  Pointer(Value):=@PNumberExpression(Expression^.GetOriginal)^.Data;
 end;

destructor Tnaddr.Done;
 begin
  Dispose(Expression, Done);

  inherited Done;
 end;

function CreatorNADDR(const Tokenizer, Actions: Pointer): Pointer;
 begin
  CreatorNADDR:=New(Pnaddr, Init(Tokenizer, Actions));
 end;

{ TechInfo }

procedure UnpackTechInfo(const TechInfo: TTechInfo; var Net, Node: Longint);
 begin
  Net:=TechInfo.Net;
  Node:=TechInfo.Node;
 end;

procedure PackTechInfo(const Net, Node: Longint; var TechInfo: TTechInfo);
 begin
  TechInfo.Net:=Net;
  TechInfo.Node:=Node;
 end;

{ AreaFormat }

procedure UnpackAreaFormat(const AreaFormat: TAreaFormat; var Style, Format: Longint; var Path: String);
 begin
  Style:=AreaFormat.Style;
  Format:=AreaFormat.Format;
  Path:=AreaFormat.Path;
 end;

procedure PackAreaFormat(const Style, Format: Longint; const Path: String; var AreaFormat: TAreaFormat);
 begin
  AreaFormat.Style:=Style;
  AreaFormat.Format:=Format;
  AreaFormat.Path:=Path;
 end;

{ Address }

procedure UnpackAddress(const Address: TAddress; var Zone, Net, Node, Point: Longint);
 begin
  Zone:=Address.Zone;
  Net:=Address.Net;
  Node:=Address.Node;
  Point:=Address.Point;
 end;

procedure PackAddress(const Zone, Net, Node, Point: Longint; var Address: TAddress);
 begin
  Address.Zone:=Zone;
  Address.Net:=Net;
  Address.Node:=Node;
  Address.Point:=Point;
 end;

{ DateTime }

procedure UnpackTime(const DateTime: TDateTime; var Hour, Min, Sec: Longint);
 begin
  Hour:=DateTime.Hour;
  Min:=DateTime.Min;
  Sec:=DateTime.Sec;
 end;

procedure UnpackDate(const DateTime: TDateTime; var Year, Month, Day: Longint);
 begin
  Year:=DateTime.Year;
  Month:=DateTime.Month;
  Day:=DateTime.Day;
 end;

procedure UnpackDateTime(const DateTime: TDateTime; var Hour, Min, Sec, Year, Month, Day: Longint);
 begin
  Hour:=DateTime.Hour;
  Min:=DateTime.Min;
  Sec:=DateTime.Sec;
  Year:=DateTime.Year;
  Month:=DateTime.Month;
  Day:=DateTime.Day;
 end;

procedure PackTime(const Hour, Min, Sec: Longint; var DateTime: TDateTime);
 begin
  DateTime.Hour:=Hour;
  DateTime.Min:=Min;
  DateTime.Sec:=Sec;
 end;

procedure PackDate(const Year, Month, Day: Longint; var DateTime: TDateTime);
 begin
  DateTime.Year:=Year;
  DateTime.Month:=Month;
  DateTime.Day:=Day;
 end;

procedure PackDateTime(const Hour, Min, Sec, Year, Month, Day: Longint; var DateTime: TDateTime);
 begin
  DateTime.Hour:=Hour;
  DateTime.Min:=Min;
  DateTime.Sec:=Sec;
  DateTime.Year:=Year;
  DateTime.Month:=Month;
  DateTime.Day:=Day;
 end;

{ Touch }

procedure Touch(const FName: String);
 begin
  if TouchFile(FName) then
   jt.LogWrite('&', FName + ' touched');
 end;

{ TouchOnce }

procedure TouchOnce(const FName: String);
 begin
  if not ExistFile(FName) and TouchFile(FName) then
   jt.LogWrite('&', FName + ' touched');
 end;

{ pstos }

function pstos(const S: PString): String;
 begin
  if S = nil then
   pstos:=''
  else
   pstos:=S^;
 end;

{ Chr }

function Chr(const N: Longint): String;
 begin
  Chr:=System.Chr(N);
 end;

{ Pos }

function Pos(const SubStr, Str: String): Longint;
 begin
  Pos:=System.Pos(SubStr, Str);
 end;

{ Copy }

function Copy(const S: String; const Index, Count: Longint): String;
 begin
  Copy:=System.Copy(S, Index, Count);
 end;

{ Delete }

procedure Delete(var S: String; const Index, Count: Longint);
 begin
  System.Delete(S, Index, Count);
 end;

{ Inc }

procedure Inc(var N: Longint);
 begin
  System.Inc(N);
 end;

{ Dec }

procedure Dec(var N: Longint);
 begin
  System.Dec(N);
 end;

{ Carbon }

procedure Carbon(const smsg: Pointer; const AreaName: String);
 var
  Area, msg, st: Pointer;
  Source, Destination: PStream;
  S: String;
 begin
  Area:=jt.area.Seek(AreaName);

  if Area = nil then
   begin
    jt.LogWrite('?', 'Reference to undefined area "' + AreaName + '"');

    Exit;
   end;

  msg:=jt.msg.Clone(smsg, mcfCloneHeader);

  st:=jt.msg.st(msg);

  jt.st.Add(st, 'Source', Pointer(tsLocalInbound), nil);

  jt.st.Add(st, 'NoCallbacks', Pointer(1), nil);

  jt.msg.SetArea(msg, jt.area.GetName(Area)^);

  jt.msg.Add(msg, 'AREA:' + jt.msg.GetArea(smsg)^);

  Source:=New(PStreamThunk, Init(jt.msg.GetText(smsg), jt.StreamLinker, False));
  Destination:=New(PStreamThunk, Init(jt.msg.GetText(msg), jt.StreamLinker, False));

  Source^.Seek(0);

  Destination^.Seek(Destination^.GetSize);

  Destination^.CopyFrom(Source^, Source^.GetSize);

  Dispose(Destination, Done);
  Dispose(Source, Done);

  jt.tossscan.TossMessage(msg);
 end;

{ Speed }

function Speed(A, B: Longint): String;
 begin
  if B = 0 then
   B:=1;

  Speed:=Real2Str(A/B, 1, 2);
 end;

{ AddSupportRoutines }

procedure AddSupportRoutines(const Target: PActions);
 begin
  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto,
    New(PProtoCreator, Init('MATCHRE', CreatorMATCHRE)));

  Target^.Identifiers^.AddIdentifier(imBooleanExpressionProto,
    New(PProtoCreator, Init('MATCHRECS', CreatorMATCHRECS)));

  Target^.Identifiers^.AddIdentifier(imActionProto,
    New(PProtoCreator, Init('EACHAREA', CreatorEACHAREA)));

  Target^.Identifiers^.AddIdentifier(imActionProto,
    New(PProtoCreator, Init('EACHMSG', CreatorEACHMSG)));

  Target^.Identifiers^.AddIdentifier(imNumberExpressionProto,
    New(PProtoCreator, Init('NADDR', CreatorNADDR)));
 end;

end.