library Check;

uses
     PlugAPI,
     PlugSIDE,

     jtMHLcmn,

     Wizard;

const
 CheckPlugin    : Pointer       = nil;

 CheckPurge                     = $1000;
 CheckLinked                    = $1001;
 CheckDomains                   = $1002;
 CheckOM                        = $1003;

type
 TPerformArea = function(const Area: Pointer): Boolean;

{ Info }

procedure Info;
 var
  Plugin: Pointer;
  Information: TPluginInformation;
  FName: String;
  K: Longint;
 begin
  jt.PrintStr('API: ' + Long2Str(SizeOf(jt)) + ' (' + Long2Str(SizeOf(jt) div 4) + ' entries)'#13#10#13#10);

  jt.PrintStr('Regular expressions:'#13#10 + jt.re.Title + #13#10#13#10);

  jt.PrintStr(' ' + Pad('filename', 12) + ' ' +
                    LeftPad('version', 7) + ' ' +
                    LeftPad('kver', 4) + ' ' +
                    'plugin name'#13#10);

  jt.PrintStr(' ' + PadCh('', '-', 12) + ' ' +
                    PadCh('', '-', 7) + ' ' +
                    PadCh('', '-', 4) + ' ' +
                    PadCh('', '-', 40) + #13#10);

  for K:=0 to jt.plugs.Count - 1 do
   begin
    Plugin:=jt.plugs.At(K);

    jt.plugs.GetInformation(Plugin, Information);
    jt.plugs.GetPluginFName(Plugin, FName);

    jt.PrintStr(' ' + Pad(Copy(JustFileName(FName), 1, 12), 12) + ' ' +
                      LeftPad(Long2Str(Information.Version), 7) + ' ' +
                      LeftPad(Long2Str(Information.KernelVersion), 4) + ' ' +
                      Information.Name + #13#10);
   end;

  jt.PrintStr(' ' + PadCh('', '-', 12) + ' ' +
                    PadCh('', '-', 7) + ' ' +
                    PadCh('', '-', 4) + ' ' +
                    PadCh('', '-', 40) + #13#10);

  jt.PrintStr(' total: ' + Long2Str(jt.plugs.Count) + ' plugins loaded'#13#10#13#10);

  jt.PrintStr('memavail/maxavail/stack: ' + Long2Str(MemAvail) + '/' +
                                            Long2Str(MaxAvail) + '/' +
                                            Long2Str(SPtr) + #13#10);
 end;

{ PerformAreasRegExp }

procedure PerformAreasRegExp(const re: Pointer; const Perform: TPerformArea);
 var
  K: Longint;
  Area: Pointer;
 begin
  if Perform(nil) then
   for K:=0 to jt.area.Count - 1 do
    begin
     Area:=jt.area.At(K);

     if jt.re.MatchStr(re, jt.area.GetName(Area)^, False) then
      if Perform(Area) then
       Break;
    end;

  jt.DestroyRegExp(re);
 end;

{ CheckPurgeSettings }

function CheckPurgeSettings(const Area: Pointer): Boolean;
 const
  DaysWidth               = 6;
  MessagesWidth           = 6;
  PurgeDays     : Pointer = nil;
  PurgeMessages : Pointer = nil;
  PurgeDisable  : Pointer = nil;
 var
  st: Pointer;
  Days, Messages: Longint;
 function Long2StrEx(const Value: Longint): String;
  begin
   if Value = 0 then
    Long2StrEx:=''
   else
    Long2StrEx:=Long2Str(Value);
  end;
 begin
  if Area = nil then
   begin
    PurgeDays:=jt.rests.Add('PurgeDays', rsIsExists);
    PurgeMessages:=jt.rests.Add('PurgeMessages', rsIsExists);
    PurgeDisable:=jt.rests.Add('PurgeDisable', rsIsExists);

    if (PurgeDays = nil) or (PurgeMessages = nil) or (PurgeDisable = nil) then
     begin
      jt.LogWrite('C', 'Purger seems like non-loaded.');

      CheckPurgeSettings:=False;
     end
    else
     begin
      jt.LogWrite('C', LeftPad('days', DaysWidth) + ' | ' + LeftPad('msgs', MessagesWidth) + ' | area // PURGE SETTINGS');

      CheckPurgeSettings:=True;
     end;
   end
  else
   begin
    { // purge days }

    jt.rests.Getst(PurgeDays, nil, jt.area.GetName(Area), st);

    if st = nil then
     Days:=0
    else
     Days:=Longint(jt.st.Get(st, 'Days'));

    { // purge messages }

    jt.rests.Getst(PurgeMessages, nil, jt.area.GetName(Area), st);

    if st = nil then
     Messages:=0
    else
     Messages:=Longint(jt.st.Get(st, 'Messages'));

    if jt.rests.Check(PurgeDisable, nil, jt.area.GetName(Area)) then
     jt.LogWrite('C', LeftPad('-', DaysWidth) + ' | ' +
                      LeftPad('-', MessagesWidth) + ' | ' +
                      jt.area.GetName(Area)^)
    else
     jt.LogWrite('C', LeftPad(Long2StrEx(Days), DaysWidth) + ' | ' +
                      LeftPad(Long2StrEx(Messages), MessagesWidth) + ' | ' +
                      jt.area.GetName(Area)^);

    CheckPurgeSettings:=jt.Cancelled;
   end;
 end;

{ CheckLinkedToArea }

function CheckLinkedToArea(const Area: Pointer): Boolean;
 var
  Link, LinkArea: Pointer;
  Address: TAddress;
  Rights: String;
  K: Longint;
 begin
  if Area = nil then
   CheckLinkedToArea:=True
  else
   begin
    for K:=0 to jt.link.Count - 1 do
     begin
      Link:=jt.link.At(K);

      LinkArea:=jt.link.SeekArea(Link, Area);

      if (LinkArea <> nil) and jt.link.area.IsSubscribed(LinkArea) then
       begin
        Rights:='';

        if jt.link.area.IsReadable(LinkArea) then Rights:=Concat(Rights, ', read');
        if jt.link.area.IsWriteable(LinkArea) then Rights:=Concat(Rights, ', write');
        if jt.link.area.IsPassive(LinkArea) then Rights:=Concat(Rights, ', passive');

        if Rights = '' then
         Rights:=', no rights (brain is dead?)';

        jt.link.GetAddress(Link, Address);

        jt.LogWrite('C', jt.area.GetName(Area)^ + ': ' + AddressToStrEx(Address) + Rights);
       end;
     end;

    CheckLinkedToArea:=jt.Cancelled;
   end;
 end;

{ PerformCheckDomains }

procedure PerformCheckDomains;
 var
  Domain, DomainOutbound: String;
  K, MainZone: Longint;
  Address: TAddress;
  Link: Pointer;
 begin
  jt.LogWrite('C', 'Domain settings:');

  for K:=0 to jt.link.Count - 1 do
   begin
    Link:=jt.link.At(K);

    jt.link.GetDomainInfo(Link, Domain, DomainOutbound, MainZone);
    jt.link.GetAddress(Link, Address);

    if Address.Zone <> MainZone then
     DomainOutbound:=Concat(DomainOutbound, '.', HexT(Address.Zone));

    jt.LogWrite('C', AddressToStrEx(Address) + '@' + Domain + ', BSO: ' + DomainOutbound);
   end;
 end;

{ PerformCheckOM }

procedure PerformCheckOM;
 function PluginName(const Plugin: Pointer): String;
  begin
   if Plugin = nil then
    PluginName:='?'
   else
    PluginName:=jt.plugs.GetPluginName(Plugin)^;
  end;
 var
  K: Longint;
  om: Pointer;
  Reader: PFormatReader;
  Writer: PFormatWriter;
  Preprocessor: PFormatPreprocessor;
  Outprocessor: PFormatOutprocessor;
  S: String;
 begin
  jt.LogWrite('C', 'Outbound modes:');

  for K:=0 to jt.om.Count - 1 do
   begin
    om:=jt.om.At(K);

    jt.LogWrite('C', ' - ' + jt.om.GetName(om) + ', plugin: ' + PluginName(jt.om.GetPlugin(om)));
   end;

  jt.LogWrite('C', 'Format readers:');

  for K:=0 to jt.fmt.r.Count - 1 do
   begin
    Reader:=jt.fmt.r.At(K);

    S:=' - ' + Reader^.Name^ + ', extension: ' + Reader^.Extension^ +
                               ', plugin: ' + PluginName(Reader^.Plugin);

    if Reader^.AssociatedWriter = nil then
     S:=S + ', no associated writer'
    else
     S:=S + ', associated writer: ' + Reader^.AssociatedWriter^.Name^;

    jt.LogWrite('C', S);
   end;

  jt.LogWrite('C', 'Format writers:');

  for K:=0 to jt.fmt.w.Count - 1 do
   begin
    Writer:=jt.fmt.w.At(K);

    S:=' - ' + Writer^.Name^ + ', extension: ' + Writer^.Extension^ +
                               ', plugin: ' + PluginName(Writer^.Plugin);

    if Writer^.AssociatedReader = nil then
     S:=S + ', no associated reader'
    else
     S:=S + ', associated reader: ' + Writer^.AssociatedReader^.Name^;

    jt.LogWrite('C', S);
   end;

  jt.LogWrite('C', 'Format preprocessors:');

  for K:=0 to jt.fmt.pp.Count - 1 do
   begin
    Preprocessor:=jt.fmt.pp.At(K);

    jt.LogWrite('C', ' - ' + Preprocessor^.Name^ + ', plugin: ' + PluginName(Preprocessor^.Plugin));
   end;

  jt.LogWrite('C', 'Format outprocessors:');

  for K:=0 to jt.fmt.op.Count - 1 do
   begin
    Outprocessor:=jt.fmt.op.At(K);

    jt.LogWrite('C', ' - ' + Outprocessor^.Name^ + ', plugin: ' + PluginName(Outprocessor^.Plugin));
   end;
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, 'Check', 1);

  CheckPlugin:=Plugin;

  PluginSetup:=True;
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 begin
  if Command = CheckPurge then PerformAreasRegExp(Pointer(Param), CheckPurgeSettings) else
  if Command = CheckLinked then PerformAreasRegExp(Pointer(Param), CheckLinkedToArea) else
  if Command = CheckDomains then PerformCheckDomains else
  if Command = CheckOM then PerformCheckOM;
 end;

{ PluginHandleCmdlineSwitch }

function PluginHandleCmdlineSwitch(Switch: String): Boolean; export;
 procedure PushRegExpCommand(const Skip: Byte; const Command: Longint; const When: TCommandWhen);
  var
   re: Pointer;
  begin
   Delete(Switch, 1, Skip);

   jt.CompileRegExp(Switch, re, jt.linker.Stub, False);

   jt.EnqueueCommand(CheckPlugin, Command, Longint(re), When);
  end;
 var
  SwitchU: String;
 begin
  SwitchU:=StUpcase(Switch);

  PluginHandleCmdlineSwitch:=True;

  if SwitchU = 'INFO' then
   begin
    Info;

    PluginHandleCmdlineSwitch:=False;
   end else
  if Copy(SwitchU, 1, 11) = 'CHECKPURGE:' then PushRegExpCommand(11, CheckPurge, cwEarliest) else
  if Copy(SwitchU, 1, 12) = 'CHECKLINKED:' then PushRegExpCommand(12, CheckLinked, cwEarliest) else
  if SwitchU = 'CHECKDOMAINS' then jt.EnqueueCommand(CheckPlugin, CheckDomains, 0, cwEarliest) else
  if SwitchU = 'OMINFO' then jt.EnqueueCommand(CheckPlugin, CheckOM, 0, cwEarliest) else
   PluginHandleCmdlineSwitch:=False;
 end;

{ PluginShowHelp }

procedure PluginShowHelp; export;
 begin
  jt.PrintStr(' INFO             print some info about plugins, kernel, etc'#13#10);
  jt.PrintStr(' CHECKPURGE:re    check purge settings for specified areas'#13#10);
  jt.PrintStr(' CHECKLINKED:re   check linked to specified areas'#13#10);
  jt.PrintStr(' CHECKDOMAINS     check domain settings'#13#10);
  jt.PrintStr(' OMINFO           show all registered outbound modes, readers, writers etc'#13#10);
 end;

exports
 PluginSetup,
 PluginHandleCommand,
 PluginHandleCmdlineSwitch,
 PluginShowHelp;

begin
end.