unit Columns;

interface
uses
     PlugAPI,
     PlugSIDE,

     Objects,

     Wizard;

type
 TColumnAlign = (caLeft, caCenter, caRight);

 PColumn = ^TColumn;
 TColumn = object(TObject)
  Width: Byte;
  Data: PStrings;

  constructor Init(const ABeforePad, AWrapPad, AWidth: Byte; const AAlign: TColumnAlign; Value: String);
  procedure PadHeightTo(const Maximum: Longint); virtual;
  destructor Done; virtual;
 end;

 PColumnFiller = ^TColumnFiller;
 TColumnFiller = object(TColumn)
  constructor Init(const ABeforePad, AWidth: Byte; const Value: Char);
 end;

 PColumnRow = ^TColumnRow;
 TColumnRow = object(TCollection)
  procedure FormatRow(const Rows: PStrings);
 end;

 PColumnRows = ^TColumnRows;
 TColumnRows = object(TCollection)
  function Add: PColumnRow;
  procedure FormatRows(const Rows: PStrings);
  procedure FormatRowsTomsg(const msg: Pointer);
 end;

implementation

{ TColumn }

constructor TColumn.Init(const ABeforePad, AWrapPad, AWidth: Byte; const AAlign: TColumnAlign; Value: String);
 var
  First: Boolean;
 function Padder(var Skipped: Byte): String;
  begin
   if First then
    begin
     Skipped:=0;

     Padder:=Pad('', ABeforePad);

     First:=False;
    end
   else
    begin
     Skipped:=AWrapPad;

     Padder:=Pad('', ABeforePad + AWrapPad);
    end;
  end;
 var
  Skipped: Byte;
 begin
  inherited Init;

  Width:=AWidth;
  Data:=New(PStrings, Init($1, $4));

  First:=True;

  while Length(Value) > Width do
   begin
    Data^.Insert(NewStr(Padder(Skipped) + Copy(Value, 1, Width - Skipped)));

    Delete(Value, 1, Width);
   end;

  if Value <> '' then
   case AAlign of
    caLeft: Data^.Insert(NewStr(Padder(Skipped) + Pad(Value, Width - Skipped)));
    caCenter: Data^.Insert(NewStr(Padder(Skipped) + Center(Value, Width - Skipped)));
    caRight: Data^.Insert(NewStr(Padder(Skipped) + LeftPad(Value, Width - Skipped)));
   end;

  Inc(Width, ABeforePad);
 end;

procedure TColumn.PadHeightTo(const Maximum: Longint);
 var
  K: Longint;
 begin
  for K:=Data^.Count + 1 to Maximum do
   Data^.Insert(NewStr(Pad('', Width)));
 end;

destructor TColumn.Done;
 begin
  if Data <> nil then
   Dispose(Data, Done);

  inherited Done;
 end;

{ TColumnFiller }

constructor TColumnFiller.Init(const ABeforePad, AWidth: Byte; const Value: Char);
 var
  S: String;
  K: Byte;
 begin
  TObject.Init;

  Width:=AWidth;
  Data:=New(PStrings, Init($1, $1));

  S:='';

  for K:=1 to ABeforePad do
   S:=S + ' ';

  for K:=1 to AWidth do
   S:=S + Value;

  Data^.Insert(NewStr(S));

  Inc(Width, ABeforePad);
 end;

{ TColumnRow }

procedure TColumnRow.FormatRow(const Rows: PStrings);
 var
  MaxHeight: Longint;
 procedure CountMaxHeight(const Column: PColumn); far;
  begin
   if Column^.Data^.Count > MaxHeight then
    MaxHeight:=Column^.Data^.Count;
  end;
 procedure PadHeightTo(const Column: PColumn); far;
  begin
   Column^.PadHeightTo(MaxHeight);
  end;
 var
  Line: Longint;
  Row: String;
 procedure AddRowData(const Column: PColumn); far;
  begin
   Row:=Row + PString(Column^.Data^.At(Line))^;
  end;
 begin
  if Count > 0 then
   begin
    MaxHeight:=1;

    ForEach(@CountMaxHeight);
    ForEach(@PadHeightTo);

    for Line:=0 to MaxHeight - 1 do
     begin
      Row:='';

      ForEach(@AddRowData);

      Rows^.Insert(NewStr(Row));
     end;
   end;
 end;

{ TColumnRows }

function TColumnRows.Add: PColumnRow;
 var
  Row: PColumnRow;
 begin
  Row:=New(PColumnRow, Init(1, 4));

  Insert(Row);

  Add:=Row;
 end;

procedure TColumnRows.FormatRows(const Rows: PStrings);
 procedure FormatRow(const Row: PColumnRow); far;
  begin
   Row^.FormatRow(Rows);
  end;
 begin
  ForEach(@FormatRow);
 end;

procedure TColumnRows.FormatRowsTomsg(const msg: Pointer);
 var
  Rows: PStrings;
 procedure Tomsg(const S: PString); far;
  begin
   if S = nil then
    jt.msg.Add(msg, '')
   else
    jt.msg.Add(msg, S^);
  end;
 begin
  Rows:=New(PStrings, Init($100, $100));

  FormatRows(Rows);

  Rows^.ForEach(@Tomsg);

  Dispose(Rows, Done);
 end;

end.