unit dchkCore;

interface
uses
     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     Objects,

     RStream,
     Thunks,

     TossAPI,
     ccAPI,

     dchkCmn,

     jtMHLcmn,

     Wizard;

type
 PAreaPoolItem = ^TAreaPoolItem;
 TAreaPoolItem = packed record
  Area: PString;
  Index: System.Integer;
 end;

 PLinkPoolItem = ^TLinkPoolItem;
 TLinkPoolItem = packed record
  Link: TAddress;
  Index: System.Integer;
 end;

 PPool = ^TPool;
 TPool = object(TStringCollection)
  Name: PString;

  constructor Init(const AName: String);
  destructor Done; virtual;
 end;

 PAreaPool = ^TAreaPool;
 TAreaPool = object(TPool)
  function KeyOf(Item: Pointer): Pointer; virtual;
  function GetItem(var S: TStream): Pointer; virtual;
  procedure PutItem(var S: TStream; Item: Pointer); virtual;
  procedure FreeItem(Item: Pointer); virtual;
 public
  function QueryIndex(const Area: Pointer): System.Integer;
  function QueryArea(const Index: System.Integer; var AArea: String): Boolean;
 end;

 PLinkPool = ^TLinkPool;
 TLinkPool = object(TPool)
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  function KeyOf(Item: Pointer): Pointer; virtual;
  function GetItem(var S: TStream): Pointer; virtual;
  procedure PutItem(var S: TStream; Item: Pointer); virtual;
  procedure FreeItem(Item: Pointer); virtual;
 public
  function QueryIndex(const Link: Pointer): System.Integer;
  function QueryLink(const Index: System.Integer; var ALink: TAddress): Boolean;
 end;

 PDupeRecord = ^TDupeRecord;
 TDupeRecord = packed record
  { information }
  Time       : Longint;                 { unixtime of msg.datetime }
  MSGIDhex   : Longint;                 { hexadecimal value of MSGID }
  Area       : System.Integer;          { area index in areas pool }
  Link       : System.Integer;          { link index in links pool }

  { comparing }
  Body       : Longint;                 { crc32 of msgbody }
  MSGID      : Longint;                 { stringcrc32 of @msgid kludge }
 end;

 PDupeBase = ^TDupeBase;
 TDupeBase = object(TSortedCollection)
  Name: PString;
  Changed: Boolean;
  Maximum: Longint;

  constructor Init(const AName: String; const AMaximum: Longint);
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  procedure FreeItem(Item: Pointer); virtual;
  destructor Done; virtual;
 public
  procedure Add(const AItem: TDupeRecord; const Index: xInteger);
 end;

const
 AreaPool               : PAreaPool             = nil;
 LinkPool               : PLinkPool             = nil;

 UnitedDupeBase         : PDupeBase             = nil;

 MessagesProcessed      : Longint               = 0;

procedure MessageTossFilter(var Block: TTossMessageBlock);

function GetDupeBase(const Area: Pointer): PDupeBase;
procedure Wakeup;
procedure Flush;
procedure Gather(const Area, Link, msg: Pointer; var Item: TDupeRecord);

function stDupeBase(const Action: TStorageAction; const Data: Pointer): Pointer; export;

implementation

{ TPool }

constructor TPool.Init(const AName: String);
 var
  Stream: PRamStream;
  Countdown: Longint;
  Item: Pointer;
 begin
  inherited Init($100, $100);

  Name:=NewStr(AName);

  Stream:=New(PRamStream, Init);

  if jt.rf.Get(DupeCheckerResourceFile, Name^, Stream, StreamLinker) then
   begin
    Stream^.Seek(0);

    Stream^.Read(Countdown, SizeOf(Countdown));

    while Countdown > 0 do
     begin
      Item:=GetItem(Stream^);

      if Item <> nil then
       Insert(Item);

      Dec(Countdown);
     end;
   end;

  Dispose(Stream, Done);
 end;

destructor TPool.Done;
 var
  Stream: PRamStream;
 procedure Perform(const Item: Pointer); far;
  begin
   PutItem(Stream^, Item);
  end;
 var
  Countdown: Longint;
 begin
  Stream:=New(PRamStream, Init);

  Countdown:=Count;

  Stream^.Write(Countdown, SizeOf(Countdown));

  ForEach(@Perform);

  jt.rf.Add(DupeCheckerResourceFile, Name^, Stream, StreamLinker);

  Dispose(Stream, Done);

  DisposeStr(Name);

  inherited Done;
 end;

{ TAreaPool }

function TAreaPool.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=PAreaPoolItem(Item)^.Area;
 end;

function TAreaPool.GetItem(var S: TStream): Pointer;
 var
  Item: PAreaPoolItem;
 begin
  New(Item);

  Item^.Area:=S.ReadStr;

  S.Read(Item^.Index, SizeOf(Item^.Index));

  if Item^.Area <> nil then
   GetItem:=Item
  else
   begin
    Dispose(Item);

    GetItem:=nil;
   end;
 end;

procedure TAreaPool.PutItem(var S: TStream; Item: Pointer);
 begin
  with PAreaPoolItem(Item)^ do
   begin
    S.WriteStr(Area);

    S.Write(Index, SizeOf(Index));
   end;
 end;

procedure TAreaPool.FreeItem(Item: Pointer);
 begin
  DisposeStr(PAreaPoolItem(Item)^.Area);

  Dispose(PAreaPoolItem(Item));
 end;

{ public methods }

function TAreaPool.QueryIndex(const Area: Pointer): System.Integer;
 var
  Index: xInteger;
  Item: PAreaPoolItem;
 begin
  if Search(jt.area.GetName(Area), Index) then
   QueryIndex:=PAreaPoolItem(At(Index))^.Index
  else
   begin
    New(Item);

    Item^.Area:=NewStr(jt.area.GetName(Area)^);
    Item^.Index:=Count;

    AtInsert(Index, Item);

    QueryIndex:=Item^.Index;
   end;
 end;

function TAreaPool.QueryArea(const Index: System.Integer; var AArea: String): Boolean;
 function Perform(const Item: PAreaPoolItem): Boolean; far;
  begin
   if Item^.Index = Index then
    begin
     AArea:=Item^.Area^;

     Perform:=True;
    end
   else
    Perform:=False;
  end;
 begin
  QueryArea:=FirstThat(@Perform) <> nil;
 end;

{ TLinkPool }

function TLinkPool.Compare(Key1, Key2: Pointer): xInteger;
 begin
  Compare:=AddressCompare(PAddress(Key1)^, PAddress(Key2)^);
 end;

function TLinkPool.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=@PLinkPoolItem(Item)^.Link;
 end;

function TLinkPool.GetItem(var S: TStream): Pointer;
 var
  Item: PLinkPoolItem;
 begin
  New(Item);

  S.Read(Item^, SizeOf(Item^));

  GetItem:=Item;
 end;

procedure TLinkPool.PutItem(var S: TStream; Item: Pointer);
 begin
  S.Write(PLinkPoolItem(Item)^, SizeOf(TLinkPoolItem));
 end;

procedure TLinkPool.FreeItem(Item: Pointer);
 begin
  Dispose(PLinkPoolItem(Item));
 end;

{ public methods }

function TLinkPool.QueryIndex(const Link: Pointer): System.Integer;
 var
  Index: xInteger;
  Item: PLinkPoolItem;
  Address: TAddress;
 begin
  if Link = nil then
   ClearAddress(Address)
  else
   jt.link.GetAddress(Link, Address);

  if Search(@Address, Index) then
   QueryIndex:=PLinkPoolItem(At(Index))^.Index
  else
   begin
    New(Item);

    Item^.Link:=Address;
    Item^.Index:=Count;

    AtInsert(Index, Item);

    QueryIndex:=Item^.Index;
   end;
 end;

function TLinkPool.QueryLink(const Index: System.Integer; var ALink: TAddress): Boolean;
 function Perform(const Item: PLinkPoolItem): Boolean; far;
  begin
   if Item^.Index = Index then
    begin
     ALink:=Item^.Link;

     Perform:=True;
    end
   else
    Perform:=False;
  end;
 begin
  QueryLink:=FirstThat(@Perform) <> nil;
 end;

{ TDupeBase }

constructor TDupeBase.Init(const AName: String; const AMaximum: Longint);
 var
  Stream: PRamStream;
  Countdown: Longint;
  Item: PDupeRecord;
 begin
  inherited Init($800, $800);

  Name:=NewStr(AName);

  Maximum:=AMaximum;

  Stream:=New(PRamStream, Init);

  if jt.rf.Get(DupeCheckerResourceFile, Name^, Stream, StreamLinker) then
   begin
    Stream^.Seek(0);

    Stream^.Read(Countdown, SizeOf(Countdown));

    if Countdown > Limit then
     SetLimit(Countdown or Limit);

    while Countdown > 0 do
     begin
      New(Item);

      Stream^.Read(Item^, SizeOf(TDupeRecord));

      AtInsert(Count, Item);

      Dec(Countdown);
     end;
   end
  else
   Changed:=True;

  Dispose(Stream, Done);
 end;

function TDupeBase.Compare(Key1, Key2: Pointer): xInteger;
 var
  K1: PDupeRecord absolute Key1;
  K2: PDupeRecord absolute Key2;
 begin
  if K1^.Time < K2^.Time then Compare:=1 else
  if K1^.Time > K2^.Time then Compare:=-1 else
   if K1^.Body < K2^.Body then Compare:=-1 else
   if K1^.Body > K2^.Body then Compare:=1 else
    if K1^.MSGID < K2^.MSGID then Compare:=-1 else
    if K1^.MSGID > K2^.MSGID then Compare:=1 else
     Compare:=0;
 end;

procedure TDupeBase.FreeItem(Item: Pointer);
 begin
  Dispose(PDupeRecord(Item));
 end;

destructor TDupeBase.Done;
 var
  Stream: PRamStream;
  Countdown: Longint;
 procedure Perform(const Item: PDupeRecord); far;
  begin
   Stream^.Write(Item^, SizeOf(TDupeRecord));
  end;
 begin
  if Changed then
   begin
    Stream:=New(PRamStream, Init);

    Stream^.Seek(SizeOf(Countdown) + Longint(Count) * Longint(SizeOf(TDupeRecord)));

    Stream^.Truncate;

    Stream^.Seek(0);

    Countdown:=Count;

    Stream^.Write(Countdown, SizeOf(Countdown));

    ForEach(@Perform);

    jt.rf.Add(DupeCheckerResourceFile, Name^, Stream, StreamLinker);

    Dispose(Stream, Done);
   end;

  DisposeStr(Name);

  inherited Done;
 end;

{ public methods }

procedure TDupeBase.Add(const AItem: TDupeRecord; const Index: xInteger);
 var
  Item: PDupeRecord;
 begin
  New(Item);

  Item^:=AItem;

  AtInsert(Index, Item);

  while Count >= Maximum do
   AtFree(Count - 1);

  Changed:=True;
 end;

{ MessageTossFilter }

procedure MessageTossFilter(var Block: TTossMessageBlock);
 var
  Source: Longint;
  Index: xInteger;
  Base: PDupeBase;
  Item: TDupeRecord;
  st, msg, msgst, msgp: Pointer;
  Link: TAddress;
  DT: TDateTime;
  S, Area: String;
  SourceStream, DestinationStream: PStream;
 begin
  st:=jt.msg.st(Block.msg);

  Source:=Longint(jt.st.Get(st, 'Source'));

  if (Source = tsBaseInbound) or LongBool(jt.st.Get(st, 'NoDupeCheck')) or LongBool(jt.st.Get(st, 'NoCallbacks')) then
   Exit;

  Base:=GetDupeBase(Block.Area);

  if Base <> nil then
   begin
    Gather(Block.Area, Block.Link, Block.msg, Item);

    if Base^.Search(@Item, Index) then
     begin
      if DupeCheckerArea <> nil then
       begin
        Item:=PDupeRecord(Base^.At(Index))^;

        msg:=jt.msg.Clone(Block.msg, mcfCloneHeader);

        jt.msg.SetArea(msg, jt.area.GetName(DupeCheckerArea)^);

        msgst:=jt.msg.st(msg);

        jt.st.Add(msgst, 'Source', Pointer(tsBaseInbound), nil);

        msgp:=jt.msg.CreateParser(msg, False);

        jt.msg.p.PutString(msgp, #1'AREA: ' + jt.area.GetName(Block.Area)^);
        jt.msg.p.PutString(msgp, #1'FROM: ' + AddressOfLink(Block.Link));

        UnixDateTimeToDateTime(Item.Time, DT);

        S:=lz(DT.Day) + ' ' + MonthNumberToMonthString(DT.Month) + ' ' + Long2Str(DT.Year) + ' ' +
           lz(DT.Hour) + ':' + lz(DT.Min) + ':' + lz(DT.Sec);

        if Item.MSGIDhex <> $FFFFFFFF then
         S:=S + ', msgid ' + HexL(Item.MSGIDhex);

        if AreaPool^.QueryArea(Item.Area, Area) then
         S:=S + ', ' + Area;

        if LinkPool^.QueryLink(Item.Link, Link) and not IsCleanAddress(Link) then
         S:=S + ', ' + AddressToStrEx(Link);

        jt.msg.p.PutString(msgp, #1'ORIGINAL: ' + S);

        jt.msg.DestroyParser(msgp);

        SourceStream:=New(PStreamThunk, Init(jt.msg.GetText(Block.msg), jt.StreamLinker, False));
        DestinationStream:=New(PStreamThunk, Init(jt.msg.GetText(msg), jt.StreamLinker, False));

        SourceStream^.Seek(0);

        DestinationStream^.CopyFrom(SourceStream^, SourceStream^.GetSize);

        Dispose(DestinationStream, Done);
        Dispose(SourceStream, Done);

        jt.tossscan.TossMessage(msg);

        if ccAvailable then
         DupeCheckerMovedHook^.Execute;
       end
      else
       if ccAvailable then
        DupeCheckerKilledHook^.Execute;

      Block.Area:=nil;
     end
    else
     Base^.Add(Item, Index);
   end;

  if DupeCheckerFlushInterval <> -1 then
   begin
    Inc(MessagesProcessed);

    if MessagesProcessed >= DupeCheckerFlushInterval then
     begin
      jt.SetStatus('flushing dupebases...');

      Flush;

      MessagesProcessed:=0;
     end;
   end;
 end;

{ GetDupeBase }

function GetDupeBase(const Area: Pointer): PDupeBase;
 var
  Base: PDupeBase;
  st, rst: Pointer;
 begin
  case DupeCheckerMode of
   dcmSeparated:
    begin
     st:=jt.area.st(Area);

     Base:=jt.st.Get(st, 'DupeChecker.DupeBase');

     if Base = nil then
      if not jt.rests.Check(DupeCheckerDupDisable, nil, jt.area.GetName(Area)) then
       begin
        jt.rests.Getst(DupeCheckerDupLimit, nil, jt.area.GetName(Area), rst);

        if rst = nil then
         Base:=New(PDupeBase, Init('DupeBase.' + jt.area.GetName(Area)^, DupeCheckerLimit))
        else
         Base:=New(PDupeBase, Init('DupeBase.' + jt.area.GetName(Area)^, Longint(jt.st.Get(rst, 'Limit'))));

        jt.st.Add(st, 'DupeChecker.DupeBase', Base, stDupeBase);
       end;

     GetDupeBase:=Base;
    end;
   dcmUnited:
    if jt.rests.Check(DupeCheckerDupDisable, nil, jt.area.GetName(Area)) then
     GetDupeBase:=nil
    else
     begin
      if UnitedDupeBase = nil then
       UnitedDupeBase:=New(PDupeBase, Init('UnitedDupeBase', DupeCheckerLimit));

      GetDupeBase:=UnitedDupeBase;
     end;
  end;
 end;

{ Wakeup }

procedure Wakeup;
 begin
  if AreaPool = nil then
   AreaPool:=New(PAreaPool, Init('DupeChecker.AreaPool'));

  if LinkPool = nil then
   LinkPool:=New(PLinkPool, Init('DupeChecker.LinkPool'));
 end;

{ Flush }

procedure Flush;
 var
  K: Longint;
 begin
  case DupeCheckerMode of
   dcmSeparated:
    for K:=0 to jt.area.Count - 1 do
     jt.st.Remove(jt.area.st(jt.area.At(K)), 'DupeChecker.DupeBase');
   dcmUnited:
    if UnitedDupeBase <> nil then
     begin
      Dispose(UnitedDupeBase, Done);

      UnitedDupeBase:=nil;
     end;
  end;

  if AreaPool <> nil then
   begin
    Dispose(AreaPool, Done);

    AreaPool:=nil;
   end;

  if LinkPool <> nil then
   begin
    Dispose(LinkPool, Done);

    LinkPool:=nil;
   end;
 end;

{ Gather }

procedure Gather(const Area, Link, msg: Pointer; var Item: TDupeRecord);
 var
  msgp: Pointer;
  Line: PChar;
  DT: TDateTime;
  S: String;
 begin
  Wakeup;

  jt.msg.GetWritten(msg, DT);

  DateTimeToUnixDateTime(DT, Item.Time);

  msgp:=jt.msg.CreateParser(msg, False);

  if not jt.msg.p.GetKludge(msgp, #1'MSGID: ', S) then
   begin
    Item.MSGID:=$FFFFFFFF;
    Item.MSGIDhex:=$FFFFFFFF;
   end
  else
   begin
    Item.MSGID:=StringCRC32(S, $FFFFFFFF);

    if not Str2Longint('$' + Copy(ExtractWord(3, S, [' ']), 1, 8), Item.MSGIDhex) then
     Item.MSGIDhex:=$FFFFFFFF;
   end;

  Item.Area:=AreaPool^.QueryIndex(Area);
  Item.Link:=LinkPool^.QueryIndex(Link);

  jt.msg.p.SetTextPos(msgp, jt.msg.p.AfterLastKludge(msgp));

  Item.Body:=$FFFFFFFF;

  while not jt.msg.p.EndOfMessage(msgp) do
   begin
    Line:=jt.msg.p.GetStringPCharEx(msgp);

    Item.Body:=PCharCRC32(Line, Item.Body);

    jt.StrDispose(Line);
   end;

  jt.msg.DestroyParser(msgp);
 end;

{ stDupeBase }

function stDupeBase(const Action: TStorageAction; const Data: Pointer): Pointer;
 var
  DupeBase: PDupeBase absolute Data;
 begin
  case Action of
   saDestroy: Dispose(DupeBase, Done);
   saClone: stDupeBase:=nil;
   saInfo: stDupeBase:=jt.NewStr('dupebase');
  end;
 end;

end.
