library DupeChk;

uses
     PlugAPI,
     PlugSIDE,

     Objects,

     TossAPI,
     ccAPI,

     Wizard,

     dchkCmn,
     dchkCore,
     dchkDump;

{ dupLimitValueParser }

procedure dupLimitValueParser(const Keyword, Linker, st: Pointer; Value: String); export;
 var
  Limit: Longint;
 begin
  Limit:=jt.linker.ParseNumber(Linker, Value);

  if Limit < 16 then
   jt.linker.Error(Linker, 'invalid dupLimit restriction value');

  jt.st.Add(st, 'Limit', Pointer(Limit), nil);
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, 'DupeChecker', 3);

  DupeCheckerPlugin:=Plugin;

  DupeCheckerDupLimit:=jt.rests.AddValueParser('dupLimit', rsMinus or rsValue or rsStorage, dupLimitValueParser);
  DupeCheckerDupDisable:=jt.rests.Add('dupDisable', rsMinus);

  PluginSetup:=True;

  jt.EnqueueCommand(DupeCheckerPlugin, DupeCheckerFlush, 0, cwFlush);

  InitCCAPI;

  if ccAvailable then
   begin
    DupeCheckerMovedHook:=New(PccHook, Init('DCAMOVED'));
    DupeCheckerKilledHook:=New(PccHook, Init('DCAKILLED'));
   end;

  jt.area.RegisterNotifyAreaDestroyed(Plugin);
 end;

{ PluginStartup }

procedure PluginStartup; export;
 begin
  if not DupeCheckerDisable then
   RegisterTosserCallback(DupeCheckerPlugin, taMessageTossFilter);
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
  if ccAvailable then
   begin
    Dispose(DupeCheckerMovedHook, Done);
    Dispose(DupeCheckerKilledHook, Done);
   end;

  DisposeTosserCallbacks;

  Flush;

  jt.rf.Destroy(DupeCheckerResourceFile);
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 var
  Linker: Pointer absolute Param;
  Area: Pointer absolute Param;
  Block: PTossMessageBlock absolute Param;
 begin
  if Command = tcMessageTossFilter then MessageTossFilter(Block^) else
  if Command = jtpcPluginSectionEnd then
   begin
    if DupeCheckerResourceFile = nil then
     jt.linker.Error(Linker, 'DUPES was not defined correctly');

    if DupeCheckerLimit < 16 then
     jt.linker.Error(Linker, 'DUPELIMIT was not defined correctly');

    if DupeCheckerMode = dcmUndefined then
     jt.linker.Error(Linker, 'DUPEMODE was not defined correctly');
   end else
  if Command = DupeCheckerFlush then Flush else
  if Command = DupeCheckerDumpDupeBase then DumpDupeBase(PString(Param)) else
  if Command = jtpcanDestroyed then
   begin
    if DupeCheckerMode = dcmSeparated then
     begin
      jt.st.Remove(jt.area.st(Area), 'DupeChecker.DupeBase');

      jt.rf.Remove(DupeCheckerResourceFile, 'DupeBase.' + jt.area.GetName(Area)^);
     end;
   end;
 end;

{ PluginHandleConfigSwitch }

procedure PluginHandleConfigSwitch(const Keyword: String; Value: String; const Linker: Pointer); export;
 var
  S: String;
 begin
  if Keyword = 'DUPES' then
   begin
    if DupeCheckerResourceFile <> nil then
     jt.linker.Error(Linker, 'redefinition of DUPES');

    DupeCheckerResourceFile:=jt.rf.Create(jt.linker.ParseStringToken(Linker, Value), rffNone);

    if DupeCheckerResourceFile = nil then
     jt.linker.Error(Linker, 'failed to create resourcefile object');
   end else
  if Keyword = 'DUPEMODE' then
   begin
    S:=jt.linker.ParseWord(Linker, Value);

    StUpcaseEx(S);

    if S = 'UNITED' then DupeCheckerMode:=dcmUnited else
    if S = 'SEPARATED' then DupeCheckerMode:=dcmSeparated else
     jt.linker.Error(Linker, 'UNITED or SEPARATED expected');
   end else
  if Keyword = 'DUPEAREA' then
   begin
    S:=jt.linker.ParseStringToken(Linker, Value);

    DupeCheckerArea:=jt.area.Seek(S);

    if DupeCheckerArea = nil then
     jt.linker.Error(Linker, 'unknown area "' + S + '"');

    jt.area.Used(DupeCheckerArea);
   end else
  if Keyword = 'DUPEFLUSH' then
   begin
    DupeCheckerFlushInterval:=jt.linker.ParseNumber(Linker, Value);

    if DupeCheckerFlushInterval < 50 then
     jt.linker.Error(Linker, '50 or greater expected');
   end else
  if Keyword = 'DUPELIMIT' then DupeCheckerLimit:=jt.linker.ParseNumber(Linker, Value) else
  if Keyword = 'DUPEDISABLE' then DupeCheckerDisable:=jt.linker.ParseBoolean(Linker, Value) else
   jt.linker.Error(Linker, 'syntax error');
 end;

{ PluginHandleCmdlineSwitch }

function PluginHandleCmdlineSwitch(Switch: String): Boolean; export;
 begin
  StUpcaseEx(Switch);

  PluginHandleCmdlineSwitch:=True;

  if Copy(Switch, 1, 13) = 'DUMPDUPEBASE:' then
   begin
    Delete(Switch, 1, 13);

    if Switch = '' then
     jt.linker.Error(jt.linker.Stub, 'invalid value of DUMPDUPEBASE');

    jt.EnqueueCommand(DupeCheckerPlugin, DupeCheckerDumpDupeBase, Longint(NewStr(Switch)), cwEarliest);
   end
  else
   PluginHandleCmdlineSwitch:=False;
 end;

{ PluginShowHelp }

procedure PluginShowHelp; export;
 begin
  jt.PrintStr(' DUMPDUPEBASE:AREA       dump dupebase of AREA'#13#10);
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleConfigSwitch,
 PluginHandleCmdlineSwitch,
 PluginShowHelp;

begin
 Assign(Output, '');
 Rewrite(Output);
end.
