library Echolist;

uses
     PlugAPI,
     PlugSIDE,

     Objects,
     Strings,

     Thunks,
     RStream,

     TossAPI,

     jtMHLcmn,

     Wizard;

type
 TImportMode = (imSoft, imHard);

 PArea = ^TArea;
 TArea = object(TObject)
  Name, Desc: PString;

  constructor Init(const AName, ADesc: String);
  constructor Load(var S: TStream);
  procedure Store(var S: TStream); virtual;
  destructor Done; virtual;
 end;

 PAreas = ^TAreas;
 TAreas = object(TSortedCollection)
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  function KeyOf(Item: Pointer): Pointer; virtual;
 end;

const
 RArea: TStreamRec = (
  ObjType: $1000;
  VmtLink: Ofs(TypeOf(TArea)^);
  Load: @TArea.Load;
  Store: @TArea.Store);

 RAreas: TStreamRec = (
  ObjType: $1001;
  VmtLink: Ofs(TypeOf(TAreas)^);
  Load: @TAreas.Load;
  Store: @TAreas.Store);

 Areas                  : PAreas                = nil;
 Echolists              : PStrings              = nil;

 EcholistPlugin         : Pointer               = nil;

 CompileForce           : Boolean               = False;

 ImportDescriptions                             = $1000;

 CompiledEcholistID                             = $FAEBFE00 or OSuid;

{ TArea }

constructor TArea.Init(const AName, ADesc: String);
 begin
  inherited Init;

  Name:=NewStr(AName);
  Desc:=NewStr(ADesc);
 end;

constructor TArea.Load(var S: TStream);
 begin
  Name:=S.ReadStr;
  Desc:=S.ReadStr;
 end;

procedure TArea.Store(var S: TStream);
 begin
  S.WriteStr(Name);
  S.WriteStr(Desc);
 end;

destructor TArea.Done;
 begin
  DisposeStr(Desc);
  DisposeStr(Name);

  inherited Done;
 end;

{ TAreas }

function TAreas.Compare(Key1, Key2: Pointer): xInteger;
 var
  K1: PString absolute Key1;
  K2: PString absolute Key2;
 begin
  if K1^ < K2^ then Compare:=-1 else
  if K1^ > K2^ then Compare:=1 else
   Compare:=0;
 end;

function TAreas.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=PArea(Item)^.Name;
 end;

{ Compile }

procedure Compile;
 var
  Stream: PRamStream;
  id, EcholistsCRC: Longint;
 procedure CalculateCRC(const Echolist: PString); far;
  begin
   EcholistsCRC:=StringCRC32(Echolist^, EcholistsCRC);

   EcholistsCRC:=StringCRC32(HexL(GetFileDate(Echolist^)), EcholistsCRC);
  end;
 procedure CompileEcholist(const Echolist: PString); far;
  var
   F: Text;
   Index: xInteger;
   Count: Longint;
   S, Name, Desc: String;
  begin
   if IOResult <> 0 then;

   Assign(F, Echolist^);
   Reset(F);

   if InOutRes = 0 then
    begin
     jt.SetStatus('compiling ' + Echolist^);

     Count:=0;

     while not Eof(F) do
      begin
       ReadLn(F, S);

       TrimEx(S);

       if (S <> '') and (S[1] <> ';') then
        begin
         if S[1] = ',' then
          begin
           Name:=ExtractWord(1, S, [',']);
           Desc:=ExtractWord(2, S, [',']);
          end
         else
          begin
           Name:=ExtractWord(2, S, [',']);
           Desc:=ExtractWord(3, S, [',']);
          end;

         TrimEx(Name);
         TrimEx(Desc);

         StUpcaseEx(Name);

         if (Name <> '') and (Desc <> '') then
          if not Areas^.Search(@Name, Index) then
           begin
            Areas^.AtInsert(Index, New(PArea, Init(Name, Desc)));

            Inc(Count);
           end;
        end;
      end;

     Close(F);

     if Count = 0 then
      jt.LogWrite('E', Echolist^ + ': no areas')
     else
      jt.LogWrite('E', Echolist^ + ': ' + Long2Str(Count) + ' area' + _s(Count));
    end
   else
    jt.LogWrite('?', 'Unable to open ' + Echolist^ + ', rc=' + Long2Str(IOResult));
  end;
 begin
  if Areas <> nil then
   Exit;

  jt.PushTitle;

  EcholistsCRC:=$FFFFFFFF;

  Echolists^.ForEach(@CalculateCRC);

  Stream:=New(PRamStream, Init);

  jt.SetStatus('loading compiled echolists');
  jt.SetTitle('loading compiled echolists');

  if not CompileForce and jt.rf.Get(jt.rf.common, 'CompiledEcholist', Stream, StreamLinker) then
   begin
    Stream^.Seek(0);

    Stream^.Read(id, SizeOf(id));

    if id = CompiledEcholistID then
     begin
      Stream^.Read(id, SizeOf(id));

      if id = EcholistsCRC then
       Areas:=PAreas(Stream^.Get);
     end;
   end;

  if Areas = nil then
   begin
    jt.SetStatus('compiling echolists');
    jt.SetTitle('compiling echolists');

    Stream^.Seek(0);

    Stream^.Truncate;

    id:=CompiledEcholistID;

    Stream^.Write(id, SizeOf(id));

    id:=EcholistsCRC;

    Stream^.Write(id, SizeOf(id));

    Areas:=New(PAreas, Init($100, $100));

    Echolists^.ForEach(@CompileEcholist);

    jt.LogWrite('E', 'Total: ' + Long2Str(Areas^.Count) + ' area' + _s(Areas^.Count) + ' with description' + _s(Areas^.Count));

    jt.SetStatus('storing echolists');
    jt.SetTitle('storing echolists');

    Stream^.Put(Areas);

    jt.rf.Add(jt.rf.common, 'CompiledEcholist', Stream, StreamLinker);
   end;

  Dispose(Stream, Done);

  jt.ClearStatus;

  jt.PopTitle;
 end;

{ ImportArea }

function ImportArea(const Area: Pointer; const Mode: TImportMode): Boolean;
 var
  AArea: PArea;
  Index: xInteger;
 begin
  ImportArea:=False;

  if (Area = nil) or ((jt.area.GetDesc(Area) <> nil) and (Mode = imSoft)) then
   Exit;

  Compile;

  if Areas^.Search(jt.area.GetName(Area), Index) then
   begin
    AArea:=Areas^.At(Index);

    if AArea^.Desc <> nil then
     begin
      jt.area.SetDesc(Area, AArea^.Desc^);

      jt.area.SetChanged(acfFull, 'descimport');

      jt.LogWrite('E', jt.area.GetName(Area)^ + ': "' + jt.area.GetDesc(Area)^ + '"');

      jt.area.NotifyAreaChanged(Area);

      ImportArea:=True;
     end;
   end;
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, 'Echolist', 1);

  EcholistPlugin:=Plugin;

  PluginSetup:=True;

  Echolists:=New(PStrings, Init($10, $10));

  RegisterObjects;
  RegisterType(RArea);
  RegisterType(RAreas);
 end;

{ PluginStartup }

procedure PluginStartup; export;
 begin
  if CompileForce then
   Compile;

  jt.area.RegisterNotifyAreaCreated(EcholistPlugin);

  RegisterTosserCallback(EcholistPlugin, taAreaCreatedFirst);
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
  DisposeTosserCallbacks;

  Dispose(Echolists, Done);

  if Areas <> nil then
   Dispose(Areas, Done);
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 var
  K, Imported: Longint;
  Area: Pointer;
 begin
  if Command = jtpcanCreated then ImportArea(Pointer(Param), imSoft) else
  if Command = tcAreaCreatedFirst then ImportArea(PTosserAutoCreateBlock(Param)^.Area, imSoft) else
  if Command = ImportDescriptions then
   begin
    jt.PushTitle;

    jt.SetTitle('importing descriptions');

    Imported:=0;

    for K:=0 to jt.area.Count - 1 do
     begin
      Area:=jt.area.At(K);

      jt.SetStatus(jt.area.GetName(Area)^);

      if ImportArea(Area, imHard) then
       Inc(Imported);
     end;

    if Imported <> 0 then
     jt.LogWrite('E', Long2Str(Imported) + ' description' + _s(Imported) + ' imported')
    else
     jt.LogWrite('E', 'No descriptions imported');

    jt.PopTitle;
   end;
 end;

{ PluginHandleConfigSwitch }

procedure PluginHandleConfigSwitch(const Keyword: String; Value: String; const Linker: Pointer); export;
 var
  Echolist: String;
 begin
  if Keyword = 'ECHOLIST' then
   begin
    Echolist:=jt.linker.ParseStringToken(Linker, Value);

    if Echolist = '' then
     jt.linker.Error(Linker, 'invalid filename');

    Echolists^.Insert(NewStr(Echolist));
   end
  else
   jt.linker.Error(Linker, 'syntax error');
 end;

{ PluginHandleCmdlineSwitch }

function PluginHandleCmdlineSwitch(Switch: String): Boolean; export;
 begin
  StUpcaseEx(Switch);

  PluginHandleCmdlineSwitch:=True;

  if Switch = 'FORCEECHOLISTCOMPILE' then CompileForce:=True else
  if (Switch = 'IMPORTDESCRIPTIONS') or (Switch = 'IMPORTDESCS') then
   jt.EnqueueCommand(EcholistPlugin, ImportDescriptions, 0, cwEarliest)
  else
   PluginHandleCmdlineSwitch:=False;
 end;

{ PluginShowHelp }

procedure PluginShowHelp; export;
 begin
  jt.PrintStr(' FORCEECHOLISTCOMPILE    force echolist(s) recompilation'#13#10);
  jt.PrintStr(' IMPORTDESCRIPTIONS      perform descriptions importing'#13#10);
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleConfigSwitch,
 PluginHandleCmdlineSwitch,
 PluginShowHelp;

begin
 Assign(Output, '');
 Rewrite(Output);
end.