library FTSbndl;

{.$DEFINE KEEPINCOMING}
{.$DEIFNE KEEPOUTGOING}

uses
     Objects,

     PlugAPI,
     PlugSIDE,

     Wizard;

const
 fmtName                              = 'FTSbundle';
 fmtVersion                           = 2;

 fmtPlugin                  : Pointer = nil;

 fmtFormatName            : String[6] = 'BUNDLE';
 fmtFormatNameStub        : String[4] = 'NONE';

 fmtLetterIn                          = '<';
 fmtLetterError                       = '!';

{ IsArcmail }

function IsArcmail(FName: String): Boolean;
 begin
  FName:=Copy(JustExtension(FName), 1, 2);

  StUpcaseEx(FName);

  IsArcmail:=(FName = 'SU') or
             (FName = 'MO') or
             (FName = 'TU') or
             (FName = 'WE') or
             (FName = 'TH') or
             (FName = 'FR') or
             (FName = 'SA');
 end;

{ PreprocessBundle }

function PreprocessBundle(const FName: String): Boolean; export;
 var
  Archiver: Pointer;
  Size, K, rc: Longint;
  ACmdline, Cmdline, LastDir: String;
  ChangeDir: Boolean;
  Reader: PFormatReader;
  F: Text;
 begin
  PreprocessBundle:=False;

  if not IsArcmail(FName) then
   Exit;

  Archiver:=jt.arc.Identify(FName);

  if Archiver = nil then
   Exit;

  Size:=GetFileSize(FName);

  if Size = -1 then
   Exit;

  ACmdline:=jt.arc.GetExtract(Archiver);

  if ACmdline = '' then
   Exit;

  jt.LogWrite(fmtLetterIn, JustFileName(FName) + '  TempInbound; ' +
                           jt.arc.GetName(Archiver) + ', ' +
                           Size2KB(Size));

  ChangeDir:=(Pos('%P', StUpcase(ACmdline)) = 0) and
             (Pos('%SP', StUpcase(ACmdline)) = 0);

  if ChangeDir then
   begin
    if IOResult <> 0 then;

    GetDir(0, LastDir);

    if IOResult <> 0 then;

    ChDir(RemoveBackSlash(jt.core.GetTempInbound^));

    if InOutRes <> 0 then
     begin
      jt.Report('Unable to change directory to ' + RemoveBackSlash(jt.core.GetTempInbound^), True);

      Exit;
     end;
   end;

  for K:=0 to jt.fmt.r.Count - 1 do
   begin
    Cmdline:=ACmdline;

    Reader:=jt.fmt.r.At(K);

    ReplaceEx(Cmdline, '%a', FName);
    ReplaceEx(Cmdline, '%f', '*.' + Reader^.Extension^);
    ReplaceEx(Cmdline, '%p', AddBackSlash(jt.core.GetTempInbound^));
    ReplaceEx(Cmdline, '%sa', jt.MakeShortName(FName));
    ReplaceEx(Cmdline, '%sp', AddBackSlash(jt.MakeShortName(RemoveBackSlash(jt.core.GetTempInbound^))));

    jt.Execute(Cmdline, rc, jt.arc.Quiet);

    if rc <> 0 then
     begin
      jt.Report('Failed to "' + Cmdline + '", rc=' + Long2Str(rc), True);

      if ChangeDir then
       begin
        if IOResult <> 0 then;

        ChDir(LastDir);

        if IOResult <> 0 then;
       end;

      Exit;
     end;
   end;

  if ChangeDir then
   begin
    if IOResult <> 0 then;

    ChDir(LastDir);

    if IOResult <> 0 then;
   end;

  {$IFNDEF KEEPINCOMING}
  if IOResult <> 0 then;

  Assign(F, FName);

  Erase(F);

  if InOutRes <> 0 then
   jt.Report('Unable to erase ' + FName + ', rc=' + Long2Str(IOResult), True);
  {$ENDIF}

  PreprocessBundle:=True;
 end;

{ PreprocessIsOur }

function PreprocessIsOur(const FName: String): Boolean; export;
 begin
  PreprocessIsOur:=IsArcmail(FName);
 end;

{ OutprocessBundle }

function OutprocessBundle(const Source, Destination: String; const Archiver: Pointer): Boolean; export;
 var
  Cmdline: String;
  rc: Longint;
  F: Text;
 begin
  OutprocessBundle:=False;

  if not jt.CreateDirectory(JustPathName(Destination)) then
   begin
    jt.Report('Failed to create directory ' + JustPathName(Destination), True);

    Exit;
   end;

  Cmdline:=jt.arc.GetAdd(Archiver);

  if Cmdline = '' then
   begin
    jt.Report('No ADD command defined for ' + jt.arc.GetName(Archiver), True);

    Exit;
   end;

  ReplaceEx(Cmdline, '%f', Source);
  ReplaceEx(Cmdline, '%a', Destination);

  if Pos('%SA', StUpcase(Cmdline)) > 0 then
   if ExistFile(Destination) then
    ReplaceEx(Cmdline, '%sa', jt.MakeShortName(Destination))
   else
    begin
     if IOResult <> 0 then;

     Assign(F, Destination);

     Rewrite(F);

     Close(F);

     if IOResult <> 0 then;

     ReplaceEx(Cmdline, '%sa', jt.MakeShortName(Destination));

     Erase(F);

     if IOResult <> 0 then;
    end;

  jt.Execute(Cmdline, rc, jt.arc.Quiet);

  if rc <> 0 then
   jt.Report('Failed to "' + Cmdline + '", rc=' + Long2Str(rc), True)
  else
   begin
    {$IFNDEF KEEPOUTGOING}
    if ExistFile(Source) then
     begin
      if IOResult <> 0 then;

      Assign(F, Source);

      Erase(F);

      if InOutRes <> 0 then
       jt.Report('Failed to erase ' + Source + ', rc='+ Long2Str(IOResult), True);
     end;
    {$ENDIF}

    OutprocessBundle:=True;
   end;
 end;

{ FreeOutprocessor }

procedure FreeOutprocessor(const Outprocessor: PFormatOutprocessor); export;
 begin
  jt.LogWrite('~', HexPtr(Outprocessor^.st));

  jt.st.Destroy(Outprocessor^.st);
 end;

{ OutprocessBundleStub }

function OutprocessBundleStub(const Source, Destination: String; const Archiver: Pointer): Boolean; export;
 begin
  OutprocessBundleStub:=False;
 end;

{ pre/outprocessor wrappers }

const
 Preprocessor: PlugSIDE.TFormatPreprocessor = (
  Name: @fmtFormatName;
  Plugin: nil;
  st: nil;

  Preprocess: PreprocessBundle;
  IsOur: PreprocessIsOur);

 Outprocessor: PlugSIDE.TFormatOutprocessor = (
  Name: @fmtFormatName;
  Plugin: nil;
  st: nil;

  Outprocess: OutprocessBundle);

 OutprocessorStub: PlugSIDE.TFormatOutprocessor = (
  Name: @fmtFormatNameStub;
  Plugin: nil;
  st: nil;

  Outprocess: OutprocessBundleStub);

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, fmtName, fmtVersion);

  fmtPlugin:=Plugin;

  Preprocessor.Plugin:=Plugin;
  Outprocessor.Plugin:=Plugin;
  OutprocessorStub.Plugin:=Plugin;

  jt.fmt.pp.Register(@Preprocessor);
  jt.fmt.op.Register(@Outprocessor);
  jt.fmt.op.Register(@OutprocessorStub);

  PluginSetup:=True;
 end;

exports
 PluginSetup;

begin
end.
