library FTSpktV2;

{.$DEFINE DEBUGWRITER}

uses
     Objects,
     Strings,

     PlugAPI,
     PlugSIDE,

     jtMHLcmn,

     Wizard;

const
 fmtName                        = 'FTSpktV2';
 fmtVersion                     = 2;

 fmtPlugin      : Pointer       = nil;

 fmtExtension   : String[3]     = 'PKT';
 fmtFormatName  : String[5]     = 'PKTV2';

 KillAttributesOnRead           = maLocal or maSent or maScanned or maRead;
 KillAttributesOnWrite          = maLocal or maSent or maScanned or maRead;

type
 PMessageV2Header = ^TMessageV2Header;
 TMessageV2Header = packed record
  OrigNode     : System.Word;
  DestNode     : System.Word;
  OrigNet      : System.Word;
  DestNet      : System.Word;
  Attributes   : System.Word;
  Cost         : System.Word;
 end;

 PPacketV2Header = ^TPacketV2Header;
 TPacketV2Header = packed record
  OrigNode     : System.Word;
  DestNode     : System.Word;
  Year         : System.Word;
  Month        : System.Word;
  Day          : System.Word;
  Hour         : System.Word;
  Minute       : System.Word;
  Second       : System.Word;
  Rate         : System.Word;
  Version      : System.Word;
  OrigNet      : System.Word;
  DestNet      : System.Word;
  PCodeLo      : System.Byte;
  PRevMajor    : System.Byte;
  Password     : array[1..8] of System.Char;
  QMOrigZone   : System.Word;
  QMDestZone   : System.Word;
  AuxNet       : System.Word;
  CWValidate   : System.Word;
  PCodeHi      : System.Byte;
  PRevMinor    : System.Byte;
  CWCapWord    : System.Word;
  OrigZone     : System.Word;
  DestZone     : System.Word;
  OrigPoint    : System.Word;
  DestPoint    : System.Word;
  Flags        : System.Longint;
 end;

 PFormatReader = ^TFormatReader;
 TFormatReader = object(TObject)
  Stream: PStream;
  Header: TPacketV2Header;
  FName: PString;

  constructor Open(const AFName: String; var rc: String; var FromA, ToA: TAddress; var Password: String; var Flags: Longint);
  function Read(var msg: Pointer; var rc: String): Boolean;
  function GetSize: Longint;
  function GetPos: Longint;
  destructor Done; virtual;
 end;

 PFormatWriter = ^TFormatWriter;
 TFormatWriter = object(TObject)
  Stream: PStream;
  Header: TPacketV2Header;
  FName: PString;

  constructor Create(const AFName: String; var rc: String; const FromA, ToA: TAddress; const Password: String;
   const Flags: Longint);
  constructor Append(const AFName: String; var rc: String);
  function Write(const msg: Pointer; var rc: String): Boolean;
  function GetSize: Longint;
  destructor Done; virtual;
 end;

const
 faPrivate           = $0001;
 faCrash             = $0002;
 faReceived          = $0004;
 faSent              = $0008;
 faAttach            = $0010;
 faTransit           = $0020;
 faOrphan            = $0040;
 faKill              = $0080;
 faLocal             = $0100;
 faHold              = $0200;
 faFRq               = $0800;
 faRRq               = $1000;
 faRRc               = $2000;
 faARq               = $4000;
 faURq               = $8000;

 AttributesMap: array[1..15, 1..2] of Longint = (
                       (faPrivate,  maPrivate),
                       (faCrash,    maCrash),
                       (faReceived, maReceived),
                       (faSent,     maSent),
                       (faAttach,   maAttach),
                       (faTransit,  maTransit),
                       (faOrphan,   maOrphan),
                       (faKill,     maKill),
                       (faLocal,    maLocal),
                       (faHold,     maHold),
                       (faFRq,      maFRq),
                       (faRRq,      maRRq),
                       (faRRc,      maRRc),
                       (faARq,      maARq),
                       (faURq,      maURq));

 FlushAfterWrite: Boolean = False;

{ TFormatReader }

constructor TFormatReader.Open(const AFName: String; var rc: String; var FromA, ToA: TAddress; var Password: String;
  var Flags: Longint);
 var
  K: Integer;
  DT: TDateTime;
 begin
  inherited Init;

  Stream:=New(PBufStream, Init(AFName, stOpenRead, 32768));

  if Stream^.Status <> stOk then
   begin
    rc:='failed to open ' + AFName + ', rc=' + Long2Str(Stream^.Status);

    Dispose(Stream, Done);

    inherited Done;

    Fail;
   end;

  Stream^.Read(Header, SizeOf(Header));

  if Stream^.Status <> stOk then
   begin
    rc:='failed to read header of ' + AFName + ', rc=' + Long2Str(Stream^.Status);

    Dispose(Stream, Done);

    inherited Done;

    Fail;
   end;

  if Header.Version <> $02 then
   begin
    rc:='invalid version of ' + AFName + ', rc=' + Long2Str(Stream^.Status);

    Dispose(Stream, Done);

    inherited Done;

    Fail;
   end;

  FromA.Zone:=Header.OrigZone;
  FromA.Net:=Header.OrigNet;
  FromA.Node:=Header.OrigNode;
  FromA.Point:=Header.OrigPoint;

  if FromA.Zone = 0 then
   FromA.Zone:=Header.QMOrigZone;

  if FromA.Net = -1 then
   FromA.Net:=Header.AuxNet;

  ToA.Zone:=Header.DestZone;
  ToA.Net:=Header.DestNet;
  ToA.Node:=Header.DestNode;
  ToA.Point:=Header.DestPoint;

  if ToA.Zone = 0 then
   ToA.Zone:=Header.QMDestZone;

  SetLength(Password, 8);

  Move(Header.Password, Password[1], 8);

  K:=Pos(#0, Password);

  if K <> 0 then
   SetLength(Password, K - 1);

  Flags:=Header.Flags;

  FName:=NewStr(AFName);
 end;

function TFormatReader.Read(var msg: Pointer; var rc: String): Boolean;
 function GetString(var S: String; var rc: String): Boolean;
  var
   Size: Longint;
   Finish: PChar;
  const
   BufferSize = 254;
  begin
   GetString:=False;

   SetLength(S, BufferSize + 1);

   Size:=Stream^.GetSize - Stream^.GetPos;

   if Size > BufferSize then
    Size:=BufferSize;

   Stream^.Read(S[1], Size);

   S[BufferSize + 1]:=#0;

   Finish:=StrEnd(@S[1]);

   if Finish = @S[BufferSize + 1] then
    rc:='unable to reach end of ASCIIZ string near ' + HexL(Stream^.GetPos) + ' in ' + FName^
   else
    begin
     SetLength(S, Finish - @S[1]);

     Stream^.Seek(Stream^.GetPos - Size + Length(S) + 1);

     if Stream^.Status <> stOk then
      rc:='ASCIIZ string read error in ' + FName^ + ', rc=' + Long2Str(Stream^.Status)
     else
      GetString:=True;
    end;
  end;
 function GetStringCR(var S: String; var rc: String): Boolean;
  var
   Size: Longint;
   Finish: PChar;
  const
   BufferSize = 254;
  begin
   GetStringCR:=False;

   SetLength(S, BufferSize + 1);

   Size:=Stream^.GetSize - Stream^.GetPos;

   if Size > BufferSize then
    Size:=BufferSize;

   Stream^.Read(S[1], Size);

   S[BufferSize + 1]:=#13;

   Finish:=StrScan(@S[1], #13);

   if Finish = @S[BufferSize + 1] then
    rc:='unable to reach end of CR string near ' + HexL(Stream^.GetPos) + ' in ' + FName^
   else
    begin
     SetLength(S, Finish - @S[1]);

     Stream^.Seek(Stream^.GetPos - Size + Length(S) + 1);

     if Stream^.Status <> stOk then
      rc:='CR string read error in ' + FName^ + ', rc=' + Long2Str(Stream^.Status)
     else
      GetStringCR:=True;
    end;
  end;
 function SearchZero(var Mast: Longint; var rc: String): Boolean;
  var
   Buffer, Finish: PChar;
   Size: Longint;
  const
   BufferSize = $400;
  begin
   GetMem(Buffer, BufferSize + 1);

   repeat
    Size:=Stream^.GetSize - Stream^.GetPos;

    if Size > BufferSize then
     Size:=BufferSize;

    if Size <= 0 then
     begin
      SearchZero:=False;

      rc:='unable to locate end of body near ' + HexL(Stream^.GetPos) + ' of ' + FName^;

      Break;
     end;

    Buffer[Size]:=#0;

    Stream^.Read(Buffer^, Size);

    Finish:=StrEnd(Buffer);

    if Finish <> @Buffer[Size] then
     begin
      SearchZero:=True;

      Mast:=Stream^.GetPos - Size + (Finish - Buffer) + 1;

      Break;
     end;
   until False;

   FreeMem(Buffer, BufferSize + 1);
  end;
 const
  BufferSize = $4000;
 var
  S, ToName, FromName, Subject, Area: String;
  Anchor, Mast, Attributes, K, Countdown, BlockSize: Longint;
  mHeader: TMessageV2Header;
  id: System.Word;
  msgp, Buffer: Pointer;
  DT: TDateTime;
 begin
  if Stream^.Status <> stOk then
   Stream^.Reset;

  Stream^.Read(id, SizeOf(id));

  if id = $00 then
   begin
    Read:=True;

    msg:=nil;

    Exit;
   end;

  Read:=False;

  if id <> $02 then
   begin
    rc:='invalid message ID of ' + FName^ + ' at ' + HexL(Stream^.GetPos - 2) + ': ' + HexW(id);

    Exit;
   end;

  Stream^.Read(mHeader, SizeOf(mHeader));

  if Stream^.Status <> stOk then
   begin
    rc:='unable to load another mheader from ' + FName^ + ', rc=' + Long2Str(Stream^.Status);

    Exit;
   end;

  if not GetString(S, rc) then
   Exit;

  if Length(S) <> 19 then
   begin
    rc:='invalid date-time stamp of ' + FName^ + ' near ' + HexL(Stream^.GetPos) + ', possible Y2k incompatibility?';

    Exit;
   end;

  Str2Word(ExtractWord(1, S, [' ']), DT.Day);

  MonthStringToMonthNumber(ExtractWord(2, S, [' ']), DT.Month);

  Str2Word(ExtractWord(3, S, [' ']), DT.Year);

  Delete(S, 1, 11);

  Str2Word(ExtractWord(1, S, [':']), DT.Hour);
  Str2Word(ExtractWord(2, S, [':']), DT.Min);
  Str2Word(ExtractWord(3, S, [':', ' ']), DT.Sec);

  DT.Year:=DoubleToY2(DT.Year);

  if not GetString(ToName, rc) then
   Exit;

  if not GetString(FromName, rc) then
   Exit;

  if not GetString(Subject, rc) then
   Exit;

  Anchor:=Stream^.GetPos;

  if not GetStringCR(Area, rc) then
   Exit;

  if Copy(Area, 1, 5) = 'AREA:' then
   begin
    Delete(Area, 1, 5);

    TrimEx(Area);

    StUpcaseEx(Area);

    Anchor:=Stream^.GetPos;
   end
  else
   Area:='';

  if not SearchZero(Mast, rc) then
   Exit;

  if MemAvail < (Mast - Anchor) * 2 then
   begin
    rc:='too low memory for read msg at ' + HexL(Anchor) + ' from ' + FName^ + ' (len=' + HexL(Mast - Anchor) + ')';

    Exit;
   end;

  Attributes:=maNone;

  for K:=Low(AttributesMap) to High(AttributesMap) do
   if mHeader.Attributes and AttributesMap[K, 1] <> 0 then
    Attributes:=Attributes or AttributesMap[K, 2];

  msg:=jt.msg.Create;

  jt.msg.SetWritten(msg, DT);
  jt.msg.SetFromName(msg, FromName);
  jt.msg.SetToName(msg, ToName);
  jt.msg.SetSubject(msg, Subject);
  jt.msg.SetArea(msg, Area);
  jt.msg.SetAttributes(msg, Attributes and not KillAttributesOnRead);

  Stream^.Seek(Anchor);

  msgp:=jt.msg.CreateParser(msg, False);

  jt.msg.p.SetTextPos(msgp, 0);

  GetMem(Buffer, BufferSize);

  Countdown:=Mast - Anchor - 1;

  while (Countdown > 0) and (Stream^.Status = stOk) do
   begin
    if Countdown > BufferSize then
     BlockSize:=BufferSize
    else
     BlockSize:=Countdown;

    Stream^.Read(Buffer^, BlockSize);

    jt.msg.p.WriteText(msgp, Buffer^, BlockSize);

    Dec(Countdown, BlockSize);
   end;

  FreeMem(Buffer, BufferSize);

  jt.msg.DestroyParser(msgp);

  Stream^.Seek(Mast);

  if Stream^.Status <> stOk then
   begin
    rc:='failed to load msg at ' + HexL(Anchor) + ' of ' + FName^ + ', rc=' + Long2Str(Stream^.Status);

    jt.msg.Destroy(msg);

    Exit;
   end;

  Read:=True;

  jt.st.Add(jt.msg.st(msg), 'Packet.FixedSize', Pointer(2 + SizeOf(TMessageV2Header) + 19), nil);
 end;

function TFormatReader.GetSize: Longint;
 begin
  GetSize:=Stream^.GetSize;
 end;

function TFormatReader.GetPos: Longint;
 begin
  GetPos:=Stream^.GetPos;
 end;

destructor TFormatReader.Done;
 begin
  Dispose(Stream, Done);
  DisposeStr(FName);

  inherited Done;
 end;

{ TFormatWriter }

constructor TFormatWriter.Create(const AFName: String; var rc: String; const FromA, ToA: TAddress; const Password: String;
  const Flags: Longint);
 var
  DT: TDateTime;
  Pwd: String[8];
 begin
  inherited Init;

  GetCurrentDateTime(DT);

  Pwd:=Copy(Password, 1, 8);

  while Length(Pwd) < 8 do
   Pwd:=Concat(Pwd, #0);

  Header.OrigNode:=FromA.Node;
  Header.DestNode:=ToA.Node;

  Header.Day:=DT.Day;
  Header.Month:=Pred(DT.Month);
  Header.Year:=DT.Year;
  Header.Hour:=DT.Hour;
  Header.Minute:=DT.Min;
  Header.Second:=DT.Sec;

  Header.Rate:=0;
  Header.Version:=2;

  Header.OrigNet:=FromA.Net;
  Header.DestNet:=ToA.Net;

  Header.PCodeLo:=0;
  Header.PRevMajor:=0;

  Move(Pwd[1], Header.Password, 8);

  Header.QMOrigZone:=FromA.Zone;
  Header.QMDestZone:=ToA.Zone;

  Header.AuxNet:=0;
  Header.CWValidate:=$100;
  Header.PCodeHi:=0;
  Header.PRevMinor:=0;
  Header.CWCapWord:=1;

  Header.OrigZone:=FromA.Zone;
  Header.DestZone:=ToA.Zone;
  Header.OrigPoint:=FromA.Point;
  Header.DestPoint:=ToA.Point;

  Header.Flags:=Flags;

  Stream:=New(PBufStream, Init(AFName, stCreate, 32768));

  if Stream^.Status <> stOk then
   begin
    rc:='unable to create ' + AFName + ', rc=' + Long2Str(Stream^.Status);

    Dispose(Stream, Done);

    inherited Done;

    Fail;
   end;

  Stream^.Write(Header, SizeOf(Header));

  if Stream^.Status <> stOk then
   begin
    rc:='unable to write header of ' + AFName + ', rc=' + Long2Str(Stream^.Status);

    Dispose(Stream, Done);

    inherited Done;

    Fail;
   end;

  FName:=NewStr(AFName);

  {$IFDEF DEBUGWRITER}
  jt.LogWrite('~', 'PKTV2: created ' + FName^ + ' ' + AddressToStrEx(FromA) + '  ' + AddressToStrEx(ToA) +
   ' (' + Password + ')');
  {$ENDIF}
 end;

constructor TFormatWriter.Append(const AFName: String; var rc: String);
 var
  id: System.Word;
 begin
  inherited Init;

  Stream:=New(PBufStream, Init(AFName, stOpen, 32768));

  if Stream^.Status <> stOk then
   begin
    rc:='failed to append ' + AFName + ', rc=' + Long2Str(Stream^.Status);

    Dispose(Stream, Done);

    inherited Done;

    Fail;
   end;

  Stream^.Read(Header, SizeOf(Header));

  if Stream^.Status <> stOk then
   begin
    rc:='failed to read header of ' + AFName + ', rc=' + Long2Str(Stream^.Status);

    Dispose(Stream, Done);

    inherited Done;

    Fail;
   end;

  if Header.Version <> 2 then
   begin
    rc:='grunged header of ' + AFName + ', rc=' + Long2Str(Stream^.Status);

    Dispose(Stream, Done);

    inherited Done;

    Fail;
   end;

  Stream^.Seek(Stream^.GetSize - 2);

  Stream^.Read(id, SizeOf(id));

  if id <> $00 then
   Stream^.Seek(Stream^.GetSize)
  else
   Stream^.Seek(Stream^.GetSize - 2);

  FName:=NewStr(AFName);

  {$IFDEF DEBUGWRITER}
  jt.LogWrite('~', 'PKTV2: appended ' + FName^);
  {$ENDIF}
 end;

function TFormatWriter.Write(const msg: Pointer; var rc: String): Boolean;
 const
  NULL: Byte = $00;
  AREA: String[5] = 'AREA:';
  CR: Byte = $0D;
 function PutString(const S: PString; var rc: String): Boolean;
  begin
   if S <> nil then
    Stream^.Write(S^[1], Length(S^));

   Stream^.Write(NULL, SizeOf(NULL));

   if Stream^.Status <> stOk then
    begin
     rc:='unable to write msg (PutString) ' + FName^ + ', rc=' + Long2Str(Stream^.Status);

     PutString:=False;
    end
   else
    PutString:=True;
  end;
 function PutStringCR(const S: PString; var rc: String): Boolean;
  begin
   if (S <> nil) and (S^ <> '') then
    begin
     Stream^.Write(Area[1], Length(Area));

     Stream^.Write(S^[1], Length(S^));

     Stream^.Write(CR, SizeOf(CR));

     if Stream^.Status <> stOk then
      begin
       rc:='unable to write msg (PutStringCR) ' + FName^ + ', rc=' + Long2Str(Stream^.Status);

       PutStringCR:=False;
      end
     else
      PutStringCR:=True;
    end
   else
    PutStringCR:=True;
  end;
 const
  BufferSize = $4000;
 var
  Attributes, Countdown, BlockSize: Longint;
  msgp, Buffer: Pointer;
  mHeader: TMessageV2Header;
  id, K: System.Word;
  DT: TDateTime;
  S: String;
 begin
  {$IFDEF DEBUGWRITER}
  jt.LogWrite('~', 'PKTV2: written ' + HexPtr(msg) + ' to ' + FName^);
  {$ENDIF}

  if Stream^.Status <> stOk then
   Stream^.Reset;

  Write:=False;

  id:=$02;

  Stream^.Write(id, SizeOf(id));

  mHeader.OrigNode:=Header.OrigNode;
  mHeader.DestNode:=Header.DestNode;
  mHeader.OrigNet:=Header.OrigNet;
  mHeader.DestNet:=Header.DestNet;

  mHeader.Attributes:=0;
  mHeader.Cost:=0;

  Attributes:=jt.msg.GetAttributes(msg) and not KillAttributesOnWrite;

  for K:=Low(AttributesMap) to High(AttributesMap) do
   if Attributes and AttributesMap[K, 2] <> 0 then
    mHeader.Attributes:=mHeader.Attributes or AttributesMap[K, 1];

  Stream^.Write(mHeader, SizeOf(mHeader));

  if Stream^.Status <> stOk then
   begin
    rc:='failed to write msg to ' + FName^ + ', rc=' + Long2Str(Stream^.Status);

    Exit;
   end;

  jt.msg.GetWritten(msg, DT);

  with DT do
   S:=lz(Day) + ' ' +
      MonthNumberToMonthString(Month) + ' ' +
      lz(Y2ToDouble(Year)) + '  ' +
      lz(Hour) + ':' +
      lz(Min) + ':' +
      lz(Sec);

  if not PutString(@S, rc) then Exit;
  if not PutString(jt.msg.GetToName(msg), rc) then Exit;
  if not PutString(jt.msg.GetFromName(msg), rc) then Exit;

  if not PutString(jt.msg.GetSubject(msg), rc) then Exit;
  if not PutStringCR(jt.msg.GetArea(msg), rc) then Exit;

  msgp:=jt.msg.CreateParser(msg, False);

  jt.msg.p.SetTextPos(msgp, 0);

  Countdown:=jt.msg.p.GetTextSize(msgp);

  GetMem(Buffer, BufferSize);

  while (Countdown > 0) and (Stream^.Status = stOk) do
   begin
    if Countdown > BufferSize then
     BlockSize:=BufferSize
    else
     BlockSize:=Countdown;

    jt.msg.p.ReadText(msgp, Buffer^, BlockSize);

    Stream^.Write(Buffer^, BlockSize);

    Dec(Countdown, BlockSize);
   end;

  FreeMem(Buffer, BufferSize);

  jt.msg.DestroyParser(msgp);

  Stream^.Write(NULL, SizeOf(NULL));

  if Stream^.Status <> stOk then
   begin
    rc:='failed to put msg to ' + FName^ + ', rc=' + Long2Str(Stream^.Status);

    Exit;
   end;

  if FlushAfterWrite then
   Stream^.Flush;

  Write:=True;
 end;

function TFormatWriter.GetSize: Longint;
 begin
  GetSize:=Stream^.GetSize;
 end;

destructor TFormatWriter.Done;
 const
  NULLTWICE: System.Word = $0000;
 begin
  {$IFDEF DEBUGWRITER}
  jt.LogWrite('~', 'PKTV2: closed ' + FName^);
  {$ENDIF}

  Stream^.Write(NULLTWICE, SizeOf(NULLTWICE));

  Dispose(Stream, Done);
  DisposeStr(FName);

  inherited Done;
 end;

{ reader wrappers }

function readerOpen(const FName: String; var rc: String; var FromA, ToA: TAddress; var Password: String;
  var Flags: Longint): Pointer; export;
 begin
  readerOpen:=New(PFormatReader, Open(FName, rc, FromA, ToA, Password, Flags));
 end;

function readerRead(const reader: Pointer; var msg: Pointer; var rc: String): Boolean; export;
 begin
  readerRead:=PFormatReader(reader)^.Read(msg, rc);
 end;

function readerGetSize(const reader: Pointer): Longint; export;
 begin
  readerGetSize:=PFormatReader(reader)^.GetSize;
 end;

function readerGetPos(const reader: Pointer): Longint; export;
 begin
  readerGetPos:=PFormatReader(reader)^.GetPos;
 end;

procedure readerClose(const reader: Pointer); export;
 begin
  Dispose(PFormatReader(reader), Done);
 end;

function readerIdent(const FName: String): Boolean; export;
 var
  Stream: PDosStream;
  Header: TPacketV2Header;
 begin
  Stream:=New(PDosStream, Init(FName, stOpenRead));

  if Stream^.Status = stOk then
   begin
    Stream^.Read(Header, SizeOf(Header));

    readerIdent:=(Stream^.Status = stOk) and (Header.Version = $02);
   end
  else
   readerIdent:=False;

  Dispose(Stream, Done);
 end;

function readerIsOur(const FName: String): Boolean; export;
 begin
  readerIsOur:=StUpcase(JustExtension(FName)) = fmtExtension;
 end;

{ writer wrappers }

function writerCreate(const AFName: String; var rc: String; const FromA, ToA: TAddress; const Password: String;
  const Flags: Longint): Pointer; export;
 begin
  writerCreate:=New(PFormatWriter, Create(AFName, rc, FromA, ToA, Password, Flags));
 end;

function writerAppend(const AFName: String; var rc: String): Pointer; export;
 begin
  writerAppend:=New(PFormatWriter, Append(AFName, rc));
 end;

function writerWrite(const writer: Pointer; const msg: Pointer; var rc: String): Boolean; export;
 begin
  writerWrite:=PFormatWriter(writer)^.Write(msg, rc);
 end;

function writerGetSize(const writer: Pointer): Longint; export;
 begin
  writerGetSize:=PFormatWriter(writer)^.GetSize;
 end;

procedure writerClose(const writer: Pointer); export;
 begin
  Dispose(PFormatWriter(writer), Done);
 end;

{ API structures }

const
 Reader: PlugSIDE.TFormatReader = (
  Extension: @fmtExtension;
  Name: @fmtFormatName;
  Plugin: nil;
  st: nil;
  AssociatedWriter: nil;

  Open: readerOpen;
  Read: readerRead;
  GetSize: readerGetSize;
  GetPos: readerGetPos;
  Close: readerClose;
  Ident: readerIdent;
  IsOur: readerIsOur);

 Writer: PlugSIDE.TFormatWriter = (
  Extension: @fmtExtension;
  Name: @fmtFormatName;
  Plugin: nil;
  st: nil;
  AssociatedReader: nil;

  Create: writerCreate;
  Append: writerAppend;
  Write: writerWrite;
  GetSize: writerGetSize;
  Close: writerClose);

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, fmtName, fmtVersion);

  fmtPlugin:=Plugin;

  Reader.Plugin:=Plugin;
  Writer.Plugin:=Plugin;

  Reader.st:=jt.st.Create;
  Writer.st:=jt.st.Create;

  Reader.AssociatedWriter:=@Writer;
  Writer.AssociatedReader:=@Reader;

  jt.fmt.r.Register(@Reader);
  jt.fmt.w.Register(@Writer);

  PluginSetup:=True;
 end;

{ PluginHandleConfigSwitch }

procedure PluginHandleConfigSwitch(const Keyword: String; Value: String; const Linker: Pointer); export;
 begin
  if Keyword = 'FLUSHAFTERWRITE' then FlushAfterWrite:=jt.linker.ParseBoolean(Linker, Value) else
   jt.linker.Error(Linker, 'syntax error');
 end;

exports
 PluginSetup,
 PluginHandleConfigSwitch;

begin
end.