unit frAfx;

interface
uses
     PlugSIDE,
     PlugAPI,

     Objects,

     Columns,

     afxAPI,

     frCmn,
     frFwdReq,
     frLists,
     frQueue,

     jtMHLcmn,

     Wizard;

procedure afxAvailHandler(const Command: String; Value: String); export;
procedure afxFAvailHandler(const Command: String; Value: String); export;
procedure afxAQueryHandler(const Command: String; Value: String); export;

implementation

const
 colListAreaName            = 41;
 colListAreaNameWrapped     = 3;
 colListDescription         = 37; { %AVAIL, %FAVAIL }
 colListRequestedBy         = 18; { %AQUERY }
 colListRequestedFrom       = 18; { %AQUERY }

 colMaxWidth                = 79;

 FlushLimit                 = 128;

type
 PAvailGroups = ^TAvailGroups;
 PAvailGroup = ^TAvailGroup;

 PFormatter = ^TFormatter;
 TFormatter = object(TObject)
  Target: PStrings;
  Groups: PAvailGroups;

  constructor Init(const ATarget: PStrings; const AGroups: PAvailGroups);
  procedure BeforeAll; virtual;
  procedure BeforeGroup(const Group: PAvailGroup); virtual;
  procedure DoItem(const Item: PForwardRequestListItem); virtual;
  procedure AfterGroup; virtual;
  procedure AfterAll; virtual;
 end;

 PAvailFormatter = ^TAvailFormatter;
 TAvailFormatter = object(TFormatter)
  Rows: PColumnRows;
  Total: Longint;

  procedure BeforeAll; virtual;
  procedure BeforeGroup(const Group: PAvailGroup); virtual;
  procedure DoItem(const Item: PForwardRequestListItem); virtual;
  procedure AfterGroup; virtual;
  procedure AfterAll; virtual;
 private
  procedure Flush;
 end;

 PAvailPureFormatter = ^TAvailPureFormatter;
 TAvailPureFormatter = object(TFormatter)
  Total: Longint;
  WriteHeader: Boolean;

  constructor Init(const ATarget: PStrings; const AGroups: PAvailGroups; const AWriteHeader: Boolean);
  procedure BeforeAll; virtual;
  procedure BeforeGroup(const Group: PAvailGroup); virtual;
  procedure DoItem(const Item: PForwardRequestListItem); virtual;
  procedure AfterAll; virtual;
 end;

 PAvailPool = ^TAvailPool;
 TAvailPool = object(TStringCollection)
  function KeyOf(Item: Pointer): Pointer; virtual;
  procedure FreeItem(Item: Pointer); virtual;
 end;

 TAvailGroup = object(TAvailPool)
  fr: PForwardRequests;

  constructor Init(const Afr: PForwardRequests);
 end;

 TAvailGroups = object(TCollection)
  Block: TAreafixGetRequestInfoBlock;
  SplitToGroups: Boolean;

  constructor Init(const ASplitToGroups: Boolean);
  procedure FillAreas; virtual;
  procedure Format(const Formatter: PFormatter); virtual;
  function CouldBeAdded(const fr: PForwardRequests; const Item: PForwardRequestListItem): Boolean; virtual;
  destructor Done; virtual;
 end;

 PQueries = ^TQueries;
 TQueries = object(TStringCollection)
  Block: TAreafixGetRequestInfoBlock;

  constructor Init;
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  procedure FreeItem(Item: Pointer); virtual;
  procedure FillAreas;
  procedure Format(const Target: PStrings); virtual;
 end;

{ TFormatter }

constructor TFormatter.Init(const ATarget: PStrings; const AGroups: PAvailGroups);
 begin
  inherited Init;

  Target:=ATarget;
  Groups:=AGroups;
 end;

procedure TFormatter.BeforeAll;
 begin
 end;

procedure TFormatter.BeforeGroup(const Group: PAvailGroup);
 begin
 end;

procedure TFormatter.DoItem(const Item: PForwardRequestListItem);
 begin
 end;

procedure TFormatter.AfterGroup;
 begin
 end;

procedure TFormatter.AfterAll;
 begin
 end;

{ TAvailFormatter }

procedure TAvailFormatter.BeforeAll;
 var
  Address: TAddress;
 begin
  Rows:=New(PColumnRows, Init($100, $100));

  Total:=0;

  jt.link.GetAddress(Groups^.Block.Link, Address);

  Rows^.Add^.Insert(New(PColumn, Init(0, 0, colMaxWidth, caLeft,
   'The following areas available to ' + AddressToStrEx(Address) + ' from uplinks of ' +
   AddressToStrEx(Groups^.Block.OurAddress) + ':')));

  Flush;
 end;

procedure TAvailFormatter.BeforeGroup(const Group: PAvailGroup);
 begin
  if Group^.Count > 0 then
   begin
    Rows^.Add^.Insert(New(PColumn, Init(0, 0, colMaxWidth, caLeft, '')));

    Rows^.Add^.Insert(New(PColumn, Init(0, 0, colMaxWidth, caLeft,
     Long2Str(Group^.Count) + ' area' + _s(Group^.Count) + ' available from ' +
     AddressToStrEx(Group^.fr^.Address) + ':')));

    Rows^.Add^.Insert(New(PColumn, Init(0, 0, colMaxWidth, caLeft, '')));

    Inc(Total, Group^.Count);
   end;
 end;

procedure TAvailFormatter.DoItem(const Item: PForwardRequestListItem);
 begin
  with Rows^.Add^ do
   begin
    Insert(New(PColumn, Init(0, colListAreaNameWrapped, colListAreaName, caLeft, Item^.Name^)));
    Insert(New(PColumn, Init(1, 0, colListDescription, caLeft, GetPString(Item^.Desc))));
   end;

  if Rows^.Count > FlushLimit then
   Flush;
 end;

procedure TAvailFormatter.AfterGroup;
 begin
  Flush;
 end;

procedure TAvailFormatter.AfterAll;
 begin
  Rows^.Add^.Insert(New(PColumn, Init(0, 0, colMaxWidth, caLeft, '')));

  Rows^.Add^.Insert(New(PColumn, Init(0, 0, colMaxWidth, caLeft, 'Total: ' + Long2Str(Total) + ' area' + _s(Total))));

  Flush;

  Dispose(Rows, Done);
 end;

{ private methods }

procedure TAvailFormatter.Flush;
 begin
  jt.SetStatus('formatting available areas list');

  Rows^.FormatRows(Target);

  Rows^.FreeAll;

  jt.SetStatus('generating available areas list');
 end;

{ TAvailPureFormatter }

constructor TAvailPureFormatter.Init(const ATarget: PStrings; const AGroups: PAvailGroups; const AWriteHeader: Boolean);
 begin
  inherited Init(ATarget, AGroups);

  WriteHeader:=AWriteHeader;
 end;

procedure TAvailPureFormatter.BeforeAll;
 var
  Address: TAddress;
 begin
  if WriteHeader then
   begin
    jt.link.GetAddress(Groups^.Block.Link, Address);

    Target^.Insert(NewStr('The following areas available to ' + AddressToStrEx(Address) + ' from uplinks of ' +
     AddressToStrEx(Groups^.Block.OurAddress) + ':'));

    Target^.Insert(nil);
   end;
 end;

procedure TAvailPureFormatter.BeforeGroup(const Group: PAvailGroup);
 begin
  Inc(Total, Group^.Count);
 end;

procedure TAvailPureFormatter.DoItem(const Item: PForwardRequestListItem);
 begin
  Target^.Insert(NewStr(Item^.Name^));
 end;

procedure TAvailPureFormatter.AfterAll;
 begin
  if WriteHeader then
   begin
    if Total > 0 then
     Target^.Insert(nil);

    Target^.Insert(NewStr('Total: ' + Long2Str(Total) + ' area' + _s(Total)));
   end;
 end;

{ TAvailPool }

function TAvailPool.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=PForwardRequestListItem(Item)^.Name;
 end;

procedure TAvailPool.FreeItem(Item: Pointer);
 begin
 end;

{ TAvailGroup }

constructor TAvailGroup.Init(const Afr: PForwardRequests);
 begin
  inherited Init($100, $100);

  fr:=Afr;
 end;

{ TAvailGroups }

constructor TAvailGroups.Init(const ASplitToGroups: Boolean);
 begin
  inherited Init($10, $10);

  SplitToGroups:=ASplitToGroups;

  afxGetRequestInfo(Block);
 end;

procedure TAvailGroups.FillAreas;
 var
  Pool: PAvailPool;
  Group: PAvailGroup;
 procedure PerformForwardRequests(const fr: PForwardRequests); far;
  procedure PerformForwardRequestList(const List: PForwardRequestList); far;
   procedure PerformForwardRequestListItem(const Item: PForwardRequestListItem); far;
    var
     PoolIndex: xInteger;
    begin
     if not Pool^.Search(Item^.Name, PoolIndex) then
      if CouldBeAdded(fr, Item) then
       begin
        Group^.Insert(Item);

        Pool^.AtInsert(PoolIndex, Item);
       end;
    end;
   begin
    List^.ForEach(@PerformForwardRequestListItem);
   end;
  begin
   if SplitToGroups then
    begin
     Group:=New(PAvailGroup, Init(fr));

     Insert(Group);
    end;

   fr^.Lists^.Compile(False);

   fr^.Lists^.ForEach(@PerformForwardRequestList);
  end;
 begin
  if not SplitToGroups then
   begin
    Group:=New(PAvailGroup, Init(nil));

    Insert(Group);
   end;

  Pool:=New(PAvailPool, Init($1000, $1000));

  fwdreqOrder^.ForEach(@PerformForwardRequests);

  Dispose(Pool, Done);
 end;

procedure TAvailGroups.Format(const Formatter: PFormatter);
 procedure FormatGroup(const Group: PAvailGroup); far;
  procedure FormatItem(const Item: PForwardRequestListItem); far;
   begin
    Formatter^.DoItem(Item);
   end;
  begin
   Formatter^.BeforeGroup(Group);

   Group^.ForEach(@FormatItem);

   Formatter^.AfterGroup;
  end;
 begin
  Formatter^.BeforeAll;

  ForEach(@FormatGroup);

  Formatter^.AfterAll;

  Dispose(Formatter, Done);
 end;

function TAvailGroups.CouldBeAdded(const fr: PForwardRequests; const Item: PForwardRequestListItem): Boolean;
 begin
  CouldBeAdded:=jt.area.Seek(Item^.Name^) = nil;
 end;

destructor TAvailGroups.Done;
 begin
  inherited Done;
 end;

{ TQueries }

constructor TQueries.Init;
 begin
  inherited Init($10, $10);

  afxGetRequestInfo(Block);
 end;

function TQueries.Compare(Key1, Key2: Pointer): xInteger;
 var
  K1: PForwardRequestQueueItem absolute Key1;
  K2: PForwardRequestQueueItem absolute Key2;
  A1: TAddress;
  A2: TAddress;
  I: xInteger;
 begin
  jt.link.GetAddress(K1^.RequestedFrom, A1);
  jt.link.GetAddress(K2^.RequestedFrom, A2);

  I:=AddressCompare(A1, A2);

  if I = 0 then
   Compare:=inherited Compare(K1^.Name, K2^.Name)
  else
   Compare:=I;
 end;

procedure TQueries.FreeItem(Item: Pointer);
 begin
 end;

procedure TQueries.FillAreas;
 procedure Perform(const Item: PForwardRequestQueueItem); far;
  begin
   if Item^.IndexOf(Block.Link) <> -1 then
    Insert(Item);
  end;
 begin
  fwdreqQueue^.ForEach(@Perform);
 end;

procedure TQueries.Format(const Target: PStrings);
 var
  Rows: PColumnRows;
 procedure Flush;
  begin
   Rows^.FormatRows(Target);

   Rows^.FreeAll;
  end;
 procedure Perform(const Item: PForwardRequestQueueItem); far;
  var
   Address: TAddress;
  begin
   with Rows^.Add^ do
    begin
     Insert(New(PColumn, Init(0, colListAreaNameWrapped, colListAreaName, caLeft, Item^.Name^)));

     jt.link.GetAddress(Item^.RequestedBy, Address);

     Insert(New(PColumn, Init(1, 0, colListRequestedBy, caRight, AddressToStrEx(Address))));

     jt.link.GetAddress(Item^.RequestedFrom, Address);

     Insert(New(PColumn, Init(1, 0, colListRequestedFrom, caRight, AddressToStrEx(Address))));
    end;

   if Rows^.Count > FlushLimit then
    Flush;
  end;
 var
  Address: TAddress;
 begin
  Rows:=New(PColumnRows, Init($100, $100));

  jt.link.GetAddress(Block.Link, Address);

  if Count = 0 then
   Rows^.Add^.Insert(New(PColumn, Init(0, 0, colMaxWidth, caLeft,
    'There is no pending forward request areas for ' + AddressToStrEx(Address))))
  else
   begin
    Rows^.Add^.Insert(New(PColumn, Init(0, 0, colMaxWidth, caLeft,
     'The following forward request areas are pending for ' + AddressToStrEx(Address) + ':')));

    Rows^.Add^.Insert(New(PColumn, Init(0, 0, colMaxWidth, caLeft, '')));
   end;

  with Rows^.Add^ do
   begin
    Insert(New(PColumn, Init(0, colListAreaNameWrapped, colListAreaName, caLeft, 'Area')));
    Insert(New(PColumn, Init(1, 0, colListRequestedBy, caRight, 'Requested by')));
    Insert(New(PColumn, Init(1, 0, colListRequestedFrom, caRight, 'Requested from')));
   end;

  with Rows^.Add^ do
   begin
    Insert(New(PColumn, Init(0, colListAreaNameWrapped, colListAreaName, caLeft, PadCh('', '-', colListAreaName))));
    Insert(New(PColumn, Init(1, 0, colListRequestedBy, caLeft, PadCh('', '-', colListRequestedBy))));
    Insert(New(PColumn, Init(1, 0, colListRequestedFrom, caLeft, PadCh('', '-', colListRequestedFrom))));
   end;

  ForEach(@Perform);

  Flush;

  Dispose(Rows, Done);
 end;

{ afxAvailHandler }

procedure afxAvailHandler(const Command: String; Value: String);
 var
  Reply: Pointer;
  Created: Boolean;
  Groups: PAvailGroups;
  List: PStrings;
  Mode: (amPure, amFull);
  Block: TAreafixGetRequestInfoBlock;
  Address: TAddress;
 procedure Transform(const Line: PString); far;
  begin
   if Line = nil then
    jt.msg.Add(Reply, '')
   else
    jt.msg.Add(Reply, RTrim(Line^));
  end;
 begin
  afxGetRequestInfo(Block);

  jt.link.GetAddress(Block.Link, Address);

  if jt.rests.Check(restNoAVAIL, @Address, nil) then
   begin
    jt.LogWrite('A', 'Prohibited');

    jt.msg.Add(afxCreateReply('Common', Created), '%AVAIL: you have no rights to execute this command');

    Exit;
   end;

  TrimEx(Value);

  StUpcaseEx(Value);

  if Value = 'PURE' then Mode:=amPure else
  if Value = 'FULL' then Mode:=amFull else
  if Value = '' then Mode:=amFull else
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%AVAIL ' + Value + ': invalid switch; PURE, FULL or nothing expected');

    Exit;
   end;

  Reply:=afxCreateReply('AVAIL.' + GenerateMSGID, Created);

  if Created then
   begin
    case Mode of
     amPure: jt.msg.SetSubject(Reply, '%AVAIL PURE response');
     amFull: jt.msg.SetSubject(Reply, '%AVAIL FULL response');
    end;

    afxFixEmptyReply(Reply);
   end;

  List:=New(PStrings, Init($100, $1000));

  Groups:=New(PAvailGroups, Init(Mode = amFull));

  jt.SetStatus('preparing %AVAIL');

  Groups^.FillAreas;

  case Mode of
   amPure: Groups^.Format(New(PAvailPureFormatter, Init(List, Groups, True)));
   amFull: Groups^.Format(New(PAvailFormatter, Init(List, Groups)));
  end;

  Dispose(Groups, Done);

  List^.ForEach(@Transform);

  Dispose(List, Done);

  jt.ClearStatus;
 end;

{ afxFAvailHandler }

procedure afxFAvailHandler(const Command: String; Value: String);
 var
  Reply, msgp: Pointer;
  Created: Boolean;
  Groups: PAvailGroups;
  List: PStrings;
  Mode: (amPure, amFull);
  FName: String;
  F: Text;
  Address: TAddress;
  Block: TAreafixGetRequestInfoBlock;
 procedure Transform(const Line: PString); far;
  begin
   if Line = nil then
    WriteLn(F)
   else
    WriteLn(F, RTrim(Line^));
  end;
 begin
  afxGetRequestInfo(Block);

  jt.link.GetAddress(Block.Link, Address);

  if jt.rests.Check(restNoFAVAIL, @Address, nil) then
   begin
    jt.LogWrite('A', 'Prohibited');

    jt.msg.Add(afxCreateReply('Common', Created), '%FAVAIL: you have no rights to execute this command');

    Exit;
   end;

  TrimEx(Value);

  StUpcaseEx(Value);

  if Value = 'PURE' then Mode:=amPure else
  if Value = 'FULL' then Mode:=amFull else
  if Value = '' then Mode:=amPure else
   begin
    jt.msg.Add(afxCreateReply('Common', Created), '%FAVAIL ' + Value + ': invalid switch; PURE, FULL or nothing expected');

    Exit;
   end;

  repeat
   FName:=jt.core.GetTempAttaches + GenerateMSGID + '.lst';
  until not ExistFile(FName);

  if IOResult <> 0 then;

  Assign(F, FName);
  Rewrite(F);

  if InOutRes <> 0 then
   begin
    jt.Report('Failed to create ' + FName + ', rc=' + Long2Str(IOResult), True);

    jt.msg.Add(afxCreateReply('Common', Created), 'Unable to execute %FAVAIL cause of internal error');

    Exit;
   end;

  Reply:=afxCreateReply('FAVAIL.' + GenerateMSGID, Created);

  afxFixEmptyReply(Reply);

  jt.msg.SetAttributes(Reply, jt.msg.GetAttributes(Reply) or maAttach);

  jt.msg.SetSubject(Reply, FName);

  msgp:=jt.msg.CreateParser(Reply, True);

  jt.msg.p.AddKludge(msgp, #1'FLAGS KFS');

  jt.msg.DestroyParser(msgp);

  List:=New(PStrings, Init($100, $1000));

  Groups:=New(PAvailGroups, Init(Mode = amFull));

  jt.SetStatus('preparing %FAVAIL');

  Groups^.FillAreas;

  case Mode of
   amPure: Groups^.Format(New(PAvailPureFormatter, Init(List, Groups, False)));
   amFull: Groups^.Format(New(PAvailFormatter, Init(List, Groups)));
  end;

  Dispose(Groups, Done);

  List^.ForEach(@Transform);

  Dispose(List, Done);

  Close(F);

  jt.msg.Add(afxCreateReply('Common', Created), Trim('%FAVAIL ' + Value) + ': ' + JustFileName(FName) + ' attached');

  jt.LogWrite('A', FName + ' attached');

  jt.ClearStatus;
 end;

{ afxAQueryHandler }

procedure afxAQueryHandler(const Command: String; Value: String);
 var
  Reply: Pointer;
  Created: Boolean;
  List: PStrings;
  Block: TAreafixGetRequestInfoBlock;
  Address: TAddress;
  Queries: PQueries;
 procedure Transform(const Line: PString); far;
  begin
   if Line = nil then
    jt.msg.Add(Reply, '')
   else
    jt.msg.Add(Reply, RTrim(Line^));
  end;
 begin
  afxGetRequestInfo(Block);

  jt.link.GetAddress(Block.Link, Address);

  if jt.rests.Check(restNoAQUERY, @Address, nil) then
   begin
    jt.LogWrite('A', 'Prohibited');

    jt.msg.Add(afxCreateReply('Common', Created), '%AQUERY: you have no rights to execute this command');

    Exit;
   end;

  TrimEx(Value);

  StUpcaseEx(Value);

  Reply:=afxCreateReply('AQUERY.' + GenerateMSGID, Created);

  if Created then
   begin
    jt.msg.SetSubject(Reply, '%AQUERY response');

    afxFixEmptyReply(Reply);
   end;

  List:=New(PStrings, Init($100, $1000));

  Queries:=New(PQueries, Init);

  Queries^.FillAreas;

  Queries^.Format(List);

  Dispose(Queries, Done);

  List^.ForEach(@Transform);

  Dispose(List, Done);

  jt.ClearStatus;
 end;

end.