unit frAfxReq;

interface
uses
     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     afxAPI,

     jtMHLcmn,

     frCmn,
     frOrder,
     frQueue,
     frFwdReq,
     frLists,

     Wizard;

procedure SubscribeAfter(var Block: TAreafixSubscribeBlock);
procedure UnsubscribeAfter(var Block: TAreafixUnsubscribeBlock);

implementation

{ SubscribeAfter }

procedure SubscribeAfter(var Block: TAreafixSubscribeBlock);
 procedure Did;
  begin
   Block.Mask:=nil;

   Inc(Block.Changes);
  end;
 var
  Info: TAreafixGetRequestInfoBlock;
  Queries: PForwardRequestQueries;
  Query: PForwardRequestQuery;
  QueueItem: PForwardRequestQueueItem;
  Address: TAddress;
  Index: xInteger;
  Max: Longint;
  st: Pointer;
 begin
  { maybe denied request came to us? }

  if jt.area.Seek(Block.Mask^) <> nil then
   Exit;

  afxGetRequestInfo(Info);

  { if requested area in queue... }

  if fwdreqQueue^.Search(Block.Mask, Index) then
   begin
    QueueItem:=fwdreqQueue^.At(Index);

    if QueueItem^.Search(Info.Link, Index) then
     begin
      jt.msg.Add(Block.Reply, '+' + Block.Mask^ + ': already subscribed (forward request pending)');

      jt.LogWrite('F', '+' + Block.Mask^ + ': already subscribed (forward request pending)');
     end
    else
     begin
      QueueItem^.AtInsert(Index, Info.Link);

      jt.msg.Add(Block.Reply, '+' + Block.Mask^ + ': subscribed (forward request pending)');

      jt.LogWrite('F', '+' + Block.Mask^ + ': subscribed (forward request pending)');
     end;

    Block.Mask:=nil;

    Inc(Block.Changes);
   end;

  { if anything did... }

  if Block.Changes <> 0 then
   Exit;

  { nothing did, check lists of uplinks }

  Queries:=New(PForwardRequestQueries, Init($10, $10));

  fwdreqOrder^.PerformQuery(Block.Mask^, Queries, Info.Link);

  if Queries^.Count > 0 then
   if (fwdreqMaxRequests <> 0) and (fwdreqQueue^.Count >= fwdreqMaxRequests) then
    begin
     jt.msg.Add(Block.Reply, '+' + Block.Mask^ + ': too many requests in queue, try later');

     jt.LogWrite('F', '+' + Block.Mask^ + ': too many requests in queue');

     Did;
    end
   else
    if not fwdreqQueue^.MaxRequestedByCheck(Info.Link) then
     begin
      jt.msg.Add(Block.Reply, '+' + Block.Mask^ + ': too many your requests in queue, try later');

      jt.LogWrite('F', '+' + Block.Mask^ + ': too many requests by ' + AddressOfLink(Info.Link) + ' in queue');

      Did;
     end
    else
     begin
      Query:=Queries^.At(0);

      if not fwdreqQueue^.MaxRequestedToCheck(Query^.fr^.Link) then
       begin
        jt.msg.Add(Block.Reply, '+' + Block.Mask^ + ': too many requests to ' + AddressOfLink(Query^.fr^.Link) +
         ' in queue, try later');

        jt.LogWrite('F', '+' + Block.Mask^ + ': too many requests to ' + AddressOfLink(Query^.fr^.Link) + ' in queue');

        Did;
       end
      else
       begin
        fwdreqQueue^.AddRequest(Info.Link, Query);

        jt.msg.Add(Block.Reply, '+' + Block.Mask^ + ': requested from ' + AddressToStrEx(Query^.fr^.Address));

        jt.LogWrite('F', '+' + Query^.Item^.Name^ + ': requested from ' + AddressToStrEx(Query^.fr^.Address) + ' (' +
         Query^.List^.FullListName + ')');

        Did;
       end;
     end;

  Dispose(Queries, Done);
 end;

{ UnsubscribeAfter }

procedure UnsubscribeAfter(var Block: TAreafixUnsubscribeBlock);
 var
  Info: TAreafixGetRequestInfoBlock;
  Index: xInteger;
  QueueItem: PForwardRequestQueueItem;
 procedure Did;
  begin
   Block.Mask:=nil;

   Inc(Block.Changes);
  end;
 function SeekAnotherRequestedBy(const Link: Pointer): Boolean; far;
  begin
   SeekAnotherRequestedBy:=(Link <> QueueItem^.RequestedBy) and (Link <> QueueItem^.RequestedFrom);
  end;
 var
  Link: Pointer;
  AAddress, BAddress: TAddress;
 begin
  if jt.area.Seek(Block.Mask^) <> nil then
   Exit;

  if Block.Changes <> 0 then
   Exit;

  if not fwdreqQueue^.Search(Block.Mask, Index) then
   Exit;

  afxGetRequestInfo(Info);

  QueueItem:=fwdreqQueue^.At(Index);

  if QueueItem^.IndexOf(Info.Link) = -1 then
   Exit;

  if Info.Link = QueueItem^.RequestedFrom then
   begin
    fwdreqQueue^.KillRequest(QueueItem, 'r/from unsub', Info.Link);

    jt.msg.Add(Block.Reply, '-' + Block.Mask^ + ': unsubscribed');

    jt.LogWrite('F', '-' + Block.Mask^ + ': unsubscribed (forward request killed - requested-from link unsubscribed)');

    Did;
   end else
  if Info.Link = QueueItem^.RequestedBy then
   begin
    Link:=QueueItem^.FirstThat(@SeekAnotherRequestedBy);

    if Link = nil then
     begin
      fwdreqQueue^.KillRequest(QueueItem, 'r/by unsub', Info.Link);

      jt.msg.Add(Block.Reply, '-' + Block.Mask^ + ': unsubscribed (forward request killed)');

      jt.LogWrite('F', '-' + Block.Mask^ + ': unsubscribed (forward request killed - requested-by link unsubscribed)');

      Did;
     end
    else
     begin
      jt.link.GetAddress(Info.Link, AAddress);
      jt.link.GetAddress(Link, BAddress);

      jt.msg.Add(Block.Reply, '-' + Block.Mask^ + ': unsubscribed');

      jt.LogWrite('F', '-' + Block.Mask^ + ': unsubscribed (requested-by changed from ' + AddressToStrEx(AAddress) + ' to ' +
       AddressToStrEx(BAddress) + ')');

      QueueItem^.Free(Info.Link);

      QueueItem^.RequestedBy:=Link;

      Did;
     end;
   end else
  if QueueItem^.Count <= 2 then
   begin
    fwdreqQueue^.KillRequest(QueueItem, 'no more subs', Info.Link);

    jt.msg.Add(Block.Reply, '-' + Block.Mask^ + ': unsubscribed');

    jt.LogWrite('F', '-' + Block.Mask^ + ': unsubscribed (forward request killed - no more subscribers)');

    Did;
   end
  else
   begin
    jt.msg.Add(Block.Reply, '-' + Block.Mask^ + ': unsubscribed');

    jt.LogWrite('F', '-' + Block.Mask^ + ': unsubscribed (forward request pending)');

    QueueItem^.Free(Info.Link);

    Did;
   end;
 end;

end.