unit frAnn;

interface
uses
     PlugAPI,
     PlugSIDE,

     Objects,

     frCmn,
     frQueue,
     frFwdReq,

     annAPI,

     Columns,
     Thunks,

     jtMHLcmn,

     RStream;

procedure AddAnnounce(Action: TForwardRequestAction; const QueueItem: PForwardRequestQueueItem; const Details: String);

procedure ProcessFirstAnnounce(const Block: TProcessAnnounceFirstBlock);
procedure ProcessAnnounce(const Block: TProcessAnnounceBlock);
procedure ProcessLastAnnounce(const Block: TProcessAnnounceLastBlock);

implementation

const
 colAction              = 12;
 colArea                = 22;
 colBy                  = 12;
 colFrom                = 12;
 colDetails             = 16;

{ AddAnnounce }

procedure AddAnnounce(Action: TForwardRequestAction; const QueueItem: PForwardRequestQueueItem; const Details: String);
 var
  Stream: PRamStream;
  Address: TAddress;
 begin
  Stream:=New(PRamStream, Init);

  Stream^.Write(Action, SizeOf(Action));
  Stream^.WriteStr(QueueItem^.Name);

  jt.link.GetAddress(QueueItem^.RequestedBy, Address);

  Stream^.Write(Address, SizeOf(Address));

  jt.link.GetAddress(QueueItem^.RequestedFrom, Address);

  Stream^.Write(Address, SizeOf(Address));

  Stream^.WriteStr(@Details);

  AddAnnounceRecord('ForwardRequestNews', Stream);

  Dispose(Stream, Done);
 end;

{ ProcessFirstAnnounce }

procedure ProcessFirstAnnounce(const Block: TProcessAnnounceFirstBlock);
 var
  Rows: PColumnRows;
 begin
  jt.msg.SetSubject(Block.msg, 'forward request news');

  jt.msg.Add(Block.msg, '');

  Rows:=New(PColumnRows, Init($10, $10));

  with Rows^.Add^ do
   begin
    Insert(New(PColumn, Init(0, 0, colAction, caLeft, 'Action')));
    Insert(New(PColumn, Init(1, 0, colArea, caLeft, 'Area')));
    Insert(New(PColumn, Init(1, 0, colBy, caRight, 'By')));
    Insert(New(PColumn, Init(1, 0, colFrom, caRight, 'From')));
    Insert(New(PColumn, Init(1, 0, colDetails, caRight, 'Details')));
   end;

  with Rows^.Add^ do
   begin
    Insert(New(PColumnFiller, Init(0, colAction, '-')));
    Insert(New(PColumnFiller, Init(1, colArea, '-')));
    Insert(New(PColumnFiller, Init(1, colBy, '-')));
    Insert(New(PColumnFiller, Init(1, colFrom, '-')));
    Insert(New(PColumnFiller, Init(1, colDetails, '-')));
   end;

  Rows^.FormatRowsTomsg(Block.msg);

  Dispose(Rows, Done);
 end;

{ ProcessAnnounce }

procedure ProcessAnnounce(const Block: TProcessAnnounceBlock);
 var
  Stream: PRamStream;
  Rows: PColumnRows;
  Action: Longint;
  Name, Details: PString;
  RequestedBy, RequestedFrom: TAddress;
 begin
  Stream:=New(PRamStream, Init);

  jt.rf.Get(Block.rf, Block.ResourceName^, Stream, StreamLinker);

  Stream^.Seek(0);

  Stream^.Read(Action, SizeOf(Action));

  Name:=Stream^.ReadStr;

  Stream^.Read(RequestedBy, SizeOf(RequestedBy));
  Stream^.Read(RequestedFrom, SizeOf(RequestedFrom));

  Details:=Stream^.ReadStr;

  if Name <> nil then
   begin
    Rows:=New(PColumnRows, Init($10, $10));

    with Rows^.Add^ do
     begin
      case Action of
       frRequested: Insert(New(PColumn, Init(0, 0, colAction, caRight, 'requested')));
       frRerequested: Insert(New(PColumn, Init(0, 0, colAction, caRight, 're-requested')));
       frKilled: Insert(New(PColumn, Init(0, 0, colAction, caRight, 'killed')));
       frGot: Insert(New(PColumn, Init(0, 0, colAction, caRight, 'got')));
      else
       Insert(New(PColumn, Init(0, 0, colAction, caRight, 'brainisdead')));
      end;

      Insert(New(PColumn, Init(1, 0, colArea, caLeft, Name^)));

      if IsCleanAddress(RequestedBy) then
       Insert(New(PColumn, Init(1, 0, colBy, caRight, '')))
      else
       Insert(New(PColumn, Init(1, 0, colBy, caRight, AddressToStrEx(RequestedBy))));

      if IsCleanAddress(RequestedFrom) then
       Insert(New(PColumn, Init(1, 0, colFrom, caRight, '')))
      else
       Insert(New(PColumn, Init(1, 0, colFrom, caRight, AddressToStrEx(RequestedFrom))));

      if Details = nil then
       Insert(New(PColumn, Init(1, 0, colDetails, caRight, '')))
      else
       Insert(New(PColumn, Init(1, 0, colDetails, caRight, Details^)));
     end;

    Rows^.FormatRowsTomsg(Block.msg);

    Dispose(Rows, Done);
   end;

  DisposeStr(Details);
  DisposeStr(Name);

  Dispose(Stream, Done);
 end;

{ ProcessLastAnnounce }

procedure ProcessLastAnnounce(const Block: TProcessAnnounceLastBlock);
 begin
  jt.msg.Add(Block.msg, '');
 end;

end.