unit frFwdReq;

interface
uses
     PlugAPI,
     PlugSIDE,

     Objects,

     frLists,

     jtMHLcmn,

     Wizard;

const
 frRequested            = $00000000;
 frRerequested          = $00000001;
 frKilled               = $00000002;
 frGot                  = $00000003;

type
 TForwardRequestAction = Longint;

 PForwardRequests = ^TForwardRequests;
 TForwardRequests = object(TObject)
  Link: Pointer;
  Address: TAddress;

  Enabled: Boolean;
  Lists: PForwardRequestLists;
  Timeout: Longint;
  Format: PAreaFormat;

  constructor Init(const ALink: Pointer);
  procedure ParseList(const Linker: Pointer; var Value: String);
  procedure ParseFormat(const Linker: Pointer; var Value: String);
  procedure ExportLink(const Linker: Pointer);
  procedure Compile(const Force: Boolean);
  destructor Done; virtual;
 public
  function GetTimeout: Longint;
  function GetFormat: PAreaFormat;
 end;

function stForwardRequests(const Action: TStorageAction; const Data: Pointer): Pointer; export;
function GetForwardRequests(const Link: Pointer): PForwardRequests;

implementation
uses
     frCmn;

{ stForwardRequests }

function stForwardRequests(const Action: TStorageAction; const Data: Pointer): Pointer;
 begin
  case Action of
   saDestroy: Dispose(PForwardRequests(Data), Done);
   saClone: stForwardRequests:=nil;
   saInfo: stForwardRequests:=NewStr('forwardrequests object');
  end;
 end;

{ GetForwardRequests }

function GetForwardRequests(const Link: Pointer): PForwardRequests;
 var
  st: Pointer;
  fr: PForwardRequests;
 begin
  st:=jt.link.st(Link);

  fr:=jt.st.Get(st, 'ForwardRequests');

  if fr = nil then
   begin
    fr:=New(PForwardRequests, Init(Link));

    jt.st.Add(st, 'ForwardRequests', fr, stForwardRequests);
   end;

  GetForwardRequests:=fr;
 end;

{ TForwardRequests }

constructor TForwardRequests.Init(const ALink: Pointer);
 begin
  inherited Init;

  Link:=ALink;

  jt.link.GetAddress(Link, Address);

  Lists:=New(PForwardRequestLists, Init($10, $10));

  Timeout:=-1;
 end;

procedure TForwardRequests.ParseList(const Linker: Pointer; var Value: String);
 var
  ListType: String;
 function FName: String;
  var
   S: String;
  begin
   S:=jt.linker.ParseStringToken(Linker, Value);

   if S = '' then
    jt.linker.Error(Linker, 'invalid filename');

   FName:=S;
  end;
 begin
  ListType:=jt.linker.ParseWord(Linker, Value);

  StUpcaseEx(ListType);

  if ListType = 'ECHOLIST' then Lists^.Insert(New(PForwardRequestEcholist, Init(FName))) else
  if ListType = 'AREADESC' then Lists^.Insert(New(PForwardRequestAreaDesc, Init(FName))) else
  if ListType = 'BCL' then Lists^.Insert(New(PForwardRequestBCL, Init(Link))) else
  if ListType = 'UNCONDITIONAL' then LIsts^.Insert(New(PForwardRequestUnconditional, Init)) else
   jt.linker.Error(Linker, 'unknown list type "' + ListType + '"');
 end;

procedure TForwardRequests.ParseFormat(const Linker: Pointer; var Value: String);
 begin
  if Format <> nil then
   jt.linker.Error(Linker, 'redefinition of FORWARDREQUESTFORMAT');

  Format:=jt.NewAreaFormat;

  Format^.Style:=asEchomail;

  jt.linker.ParseAreaFormat(Linker, jt.linker.ParseWord(Linker, Value), Format);

  jt.linker.ParseAreaPath(Linker, Value, Format);
 end;

procedure TForwardRequests.ExportLink(const Linker: Pointer);
 procedure ExportList(const List: PForwardRequestList); far;
  begin
   List^.ExportLink(Linker);
  end;
 begin
  if Enabled then
   begin
    jt.linker.ExportLine(Linker, ' ForwardRequests Yes');

    if Timeout <> -1 then
     jt.linker.ExportLine(Linker, '   ForwardRequestTimeout ' + jt.linker.UnparseNumber(Linker, Timeout));

    if Format <> nil then
     jt.linker.ExportLine(Linker, '   ForwardRequestFormat ' + jt.linker.UnparseAreaFormat(Linker, Format) + ' ' +
                                                               jt.linker.UnparseAreaPath(Linker, Format));

    Lists^.ForEach(@ExportList);
   end;
 end;

procedure TForwardRequests.Compile(const Force: Boolean);
 begin
  Lists^.Compile(Force);
 end;

destructor TForwardRequests.Done;
 begin
  Dispose(Lists, Done);

  if Format <> nil then
   jt.DestroyAreaFormat(Format);

  inherited Done;
 end;

{ public methods }

function TForwardRequests.GetTimeout: Longint;
 begin
  if Timeout = -1 then
   GetTimeout:=fwdreqDefaultTimeout
  else
   GetTimeout:=Timeout;
 end;

function TForwardRequests.GetFormat: PAreaFormat;
 begin
  if Format = nil then
   GetFormat:=fwdreqDefaultFormat
  else
   GetFormat:=Format;
 end;

end.
