unit frLists;

interface
uses
     PlugAPI,
     PlugSIDE,

     bclAPI,

     Objects,

     RStream,

     Thunks,

     Wizard;

const
 ValidAreaNameSet: TCharSet = [#33..#127] - ['*', '?'];

type
 PForwardRequestListItem = ^TForwardRequestListItem;
 TForwardRequestListItem = object(TObject)
  Name, Desc: PString;

  constructor Init(const AName, ADesc: String);
  constructor Load(var S: TStream);
  procedure Store(var S: TStream);
  destructor Done; virtual;
 end;

 PForwardRequestList = ^TForwardRequestList;
 TForwardRequestList = object(TStringCollection)
  Stamp: Longint;
  FName: PString;
  Compiled: Boolean;

  constructor Init(const AFName: String);
  procedure FreeItem(Item: Pointer); virtual;
  function KeyOf(Item: Pointer): Pointer; virtual;
  destructor Done; virtual;
 public
  function id: String; virtual;
  function CreateFileParser(var rc: Longint): Pointer; virtual;
  function ListName: String; virtual;
  function FullListName: String; virtual;
  procedure Compile(Force: Boolean); virtual;
  procedure ExportLink(const Linker: Pointer); virtual;
 end;

 PForwardRequestEcholist = ^TForwardRequestEcholist;
 TForwardRequestEcholist = object(TForwardRequestList)
  function ListName: String; virtual;
  function FullListName: String; virtual;
  procedure Compile(Force: Boolean); virtual;
 end;

 PForwardRequestAreaDesc = ^TForwardRequestAreaDesc;
 TForwardRequestAreaDesc = object(TForwardRequestList)
  function ListName: String; virtual;
  function FullListName: String; virtual;
  procedure Compile(Force: Boolean); virtual;
 end;

 PForwardRequestBCL = ^TForwardRequestBCL;
 TForwardRequestBCL = object(TForwardRequestList)
  Link: Pointer;

  constructor Init(const ALink: Pointer);
  function ListName: String; virtual;
  function FullListName: String; virtual;
  procedure Compile(Force: Boolean); virtual;
  procedure ExportLink(const Linker: Pointer); virtual;
 end;

 PForwardRequestUnconditional = ^TForwardRequestUnconditional;
 TForwardRequestUnconditional = object(TForwardRequestList)
  constructor Init;
  function ListName: String; virtual;
  function FullListName: String; virtual;
  procedure Compile(Force: Boolean); virtual;
  procedure ExportLink(const Linker: Pointer); virtual;
  function Search(Key: Pointer; var Index: xInteger): Boolean; virtual;
 end;

 PForwardRequestLists = ^TForwardRequestLists;
 TForwardRequestLists = object(TCollection)
  procedure Compile(const Force: Boolean); virtual;
  function Search(const Name: String; var Item: PForwardRequestListItem; var List: PForwardRequestList): Boolean; virtual;
 end;

function bclEnumerator(const Name, Desc: PString; const Param: Pointer): Boolean; export;

implementation
uses
     frCmn;

{ TForwardRequestListItem }

constructor TForwardRequestListItem.Init(const AName, ADesc: String);
 begin
  inherited Init;

  Name:=NewStr(AName);
  Desc:=NewStr(ADesc);
 end;

constructor TForwardRequestListItem.Load(var S: TStream);
 begin
  Name:=S.ReadStr;
  Desc:=S.ReadStr;
 end;

procedure TForwardRequestListItem.Store(var S: TStream);
 begin
  S.WriteStr(Name);
  S.WriteStr(Desc);
 end;

destructor TForwardRequestListItem.Done;
 begin
  DisposeStr(Desc);
  DisposeStr(Name);

  inherited Done;
 end;

{ TForwardRequestList }

constructor TForwardRequestList.Init(const AFName: String);
 begin
  inherited Init($100, $1000);

  FName:=NewStr(AFName);

  if FName <> nil then
   Stamp:=GetFileDate(FName^);
 end;

procedure TForwardRequestList.FreeItem(Item: Pointer);
 begin
  TCollection.FreeItem(Item);
 end;

function TForwardRequestList.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=PForwardRequestListItem(Item)^.Name;
 end;

destructor TForwardRequestList.Done;
 begin
  DisposeStr(FName);

  inherited Done;
 end;

{ public methods }

function TForwardRequestList.id: String;
 begin
  id:='ForwardRequestList.' + FName^;
 end;

function TForwardRequestList.CreateFileParser(var rc: Longint): Pointer;
 var
  Stream: PDosStream;
 begin
  Stream:=New(PDosStream, Init(FName^, stOpenRead));

  if Stream^.Status = stOk then
   CreateFileParser:=jt.msg.p.CreateEx(Stream, False, StreamLinker, True)
  else
   begin
    rc:=Stream^.Status;

    Dispose(Stream, Done);

    CreateFileParser:=nil;
   end;
 end;

function TForwardRequestList.ListName: String;
 begin
  Abstract;
 end;

function TForwardRequestList.FullListName: String;
 begin
  Abstract;
 end;

procedure TForwardRequestList.Compile(Force: Boolean);
 begin
  Abstract;
 end;

procedure TForwardRequestList.ExportLink(const Linker: Pointer);
 begin
  jt.linker.ExportLine(Linker, '   ForwardRequestList ' + ListName + ' ' +
    jt.linker.UnparseString(Linker, FName^));
 end;

{ TForwardRequestEcholist }

function TForwardRequestEchoList.ListName: String;
 begin
  ListName:='Echolist';
 end;

function TForwardRequestEchoList.FullListName: String;
 begin
  FullListName:='echolist "' + FName^ + '"';
 end;

procedure TForwardRequestEcholist.Compile(Force: Boolean);
 var
  Stream: PRamStream;
  Parser: Pointer;
  AStamp, Countdown, rc: Longint;
 procedure WriteItem(const Item: PForwardRequestListItem); far;
  begin
   Item^.Store(Stream^);
  end;
 var
  S, Name, Desc: String;
  Index: xInteger;
  Item: PForwardRequestListItem;
  K: Byte;
 begin
  if Compiled and not Force then
   Exit;

  jt.SetStatus('loading ' + FName^);

  FreeAll;

  Stream:=New(PRamStream, Init);

  if not Force and jt.rf.Get(jt.rf.common, id, Stream, StreamLinker) then
   begin
    Stream^.Seek(0);

    Stream^.Read(AStamp, SizeOf(AStamp));

    if Stamp = AStamp then
     begin
      Stream^.Read(Countdown, SizeOf(Countdown));

      while Countdown <> 0 do
       begin
        Insert(New(PForwardRequestListItem, Load(Stream^)));

        Dec(Countdown);
       end;
     end
    else
     Force:=True;
   end
  else
   Force:=True;

  if Force then
   begin
    jt.SetStatus('compiling ' + FName^);

    Parser:=CreateFileParser(rc);

    if Parser <> nil then
     begin
      while not jt.msg.p.EndOfMessage(Parser) do
       begin
        jt.msg.p.GetString(Parser, S);

        if (S <> '') and (S[1] <> ';') then
         begin
          K:=Pos(',', S);

          if K <> 0 then
           begin
            System.Delete(S, 1, K);

            K:=Pos(',', S);

            if K <> 0 then
             begin
              Name:=Copy(S, 1, K - 1);

              System.Delete(S, 1, K);

              K:=Pos(',', S);

              if K <> 0 then
               begin
                Desc:=Copy(S, 1, K - 1);

                StUpcaseEx(Name);

                if not Search(@Name, Index) then
                 AtInsert(Index, New(PForwardRequestListItem, Init(Name, Desc)))
                else
                 begin
                  Item:=At(Index);

                  if Item^.Desc = nil then
                   Item^.Desc:=NewStr(Desc);
                 end;
               end;
             end;
           end;
         end;
       end;

      jt.msg.p.Destroy(Parser);

      if Count = 0 then
       jt.LogWrite('F', FName^ + ': no areas')
      else
       jt.LogWrite('F', FName^ + ': ' + Long2Str(Count) + ' area' + _s(Count));

      Stream^.Seek(0);

      Stream^.Write(Stamp, SizeOf(Stamp));

      Countdown:=Count;

      Stream^.Write(Countdown, SizeOf(Countdown));

      ForEach(@WriteItem);

      Stream^.Truncate;

      jt.SetStatus('storing compiled ' + FName^);

      jt.rf.Add(jt.rf.common, id, Stream, StreamLinker);
     end
    else
     begin
      jt.LogWrite('?', 'Unable to open ' + FName^ + ', rc=' + Long2Str(rc));

      jt.rf.Remove(jt.rf.common, id);
     end;
   end;

  Dispose(Stream, Done);

  Compiled:=True;
 end;

{ TForwardRequestAreaDesc }

function TForwardRequestAreaDesc.ListName: String;
 begin
  ListName:='AreaDesc';
 end;

function TForwardRequestAreaDesc.FullListName: String;
 begin
  FullListName:='areadesc "' + FName^ + '"';
 end;

procedure TForwardRequestAreaDesc.Compile(Force: Boolean);
 var
  Stream: PRamStream;
  Parser: Pointer;
  AStamp, Countdown, rc: Longint;
 procedure WriteItem(const Item: PForwardRequestListItem); far;
  begin
   Item^.Store(Stream^);
  end;
 var
  S, Name, Desc: String;
  Index: xInteger;
  Item: PForwardRequestListItem;
  Ok: Boolean;
  K: Byte;
 begin
  if Compiled and not Force then
   Exit;

  jt.SetStatus('loading ' + FName^);

  FreeAll;

  Stream:=New(PRamStream, Init);

  if not Force and jt.rf.Get(jt.rf.common, id, Stream, StreamLinker) then
   begin
    Stream^.Seek(0);

    Stream^.Read(AStamp, SizeOf(AStamp));

    if Stamp = AStamp then
     begin
      Stream^.Read(Countdown, SizeOf(Countdown));

      while Countdown <> 0 do
       begin
        Insert(New(PForwardRequestListItem, Load(Stream^)));

        Dec(Countdown);
       end;
     end
    else
     Force:=True;
   end
  else
   Force:=True;

  if Force then
   begin
    jt.SetStatus('compiling ' + FName^);

    Parser:=CreateFileParser(rc);

    if Parser <> nil then
     begin
      while not jt.msg.p.EndOfMessage(Parser) do
       begin
        jt.msg.p.GetString(Parser, S);

        if Pos(#9, S) > 0 then
         for K:=1 to Length(S) do
          if S[K] = #9 then
           S[K]:=' ';

        TrimEx(S);

        if (S <> '') and (S[1] <> ';') then
         begin
          K:=Pos(' ', S);

          if K = 0 then
           K:=255;

          Name:=Copy(S, 1, K - 1);
          Desc:=Copy(S, K + 1, 255);

          TrimEx(Desc);

          Ok:=True;

          for K:=1 to Length(Name) do
           if not (Name[K] in ValidAreaNameSet) then
            begin
             Ok:=False;

             Break;
            end;

          if Length(Name) > 2 then
           begin
            StUpcaseEx(Name);

            if not Search(@Name, Index) then
             AtInsert(Index, New(PForwardRequestListItem, Init(Name, Desc)))
            else
             begin
              Item:=At(Index);

              if Item^.Desc = nil then
               Item^.Desc:=NewStr(Desc);
             end;
           end;
         end;
       end;

      jt.msg.p.Destroy(Parser);

      if Count = 0 then
       jt.LogWrite('F', FName^ + ': no areas')
      else
       jt.LogWrite('F', FName^ + ': ' + Long2Str(Count) + ' area' + _s(Count));

      Stream^.Seek(0);

      Stream^.Write(Stamp, SizeOf(Stamp));

      Countdown:=Count;

      Stream^.Write(Countdown, SizeOf(Countdown));

      ForEach(@WriteItem);

      Stream^.Truncate;

      jt.SetStatus('storing compiled ' + FName^);

      jt.rf.Add(jt.rf.common, id, Stream, StreamLinker);
     end
    else
     begin
      jt.LogWrite('?', 'Unable to open ' + FName^ + ', rc=' + Long2Str(rc));

      jt.rf.Remove(jt.rf.common, id);
     end;
   end;

  Dispose(Stream, Done);

  Compiled:=True;
 end;

{ TForwardRequestBCL }

function bclEnumerator(const Name, Desc: PString; const Param: Pointer): Boolean;
 var
  Index: xInteger;
  List: PForwardRequestBCL absolute Param;
  Item: PForwardRequestListItem;
  S: String;
 begin
  if Name <> nil then
   begin
    if Desc <> nil then
     S:=Desc^
    else
     S:='';

    if List^.Search(Name, Index) then
     if S <> '' then
      begin
       Item:=List^.At(Index);

       if Item^.Desc = nil then
        Item^.Desc:=NewStr(S);
      end
     else
    else
     List^.AtInsert(Index, New(PForwardRequestListItem, Init(Name^, S)));
   end;

  bclEnumerator:=False;
 end;

constructor TForwardRequestBCL.Init(const ALink: Pointer);
 begin
  inherited Init('');

  Link:=ALink;
 end;

function TForwardRequestBCL.ListName: String;
 begin
  ListName:='BCL';
 end;

function TForwardRequestBCL.FullListName: String;
 begin
  FullListName:='BCL';
 end;

procedure TForwardRequestBCL.Compile(Force: Boolean);
 var
  bcl: Pointer;
 begin
  if Compiled and not Force then
   Exit;

  FreeAll;

  jt.SetStatus('enumerating BCL');

  bcl:=bclOpen(Link);

  if bcl <> nil then
   bclEnumerate(bcl, bclEnumerator, @Self);

  Compiled:=True;
 end;

procedure TForwardRequestBCL.ExportLink(const Linker: Pointer);
 begin
  jt.linker.ExportLine(Linker, '   ForwardRequestList ' + ListName);
 end;

{ TForwardRequestUnconditional }

constructor TForwardRequestUnconditional.Init;
 begin
  inherited Init('');
 end;

function TForwardRequestUnconditional.ListName: String;
 begin
  ListName:='Unconditional';
 end;

function TForwardRequestUnconditional.FullListName: String;
 begin
  FullListName:='unconditional';
 end;

procedure TForwardRequestUnconditional.Compile(Force: Boolean);
 begin
 end;

procedure TForwardRequestUnconditional.ExportLink(const Linker: Pointer);
 begin
  jt.linker.ExportLine(Linker, '   ForwardRequestList ' + ListName);
 end;

function TForwardRequestUnconditional.Search(Key: Pointer; var Index: xInteger): Boolean;
 var
  Name: PString absolute Key;
  Item: PForwardRequestListItem;
 begin
  if not jt.rests.Check(restUnconditional, nil, Name) then
   Search:=False
  else
   begin
    Search:=True;

    if not inherited Search(Key, Index) then
     begin
      Item:=New(PForwardRequestListItem, Init(Name^, ''));

      AtInsert(Index, Item);
     end;
   end;
 end;

{ TForwardRequestLists }

procedure TForwardRequestLists.Compile(const Force: Boolean);
 procedure Perform(const List: PForwardRequestList); far;
  begin
   List^.Compile(Force);
  end;
 begin
  ForEach(@Perform);
 end;

function TForwardRequestLists.Search(const Name: String; var Item: PForwardRequestListItem;
  var List: PForwardRequestList): Boolean;
 function Perform(const AList: PForwardRequestList): Boolean; far;
  var
   Index: xInteger;
  begin
   if AList^.Search(@Name, Index) then
    begin
     Item:=AList^.At(Index);

     List:=AList;

     Perform:=True;
    end
   else
    Perform:=False;
  end;
 begin
  Search:=FirstThat(@Perform) <> nil;
 end;

end.