library FwdReq;

uses
     Objects,

     PlugAPI,
     PlugSIDE,

     AnnAPI,
     TossAPI,
     afxAPI,
     acrAPI,

     frCmn,
     frFwdReq,
     frAfx,
     frAfxReq,
     frOrder,
     frQueue,
     frInfo,
     frAnn,
     frACR,

     rlopAPI,

     jtMHLcmn,

     Wizard;

{ CheckLinkValidity }

procedure CheckLinkValidity(var Stuff);
 var
  Data: packed record
   Link, Linker: Pointer;
  end absolute Stuff;
  rc: String;
 begin
{
  if not rlopIsLinkValid(Data.Link, rc) then
   jt.linker.Error(Data.Linker, rc);
}
 end;

{ fwdreqMaxRequestsBy }

procedure fwdreqMaxRequestsBy(const Keyword, Linker, st: Pointer; Value: String); export;
 var
  Max: Longint;
 begin
  Max:=jt.linker.ParseNumber(Linker, Value);

  if Max < 0 then
   Max:=0;

  jt.st.Add(st, 'Max', Pointer(Max), nil);
 end;

{ fwdreqMaxRequestsFrom }

procedure fwdreqMaxRequestsFrom(const Keyword, Linker, st: Pointer; Value: String); export;
 var
  Max: Longint;
 begin
  Max:=jt.linker.ParseNumber(Linker, Value);

  if Max < 0 then
   Max:=0;

  jt.st.Add(st, 'Max', Pointer(Max), nil);
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, 'ForwardRequests', 1);

  fwdreqPlugin:=Plugin;

  fwdreqOrder:=New(PForwardRequestOrder, Init($10, $10));
  fwdreqQueue:=New(PForwardRequestQueue, Init($10, $10));

  restNoAVAIL:=jt.rests.Add('afxNoAVAIL', rsMinus);
  restNoFAVAIL:=jt.rests.Add('afxNoFAVAIL', rsMinus);
  restNoAQUERY:=jt.rests.Add('afxNoAQUERY', rsMinus);
  restUnconditional:=jt.rests.Add('frUnconditional', rsMinus);
  restMaxRequestsBy:=jt.rests.AddValueParser('frMaxRequestsBy', rsValue or rsStorage, fwdreqMaxRequestsBy);
  restMaxRequestsTo:=jt.rests.AddValueParser('frMaxRequestsTo', rsValue or rsStorage, fwdreqMaxRequestsFrom);

  RegisterAreafixCallback(Plugin, aaSubscribeAfter);
  RegisterAreafixCallback(Plugin, aaUnsubscribeAfter);

  jt.EnqueueCommand(Plugin, fwdreqFlush, 0, cwFlush);

  jt.linker.RegisterLinkSectionCallback(Plugin);

  PluginSetup:=True;
 end;

{ PluginStartup }

procedure PluginStartup; export;
 function CheckLink(const fr: PForwardRequests): Boolean; far;
  var
   rc: String;
  begin
   if not rlopIsLinkValid(fr^.Link, rc) then
    begin
     jt.SetError(rc);

     CheckLink:=True;
    end
   else
    CheckLink:=False;
  end;
 var
  K: Longint;
  fr: PForwardRequests;
 begin
  InitAnnouncerAPI;

  RegisterAnnounceType(fwdreqPlugin, 'ForwardRequestNews');

  afxRegisterCommandHandler('AVAIL', afxAvailHandler);
  afxRegisterCommandHandler('FAVAIL', afxFAvailHandler);
  afxRegisterCommandHandler('AQUERY', afxAQueryHandler);

  if fwdreqForceCompile then
   begin
    jt.LogWrite('F', #0'Forced compile of forwardrequest lists');

    for K:=0 to jt.link.Count - 1 do
     GetForwardRequests(jt.link.At(K))^.Compile(True);
   end;

  if fwdreqOrder^.Count = 0 then
   for K:=0 to jt.link.Count - 1 do
    begin
     fr:=GetForwardRequests(jt.link.At(K));

     if fr^.Enabled then
      fwdreqOrder^.Insert(fr);
    end;

  LoadQueue;

  if fwdreqOrder^.FirstThat(@CheckLink) = nil then
   begin
    fwdreqQueue^.CheckExpiredRequests;

    RegisterTosserCallback(fwdreqPlugin, taAutoCreateFirst);
   end;
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
  DisposeTosserCallbacks;

  DisposeAreafixCallbacks;

  jt.rf.Destroy(fwdreqrf);

  Dispose(fwdreqQueue, Done);
  Dispose(fwdreqOrder, Done);

  jt.DestroyAreaFormat(fwdreqDefaultFormat);
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 var
  Linker: Pointer absolute Param;
 begin
  if Command = tcAutoCreateFirst then AutoCreateFirst(PTosserAutoCreateBlock(Param)^) else
  if Command = acSubscribeAfter then SubscribeAfter(PAreafixSubscribeBlock(Param)^) else
  if Command = acUnsubscribeAfter then UnsubscribeAfter(PAreafixUnsubscribeBlock(Param)^) else
  if Command = jtpcLinkSectionEnd then CheckLinkValidity(Pointer(Param)^) else
  if Command = jtpcPluginSectionEnd then
   begin
    if fwdreqrf = nil then
     jt.linker.Error(Linker, 'QUEUE was not defined');

    if fwdreqDefaultFormat = nil then
     jt.linker.Error(Linker, 'DEFAULTFORMAT was not defined');
   end else
  if Command = fwdreqFlush then StoreQueue else
  if Command = fwdreqInfo then Info else
  if Command = AnnouncerProcessFirst then ProcessFirstAnnounce(PProcessAnnounceFirstBlock(Param)^) else
  if Command = AnnouncerProcess then ProcessAnnounce(PProcessAnnounceBlock(Param)^) else
  if Command = AnnouncerProcessLast then ProcessLastAnnounce(PProcessAnnounceLastBlock(Param)^);
 end;

{ PluginHandleConfigSwitch }

procedure PluginHandleConfigSwitch(const Keyword: String; Value: String; const Linker: Pointer); export;
 var
  Address: TAddress;
  Link: Pointer;
  fr: PForwardRequests;
 begin
  if Keyword = 'REQUESTORDER' then
   begin
    jt.core.GetMainAddress(Address);

    if not StrToAddress(jt.linker.ParseWord(Linker, Value), Address) then
     jt.linker.Error(Linker, 'failed to parse address');

    Link:=jt.link.Seek(Address);

    if Link = nil then
     jt.linker.Error(Linker, 'unknown link ' + AddressToStrEx(Address));

    fr:=GetForwardRequests(Link);

    if not fr^.Enabled then
     jt.linker.Error(Linker, 'forwardrequests must be enabled for ' + AddressToStrEx(Address));

    fwdreqOrder^.Insert(fr);
   end else
  if Keyword = 'QUEUE' then
   begin
    if fwdreqrf <> nil then
     jt.linker.Error(Linker, 'redefinition of QUEUE');

    fwdreqrf:=jt.rf.Create(jt.linker.ParseStringToken(Linker, Value), rffForcedPack);

    if fwdreqrf = nil then
     jt.linker.Error(Linker, 'failed to create QUEUE resourcefile');
   end else
  if Keyword = 'DEFAULTTIMEOUT' then
   begin
    fwdreqDefaultTimeout:=jt.linker.ParseNumber(Linker, Value);

    if fwdreqDefaultTimeout < 1 then
     jt.linker.Error(Linker, 'invalid value specified (1 or greater expected)');
   end else
  if Keyword = 'DEFAULTFORMAT' then
   begin
    if fwdreqDefaultFormat <> nil then
     jt.linker.Error(Linker, 'redefinition of DEFAULTFORMAT');

    fwdreqDefaultFormat:=jt.NewAreaFormat;

    fwdreqDefaultFormat^.Style:=asEchomail;

    jt.linker.ParseAreaFormat(Linker, jt.linker.ParseWord(Linker, Value), fwdreqDefaultFormat);

    jt.linker.ParseAreaPath(Linker, Value, fwdreqDefaultFormat);
   end else
  if Keyword = 'MAXREQUESTS' then
   begin
    fwdreqMaxRequests:=jt.linker.ParseNumber(Linker, Value);

    if fwdreqMaxRequests < 0 then
     fwdreqMaxRequests:=0;
   end
  else
   jt.linker.Error(Linker, 'syntax error');
 end;

{ PluginHandleLinkSwitch }

function PluginHandleLinkSwitch(const Link: Pointer; const Keyword: String; Value: String; const Linker: Pointer): Boolean;
 export;
 var
  fr: PForwardRequests;
 begin
  if Copy(Keyword, 1, 14) <> 'FORWARDREQUEST' then
   begin
    PluginHandleLinkSwitch:=False;

    Exit;
   end;

  PluginHandleLinkSwitch:=True;

  fr:=GetForwardRequests(Link);

  if Keyword = 'FORWARDREQUESTS' then fr^.Enabled:=jt.linker.ParseBoolean(Linker, Value) else
  if Keyword = 'FORWARDREQUESTLIST' then fr^.ParseList(Linker, Value) else
  if Keyword = 'FORWARDREQUESTTIMEOUT' then
   begin
    fr^.Timeout:=jt.linker.ParseNumber(Linker, Value);

    if fr^.Timeout < 1 then
     jt.linker.Error(Linker, 'invalid value (1 or greater expected)');
   end else
  if Keyword = 'FORWARDREQUESTFORMAT' then fr^.ParseFormat(Linker, Value) else
   PluginHandleLinkSwitch:=False;
 end;

{ PluginExportLinkSwitches }

procedure PluginExportLinkSwitches(const Link, Linker: Pointer); export;
 begin
  GetForwardRequests(Link)^.ExportLink(Linker);
 end;

{ PluginHandleCmdlineSwitch }

function PluginHandleCmdlineSwitch(Switch: String): Boolean; export;
 begin
  StUpcaseEx(Switch);

  PluginHandleCmdlineSwitch:=True;

  if Switch = 'FORCEFWDREQCOMPILE' then fwdreqForceCompile:=True else
  if Switch = 'FWDREQINFO' then jt.EnqueueCommand(fwdreqPlugin, fwdreqInfo, 0, cwEarliest) else
   PluginHandleCmdlineSwitch:=False;
 end;

{ PluginShowHelp }

procedure PluginShowHelp; export;
 begin
  jt.PrintStr(' FORCEFWDREQCOMPILE      force fwdreq lists recompilation'#13#10);
  jt.PrintStr(' FWDREQINFO              show current forward request area pendings'#13#10);
 end;

{ PluginExportLinksHeader }

procedure PluginExportLinksHeader(const Linker: Pointer); export;
 begin
  jt.linker.ExportLine(Linker, '; ForwardRequests switches:');
  jt.linker.ExportLine(Linker, ';   ForwardRequests Yes|No');
  jt.linker.ExportLine(Linker, ';   ForwardRequestList Echolist "fname"');
  jt.linker.ExportLine(Linker, ';   ForwardRequestList AreaDesc "fname"');
  jt.linker.ExportLine(Linker, ';   ForwardRequestList BCL');
  jt.linker.ExportLine(Linker, ';   ForwardRequestTimeout days');
  jt.linker.ExportLine(Linker, ';   ForwardRequestFormat Passthrough "path"');
  jt.linker.ExportLine(Linker, ';');
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleConfigSwitch,
 PluginHandleLinkSwitch,
 PluginExportLinkSwitches,
 PluginHandleCmdlineSwitch,
 PluginShowHelp,
 PluginExportLinksHeader;

begin
 Assign(Output, '');
 Rewrite(Output);
end.
