library H;

uses
     PlugAPI,
     PlugSIDE,

     Objects,
     Strings,

     ScanAPI,

     Thunks;

const
 SearchFor      = #141; { // russian H }
 ReplaceTo      = #72;  { // english H }

{ ProcessStream }

procedure ProcessStream(const Stream: PStream);
 const
  BufferSize = $1000;
 var
  Buffer, BufferAnchor, H: PChar;
  Size, Anchor: Longint;
  Replaced: Boolean;
 begin
  GetMem(Buffer, BufferSize + 1);

  Stream^.Seek(0);

  repeat
   Anchor:=Stream^.GetPos;

   Size:=Stream^.GetSize - Anchor;

   if Size > BufferSize then
    Size:=BufferSize;

   if Size > 0 then
    begin
     Stream^.Read(Buffer[0], Size);

     Buffer[Size]:=#0;

     BufferAnchor:=Buffer;

     Replaced:=False;

     repeat
      H:=StrScan(BufferAnchor, SearchFor);

      if H <> nil then
       begin
        H[0]:=ReplaceTo;

        BufferAnchor:=H + 1;

        Replaced:=True;
       end;
     until H = nil;

     if Replaced then
      begin
       Stream^.Seek(Anchor);

       Stream^.Write(Buffer[0], Size);

       Stream^.Seek(Anchor + Size);
      end;
    end;
  until Size = 0;

  FreeMem(Buffer, BufferSize + 1);
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, 'H', 1);

  PluginSetup:=True;

  RegisterScannerCallback(Plugin, saMessageScanGeneral);
 end;

{ ScanGeneral }

procedure ScanGeneral(const Block: TScanMessageBlock);
 var
  Stream: PStreamThunk;
 begin
  Stream:=New(PStreamThunk, Init(jt.msg.GetText(Block.msg), jt.StreamLinker, False));

  ProcessStream(Stream);

  Dispose(Stream, Done);
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
  DisposeScannerCallbacks;
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 begin
  if Command = scMessageScanGeneral then ScanGeneral(PScanMessageBlock(Param)^);
 end;

exports
 PluginSetup,
 PluginShutdown,
 PluginHandleCommand;

begin
end.