unit jtmAConv;

interface
uses
     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     Objects,
     Drivers,

     Views,
     App,
     Dialogs,
     MsgBox,

     Wizard,

     acrAPI,

     jtmCmn,
     jtmHelp,
     jtmUtil,
     jtmTV,

     jtMHLcmn;

const
 cmConvertOnly                  = $0000;
 cmConvertAndMove               = $0001;

 coNoErrorMessages              = $0001;
 coUniqueAll                    = $0002;
 coUseACR                       = $0004;

 cmUnique                       = $2000;
 cmUniqueAll                    = $2001;

type
 PAreaConverterRadioButtons = ^TAreaConverterRadioButtons;
 TAreaConverterRadioButtons = object(TRadioButtons)
  procedure Press(Item: xInteger); virtual;
  procedure MovedTo(Item: xInteger); virtual;
 end;

 PAreaConvertProgress = ^TAreaConvertProgress;

 PAreaConverter = ^TAreaConverter;
 TAreaConverter = object(TtuiDialog)
  Selected: PAreasSelected;
  Format: PAreaFormats;
  Mode: PAreaConverterRadioButtons;
  Path: PtuiInputLine;
  Setup: PCheckBoxes;
  Errors: PStrings;

  constructor Init(const ASelected: PAreasSelected);
  procedure HandleEvent(var Event: TEvent); virtual;
  function Valid(Command: xWord): Boolean; virtual;
  procedure EndModal(Command: xWord); virtual;
  destructor Done; virtual;
 public
  function PrepareAreaFormat(const Area: Pointer; var AreaFormat: TAreaFormat): Boolean;
 end;

 TAreaConvertProgress = object(TtuiDialog)
  Area: Pointer;
  Converter: PAreaConverter;
  Progress: PProgress;

  constructor Init(const AArea: Pointer; const AConverter: PAreaConverter);
  procedure EndModal(Command: xWord); virtual;
 public
  procedure Error(const AArea: Pointer; const Message: String; var Command: xWord);
 end;

 PAreaConvertPathProblem = ^TAreaConvertPathProblem;
 TAreaConvertPathProblem = object(TtuiDialog)
  Area: Pointer;
  Format: PAreaFormat;
  Path: PtuiInputLine;
  UniqueAll: Boolean;

  constructor Init(const AArea: Pointer; const AFormat: PAreaFormat);
  procedure HandleEvent(var Event: TEvent); virtual;
  function Valid(Command: xWord): Boolean; virtual;
  procedure EndModal(Command: xWord); virtual;
 end;

 PAreaConvertResultsBox = ^TAreaConvertResultsBox;
 TAreaConvertResultsBox = object(TListViewer)
  List: PStrings;

  function GetText(Item: xInteger; MaxLen: xInteger): String; virtual;
 end;

 PAreaConvertResults = ^TAreaConvertResults;
 TAreaConvertResults = object(TtuiDialog)
  constructor Init(const List: PStrings);
 end;

implementation

const
 cmModeChanged                  = $3000;

{ TAreaConverterRadioButtons }

procedure TAreaConverterRadioButtons.Press(Item: xInteger);
 begin
  inherited Press(Item);

  Message(Owner, evCommand, cmModeChanged, nil);
 end;

procedure TAreaConverterRadioButtons.MovedTo(Item: xInteger);
 begin
  inherited MovedTo(Item);

  Message(Owner, evCommand, cmModeChanged, nil);
 end;

{ TAreaConverter }

constructor TAreaConverter.Init(const ASelected: PAreasSelected);
 var
  Bounds: TRect;
 begin
  Bounds.Assign(0, 0, 48, 15);

  inherited Init(Bounds, 'Convert');

  HelpCtx:=hcAreaConverter;

  Options:=Options or ofCentered;

  Selected:=ASelected;

  Errors:=New(PStrings, Init($10, $10));

  { // header }

  Bounds.Assign(2, 2, Size.X - 2, 3);

  Insert(New(PStaticText, Init(Bounds, ^C + Long2Str(Selected^.Count) + ' area' + _s(Selected^.Count) + ' to convert')));

  { // format }

  Bounds.Assign(2, 5, 22, 10);

  Format:=New(PAreaFormats, Init(Bounds, @Self));

  if Selected^.Count = 1 then
   Format^.SetSingle(Selected^.At(0))
  else
   Format^.SetDefault;

  Insert(Format);

  Bounds.Assign(1, 4, 22, 5);

  Insert(New(PLabel, Init(Bounds, 'Target ~f~ormat:', Format)));

  { // mode }

  Bounds.Assign(24, 5, 46, 7);

  Mode:=New(PAreaConverterRadioButtons, Init(Bounds,
   NewSItem('convert ~o~nly',
   NewSItem('convert and ~m~ove',
    nil))));

  Insert(Mode);

  Bounds.Assign(23, 4, 46, 5);

  Insert(New(PLabel, Init(Bounds, 'Con~v~ert mode:', Mode)));

  { // mode }

  Bounds.Assign(24, 9, 46, 10);

  Path:=New(PtuiInputLine, Init(Bounds, 255));

  Insert(Path);

  Bounds.Assign(23, 8, 46, 9);

  Insert(New(PLabel, Init(Bounds, 'Target ~p~ath:', Path)));

  Path^.SetState(sfDisabled, True);

  { // setup }

  Bounds.Assign(2, Size.Y - 4, 22, Size.Y - 1);

  Setup:=New(PCheckBoxes, Init(Bounds,
   NewSItem('no error msgs',
   NewSItem('unique all',
   NewSItem('use ACR',
   nil)))));

  if not acrAvailable then
   Setup^.EnableMask:=Setup^.EnableMask and not coUseACR;

  Insert(Setup);

  { // buttons }

  Bounds.Assign(0, 0, 12, 2);
  Bounds.Move(Size.X - Bounds.B.X * 2 - 2, Size.Y - 3);

  Insert(New(PButton, Init(Bounds, 'O~K~', cmOk, bfDefault)));

  Bounds.Move(Bounds.B.X - Bounds.A.X, 0);

  Insert(New(PButton, Init(Bounds, 'Cancel', cmCancel, bfNormal)));

  SelectNext(False);
 end;

procedure TAreaConverter.HandleEvent(var Event: TEvent);
 begin
  if Event.What = evCommand then
   repeat
    case Event.Command of
     cmModeChanged: Path^.SetState(sfDisabled, Mode^.Value = 0);
    else
     Break;
    end;

    ClearEvent(Event);
   until True;

  inherited HandleEvent(Event);
 end;

function TAreaConverter.Valid(Command: xWord): Boolean;
 procedure Error(const FocusControl: PView; const Message: String);
  begin
   FocusControl^.Focus;

   MessageBox(^C + Message, nil, mfError or mfCancelButton);

   Valid:=False;
  end;
 begin
  if Command = cmOk then
   if (Mode^.Value = cmConvertAndMove) and (Path^.Data^ = '') then
    Error(Path, 'Path must be defined for convert and move.')
   else
    Valid:=inherited Valid(Command)
  else
   Valid:=inherited Valid(Command);
 end;

procedure TAreaConverter.EndModal(Command: xWord);
 function Perform(const Area: Pointer): Boolean; far;
  var
   Dialog: PAreaConvertProgress;
  begin
   Dialog:=New(PAreaConvertProgress, Init(Area, @Self));

   Perform:=Desktop^.ExecView(Dialog) <> cmYes;

   Dispose(Dialog, Done);
  end;
 var
  Dialog: PAreaConvertResults;
 begin
  if Command = cmOk then
   begin
    Errors^.FreeAll;

    Selected^.FirstThat(@Perform);

    if Errors^.Count <> 0 then
     begin
      Dialog:=New(PAreaConvertResults, Init(Errors));

      Desktop^.ExecView(Dialog);

      Dispose(Dialog, Done);
     end
    else
     inherited EndModal(Command);
   end
  else
   inherited EndModal(Command);
 end;

destructor TAreaConverter.Done;
 begin
  Dispose(Errors, Done);

  inherited Done;
 end;

{ public methods }

function TAreaConverter.PrepareAreaFormat(const Area: Pointer; var AreaFormat: TAreaFormat): Boolean;
 var
  TargetFormat: Longint;
  FullName, ShortName: String;
  Dialog: PAreaConvertPathProblem;
  Uplink: TAddress;
  Event: TEvent;
  Ok: Boolean;
 begin
  if not jt.core.AtFormat(Format^.Focused, TargetFormat, FullName, ShortName) then
   begin
    MessageBox(^C'Brain is dead.', nil, mfError or mfCancelButton);

    PrepareAreaFormat:=False;

    Exit;
   end;

  AreaFormat.Format:=TargetFormat;

  if Mode^.Value = cmConvertAndMove then
   if (Setup^.Value and Setup^.EnableMask) and coUseACR <> 0 then
    begin
     AreaFormat.Path:=AddBackSlash(Path^.Data^);

     jt.area.GetUplink(Area, Uplink);

     acrPrepareAreaFormat(AreaFormat, jt.area.GetName(Area)^, @Uplink);
    end
   else
    AreaFormat.Path:=AddBackSlash(Path^.Data^) + JustFileName(RemoveBackSlash(AreaFormat.Path));

  if jt.area.IsBaseExists(AreaFormat) then
   begin
    Dialog:=New(PAreaConvertPathProblem, Init(Area, @AreaFormat));

    if Setup^.Value and coUniqueAll <> 0 then
     begin
      Event.What:=evCommand;
      Event.Command:=cmUniqueAll;

      Application^.PutEvent(Event);

      Desktop^.Lock;
     end;

    Ok:=Desktop^.ExecView(Dialog) = cmOk;

    if Setup^.Value and coUniqueAll <> 0 then
     Desktop^.Unlock;

    if Dialog^.UniqueAll then
     begin
      Setup^.Value:=Setup^.Value or coUniqueAll;

      Setup^.DrawView;
     end;

    Dispose(Dialog, Done);
   end
  else
   Ok:=True;

  if Ok then
   jt.LogWrite('C', 'Converting ' + jt.area.GetName(Area)^ + ' to ' + FullName + ', "' + AreaFormat.Path + '"');

  PrepareAreaFormat:=Ok;
 end;

{ TAreaConvertProgress }

constructor TAreaConvertProgress.Init(const AArea: Pointer; const AConverter: PAreaConverter);
 var
  Bounds: TRect;
  Event: TEvent;
  Control: PView;
 begin
  Bounds.Assign(0, 0, 50, 9);

  inherited Init(Bounds, 'Progress');

  Area:=AArea;
  Converter:=AConverter;

  HelpCtx:=hcAreaConvertProgress;

  Options:=Options or ofCentered;

  Bounds.Assign(2, 2, Size.X - 2, 3);

  Insert(New(PStaticText, Init(Bounds, ^C'Converting ' + jt.area.GetName(AArea)^)));

  Bounds.Move(0, 2);

  Progress:=New(PProgress, Init(Bounds, 0, 1));

  Progress^.SetState(sfVisible, False);

  Insert(Progress);

  { // button }

  Bounds.Assign(0, 0, 12, 2);
  Bounds.Move(0, Size.Y - 3);

  Control:=New(PButton, Init(Bounds, 'Cancel', cmCancel, bfNormal));

  Control^.Options:=Control^.Options or ofCenterX;

  Insert(Control);

  Event.What:=evCommand;
  Event.Command:=cmOk;

  Application^.PutEvent(Event);
 end;

procedure TAreaConvertProgress.EndModal(Command: xWord);
 var
  Format: TAreaFormat;
  TempAreaName, rc: String;
  TempArea, mbSource, mbTarget, msg, st: Pointer;
  Locked: Boolean;
  Event: TEvent;
  No: Longint;
 procedure Cleanup(const Command: Word);
  begin
   if mbSource <> nil then
    begin
     jt.SetStatus('closing source base');

     if Locked then
      jt.mb.Unlock(mbSource);

     jt.area.Unlock(Area, mbSource);
    end;

   jt.SetStatus('flushing internal JT buffers');

   jt.area.MinimizeHandles;

   if TempArea <> nil then
    begin
     jt.SetStatus('destroying temp area');

     if jt.area.Kill(TempArea, rc) then
      jt.area.killed.Destroy(TempArea)
     else
      jt.LogWrite('?', 'jtm/conv/c_ok: failed to kill temparea ' + jt.area.GetName(TempArea)^ + ': ' + rc);
    end;

   jt.ClearStatus;

   inherited EndModal(Command);
  end;
 procedure SwapFormats(const Area1, Area2: Pointer);
  var
   Format1: TAreaFormat;
   Format2: TAreaFormat;
  begin
   jt.area.GetFormat(Area1, Format1);
   jt.area.GetFormat(Area2, Format2);

   jt.area.SetFormat(Area1, Format2);
   jt.area.SetFormat(Area2, Format1);
  end;
 function Cancelled(var Event: TEvent): Boolean;
  begin
   Cancelled:=(((Event.What = evCommand) and (Event.Command = cmCancel)) or
              ((Event.What = evKeyDown) and ((Event.KeyCode = kbEsc) or
                                             (Event.KeyCode = kbSpace) or
                                             (Event.KeyCode = kbEnter)))) and
              (MessageBox(^C'Are you sure to stop converting?', nil, mfConfirmation or mfYesButton or mfNoButton) = cmYes);
  end;
 begin
  TempArea:=nil;
  mbSource:=nil;
  Locked:=False;

  Message(Application, evCommand, cmSetTitle, NewStr('converting ' + jt.area.GetName(Area)^));

  { // opening & locking source base }

  jt.SetStatus('opening ' + jt.area.GetName(Area)^);

  if not jt.area.Lock(Area, mbSource, rc) then
   begin
    Error(Area, 'failed to open source area: ' + rc, Command);

    Cleanup(Command);

    Exit;
   end;

  jt.SetStatus('locking ' + jt.area.GetName(Area)^);

  if not jt.mb.Lock(mbSource) then
   begin
    Error(Area, 'failed to lock area', Command);

    Cleanup(Command);

    Exit;
   end;

  Locked:=True;

  { // preparing target area format }

  jt.SetStatus('preparing target area format');

  jt.area.GetFormat(Area, Format);

  if not Converter^.PrepareAreaFormat(Area, Format) then
   begin
    Cleanup(cmCancel);

    Exit;
   end;

  { // creating temp fictive area }

  jt.SetStatus('calculating target area name');

  repeat
   TempAreaName:='JTM.CONVERT.' + GenerateMSGID + '.' + jt.area.GetName(Area)^;

   StUpcaseEx(TempAreaName);
  until jt.area.Seek(TempAreaName) = nil;

  jt.SetStatus('creating ' + TempAreaName);

  TempArea:=jt.area.Create(TempAreaName, @Format);

  { // try to open target }

  jt.SetStatus('trying to open target base');

  if not jt.area.Lock(TempArea, mbTarget, rc) then
   begin
    Error(Area, 'failed to open target area: ' + rc, Command);

    Cleanup(Command);

    Exit;
   end;

  jt.area.Unlock(TempArea, mbTarget);

  { // prepare progress }

  Progress^.SetState(sfVisible, True);

  Progress^.Max:=jt.mb.Count(mbSource) - 1;

  { // check, if any }

  GetEvent(Event);

  if Cancelled(Event) then
   begin
    Error(Area, 'aborted by used', Command);

    Cleanup(Command);

    Exit;
   end;

  { // convert }

  for No:=0 to jt.mb.Count(mbSource) - 1 do
   begin
    Progress^.SetPosition(No);

    GetEvent(Event);

    if Cancelled(Event) then
     begin
      Error(Area, 'aborted by used', Command);

      Cleanup(Command);

      Exit;
     end;

    if jt.mb.Load(mbSource, No, msg, rc, mfNone) then
     begin
      st:=jt.msg.st(msg);

      jt.st.Add(st, 'Source', Pointer(tsBaseInbound), nil);

      jt.st.Add(st, 'Don''tAddToImportList', Pointer(1), nil);

      jt.msg.SetArea(msg, TempAreaName);

      jt.tossscan.TossMessage(msg);
     end
    else
     begin
      Error(Area, 'failed to load msg ' + Long2Str(No) + ': ' + rc, Command);

      Cleanup(Command);

      Exit;
     end;
   end;

  { // closing source base }

  jt.SetStatus('closing source base');

  jt.area.Unlock(Area, mbSource);

  { // swap source & target formats }

  jt.SetStatus('final cooks');

  SwapFormats(Area, TempArea);

  { // flush internal JT buffers ;)) }

  jt.SetStatus('flushing internal JT buffers');

  jt.area.MinimizeHandles;

  { // destroying temp fictive area }

  jt.SetStatus('destroying temp area');

  jt.st.Add(jt.area.st(TempArea), 'NoDestroyNotify', Pointer(1), nil);

  if jt.area.Kill(TempArea, rc) then
   jt.area.killed.Destroy(TempArea)
  else
   jt.LogWrite('?', 'jtm/conv/c_ok: failed to kill temparea ' + jt.area.GetName(TempArea)^ + ': ' + rc);

  jt.area.SetChanged(acfFull, 'jtm/converted');

  { // to avoid strange problems ;)) }

  jt.ClearStatus;

  inherited EndModal(cmYes);
 end;

{ public methods }

procedure TAreaConvertProgress.Error(const AArea: Pointer; const Message: String; var Command: xWord);
 begin
  Converter^.Errors^.Insert(NewStr(jt.area.GetName(AArea)^ + ': ' + Message));

  if Converter^.Setup^.Value and coNoErrorMessages <> 0 then
   Command:=cmYes
  else
   begin
    Command:=MessageBox(^C + jt.area.GetName(AArea)^ + ': ' + Message + '. Continue converting?', nil,
     mfWarning or mfYesButton or mfNoButton);

    if (Command <> cmYes) and (Command <> cmNo) then
     Command:=cmYes;
   end;
 end;

{ TAreaConvertPathProblem }

constructor TAreaConvertPathProblem.Init(const AArea: Pointer; const AFormat: PAreaFormat);
 var
  Bounds: TRect;
 begin
  Bounds.Assign(0, 0, 59, 9);

  inherited Init(Bounds, 'Converting ' + jt.area.GetName(AArea)^ + ': path problem');

  HelpCtx:=hcAreaConvertPathProblem;

  Area:=AArea;
  Format:=AFormat;

  Options:=Options or ofCentered;

  Bounds.Assign(2, 2, Size.X - 2, 3);

  Insert(New(PStaticText, Init(Bounds, ^C'Base with same path already exists.')));

  Bounds.Assign(8, 4, Size.X - 2, 5);

  Path:=New(PtuiInputLine, Init(Bounds, 255));

  Path^.Data^:=Format^.Path;

  Insert(Path);

  Bounds.Assign(1, 4, 8, 5);

  Insert(New(PLabel, Init(Bounds, '~P~ath:', Path)));

  Bounds.Assign(0, 0, 14, 2);

  Bounds.Move(Size.X - 2 - Bounds.B.X * 4, Size.Y - 3);

  Insert(New(PButton, Init(Bounds, '~U~nique', cmUnique, bfNormal)));

  Bounds.Move(Bounds.B.X - Bounds.A.X, 0);
  Insert(New(PButton, Init(Bounds, 'Unique ~a~ll', cmUniqueAll, bfNormal)));

  Bounds.Move(Bounds.B.X - Bounds.A.X, 0);
  Insert(New(PButton, Init(Bounds, '~O~K', cmOk, bfDefault)));

  Bounds.Move(Bounds.B.X - Bounds.A.X, 0);
  Insert(New(PButton, Init(Bounds, 'Cancel', cmCancel, bfNormal)));

  SelectNext(False);
 end;

procedure TAreaConvertPathProblem.HandleEvent(var Event: TEvent);
 begin
  if Event.What = evCommand then
   repeat
    case Event.Command of
     cmUnique, cmUniqueAll: EndModal(Event.Command);
    else
     Break;
    end;

    ClearEvent(Event);
   until True;

  inherited HandleEvent(Event);
 end;

function TAreaConvertPathProblem.Valid(Command: xWord): Boolean;
 begin
  if Command = cmOk then
   begin
    Format^.Path:=Path^.Data^;

    if jt.area.IsBaseExists(Format^) then
     begin
      Path^.Focus;

      MessageBox(^C + 'Base with the same path already exists.', nil, mfError or mfCancelButton);

      Valid:=False;
     end
    else
     Valid:=inherited Valid(Command);
   end
  else
   Valid:=inherited Valid(Command);
 end;

procedure TAreaConvertPathProblem.EndModal(Command: xWord);
 var
  UniqueAllEnable: Boolean;
 begin
  if (Command = cmUnique) or (Command = cmUniqueAll) then
   begin
    repeat
     Path^.Data^:=AddBackSlash(JustPathName(RemoveBackSlash(Path^.Data^))) + GenerateMSGID;

     Format^.Path:=Path^.Data^;
    until not jt.area.IsBaseExists(Format^);

    UniqueAllEnable:=Command = cmUniqueAll;

    Command:=cmOk;
   end
  else
   UniqueAllEnable:=False;

  if Command <> cmOk then
   inherited EndModal(Command)
  else
   if Valid(cmOk) then
    begin
     if UniqueAllEnable then
      UniqueAll:=True;

     inherited EndModal(cmOk);
    end;
 end;

{ TAreaConvertResultsBox }

function TAreaConvertResultsBox.GetText(Item: xInteger; MaxLen: xInteger): String;
 begin
  GetText:=GetPString(List^.At(Item));
 end;

{ TAreaConvertResults }

constructor TAreaConvertResults.Init(const List: PStrings);
 var
  MaxWidth: Byte;
 procedure CountMaxWidth(const S: PString); far;
  begin
   if S <> nil then
    if Length(S^) > MaxWidth then
     MaxWidth:=Length(S^);
  end;
 var
  Bounds: TRect;
  HSB, VSB: PScrollBar;
  Box: PAreaConvertResultsBox;
 begin
  Bounds.Assign(0, 0, 65, 14);

  inherited Init(Bounds, 'Convert results');

  Options:=Options or ofCentered;

  HelpCtx:=hcAreaConvertResults;

  { // max width }

  HSB:=StandardScrollBar(sbHorizontal or sbHandleKeyboard);

  MaxWidth:=1;

  List^.ForEach(@CountMaxWidth);

  HSB^.SetRange(1, MaxWidth);

  { // vertical scroll bar }

  Bounds.Assign(Size.X - 1, 4, Size.X, Size.Y - 1);

  VSB:=New(PScrollBar, Init(Bounds));

  Insert(VSB);

  { // box }

  Bounds.Assign(1, 4, Size.X - 1, Size.Y - 1);

  Box:=New(PAreaConvertResultsBox, Init(Bounds, 1, HSB, VSB));

  Box^.List:=List;

  Box^.SetRange(List^.Count);

  Insert(Box);

  { // close button }

  Bounds.Assign(Size.X - 15, 2, Size.X - 2, 4);

  Insert(New(PButton, Init(Bounds, 'Close', cmOk, bfDefault)));

  { // misc stuff }

  Bounds.Assign(3, 2, Size.X - 16, 3);

  Insert(New(PStaticText, Init(Bounds, 'The following errors occured while converting:')));

  SelectNext(False);
 end;

end.
