unit jtmAEdit;

interface
uses
     PlugAPI,
     PlugSIDE,

     Objects,
     Drivers,

     Dialogs,
     Menus,
     Views,
     App,
     MsgBox,

     jtmCmn,
     jtmUtil,
     jtmHelp,
     jtmTV,

     jtmAMisc,

     Wizard;

type
 PAreaEditor = ^TAreaEditor;
 TAreaEditor = object(TtuiDialog)
  Selected: PAreasSelected;
  Creation: Boolean;
  Mode: (aemEditSingle, aemEditMultiple);
  MiscMenu: PMenu;

  constructor Init(const ASelected: PAreasSelected; const ACreation: Boolean; const AMiscMenu: PMenu);
  procedure HandleEvent(var Event: TEvent); virtual;
  function Valid(Command: xWord): Boolean; virtual;
  procedure EndModal(Command: xWord); virtual;
 public
  Area: Pointer;
  AreaName: PtuiInputLine;
  AreaDesc: PtuiInputLine;
  AreaStyle: PAreaStyles;
  AreaFormat: PAreaFormats;
  AreaPath: PtuiInputLine;
  AreaSubscribers: PAreaSubscribers;
  MiscButton: PButton;

  procedure MiscPopup;
 end;

implementation

{ TAreaEditor }

constructor TAreaEditor.Init(const ASelected: PAreasSelected; const ACreation: Boolean; const AMiscMenu: PMenu);
 var
  Bounds: TRect;
  Format: TAreaFormat;
  Base: Integer;
  S: String;
 begin
  Bounds.Assign(0, 0, 80, 23);

  if ASelected^.Count = 0 then
   inherited Init(Bounds, 'Create area')
  else
   inherited Init(Bounds, 'Edit area' + _s(ASelected^.Count));

  Creation:=ACreation;

  Selected:=ASelected;

  MiscMenu:=AMiscMenu;

  if ASelected^.Count = 1 then
   begin
    Mode:=aemEditSingle;

    Area:=ASelected^.At(0);
   end
  else
   Mode:=aemEditMultiple;

  HelpCtx:=hcEditAreaDialog;

  Options:=Options or ofCentered;

  jt.SetStatus('loading settings');

  { // area name }

  case Mode of
   aemEditSingle:
    begin
     Bounds.Assign(2, 2, Size.X - 2, 3);

     AreaName:=New(PtuiInputLine, Init(Bounds, 255));

     Insert(AreaName);

     Bounds.Move(-1, -1);

     Insert(New(PLabel, Init(Bounds, '~N~ame:', AreaName)));

     AreaName^.Data^:=jt.area.GetName(Area)^;
    end;
   aemEditMultiple:
    begin
     S:=Long2Str(Selected^.Count) + ' area' + _s(Selected^.Count) + ' selected';

     Bounds.Assign(Size.X - 2 - Length(S), 2, Size.X - 2, 3);

     Insert(New(PStaticText, Init(Bounds, S)));
    end;
  end;

  { // area desc }

  case Mode of
   aemEditSingle:
    begin
     Bounds.Assign(2, 5, Size.X - 2, 6);

     AreaDesc:=New(PtuiInputLine, Init(Bounds, 255));

     Insert(AreaDesc);

     Bounds.Move(-1, -1);

     Insert(New(PLabel, Init(Bounds, '~D~escription:', AreaDesc)));

     GetPStringEx(jt.area.GetDesc(Area), AreaDesc^.Data^);

     Base:=8;
    end;
   aemEditMultiple:
    Base:=2;
  else
   Base:=5;
  end;

  { // area style }

  Bounds.Assign(0, 0, 20, 3);
  Bounds.Move(2, Base);

  AreaStyle:=New(PAreaStyles, Init(Bounds, @Self));

  case Mode of
   aemEditSingle: AreaStyle^.SetSingle(Area);
   aemEditMultiple: AreaStyle^.SetMultiple(Selected)
  end;

  Bounds.B.Y:=Bounds.A.Y + 1;

  Bounds.Move(-1, -1);

  Insert(New(PLabel, Init(Bounds, '~S~tyle:', AreaStyle)));

  { // area format }

  Bounds.Assign(0, 0, 20, 3);
  Bounds.Move(24, Base);

  AreaFormat:=New(PAreaFormats, Init(Bounds, @Self));

  case Mode of
   aemEditSingle: AreaFormat^.SetSingle(Area);
   aemEditMultiple: AreaFormat^.SetMultiple(Selected)
  end;

  Bounds.B.Y:=Bounds.A.Y + 1;

  Bounds.Move(-1, -1);

  Insert(New(PLabel, Init(Bounds, '~F~ormat:', AreaFormat)));

  { // area path }

  if Mode = aemEditSingle then
   begin
    Bounds.Assign(46, Base, Size.X - 2, Base + 1);

    AreaPath:=New(PtuiInputLine, Init(Bounds, 255));

    Insert(AreaPath);

    Bounds.Move(-1, -1);

    Insert(New(PLabel, Init(Bounds, '~P~ath:', AreaPath)));

    jt.area.GetFormat(Area, Format);

    AreaPath^.Data^:=Format.Path;
   end;

  jt.SetStatus('loading subscribe');

  { // subscribers }

  Bounds.Assign(1, Base + 5, Size.X - 1, Size.Y - 1);

  AreaSubscribers:=New(PAreaSubscribers, Init(Bounds, @Self));

  AreaSubscribers^.Options:=AreaSubscribers^.Options or ofFramed;

  AreaSubscribers^.HelpCtx:=hcEditAreaDialogSubscribers;

  Bounds.Assign(2, Base + 4, 16, Base + 5);

  Insert(New(PLabel, Init(Bounds, 'S~u~bscribers:', AreaSubscribers)));

  case Mode of
   aemEditSingle: AreaSubscribers^.SetSingle(Area);
   aemEditMultiple: AreaSubscribers^.SetMultiple(Selected)
  end;

  jt.ClearStatus;

  { // buttons }

  Bounds.Assign(0, 0, 10, 2);
  Bounds.Move(48, Base + 2);

  MiscButton:=New(PButton, Init(Bounds, '~M~isc', cmaeMisc, bfNormal));

  Insert(MiscButton);

  Bounds.Move(Bounds.B.X - Bounds.A.X, 0);
  Insert(New(PButton, Init(Bounds, 'O~K', cmOk, bfDefault)));

  Bounds.Move(Bounds.B.X - Bounds.A.X, 0);
  Insert(New(PButton, Init(Bounds, 'Cancel', cmCancel, bfNormal)));

  SelectNext(False);
 end;

procedure TAreaEditor.HandleEvent(var Event: TEvent);
 begin
  if Event.What = evCommand then
   repeat
    case Event.Command of
     cmaeMisc: MiscPopup;
     cmamMiscStart..cmamMiscEnd: ExecuteMiscCommand(Event.Command, Selected, mmAreasEditor);
    else
     Break;
    end;

    ClearEvent(Event);
   until True;

  inherited HandleEvent(Event);
 end;

function TAreaEditor.Valid(Command: xWord): Boolean;
 var
  Ok: Boolean;
 procedure Error(const FocusControl: PView; const Message: String);
  begin
   if FocusControl <> nil then
    FocusControl^.Focus;

   MessageBox(^C + Message, nil, mfError or mfCancelButton);

   Ok:=False;
  end;
 var
  AArea: Pointer;
 begin
  if Command = cmOk then
   begin
    Ok:=True;

    if Mode = aemEditSingle then
     if AreaName^.Data^ = '' then
      Error(AreaName, 'Area name cannot be empty.')
     else
      begin
       AArea:=jt.area.Seek(StUpcase(AreaName^.Data^));

       if (AArea <> nil) and (AArea <> Area) then
        Error(AreaName, 'Area with the same name already exists.');
      end;

    Valid:=Ok and inherited Valid(Command);
   end
  else
   Valid:=inherited Valid(Command);
 end;

procedure TAreaEditor.EndModal(Command: xWord);
 var
  Format: TAreaFormat;
  Killed: Pointer;
 begin
  if Command = cmOk then
   begin
    if not Valid(Command) then
     Exit;

    { // AreaName }

    jt.SetStatus('applying changes');

    case Mode of
     aemEditSingle:
      begin
       jt.area.killed.Create(jt.area.GetName(Area)^);

       jt.area.SetName(Area, StUpcase(AreaName^.Data^));

       Killed:=jt.area.killed.Seek(AreaName^.Data^);

       if Killed <> nil then
        jt.area.killed.Destroy(Killed);
      end;
    end;

    { // AreaDesc }

    case Mode of
     aemEditSingle: jt.area.SetDesc(Area, AreaDesc^.Data^);
    end;

    { // AreaStyle }

    case Mode of
     aemEditSingle: AreaStyle^.GetSingle(Area);
     aemEditMultiple: AreaStyle^.GetMultiple(Selected);
    end;

    { // AreaFormat }

    case Mode of
     aemEditSingle: AreaFormat^.GetSingle(Area);
     aemEditMultiple: AreaFormat^.GetMultiple(Selected);
    end;

    { // AreaPath }

    case Mode of
     aemEditSingle:
      begin
       jt.area.GetFormat(Area, Format);

       Format.Path:=AreaPath^.Data^;

       jt.area.SetFormat(Area, Format);
      end;
    end;

    { // AreaSubscribes }

    jt.SetStatus('applying subscribe');

    case Mode of
     aemEditSingle: AreaSubscribers^.GetSingle(Area);
     aemEditMultiple: AreaSubscribers^.GetMultiple(Selected);
    end;

    jt.ClearStatus;

    { // wink to kernel }

    jt.area.SetChanged(acfFull, 'jtm/aeditor');
    jt.link.SetChanged(acfFull, 'jtm/aeditor');
   end;

  inherited EndModal(Command);
 end;

{ public methods }

procedure TAreaEditor.MiscPopup;
 var
  PopupCoords: TPoint;
  MenuPopup: PMenuPopup;
  Bounds: TRect;
  Event: TEvent;
 begin
  PopupCoords:=MiscButton^.Origin;

  Dec(PopupCoords.Y);
  Dec(PopupCoords.X);

  MakeGlobal(PopupCoords, PopupCoords);

  Bounds.Assign(0, 0, 40, 20);

  MenuPopup:=New(PtuiMenuPopup, Init(Bounds, MiscMenu));

  MenuPopup^.GetBounds(Bounds);

  Bounds.Move(PopupCoords.X, PopupCoords.Y);

  while Bounds.B.X > Application^.Size.X do Bounds.Move(-1, 0);
  while Bounds.B.Y > Application^.Size.Y do Bounds.Move(0, -1);
  while Bounds.A.X < 0 do Bounds.Move(1, 0);
  while Bounds.A.Y < 0 do Bounds.Move(0, 1);

  MenuPopup^.Locate(Bounds);

  Event.What:=evCommand;
  Event.Command:=Application^.ExecView(MenuPopup);

  Dispose(MenuPopup, Done);

  if Event.Command <> cmCancel then
   PutEvent(Event);
 end;

end.