unit jtmAMisc;

interface
uses
     PlugAPI,
     PlugSIDE,

     Objects,
     Drivers,

     App,
     Dialogs,
     MsgBox,
     Views,

     jtmCmn,
     jtmHelp,

     jtMHLcmn,

     jtmTV,

     Wizard;

type
 TMiscMode = (mmAreasManager, mmAreasEditor);

 PAreaUplinks = ^TAreaUplinks;
 TAreaUplinks = object(TListViewer)
  Selected: PAreasSelected;

  constructor Init(var Bounds: TRect; const AOwner: PWindow; const ASelected: PAreasSelected);
  function GetText(Item: xInteger; MaxLen: xInteger): String; virtual;
 end;

 PMiscUplinkDialog = ^TMiscUplinkDialog;
 TMiscUplinkDialog = object(TtuiDialog)
  Selected: PAreasSelected;
  Box: PAreaUplinks;

  constructor Init(const ASelected: PAreasSelected);
  procedure EndModal(Command: xWord); virtual;
 end;

 PMiscDescriptionDialog = ^TMiscDescriptionDialog;
 TMiscDescriptionDialog = object(TtuiDialog)
  Selected: PAreasSelected;
  Description: PtuiInputLine;

  constructor Init(const ASelected: PAreasSelected);
  procedure EndModal(Command: xWord); virtual;
 end;

procedure ExecuteMiscCommand(const Command: xWord; const Selected: PAreasSelected; const Mode: TMiscMode);

implementation

{ TAreaUplinks }

constructor TAreaUplinks.Init(var Bounds: TRect; const AOwner: PWindow; const ASelected: PAreasSelected);
 var
  BarBounds: TRect;
  Bar: PScrollBar;
  Address: TAddress;
 function Perform(const Area: Pointer): Boolean; far;
  var
   Uplink: TAddress;
  begin
   jt.area.GetUplink(Area, Uplink);

   if jt.link.Seek(Uplink) = nil then
    begin
     ClearAddress(Address);

     Perform:=True;
    end
   else
    if IsCleanAddress(Address) then
     begin
      Address:=Uplink;

      Perform:=False;
     end
    else
     if not AddressEqu(Address, Uplink) then
      begin
       ClearAddress(Address);

       Perform:=True;
      end
     else
      Perform:=False;
  end;
 begin
  BarBounds.Assign(Bounds.B.X, Bounds.A.Y, Bounds.B.X + 1, Bounds.B.Y);

  Bar:=New(PScrollBar, Init(BarBounds));

  AOwner^.Insert(Bar);

  inherited Init(Bounds, 1, nil, Bar);

  Selected:=ASelected;

  AOwner^.Insert(@Self);

  SetRange(jt.link.Count);

  ClearAddress(Address);

  Selected^.FirstThat(@Perform);

  if IsCleanAddress(Address) then
   begin
    SetRange(Range + 1);

    FocusItem(Range - 1);
   end
  else
   FocusItem(jt.link.IndexOf(jt.link.Seek(Address)));
 end;

function TAreaUplinks.GetText(Item: xInteger; MaxLen: xInteger): String;
 var
  Link: Pointer;
  Address: TAddress;
 begin
  Link:=jt.link.At(Item);

  if Link = nil then
   GetText:='no changes'
  else
   begin
    jt.link.GetAddress(Link, Address);

    GetText:=AddressToStrEx(Address);
   end;
 end;

{ TMiscUplinkDialog }

constructor TMiscUplinkDialog.Init(const ASelected: PAreasSelected);
 var
  Bounds: TRect;
 begin
  Bounds.Assign(0, 0, 60, 13);

  inherited Init(Bounds, 'Uplink');

  Selected:=ASelected;

  Options:=Options or ofCentered;

  HelpCtx:=hcMiscUplinkDialog;

  Bounds.Assign(3, 2, Size.X - 3, 3);

  if Selected^.Count = 1 then
   Insert(New(PStaticText, Init(Bounds, 'Choose uplink for ' + jt.area.GetName(Selected^.At(0))^ + ':')))
  else
   Insert(New(PStaticText, Init(Bounds, 'Choose uplink for ' + Long2Str(Selected^.Count) + ' area' +
    _s(Selected^.Count) + ':')));

  Bounds.Assign(0, 0, 10, 2);
  Bounds.Move(Size.X - 2 - Bounds.B.X * 2, 2);

  Insert(New(PButton, Init(Bounds, '~O~K', cmOk, bfDefault)));

  Bounds.Move(Bounds.B.X - Bounds.A.X, 0);
  Insert(New(PButton, Init(Bounds, 'Cancel', cmCancel, bfNormal)));

  Bounds.Assign(1, 4, Size.X - 1, Size.Y - 1);

  Box:=New(PAreaUplinks, Init(Bounds, @Self, Selected));
 end;

procedure TMiscUplinkDialog.EndModal(Command: xWord);
 var
  Uplink: Pointer;
  Address: TAddress;
 procedure Perform(const Area: Pointer); far;
  begin
   jt.area.SetUplink(Area, Address);
  end;
 begin
  if Command = cmOk then
   begin
    Uplink:=jt.link.At(Box^.Focused);

    if Uplink <> nil then
     begin
      jt.link.GetAddress(Uplink, Address);

      Selected^.ForEach(@Perform);

      jt.area.SetChanged(acfFull, 'jtm/misc/setuplink');
     end;
   end;

  inherited EndModal(Command);
 end;

{ TMiscDescriptionDialog }

constructor TMiscDescriptionDialog.Init(const ASelected: PAreasSelected);
 var
  Bounds: TRect;
 begin
  Bounds.Assign(0, 0, 70, 8);

  inherited Init(Bounds, 'Description');

  Options:=Options or ofCentered;

  Selected:=ASelected;

  Bounds.Assign(2, 3, Size.X - 2, 4);

  Description:=New(PtuiInputLine, Init(Bounds, 255));

  Insert(Description);

  Bounds.Move(0, -1);

  if Selected^.Count = 1 then
   begin
    Insert(New(PLabel, Init(Bounds, '~D~escription for ' + jt.area.GetName(Selected^.At(0))^ + ':', Description)));

    GetPStringEx(jt.area.GetDesc(Selected^.At(0)), Description^.Data^);
   end
  else
   Insert(New(PLabel, Init(Bounds, '~D~escription for ' + Long2Str(Selected^.Count) + ' area' +
    _s(Selected^.Count) + ':', Description)));

  Bounds.Assign(0, 0, 10, 2);
  Bounds.Move(Size.X - 2 - Bounds.B.X * 2, Size.Y - 3);

  Insert(New(PButton, Init(Bounds, '~O~K', cmOk, bfDefault)));

  Bounds.Move(Bounds.B.X - Bounds.A.X, 0);
  Insert(New(PButton, Init(Bounds, 'Cancel', cmCancel, bfNormal)));

  SelectNext(False);

  HelpCtx:=hcMiscDescriptionDialog;
 end;

procedure TMiscDescriptionDialog.EndModal(Command: xWord);
 procedure Perform(const Area: Pointer); far;
  begin
   jt.area.SetDesc(Area, Description^.Data^);
  end;
 begin
  if Command = cmOk then
   begin
    Selected^.ForEach(@Perform);

    jt.area.SetChanged(acfFull, 'jtm/misc/setdesc');
   end;

  inherited EndModal(Command);
 end;

{ MiscUplink }

procedure MiscUplink(const Selected: PAreasSelected);
 var
  Dialog: PMiscUplinkDialog;
 begin
  Dialog:=New(PMiscUplinkDialog, Init(Selected));

  Desktop^.ExecView(Dialog);

  Dispose(Dialog, Done);
 end;

{ MiscDescription }

procedure MiscDescription(const Selected: PAreasSelected; const Mode: TMiscMode);
 var
  Dialog: PMiscDescriptionDialog;
 begin
  if Mode = mmAreasEditor then
   begin
    MessageBox(^C'This function does not work in areas editor.', nil, mfError or mfCancelButton);

    Exit;
   end;

  Dialog:=New(PMiscDescriptionDialog, Init(Selected));

  Desktop^.ExecView(Dialog);

  Dispose(Dialog, Done);
 end;

{ ExecuteMiscCommand }

procedure ExecuteMiscCommand(const Command: xWord; const Selected: PAreasSelected; const Mode: TMiscMode);
 begin
  case Command of
   cmamUplink: MiscUplink(Selected);
   cmamDescription: MiscDescription(Selected, Mode);
  end;
 end;

end.