unit jtmAreas;

interface
uses
     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     Objects,
     Drivers,

     Views,
     MsgBox,
     Dialogs,
     Menus,
     App,

     rlopAPI,

     jtMHLcmn,

     jtmCmn,
     jtmUtil,
     jtmPerf,
     jtmLog,
     jtmHelp,
     jtmAEdit,
     jtmAConv,
     jtmAMisc,
     jtmTV,

     Wizard;

type
 TSelectRegExpAction = (sreSelect, sreUnselect, sreToggle);

 PAreasBox = ^TAreasBox;
 TAreasBox = object(TListViewer)
  Selected: PAreasSelected;
  LocalMenu, PassthroughMenu, MiscMenu: PMenu;
  SearchPos: Byte;

  constructor Init(var Bounds: TRect; const HSB, VSB: PScrollBar);
  procedure FocusItem(Item: xInteger); virtual;
  function GetText(Item: xInteger; MaxLen: xInteger): String; virtual;
  function IsSelected(Item: xInteger): Boolean; virtual;
  procedure Draw; virtual;
  procedure HandleEvent(var Event: TEvent); virtual;
  destructor Done; virtual;
 private
  SingleSelected: Pointer;

  procedure Update(const Area: Pointer);
  procedure DoSelect(const MoveDown: Boolean);
  procedure DoSelectRegExp(const Action: TSelectRegExpAction);
  procedure Menu(const AMenu: PMenu; const Mouse: Boolean);
  procedure EditArea;
  procedure CreateArea;
  procedure DestroyArea;
  procedure ConvertArea;
  procedure MakeAreaPassthrough;
  procedure UnsubscribeAndNotifyLinks;
  procedure SubscribeArea(const Subscribe: Boolean);
  procedure RelinkArea;
  procedure CaptureSelected;
  procedure ReleaseSelected;
 end;

 PAreasSelectedInfo = ^TAreasSelectedInfo;
 TAreasSelectedInfo = object(TView)
  Selected: PAreasSelected;
  Info: PString;

  constructor Init(var Bounds: TRect; const ASelected: PAreasSelected);
  procedure Draw; virtual;
  destructor Done; virtual;
 public
  procedure Update;
 end;

 PAreasManager = ^TAreasManager;
 TAreasManager = object(TtuiDialog)
  Box: PAreasBox;
  Info: PAreasSelectedInfo;

  constructor Init;
  procedure SizeLimits(var Min, Max: TPoint); virtual;
  destructor Done; virtual;
 end;

implementation
uses
     jtmTUI;

{ TAreasBox }

constructor TAreasBox.Init(var Bounds: TRect; const HSB, VSB: PScrollBar);
 begin
  inherited Init(Bounds, 1, HSB, VSB);

  HSB^.SetRange(1, 255);

  GrowMode:=gfGrowHiX or gfGrowHiY;

  Selected:=New(PAreasSelected, Init(jt.area.Count, $100));

  SearchPos:=1;

  ShowCursor;

  Update(nil);

  MiscMenu:=NewMenu(
   NewItem('~U~plink...', '', kbNoKey, cmamUplink, hcMenu,
   NewItem('~D~escription...', '', kbNoKey, cmamDescription, hcMenu,
    nil)));

  PassthroughMenu:=NewMenu(
   NewItem('~M~ake passthrough', '', kbNoKey, cmaMakeAreaPassthrough, hcMenu,
   NewItem('~U~nsubscribe and notify links', '', kbNoKey, cmaUnsubscribeAreaAndNotifyLinks, hcMenu,
   nil)));

  LocalMenu:=NewMenu(
          NewItem('~E~dit...', 'Enter', kbNoKey, cmaEditArea, hcMenu,
          NewLine(
          NewItem('Cre~a~te...', 'Alt-N', kbAltN, cmaCreateArea, hcMenu,
          NewItem('~D~estroy...', 'Del', kbDel, cmaDestroyArea, hcMenu,
          NewLine(
          NewItem('~C~onvert...', 'Alt-C', kbAltC, cmaConvertArea, hcMenu,
          NewLine(
          NewSubMenu('~P~assthrough...', hcMenu, PassthroughMenu,
          NewLine(
          NewItem('~S~ubscribe...', 'Alt-S', kbAltS, cmaSubscribeArea, hcMenu,
          NewItem('~U~nsubscribe...', 'Alt-U', kbAltU, cmaUnsubscribeArea, hcMenu,
          NewItem('~R~elink...', 'Alt-R', kbAltR, cmaRelinkArea, hcMenu,
          NewLine(
          NewSubMenu('~M~isc...', hcMenu, MiscMenu,
          nil)))))))))))))));
 end;

procedure TAreasBox.FocusItem(Item: xInteger);
 begin
  if Item >= Range then
   Item:=Range - 1;

  inherited FocusItem(Item);
 end;

function TAreasBox.GetText(Item: xInteger; MaxLen: xInteger): String;
 var
  Area: Pointer;
  Format: TAreaFormat;
  Address: TAddress;
  AreaUplink: String;
 const
  WidthAreaName         : Byte          = 25;
  WidthAreaUplink       : Byte          = 15;
  WidthAreaStyle                        =  5;
  WidthAreaFormat                       =  6;
 function AreaStyle: String;
  begin
   case Format.Style of
    asNetmail: AreaStyle:='Net';
    asEchomail: AreaStyle:='Echo';
    asLocal: AreaStyle:='Local';
   else
    AreaStyle:=Long2Str(Format.Style);
   end;
  end;
 function AreaFormat: String;
  var
   Index, id: Longint;
   FullBaseName, ShortBaseName: String;
  begin
   jt.core.SeekFormat(Format.Format, Index);
   jt.core.AtFormat(Index, id, FullBaseName, ShortBaseName);

   AreaFormat:=ShortBaseName;
  end;
 begin
  Area:=jt.area.At(Item);

  if Area <> nil then
   begin
    jt.area.GetUplink(Area, Address);

    if IsCleanAddress(Address) then
     AreaUplink:=''
    else
     AreaUplink:=AddressToStrEx(Address);

    jt.area.GetFormat(Area, Format);

    WidthAreaUplink:=Length(AreaUplink);

    WidthAreaName:=Size.X - WidthAreaUplink - WidthAreaStyle - WidthAreaFormat - 5;

    GetText:=Pad(Copy(jt.area.GetName(Area)^, MaxLen, WidthAreaName), WidthAreaName) +
       ' ' + AreaUplink +
       ' ' + Pad(AreaStyle, WidthAreaStyle) +
       ' ' + Pad(Copy(AreaFormat, 1, WidthAreaFormat), WidthAreaFormat);
   end
  else
   Area:=nil;
 end;

function TAreasBox.IsSelected(Item: xInteger): Boolean;
 var
  Index: xInteger;
 begin
  if Item < jt.area.Count then
   IsSelected:=Selected^.Search(jt.area.At(Item), Index)
  else
   IsSelected:=False;
 end;

procedure TAreasBox.Draw;
 var
  NormalColor, FocusedColor, SelectedColor, FocusedSelectedColor, Color: Byte;
  ItemSelected: Boolean;
  I, Item: Integer;
  B: TDrawBuffer;
 begin
  NormalColor:=$17;
  SelectedColor:=$1E;

  if State and (sfSelected + sfActive) = (sfSelected + sfActive) then
   begin
    FocusedColor:=$2F;
    FocusedSelectedColor:=$2E;
   end
  else
   begin
    FocusedColor:=$1F;
    FocusedSelectedColor:=$1E;
   end;

  if HScrollBar^.Value <> 1 then
   if GetState(sfCursorVis) then
    begin
     HideCursor;

     SearchPos:=1;
    end;

  Item:=TopItem;

  for I:=0 to Size.Y - 1 do
   begin
    ItemSelected:=IsSelected(Item);

    if (State and (sfSelected + sfActive) = (sfSelected + sfActive)) and (Focused = Item) and (Range > 0) then
     begin
      if ItemSelected then
       Color:=FocusedSelectedColor
      else
       Color:=FocusedColor;

      if HScrollBar^.Value = 1 then
       begin
        if not GetState(sfCursorVis) then
         ShowCursor;

        SetCursor(SearchPos, I);
       end;
     end
    else
     if (Item < Range) and ItemSelected then
      Color:=SelectedColor
     else
      Color:=NormalColor;

    MoveChar(B, ' ', Color, Size.X);

    if ItemSelected then
     MoveChar(B, '*', Color, 1);

    if Item < Range then
     MoveStr(B[1], GetText(Item, HScrollBar^.Value), Color);

    WriteLine(0, I, Size.X, 1, B);

    Inc(Item);
   end;
 end;

procedure TAreasBox.HandleEvent(var Event: TEvent);
 var
  OldFocused: xInteger;
  S: String;
  K: Longint;
 begin
  if (Event.What = evKeyDown) and (HScrollBar^.Value = 1) then
   case Event.CharCode of
    #0:;
    #8:
     if SearchPos > 1 then
      begin
       Dec(SearchPos);

       DrawView;
      end;
   else
    S:=Copy(jt.area.GetName(jt.area.At(Focused))^, 1, SearchPos - 1) + Event.CharCode;

    StUpcaseEx(S);

    for K:=0 to jt.area.Count - 1 do
     if Copy(jt.area.GetName(jt.area.At(K))^, 1, Length(S)) = S then
      begin
       inherited FocusItem(K);

       Inc(SearchPos);

       DrawView;

       ClearEvent(Event);

       Exit;
      end;
   end;

  OldFocused:=Focused;

  if Event.What = evCommand then
   repeat
    case Event.Command of
     cmaSelectToggle: DoSelect(False);
     cmaSelect: DoSelect(True);
     cmaSelectRegExp: DoSelectRegExp(sreSelect);
     cmaUnselectRegExp: DoSelectRegExp(sreUnselect);
     cmaToggleRegExp: DoSelectRegExp(sreToggle);
     cmaPopupMenu: Menu(LocalMenu, False);
     cmaMiscMenu: Menu(MiscMenu, False);
     cmaPassthroughMenu: Menu(PassthroughMenu, False);
     cmaEditArea: EditArea;
     cmaCreateArea: CreateArea;
     cmaDestroyArea: DestroyArea;
     cmaConvertArea: ConvertArea;
     cmaMakeAreaPassthrough: MakeAreaPassthrough;
     cmaUnsubscribeAreaAndNotifyLinks: UnsubscribeAndNotifyLinks;
     cmaSubscribeArea: SubscribeArea(True);
     cmaUnsubscribeArea: SubscribeArea(False);
     cmaRelinkArea: RelinkArea;
     cmamMiscStart..cmamMiscEnd:
      begin
       CaptureSelected;

       ExecuteMiscCommand(Event.Command, Selected, mmAreasManager);

       ReleaseSelected;

       Update(nil);
      end;
    else
     Break;
    end;

    OldFocused:=-1;

    ClearEvent(Event);
   until True;

  if (Event.What = evMouseDown) and (Event.Buttons = mbRightButton) and MouseInView(Event.Where) then
   begin
    Menu(LocalMenu, True);

    ClearEvent(Event);
   end;

  inherited HandleEvent(Event);

  if Focused <> OldFocused then
   if SearchPos <> 1 then
    begin
     SearchPos:=1;

     DrawView;
    end;
 end;

destructor TAreasBox.Done;
 begin
  DisposeMenu(LocalMenu);

  Dispose(Selected, Done);

  inherited Done;
 end;

{ private methods }

procedure TAreasBox.Update(const Area: Pointer);
 var
  AFocused, AAFocused: Longint;
 begin
  AFocused:=Focused;

  SetRange(jt.area.Count);

  if Area <> nil then
   begin
    AAFocused:=jt.area.IndexOf(Area);

    if AAFocused <> -1 then
     AFocused:=AAFocused;
   end;

  FocusItem(AFocused);

  DrawView;
 end;

procedure TAreasBox.DoSelect(const MoveDown: Boolean);
 var
  Area: Pointer;
  Index: xInteger;
 begin
  Area:=jt.area.At(Focused);

  if Area <> nil then
   begin
    if Selected^.Search(Area, Index) then
     Selected^.AtDelete(Index)
    else
     Selected^.AtInsert(Index, Area);

    if MoveDown then
     FocusItem(Focused + 1);

    DrawView;

    PAreasManager(Owner)^.Info^.Update;
   end;
 end;

procedure TAreasBox.DoSelectRegExp(const Action: TSelectRegExpAction);
 var
  re, Area: Pointer;
  Index: xInteger;
  Format: TAreaFormat;
  K, Styles, Formats: Longint;
 begin
  case Action of
   sreSelect: re:=AskRegExpAreas('Select', hidAreasManagerSelect, Styles, Formats);
   sreUnselect: re:=AskRegExpAreas('Unselect', hidAreasManagerUnselect, Styles, Formats);
   sreToggle: re:=AskRegExpAreas('Toggle', hidAreasManagerToggle, Styles, Formats);
  else
   re:=nil;
  end;

  if re <> nil then
   begin
    for K:=0 to jt.area.Count - 1 do
     begin
      Area:=jt.area.At(K);

      if jt.re.MatchStr(re, jt.area.GetName(Area)^, False) then
       begin
        jt.area.GetFormat(Area, Format);

        if (Styles and Format.Style <> 0) and (Formats and (1 shl (Format.Format)) <> 0) then
         case Action of
          sreSelect:
           if not Selected^.Search(Area, Index) then
            Selected^.AtInsert(Index, Area);
          sreUnselect:
           if Selected^.Search(Area, Index) then
            Selected^.AtDelete(Index);
          sreToggle:
           if Selected^.Search(Area, Index) then
            Selected^.AtDelete(Index)
           else
            Selected^.AtInsert(Index, Area);
         end;
       end;
     end;

    jt.re.Destroy(re);

    DrawView;

    PAreasManager(Owner)^.Info^.Update;
   end;
 end;

procedure TAreasBox.Menu(const AMenu: PMenu; const Mouse: Boolean);
 var
  PopupCoords: TPoint;
  MenuPopup: PMenuPopup;
  Bounds: TRect;
  Event: TEvent;
 begin
  if Mouse then
   begin
    PopupCoords:=MouseWhere;

    Event.What:=evMouseDown;
    Event.Where:=MouseWhere;
    Event.Buttons:=MouseButtons;
    Event.Double:=False;

    PutEvent(Event);

    Dec(PopupCoords.X, 2);
    Dec(PopupCoords.Y);
   end
  else
   begin
    PopupCoords.Y:=Focused - TopItem + 1;
    PopupCoords.X:=2;

    MakeGlobal(PopupCoords, PopupCoords);
   end;

  Bounds.Assign(0, 0, 40, 20);

  MenuPopup:=New(PtuiMenuPopup, Init(Bounds, AMenu));

  MenuPopup^.GetBounds(Bounds);

  Bounds.Move(PopupCoords.X, PopupCoords.Y);

  while Bounds.B.X > Application^.Size.X do Bounds.Move(-1, 0);
  while Bounds.B.Y > Application^.Size.Y do Bounds.Move(0, -1);
  while Bounds.A.X < 0 do Bounds.Move(1, 0);
  while Bounds.A.Y < 0 do Bounds.Move(0, 1);

  MenuPopup^.Locate(Bounds);

  Event.What:=evCommand;
  Event.Command:=Application^.ExecView(MenuPopup);

  Dispose(MenuPopup, Done);

  if Event.Command <> cmCancel then
   PutEvent(Event);
 end;

procedure TAreasBox.EditArea;
 var
  Dialog: PAreaEditor;
  Area: Pointer;
  OldDesc: String;
 begin
  CaptureSelected;

  Dialog:=New(PAreaEditor, Init(Selected, False, MiscMenu));

  if Dialog^.Area <> nil then
   GetPStringEx(jt.area.GetDesc(Dialog^.Area), OldDesc);

  if Desktop^.ExecView(Dialog) = cmOk then
   begin
    if Dialog^.Area <> nil then
     begin
      Area:=Dialog^.Area;

      if OldDesc <> GetPString(jt.area.GetDesc(Dialog^.Area)) then
       jt.area.NotifyAreaChanged(Area);
     end
    else
     Area:=nil;
   end
  else
   Area:=nil;

  Dispose(Dialog, Done);

  ReleaseSelected;

  Update(Area);
 end;

procedure TAreasBox.CreateArea;
 var
  ASelected: PAreasSelected;
  Dialog: PAreaEditor;
  Format: TAreaFormat;
  Area, FocusArea: Pointer;
  S, rc: String;
  K: Longint;
 begin
  ASelected:=New(PAreasSelected, Init($10, $10));

  Format.Style:=asEchomail;
  Format.Format:=afPassthrough;
  Format.Path:='';

  K:=0;
  S:='NEWAREA';

  while jt.area.Seek(S) <> nil do
   begin
    Inc(K);

    S:='NEWAREA' + Long2Str(K);
   end;

  Area:=jt.area.Create(S, @Format);

  ASelected^.Insert(Area);

  Dialog:=New(PAreaEditor, Init(ASelected, True, MiscMenu));

  FocusArea:=nil;

  if Desktop^.ExecView(Dialog) = cmOk then
   begin
    jt.area.NotifyAreaCreated(Area);

    FocusArea:=Area;
   end
  else
   if not jt.area.Kill(Area, rc) then
    jt.LogWrite('?', 'jtm/ab/CreateArea: ' + rc + ' (brain is dead?')
   else
    jt.area.killed.Destroy(Area);

  Dispose(Dialog, Done);

  Dispose(ASelected, Done);

  Update(FocusArea);
 end;

procedure TAreasBox.DestroyArea;
 var
  Killed: PAreasSelected;
 function Perform(const Area: Pointer): Boolean; far;
  var
   rc: String;
  begin
   jt.SetStatus('killing area ' + jt.area.GetName(Area)^);

   if jt.area.Kill(Area, rc) then
    begin
     Killed^.Insert(Area);

     Perform:=False;
    end
   else
    Perform:=MessageBox(^C'Failed to kill ' + jt.area.GetName(Area)^ + ': ' + rc + '. Continue?', nil,
     mfConfirmation or mfYesButton or mfNoButton) <> cmYes;
  end;
 procedure Notify(const Area: Pointer); far;
  begin
   Selected^.Delete(Area);
  end;
 var
  Ok: Boolean;
 begin
  CaptureSelected;

  Killed:=New(PAreasSelected, Init($10, $10));

  if Selected^.Count = 1 then
   Ok:=MessageBox(^C'Do you wish to destroy ' + jt.area.GetName(Selected^.At(0))^ + '?', nil,
    mfConfirmation or mfYesButton or mfNoButton) = cmYes
  else
   Ok:=MessageBox(^C'Do you wish to destroy ' + Long2Str(Selected^.Count) + ' area' + _s(Selected^.Count) + '?', nil,
    mfConfirmation or mfYesButton or mfNoButton) = cmYes;

  if Ok then
   Selected^.FirstThat(@Perform);

  ReleaseSelected;

  Killed^.ForEach(@Notify);

  if Killed^.Count <> 0 then
   begin
    jt.area.SetChanged(acfFull, 'jtm/akilled');
    jt.link.SetChanged(lcfFull, 'jtm/akilled');
   end;

  jt.ClearStatus;

  Dispose(Killed, Done);

  Update(nil);
 end;

procedure TAreasBox.ConvertArea;
 var
  Dialog: PAreaConverter;
 begin
  CaptureSelected;

  Dialog:=New(PAreaConverter, Init(Selected));

  Desktop^.ExecView(Dialog);

  Dispose(Dialog, Done);

  ReleaseSelected;

  Update(nil);
 end;

procedure TAreasBox.MakeAreaPassthrough;
 var
  SelectedKilled: PAreasSelected;
 function Perform(const Area: Pointer): Boolean; far;
  var
   Format: TAreaFormat;
   Links: PAreaLinks;
   KillMode: (kmDoNot, kmNoLinks, kmAlreadyPT);
   Name, rc: String;
  procedure Unsubscribe(const Link: PAreaLink); far;
   begin
    if rlopIsLinkValid(Link^.Link, rc) then
     begin
      rlopSendAreaUnsubscribe(Link^.Link, Name);

      jt.LogWrite('J', Name + ': sending unsubscribe to ' + AddressOfLink(Link^.Link));
     end
    else
     jt.LogWrite('?', Name + ': failed to unsubscribe ' + AddressOfLink(Link^.Link) + ' - ' + rc);
   end;
  begin
   Update(Area);

   Links:=New(PAreaLinks, Init(Area, True));

   Name:=jt.area.GetName(Area)^;

   jt.area.GetFormat(Area, Format);

   if Format.Format <> afPassthrough then
    if Links^.Count <> 0 then
     begin
      if not jt.area.KillBase(Format, rc) then
       jt.LogWrite('?', Name + ': failed to kill base - ' + rc);

      Format.Format:=afPassthrough;

      jt.area.SetFormat(Area, Format);

      jt.LogWrite('J', Name + ': made passthrough');

      Perform:=False;

      KillMode:=kmDoNot;
     end
    else
     KillMode:=kmNoLinks
   else
    KillMode:=kmAlreadyPT;

   if KillMode <> kmDoNot then
    begin
     case KillMode of
      kmNoLinks: rc:='have no valid subscribed links (exclude uplink)';
      kmAlreadyPT: rc:='already passthrough';
     end;

     case MessageBox(^C + Name + ' ' + rc + '. Sure to kill and unsubscribe all linked?', nil,
       mfConfirmation or mfYesButton or mfNoButton or mfCancelButton) of
      cmYes:
       begin
        Dispose(Links, Done);

        Links:=New(PAreaLinks, Init(Area, False));

        if jt.area.Kill(Area, rc) then
         begin
          jt.LogWrite('J', Name + ': killed');

          Links^.ForEach(@Unsubscribe);

          SelectedKilled^.Insert(Area);
         end
        else
         jt.LogWrite('?', Name + ': failed to kill - ' + rc);

        Perform:=False;
       end;
      cmNo: Perform:=False;
      cmCancel: Perform:=True;
     end;
   end;

   Dispose(Links, Done);
  end;
 procedure KillInSelected(const Area: Pointer); far;
  begin
   Selected^.Delete(Area);
  end;
 begin
  CaptureSelected;

  if MessageBox(^C + 'Sure to make passthrough ' + Long2Str(Selected^.Count) + ' area' + _s(Selected^.Count) + '?', nil,
    mfConfirmation or mfYesButton or mfNoButton) = cmYes then
   begin
    SelectedKilled:=New(PAreasSelected, Init(Selected^.Count, $10));

    Selected^.FirstThat(@Perform);

    jt.area.SetChanged(acfFull, 'jtm/makept');
    jt.link.SetChanged(lcfFull, 'jtm/makept');

    SelectedKilled^.ForEach(@KillInSelected);

    Dispose(SelectedKilled, Done);

    ReleaseSelected;

    Update(nil);

    DrawView;
   end
  else
   ReleaseSelected;
 end;

procedure TAreasBox.UnsubscribeAndNotifyLinks;
 var
  SelectedKilled: PAreasSelected;
 function Perform(const Area: Pointer): Boolean; far;
  var
   Format: TAreaFormat;
   Links: PAreaLinks;
   Name, rc: String;
  procedure Unsubscribe(const Link: PAreaLink); far;
   begin
    if rlopIsLinkValid(Link^.Link, rc) then
     begin
      rlopSendAreaUnsubscribe(Link^.Link, Name);

      jt.LogWrite('J', Name + ': sending unsubscribe to ' + AddressOfLink(Link^.Link));
     end
    else
     jt.LogWrite('?', Name + ': failed to unsubscribe ' + AddressOfLink(Link^.Link) + ' - ' + rc);
   end;
  begin
   Update(Area);

   Links:=New(PAreaLinks, Init(Area, False));

   Name:=jt.area.GetName(Area)^;

   jt.area.GetFormat(Area, Format);

   case MessageBox(^C + Name + ': sure to kill and unsubscribe all linked?', nil,
     mfConfirmation or mfYesButton or mfNoButton or mfCancelButton) of
    cmYes:
     begin
      if jt.area.Kill(Area, rc) then
       begin
        jt.LogWrite('J', Name + ': killed');

        Links^.ForEach(@Unsubscribe);

        SelectedKilled^.Insert(Area);
       end
      else
       jt.LogWrite('?', Name + ': failed to kill - ' + rc);

      Perform:=False;
     end;
    cmNo: Perform:=False;
    cmCancel: Perform:=True;
   end;

   Dispose(Links, Done);
  end;
 procedure KillInSelected(const Area: Pointer); far;
  begin
   Selected^.Delete(Area);
  end;
 begin
  CaptureSelected;

  if MessageBox(^C + 'Sure to kill ' + Long2Str(Selected^.Count) + ' area' + _s(Selected^.Count) +
    ' and unsubscribe all linked?', nil, mfConfirmation or mfYesButton or mfNoButton) = cmYes then
   begin
    FocusLog;

    SelectedKilled:=New(PAreasSelected, Init(Selected^.Count, $10));

    Selected^.FirstThat(@Perform);

    jt.area.SetChanged(acfFull, 'jtm/unsub&notify');
    jt.link.SetChanged(lcfFull, 'jtm/unsub&notify');

    SelectedKilled^.ForEach(@KillInSelected);

    Dispose(SelectedKilled, Done);

    Update(nil);
   end;

  ReleaseSelected;
 end;

procedure TAreasBox.SubscribeArea(const Subscribe: Boolean);
 var
  Links: PLinksSelected;
  Dialog: PLinksSelectorDialog;
  Operation: String;
 procedure PerformArea(const Area: Pointer); far;
  procedure PerformLink(const Link: Pointer); far;
   begin
    jt.LogWrite('J', jt.area.GetName(Area)^ + ': ' + Operation + ' to ' + AddressOfLink(Link));

    if Subscribe then
     rlopSendAreaSubscribe(Link, jt.area.GetName(Area)^)
    else
     rlopSendAreaUnsubscribe(Link, jt.area.GetName(Area)^);
   end;
  begin
   jt.LogWrite('J', 'Sending ' + Operation + '-requests for area ' + jt.area.GetName(Area)^ + ':');

   Links^.ForEach(@PerformLink);
  end;
 begin
  CaptureSelected;

  Links:=New(PLinksSelected, Init(jt.link.Count, $10));

  if Subscribe then
   Operation:='subscribe'
  else
   Operation:='unsubscribe';

  Dialog:=New(PrlopLinksSelectorDialog, Init(Links, 'Choose links', 'Choose links to send ' + Operation + '-requests:'));

  if Dialog^.Valid(cmValid) then
   if Desktop^.ExecView(Dialog) = cmOk then
    if MessageBox(^C'Sure to send ' + Operation + '-requests to ' + Long2Str(Links^.Count) + ' link' + _s(Links^.Count) +
      ' for ' + Long2Str(Selected^.Count) + ' area' + _s(Selected^.Count) + '?', nil, mfConfirmation or mfYesButton or
      mfNoButton) = cmYes then
     begin
      FocusLog;

      Selected^.ForEach(@PerformArea);
     end;

  Dispose(Dialog, Done);

  Dispose(Links, Done);

  ReleaseSelected;
 end;

procedure TAreasBox.RelinkArea;
 var
  Link: Pointer;
  Uplink: TAddress;
 procedure Perform(const Area: Pointer); far;
  var
   rc: String;
   OldLink: Pointer;
   OldUplink: TAddress;
  begin
   jt.area.GetUplink(Area, OldUplink);

   OldLink:=jt.link.Seek(OldUplink);

   if (OldLink <> nil) and rlopIsLinkValid(OldLink, rc) then
    begin
     jt.LogWrite('J', 'Relinking ' + jt.area.GetName(Area)^ + ' from ' + AddressToStrEx(OldUplink) + ' to ' +
      AddressToStrEx(Uplink));

     rlopSendAreaUnsubscribe(OldLink, jt.area.GetName(Area)^);
     rlopSendAreaSubscribe(Link, jt.area.GetName(Area)^);
    end
   else
    begin
     jt.LogWrite('J', 'Relinking ' + jt.area.GetName(Area)^ + ' to ' + AddressToStrEx(Uplink));

     rlopSendAreaSubscribe(Link, jt.area.GetName(Area)^);
    end;

   jt.area.SetUplink(Area, Uplink);

   jt.link.SetChanged(lcfFull, 'jtm/relink');
   jt.area.SetChanged(acfFull, 'jtm/relink');
  end;
 var
  LinksSelected: PLinksSelected;
  Dialog: PrlopLinksSelectorDialog;
 begin
  CaptureSelected;

  LinksSelected:=New(PLinksSelected, Init($1, $1));

  Dialog:=New(PrlopLinksSelectorDialog, Init(LinksSelected, 'Relink', 'Relink ' + Long2Str(Selected^.Count) + ' area' +
   _s(Selected^.Count) + ' to:'));

  Dialog^.HelpCtx:=hcRelinkSelectorDialog;

  if Dialog^.Valid(cmValid) then
   if Desktop^.ExecView(Dialog) = cmOk then
    begin
     Link:=LinksSelected^.At(0);

     jt.link.GetAddress(Link, Uplink);

     if MessageBox(^C'Sure to relink ' + Long2Str(Selected^.Count) + ' area' + _s(Selected^.Count) + ' to ' +
       AddressToStrEx(Uplink) + '?', nil, mfConfirmation or mfYesButton or mfNoButton) = cmYes then
      Selected^.ForEach(@Perform);
    end;

  Dispose(Dialog, Done);

  Dispose(LinksSelected, Done);

  ReleaseSelected;

  Update(nil);
 end;

procedure TAreasBox.CaptureSelected;
 begin
  if Selected^.Count = 0 then
   begin
    SingleSelected:=jt.area.At(Focused);

    Selected^.Insert(SingleSelected);
   end
  else
   SingleSelected:=nil;
 end;

procedure TAreasBox.ReleaseSelected;
 begin
  if SingleSelected <> nil then
   Selected^.Delete(SingleSelected);
 end;

{ TAreasSelectedInfo }

constructor TAreasSelectedInfo.Init(var Bounds: TRect; const ASelected: PAreasSelected);
 begin
  inherited Init(Bounds);

  Selected:=ASelected;

  Update;
 end;

procedure TAreasSelectedInfo.Draw;
 var
  B: TDrawBuffer;
 begin
  MoveStr(B, Info^, GetColor(2));

  WriteLine(0, 0, Size.X, Size.Y, B);
 end;

destructor TAreasSelectedInfo.Done;
 begin
  DisposeStr(Info);

  inherited Done;
 end;

{ public methods }

procedure TAreasSelectedInfo.Update;
 var
  Bounds: TRect;
 begin
  DisposeStr(Info);

  Info:=NewStr(Long2Str(Selected^.Count) + '/' + Long2Str(jt.area.Count));

  GetBounds(Bounds);

  Bounds.B.X:=Bounds.A.X + Length(Info^);

  Locate(Bounds);

  DrawView;
 end;

{ TAreasManager }

constructor TAreasManager.Init;
 var
  Bounds: TRect;
  HSB: PScrollBar;
 begin
  Desktop^.GetExtent(Bounds);

  Bounds.B.Y:=Bounds.B.Y - VerticalMargin;

  inherited Init(Bounds, 'Areas Manager');

  if NoShadow then
   SetState(sfShadow, False);

  HSB:=StandardScrollBar(sbHorizontal or sbHandleKeyboard);

  HSB^.GetBounds(Bounds);

  Inc(Bounds.A.X, 11);

  HSB^.Locate(Bounds);

  GetExtent(Bounds);

  Bounds.Grow(-1, -1);

  Box:=New(PAreasBox, Init(Bounds, HSB,
                                   StandardScrollBar(sbVertical or sbHandleKeyboard)));

  Insert(Box);

  Number:=wnAreasManager;

  Palette:=dpBlueDialog;

  HelpCtx:=hcAreasManager;

  Flags:=wfClose or wfMove or wfGrow or wfZoom;

  Bounds.Assign(2, Size.Y - 1, 2, Size.Y);

  Info:=New(PAreasSelectedInfo, Init(Bounds, Box^.Selected));

  Info^.GrowMode:=gfGrowLoY or gfGrowHiY;

  Insert(Info);
 end;

procedure TAreasManager.SizeLimits(var Min, Max: TPoint);
 begin
  inherited SizeLimits(Min, Max);

  Min.X:=40;
  Min.Y:=6;
 end;

destructor TAreasManager.Done;
 begin
  inherited Done;

  PtuiApplication(Application)^.AreasManager:=nil;
 end;

end.
