unit jtmLog;

interface
uses
     PlugAPI,
     PlugSIDE,

     Objects,
     Drivers,

     Views,
     Dialogs,
     App,

     jtmCmn,
     jtmHelp,
     jtmTV,

     Wizard;

const
 CLogBox = #$0F#$0F#$0C#$09#$00;

type
 PLogBox = ^TLogBox;
 TLogBox = object(TListViewer)
  Log: PStrings;

  constructor Init(var Bounds: TRect; const HSB, VSB: PScrollBar);
  function GetText(Item: xInteger; MaxLen: xInteger): String; virtual;
  function GetPalette: PPalette; virtual;
  destructor Done; virtual;
 public
  procedure Update;
 private
  procedure CheckLimitX(const X: Integer);
 end;

 PLogViewer = ^TLogViewer;
 TLogViewer = object(TtuiDialog)
  Box: PLogBox;

  constructor Init;
  procedure Close; virtual;
  procedure SizeLimits(var Min, Max: TPoint); virtual;
  destructor Done; virtual;
 public
  LastStatus: Boolean;

  procedure LogWrite(const Symbol: Char; const Message: String);
  procedure SetStatus(const Status: String);
  procedure ClearStatus;
 end;

procedure FocusLog;

implementation
uses
     jtmTUI;

const
 Log                    : PLogViewer    = nil;

{ TLogBox }

constructor TLogBox.Init(var Bounds: TRect; const HSB, VSB: PScrollBar);
 begin
  inherited Init(Bounds, 1, HSB, VSB);

  Log:=New(PStrings, Init($100, $100));

  HelpCtx:=hcLogViewer;

  Update;
 end;

function TLogBox.GetPalette: PPalette;
 const
  S: String[Length(CLogBox)] = CLogBox;
 begin
  GetPalette:=@S;
 end;

function TLogBox.GetText(Item: xInteger; MaxLen: xInteger): String;
 var
  S: PString;
 begin
  S:=Log^.At(Item);

  if S <> nil then
   GetText:=S^
  else
   GetText:='';
 end;

destructor TLogBox.Done;
 begin
  Dispose(Log, Done);

  inherited Done;
 end;

{ public methods }

procedure TLogBox.Update;
 begin
  SetRange(Log^.Count);

  if Log^.Count > 0 then
   FocusItem(Log^.Count - 1);

  DrawView;
 end;

{ private methods }

procedure TLogBox.CheckLimitX(const X: Integer);
 begin
  if X > HScrollBar^.Max then
   HScrollBar^.SetRange(1, X);
 end;

{ TLogViewer }

constructor TLogViewer.Init;
 var
  Bounds: TRect;
 begin
  Log:=@Self;

  Desktop^.GetExtent(Bounds);

  Bounds.A.Y:=Bounds.B.Y - VerticalMargin;

  inherited Init(Bounds, 'Log');

  if NoShadow then
   SetState(sfShadow, False);

  GetExtent(Bounds);

  Bounds.Grow(-1, -1);

  Box:=New(PLogBox, Init(Bounds, StandardScrollBar(sbHorizontal or sbHandleKeyboard),
                                 StandardScrollBar(sbVertical or sbHandleKeyboard)));

  Box^.GrowMode:=gfGrowHiX or gfGrowHiY;

  Insert(Box);

  Number:=wnLogViewer;

  Palette:=dpGrayDialog;

  Flags:=wfClose or wfMove or wfGrow or wfZoom;
 end;

procedure TLogViewer.Close;
 begin
  SetState(sfVisible, False);
 end;

procedure TLogViewer.SizeLimits(var Min, Max: TPoint);
 begin
  inherited SizeLimits(Min, Max);

  Min.X:=20;
  Min.Y:=6;
 end;

destructor TLogViewer.Done;
 begin
  inherited Done;

  Log:=nil;
 end;

{ public methods }

procedure TLogViewer.LogWrite(const Symbol: Char; const Message: String);
 begin
  if LastStatus then
   begin
    Box^.Log^.AtFree(Box^.Log^.Count - 1);

    LastStatus:=False;
   end;

  Box^.Log^.Insert(NewStr(Symbol + ' ' + Copy(GetBinkDateTime, 8, 8) + ' ' + Message));

  Box^.CheckLimitX(11 + Length(Message));

  Box^.Update;
 end;

procedure TLogViewer.SetStatus(const Status: String);
 var
  B: TDrawBuffer;
 begin
  if LastStatus then
   Box^.Log^.AtFree(Box^.Log^.Count - 1);

  Box^.CheckLimitX(Length(Status));

  Box^.Log^.Insert(NewStr(Status));

  Box^.Update;

  LastStatus:=True;

  with StatusLine^ do
   begin
    MoveChar(B, ' ', GetColor(1), Size.X);

    MoveStr(B[1], Copy(Status, 1, Size.X), GetColor(1));

    WriteLine(0, 0, Size.X, Size.Y, B);
   end;
 end;

procedure TLogViewer.ClearStatus;
 begin
  if LastStatus then
   begin
    Box^.Log^.AtFree(Box^.Log^.Count - 1);

    Box^.Update;

    LastStatus:=False;
   end;

  StatusLine^.DrawView;
 end;

{ FocusLog }

procedure FocusLog;
 begin
  if Log <> nil then
   Log^.Focus;
 end;

end.