unit jtmTUI;

interface
uses
{$IFDEF VIRTUALPASCAL}
     vpSysLow,
{$ENDIF}
     PlugAPI,
     PlugSIDE,

     PrioAPI,

     jtmCmn,
     jtmLog,
     jtmAreas,
     jtmHelp,

     ScrState,

     Objects,
     Drivers,
     Dialogs,
     Views,
     App,
     Menus,
     MsgBox,

     Wizard,

     TheClip,
     TheHelp,

     jtMHLcmn,

     jtmTV;

const
 CApplication =
        #$71#$70#$78#$74#$20#$28#$24#$17#$1F#$1A#$31#$31#$1E#$71#$1F +
    #$37#$3F#$3A#$13#$13#$3E#$21#$3F#$70#$7F#$7A#$13#$13#$70#$7F#$7E +
    #$70#$7F#$7A#$13#$13#$70#$70#$7F#$7E#$20#$2B#$2F#$78#$2E#$70#$30 +
    #$3F#$3E#$1F#$2F#$1A#$20#$72#$31#$31#$30#$2F#$3E#$31#$13#$38#$00 +
    #$17#$1F#$1A#$71#$71#$1E#$17#$1F#$1E#$20#$2B#$2F#$78#$2E#$10#$30 +
    #$3F#$3E#$70#$2F#$7A#$20#$12#$31#$31#$30#$2F#$3E#$31#$13#$38#$00 +
    #$37#$3F#$3A#$13#$13#$3E#$30#$3F#$3E#$20#$2B#$2F#$78#$2E#$30#$70 +
    #$7F#$7E#$1F#$2F#$1A#$20#$32#$31#$71#$70#$2F#$7E#$71#$13#$38#$00 +
    #$37#$3F#$3A#$13#$13#$30#$3E#$1E#$3F#$2F#$4E#$00#$00#$00#$00#$00 + { help }
    '';

type
 PtuiPendings = ^TtuiPendings;
 TtuiPendings = object(TCollection)
  procedure FreeItem(Item: Pointer); virtual;
 end;

 PtuiClocks = ^TtuiClocks;
 TtuiClocks = object(TView)
  LastCOA, LastPOA, LastTime, LastMem: Longint;
  Info: String;

  constructor Init(var Bounds: TRect);
  procedure Draw; virtual;
 public
  procedure Update;
 end;

 PtuiApplication = ^TtuiApplication;
 TtuiApplication = object(TApplication)
  constructor Init;
  function GetPalette: PPalette; virtual;
  procedure InitMenuBar; virtual;
  procedure InitStatusLine; virtual;
  procedure PutEvent(var Event: TEvent); virtual;
  procedure GetEvent(var Event: TEvent); virtual;
  procedure HandleEvent(var Event: TEvent); virtual;
  procedure Idle; virtual;
  procedure Run; virtual;
  destructor Done; virtual;
 public
  Pendings: PtuiPendings;
  Clocks: PtuiClocks;
  LogViewer: PLogViewer;
  AreasManager: PAreasManager;

  procedure coreLogWrite(var Block: TLogWriteBlock);
  procedure coreSetStatus(var Block: TSetStatusBlock);
  procedure coreClearStatus(var Block: TClearStatusBlock);
  procedure coreCancelled(var Block: TCancelledBlock);
  procedure Help(Context: Word);
 end;

const
 tuiScreenState         : PScreenState          = nil;

procedure tui;

function tuiMessageBoxDialogCreator(var Bounds: TRect; const Title: String): PDialog;

implementation

{ TtuiPendings }

procedure TtuiPendings.FreeItem(Item: Pointer);
 begin
  Dispose(PEvent(Item));
 end;

{ TtuiClocks }

constructor TtuiClocks.Init(var Bounds: TRect);
 begin
  inherited Init(Bounds);

  Update;
 end;

procedure TtuiClocks.Draw;
 var
  B: TDrawBuffer;
 begin
  MoveStr(B, Info, GetColor(1));

  WriteLine(0, 0, Size.X, Size.Y, B);
 end;

{ public methods }

procedure TtuiClocks.Update;
 var
  COA, POA, Time, Mem: Longint;
  NeedUpdate: Boolean;
  Bounds: TRect;
  DT: TDateTime;
 begin
  COA:=jt.COA;
  POA:=jt.POA;
  Mem:=MemAvail;

  GetCurrentDateTime(DT);
  DateTimeToUnixDateTime(DT, Time);

  NeedUpdate:=(COA <> LastCOA) or (POA <> LastPOA) or (Time <> LastTime) or (Mem <> LastMem);

  if NeedUpdate then
   begin
    LastCOA:=COA;
    LastPOA:=POA;
    LastTime:=Time;
    LastMem:=Mem;

    Info:=' [COA: ' + Long2Str(LastCOA) + ',' +
           ' POA: ' + Long2Str(LastPOA) + ',' +
           ' Mem: ' + Size2KB(LastMem) +
               '] ' + LeftPad(Long2Str(DT.Day), 2) +
                ' ' + MonthNumberToMonthString(DT.Month) +
                ' ' + LeftPad(Long2Str(DT.Year), 4) +
                ' ' + lz(DT.Hour) +
                ':' + lz(DT.Min) +
                ':' + lz(DT.Sec) +
                ' ';

    GetBounds(Bounds);

    Bounds.A.X:=Bounds.B.X - Length(Info);

    Locate(Bounds);

    DrawView;
   end;
 end;

{ TtuiApplication }

constructor TtuiApplication.Init;
 var
  Bounds: TRect;
 begin
  jt.PushTitle;

  {$IFDEF VIRTUALPASCAL}
  CodePage:=SysGetCodePage;
  SysTVInitCursor;
  InitKeyboard;
  DetectMouse;
  DetectVideo;
  {$ENDIF}

  SaveScreenState(tuiScreenState);

  inherited Init;

  RegisterObjects;
  RegisterType(RHelpTopic);
  RegisterType(RHelpIndex);

  InitClipboard(tcNormal);

  Pendings:=New(PtuiPendings, Init($10, $10));

  jt.core.ScreenHook(jtmPlugin, True);

  LogViewer:=New(PLogViewer, Init);

  Desktop^.Insert(LogViewer);

  Bounds.Assign(Size.X - 1, 0, Size.X, 1);

  Clocks:=New(PtuiClocks, Init(Bounds));

  Insert(Clocks);

  Message(@Self, evCommand, cmLogViewer, nil);
  Message(@Self, evCommand, cmAreasManager, nil);
 end;

function TtuiApplication.GetPalette: PPalette;
 const
  S: String[Length(CApplication)] = CApplication;
 begin
  GetPalette:=@S;
 end;

procedure TtuiApplication.InitMenuBar;
 var
  Bounds: TRect;
 begin
  GetExtent(Bounds);

  Bounds.B.Y:=Bounds.A.Y + 1;

  MenuBar:=New(PMenuBar, Init(Bounds, NewMenu(
   NewSubMenu('~F~ile', hcMenu, NewMenu(
    NewItem('~A~reas manager...', 'Alt-A', kbAltA, cmAreasManager, hcMenu,
    NewItem('~L~og viewer...', 'Alt-L', kbAltL, cmLogViewer, hcMenu,
    NewLine(
    NewItem('E~x~it', 'Alt-X', kbAltX, cmQuit, hcMenu,
    nil))))),
   NewSubMenu('~H~elp', hcMenu, NewMenu(
    NewItem('~H~elp', 'F1', kbF1, cmHelp, hcMenu,
    NewItem('~C~ontents', 'Ctrl-F1', kbCtrlF1, cmHelpContents, hcMenu,
    NewItem('~I~ndex', 'Shift-F1', kbShiftF1, cmHelpIndex, hcMenu,
    NewItem('~P~revious', 'Alt-F1', kbAltF1, cmHelpPrevious, hcMenu,
    nil))))),
   nil)))));
 end;

procedure TtuiApplication.InitStatusLine;
 function StdStatusKeys(Next: PStatusItem): PStatusItem;
  begin
   StdStatusKeys:=
    NewStatusKey('', kbF1, cmHelp,
    NewStatusKey('', kbCtrlF1, cmHelpContents,
    NewStatusKey('', kbShiftF1, cmHelpIndex,
    NewStatusKey('', kbAltF1, cmHelpPrevious,
    NewStatusKey('', kbAltF2, cmHelpHistory,
    App.StdStatusKeys(Next))))));
  end;
 var
  Bounds: TRect;
 begin
  GetExtent(Bounds);

  Bounds.A.Y:=Bounds.B.Y - 1;

  StatusLine:=New(PStatusLine, Init(Bounds,
   NewStatusDef(hcDesktop, hcDesktop,
    StdStatusKeys(
    NewStatusKey('~F10~ menu', kbF10, cmMenu,
    nil)),
   NewStatusDef(hcLogViewer, hcLogViewer,
    StdStatusKeys(
    NewStatusKey('~Alt-F3~ hide', kbAltF3, cmClose,
    nil)),
   NewStatusDef(hcAreasManager, hcAreasManager,
    StdStatusKeys(
    NewStatusKey('~Space~', kbSpace, cmaSelectToggle,
    NewStatusKey('~Ins~', kbIns, cmaSelect,
    NewStatusKey('~+~', kbGrayPlus, cmaSelectRegExp,
    NewStatusKey('~-~', kbGrayMinus, cmaUnselectRegExp,
    NewStatusKey('~*~', kbGrayAsterik, cmaToggleRegExp,
    NewStatusKey('~Alt-F10~', kbAltF10, cmaPopupMenu,
    NewStatusKey('~Enter~', kbEnter, cmaEditArea,
    NewStatusKey('~Alt-N~', kbAltN, cmaCreateArea,
    NewStatusKey('~Del~', kbDel, cmaDestroyArea,
    NewStatusKey('~Alt-C~', kbAltC, cmaConvertArea,
    NewStatusKey('~Alt-P~', kbAltP, cmaPassthroughMenu,
    NewStatusKey('~Alt-S~', kbAltS, cmaSubscribeArea,
    NewStatusKey('~Alt-U~', kbAltU, cmaUnsubscribeArea,
    NewStatusKey('~Alt-R~', kbAltR, cmaRelinkArea,
    NewStatusKey('~Alt-M~', kbAltM, cmaMiscMenu,
    nil)))))))))))))))),
   NewStatusDef(hcEditAreaDialog, hcEditAreaDialog,
    StdStatusKeys(
    nil),
   NewStatusDef(hcEditAreaDialogSubscribers, hcEditAreaDialogSubscribers,
    StdStatusKeys(
    NewStatusKey('~Enter~', kbEnter, cmasEditSubscriber,
    NewStatusKey('~Ins~', kbIns, cmasCreateSubscriber,
    NewStatusKey('~Del~', kbDel, cmasDestroySubscriber,
    nil)))),
   NewStatusDef(hcEditAreaSubscriberDialog, hcEditAreaSubscriberDialog,
    StdStatusKeys(
    NewStatusKey('~F2~ default', kbF2, cmaseDefault,
    NewStatusKey('~F3~ no changes', kbF3, cmaseNoChanges,
    nil))),
   NewStatusDef(hcChooseAreaSubscriberDialog, hcChooseAreaSubscriberDialog,
    StdStatusKeys(
    NewStatusKey('~Enter~ accept', kbEnter, cmOk,
    NewStatusKey('~Esc~ reject', kbEsc, cmCancel,
    nil))),
   NewStatusDef(hcLinksSelectorDialog, hcLinksSelectorDialog,
    StdStatusKeys(
    NewStatusKey('~Space~', kbSpace, cmlsbSelectToggle,
    NewStatusKey('~Ins~', kbIns, cmlsbSelect,
    nil))),
   NewStatusDef(hcRelinkSelectorDialog, hcRelinkSelectorDialog,
    StdStatusKeys(
    nil),
   NewStatusDef(hcHelpSystem, hcHelpSystem,
    StdStatusKeys(
    NewStatusKey('~Ctrl-F1~ contents', kbCtrlF1, cmHelpContents,
    NewStatusKey('~Shift-F1~ index', kbShiftF1, cmHelpIndex,
    NewStatusKey('~Alt-F1~ previous', kbAltF1, cmHelpPrevious,
    NewStatusKey('~Alt-F2~ history', kbAltF2, cmHelpHistory,
    NewStatusKey('', kbCtrlIns, cmCopy,
    nil)))))),
   NewStatusDef($0000, $FFFF,
    StdStatusKeys(
    nil),
   nil)))))))))))));
 end;

procedure TtuiApplication.PutEvent(var Event: TEvent);
 var
  AEvent: PEvent;
 begin
  New(AEvent);

  AEvent^:=Event;

  Pendings^.Insert(AEvent);
 end;

procedure TtuiApplication.GetEvent(var Event: TEvent);
 begin
  if Pendings^.Count > 0 then
   begin
    Event:=PEvent(Pendings^.At(0))^;

    Pendings^.AtFree(0);
   end
  else
   begin
    if Event.What = evCommand then
     repeat
      case Event.Command of
       cmHelp: Help(GetHelpCtx);
       cmHelpPrevious: Help(hcPrevious);
       cmHelpIndex: Help(hcIndex);
       cmHelpContents: Help(hcContents);
       cmHelpHistory: Help(hcHistory);
      else
       Break;
      end;

      ClearEvent(Event);
     until True;

    inherited GetEvent(Event);
   end;
 end;

procedure TtuiApplication.HandleEvent(var Event: TEvent);
 var
  R: ^TRect;
  R1: TRect;
  S: String;
 begin
  if Event.What = evCommand then
   repeat
    case Event.Command of
     cmLogViewer:
      begin
       if not LogViewer^.GetState(sfVisible) then
        LogViewer^.SetState(sfVisible, True);

       LogViewer^.Focus
      end;
     cmAreasManager:
      if AreasManager = nil then
       begin
        AreasManager:=New(PAreasManager, Init);

        Desktop^.Insert(AreasManager);
       end
      else
       AreasManager^.Focus;
     cmQueryRect:
      begin
       R:=Event.InfoPtr;

       Desktop^.GetExtent(R^);

       LogViewer^.GetBounds(R1);

       if LogViewer^.GetState(sfVisible) and not R^.Equals(R1) and (R^.B.X = R1.B.X) and (R^.B.Y = R1.B.Y) and
         (R^.A.X = R1.A.X) then
        R^.B.Y:=R1.A.Y;

       if R^.B.Y < Desktop^.Size.Y div 2 then
        Desktop^.GetExtent(R^);

       ClearEvent(Event);
      end;
     cmSetTitle:
      begin
       GetPStringEx(Event.InfoPtr, S);

       jt.SetTitle(S);

       DisposeStr(Event.InfoPtr);
      end;
     cmResetTitle: jt.SetTitle('jtm');
    else
     Break;
    end;

    ClearEvent(Event);
   until True;

  inherited HandleEvent(Event);
 end;

procedure TtuiApplication.Idle;
 begin
  {$IFDEF VIRTUALPASCAL}
  SysCtrlSleep(1);
  {$ENDIF}

  Clocks^.Update;

  inherited Idle;
 end;

procedure TtuiApplication.Run;
 begin
  inherited Run;
 end;

destructor TtuiApplication.Done;
 begin
  Dispose(Pendings, Done);

  DoneClipboard;

  inherited Done;

  LoadScreenState(tuiScreenState);

  jt.core.ScreenHook(jtmPlugin, False);

  {$IFDEF VIRTUALPASCAL}
  DoneSysError;
  DoneEvents;
  SysTVDoneMouse(True);

  if ExitProc <> nil then
   begin
    asm
     call ExitProc
    end;

    ExitProc:=nil;
   end;
  {$ENDIF}

  jt.PopTitle;
 end;

{ public methods }

procedure TtuiApplication.coreLogWrite(var Block: TLogWriteBlock);
 begin
  LogViewer^.LogWrite(Block.Symbol, Block.Message^);

  Block.Core:=False;
 end;

procedure TtuiApplication.coreSetStatus(var Block: TSetStatusBlock);
 begin
  LogViewer^.SetStatus(Block.Status^ + Block.Suffix^);

  Block.Core:=False;
 end;

procedure TtuiApplication.coreClearStatus(var Block: TClearStatusBlock);
 begin
  LogViewer^.ClearStatus;

  Block.Core:=False;
 end;

procedure TtuiApplication.coreCancelled(var Block: TCancelledBlock);
 begin
  Block.Core:=False;
 end;

procedure TtuiApplication.Help(Context: Word);
 var
  HelpWindow: PHelpWindow;
  HelpFile: PHelpFile;
  HelpStream: PStream;
  WindowNum: Longint;
  Target: PGroup;
  FName: String;
  Modal: Boolean;
  R: TRect;
 const
  Busy: Boolean = False;
 begin
  Modal:=((Application^.Current <> nil) and Application^.Current^.GetState(sfModal)) or
         ((Desktop^.Current <> nil) and Desktop^.Current^.GetState(sfModal));

  HelpWindow:=Message(Application, evBroadcast, cmHelpRespondMe, nil);

  if (HelpWindow <> nil) and (HelpWindow^.GetState(sfModal) = Modal) then
   begin
    Message(HelpWindow, evCommand, cmHelpSetTopic, Pointer(Context));

    Exit;
   end;

  if Busy then
   Exit;

  if Modal then
   Target:=Application
  else
   Target:=Desktop;

  {$IFDEF VIRTUALPASCAL}
  FName:='jtm32.hlp';
  {$ELSE}
  FName:='jtm.hlp';
  {$ENDIF}

  HelpStream:=New(PBufStream, Init(FName, stOpenRead, 2048));

  if (HelpStream = nil) or (HelpStream^.Status <> stOk) then
   begin
    Busy:=True;

    MessageBox(^C'Unable to open ' + FName + '.', nil, mfError or mfOkButton or mfInsertInApp);

    Busy:=False;

    Exit;
   end;

  HelpFile:=New(PHelpFile, Init(HelpStream));

  if (HelpFile = nil) or HelpFile^.Modified then
   begin
    Busy:=True;

    MessageBox(^C'Invalid format of ' + FName + '.', nil, mfError or mfOkButton or mfInsertInApp);

    Busy:=False;

    if HelpFile <> nil then
     begin
      HelpFile^.Modified:=False;

      Dispose(HelpFile, Done);
     end;

    Exit;
   end;

  if Modal then
   GetExtent(R)
  else
   Message(Application, evCommand, cmQueryRect, @R);

  if Modal then
   WindowNum:=0
  else
   WindowNum:=8;

  HelpWindow:=New(PHelpWindow, Init(R, HelpFile, Context, WindowNum));

  HelpWindow^.SetState(sfShadow, Modal);

  if Modal then
   begin
    Target^.ExecView(HelpWindow);

    Dispose(HelpWindow, Done);
   end
  else
   Target^.Insert(HelpWindow);
 end;

{ tui }

procedure tui;
 var
  tuiApplication: TtuiApplication;
 begin
  PerformPriorityAction('jtm');

  MessageBoxDialogCreator:=tuiMessageBoxDialogCreator;

  tuiApplication.Init;
  tuiApplication.Run;
  tuiApplication.Done;
 end;

{ tuiMessageBoxDialogCreator }

function tuiMessageBoxDialogCreator(var Bounds: TRect; const Title: String): PDialog;
 var
  Dialog: PtuiDialog;
 begin
  Dialog:=New(PtuiDialog, Init(Bounds, Title));

  Dialog^.HelpCtx:=hcMessageBox;

  tuiMessageBoxDialogCreator:=Dialog;
 end;

end.
