unit jtmUtil;

interface
uses
     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     Objects,
     Drivers,

     Views,
     Dialogs,
     Menus,
     App,

     MsgBox,

     rlopAPI,

     jtmCmn,
     jtmDef,
     jtmHelp,
     jtmTV,

     jtMHLcmn,

     Wizard;

const
 acsNone                = $0000;

 acsReadable            = $0001;
 acsWriteable           = $0002;
 acsPassive             = $0004;
 acsSubscribed          = $0008;

 acsAll                 = acsReadable or
                          acsWriteable or
                          acsPassive or
                          acsSubscribed;

type
 TACSStatus = (acssEnabled, acssDisabled, acssUnknown);

 { // Ask RegExp base dialog }

 PAskRegExpDialog = ^TAskRegExpDialog;
 TAskRegExpDialog = object(TtuiDialog)
  Expression: PtuiInputLine;
  re: Pointer;

  constructor Init(const ATitle: String; const HistoryId: Byte);
  procedure InitAdditionalControls; virtual;
  function Valid(Command: xWord): Boolean; virtual;
 end;

 { // Ask RegExp areas dialog }

 PAskRegExpAreasDialog = ^TAskRegExpAreasDialog;
 TAskRegExpAreasDialog = object(TAskRegExpDialog)
  Styles: PCheckBoxes;
  Formats: PCheckBoxes;

  procedure InitAdditionalControls; virtual;
  function Valid(Command: xWord): Boolean; virtual;
 end;

 { // AreaStyles control }

 PAreaStyles = ^TAreaStyles;
 TAreaStyles = object(TListBox)
  constructor Init(var Bounds: TRect; const AOwner: PWindow);
  function GetText(Item: xInteger; MaxLen: xInteger): String; virtual;
 public
  procedure SetDefault;
  procedure SetSingle(const Area: Pointer);
  procedure SetMultiple(const Selected: PAreasSelected);
  procedure GetSingle(const Area: Pointer);
  procedure GetMultiple(const Selected: PAreasSelected);
 end;

 { // AreaFormats control }

 PAreaFormats = ^TAreaFormats;
 TAreaFormats = object(TListBox)
  constructor Init(var Bounds: TRect; const AOwner: PWindow);
  function GetText(Item: xInteger; MaxLen: xInteger): String; virtual;
 public
  procedure SetDefault;
  procedure SetSingle(const Area: Pointer);
  procedure SetMultiple(const Selected: PAreasSelected);
  procedure GetSingle(const Area: Pointer);
  procedure GetMultiple(const Selected: PAreasSelected);
 end;

 { // AreaSubscribers control supplement }

 PAreaSubscribe = ^TAreaSubscribe;
 TAreaSubscribe = object(TObject)
  Area: Pointer;
  ACS: Longint;

  constructor Init(const AArea: Pointer; const AACS: Longint);
  procedure SetLinkArea(const LinkArea: Pointer);
 end;

 PAreaSubscriber = ^TAreaSubscriber;
 TAreaSubscriber = object(TSortedCollection)
  Link: Pointer;

  constructor Init(const ALink: Pointer);
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  function KeyOf(Item: Pointer): Pointer; virtual;
 public
  procedure SetLinkArea(const LinkArea: Pointer; const Area: Pointer);
  procedure GetLinkAreas(const Area: Pointer);
  function GetACS: String;
  function GetACSStatus(const ACS: Longint): TACSStatus;
  procedure SetACSStatus(const ACS: Longint; const Status: TACSStatus);
  function Empty: Boolean;
 end;

 PAreaSubscribersList = ^TAreaSubscribersList;
 TAreaSubscribersList = object(TSortedCollection)
  constructor Init;
  constructor InitSimple;
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  function Lookup(const Link: Pointer): PAreaSubscriber; virtual;
  destructor Done; virtual;
 public
  procedure SetArea(const Area: Pointer);
  procedure GetArea(const Area: Pointer);
  procedure Resort;
  function Joint: xInteger;
 private
  Temp: PAreaSubscribersList;
 end;

 { // EditAreaSubscriber dialog }

 PEditAreaSubscriberDialog = ^TEditAreaSubscriberDialog;
 TEditAreaSubscriberDialog = object(TtuiDialog)
  Subscriber: PAreaSubscriber;
  Creation: Boolean;

  constructor Init(const ASubscriber: PAreaSubscriber; const ACreation: Boolean);
  procedure HandleEvent(var Event: TEvent); virtual;
  procedure EndModal(Command: xWord); virtual;
  function Valid(Command: xWord): Boolean; virtual;
 private
  Readable,
  Writeable,
  Passive,
  Subscribed: PRadioButtons;

  procedure SetRadioStatus(const RadioButtons: PRadioButtons; const Status: TACSStatus);
 end;

 { // ChooseAreaSubscriber dialog }

 PChooseAreaSubscriberBox = ^TChooseAreaSubscriberBox;
 TChooseAreaSubscriberBox = object(TListViewer)
  List: PAreaSubscribersList;
  Joint: xInteger;

  constructor Init(var Bounds: TRect; const VSB: PScrollBar; const AList: PAreaSubscribersList);
  function GetText(Item: xInteger; MaxLen: xInteger): String; virtual;
 end;

 PChooseAreaSubscriberDialog = ^TChooseAreaSubscriberDialog;
 TChooseAreaSubscriberDialog = object(TtuiDialog)
  Box: PChooseAreaSubscriberBox;

  constructor Init(const AList: PAreaSubscribersList);
  function GetSubscriber: PAreaSubscriber;
 end;

 { // AreaSubscribers control }

 PAreaSubscribers = ^TAreaSubscribers;
 TAreaSubscribers = object(TListViewer)
  List: PAreaSubscribersList;

  constructor Init(var Bounds: TRect; const AOwner: PWindow);
  function GetText(Item: xInteger; MaxLen: xInteger): String; virtual;
  procedure HandleEvent(var Event: TEvent); virtual;
  procedure Update(const NewFocus: PAreaSubscriber);
  destructor Done; virtual;
 public
  procedure SetDefault;
  procedure SetSingle(const Area: Pointer);
  procedure SetMultiple(const Selected: PAreasSelected);
  procedure GetSingle(const Area: Pointer);
  procedure GetMultiple(const Selected: PAreasSelected);
  procedure EditSubscriber(const Subscriber: PAreaSubscriber);
  procedure CreateSubscriber;
  procedure DestroySubscriber;
 end;

 { // Progress control }

 PProgress = ^TProgress;
 TProgress = object(TView)
  Min, Max, Pos: Longint;

  constructor Init(var Bounds: TRect; const AMin, AMax: Longint);
  procedure SetPosition(const APos: Longint);
  procedure Draw; virtual;
 end;

 { // MenuPopup control }

 PtuiMenuPopup = ^TtuiMenuPopup;
 TtuiMenuPopup = object(TMenuPopup)
  destructor Done; virtual;
 end;

 { // LinksSelectorBox }

 PLinksSelectorBox = ^TLinksSelectorBox;
 TLinksSelectorBox = object(TListViewer)
  Links: PLinksSelected;
  Selected: PLinksSelected;

  constructor Init(var Bounds: TRect; const ALinks, ASelected: PLinksSelected; const VSB: PScrollBar);
  procedure Draw; virtual;
  function GetText(Item: xInteger; MaxLen: xInteger): String; virtual;
  function IsSelected(Item: xInteger): Boolean; virtual;
  procedure HandleEvent(var Event: TEvent); virtual;
  function Valid(Command: xWord): Boolean; virtual;
  destructor Done; virtual;
 public
  procedure DoSelect(const MoveDown: Boolean);
 end;

 { // LinksSelector dialog }

 PLinksSelectorDialog = ^TLinksSelectorDialog;
 TLinksSelectorDialog = object(TtuiDialog)
  constructor Init(const ASelected: PLinksSelected; const ATitle, AHeader: String);
 public
  function IsValidLink(const Link: Pointer): Boolean; virtual;
  procedure NoValidLinks; virtual;
 end;

 { // RLOP LinksSelector descendant }

 PrlopLinksSelectorDialog = ^TrlopLinksSelectorDialog;
 TrlopLinksSelectorDialog = object(TLinksSelectorDialog)
  function IsValidLink(const Link: Pointer): Boolean; virtual;
  procedure NoValidLinks; virtual;
 end;

function AskRegExp(const ATitle: String; const HistoryId: Byte): Pointer;
function AskRegExpAreas(const ATitle: String; const HistoryId: Byte; var Styles, Formats: Longint): Pointer;

function LinkAreaACS(const LinkArea: Pointer): Longint;
function MeanlessACS(const ACS: Longint): Boolean;

implementation
uses
     mbAPI;

{ TAskRegExpDialog }

constructor TAskRegExpDialog.Init(const ATitle: String; const HistoryId: Byte);
 var
  Bounds: TRect;
 begin
  Bounds.Assign(0, 0, 50, 7);

  inherited Init(Bounds, ATitle);

  Bounds.Assign(15, 2, Size.X - 5, 3);

  Expression:=New(PtuiInputLine, Init(Bounds, 255));

  Insert(Expression);

  Bounds.Assign(2, 2, 15, 3);

  Insert(New(PLabel, Init(Bounds, '~E~xpression:', Expression)));

  Bounds.Assign(Size.X - 5, 2, Size.X - 2, 3);

  Insert(New(PHistory, Init(Bounds, Expression, HistoryId)));

  InitAdditionalControls;

  Bounds.Assign(0, 0, Size.X div 4, 2);

  Bounds.Move(Size.X div 2 - 1, Size.Y - 3);

  Insert(New(PButton, Init(Bounds, '~O~K', cmOk, bfDefault)));

  Bounds.Move(Bounds.B.X - Bounds.A.X, 0);

  Insert(New(PButton, Init(Bounds, 'Cancel', cmCancel, bfNormal)));

  SelectNext(False);

  Options:=Options or ofCentered;
 end;

procedure TAskRegExpDialog.InitAdditionalControls;
 begin
 end;

function TAskRegExpDialog.Valid(Command: xWord): Boolean;
 procedure Error(const Message: String);
  begin
   MessageBox(^C + Message, nil, mfError or mfOKButton);

   Valid:=False;

   Expression^.Focus;
  end;
 begin
  if Command = cmOk then
   begin
    re:=jt.re.CreateStr(Expression^.Data^);

    if re = nil then Error('Invalid regexp.') else
    if not jt.re.IsValid(re) then
     begin
      Error(jt.re.GetErrorMsg(re));

      jt.re.Destroy(re);
     end
    else
     Valid:=True;
   end
  else
   Valid:=inherited Valid(Command);
 end;

{ TAskRegExpAreasDialog }

procedure TAskRegExpAreasDialog.InitAdditionalControls;
 var
  Bounds: TRect;
  K, id, Value: Longint;
  Name, ShortName: String;
 function CreateStyle: PSItem;
  begin
   if AtStyle(K, id, Name) then
    begin
     if K mod 2 = 0 then
      Inc(Bounds.B.Y);

     Inc(K);

     CreateStyle:=NewSItem(Pad(Name, Size.X div 2 - 9), CreateStyle);
    end
   else
    CreateStyle:=nil;
  end;
 function CreateFormat: PSItem;
  begin
   if jt.core.AtFormat(K, id, Name, ShortName) then
    begin
     if K mod 2 = 0 then
      Inc(Bounds.B.Y);

     Inc(K);

     Value:=Value or id;

     CreateFormat:=NewSItem(Pad(Name, Size.X div 2 - 9), CreateFormat);
    end
   else
    CreateFormat:=nil;
  end;
 begin
  { // styles }

  Bounds.Assign(2, 0, Size.X - 2, 0);

  K:=0;

  Styles:=New(PCheckBoxes, Init(Bounds, CreateStyle));

  Bounds.Move(0, 4);

  Styles^.Locate(Bounds);

  Insert(Styles);

  { // formats }

  Bounds.Assign(2, 0, Size.X - 2, 0);

  K:=0;
  Value:=0;

  Formats:=New(PCheckBoxes, Init(Bounds, CreateFormat));

  Bounds.Move(0, 5 + Styles^.Size.Y);

  Formats^.Locate(Bounds);

  Insert(Formats);

  { // change dialog's size }

  GetBounds(Bounds);

  Inc(Bounds.B.Y, Styles^.Size.Y + 1);
  Inc(Bounds.B.Y, Formats^.Size.Y + 1);

  Locate(Bounds);

  { // add another button }

  Bounds.Assign(1, Size.Y - 3, 1 + Size.X div 4, Size.Y - 1);

  Insert(New(PButton, Init(Bounds, '~A~ll', cmYes, bfNormal)));

  HelpCtx:=hcAreasSelectDialog;
 end;

function TAskRegExpAreasDialog.Valid(Command: xWord): Boolean;
 begin
  if Command = cmYes then
   begin
    Expression^.Data^:='.*';

    Styles^.Value:=$FFFFFFFF;

    Formats^.Value:=$FFFFFFFF;

    Command:=cmOk;
   end;

  Valid:=inherited Valid(Command);
 end;

{ TAreaStyles }

constructor TAreaStyles.Init(var Bounds: TRect; const AOwner: PWindow);
 var
  BarBounds: TRect;
  Bar: PScrollBar;
  Highest, id: Longint;
  Name: String;
 begin
  BarBounds.Assign(Bounds.B.X, Bounds.A.Y, Bounds.B.X + 1, Bounds.B.Y);

  Bar:=New(PScrollBar, Init(BarBounds));

  AOwner^.Insert(Bar);

  inherited Init(Bounds, 1, Bar);

  AOwner^.Insert(@Self);

  Highest:=0;

  while AtStyle(Highest, id, Name) do
   Inc(Highest);

  SetRange(Highest);
 end;

function TAreaStyles.GetText(Item: xInteger; MaxLen: xInteger): String;
 var
  id: Longint;
  Name: String;
 begin
  if State and sfDisabled <> 0 then
   if Item = 0 then
    GetText:='?'
   else
    GetText:=''
  else
   if AtStyle(Item, id, Name) then
    GetText:=Name
   else
    GetText:='no changes';
 end;

{ public methods }

procedure TAreaStyles.SetDefault;
 var
  Index: Longint;
 begin
  if SeekStyle(asEchomail, Index) then
   FocusItem(Index);
 end;

procedure TAreaStyles.SetSingle(const Area: Pointer);
 var
  Format: TAreaFormat;
  Index: Longint;
 begin
  jt.area.GetFormat(Area, Format);

  if SeekStyle(Format.Style, Index) then
   FocusItem(Index)
  else
   SetState(sfDisabled, True);
 end;

procedure TAreaStyles.SetMultiple(const Selected: PAreasSelected);
 var
  Style: Longint;
 function Perform(const Area: Pointer): Boolean; far;
  var
   Format: TAreaFormat;
  begin
   jt.area.GetFormat(Area, Format);

   if Style = -1 then
    begin
     Style:=Format.Style;

     Perform:=False;
    end
   else
    if Style <> Format.Style then
     begin
      Style:=-1;

      Perform:=True;
     end
    else
     Perform:=False;
  end;
 var
  Index: Longint;
 begin
  Style:=-1;

  Selected^.FirstThat(@Perform);

  if Style = -1 then
   begin
    SetRange(Range + 1);

    FocusItem(Range - 1);
   end
  else
   if SeekStyle(Style, Index) then
    FocusItem(Index)
   else
    SetState(sfDisabled, True);
 end;

procedure TAreaStyles.GetSingle(const Area: Pointer);
 var
  Format: TAreaFormat;
  Name: String;
  id: Longint;
 begin
  if not GetState(sfDisabled) then
   begin
    jt.area.GetFormat(Area, Format);

    if AtStyle(Focused, id, Name) then
     Format.Style:=id;

    jt.area.SetFormat(Area, Format);
   end;
 end;

procedure TAreaStyles.GetMultiple(const Selected: PAreasSelected);
 var
  id: Longint;
  Name: String;
 procedure Perform(const Area: Pointer); far;
  var
   Format: TAreaFormat;
  begin
   jt.area.GetFormat(Area, Format);

   Format.Style:=id;

   jt.area.SetFormat(Area, Format);
  end;
 begin
  if not GetState(sfDisabled) and AtStyle(Focused, id, Name) then
   Selected^.ForEach(@Perform);
 end;

{ TAreaFormats }

constructor TAreaFormats.Init(var Bounds: TRect; const AOwner: PWindow);
 var
  BarBounds: TRect;
  Bar: PScrollBar;
  Highest, id: Longint;
  FullName, ShortName: String;
 begin
  BarBounds.Assign(Bounds.B.X, Bounds.A.Y, Bounds.B.X + 1, Bounds.B.Y);

  Bar:=New(PScrollBar, Init(BarBounds));

  AOwner^.Insert(Bar);

  inherited Init(Bounds, 1, Bar);

  AOwner^.Insert(@Self);

  Highest:=0;

  while jt.core.AtFormat(Highest, id, FullName, ShortName) do
   Inc(Highest);

  SetRange(Highest);
 end;

function TAreaFormats.GetText(Item: xInteger; MaxLen: xInteger): String;
 var
  id: Longint;
  FullName, ShortName: String;
 begin
  if State and sfDisabled <> 0 then
   if Item = 0 then
    GetText:='?'
   else
    GetText:=''
  else
   if jt.core.AtFormat(Item, id, FullName, ShortName) then
    GetText:=FullName
   else
    GetText:='no changes';
 end;

{ public methods }

procedure TAreaFormats.SetDefault;
 var
  Index: Longint;
  mbi: PMessageBaseInterface;
 begin
  FocusItem(0);
{
  Index:=0;

  repeat
   mbi:=jt.core.mbi(Index);

   if (mbi <> nil) and (mbi^.AreaType = atSquish) then
    begin
     FocusItem(Index);

     Break;
    end;

   Inc(Index);
  until mbi = nil;
}
 end;

procedure TAreaFormats.SetSingle(const Area: Pointer);
 var
  Format: TAreaFormat;
  Index: Longint;
 begin
  jt.area.GetFormat(Area, Format);

  if jt.core.SeekFormat(Format.Format, Index) then
   FocusItem(Index)
  else
   SetState(sfDisabled, True);
 end;

procedure TAreaFormats.SetMultiple(const Selected: PAreasSelected);
 var
  Format: Longint;
 function Perform(const Area: Pointer): Boolean; far;
  var
   AreaFormat: TAreaFormat;
  begin
   jt.area.GetFormat(Area, AreaFormat);

   if Format = -1 then
    begin
     Format:=AreaFormat.Format;

     Perform:=False;
    end
   else
    if Format <> AreaFormat.Format then
     begin
      Format:=-1;

      Perform:=True;
     end
    else
     Perform:=False;
  end;
 var
  Index: Longint;
 begin
  Format:=-1;

  Selected^.FirstThat(@Perform);

  if Format = -1 then
   begin
    SetRange(Range + 1);

    FocusItem(Range - 1);
   end
  else
   if jt.core.SeekFormat(Format, Index) then
    FocusItem(Index)
   else
    SetState(sfDisabled, True);
 end;

procedure TAreaFormats.GetSingle(const Area: Pointer);
 var
  Format: TAreaFormat;
  Name: String;
  id: Longint;
 begin
  if not GetState(sfDisabled) then
   begin
    jt.area.GetFormat(Area, Format);

    if jt.core.AtFormat(Focused, id, Name, Name) then
     Format.Format:=id;

    jt.area.SetFormat(Area, Format);
   end;
 end;

procedure TAreaFormats.GetMultiple(const Selected: PAreasSelected);
 var
  id: Longint;
  Name: String;
 procedure Perform(const Area: Pointer); far;
  var
   Format: TAreaFormat;
  begin
   jt.area.GetFormat(Area, Format);

   Format.Format:=id;

   jt.area.SetFormat(Area, Format);
  end;
 begin
  if not GetState(sfDisabled) and jt.core.AtFormat(Focused, id, Name, Name) then
   Selected^.ForEach(@Perform);
 end;

{ TAreaSubscribe }

constructor TAreaSubscribe.Init(const AArea: Pointer; const AACS: Longint);
 begin
  inherited Init;

  Area:=AArea;
  ACS:=AACS;
 end;

procedure TAreaSubscribe.SetLinkArea(const LinkArea: Pointer);
 begin
  jt.link.area.SetReadable(LinkArea, ACS and acsReadable <> 0);
  jt.link.area.SetWriteable(LinkArea, ACS and acsWriteable <> 0);
  jt.link.area.SetPassive(LinkArea, ACS and acsPassive <> 0);
  jt.link.area.SetSubscribed(LinkArea, ACS and acsSubscribed <> 0);
 end;

{ TAreaSubscriber }

constructor TAreaSubscriber.Init(const ALink: Pointer);
 begin
  inherited Init($10, $10);

  Link:=ALink;
 end;

function TAreaSubscriber.Compare(Key1, Key2: Pointer): xInteger;
 var
  K1: Longint absolute Key1;
  K2: Longint absolute Key2;
 begin
  Compare:=CompareLongint(K1, K2);
 
{
  if K1 < K2 then Compare:=-1 else
  if K1 > K2 then Compare:=1 else
   Compare:=0;
}
 end;

function TAreaSubscriber.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=PAreaSubscribe(Item)^.Area;
 end;

{ public methods }

procedure TAreaSubscriber.SetLinkArea(const LinkArea: Pointer; const Area: Pointer);
 begin
  Insert(New(PAreaSubscribe, Init(Area, LinkAreaACS(LinkArea))));
 end;

procedure TAreaSubscriber.GetLinkAreas(const Area: Pointer);
 var
  LinkArea: Pointer;
  Index: xInteger;
 begin
  if Search(Area, Index) then
   begin
    LinkArea:=jt.link.SeekArea(Link, Area);

    if LinkArea = nil then
     LinkArea:=jt.link.CreateLinkArea(Link, Area);

    PAreaSubscribe(At(Index))^.SetLinkArea(LinkArea);
   end
  else
   begin
    LinkArea:=jt.link.SeekArea(Link, Area);

    if LinkArea <> nil then
     begin
      jt.link.DestroyLinkArea(Link, LinkArea);

      jt.LogWrite('?', 'JTM/as-rGetLinkAreas: brain is dead?');
     end;
   end;
 end;

function TAreaSubscriber.GetACS: String;
 var
  S: String[acsMaxLength];
 type
  TacsType = (acstReadable, acstWriteable, acstPassive, acstSubscribed);
 const
  acsChars: array[TacsType, TACSStatus] of Char = (
   ('R', '-', '?'),
   ('W', '-', '?'),
   ('P', '-', '?'),
   ('-', 'U', '?'));
 begin
  SetLength(S, acsMaxLength);

  S[1]:=acsChars[acstReadable, GetACSStatus(acsReadable)];
  S[2]:=acsChars[acstWriteable, GetACSStatus(acsWriteable)];
  S[3]:=acsChars[acstPassive, GetACSStatus(acsPassive)];
  S[4]:=acsChars[acstSubscribed, GetACSStatus(acsSubscribed)];

  GetACS:=S;
 end;

function TAreaSubscriber.GetACSStatus(const ACS: Longint): TACSStatus;
 var
  K: Longint;
 procedure Perform(const Subscribe: PAreaSubscribe); far;
  begin
   if Subscribe^.ACS and ACS <> 0 then
    Inc(K);
  end;
 begin
  K:=0;

  ForEach(@Perform);

  if K = Count then GetACSStatus:=acssEnabled else
  if K = 0 then GetACSStatus:=acssDisabled else
   GetACSStatus:=acssUnknown;
 end;

procedure TAreaSubscriber.SetACSStatus(const ACS: Longint; const Status: TACSStatus);
 procedure Perform(const Subscribe: PAreaSubscribe); far;
  begin
   case Status of
    acssEnabled: Subscribe^.ACS:=Subscribe^.ACS or ACS;
    acssDisabled: Subscribe^.ACS:=Subscribe^.ACS and not ACS;
   end;
  end;
 begin
  if Status <> acssUnknown then
   ForEach(@Perform);
 end;

function TAreaSubscriber.Empty: Boolean;
 function Perform(const Subscribe: PAreaSubscribe): Boolean; far;
  begin
   Perform:=not MeanlessACS(Subscribe^.ACS);
  end;
 begin
  Empty:=FirstThat(@Perform) = nil;
 end;

{ TAreaSubscribersList }

constructor TAreaSubscribersList.Init;
 begin
  inherited Init(jt.link.Count, $10);

  Temp:=New(PAreaSubscribersList, InitSimple);
 end;

constructor TAreaSubscribersList.InitSimple;
 begin
  inherited Init(jt.link.Count, $10);
 end;

function TAreaSubscribersList.Compare(Key1, Key2: Pointer): xInteger;
 var
  K1: PAreaSubscriber absolute Key1;
  K2: PAreaSubscriber absolute Key2;
  Address1, Address2: TAddress;
 begin
  if K1^.Empty and not K2^.Empty then Compare:=1 else
  if not K1^.Empty and K2^.Empty then Compare:=-1 else
   begin
    jt.link.GetAddress(K1^.Link, Address1);
    jt.link.GetAddress(K2^.Link, Address2);

    Compare:=AddressCompare(Address1, Address2);
   end;
 end;

function TAreaSubscribersList.Lookup(const Link: Pointer): PAreaSubscriber;
 function Perform(const Subscriber: PAreaSubscriber): Boolean; far;
  begin
   Perform:=Subscriber^.Link = Link;
  end;
 var
  Subscriber: PAreaSubscriber;
 begin
  Subscriber:=FirstThat(@Perform);

  if Subscriber = nil then
   begin
    Subscriber:=New(PAreaSubscriber, Init(Link));

    Insert(Subscriber);
   end;

  Lookup:=Subscriber;
 end;

destructor TAreaSubscribersList.Done;
 begin
  if Temp <> nil then
   Dispose(Temp, Done);

  inherited Done;
 end;

{ public methods }

procedure TAreaSubscribersList.SetArea(const Area: Pointer);
 var
  K: Longint;
  Link: Pointer;
  Subscriber: PAreaSubscriber;
 begin
  for K:=0 to jt.link.Count - 1 do
   begin
    Link:=jt.link.At(K);

    Subscriber:=Lookup(Link);

    if Area <> nil then
     Subscriber^.SetLinkArea(jt.link.SeekArea(Link, Area), Area);
   end;
 end;

procedure TAreaSubscribersList.GetArea(const Area: Pointer);
 procedure Perform(const Subscriber: PAreaSubscriber); far;
  begin
   Subscriber^.GetLinkAreas(Area);
  end;
 begin
  ForEach(@Perform);
 end;

procedure TAreaSubscribersList.Resort;
 procedure TransformToTemp(const Subscriber: PAreaSubscriber); far;
  begin
   Temp^.Insert(Subscriber);
  end;
 procedure TransformFromTemp(const Subscriber: PAreaSubscriber); far;
  begin
   Insert(Subscriber);
  end;
 begin
  ForEach(@TransformToTemp);

  DeleteAll;

  Temp^.ForEach(@TransformFromTemp);

  Temp^.DeleteAll;
 end;

function TAreaSubscribersList.Joint: xInteger;
 var
  Index: xInteger;
 function Perform(const Subscriber: PAreaSubscriber): Boolean; far;
  begin
   Dec(Index);

   Perform:=not Subscriber^.Empty;
  end;
 begin
  Index:=Count;

  if LastThat(@Perform) = nil then
   Index:=0
  else
   Inc(Index);

  Joint:=Index;
 end;

{ TEditAreaSubscriberDialog }

constructor TEditAreaSubscriberDialog.Init(const ASubscriber: PAreaSubscriber; const ACreation: Boolean);
 procedure CreateRadioButtons(var Bounds: TRect; var Buttons: PRadioButtons; const ALabel: String; const ACS: Longint);
  begin
   Buttons:=New(PRadioButtons, Init(Bounds,
    NewSItem('~e~nable',
    NewSItem('~d~isable',
    NewSItem('do ~n~ot change',
    nil)))));

   if Creation then
    Buttons^.EnableMask:=Buttons^.EnableMask and not (1 shl xInteger(acssUnknown));

   Insert(Buttons);

   Bounds.Move(-1, -1);

   Insert(New(PLabel, Init(Bounds, ALabel, Buttons)));

   Bounds.Move(1, 3);

   SetRadioStatus(Buttons, Subscriber^.GetACSStatus(ACS));
  end;
 var
  Bounds: TRect;
 begin
  Bounds.Assign(0, 0, 48, 13);

  inherited Init(Bounds, AddressOfLink(ASubscriber^.Link) + ' subscribe');

  Creation:=ACreation;

  Palette:=dpBlueDialog;

  Options:=Options or ofCentered;

  Subscriber:=ASubscriber;

  HelpCtx:=hcEditAreaSubscriberDialog;

  Bounds.Assign(2, 2, Size.X - 2, 3);

  CreateRadioButtons(Bounds, Readable, '~R~eadable:', acsReadable);
  CreateRadioButtons(Bounds, Writeable, '~W~riteable:', acsWriteable);
  CreateRadioButtons(Bounds, Passive, '~P~assive:', acsPassive);
  CreateRadioButtons(Bounds, Subscribed, '~S~ubscribed:', acsSubscribed);

  Bounds.Assign(0, 0, 11, 2);
  Bounds.Move(1, 10);

  Insert(New(PButton, Init(Bounds, 'De~f~ault', cmaseDefault, bfNormal)));

  Bounds.Move(Bounds.B.X - Bounds.A.X, 0);
  Inc(Bounds.B.X, 3);
  Insert(New(PButton, Init(Bounds, 'No ~c~hanges', cmaseNoChanges, bfNormal)));

  if Creation then
   DisableCommands([cmaseNoChanges]);

  Bounds.Move(Bounds.B.X - Bounds.A.X, 0);
  Dec(Bounds.B.X, 4);
  Insert(New(PButton, Init(Bounds, '~O~K', cmOk, bfDefault)));

  Bounds.Move(Bounds.B.X - Bounds.A.X, 0);
  Insert(New(PButton, Init(Bounds, 'Cancel', cmCancel, bfNormal)));

  SelectNext(False);
 end;

procedure TEditAreaSubscriberDialog.HandleEvent(var Event: TEvent);
 begin
  if Event.What = evCommand then
   repeat
    case Event.Command of
     cmaseDefault:
      begin
       SetRadioStatus(Readable, acssEnabled);
       SetRadioStatus(Writeable, acssEnabled);
       SetRadioStatus(Passive, acssDisabled);
       SetRadioStatus(Subscribed, acssEnabled);
      end;
     cmaseNoChanges:
      begin
       SetRadioStatus(Readable, acssUnknown);
       SetRadioStatus(Writeable, acssUnknown);
       SetRadioStatus(Passive, acssUnknown);
       SetRadioStatus(Subscribed, acssUnknown);
      end;
    else
     Break;
    end;

    ClearEvent(Event);
   until True;

  inherited HandleEvent(Event);
 end;

function TEditAreaSubscriberDialog.Valid(Command: xWord): Boolean;
 begin
  Valid:=inherited Valid(Command);
 end;

procedure TEditAreaSubscriberDialog.EndModal(Command: xWord);
 procedure Perform(const RadioButtons: PRadioButtons; const ACS: Longint);
  begin
   Subscriber^.SetACSStatus(ACS, TACSStatus(RadioButtons^.Value));
  end;
 begin
  if Command = cmOk then
   begin
    Perform(Readable, acsReadable);
    Perform(Writeable, acsWriteable);
    Perform(Passive, acsPassive);
    Perform(Subscribed, acsSubscribed);
   end;

  inherited EndModal(Command);
 end;

{ private methods }

procedure TEditAreaSubscriberDialog.SetRadioStatus(const RadioButtons: PRadioButtons; const Status: TACSStatus);
 begin
  RadioButtons^.Value:=Integer(Status);
  RadioButtons^.Sel:=Integer(Status);

  RadioButtons^.DrawView;
 end;

{ TChooseAreaSubscriberBox }

constructor TChooseAreaSubscriberBox.Init(var Bounds: TRect; const VSB: PScrollBar; const AList: PAreaSubscribersList);
 begin
  inherited Init(Bounds, 1, nil, VSB);

  List:=AList;

  Joint:=List^.Joint;

  SetRange(List^.Count - Joint);
 end;

function TChooseAreaSubscriberBox.GetText(Item: xInteger; MaxLen: xInteger): String;
 var
  Subscriber: PAreaSubscriber;
 begin
  Subscriber:=List^.At(Joint + Item);

  GetText:=AddressOfLink(Subscriber^.Link);
 end;

{ TChooseAreaSubscriberDialog }

constructor TChooseAreaSubscriberDialog.Init(const AList: PAreaSubscribersList);
 var
  Bounds: TRect;
 begin
  Bounds.Assign(0, 0, 30, 10);

  inherited Init(Bounds, 'Choose link');

  Options:=Options or ofCentered;

  HelpCtx:=hcChooseAreaSubscriberDialog;

  Bounds.Grow(-1, -1);

  Box:=New(PChooseAreaSubscriberBox, Init(Bounds, StandardScrollBar(sbVertical or sbHandleKeyboard), AList));

  Insert(Box);
 end;

function TChooseAreaSubscriberDialog.GetSubscriber: PAreaSubscriber;
 begin
  GetSubscriber:=Box^.List^.At(Box^.Joint + Box^.Focused);
 end;

{ TAreaSubscribers }

constructor TAreaSubscribers.Init(var Bounds: TRect; const AOwner: PWindow);
 var
  BarBounds: TRect;
  Bar: PScrollBar;
 begin
  BarBounds.Assign(Bounds.B.X, Bounds.A.Y, Bounds.B.X + 1, Bounds.B.Y);

  Bar:=New(PScrollBar, Init(BarBounds));

  AOwner^.Insert(Bar);

  inherited Init(Bounds, 3, nil, Bar);

  List:=New(PAreaSubscribersList, Init);

  AOwner^.Insert(@Self);
 end;

procedure TAreaSubscribers.HandleEvent(var Event: TEvent);
 begin
  if Event.What = evCommand then
   repeat
    case Event.Command of
     cmasEditSubscriber: EditSubscriber(List^.At(Focused));
     cmasCreateSubscriber: CreateSubscriber;
     cmasDestroySubscriber: DestroySubscriber;
    else
     Break;
    end;

    ClearEvent(Event);
   until True;

  inherited HandleEvent(Event);
 end;

function TAreaSubscribers.GetText(Item: xInteger; MaxLen: xInteger): String;
 var
  Subscriber: PAreaSubscriber;
  Address: TAddress;
 begin
  Subscriber:=List^.At(Item);

  if Subscriber <> nil then
   begin
    jt.link.GetAddress(Subscriber^.Link, Address);

    GetText:=Pad(AddressToStrEx(Address), MaxLen - acsMaxLength - 3) +
             Subscriber^.GetACS;
   end
  else
   GetText:='';
 end;

procedure TAreaSubscribers.Update(const NewFocus: PAreaSubscriber);
 var
  Index: xInteger;
 begin
  List^.Resort;

  SetRange(List^.Joint);

  if NewFocus <> nil then
   Index:=List^.IndexOf(NewFocus)
  else
   Index:=-1;

  if (Index <> -1) and (Focused <> Index) and (Index < Range) then
   FocusItem(Index);

  DrawView;
 end;

destructor TAreaSubscribers.Done;
 begin
  Dispose(List, Done);

  inherited Done;
 end;

{ public methods }

procedure TAreaSubscribers.SetDefault;
 begin
  Update(nil);
 end;

procedure TAreaSubscribers.SetSingle(const Area: Pointer);
 begin
  List^.SetArea(Area);

  Update(nil);
 end;

procedure TAreaSubscribers.SetMultiple(const Selected: PAreasSelected);
 procedure Perform(const Area: Pointer); far;
  begin
   List^.SetArea(Area);
  end;
 begin
  Selected^.ForEach(@Perform);

  Update(nil);
 end;

procedure TAreaSubscribers.GetSingle(const Area: Pointer);
 begin
  List^.GetArea(Area);
 end;

procedure TAreaSubscribers.GetMultiple(const Selected: PAreasSelected);
 procedure Perform(const Area: Pointer); far;
  begin
   List^.GetArea(Area);
  end;
 begin
  Selected^.ForEach(@Perform);
 end;

procedure TAreaSubscribers.EditSubscriber(const Subscriber: PAreaSubscriber);
 var
  Dialog: PEditAreaSubscriberDialog;
 begin
  if (Subscriber <> nil) and not Subscriber^.Empty then
   begin
    Dialog:=New(PEditAreaSubscriberDialog, Init(Subscriber, False));

    if Desktop^.ExecView(Dialog) = cmOk then
     Update(nil);

    Dispose(Dialog, Done);
   end;
 end;

procedure TAreaSubscribers.CreateSubscriber;
 var
  ChooseDialog: PChooseAreaSubscriberDialog;
  EditDialog: PEditAreaSubscriberDialog;
  Subscriber: PAreaSubscriber;
 begin
  ChooseDialog:=New(PChooseAreaSubscriberDialog, Init(List));

  if Desktop^.ExecView(ChooseDialog) = cmOk then
   Subscriber:=ChooseDialog^.GetSubscriber
  else
   Subscriber:=nil;

  Dispose(ChooseDialog, Done);

  if Subscriber <> nil then
   begin
    EditDialog:=New(PEditAreaSubscriberDialog, Init(Subscriber, True));

    Message(EditDialog, evCommand, cmaseDefault, nil);

    if Desktop^.ExecView(EditDialog) = cmOk then
     Update(Subscriber);

    Dispose(EditDialog, Done);
   end;
 end;

procedure TAreaSubscribers.DestroySubscriber;
 var
  Subscriber: PAreaSubscriber;
 begin
  Subscriber:=List^.At(Focused);

  if (Subscriber <> nil) and not Subscriber^.Empty then
   if MessageBox(^C'Sure to kill ' + AddressOfLink(Subscriber^.Link) + ' subscribe?', nil,
     mfConfirmation or mfYesButton or mfNoButton) = cmYes then
    begin
     Subscriber^.SetACSStatus(acsReadable, acssDisabled);
     Subscriber^.SetACSStatus(acsWriteable, acssDisabled);
     Subscriber^.SetACSStatus(acsPassive, acssDisabled);
     Subscriber^.SetACSStatus(acsSubscribed, acssDisabled);

     Update(nil);
    end;
 end;

{ TProgress }

constructor TProgress.Init(var Bounds: TRect; const AMin, AMax: Longint);
 begin
  inherited Init(Bounds);

  Min:=AMin;
  Max:=AMax;
  Pos:=AMin;
 end;

procedure TProgress.SetPosition(const APos: Longint);
 begin
  Pos:=APos;

  DrawView;
 end;

procedure TProgress.Draw;
 var
  B: TDrawBuffer;
  K: Longint;
 begin
  if Max - Min <> 0 then
   K:=Round(Pos / (Max - Min) * (Size.X - 1))
  else
   K:=Size.X - 1;

  if K < 0 then
   K:=0;

  if K > Size.X - 1 then
   K:=Size.X - 1;

  MoveChar(B, #$DB, GetColor(1), K + 1);
  MoveChar(B[K + 1], #$B0, GetColor(1), Size.X - K);

  WriteLine(0, 0, Size.X, Size.Y, B);
 end;

{ TtuiMenuPopup }

destructor TtuiMenuPopup.Done;
 begin
  TMenuBox.Done;
 end;

{ TLinksSelectorBox }

constructor TLinksSelectorBox.Init(var Bounds: TRect; const ALinks, ASelected: PLinksSelected; const VSB: PScrollBar);
 begin
  inherited Init(Bounds, 1, nil, VSB);

  Links:=ALinks;
  Selected:=ASelected;

  SetRange(Links^.Count);
 end;

procedure TLinksSelectorBox.Draw;
 var
  NormalColor, FocusedColor, SelectedColor, FocusedSelectedColor, Color: Byte;
  ItemSelected: Boolean;
  I, Item: Integer;
  B: TDrawBuffer;
 begin
  NormalColor:=$30;
  SelectedColor:=$3E;

  if State and (sfSelected + sfActive) = (sfSelected + sfActive) then
   begin
    FocusedColor:=$2F;
    FocusedSelectedColor:=$2E;
   end
  else
   begin
    FocusedColor:=$2F;
    FocusedSelectedColor:=$2E;
   end;

  Item:=TopItem;

  for I:=0 to Size.Y - 1 do
   begin
    ItemSelected:=IsSelected(Item);

    if (State and (sfSelected + sfActive) = (sfSelected + sfActive)) and (Focused = Item) and (Range > 0) then
     if ItemSelected then
      Color:=FocusedSelectedColor
     else
      Color:=FocusedColor
    else
     if (Item < Range) and ItemSelected then
      Color:=SelectedColor
     else
      Color:=NormalColor;

    MoveChar(B, ' ', Color, Size.X);

    if ItemSelected then
     MoveChar(B, '*', Color, 1);

    if Item < Range then
     MoveStr(B[1], GetText(Item, 0), Color);

    WriteLine(0, I, Size.X, 1, B);

    Inc(Item);
   end;
 end;

function TLinksSelectorBox.GetText(Item: xInteger; MaxLen: xInteger): String;
 var
  Link: Pointer;
  Address: TAddress;
 begin
  Link:=Links^.At(Item);

  if Link = nil then
   GetText:=''
  else
   begin
    jt.link.GetAddress(Link, Address);

    GetText:=AddressToStrEx(Address);
   end;
 end;

function TLinksSelectorBox.IsSelected(Item: xInteger): Boolean;
 var
  Index: xInteger;
 begin
  if Item < Links^.Count then
   IsSelected:=Selected^.Search(Links^.At(Item), Index)
  else
   IsSelected:=False;
 end;

procedure TLinksSelectorBox.HandleEvent(var Event: TEvent);
 begin
  if Event.What = evCommand then
   repeat
    case Event.Command of
     cmlsbSelectToggle: DoSelect(False);
     cmlsbSelect: DoSelect(True);
    else
     Break;
    end;

    ClearEvent(Event);
   until True;

  inherited HandleEvent(Event);
 end;

function TLinksSelectorBox.Valid(Command: xWord): Boolean;
 begin
  if (Command = cmOk) and (Selected^.Count = 0) then
   if Links^.At(Focused) <> nil then
    Selected^.Insert(Links^.At(Focused));

  if Command = cmValid then
   if Links^.Count = 0 then
    PLinksSelectorDialog(Owner)^.NoValidLinks;

  Valid:=inherited Valid(Command);
 end;

destructor TLinksSelectorBox.Done;
 begin
  Dispose(Links, Done);

  inherited Done;
 end;

{ public methods }

procedure TLinksSelectorBox.DoSelect(const MoveDown: Boolean);
 var
  Link: Pointer;
  Index: xInteger;
 begin
  Link:=Links^.At(Focused);

  if Link <> nil then
   begin
    if Selected^.Search(Link, Index) then
     Selected^.AtDelete(Index)
    else
     Selected^.AtInsert(Index, Link);

    if MoveDown then
     if Focused + 1 < Range then
      FocusItem(Focused + 1);

    DrawView;
   end;
 end;

{ TLinksSelectorDialog }

constructor TLinksSelectorDialog.Init(const ASelected: PLinksSelected; const ATitle, AHeader: String);
 var
  Bounds: TRect;
  Margin, K: Integer;
  Links: PLinksSelected;
  Link: Pointer;
 begin
  Bounds.Assign(0, 0, 70, 12);

  inherited Init(Bounds, ATitle);

  Options:=Options or ofCentered;

  HelpCtx:=hcLinksSelectorDialog;

  Links:=New(PLinksSelected, Init(jt.link.Count, $10));

  for K:=0 to jt.link.Count - 1 do
   begin
    Link:=jt.link.At(K);

    if IsValidLink(Link) then
     Links^.Insert(Link);
   end;

  Bounds.Assign(0, 0, 10, 2);
  Bounds.Move(Size.X - 2 - Bounds.B.X * 2, 2);

  Margin:=Bounds.A.X;

  Insert(New(PButton, Init(Bounds, '~O~K', cmOk, bfDefault)));

  Bounds.Move(Bounds.B.X - Bounds.A.X, 0);
  Insert(New(PButton, Init(Bounds, 'Cancel', cmCancel, bfNormal)));

  GetExtent(Bounds);

  Bounds.Grow(-1, -1);

  Inc(Bounds.A.Y, 3);

  Insert(New(PLinksSelectorBox, Init(Bounds, Links, ASelected, StandardScrollBar(sbVertical))));

  Bounds.Assign(3, 2, Margin, 4);
  Insert(New(PStaticText, Init(Bounds, AHeader)));
 end;

{ public methods }

function TLinksSelectorDialog.IsValidLink(const Link: Pointer): Boolean;
 begin
  IsValidLink:=True;
 end;

procedure TLinksSelectorDialog.NoValidLinks;
 begin
 end;

{ TrlopLinksSelectorDialog }

function TrlopLinksSelectorDialog.IsValidLink(const Link: Pointer): Boolean;
 var
  rc: String;
 begin
  IsValidLink:=rlopIsLinkValid(Link, rc);
 end;

procedure TrlopLinksSelectorDialog.NoValidLinks;
 begin
  MessageBox(^C'No valid links supported by RemoteLinkOperations plugin.', nil, mfError or mfCancelButton);
 end;

{ AskRegExp }

function AskRegExp(const ATitle: String; const HistoryId: Byte): Pointer;
 var
  Dialog: PAskRegExpDialog;
 begin
  Dialog:=New(PAskRegExpDialog, Init(ATitle, HistoryId));

  if Desktop^.ExecView(Dialog) = cmOk then
   AskRegExp:=Dialog^.re
  else
   AskRegExp:=nil;

  Dispose(Dialog, Done);
 end;

{ AskRegExpAreas }

function AskRegExpAreas(const ATitle: String; const HistoryId: Byte; var Styles, Formats: Longint): Pointer;
 var
  Dialog: PAskRegExpAreasDialog;
  Command: xWord;
 begin
  Dialog:=New(PAskRegExpAreasDialog, Init(ATitle, HistoryId));

  LoadDialogDefaults('AskRegExpAreasDialog', Dialog, @AskRegExpAreasDialogDefaults);

  Command:=Desktop^.ExecView(Dialog);

  case Command of
   cmOk, cmYes:
    begin
     if Command = cmOk then
      StoreDialogDefaults('AskRegExpAreasDialog', Dialog);

     AskRegExpAreas:=Dialog^.re;

     Styles:=Dialog^.Styles^.Value;
     Formats:=Dialog^.Formats^.Value;
    end
  else
   AskRegExpAreas:=nil;
  end;

  Dispose(Dialog, Done);
 end;

{ LinkAreaACS }

function LinkAreaACS(const LinkArea: Pointer): Longint;
 var
  ACS: Longint;
 begin
  ACS:=acsNone;

  if LinkArea <> nil then
   begin
    if jt.link.area.IsReadable(LinkArea) then ACS:=ACS or acsReadable;
    if jt.link.area.IsWriteable(LinkArea) then ACS:=ACS or acsWriteable;
    if jt.link.area.IsPassive(LinkArea) then ACS:=ACS or acsPassive;
    if jt.link.area.IsSubscribed(LinkArea) then ACS:=ACS or acsSubscribed;
   end;

  LinkAreaACS:=ACS;
 end;

{ MeanlessACS }

function MeanlessACS(const ACS: Longint): Boolean;
 begin
  MeanlessACS:=ACS and (acsReadable or acsWriteable) = 0;
 end;

end.