library Linkor;

uses
     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     Objects,
     Strings,

     LinkLink,

     PrioAPI,

     Wizard;

const
 LinkorName                             = 'Linkor';
 LinkorVersion                          = 2;

 LinkorPlugin           : Pointer       = nil;

 LinkFirst              : Pointer       = nil;
 LinkLast               : Pointer       = nil;
 LinkDisable            : Pointer       = nil;
 LinkInfo               : Pointer       = nil;

 LinkAreas              : PAreaList     = nil;

 LinkHaveSense          : Boolean       = False;

 LinkPerform                            = $1000;

 lfNone                                 = $0000;
 lfInfo                                 = $0001;

type
 PLinkAreaList = ^TLinkAreaList;
 TLinkAreaList = object(TAreaList)
  procedure SetAdditionalFlags(const Area: Pointer; const AreaName: String; var Flags: Longint); virtual;
 end;

{ TLinkAreaList }

procedure TLinkAreaList.SetAdditionalFlags(const Area: Pointer; const AreaName: String; var Flags: Longint);
 begin
  if jt.rests.Check(LinkInfo, nil, @AreaName) then
   Flags:=Flags or lfInfo;
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 var
  ProhibitFormats: PNumbers;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, LinkorName, LinkorVersion);

  LinkorPlugin:=Plugin;

  LinkFirst:=jt.rests.Add('LinkFirst', rsMinus);
  LinkLast:=jt.rests.Add('LinkLast', rsMinus);
  LinkDisable:=jt.rests.Add('LinkDisable', rsMinus);
  LinkInfo:=jt.rests.Add('LinkInfo', rsMinus);

  ProhibitFormats:=New(PNumbers, Init($2, $4));

  ProhibitFormats^.Add(afPassthrough);

  LinkAreas:=New(PLinkAreaList, Init(LinkFirst, LinkLast, LinkDisable, ProhibitFormats, 0));

  RegisterPriorityAction('linking');

  PluginSetup:=True;
 end;

{ PluginStartup }

procedure PluginStartup; export;
 begin
  if LinkHaveSense then
   jt.EnqueueCommand(LinkorPlugin, LinkPerform, 0, cwLink);
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
  Dispose(LinkAreas, Done);
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 const
  FirstTime: Boolean = True;
 function PerformLink(const Area: PAreaListItem): Boolean; far;
  var
   Linker: PLinker;
   Format: TAreaFormat;
  begin
   if FirstTime then
    begin
     jt.LogWrite('L', 'Linking areas');

     PerformPriorityAction('linking');

     FirstTime:=False;
    end;

   jt.area.GetFormat(Area^.Area, Format);

   Linker:=New(PLinker, Init(Area^.Area, LinkAreas^.IndexOf(Area), LinkAreas^.Count, Area^.Flags and lfInfo <> 0));

   if Linker <> nil then
    begin
     Linker^.Link;

     Dispose(Linker, Done);
    end;

   PerformLink:=jt.Cancelled;
  end;
 begin
  if Command = LinkPerform then
   begin
    if LinkAreas^.Full then
     LinkAreas^.AddList(jt.list.ImportList, LinkAreas^.FullFlags);

    if LinkAreas^.HaveSense then
     LinkAreas^.Prepare;

    if LinkAreas^.Count = 0 then
     jt.LogWrite('L', #0'Nothing to link.')
    else
     begin
      jt.PushTitle;

      LinkAreas^.FirstThat(@PerformLink);

      jt.PopTitle;
     end;
   end;
 end;

{ PluginHandleCmdlineSwitch }

function PluginHandleCmdlineSwitch(Switch: String): Boolean; export;
 procedure Full(const Flags: Longint);
  begin
   LinkAreas^.AddFull(Flags, jt.linker.Stub);

   LinkHaveSense:=True;
  end;
 procedure Mask(const Skip: Byte; const Flags: Longint);
  begin
   LinkAreas^.AddMask(Copy(Switch, Skip + 1, 255), Flags, jt.linker.Stub);

   LinkHaveSense:=True;
  end;
 begin
  StUpcaseEx(Switch);

  PluginHandleCmdlineSwitch:=True;

  if Switch = 'LINK' then Full(lfNone) else
  if Switch = 'LINKFORCE' then Full(alifIgnoreDisable) else
  if Copy(Switch, 1, 5) = 'LINK:' then Mask(5, lfNone) else
  if Copy(Switch, 1, 10) = 'LINKFORCE:' then Mask(10, alifIgnoreDisable) else
  if Switch = 'DEBUGLINKOR' then Debugger.Enabled:=True else
  if Switch = 'DEBUGDUMPLINKOR' then DebuggerDump:=True else
   PluginHandleCmdlineSwitch:=False;
 end;

{ PluginShowHelp }

procedure PluginShowHelp; export;
 begin
  jt.PrintStr(' LINK[:re]               link importlist or regexp if not omitted'#13#10);
  jt.PrintStr(' LINKFORCE[:re]          link, LinkDisable have no influence'#13#10);
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleCmdlineSwitch,
 PluginShowHelp;

begin
 Assign(Output, '');
 Rewrite(Output);
end.