unit mbjmCore;

interface
uses
     mbAPI,
     mbPLUG,

     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     LinkAPI,

     Objects,
     Strings,

     Thunks,

     jtMHLcmn,
     jtMHLstm,

     RStream,

     mbjmCmn,

     Wizard;

type
 PLongint = ^Longint;

 PJamMapItem = ^TJamMapItem;
 TJamMapItem = packed record
  No, IndexNo: Longint;
 end;

 PJamMap = ^TJamMap;
 TJamMap = object(TCollection)
  procedure FreeItem(Item: Pointer); virtual;
 public
  ModifyCounter: Longint;
  Synchronous: Boolean;
 end;

 PJamMessageBase = ^TJamMessageBase;
 TJamMessageBase = object(TMessageBase)
  constructor Init(const APath: String; const Create: Boolean; const ABaseType: TBaseType; var rc: String);
  destructor Done; virtual;
 public
  Path: PString;
  HeaderStream, DataStream, IndexStream: PDosStreamEx;
  BaseHeader: TJamBaseHeader;
  Map: PJamMap;

  function Add(const Message: Pointer; var rc: String; const Flags: Longint): Boolean; virtual;
  function Count: Longint; virtual;
  function Load(const No: Longint; var Message: Pointer; var rc: String; const Flags: Longint): Boolean; virtual;
  function Store(const No: Longint; const Message: Pointer; var rc: String; const Flags: Longint): Boolean; virtual;
  function Kill(const No: Longint; var rc: String): Boolean; virtual;
  function NoToGlobal(const No: Longint): Longint; virtual;
  function GlobalToNo(const Global: Longint): Longint; virtual;

  function LockBase: Boolean; virtual;
  procedure UnlockBase; virtual;
 private
  Inited: Boolean;

  function ReadBaseHeader(var rc: String): Boolean;
  function WriteBaseHeader(var rc: String): Boolean;

  function CheckMap(const Force: Boolean; var rc: String): Boolean;
  function GetIndexNo(const No: Longint; var IndexNo: Longint; var rc: String): Boolean;

  procedure JTMHLToJam(const msg: Pointer; var MessageHeader: TJamMessageHeader;
   var MessageIndex: TJamMessageIndex; var SubFields: PSubFields; var Body: PRamStream);
  procedure JamToJTMHL(var MessageHeader: TJamMessageHeader; const SubFields: PSubFields; const Body: PRamStream;
   var msg: Pointer);

  procedure DegrowLastreads(const IndexNo: Longint);
 public
  procedure LoadLinkInfo(var Block: TLoadLinkMessageBaseBlock); virtual;
  procedure StoreLinkInfo(var Block: TStoreLinkMessageBaseBlock); virtual;
  function ExtendedOpened(const Operation: Longint; var Data): Boolean; virtual;

  function GetLastRead(var LastRead: Longint): Boolean;
  function SetLastRead(var LastRead: Longint): Boolean;
 end;

procedure AdditionalDisposer(const Additional: Pointer); export;

implementation

{ TJamMap }

procedure TJamMap.FreeItem(Item: Pointer);
 begin
  Dispose(PJamMapItem(Item));
 end;

{ TJamMessageBase }

constructor TJamMessageBase.Init(const APath: String; const Create: Boolean; const ABaseType: TBaseType; var rc: String);
 var
  Wrong: Byte;
  DT: TDateTime;
 begin
  inherited Init(APath, Create, ABaseType, rc);

  Path:=NewStr(ForceExtension(APath, ''));

  if Path = nil then
   begin
    rc:='invalid path';

    Done;

    Fail;
   end;

  HeaderStream:=New(PDosStreamEx, Init(Path^ + 'jhr', stOpen or stDenyNone));
  DataStream:=New(PDosStreamEx, Init(Path^ + 'jdt', stOpen or stDenyNone));
  IndexStream:=New(PDosStreamEx, Init(Path^ + 'jdx', stOpen or stDenyNone));

  Wrong:=0;

  if HeaderStream^.Status <> stOk then
   Inc(Wrong);

  if IndexStream^.Status <> stOk then
   Inc(Wrong);

  if DataStream^.Status <> stOk then
   if Wrong = 0 then
    begin
     Dispose(DataStream, Done);

     DataStream:=New(PDosStreamEx, Init(Path^ + 'jdt', stCreate or stDenyNone));

     if DataStream^.Status <> stOk then
      Inc(Wrong);
    end
   else
    Inc(Wrong);

  case Wrong of
   0:
    begin
     HeaderStream^.Read(BaseHeader, SizeOf(BaseHeader));

     if HeaderStream^.Status <> stOk then
      begin
       rc:='unable to read header of ' + Path^ + 'jhr: rc=' + Long2Str(HeaderStream^.Status);

       Done;

       Fail;
      end;

     Map:=New(PJamMap, Init(BaseHeader.ActiveMessages or $100, $100));

     Map^.ModifyCounter:=not BaseHeader.ModifyCounter;

     Inited:=True;

     Exit;
    end;
   3:
    begin
     if ExistFile(Path^ + 'jhr') or ExistFile(Path^ + 'jdt') or ExistFile(Path^ + 'jdx') then
      begin
       rc:='unable to open ' + Path^ + ' jam base (used by another process or grunged?)';

       Done;

       Fail;
      end;

     if not Create then
      begin
       rc:='jam base ' + Path^ + ' absent';

       Done;

       Fail;
      end;

     if not CreateDirectory(JustPathName(Path^)) then
      begin
       rc:='failed to create directory ' + JustPathName(Path^);

       Done;

       Fail;
      end;

     Dispose(HeaderStream, Done);
     Dispose(DataStream, Done);
     Dispose(IndexStream, Done);

     HeaderStream:=New(PDosStreamEx, Init(Path^ + 'jhr', stCreate or stDenyNone));
     DataStream:=New(PDosStreamEx, Init(Path^ + 'jdt', stCreate or stDenyNone));
     IndexStream:=New(PDosStreamEx, Init(Path^ + 'jdx', stCreate or stDenyNone));

     if HeaderStream^.Status <> stOk then
      begin
       rc:='unable to create ' + Path^ + 'jhr: rc=' + Long2Str(HeaderStream^.Status);

       Done;

       Fail;
      end;

     if DataStream^.Status <> stOk then
      begin
       rc:='unable to create ' + Path^ + 'jdt: rc=' + Long2Str(DataStream^.Status);

       Done;

       Fail;
      end;

     if IndexStream^.Status <> stOk then
      begin
       rc:='unable to create ' + Path^ + 'jdx: rc=' + Long2Str(IndexStream^.Status);

       Done;

       Fail;
      end;

     with BaseHeader do
      begin
       StrPCopy(Signature, 'JAM');

       GetCurrentDateTime(DT);
       DateTimeToUnixDateTime(DT, CreatedAt);

       ModifyCounter:=1;
       ActiveMessages:=0;
       PasswordCRC32:=-1;
       BaseMessageNumber:=1;
       HighWater:=0;

       FillChar(Extra, SizeOf(Extra), 0);
      end;

     HeaderStream^.Write(BaseHeader, SizeOf(BaseHeader));

     if HeaderStream^.Status <> stOk then
      begin
       rc:='unable to write header of ' + Path^ + 'jhr: rc=' + Long2Str(HeaderStream^.Status);

       Done;

       Fail;
      end;

     HeaderStream^.Flush;

     Map:=New(PJamMap, Init($100, $100));

     Map^.ModifyCounter:=not BaseHeader.ModifyCounter;

     Inited:=True;
    end;
  else
   if HeaderStream^.Status <> stOk then
    rc:='unable to open ' + Path^ + 'jhr: rc=' + Long2Str(HeaderStream^.Status)
   else
    if DataStream^.Status <> stOk then
     rc:='unable to open ' + Path^ + 'jdt: rc=' + Long2Str(DataStream^.Status)
    else
     if IndexStream^.Status <> stOk then
      rc:='unable to open ' + Path^ + 'jdx: rc=' + Long2Str(IndexStream^.Status);

   Done;

   Fail;
  end;
 end;

destructor TJamMessageBase.Done;
 begin
  if Map <> nil then
   Dispose(Map, Done);

  DisposeStr(Path);

  Dispose(IndexStream, Done);

  Dispose(DataStream, Done);

  Dispose(HeaderStream, Done);

  inherited Done;
 end;

{ public methods }

function TJamMessageBase.Add(const Message: Pointer; var rc: String; const Flags: Longint): Boolean;
 var
  MessageHeader: TJamMessageHeader;
  MessageIndex: TJamMessageIndex;
  SubFields: PSubFields;
  Body: PRamStream;
 begin
  Add:=False;

  if Lock then
   begin
    if ReadBaseHeader(rc) then
     begin
      JTMHLToJam(Message, MessageHeader, MessageIndex, SubFields, Body);

      MessageHeader.MessageNumber:=BaseHeader.BaseMessageNumber + BaseHeader.ActiveMessages;
      MessageHeader.TextOffset:=DataStream^.GetSize;

      MessageIndex.HeaderOffset:=HeaderStream^.GetSize;

      Inc(BaseHeader.ActiveMessages);
      Inc(BaseHeader.ModifyCounter);

      HeaderStream^.Seek(MessageIndex.HeaderOffset);
      HeaderStream^.Write(MessageHeader, SizeOf(MessageHeader));

      if HeaderStream^.Status = stOk then
       SubFields^.Write(HeaderStream);

      if HeaderStream^.Status = stOk then
       begin
        Body^.Seek(0);
        DataStream^.Seek(MessageHeader.TextOffset);
        DataStream^.CopyFrom(Body^, Body^.GetSize);

        if DataStream^.Status = stOk then
         begin
          IndexStream^.Seek(IndexStream^.GetSize);
          IndexStream^.Write(MessageIndex, SizeOf(MessageIndex));

          if IndexStream^.Status = stOk then
           if WriteBaseHeader(rc) then
            Add:=True
           else
          else
           begin
            rc:='failed to write message index: rc=' + Long2Str(IndexStream^.Status);

            IndexStream^.Reset;
           end;
         end
        else
         begin
          rc:='failed to write message body: rc=' + Long2Str(DataStream^.Status);

          DataStream^.Reset;
         end;
       end
      else
       begin
        rc:='failed to write message header: rc=' + Long2Str(HeaderStream^.Status);

        HeaderStream^.Reset;
       end;

      Dispose(SubFields, Done);
      Dispose(Body, Done);
     end;

    Unlock;
   end
  else
   rc:='unable to lock base';

  if FlushAfterAdd then
   begin
    HeaderStream^.Flush;
    DataStream^.Flush;
    IndexStream^.Flush;
   end;
 end;

function TJamMessageBase.Count: Longint;
 begin
  Count:=BaseHeader.ActiveMessages;
 end;

function TJamMessageBase.Load(const No: Longint; var Message: Pointer; var rc: String; const Flags: Longint): Boolean;
 var
  IndexNo: Longint;
  MessageIndex: TJamMessageIndex;
  MessageHeader: TJamMessageHeader;
  SubFields: PSubFields;
  Body: PRamStream;
 begin
  Load:=False;

  if not GetIndexNo(No, IndexNo, rc) then
   Exit;

  IndexStream^.Seek(IndexNo * SizeOf(MessageIndex));
  IndexStream^.Read(MessageIndex, SizeOf(MessageIndex));

  if IndexStream^.Status <> stOk then
   begin
    rc:='unable to load message index at ' + HexL(IndexNo * SizeOf(MessageIndex)) + ': rc=' + Long2Str(IndexStream^.Status);

    IndexStream^.Reset;

    Exit;
   end;

  if MessageIndex.HeaderOffset = -1 then
   begin
    rc:='message left out';

    Exit;
   end;

  HeaderStream^.Seek(MessageIndex.HeaderOffset);
  HeaderStream^.Read(MessageHeader, SizeOf(MessageHeader));

  if HeaderStream^.Status <> stOk then
   begin
    rc:='unable to load message header at ' + HexL(MessageIndex.HeaderOffset) + ': rc=' + Long2Str(HeaderStream^.Status);

    HeaderStream^.Reset;

    Exit;
   end;

  if MessageHeader.Attributes and jamaDeleted <> 0 then
   begin
    rc:='unable to load deleted message ' + Long2Str(No) + ' at ' + HexL(MessageIndex.HeaderOffset) + ' (brain is dead?)';

    Exit;
   end;

  if MessageHeader.Attributes and jamaLocked <> 0 then
   begin
    rc:='message is locked';

    Exit;
   end;

  SubFields:=New(PSubFields, Init($10, $10));

  if not SubFields^.Read(HeaderStream, MessageHeader.SubFieldsLength, rc) then
   begin
    Dispose(SubFields, Done);

    Exit;
   end;

  Body:=New(PRamStream, Init);

  if Flags and mfNoText = 0 then
   begin
    DataStream^.Seek(MessageHeader.TextOffset);
    Body^.CopyFrom(DataStream^, MessageHeader.TextLength);

    if DataStream^.Status <> stOk then
     begin
      rc:='failed to load body at ' + HexL(MessageHeader.TextOffset) + ': rc=' + Long2Str(DataStream^.Status);

      DataStream^.Reset;

      Exit;
     end;
   end;

  JamToJTMHL(MessageHeader, SubFields, Body, Message);

  Dispose(SubFields, Done);
  Dispose(Body, Done);

  Load:=True;
 end;

function TJamMessageBase.Store(const No: Longint; const Message: Pointer; var rc: String; const Flags: Longint): Boolean;
 var
  IndexNo: Longint;
  MessageHeader, AMessageHeader: TJamMessageHeader;
  MessageIndex: TJamMessageIndex;
  SubFields: PSubFields;
  Body: PRamStream;
 begin
  Store:=False;

  if not GetIndexNo(No, IndexNo, rc) then
   Exit;

  if Lock then
   begin
    IndexStream^.Seek(IndexNo * SizeOf(MessageIndex));
    IndexStream^.Read(MessageIndex, SizeOf(MessageIndex));

    if IndexStream^.Status = stOk then
     begin
      if ReadBaseHeader(rc) then
       begin
        JTMHLToJam(Message, MessageHeader, MessageIndex, SubFields, Body);

        HeaderStream^.Seek(MessageIndex.HeaderOffset);
        HeaderStream^.Read(AMessageHeader, SizeOf(AMessageHeader));

        if HeaderStream^.Status = stOk then
         if (AMessageHeader.TextLength >= Body^.GetSize) and
            (AMessageHeader.SubFieldsLength >= MessageHeader.SubFieldsLength) then
          begin
           MessageHeader.MessageNumber:=AMessageHeader.MessageNumber;
           MessageHeader.TextOffset:=AMessageHeader.TextOffset;

           if Flags and mfNoText = 0 then
            MessageHeader.TextLength:=Body^.GetSize
           else
            MessageHeader.TextLength:=AMessageHeader.TextLength;

           Inc(BaseHeader.ModifyCounter);

           HeaderStream^.Seek(MessageIndex.HeaderOffset);
           HeaderStream^.Write(MessageHeader, SizeOf(MessageHeader));

           if HeaderStream^.Status = stOk then
            SubFields^.Write(HeaderStream);

           if HeaderStream^.Status = stOk then
            begin
             if Flags and mfNoText = 0 then
              begin
               Body^.Seek(0);

               DataStream^.Seek(MessageHeader.TextOffset);
               DataStream^.CopyFrom(Body^, Body^.GetSize);
              end;

             if DataStream^.Status = stOk then
              begin
               IndexStream^.Seek(IndexNo * SizeOf(MessageIndex));
               IndexStream^.Write(MessageIndex, SizeOf(MessageIndex));

               Inc(BaseHeader.ModifyCounter);

               if IndexStream^.Status = stOk then
                if WriteBaseHeader(rc) then
                 Store:=True
                else
               else
                begin
                 rc:='failed to write message index: rc=' + Long2Str(IndexStream^.Status);

                 IndexStream^.Reset;
                end;
              end
             else
              begin
               rc:='failed to write message body: rc=' + Long2Str(DataStream^.Status);

               DataStream^.Reset;
              end;
            end
           else
            begin
             rc:='failed to write message header: rc=' + Long2Str(HeaderStream^.Status);

             HeaderStream^.Reset;
            end;
          end
         else
          begin
           MessageHeader.MessageNumber:=AMessageHeader.MessageNumber;

           if Flags and mfNoText = 0 then
            begin
             AMessageHeader.TextLength:=0;
             AMessageHeader.TextOffset:=-1;

             MessageHeader.TextLength:=Body^.GetSize;
             MessageHeader.TextOffset:=DataStream^.GetSize;
            end
           else
            begin
             MessageHeader.TextLength:=AMessageHeader.TextLength;
             MessageHeader.TextOffset:=AMessageHeader.TextOffset;

             AMessageHeader.TextLength:=0;
             AMessageHeader.TextOffset:=-1;
            end;

           AMessageHeader.Attributes:=AMessageHeader.Attributes or jamaDeleted;
           AMessageHeader.MessageNumber:=-1;

           HeaderStream^.Seek(MessageIndex.HeaderOffset);
           HeaderStream^.Write(AMessageHeader, SizeOf(AMessageHeader));

           if HeaderStream^.Status <> stOk then
            HeaderStream^.Reset;

           MessageIndex.HeaderOffset:=HeaderStream^.GetSize;

           HeaderStream^.Seek(MessageIndex.HeaderOffset);
           HeaderStream^.Write(MessageHeader, SizeOf(MessageHeader));

           if HeaderStream^.Status = stOk then
            SubFields^.Write(HeaderStream);

           if HeaderStream^.Status = stOk then
            begin
             Body^.Seek(0);
             DataStream^.Seek(MessageHeader.TextOffset);
             DataStream^.CopyFrom(Body^, Body^.GetSize);

             if DataStream^.Status = stOk then
              begin
               IndexStream^.Seek(IndexNo * SizeOf(MessageIndex));
               IndexStream^.Write(MessageIndex, SizeOf(MessageIndex));

               Inc(BaseHeader.ModifyCounter);

               if IndexStream^.Status = stOk then
                if WriteBaseHeader(rc) then
                 Store:=True
                else
               else
                begin
                 rc:='failed to write message index: rc=' + Long2Str(IndexStream^.Status);

                 IndexStream^.Reset;
                end;
              end
             else
              begin
               rc:='failed to write message body: rc=' + Long2Str(DataStream^.Status);

               DataStream^.Reset;
              end;
            end
           else
            begin
             rc:='failed to write message header: rc=' + Long2Str(HeaderStream^.Status);

             HeaderStream^.Reset;
            end;
          end
        else
         rc:='failed to read message header at ' + HexL(MessageIndex.HeaderOffset) + ': rc=' + Long2Str(HeaderStream^.Status);

        Dispose(SubFields, Done);
        Dispose(Body, Done);
       end;
     end
    else
     rc:='failed to read index at ' + HexL(IndexNo * SizeOf(MessageIndex));

    Unlock;
   end
  else
   rc:='unable to lock base';

  if FlushAfterStore then
   begin
    HeaderStream^.Flush;
    DataStream^.Flush;
    IndexStream^.Flush;
   end;
 end;

function TJamMessageBase.Kill(const No: Longint; var rc: String): Boolean;
 var
  MessageIndex: TJamMessageIndex;
  MessageHeader: TJamMessageHeader;
  TempStream: PRamStream;
  IndexNo: Longint;
 begin
  Kill:=False;

  if not GetIndexNo(No, IndexNo, rc) then
   Exit;

  if not Lock then
   begin
    rc:='failed to lock base';

    Exit;
   end;

  if not ReadBaseHeader(rc) then
   Exit;

  IndexStream^.Seek(IndexNo * SizeOf(MessageIndex));
  IndexStream^.Read(MessageIndex, SizeOf(MessageIndex));

  if HardDelete then
   begin
    HeaderStream^.Seek(MessageIndex.HeaderOffset);
    HeaderStream^.Read(MessageHeader, SizeOf(MessageHeader));

    if HeaderStream^.Status = stOk then
     begin
      MessageHeader.Attributes:=MessageHeader.Attributes or jamaDeleted;

      HeaderStream^.Seek(MessageIndex.HeaderOffset);
      HeaderStream^.Write(MessageHeader, SizeOf(MessageHeader));
     end;

    if HeaderStream^.Status = stOk then
     begin
      MessageIndex.MSGTOCRC:=-1;
      MessageIndex.HeaderOffset:=-1;

      IndexStream^.Seek(IndexNo * SizeOf(MessageIndex));
      IndexStream^.Write(MessageIndex, SizeOf(MessageIndex));

      if IndexStream^.Status = stOk then
       begin
        Dec(BaseHeader.ActiveMessages);
        Inc(BaseHeader.ModifyCounter);

        Kill:=WriteBaseHeader(rc) and CheckMap(True, rc);
       end
      else
       begin
        rc:='failed to update index at ' + HexL(IndexNo * SizeOf(MessageIndex)) + ': rc=' + Long2Str(IndexStream^.Status);

        IndexStream^.Reset;
       end;
{
      TempStream:=New(PRamStream, Init);

      TempStream^.CopyFrom(IndexStream^, IndexStream^.GetSize - IndexStream^.GetPos);

      if IndexStream^.Status = stOk then
       begin
        TempStream^.Seek(0);

        IndexStream^.Seek(IndexNo * SizeOf(MessageIndex));
        IndexStream^.CopyFrom(TempStream^, TempStream^.GetSize);
        IndexStream^.Truncate;
       end;

      if IndexStream^.Status = stOk then
       begin
        if not Map^.Synchronous then
         Map^.AtFree(No);

        Dec(BaseHeader.ActiveMessages);
        Inc(BaseHeader.ModifyCounter);

        Kill:=WriteBaseHeader(rc);

        DegrowLastreads(IndexNo);
       end
      else
       begin
        rc:='failed to shift index at ' + HexL(IndexNo * SizeOf(MessageIndex)) + ': rc=' + Long2Str(IndexStream^.Status);

        IndexStream^.Reset;
       end;

      Dispose(TempStream, Done);
}
     end
    else
     begin
      rc:='unable to update header at ' + HexL(MessageIndex.HeaderOffset) + ': rc=' + Long2Str(HeaderStream^.Status);

      HeaderStream^.Reset;
     end;
   end
  else
   begin
    HeaderStream^.Seek(MessageIndex.HeaderOffset);
    HeaderStream^.Read(MessageHeader, SizeOf(MessageHeader));

    if HeaderStream^.Status = stOk then
     begin
      MessageHeader.Attributes:=MessageHeader.Attributes or jamaDeleted;

      HeaderStream^.Seek(MessageIndex.HeaderOffset);
      HeaderStream^.Write(MessageHeader, SizeOf(MessageHeader));
     end;

    if HeaderStream^.Status = stOk then
     begin
      Dec(BaseHeader.ActiveMessages);
      Inc(BaseHeader.ModifyCounter);

      if WriteBaseHeader(rc) then
       if CheckMap(True, rc) then
        begin
{
         DegrowLastreads(IndexNo);
}
         Kill:=True;
        end;
     end
    else
     begin
      rc:='unable to update header at ' + HexL(MessageIndex.HeaderOffset) + ': rc=' + Long2Str(HeaderStream^.Status);

      HeaderStream^.Reset;
     end;
   end;

  Unlock;
 end;

function TJamMessageBase.NoToGlobal(const No: Longint): Longint;
 var
  Item: PJamMapItem;
  MessageIndex: TJamMessageIndex;
  MessageHeader: TJamMessageHeader;
  IndexNo: Longint;
  rc: String;
 begin
  if not CheckMap(False, rc) then
   NoToGlobal:=-1
  else
   if GetIndexNo(No, IndexNo, rc) then
    begin
     IndexStream^.Seek(IndexNo * SizeOf(MessageIndex));
     IndexStream^.Read(MessageIndex, SizeOf(MessageIndex));

     if IndexStream^.Status = stOk then
      begin
       HeaderStream^.Seek(MessageIndex.HeaderOffset);
       HeaderStream^.Read(MessageHeader, SizeOf(MessageHeader));

       if HeaderStream^.Status = stOk then
        NoToGlobal:=MessageHeader.MessageNumber
       else
        begin
         NoToGlobal:=-1;

         HeaderStream^.Reset;
        end;
      end
    end
   else
    begin
     NoToGlobal:=-1;

     IndexStream^.Reset;
    end;
 end;

function TJamMessageBase.GlobalToNo(const Global: Longint): Longint;
 var
  Countdown, No: Longint;
  MessageIndex: TJamMessageIndex;
  MessageHeader: TJamMessageHeader;
 begin
  Countdown:=IndexStream^.GetSize div SizeOf(MessageIndex);
  No:=0;

  while Countdown > 0 do
   begin
    IndexStream^.Read(MessageIndex, SizeOf(MessageIndex));

    if IndexStream^.Status <> stOk then
     begin
      GlobalToNo:=-1;

      IndexStream^.Reset;

      Exit;
     end;

    if MessageIndex.HeaderOffset <> -1 then
     begin
      HeaderStream^.Seek(MessageIndex.HeaderOffset);
      HeaderStream^.Read(MessageHeader, SizeOf(MessageHeader));

      if HeaderStream^.Status = stOk then
       if MessageHeader.MessageNumber = Global then
        begin
         GlobalToNo:=No;

         Exit;
        end
       else
      else
       begin
        GlobalToNo:=-1;

        HeaderStream^.Reset;

        Exit;
       end;

      Inc(No);
     end;

    Dec(Countdown);
   end;

  GlobalToNo:=-1;
 end;


function TJamMessageBase.LockBase: Boolean;
 begin
  LockBase:=HeaderStream^.Lock(0, 1) = 0;
 end;

procedure TJamMessageBase.UnlockBase;
 begin
  HeaderStream^.Unlock(0, 1);
 end;

{ private methods }

function TJamMessageBase.ReadBaseHeader(var rc: String): Boolean;
 begin
  HeaderStream^.Seek(0);
  HeaderStream^.Read(BaseHeader, SizeOf(BaseHeader));

  if HeaderStream^.Status = stOk then
   ReadBaseHeader:=True
  else
   begin
    rc:='unable to read base header, rc=' + Long2Str(HeaderStream^.Status);

    HeaderStream^.Reset;

    ReadBaseHeader:=False;
   end;
 end;

function TJamMessageBase.WriteBaseHeader(var rc: String): Boolean;
 begin
  if BaseHeader.ActiveMessages < 0 then
   BaseHeader.ActiveMessages:=0;

  if BaseHeader.BaseMessageNumber < 0 then
   BaseHeader.BaseMessageNumber:=0;

  HeaderStream^.Seek(0);
  HeaderStream^.Write(BaseHeader, SizeOf(BaseHeader));

  if HeaderStream^.Status = stOk then
   WriteBaseHeader:=True
  else
   begin
    rc:='unable to write base header, rc=' + Long2Str(HeaderStream^.Status);

    HeaderStream^.Reset;

    WriteBaseHeader:=False;
   end;

  Map^.ModifyCounter:=BaseHeader.ModifyCounter;
 end;


function TJamMessageBase.CheckMap(const Force: Boolean; var rc: String): Boolean;
 var
  MessageIndex: TJamMessageIndex;
  MessageHeader: TJamMessageHeader;
  Countdown, K, No, IndexNo: Longint;
  Item: PJamMapItem;
 begin
  if not Force and (BaseHeader.ModifyCounter = Map^.ModifyCounter) then
   CheckMap:=True
  else
   begin
    Map^.FreeAll;

    Countdown:=IndexStream^.GetSize div SizeOf(MessageIndex);

    if Countdown <> BaseHeader.ActiveMessages then
     begin
      Map^.Synchronous:=False;

      IndexStream^.Seek(0);

      No:=0;
      IndexNo:=0;

      Countdown:=IndexStream^.GetSize div SizeOf(MessageIndex);

      while Countdown > 0 do
       begin
        IndexStream^.Read(MessageIndex, SizeOf(MessageIndex));

        if IndexStream^.Status <> stOk then
         begin
          rc:='failed to read message index at ' + HexL(IndexNo * SizeOf(MessageIndex)) + ': rc=' +
           Long2Str(IndexStream^.Status);

          IndexStream^.Reset;

          CheckMap:=False;

          Exit;
         end;

        if MessageIndex.HeaderOffset <> -1 then
         begin
          HeaderStream^.Seek(MessageIndex.HeaderOffset);
          HeaderStream^.Read(MessageHeader, SizeOf(MessageHeader));

          if HeaderStream^.Status = stOk then
           if MessageHeader.Attributes and jamaDeleted = 0 then
            begin
{
             jt.logwrite('~', 'jam/checkmap/message ' + long2str(no) + ' (index ' +
                                                        long2str(indexno) + ', global ' +
                                                        long2str(messageheader.messagenumber) + ') ' +
                                                        'offset ' + hexl(messageindex.headeroffset));
}
             New(Item);

             Item^.No:=No;
             Item^.IndexNo:=IndexNo;

             Map^.Insert(Item);

             Inc(No);
            end
           else
          else
           begin
            rc:='failed to load message header at ' + HexL(MessageIndex.HeaderOffset) + ': rc=' +
             Long2Str(HeaderStream^.Status);

            HeaderStream^.Reset;

            CheckMap:=False;

            Exit;
           end;
         end;

        Dec(Countdown);
        Inc(IndexNo);
       end;

      CheckMap:=True;
     end
    else
     begin
      Map^.Synchronous:=True;

      CheckMap:=True;
     end;
   end;
 end;

function TJamMessageBase.GetIndexNo(const No: Longint; var IndexNo: Longint; var rc: String): Boolean;
 var
  Item: PJamMapItem;
 begin
  if CheckMap(False, rc) then
   if (No < 0) and (No >= BaseHeader.ActiveMessages) then
    begin
     GetIndexNo:=False;

     rc:='message ' + Long2Str(No) + ' does not exist';
    end
   else
    if Map^.Synchronous then
     begin
      IndexNo:=No;

      GetIndexNo:=True;
     end
    else
     begin
      Item:=Map^.At(No);

      if Item = nil then
       begin
        GetIndexNo:=False;

        rc:='message ' + Long2Str(No) + ' does not exist and brain is dead';
       end
      else
       begin
        IndexNo:=Item^.IndexNo;

        GetIndexNo:=True;
       end;
     end
  else
   GetIndexNo:=False;
 end;


procedure TJamMessageBase.JTMHLToJam(const msg: Pointer; var MessageHeader: TJamMessageHeader;
  var MessageIndex: TJamMessageIndex; var SubFields: PSubFields; var Body: PRamStream);
 var
  FromAddress, ToAddress: TAddress;
  Line: PChar;
  msgp: Pointer;
  DT: TDateTime;
  K: Longint;
 const
  CR: Char = #13;
 begin
  FillChar(MessageHeader, SizeOf(MessageHeader), 0);

  with MessageHeader do
   begin
    MSGIDCRC32:=-1;
    REPLYCRC32:=-1;
    PasswordCRC32:=-1;
   end;

  if jt.msg.GetToName(msg) = nil then
   MessageIndex.MSGTOCRC:=-1
  else
   MessageIndex.MSGTOCRC:=StringCRC32(StLocase(jt.msg.GetToName(msg)^), -1);

  SubFields:=New(PSubFields, Init($10, $10));

  Body:=New(PRamStream, Init);

  msgp:=jt.msg.CreateParser(msg, Netmail);

  Body^.SetSize(jt.msg.p.GetTextSize(msgp));

  jt.msg.p.GetFromAndToAddress(msgp, FromAddress, ToAddress);

  SubFields^.Insert(New(PSubField, InitStr(jamSource, AddressToStrEx(FromAddress))));

  if IsCleanAddress(ToAddress) then
   SubFields^.Insert(New(PSubField, InitStr(jamDestination, AddressToStrEx(FromAddress))))
  else
   SubFields^.Insert(New(PSubField, InitStr(jamDestination, AddressToStrEx(ToAddress))));

  SubFields^.Insert(New(PSubField, InitPStr(jamFrom, jt.msg.GetFromName(msg))));
  SubFields^.Insert(New(PSubField, InitPStr(jamTo, jt.msg.GetToName(msg))));

  if jt.msg.GetSubject(msg) <> nil then
   if jt.msg.GetAttributes(msg) and maAttach <> 0 then
    SubFields^.Insert(New(PSubField, InitPStr(jamAttach, jt.msg.GetSubject(msg))))
   else
    if jt.msg.GetAttributes(msg) and maFRQ <> 0 then
     SubFields^.Insert(New(PSubField, InitPStr(jamFRQ, jt.msg.GetSubject(msg))))
    else
     SubFields^.Insert(New(PSubField, InitPStr(jamSubject, jt.msg.GetSubject(msg))));

  MessageHeader.Attributes:=jamaNone;

  for K:=Low(AttributesMap) to High(AttributesMap) do
   if jt.msg.GetAttributes(msg) and AttributesMap[K, amtJTMHL] <> 0 then
    MessageHeader.Attributes:=MessageHeader.Attributes or AttributesMap[K, amtJam];

  jt.msg.p.SetTextPos(msgp, 0);

  while not jt.msg.p.EndOfMessage(msgp) do
   begin
    Line:=jt.msg.p.GetStringPCharEx(msgp);

    if Line[0] = #1 then
     if StrLComp(@Line[1], 'MSGID: ', 7) = 0 then
      begin
       SubFields^.Insert(New(PSubField, Init(jamMSGID, StrNew(@Line[8]))));

       MessageHeader.MSGIDCRC32:=StringCRC32(Trim(StrPas(@Line[8])), -1);
      end else
     if StrLComp(@Line[1], 'REPLY: ', 7) = 0 then
      begin
       SubFields^.Insert(New(PSubField, Init(jamREPLY, StrNew(@Line[8]))));

       MessageHeader.REPLYCRC32:=StringCRC32(Trim(StrPas(@Line[8])), -1);
      end else
     if StrLComp(@Line[1], 'PID: ', 5) = 0 then SubFields^.Insert(New(PSubField, Init(jamPID, StrNew(@Line[6])))) else
     if StrLComp(@Line[1], 'FLAGS: ', 7) = 0 then SubFields^.Insert(New(PSubField, Init(jamFLAGS, StrNew(@Line[8])))) else
     if StrLComp(@Line[1], 'Via ', 4) = 0 then SubFields^.Insert(New(PSubField, Init(jamVia, StrNew(@Line[5])))) else
     if StrLComp(@Line[1], 'PATH: ', 6) = 0 then SubFields^.Insert(New(PSubField, Init(jamPATH, StrNew(@Line[7])))) else
     if StrLComp(@Line[1], 'INTL', 4) = 0 then else
     if StrLComp(@Line[1], 'FMPT', 4) = 0 then else
     if StrLComp(@Line[1], 'TOPT', 4) = 0 then else
      SubFields^.Insert(New(PSubField, Init(jamUnknown, StrNew(@Line[1]))))
    else
     if StrLComp(@Line, 'SEEN-BY: ', 9) = 0 then SubFields^.Insert(New(PSubField, Init(jamSEENBY, StrNew(@Line[9])))) else
      begin
       Body^.Write(Line[0], StrLen(Line));
       Body^.Write(CR, SizeOf(CR));
      end;

    jt.StrDispose(Line);
   end;

  jt.msg.DestroyParser(msgp);

  with MessageHeader do
   begin
    StrPCopy(Signature, 'JAM');

    Revision:=1;

    SubFieldsLength:=SubFields^.CountSize;

    if jt.msg.GetAttributes(msg) and maRead <> 0 then
     TimesRead:=1
    else
     TimesRead:=0;

    DateReceived:=Longint(jt.st.Get(jt.msg.st(msg), 'Jam.DateRcvd'));

    jt.msg.GetWritten(msg, DT);

    DateTimeToUnixDateTime(DT, DateWritten);

    jt.msg.GetArrived(msg, DT);

    DateTimeToUnixDateTime(DT, DateReceived);

    GetCurrentDateTime(DT);

    DateTimeToUnixDateTime(DT, DateProcessed);

    TextLength:=Body^.GetSize;
   end;
 end;

procedure TJamMessageBase.JamToJTMHL(var MessageHeader: TJamMessageHeader; const SubFields: PSubFields;
  const Body: PRamStream; var msg: Pointer);
 var
  msgBody: PStreamThunk;
  FromAddress, ToAddress: TAddress;
 procedure PutSubField(const SubField: PSubField; const Prefix: String);
  const
   CR: Char = #13;
  begin
   msgBody^.Write((@Prefix[1])^, Length(Prefix));

   if SubField^.Data <> nil then
    msgBody^.Write(SubField^.Data[0], StrLen(SubField^.Data));

   msgBody^.Write(CR, SizeOf(CR));
  end;
 procedure Prepare(const SubField: PSubField); far;
  function S: String;
   var
    Data: String;
   begin
    if SubField^.Data = nil then
     S:=''
    else
     if StrLen(SubField^.Data) > 255 then
      begin
       SetLength(Data, 255);

       Move(SubField^.Data[0], Data[1], 255);

       S:=Data;
      end
     else
      S:=StrPas(SubField^.Data);
   end;
  begin
   case SubField^.ID of
    jamSource: StrToAddress(S, FromAddress);
    jamDestination: StrToAddress(S, ToAddress);
    jamFrom: jt.msg.SetFromName(msg, S);
    jamTo: jt.msg.SetToName(msg, S);
    jamSubject: jt.msg.SetSubject(msg, S);
    jamAttach:
     if jt.msg.GetAttributes(msg) and maAttach = 0 then
      jt.msg.SetSubject(msg, S);
    jamFRQ:
     if jt.msg.GetAttributes(msg) and maFRQ = 0 then
      jt.msg.SetSubject(msg, S);
   end;
  end;
 procedure AddBeforeKludges(const SubField: PSubField); far;
  begin
   case SubField^.ID of
    jamMSGID: PutSubField(SubField, #1'MSGID: ');
    jamREPLY: PutSubField(SubField, #1'REPLY: ');
    jamPID: PutSubField(SubField, #1'PID: ');
    jamFLAGS: PutSubField(SubField, #1'FLAGS: ');
    jamUnknown: PutSubField(SubField, #1);
   end;
  end;
 procedure AddAfterKludges(const SubField: PSubField); far;
  begin
   case SubField^.ID of
    jamVia: PutSubField(SubField, #1'Via ');
    jamSEENBY: PutSubField(SubField, 'SEEN-BY: ');
    jamPATH: PutSubField(SubField, #1'PATH: ');
   end;
  end;
 var
  Attributes, K: Longint;
 begin
  msg:=jt.msg.Create;

  Attributes:=maNone;

  for K:=Low(AttributesMap) to High(AttributesMap) do
   if MessageHeader.Attributes and AttributesMap[K, amtJam] <> 0 then
    Attributes:=Attributes or AttributesMap[K, amtJTMHL];

  if MessageHeader.TimesRead <> 0 then
   Attributes:=Attributes or maRead;

  jt.msg.SetAttributes(msg, Attributes);

  ClearAddress(FromAddress);
  ClearAddress(ToAddress);

  SubFields^.ForEach(@Prepare);

  if Netmail then
   begin
    jt.msg.Add(msg, #1'INTL ' + AddressToStrPointless(ToAddress) + ' ' + AddressToStrPointless(FromAddress));

    if FromAddress.Point <> 0 then
     jt.msg.Add(msg, #1'FMPT ' + Long2Str(FromAddress.Point));

    if ToAddress.Point <> 0 then
     jt.msg.Add(msg, #1'TOPT ' + Long2Str(ToAddress.Point));
   end;

  msgBody:=New(PStreamThunk, Init(jt.msg.GetText(msg), jt.StreamLinker, False));

  SubFields^.ForEach(@AddBeforeKludges);

  Body^.Seek(0);

  msgBody^.CopyFrom(Body^, Body^.GetSize);

  SubFields^.ForEach(@AddAfterKludges);

  Dispose(msgBody, Done);

  jt.st.Add(jt.msg.st(msg), 'Jam.DateRcvd', Pointer(MessageHeader.DateReceived), nil);

  jt.msg.SetGlobal(msg, MessageHeader.MessageNumber);
 end;

procedure TJamMessageBase.DegrowLastreads(const IndexNo: Longint);
 var
  LastReadStream: PDosStreamEx;
  LastRead: TJamLastRead;
  K: Longint;
 begin
  LastReadStream:=New(PDosStreamEx, Init(Path^ + 'jlr', stOpen));

  if LastReadStream^.Status = stOk then
   for K:=0 to LastReadStream^.GetSize div SizeOf(LastRead) - 1 do
    begin
     LastReadStream^.Seek(K * SizeOf(LastRead));
     LastReadStream^.Read(LastRead, SizeOf(LastRead));

     if LastReadStream^.Status <> stOk then
      Break;

     if IndexNo <= LastRead.LastRead then
      begin
       Dec(LastRead.LastRead);

       if LastRead.LastRead < 1 then
        LastRead.LastRead:=1;
      end;

     LastReadStream^.Seek(K * SizeOf(LastRead));
     LastReadStream^.Write(LastRead, SizeOf(LastRead));

     if LastReadStream^.Status <> stOk then
      Break;
    end;

  Dispose(LastReadStream, Done);
 end;

type
 PAdditionalBlock = ^TAdditionalBlock;
 TAdditionalBlock = packed record
  ReplyTo: Longint;
  ReplyFirst: Longint;
  ReplyNext: Longint;
 end;

{ AdditionalDisposer }

procedure AdditionalDisposer(const Additional: Pointer);
 begin
  Dispose(PAdditionalBlock(Additional));
 end;

{ public methods }

procedure TJamMessageBase.LoadLinkInfo(var Block: TLoadLinkMessageBaseBlock);
 var
  Errors: Longint;
 procedure Error(const Message: String);
  begin
   jt.LogWrite('?', Message + ' (' + Path^ + ')');

   Inc(Errors);
  end;
 var
  No, BufferSize: Longint;
  Index: TJamMessageIndex;
  Header: TJamMessageHeader;
  Additional: PAdditionalBlock;
 begin
  Block.Supported:=True;
  Block.Did:=True;
  Block.Dispose:=AdditionalDisposer;
  Block.Flags:=llmbbfNone;

  Errors:=0;

  IndexStream^.Seek(0);

  for No:=0 to IndexStream^.GetSize div SizeOf(Index) - 1 do
   begin
    IndexStream^.Read(Index, SizeOf(Index));

    if IndexStream^.Status = stOk then
     if Index.HeaderOffset <> -1 then
      begin
       HeaderStream^.Seek(Index.HeaderOffset + 16);
       HeaderStream^.Read(Header.MSGIDCRC32, SizeOf(Longint) * 10);

       if HeaderStream^.Status = stOk then
        if Header.Attributes and jamaDeleted = 0 then
         begin
          New(Additional);

          Additional^.ReplyTo:=Header.ReplyTo;
          Additional^.ReplyFirst:=Header.ReplyFirst;
          Additional^.ReplyNext:=Header.ReplyNext;

          if not Block.Add(Block.Param, No, No + 1, Header.MSGIDCRC32, Header.REPLYCRC32, Additional) then
           begin
            Block.Did:=False;

            Break;
           end;

          Errors:=0;
         end
        else
       else
        begin
         Error('unable to load message header at ' + HexL(Index.HeaderOffset) + ', rc=' + Long2Str(HeaderStream^.Status));

         HeaderStream^.Reset;
        end;
      end
     else
    else
     begin
      Error('unable to read index entry #' + Long2Str(No) + ' at ' + HexL(No * SizeOf(Index)) +
        ', rc=' + Long2Str(IndexStream^.Status));

      IndexStream^.Reset;
     end;

    if Errors > 10 then
     begin
      Block.Did:=False;

      Break;
     end;
   end;
 end;

procedure TJamMessageBase.StoreLinkInfo(var Block: TStoreLinkMessageBaseBlock);
 var
  K, L, Errors: Longint;
  Message, M: PLinkerMessage;
  NewAdditional: TAdditionalBlock;
  Update: Boolean;
  Index: TJamMessageIndex;
  Header: TJamMessageHeader;
  rc: String;
 procedure Error(const Message: String);
  begin
   jt.LogWrite('?', Message + ' (' + Path^ + ')');

   Inc(Errors);
  end;
 begin
  Block.Supported:=True;
  Block.Did:=ReadBaseHeader(rc);

  if not Block.Did then
   begin
    Error(rc);

    Exit;
   end;

  Errors:=0;

  for K:=0 to Block.Count - 1 do
   begin
    if not Block.Callback(Block.Param) then
     begin
      Block.Did:=False;

      Break;
     end;

    Message:=Block.Items^[K];

    if Message^.Owner^.Global = -1 then
     NewAdditional.ReplyTo:=0
    else
     NewAdditional.ReplyTo:=Message^.Owner^.Global;

    Update:=NewAdditional.ReplyTo <> PAdditionalBlock(Message^.Additional)^.ReplyTo;

    if Message^.ChildFirst = nil then
     NewAdditional.ReplyFirst:=0
    else
     NewAdditional.ReplyFirst:=Message^.ChildFirst^.Global;

    Update:=Update or (NewAdditional.ReplyFirst <> PAdditionalBlock(Message^.Additional)^.ReplyFirst);

    if (Message^.Next = nil) or (Message^.Owner^.Global = -1) then
     NewAdditional.ReplyNext:=0
    else
     NewAdditional.ReplyNext:=Message^.Next^.Global;

    Update:=Update or (NewAdditional.ReplyNext <> PAdditionalBlock(Message^.Additional)^.ReplyNext);

    if Update then
     begin
      IndexStream^.Seek(Message^.No * SizeOf(Index));
      IndexStream^.Read(Index, SizeOf(Index));

      if IndexStream^.Status = stOk then
       begin
        HeaderStream^.Seek(Index.HeaderOffset + 24);
        HeaderStream^.Write(NewAdditional, SizeOf(NewAdditional));

        if HeaderStream^.Status <> stOk then
         begin
          Error('unable to set replyinfo at ' + HexL(Index.HeaderOffset) + ', rc=' + Long2Str(HeaderStream^.Status));

          HeaderStream^.Reset;
         end
        else
         Errors:=0;
       end
      else
       begin
        Error('unable to read index entry #' + Long2Str(Message^.No) + ' at ' + HexL(Message^.No * SizeOf(Index)) +
         ', rc=' + Long2Str(IndexStream^.Status));

        IndexStream^.Reset;
       end;
     end;

    if Errors > 10 then
     begin
      Block.Did:=False;

      Break;
     end;
   end;

  Inc(BaseHeader.ModifyCounter);

  if not WriteBaseHeader(rc) then
   Error(rc);
 end;

function TJamMessageBase.ExtendedOpened(const Operation: Longint; var Data): Boolean;
 var
  HighWater: Longint absolute Data;
  Attributes: Longint absolute Data;
  LastRead: Longint absolute Data;
  rc: String;
 begin
  if Operation = eooGetHighWater then
   begin
    HighWater:=BaseHeader.HighWater;

    ExtendedOpened:=True;
   end else
  if Operation = eooSetHighWater then
   if Lock then
    begin
     if ReadBaseHeader(rc) then
      begin
       BaseHeader.Highwater:=HighWater;

       ExtendedOpened:=WriteBaseHeader(rc);
      end
     else
      ExtendedOpened:=False;

     Unlock;
    end
   else
    ExtendedOpened:=False else
  if Operation = eooGetLastRead then ExtendedOpened:=GetLastRead(LastRead) else
  if Operation = eooSetLastRead then ExtendedOpened:=SetLastRead(LastRead) else
   ExtendedOpened:=inherited ExtendedOpened(Operation, Data);
 end;

function TJamMessageBase.GetLastRead(var LastRead: Longint): Boolean;
 var
  UserName: String;
  NAMECRC, Countdown: Longint;
  Data: TJamLastRead;
  Stream: PStream;
 begin
  jt.core.GetMainSysop(UserName);

  StLocaseEx(UserName);

  NAMECRC:=StringCRC32(UserName, -1);

  GetLastRead:=False;

  Stream:=New(PDosStreamEx, Init(Path^ + 'jlr', stOpen));

  if Stream^.Status = stOk then
   begin
    Countdown:=Stream^.GetSize div SizeOf(Data);

    while Countdown > 0 do
     begin
      Stream^.Read(Data, SizeOf(Data));

      if Data.NAMECRC = NAMECRC then
       begin
        LastRead:=Data.LastRead;

        GetLastRead:=True;

        Break;
       end;

      Dec(Countdown);
     end;
   end;

  Dispose(Stream, Done);
 end;

function TJamMessageBase.SetLastRead(var LastRead: Longint): Boolean;
 var
  UserName: String;
  NAMECRC, Countdown, MaxUserNumber: Longint;
  Data: TJamLastRead;
  Stream: PDosStreamEx;
  Found: Boolean;
 begin
  jt.core.GetMainSysop(UserName);

  StLocaseEx(UserName);

  NAMECRC:=StringCRC32(UserName, -1);

  Stream:=New(PDosStreamEx, Init(Path^ + 'jlr', stOpen or stDenyAll));

  if Stream^.Status <> stOk then
   begin
    Dispose(Stream, Done);

    Stream:=New(PDosStreamEx, Init(Path^ + 'jlr', stCreate or stDenyAll));
   end;

  if Stream^.Status = stOk then
   begin
    Found:=False;

    MaxUserNumber:=-1;

    Countdown:=Stream^.GetSize div SizeOf(Data);

    while Countdown > 0 do
     begin
      Stream^.Read(Data, SizeOf(Data));

      if Data.UserNumber > MaxUserNumber then
       MaxUserNumber:=Data.UserNumber;

      if Data.NAMECRC = NAMECRC then
       begin
        Data.LastRead:=LastRead;

        Stream^.Seek(Stream^.GetPos - SizeOf(Data));

        Found:=True;

        Break;
       end;

      Dec(Countdown);
     end;

    if not Found then
     begin
      Data.NAMECRC:=NAMECRC;
      Data.UserNumber:=MaxUserNumber + 1;
      Data.LastRead:=LastRead;
      Data.HighRead:=LastRead;
     end;

    Stream^.Write(Data, SizeOf(Data));
   end;

  SetLastRead:=Stream^.Status = stOk;

  Dispose(Stream, Done);
 end;

end.