unit mbjmPack;

interface
uses
     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     Objects,
     Strings,

     RStream,

     mbjmCmn,

     jtMHLcmn,
     jtMHLstm,

     Wizard;

procedure PackJamBase(Path: String; var Block: TPackMessageBaseBlock);

implementation

type
 PMessageNumberMapItem = ^TMessageNumberMapItem;
 TMessageNumberMapItem = packed record
  OldNo, NewNo, IndexNo, TargetHeaderOffset, ReplyTo, ReplyFirst, ReplyNext: Longint;
 end;

 PMessageNumberMap = ^TMessageNumberMap;
 TMessageNumberMap = object(TSortedCollection)
  procedure FreeItem(Item: Pointer); virtual;
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  function KeyOf(Item: Pointer): Pointer; virtual;
 end;

{ TMessageNumberMap }

procedure TMessageNumberMap.FreeItem(Item: Pointer);
 begin
  Dispose(PMessageNumberMapItem(Item));
 end;

function TMessageNumberMap.Compare(Key1, Key2: Pointer): xInteger;
 var
  K1: Longint absolute Key1;
  K2: Longint absolute Key2;
 begin
  if K1 < K2 then Compare:=-1 else
  if K1 > K2 then Compare:=1 else
   Compare:=0;
 end;

function TMessageNumberMap.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=Pointer(PMessageNumberMapItem(Item)^.OldNo);
 end;

{ PackJamBase }

procedure PackJamBase(Path: String; var Block: TPackMessageBaseBlock);
 function CheckStream(const Stream: PStream; const Action, Extension: String): Boolean;
  begin
   if Stream^.Status = stOk then
    CheckStream:=True
   else
    begin
     jt.LogWrite('?', 'Failed to ' + Action + ' ' + Path + Extension + ', rc=' + Long2Str(Stream^.Status) +
      ' (while packing ' + jt.area.GetName(Block.Area)^ + ')');

     CheckStream:=False;
    end;
  end;
 function CheckStreamStatus(const Stream: PStream; const Operation, Extension: String): Boolean;
  begin
   if Stream^.Status = stOk then
    CheckStreamStatus:=True
   else
    begin
     CheckStreamStatus:=False;

     jt.LogWrite('?', 'Failed to ' + Operation + ' of ' + Path + Extension + ': rc=' + Long2Str(Stream^.Status));
    end;
  end;
 procedure FailedToRename(const Original, Target: String; const rc: Longint);
  begin
   jt.LogWrite('?', 'Failed to rename ' + Path + Original + ' to ' + Path + Target +
    ', rc=' + Long2Str(rc) + ' (while packing ' + jt.area.GetName(Block.Area)^ + ')');

   Block.Did:=False;
  end;
 var
  SourceIndexStream,
  SourceHeaderStream,
  SourceDataStream,
  TargetIndexStream,
  TargetHeaderStream,
  TargetDataStream,
  Stream: PDosStreamEx;
  BaseHeader: TJamBaseHeader;
  MessageIndex: TJamMessageIndex;
  MessageHeader: TJamMessageHeader;
  SourceDataOffset, Countdown, rc, OldNo, NewNo: Longint;
  TotalHeaderStreamSize, TotalDataStreamSize: Longint;
  NewLastRead, NewHighRead: Longint;
  RenameFiles: Boolean;
  Map: PMessageNumberMap;
  MapItem: PMessageNumberMapItem;
 procedure FixMessageNumber(var Old, New: Longint; const Smart: Boolean; var Update: Boolean);
  var
   Index: xInteger;
  begin
   if Map^.Search(Pointer(Old), Index) then
    New:=PMessageNumberMapItem(Map^.At(Index))^.NewNo
   else
    if Smart and (Old <> 0) then
     begin
      if Index > Map^.Count - 1 then
       Index:=Map^.Count - 1;

      if (Index >= 0) and (Map^.At(Index) <> nil) then
       New:=PMessageNumberMapItem(Map^.At(Index))^.NewNo
      else
       New:=0;
     end
    else
     New:=0;

   Update:=Update or (Old <> New);
  end;
 function FixReplies(const MapItem: PMessageNumberMapItem): Boolean; far;
  var
   Update: Boolean;
   NewReplyTo, NewReplyFirst, NewReplyNext: Longint;
  begin
   Update:=False;

   FixMessageNumber(MapItem^.ReplyTo, NewReplyTo, False, Update);
   FixMessageNumber(MapItem^.ReplyFirst, NewReplyFirst, False, Update);
   FixMessageNumber(MapItem^.ReplyNext, NewReplyNext, False, Update);

   if Update then
    begin
     TargetHeaderStream^.Seek(MapItem^.TargetHeaderOffset);
     TargetHeaderStream^.Read(MessageHeader, SizeOf(MessageHeader));

     if CheckStreamStatus(TargetHeaderStream, 'read header', 'j$r') then
      begin
       MessageHeader.ReplyTo:=NewReplyTo;
       MessageHeader.ReplyFirst:=NewReplyFirst;
       MessageHeader.ReplyNext:=NewReplyNext;

       TargetHeaderStream^.Seek(MapItem^.TargetHeaderOffset);
       TargetHeaderStream^.Write(MessageHeader, SizeOf(MessageHeader));

       FixReplies:=not CheckStreamStatus(TargetHeaderStream, 'write header', 'j$r');
      end
     else
      FixReplies:=True;
    end
   else
    FixReplies:=False;
  end;
 var
  LastRead: TJamLastRead;
  Update: Boolean;
 begin
  Path:=AddDot(Path);

  Block.Supported:=True;

  if not jmbIsBaseExists(Path) then
   begin
    Block.Did:=True;

    Block.BeforePack:=0;
    Block.AfterPack:=0;

    Exit;
   end;

  Block.Did:=False;

  SourceIndexStream:=New(PDosStreamEx, Init(Path + 'jdx', stOpen or stDenyAll));
  SourceHeaderStream:=New(PDosStreamEx, Init(Path + 'jhr', stOpen or stDenyAll));
  SourceDataStream:=New(PDosStreamEx, Init(Path + 'jdt', stOpen or stDenyAll));

  TargetIndexStream:=New(PDosStreamEx, Init(Path + 'j$x', stCreate or stDenyAll));
  TargetHeaderStream:=New(PDosStreamEx, Init(Path + 'j$r', stCreate or stDenyAll));
  TargetDataStream:=New(PDosStreamEx, Init(Path + 'j$t', stCreate or stDenyAll));

  repeat
   if not CheckStream(SourceIndexStream, 'open', 'jdx') or
      not CheckStream(SourceHeaderStream, 'open', 'jhr') or
      not CheckStream(SourceDataStream, 'open', 'jdt') or
      not CheckStream(TargetIndexStream, 'create', 'j$x') or
      not CheckStream(TargetHeaderStream, 'create', 'j$r') or
      not CheckStream(TargetDataStream, 'create', 'j$t') then Break;

   SourceHeaderStream^.Read(BaseHeader, SizeOf(BaseHeader));

   if not CheckStreamStatus(SourceHeaderStream, 'read base header', 'jhr') then Break;

   Block.BeforePack:=SourceIndexStream^.GetSize + SourceHeaderStream^.GetSize + SourceDataStream^.GetSize;
   Block.AfterPack:=Block.BeforePack;

   if SourceIndexStream^.GetSize div SizeOf(MessageIndex) = BaseHeader.ActiveMessages then
    begin
     TotalHeaderStreamSize:=SizeOf(BaseHeader);
     TotalDataStreamSize:=0;

     Countdown:=SourceIndexStream^.GetSize div SizeOf(MessageIndex);

     while Countdown > 0 do
      begin
       SourceIndexStream^.Read(MessageIndex, SizeOf(MessageIndex));

       if SourceIndexStream^.Status <> stOk then
        begin
         SourceIndexStream^.Reset;

         Break;
        end;

       SourceHeaderStream^.Seek(MessageIndex.HeaderOffset);
       SourceHeaderStream^.Read(MessageHeader, SizeOf(MessageHeader));

       if SourceHeaderStream^.Status <> stOk then
        begin
         SourceHeaderStream^.Reset;

         Break;
        end;

       Inc(TotalHeaderStreamSize, SizeOf(MessageHeader));
       Inc(TotalHeaderStreamSize, MessageHeader.SubFieldsLength);
       Inc(TotalDataStreamSize, MessageHeader.TextLength);

       Dec(Countdown);
      end;

     if (SourceHeaderStream^.GetSize = TotalHeaderStreamSize) and
        (SourceDataStream^.GetSize = TotalDataStreamSize) and
        not ForcePacking then Break;
    end;

   Inc(BaseHeader.ModifyCounter);

   TargetHeaderStream^.Write(BaseHeader, SizeOf(BaseHeader));

   if not CheckStreamStatus(TargetHeaderStream, 'write base header', 'j$r') then Break;

   SourceIndexStream^.Seek(0);

   Countdown:=SourceIndexStream^.GetSize div SizeOf(MessageIndex);

   NewNo:=0;

   Map:=New(PMessageNumberMap, Init(Countdown, $10));

   BaseHeader.ActiveMessages:=0;
   BaseHeader.BaseMessageNumber:=$7FFFFFFF;

   while Countdown > 0 do
    begin
     if Block.Callback(Block.Area, Block.Param, paDoing) then Break;

     SourceIndexStream^.Read(MessageIndex, SizeOf(MessageIndex));

     if not CheckStreamStatus(SourceIndexStream, 'read index', 'jdx') then Break;

     if MessageIndex.HeaderOffset <> -1 then
      begin
       SourceHeaderStream^.Seek(MessageIndex.HeaderOffset);
       SourceHeaderStream^.Read(MessageHeader, SizeOf(MessageHeader));

       if not CheckStreamStatus(SourceHeaderStream, 'read header', 'jhr') then Break;

       if MessageHeader.Attributes and jamaDeleted = 0 then
        begin
         OldNo:=MessageHeader.MessageNumber;

         SourceDataOffset:=MessageHeader.TextOffset;

         MessageIndex.HeaderOffset:=TargetHeaderStream^.GetSize;
         MessageHeader.TextOffset:=TargetDataStream^.GetSize;

         if CleanReplyOnPack and not FixReplyThreadsOnPack then
          begin
           MessageHeader.ReplyTo:=0;
           MessageHeader.ReplyFirst:=0;
           MessageHeader.ReplyNext:=0;
          end;

         TargetIndexStream^.Write(MessageIndex, SizeOf(MessageIndex));

         if not CheckStreamStatus(TargetIndexStream, 'write index', 'j$x') then Break;

         Inc(NewNo);

         if RenumberOnPack then
          MessageHeader.MessageNumber:=NewNo;

         Inc(BaseHeader.ActiveMessages);

         TargetHeaderStream^.Write(MessageHeader, SizeOf(MessageHeader));
         TargetHeaderStream^.CopyFrom(SourceHeaderStream^, MessageHeader.SubFieldsLength);

         if not CheckStreamStatus(TargetHeaderStream, 'write header', 'j$r') then Break;
         if not CheckStreamStatus(SourceHeaderStream, 'read header subfields', 'jhr') then Break;

         SourceDataStream^.Seek(SourceDataOffset);
         TargetDataStream^.CopyFrom(SourceDataStream^, MessageHeader.TextLength);

         if not CheckStreamStatus(TargetDataStream, 'write body', 'j$t') then Break;
         if not CheckStreamStatus(SourceDataStream, 'read body', 'jdt') then Break;

         New(MapItem);

         MapItem^.OldNo:=OldNo;
         MapItem^.NewNo:=MessageHeader.MessageNumber;
         MapItem^.IndexNo:=NewNo - 1;
         MapItem^.TargetHeaderOffset:=MessageIndex.HeaderOffset;
         MapItem^.ReplyTo:=MessageHeader.ReplyTo;
         MapItem^.ReplyFirst:=MessageHeader.ReplyFirst;
         MapItem^.ReplyNext:=MessageHeader.ReplyNext;

         Map^.Insert(MapItem);

         if MessageHeader.MessageNumber < BaseHeader.BaseMessageNumber then
          BaseHeader.BaseMessageNumber:=MessageHeader.MessageNumber;
        end;
      end;

     Dec(Countdown);
    end;

   if Countdown = 0 then
    begin
     if BaseHeader.BaseMessageNumber = $7FFFFFFF then
      BaseHeader.BaseMessageNumber:=1;

     TargetHeaderStream^.Seek(0);
     TargetHeaderStream^.Write(BaseHeader, SizeOf(BaseHeader));

     if not CheckStreamStatus(TargetHeaderStream, 'write base header', 'j$r') then
      Break;

     if FixReplyThreadsOnPack and (Map^.FirstThat(@FixReplies) <> nil) then
      Break;

     if FixLastreadsOnPack then
      begin
       Stream:=New(PDosStreamEx, Init(Path + 'jlr', stOpen or stDenyAll));

       if Stream^.Status = stOk then
        for Countdown:=0 to Stream^.GetSize div SizeOf(LastRead) - 1 do
         begin
          Stream^.Read(LastRead, SizeOf(LastRead));

          if Stream^.Status <> stOk then
           Break;

          Update:=False;

          FixMessageNumber(LastRead.LastRead, NewLastRead, True, Update);
          FixMessageNumber(LastRead.HighRead, NewHighRead, True, Update);

          if Update then
           begin
            LastRead.LastRead:=NewLastRead;
            LastRead.HighRead:=NewHighRead;

            Stream^.Seek(Countdown * SizeOf(LastRead));
            Stream^.Write(LastRead, SizeOf(LastRead));

            if Stream^.Status <> stOk then
             Break;
           end;
         end;

       Dispose(Stream, Done);
      end;

     Block.Did:=True;

     Block.AfterPack:=TargetIndexStream^.GetSize + TargetHeaderStream^.GetSize + TargetDataStream^.GetSize;
    end;

   Dispose(Map, Done);
  until True;

  Dispose(TargetDataStream, Done);
  Dispose(TargetHeaderStream, Done);
  Dispose(TargetIndexStream, Done);

  Dispose(SourceDataStream, Done);
  Dispose(SourceHeaderStream, Done);
  Dispose(SourceIndexStream, Done);

  if Block.Did then
   begin
    EraseFile(Path + 'j!x');
    EraseFile(Path + 'j!r');
    EraseFile(Path + 'j!t');

    if not RenameFileEx(Path + 'jdx', Path + 'j!x', rc) then FailedToRename('jdx', 'j!x', rc) else
    if not RenameFileEx(Path + 'jhr', Path + 'j!r', rc) then FailedToRename('jhr', 'j!r', rc) else
    if not RenameFileEx(Path + 'jdt', Path + 'j!t', rc) then FailedToRename('jdt', 'j!t', rc) else

    if not RenameFileEx(Path + 'j$x', Path + 'jdx', rc) then FailedToRename('j$x', 'jdx', rc) else
    if not RenameFileEx(Path + 'j$r', Path + 'jhr', rc) then FailedToRename('j$r', 'jhr', rc) else
    if not RenameFileEx(Path + 'j$t', Path + 'jdt', rc) then FailedToRename('j$t', 'jdt', rc) else
     begin
      EraseFile(Path + 'j!x');
      EraseFile(Path + 'j!r');
      EraseFile(Path + 'j!t');
     end;
   end
  else
   begin
    EraseFile(Path + 'j$x');
    EraseFile(Path + 'j$r');
    EraseFile(Path + 'j$t');
   end;
 end;

end.