unit mbmsCore;

interface
uses
     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     mbAPI,
     mbPLUG,

     LinkAPI,

     mbmsCmn,

     Thunks,

     jtMHLstm,
     jtMHLcmn,

     Objects,
     Strings,

     Dos,

     Wizard;

const
 DefaultLastReadUserNumber              = 0;

type
 PFidoMessageBase = ^TFidoMessageBase;
 TFidoMessageBase = object(TMessageBase)
  constructor Init(const APath: String; const Create: Boolean; const ABaseType: TBaseType; var rc: String);
  destructor Done; virtual;
 public
  function Add(const Message: Pointer; var rc: String; const Flags: Longint): Boolean; virtual;

  function Count: Longint; virtual;
  function Load(const No: Longint; var Message: Pointer; var rc: String; const Flags: Longint): Boolean; virtual;
  function Store(const No: Longint; const Message: Pointer; var rc: String; const Flags: Longint): Boolean; virtual;
  function Kill(const No: Longint; var rc: String): Boolean; virtual;
  function NoToGlobal(const No: Longint): Longint; virtual;
  function GlobalToNo(const Global: Longint): Longint; virtual;
  function LockBase: Boolean; virtual;
  procedure UnlockBase; virtual;

  procedure LoadLinkInfo(var Block: TLoadLinkMessageBaseBlock); virtual;
  procedure StoreLinkInfo(var Block: TStoreLinkMessageBaseBlock); virtual;

  function ExtendedOpened(const Operation: Longint; var Data): Boolean; virtual;

  function GetLastRead(var LastRead: Longint): Boolean;
  function SetLastRead(var LastRead: Longint): Boolean;
 private
  Map: PFidoMessageBaseMap;
  LastReadStream: PDosStreamEx;
  Path: PString;

  procedure Remap;

  procedure FidoToJTMHL(var Header: TFidoMessageHeader; var Message: Pointer);
  procedure JTMHLToFido(const Message: Pointer; var Header: TFidoMessageHeader; const Flags: Longint);

  function JTMHLToFidoAttributes(const JTMHLAttributes: Longint): Longint;
  function FidoToJTMHLAttributes(const FidoAttributes: Longint): Longint;
 end;

procedure AdditionalDisposer(const Additional: Pointer); export;

implementation

{ TFidoMessageBase }

constructor TFidoMessageBase.Init(const APath: String; const Create: Boolean; const ABaseType: TBaseType; var rc: String);
 begin
  inherited Init(APath, Create, ABaseType, rc);

  Path:=NewStr(AddBackSlash(APath));

  if not ExistDir(Path^) then
   if Create then
    if not CreateDirectory(Path^) then
     begin
      rc:='failed to create directory ' + RemoveBackSlash(Path^);

      Done;

      Fail;
     end
    else
   else
    begin
     rc:='there is no directory ' + RemoveBackSlash(Path^);

     Done;

     Fail;
    end;

  LastReadStream:=New(PDosStreamEx, Init(Path^ + 'lastread', stOpen));

  if LastReadStream^.Status <> stOk then
   begin
    Dispose(LastReadStream, Done);

    LastReadStream:=New(PDosStreamEx, Init(Path^ + 'lastread', stCreate));

    if LastReadStream^.Status <> stOk then
     begin
      rc:='unable to open/create ' + Path^ + 'lastread, rc=' + Long2Str(LastReadStream^.Status);

      Done;

      Fail;
     end;
   end;

  Map:=New(PFidoMessageBaseMap, Init($10, $10));

  Map^.Duplicates:=False;

  Remap;
 end;

destructor TFidoMessageBase.Done;
 begin
  Dispose(Map, Done);

  if LastReadStream <> nil then
   Dispose(LastReadStream, Done);

  DisposeStr(Path);

  inherited Done;
 end;

{ public methods }

function TFidoMessageBase.Add(const Message: Pointer; var rc: String; const Flags: Longint): Boolean;
 const
  Null: System.Byte = 0;
 var
  Global: Longint;
  No: xInteger;
  Stream: PDosStreamEx;
  Header: TFidoMessageHeader;
  Text: PStreamThunk;
 begin
  Add:=False;

  Global:=Longint(Map^.At(Map^.Count - 1));

  repeat
   Inc(Global);
  until not Map^.Search(Pointer(Global), No) and not ExistFile(Path^ + Long2Str(Global) + '.msg');

  Map^.AtInsert(No, Pointer(Global));

  Stream:=New(PDosStreamEx, Init(Path^ + Long2Str(Global) + '.msg', stCreate));

  if Stream^.Status <> stOk then
   begin
    rc:='unable to store(add) msg ' + Long2Str(No) + ' (' + Path^ + Long2Str(Global) + '.msg), ' +
        'rc=' + Long2Str(Stream^.Status);

    Dispose(Stream, Done);

    Exit;
   end;

  FillChar(Header, SizeOf(Header), 0);

  JTMHLToFido(Message, Header, Flags);

  Stream^.Write(Header, SizeOf(Header));

  if Stream^.Status <> stOk then
   begin
    rc:='unable to store(add) header of msg ' + Long2Str(No) + ' (' + Path^ + Long2Str(Global) + '.msg), ' +
        'rc=' + Long2Str(Stream^.Status);

    Dispose(Stream, Done);

    Exit;
   end;

  Text:=New(PStreamThunk, Init(jt.msg.GetText(Message), jt.StreamLinker, False));

  Text^.Seek(0);

  Stream^.CopyFrom(Text^, Text^.GetSize);

  Dispose(Text, Done);

  if Stream^.Status <> stOk then
   begin
    rc:='unable to store(add) body of msg ' + Long2Str(No) + ' (' + Path^ + Long2Str(Global) + '.msg), ' +
        'rc=' + Long2Str(Stream^.Status);

    Dispose(Stream, Done);

    Exit;
   end;

  Stream^.Write(Null, SizeOf(Null));

  Dispose(Stream, Done);

  Add:=True;
 end;


function TFidoMessageBase.Count: Longint;
 begin
  Count:=Map^.Count;
 end;

function TFidoMessageBase.Load(const No: Longint; var Message: Pointer; var rc: String; const Flags: Longint): Boolean;
 var
  Global: Longint;
  Stream: PDosStreamEx;
  Header: TFidoMessageHeader;
  Parser: Pointer;
  Text: PStreamThunk;
  S: String;
 begin
  Load:=False;

  Global:=Longint(Map^.At(No));

  if Global = 0 then
   begin
    rc:='unable to load msg ' + Long2Str(No) + ': isn''t exists';

    Exit;
   end;

  Stream:=New(PDosStreamEx, Init(Path^ + Long2Str(Global) + '.msg', stOpenRead));

  if Stream^.Status <> stOk then
   begin
    rc:='unable to load msg ' + Long2Str(No) + ' (' + Path^ + Long2Str(Global) + '.msg), ' +
        'rc=' + Long2Str(Stream^.Status);

    Dispose(Stream, Done);

    Exit;
   end;

  Stream^.Read(Header, SizeOf(Header));

  if Stream^.Status <> stOk then
   begin
    rc:='unable to read header of msg ' + Long2Str(No) + ' (' + Path^ + Long2Str(Global) + '.msg), ' +
        'rc=' + Long2Str(Stream^.Status);

    Dispose(Stream, Done);

    Exit;
   end;

  FidoToJTMHL(Header, Message);

  jt.msg.SetGlobal(Message, Global);

  if Flags and mfNoText = 0 then
   begin
    Text:=New(PStreamThunk, Init(jt.msg.GetText(Message), jt.StreamLinker, False));

    Text^.Seek(Text^.GetSize);

    Text^.CopyFrom(Stream^, Stream^.GetSize - Stream^.GetPos);

    Dispose(Text, Done);

    if Stream^.Status <> stOk then
     begin
      rc:='unable to read body of msg ' + Long2Str(No) + ' (' + Path^ + Long2Str(Global) + '.msg), ' +
          'rc=' + Long2Str(Stream^.Status);

      jt.msg.Destroy(Message);

      Dispose(Stream, Done);

      Exit;
     end;

    Parser:=jt.msg.CreateParser(Message, False);

    if jt.msg.p.GetKludge(Parser, #1'FLAGS', S) then
     if Pos(' LOK', S) <> 0 then
      begin
       jt.msg.DestroyParser(Parser);

       Dispose(Stream, Done);

       jt.msg.Destroy(Message);

       rc:='message is locked';

       Exit;
      end;

    jt.msg.DestroyParser(Parser);
   end;

  Dispose(Stream, Done);

  Load:=True;
 end;
{
function TFidoMessageBase.LoadReplyInfo(const No: Longint; var Global: Longint; var MSGID, REPLY, rc: String): Boolean;
 var
  Skip: Longint;
  Stream: PStream;
  Parser: PStreamParser;
  S: String;
 begin
  LoadReplyInfo:=False;

  Global:=Longint(Map^.At(No));

  if Global = 0 then
   begin
    rc:='unable to load msg ' + Long2Str(No) + ': isn''t exists';

    Exit;
   end;

  Stream:=New(PDosStreamEx, Init(Path^ + Long2Str(Global) + '.msg', stOpenRead));

  if Stream^.Status <> stOk then
   begin
    rc:='unable to load msg ' + Long2Str(No) + ' (' + Path^ + Long2Str(Global) + '.msg), ' +
        'rc=' + Long2Str(Stream^.Status);

    Dispose(Stream, Done);

    Exit;
   end;

  Stream^.Seek(SizeOf(TFidoMessageHeader));

  Parser:=New(PStreamParser, Init(Stream, True));

  LoadReplyInfo:=True;

  MSGID:='';
  REPLY:='';

  Skip:=0;

  while (not Parser^.EndOfMessage) and (Stream^.Status = stOk) and ((MSGID = '') or (REPLY = '')) do
   begin
    Parser^.GetString(S);

    if (S = '') or (S[1] <> #1) then
     begin
      Inc(Skip);

      if Skip > 10 then
       Break;
     end
    else
     begin
      if Copy(S, 1, 7) = #1'MSGID:' then MSGID:=S else
      if Copy(S, 1, 7) = #1'REPLY:' then REPLY:=S else
       Continue;

      Skip:=0;
     end;
   end;

  Dispose(Parser, Done);
  Dispose(Stream, Done);
 end;
}
function TFidoMessageBase.Store(const No: Longint; const Message: Pointer; var rc: String; const Flags: Longint): Boolean;
 const
  Null: System.Byte = 0;
 var
  Global: Longint;
  Stream: PDosStreamEx;
  Header: TFidoMessageHeader;
  Text: PStreamThunk;
 begin
  Store:=False;

  Global:=Longint(Map^.At(No));

  if Global = 0 then
   begin
    rc:='unable to store msg ' + Long2Str(No) + ': isn''t exists';

    Exit;
   end;

  Stream:=New(PDosStreamEx, Init(Path^ + Long2Str(Global) + '.msg', stOpen));

  if Stream^.Status <> stOk then
   begin
    rc:='unable to store msg ' + Long2Str(No) + ' (' + Path^ + Long2Str(Global) + '.msg), ' +
        'rc=' + Long2Str(Stream^.Status);

    Dispose(Stream, Done);

    Exit;
   end;

  JTMHLToFido(Message, Header, Flags);

  Stream^.Write(Header, SizeOf(Header));

  if Stream^.Status <> stOk then
   begin
    rc:='unable to store header of msg ' + Long2Str(No) + ' (' + Path^ + Long2Str(Global) + '.msg), ' +
        'rc=' + Long2Str(Stream^.Status);

    Dispose(Stream, Done);

    Exit;
   end;

  if Flags and mfNoText = 0 then
   begin
    Text:=New(PStreamThunk, Init(jt.msg.GetText(Message), jt.StreamLinker, False));

    Text^.Seek(0);

    Stream^.CopyFrom(Text^, Text^.GetSize);

    Dispose(Text, Done);

    if Stream^.Status <> stOk then
     begin
      rc:='unable to store body of msg ' + Long2Str(No) + ' (' + Path^ + Long2Str(Global) + '.msg), ' +
          'rc=' + Long2Str(Stream^.Status);

      Dispose(Stream, Done);

      Exit;
     end;

    Stream^.Write(Null, SizeOf(Null));
   end;

  Dispose(Stream, Done);

  Store:=True;
 end;

function TFidoMessageBase.Kill(const No: Longint; var rc: String): Boolean;
 var
  F: Text;
  K: xInteger;
 begin
  if Map^.At(No) <> nil then
   begin
    if IOResult <> 0 then;

    Assign(F, Path^ + Long2Str(Longint(Map^.At(No))) + '.msg');

    Erase(F);

    K:=IOResult;

    Kill:=K = 0;

    if K = 0 then
     Map^.AtDelete(No)
    else
     rc:='unable to kill msg ' + Long2Str(No) + ' (' + Path^ + Long2Str(Longint(Map^.At(No))) + '.msg), ' +
         'rc=' + Long2Str(K);
   end
  else
   rc:='unable to kill msg ' + Long2Str(No) + ': isn''t exists';
 end;

function TFidoMessageBase.NoToGlobal(const No: Longint): Longint;
 begin
  if Map^.At(No) = nil then
   NoToGlobal:=-1
  else
   NoToGlobal:=Longint(Map^.At(No));
 end;

function TFidoMessageBase.GlobalToNo(const Global: Longint): Longint;
 var
  Index: xInteger;
 begin
  if Map^.Search(Pointer(Global), Index) then
   GlobalToNo:=Index
  else
   GlobalToNo:=-1;
 end;

function TFidoMessageBase.LockBase: Boolean;
 begin
  LockBase:=(LastReadStream^.GetSize = 0) or (LastReadStream^.Lock(0, 1) = 0);
 end;

procedure TFidoMessageBase.UnlockBase;
 begin
  LastReadStream^.Unlock(0, 1);
 end;

{
function TFidoMessageBase.SetReplyInfo(const No: Longint; ReplyTo, ReplyNext: System.Word; var rc: String): Boolean;
 var
  Global: Longint;
  Stream: PStream;
 begin
  SetReplyInfo:=False;

  Global:=Longint(Map^.At(No));

  if Global = 0 then
   begin
    rc:='unable to setreplyinfo msg ' + Long2Str(No) + ': isn''t exists';

    Exit;
   end;

  Stream:=New(PDosStreamEx, Init(Path^ + Long2Str(Global) + '.msg', stOpen));

  if Stream^.Status <> stOk then
   begin
    rc:='unable to setreplyinfo/open msg ' + Long2Str(No) + ' (' + Path^ + Long2Str(Global) + '.msg), ' +
        'rc=' + Long2Str(Stream^.Status);

    Dispose(Stream, Done);

    Exit;
   end;

  Stream^.Seek(184);

  Stream^.Write(ReplyTo, SizeOf(ReplyTo));

  Stream^.Seek(188);

  Stream^.Write(ReplyNext, SizeOf(ReplyNext));

  if Stream^.Status <> stOk then
   begin
    rc:='unable to setreplyinfo/write msg ' + Long2Str(No) + ' (' + Path^ + Long2Str(Global) + '.msg), ' +
        'rc=' + Long2Str(Stream^.Status);

    Dispose(Stream, Done);

    Exit;
   end;

  Dispose(Stream, Done);

  SetReplyInfo:=True;
 end;
}
{ private methods }

procedure TFidoMessageBase.Remap;
 var
  Number: Longint;
  SR: SearchRec;
  K: Byte;
 begin
  Map^.DeleteAll;

  FindFirst(Path^ + '*.msg', AnyFile, SR);

  while DosError = 0 do
   begin
    if SR.Attr and (Dos.Directory or Dos.VolumeID) = 0 then
     begin
      K:=Pos('.', SR.Name);

      if K <> 0 then
       begin
        SR.Name:=Copy(SR.Name, 1, K - 1);

        if Str2Longint(SR.Name, Number) then
         if Number > 0 then
          Map^.Insert(Pointer(Number));
       end;
     end;

    FindNext(SR);
   end;

  FindClose(SR);
 end;


procedure TFidoMessageBase.FidoToJTMHL(var Header: TFidoMessageHeader; var Message: Pointer);
 procedure ConvertDate;
  var
   DateTime: TDateTime;
   K: Integer;
   S: String;
  begin
   FillChar(DateTime, SizeOf(DateTime), 0);

   S:=StrPas(Header.DateTime);

   if S[3] = ' ' then
    if S[11] = ' ' then {Fido - "DD MMM YY  HH:MM:SS"}
     begin
      MonthStringToMonthNumber(ExtractWord(2, S, [' ']), DateTime.Month);

      Str2Word(ExtractWord(1, S, [' ']), DateTime.Day);
      Str2Word(ExtractWord(3, S, [' ']), DateTime.Year);

      Delete(S, 1, 11);

      Str2Word(ExtractWord(1, S, [':']), DateTime.Hour);
      Str2Word(ExtractWord(2, S, [':']), DateTime.Min);
      Str2Word(ExtractWord(3, S, [':', ' ']), DateTime.Sec);
     end
    else {Opus - "DD MMM YY HH:MM:SS"}
     begin
      MonthStringToMonthNumber(ExtractWord(2, S, [' ']), DateTime.Month);

      Str2Word(ExtractWord(1, S, [' ']), DateTime.Day);
      Str2Word(ExtractWord(3, S, [' ']), DateTime.Year);

      Delete(S, 1, 10);

      Str2Word(ExtractWord(1, S, [':']), DateTime.Hour);
      Str2Word(ExtractWord(2, S, [':']), DateTime.Min);
      Str2Word(ExtractWord(3, S, [':', ' ']), DateTime.Sec);
     end
   else
    if S[4] = ' ' then {SeaDog - "DOW DD MMM YY HH:MM"}
     begin
      Delete(S, 1, 4);

      MonthStringToMonthNumber(ExtractWord(2, S, [' ']), DateTime.Month);

      Str2Word(ExtractWord(1, S, [' ']), DateTime.Day);
      Str2Word(ExtractWord(3, S, [' ']), DateTime.Year);

      Delete(S, 1, 10);

      Str2Word(ExtractWord(1, S, [':']), DateTime.Hour);
      Str2Word(ExtractWord(2, S, [':']), DateTime.Min);

      DateTime.Sec:=0;
     end
    else
     if S[3] = '-' then {Wierd - "DD-MM-YYYY HH:MM:SS"}
      begin
       for K:=1 to Length(S) do
        if S[K] in ['-', ':'] then S[K]:=' ';

       Str2Word(ExtractWord(1, S, [' ']), DateTime.Day);
       Str2Word(ExtractWord(2, S, [' ']), DateTime.Month);
       Str2Word(ExtractWord(3, S, [' ']), DateTime.Year);
       Str2Word(ExtractWord(4, S, [' ']), DateTime.Hour);
       Str2Word(ExtractWord(5, S, [' ']), DateTime.Min);
       Str2Word(ExtractWord(6, S, [' ']), DateTime.Sec);
      end
     else
      begin
       MSGDateTimeToDateTime(Header.DateWritten, DateTime);

       jt.msg.SetWritten(Message, DateTime);

       Exit;
      end;

   DateTime.Year:=DoubleToY2(DateTime.Year);

   jt.msg.SetWritten(Message, DateTime);
  end;
 var
  DT: TDateTime;
  Attributes: Longint;
 begin
  Message:=jt.msg.Create;

  jt.msg.SetFromName(Message, StrPas(Header.FromName));
  jt.msg.SetToName(Message, StrPas(Header.ToName));
  jt.msg.SetSubject(Message, StrPas(Header.Subject));

  ConvertDate;

  MSGDateTimeToDateTime(Header.DateArrived, DT);

  jt.msg.SetArrived(Message, DT);

  Attributes:=FidoToJTMHLAttributes(Header.Attributes);

  if Header.TimesRead <> 0 then
   Attributes:=Attributes or maRead;

  jt.msg.SetAttributes(Message, Attributes);
 end;

procedure TFidoMessageBase.JTMHLToFido(const Message: Pointer; var Header: TFidoMessageHeader; const Flags: Longint);
 procedure PStringToPChar(const Source: PString; var Destination: array of Char);
  begin
   FillChar(Destination, SizeOf(Destination), 0);

   if Source <> nil then
    StrPCopy(@Destination, Copy(Source^, 1, High(Destination) - Low(Destination)));
  end;
 procedure ConvertDate;
  var
   DT: TDateTime;
  begin
   jt.msg.GetWritten(Message, DT);

   StrPCopy(Header.DateTime,
            lz(DT.Day) + ' ' +
            MonthNumberToMonthString(DT.Month) + ' ' +
            lz(Y2ToDouble(DT.Year)) + '  ' +
            lz(DT.Hour) + ':' +
            lz(DT.Min) + ':' +
            lz(DT.Sec));
  end;
 var
  FromAddress, ToAddress: TAddress;
  Parser: Pointer;
  DT: TDateTime;
 begin
  PStringToPChar(jt.msg.GetFromName(Message), Header.FromName);
  PStringToPChar(jt.msg.GetToName(Message), Header.ToName);
  PStringToPChar(jt.msg.GetSubject(Message), Header.Subject);

  ConvertDate;

  jt.msg.GetWritten(Message, DT);

  DateTimeToMSGDateTime(DT, Header.DateWritten);

  jt.msg.GetArrived(Message, DT);

  DateTimeToMSGDateTime(DT, Header.DateArrived);

  Header.Attributes:=JTMHLToFidoAttributes(jt.msg.GetAttributes(Message));

  if jt.msg.GetAttributes(Message) and maRead <> 0 then
   Inc(Header.TimesRead)
  else
   Header.TimesRead:=0;

  if Flags and mfNoText = 0 then
   begin
    Parser:=jt.msg.CreateParser(Message, Netmail);

    jt.msg.p.GetFromAndToAddress(Parser, FromAddress, ToAddress);

    Header.FromNode:=FromAddress.Node;
    Header.FromNet:=FromAddress.Net;

    Header.ToNode:=ToAddress.Node;
    Header.ToNet:=ToAddress.Net;

    jt.msg.DestroyParser(Parser);
   end;
 end;


function TFidoMessageBase.JTMHLToFidoAttributes(const JTMHLAttributes: Longint): Longint;
 var
  FidoAttributes: Longint;
  K: Byte;
 begin
  FidoAttributes:=faNone;

  for K:=Low(AttributesMap) to High(AttributesMap) do
   if JTMHLAttributes and AttributesMap[K, amtJTMHL] <> 0 then
    FidoAttributes:=FidoAttributes or AttributesMap[K, amtFido];

  JTMHLToFidoAttributes:=FidoAttributes;
 end;

function TFidoMessageBase.FidoToJTMHLAttributes(const FidoAttributes: Longint): Longint;
 var
  JTMHLAttributes: Longint;
  K: Byte;
 begin
  JTMHLAttributes:=maNone;

  for K:=Low(AttributesMap) to High(AttributesMap) do
   if FidoAttributes and AttributesMap[K, amtFido] <> 0 then
    JTMHLAttributes:=JTMHLAttributes or AttributesMap[K, amtJTMHL];

  FidoToJTMHLAttributes:=JTMHLAttributes;
 end;


type
 PAdditionalBlock = ^TAdditionalBlock;
 TAdditionalBlock = packed record
  ReplyTo: System.Word;
  ReplyNext: System.Word;
 end;

{ AdditionalDisposer }

procedure AdditionalDisposer(const Additional: Pointer);
 begin
  Dispose(PAdditionalBlock(Additional));
 end;


procedure TFidoMessageBase.LoadLinkInfo(var Block: TLoadLinkMessageBaseBlock);
 var
  Errors, No: Longint;
 procedure Error(const Message: String);
  begin
   jt.LogWrite('?', Message + ' (' + Path^ + ')');

   Inc(Errors);
  end;
 function Perform(const Global: Longint): Boolean; far;
  var
   Stream: PDosStreamEx;
   Header: TFidoMessageHeader;
   Parser: Pointer;
   MSGIDCRC, REPLYCRC, Skip: Longint;
   Additional: PAdditionalBlock;
   Stop: Boolean;
   S: String;
  begin
   Stop:=False;

   if Global = 0 then
    Error('unable to load msg ' + Long2Str(No) + ': isn''t exists')
   else
    begin
     Stream:=New(PDosStreamEx, Init(Path^ + Long2Str(Global) + '.msg', stOpenRead));

     if Stream^.Status <> stOk then
      begin
       Error('unable to load msg ' + Long2Str(No) + ' (' + Path^ + Long2Str(Global) + '.msg), ' +
             'rc=' + Long2Str(Stream^.Status));

       Dispose(Stream, Done);
      end
     else
      begin
       Stream^.Read(Header, SizeOf(Header));

       Parser:=jt.msg.p.CreateEx(Stream, False, StreamLinker, False);

       MSGIDCRC:=-1;
       REPLYCRC:=-1;

       Skip:=0;

       while (not jt.msg.p.EndOfMessage(Parser)) and (Stream^.Status = stOk) and ((MSGIDCRC = -1) or (REPLYCRC = -1)) do
        begin
         jt.msg.p.GetString(Parser, S);

         if (S = '') or (S[1] <> #1) then
          begin
           Inc(Skip);

           if Skip > 10 then
            Break;
          end
         else
          begin
           if Copy(S, 1, 7) = #1'MSGID:' then MSGIDCRC:=StringCRC32(Copy(S, 9, 255), -1) else
           if Copy(S, 1, 7) = #1'REPLY:' then REPLYCRC:=StringCRC32(Copy(S, 9, 255), -1) else
            Continue;

           Skip:=0;
          end;
        end;

       jt.msg.p.Destroy(Parser);

       Dispose(Stream, Done);

       New(Additional);

       Additional^.ReplyTo:=Header.ReplyTo;
       Additional^.ReplyNext:=Header.ReplyNext;

       Stop:=not Block.Add(Block.Param, No, Global, MSGIDCRC, REPLYCRC, Additional);
      end;
    end;

   Perform:=(Errors > 10) or Stop;

   Inc(No);
  end;
 begin
  Block.Supported:=True;
  Block.Dispose:=AdditionalDisposer;
  Block.Flags:=llmbbfMsgLike;

  Errors:=0;
  No:=0;

  Block.Did:=Map^.FirstThat(@Perform) = nil;
 end;

procedure TFidoMessageBase.StoreLinkInfo(var Block: TStoreLinkMessageBaseBlock);
 var
  Errors, K: Longint;
  NewAdditional: TAdditionalBlock;
  Message: PLinkerMessage;
  Update: Boolean;
  Stream: PDosStreamEx;
 procedure Error(const Message: String);
  begin
   jt.LogWrite('?', Message + ' (' + Path^ + ')');

   Inc(Errors);
  end;
 begin
  Block.Supported:=True;
  Block.Did:=True;

  Errors:=0;

  for K:=0 to Block.Count - 1 do
   begin
    if Errors > 10 then
     begin
      Block.Did:=False;

      Break;
     end;

    if not Block.Callback(Block.Param) then
     begin
      Block.Did:=False;

      Exit;
     end;

    Message:=Block.Items^[K];

    if Message^.Owner^.Global = -1 then
     NewAdditional.ReplyTo:=0
    else
     NewAdditional.ReplyTo:=Message^.Owner^.Global;

    if Message^.ChildFirst = nil then
     NewAdditional.ReplyNext:=0
    else
     NewAdditional.ReplyNext:=Message^.ChildFirst^.Global;

    with PAdditionalBlock(Message^.Additional)^ do
     Update:=(ReplyTo <> NewAdditional.ReplyTo) or (ReplyNext <> NewAdditional.ReplyNext);

    if Update then
     begin
      Stream:=New(PDosStreamEx, Init(Path^ + Long2Str(Message^.Global) + '.msg', stOpen));

      if Stream^.Status <> stOk then
       begin
        Error('unable to setreplyinfo/open msg ' + Long2Str(Message^.No) + ' (' + Path^ + Long2Str(Message^.Global) +
              '.msg), rc=' + Long2Str(Stream^.Status));

        Dispose(Stream, Done);
       end
      else
       begin
        Stream^.Seek(184);

        Stream^.Write(NewAdditional.ReplyTo, SizeOf(NewAdditional.ReplyTo));

        Stream^.Seek(188);

        Stream^.Write(NewAdditional.ReplyNext, SizeOf(NewAdditional.ReplyNext));

        if Stream^.Status <> stOk then
         Error('unable to setreplyinfo/write msg ' + Long2Str(Message^.No) + ' (' + Path^ +
               Long2Str(Message^.Global) + '.msg), rc=' + Long2Str(Stream^.Status));

        Dispose(Stream, Done);
       end;
     end;
   end;
 end;

function TFidoMessageBase.ExtendedOpened(const Operation: Longint; var Data): Boolean;
 var
  LastRead: Longint absolute Data;
 begin
  if Operation = eooGetLastRead then ExtendedOpened:=GetLastRead(LastRead) else
  if Operation = eooSetLastRead then ExtendedOpened:=SetLastRead(LastRead) else
   ExtendedOpened:=inherited ExtendedOpened(Operation, Data);
 end;

function TFidoMessageBase.GetLastRead(var LastRead: Longint): Boolean;
 var
  Data: System.Word;
 begin
  LastReadStream^.Seek(DefaultLastReadUserNumber * SizeOf(Data));

  LastReadStream^.Read(Data, SizeOf(Data));

  if LastReadStream^.Status = stOk then
   begin
    LastRead:=Data;

    GetLastRead:=True;
   end
  else
   begin
    LastReadStream^.Reset;

    GetLastRead:=False;
   end;
 end;

function TFidoMessageBase.SetLastRead(var LastRead: Longint): Boolean;
 var
  Data: System.Word;
 begin
  LastReadStream^.Seek(DefaultLastReadUserNumber * SizeOf(Data));

  Data:=LastRead;

  LastReadStream^.Write(Data, SizeOf(Data));

  if LastReadStream^.Status = stOk then
   SetLastRead:=True
  else
   begin
    LastReadStream^.Reset;

    SetLastRead:=False;
   end;
 end;

end.