unit mbPLUG;

interface
uses
     PlugAPI,
     PlugSIDE,

     Objects,

     mbAPI;

type
 PMessageBase = ^TMessageBase;
 TMessageBase = object(TObject)
  BaseType: TBaseType;

  constructor Init(const Path: String; const Create: Boolean; const ABaseType: TBaseType; var rc: String);
  destructor Done; virtual;
 public
  function Netmail: Boolean; virtual;
  function Add(const Message: Pointer; var rc: String; const Flags: Longint): Boolean; virtual;
  function Count: Longint; virtual;
  function Load(const No: Longint; var Message: Pointer; var rc: String; const Flags: Longint): Boolean; virtual;
  function Store(const No: Longint; const Message: Pointer; var rc: String; const Flags: Longint): Boolean; virtual;
  function Kill(const No: Longint; var rc: String): Boolean; virtual;
  function NoToGlobal(const No: Longint): Longint; virtual;
  function GlobalToNo(const Global: Longint): Longint; virtual;
  function Lock: Boolean; virtual;
  procedure Unlock; virtual;
  function LockBase: Boolean; virtual;
  procedure UnlockBase; virtual;
 public
  procedure LoadLinkInfo(var Block: TLoadLinkMessageBaseBlock); virtual;
  procedure StoreLinkInfo(var Block: TStoreLinkMessageBaseBlock); virtual;
  function ExtendedOpened(const Operation: Longint; var Data): Boolean; virtual;
 private
  LockCount: Longint;
 end;

procedure mbDone(const mb: Pointer); export;
function mbNetmail(const mb: Pointer): Boolean; export;
function mbAdd(const mb: Pointer; const Message: Pointer; var rc: String; const Flags: Longint): Boolean; export;
function mbCount(const mb: Pointer): Longint; export;
function mbLoad(const mb: Pointer; const No: Longint; var Message: Pointer; var rc: String;
  const Flags: Longint): Boolean; export;
function mbStore(const mb: Pointer; const No: Longint; const Message: Pointer; var rc: String;
  const Flags: Longint): Boolean; export;
function mbKill(const mb: Pointer; const No: Longint; var rc: String): Boolean; export;
function mbNoToGlobal(const mb: Pointer; const No: Longint): Longint; export;
function mbGlobalToNo(const mb: Pointer; const Global: Longint): Longint; export;
function mbLock(const mb: Pointer): Boolean; export;
procedure mbUnlock(const mb: Pointer); export;

procedure mbLoadLinkInfo(const mb: Pointer; var Block: TLoadLinkMessageBaseBlock); export;
procedure mbStoreLinkInfo(const mb: Pointer; var Block: TStoreLinkMessageBaseBlock); export;

function mbExtendedOpened(const mb: Pointer; const Operation: Longint; var Data): Boolean; export;
function mbExtended(const Operation: Longint; var Data): Boolean; export;

procedure mbPack(const Path: String; var Block: TPackMessageBaseBlock); export;
function mbKillBase(const Path: String; var rc: String): Boolean; export;
function mbIsBaseExists(const Path: String): Boolean; export;

const
 MessageBaseInterface: TMessageBaseInterface = (
  FullName: nil;
  ShortName: nil;
  FullNameUp: nil;
  ShortNameUp: nil;
  Name: nil;
  id: -1;
  AreaType: atNew;
  mbo: nil;
  Reserved: (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);

  Init: nil;
  Done: mbDone;
  Netmail: mbNetmail;
  Add: mbAdd;
  Count: mbCount;
  Load: mbLoad;
  Store: mbStore;
  Kill: mbKill;
  NoToGlobal: mbNoToGlobal;
  GlobalToNo: mbGlobalToNo;
  Lock: mbLock;
  Unlock: mbUnlock;

  LoadLinkInfo: mbLoadLinkInfo;
  StoreLinkInfo: mbStoreLinkInfo;

  ExtendedOpened: mbExtendedOpened;
  Extended: mbExtended;

  Pack: mbPack;
  KillBase: mbKillBase;
  IsBaseExists: mbIsBaseExists);

implementation

{ TMessageBase }

constructor TMessageBase.Init(const Path: String; const Create: Boolean; const ABaseType: TBaseType; var rc: String);
 begin
  inherited Init;

  BaseType:=ABaseType;
 end;

destructor TMessageBase.Done;
 begin
  inherited Done;
 end;

function TMessageBase.Netmail: Boolean;
 begin
  Netmail:=BaseType = btNetmail;
 end;

function TMessageBase.Add(const Message: Pointer; var rc: String; const Flags: Longint): Boolean;
 begin
  Abstract;
 end;

function TMessageBase.Count: Longint;
 begin
  Abstract;
 end;

function TMessageBase.Load(const No: Longint; var Message: Pointer; var rc: String; const Flags: Longint): Boolean;
 begin
  Abstract;
 end;

function TMessageBase.Store(const No: Longint; const Message: Pointer; var rc: String; const Flags: Longint): Boolean;
 begin
  Abstract;
 end;

function TMessageBase.Kill(const No: Longint; var rc: String): Boolean;
 begin
  Abstract;
 end;

function TMessageBase.NoToGlobal(const No: Longint): Longint;
 begin
  Abstract;
 end;

function TMessageBase.GlobalToNo(const Global: Longint): Longint;
 begin
  Abstract;
 end;

function TMessageBase.Lock: Boolean;
 begin
  if LockCount > 0 then
   begin
    Inc(LockCount);

    Lock:=True;
   end
  else
   if LockBase then
    begin
     Inc(LockCount);

     Lock:=True;
    end
   else
    Lock:=False;
 end;

procedure TMessageBase.Unlock;
 begin
  Dec(LockCount);

  if LockCount = 0 then
   UnlockBase;
 end;

function TMessageBase.LockBase: Boolean;
 begin
  Abstract;
 end;

procedure TMessageBase.UnlockBase;
 begin
  Abstract;
 end;


procedure TMessageBase.LoadLinkInfo(var Block: TLoadLinkMessageBaseBlock);
 begin
  Block.Supported:=False;
  Block.Did:=False;
 end;

procedure TMessageBase.StoreLinkInfo(var Block: TStoreLinkMessageBaseBlock);
 begin
  Block.Supported:=False;
  Block.Did:=False;
 end;


function TMessageBase.ExtendedOpened(const Operation: Longint; var Data): Boolean;
 begin
  ExtendedOpened:=False;
 end;

{ thunks for TMessageBaseInterface }

procedure mbDone(const mb: Pointer);
 begin
  Dispose(PMessageBase(mb), Done);
 end;

function mbNetmail(const mb: Pointer): Boolean;
 begin
  mbNetmail:=PMessageBase(mb)^.Netmail;
 end;

function mbAdd(const mb: Pointer; const Message: Pointer; var rc: String; const Flags: Longint): Boolean;
 begin
  mbAdd:=PMessageBase(mb)^.Add(Message, rc, Flags);
 end;

function mbCount(const mb: Pointer): Longint;
 begin
  mbCount:=PMessageBase(mb)^.Count;
 end;

function mbLoad(const mb: Pointer; const No: Longint; var Message: Pointer; var rc: String; const Flags: Longint): Boolean;
 begin
  mbLoad:=PMessageBase(mb)^.Load(No, Message, rc, Flags);
 end;

function mbStore(const mb: Pointer; const No: Longint; const Message: Pointer; var rc: String; const Flags: Longint): Boolean;
 begin
  mbStore:=PMessageBase(mb)^.Store(No, Message, rc, Flags);
 end;

function mbKill(const mb: Pointer; const No: Longint; var rc: String): Boolean;
 begin
  mbKill:=PMessageBase(mb)^.Kill(No, rc);
 end;

function mbNoToGlobal(const mb: Pointer; const No: Longint): Longint;
 begin
  mbNoToGlobal:=PMessageBase(mb)^.NoToGlobal(No);
 end;

function mbGlobalToNo(const mb: Pointer; const Global: Longint): Longint;
 begin
  mbGlobalToNo:=PMessageBase(mb)^.GlobalToNo(Global);
 end;

function mbLock(const mb: Pointer): Boolean;
 begin
  mbLock:=PMessageBase(mb)^.Lock;
 end;

procedure mbUnlock(const mb: Pointer);
 begin
  PMessageBase(mb)^.Unlock;
 end;

procedure mbLoadLinkInfo(const mb: Pointer; var Block: TLoadLinkMessageBaseBlock);
 begin
  PMessageBase(mb)^.LoadLinkInfo(Block);
 end;

procedure mbStoreLinkInfo(const mb: Pointer; var Block: TStoreLinkMessageBaseBlock);
 begin
  PMessageBase(mb)^.StoreLinkInfo(Block);
 end;

function mbExtendedOpened(const mb: Pointer; const Operation: Longint; var Data): Boolean;
 begin
  mbExtendedOpened:=PMessageBase(mb)^.ExtendedOpened(Operation, Data);
 end;

function mbExtended(const Operation: Longint; var Data): Boolean;
 begin
  mbExtended:=False;
 end;

procedure mbPack(const Path: String; var Block: TPackMessageBaseBlock);
 begin
  Block.Supported:=False;
  Block.Did:=False;
 end;

function mbKillBase(const Path: String; var rc: String): Boolean;
 begin
  rc:='unsupported';

  mbKillBase:=False;
 end;

function mbIsBaseExists(const Path: String): Boolean;
 begin
  mbIsBaseExists:=False;
 end;

end.