unit mbsqCche;

interface
uses
     PlugAPI,
     PlugSIDE,

     Objects,
     Drivers,

     Thunks,

     RStream,

     mbsqCmn,

     Wizard;

type
 PAdditionalBlock = ^TAdditionalBlock;
 TAdditionalBlock = packed record
  ReplyTo: Longint;
  ReplyInfo: TSquishReplyInfo;
 end;

 PLinkerCacheItem = ^TLinkerCacheItem;
 TLinkerCacheItem = packed record
  No: Longint;
  Global: Longint;
  MSGIDCRC: Longint;
  REPLYCRC: Longint;
  Additional: TAdditionalBlock;
  Used: Boolean;
 end;

 PLinkerCacheItems = ^TLinkerCacheItems;
 TLinkerCacheItems = object(TSortedCollection)
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  function KeyOf(Item: Pointer): Pointer; virtual;
  procedure FreeItem(Item: Pointer); virtual;
 end;

 PLinkerCache = ^TLinkerCache;
 TLinkerCache = object(TLinkerCacheItems)
  Area: Pointer;
  Changed: Boolean;

  constructor Init(const AArea: Pointer);
  destructor Done; virtual;
 end;

procedure AdditionalDisposer(const Additional: Pointer); export;

implementation

{ AdditionalDisposer }

procedure AdditionalDisposer(const Additional: Pointer);
 begin
  Dispose(PAdditionalBlock(Additional));
 end;

{ TLinkerCacheItems }

function TLinkerCacheItems.Compare(Key1, Key2: Pointer): xInteger;
 begin
  Compare:=CompareLongint(Longint(Key1), Longint(Key2));
 end;

function TLinkerCacheItems.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=Pointer(PLinkerCacheItem(Item)^.Global);
 end;

procedure TLinkerCacheItems.FreeItem(Item: Pointer);
 begin
  Dispose(PLinkerCacheItem(Item));
 end;

{ TLinkerCache }

constructor TLinkerCache.Init(const AArea: Pointer);
 var
  Stream: PRamStream;
  Countdown: Longint;
  Item: PLinkerCacheItem;
 begin
  inherited Init($1000, $1000);

  Area:=AArea;

  Stream:=New(PRamStream, Init);

  if jt.rf.Get(SquishResourceFile, 'SquishLinkerCache.' + jt.area.GetName(Area)^, Stream, StreamLinker) then
   begin
    Stream^.Seek(0);

    Stream^.Read(Countdown, SizeOf(Countdown));

    if Countdown > Limit then
     SetLimit(Countdown or Limit);

    while Countdown > 0 do
     begin
      New(Item);

      Stream^.Read(Item^, SizeOf(TLinkerCacheItem));

      Item^.Used:=False;

      AtInsert(Count, Item);

      Dec(Countdown);
     end;
   end
  else
   Changed:=True;

  Dispose(Stream, Done);
 end;

destructor TLinkerCache.Done;
 var
  Stream: PRamStream;
  Countdown: Longint;
 procedure Perform(const Item: PLinkerCacheItem); far;
  begin
   Stream^.Write(Item^, SizeOf(TLinkerCacheItem));
  end;
 begin
  if Changed then
   begin
    Stream:=New(PRamStream, Init);

    Stream^.Seek(SizeOf(Countdown) + Longint(Count) * Longint(SizeOf(TLinkerCacheItem)));

    Stream^.Truncate;

    Stream^.Seek(0);

    Countdown:=Count;

    Stream^.Write(Countdown, SizeOf(Countdown));

    ForEach(@Perform);

    jt.rf.Add(SquishResourceFile, 'SquishLinkerCache.' + jt.area.GetName(Area)^, Stream, StreamLinker);

    Dispose(Stream, Done);
   end;

  inherited Done;
 end;

end.