unit mbsqCmn;

interface
uses
     Wizard,

     jtMHLcmn;

type
 TAttributesMapType = (amtSquish, amtJTMHL);

const
 SquishMaxMsg           : Longint = $FFFF;
 SquishKeepDays         : Longint = $7FFF;

 squishFrameID          = $AFAE4453;

 squishFrameMessage     = $0000;
 squishFrameFree        = $0001;
 squishFrameUpdate      = $0003;

 squishaNone            = $00000;
 squishaPrivate         = $00001;
 squishaCrash           = $00002;
 squishaReceived        = $00004;
 squishaSent            = $00008;
 squishaAttach          = $00010;
 squishaTransit         = $00020;
 squishaOrphan          = $00040;
 squishaKill            = $00080;
 squishaLocal           = $00100;
 squishaHold            = $00200;
 squishaXX2             = $00400;
 squishaFRq             = $00800;
 squishaRRq             = $01000;
 squishaRRc             = $02000;
 squishaARq             = $04000;
 squishaURq             = $08000;
 squishaScanned         = $10000;
 squishaUID             = $20000;
 squishaRead            = $80000;

 AttributesMap: array[1..19, TAttributesMapType] of Longint =
   ((squishaPrivate,  maPrivate),
    (squishaCrash,    maCrash),
    (squishaReceived, maReceived),
    (squishaSent,     maSent),
    (squishaAttach,   maAttach),
    (squishaTransit,  maTransit),
    (squishaOrphan,   maOrphan),
    (squishaKill,     maKill),
    (squishaLocal,    maLocal),
    (squishaHold,     maHold),
    (squishaXX2,      maNone),
    (squishaFRq,      maFRq),
    (squishaRRq,      maRRq),
    (squishaRRc,      maRRc),
    (squishaARq,      maARq),
    (squishaURq,      maURq),
    (squishaScanned,  maScanned),
    (squishaUID,      maNone),
    (squishaRead,     maRead));

type
 TSquishBaseHeader = packed record
  Size: System.Word;
  Rsvd1: System.Word;
  NumMsg: Longint;
  HighMsg: Longint;
  SkipMsg: Longint;
  HighWater: Longint;
  UID: Longint;
  Base: array[0..79] of Char;
  FirstMessageFrame: Longint;
  LastMessageFrame: Longint;
  FirstFreeFrame: Longint;
  LastFreeFrame: Longint;
  EndOfFile: Longint;
  MaxMsg: Longint;
  KeepDays: System.Word;
  FrameSize: System.Word;
  Rsvd2: array[1..124] of Byte;
 end;

 TSquishMessageFrame = packed record
  id: Longint;
  NextFrame: Longint;
  PrevFrame: Longint;
  FrameLength: Longint;
  MsgLength: Longint;
  ControlLength: Longint;
  FrameType: System.Word;
  Reserved: System.Word;
 end;

 PSquishReplyInfo = ^TSquishReplyInfo;
 TSquishReplyInfo = array[1..9] of Longint;

 TSquishMessageHeader = packed record
  Attributes: Longint;                          { 04 }
  FromName: array[0..35] of Char;               { 36 }
  ToName: array[0..35] of Char;                 { 36 }
  Subject: array[0..71] of Char;                { 72 }
  FromAddress: TAddress;                        { 08 }
  ToAddress: TAddress;                          { 08 }
  DateWritten: Longint;                         { 04 }
  DateArrived: Longint;                         { 04 }
  UTCoffset: System.Word;                       { 02 } { 174 }
  ReplyTo: Longint;                             { 04 }
  ReplyInfo: TSquishReplyInfo;                  { 36 }
  UID: Longint;                                 { 04 }
  Date: array[0..19] of Char;                   { 20 } { 238 }
 end;

 PSquishMessageIndex = ^TSquishMessageIndex;
 TSquishMessageIndex = packed record
  Offset: Longint;
  UID: Longint;
  Hash: Longint;
 end;

const
 FlushAfterAdd          : Boolean       = False;
 FlushAfterStore        : Boolean       = False;

 SquishResourceFile     : Pointer       = nil;

function MakeSquishHash(const Name: String): Longint;

function smbIsBaseExists(const Path: String): Boolean; export;

implementation

{ MakeSquishHash }

function MakeSquishHash(const Name: String): Longint;
 var
  K: Byte;
  Hash, Tmp: Longint;
 begin
  Hash:=0;

  for K:=1 to Length(Name) do
   begin
    Hash:=(Hash shl 4) + Byte(LoCase(Name[K]));

    Tmp:=Hash and $F0000000;

    if Tmp <> 0 then
     Hash:=(Hash or (Tmp shr 24)) or Tmp;
   end;

  MakeSquishHash:=Hash and $7FFFFFFF;
 end;

{ smbIsBaseExists }

function smbIsBaseExists(const Path: String): Boolean;
 begin
  smbIsBaseExists:=ExistFile(ForceExtension(AddDot(Path), 'sqd')) or
                   ExistFile(ForceExtension(AddDot(Path), 'sqi')) or
                   ExistFile(ForceExtension(AddDot(Path), 'sql'))
 end;

end.
