unit mbsqCore;

interface
uses
     mbAPI,
     mbPLUG,

     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     LinkAPI,

     Objects,
     Strings,

     Thunks,

     jtMHLcmn,
     jtMHLstm,

     RStream,

     mbsqCmn,
     mbsqCche,

     Wizard;

const
 DefaultLastReadUserNumber              = 0;

type
 PLongint = ^Longint;

 PSquishMessageBase = ^TSquishMessageBase;
 TSquishMessageBase = object(TMessageBase)
  constructor Init(const APath: String; const Create: Boolean; const ABaseType: TBaseType; var rc: String);
  destructor Done; virtual;
 public
  Path: PString;
  DataStream, IndexStream: PDosStreamEx;
  BaseHeader: TSquishBaseHeader;

  function Add(const Message: Pointer; var rc: String; const Flags: Longint): Boolean; virtual;
  function Count: Longint; virtual;
  function Load(const No: Longint; var Message: Pointer; var rc: String; const Flags: Longint): Boolean; virtual;
  function Store(const No: Longint; const Message: Pointer; var rc: String; const Flags: Longint): Boolean; virtual;
  function Kill(const No: Longint; var rc: String): Boolean; virtual;
  function NoToGlobal(const No: Longint): Longint; virtual;
  function GlobalToNo(const Global: Longint): Longint; virtual;

  function LockBase: Boolean; virtual;
  procedure UnlockBase; virtual;
 private
  procedure MessageToSquish(const Message: Pointer; var Header: TSquishMessageHeader;
   var ControlInformationSize: Longint; var ControlInformation: PRamStream; const Adding: Boolean);
  procedure SquishToMessage(var Header: TSquishMessageHeader; const ControlInformation: PRamStream;
   const Message: Pointer);

  function JTMHLToSquishAttributes(const JTMHLAttributes: Longint): Longint;
  function SquishToJTMHLAttributes(const SquishAttributes: Longint): Longint;

  function ReadBaseHeader(var rc: String): Boolean;
  function WriteBaseHeader(var rc: String): Boolean;

  function SearchForFreeFrame(const FrameLength: Longint; var Frame: TSquishMessageFrame; var FramePos: Longint;
   var rc: String): Boolean;

  function UnlinkFrameFromChain(var Frame: TSquishMessageFrame; var FramePos, FirstFrame, LastFrame: Longint;
   var rc: String): Boolean;
  function LinkFrameToEndOfChain(var Frame: TSquishMessageFrame; var FramePos, FirstFrame, LastFrame: Longint;
   var rc: String): Boolean;
  function LinkFrameIntoChain(var Frame: TSquishMessageFrame; var FramePos, FirstFrame, LastFrame: Longint;
   var rc: String): Boolean;

  procedure SquishSwap(var Number: Longint);
 public
  procedure LoadLinkInfo(var Block: TLoadLinkMessageBaseBlock); virtual;
  procedure StoreLinkInfo(var Block: TStoreLinkMessageBaseBlock); virtual;
  function ExtendedOpened(const Operation: Longint; var Data): Boolean; virtual;

  function GetLastRead(var LastRead: Longint): Boolean;
  function SetLastRead(var LastRead: Longint): Boolean;
 end;

implementation

{ TSquishMessageBase }

constructor TSquishMessageBase.Init(const APath: String; const Create: Boolean; const ABaseType: TBaseType; var rc: String);
 var
  sqdExists, sqiExists: Boolean;
 begin
  inherited Init(APath, Create, ABaseType, rc);

  Path:=NewStr(ForceExtension(APath, ''));

  if Path = nil then
   begin
    rc:='invalid path';

    Done;

    Fail;
   end;

  DataStream:=New(PDosStreamEx, Init(Path^ + 'sqd', stOpen or stDenyNone));
  IndexStream:=New(PDosStreamEx, Init(Path^ + 'sqi', stOpen or stDenyNone));

  if (DataStream^.Status = stOk) and (IndexStream^.Status = stOk) then
   begin
    DataStream^.Read(BaseHeader, SizeOf(BaseHeader));

    if DataStream^.Status <> stOk then
     begin
      rc:='unable to read header of ' + Path^ + 'sqd: rc=' + Long2Str(DataStream^.Status);

      Done;

      Fail;
     end;

    Exit;
   end;

  if (DataStream^.Status = stOk) and (IndexStream^.Status <> stOk) then
   begin
    rc:='unable to open ' + Path^ + 'sqi (but sqd ok): rc=' + Long2Str(IndexStream^.Status);

    Done;

    Fail;
   end;

  if (DataStream^.Status <> stOk) and (IndexStream^.Status = stOk) then
   begin
    rc:='unable to open ' + Path^ + 'sqd (but sqi ok): rc=' + Long2Str(DataStream^.Status);

    Done;

    Fail;
   end;

  sqdExists:=ExistFile(Path^ + 'sqd');
  sqiExists:=ExistFile(Path^ + 'sqi');

  if sqdExists or sqiExists then
   begin
    rc:='unable to open ';

    if sqdExists then
     begin
      rc:=rc + Path^ + 'sqd (exists), rc=' + Long2Str(DataStream^.Status);

      if sqiExists then
       rc:=rc + ', ';
     end;

    if sqiExists then
     rc:=rc + Path^ + 'sqi (exists), rc=' + Long2Str(IndexStream^.Status);

    Done;

    Fail;
   end;

  if not Create then
   begin
    rc:='squish base ' + Path^ + ' absent';

    Done;

    Fail;
   end;

  if not CreateDirectory(JustPathName(Path^)) then
   begin
    rc:='failed to create directory ' + JustPathName(Path^);

    Done;

    Fail;
   end;

  if DataStream <> nil then
   Dispose(DataStream, Done);

  if IndexStream <> nil then
   Dispose(IndexStream, Done);

  DataStream:=New(PDosStreamEx, Init(Path^ + 'sqd', stCreate or stDenyNone));
  IndexStream:=New(PDosStreamEx, Init(Path^ + 'sqi', stCreate or stDenyNone));

  if DataStream^.Status <> stOk then
   begin
    rc:='unable to create ' + Path^ + 'sqd: rc=' + Long2Str(DataStream^.Status);

    Done;

    Fail;
   end;

  if IndexStream^.Status <> stOk then
   begin
    rc:='unable to create ' + Path^ + 'sqi: rc=' + Long2Str(IndexStream^.Status);

    if DataStream^.GetSize = 0 then
     begin
      Dispose(DataStream, Done);

      DataStream:=nil;

      EraseFile(Path^ + 'sqd');
     end;

    Done;

    Fail;
   end;

  FillChar(BaseHeader, SizeOf(BaseHeader), 0);

  with BaseHeader do
   begin
    Size:=SizeOf(BaseHeader);

    UID:=1;

    StrPCopy(Base, Copy(Path^, 1, 79));

    EndOfFile:=SizeOf(BaseHeader);
    MaxMsg:=SquishMaxMsg;
    KeepDays:=SquishKeepDays;
    FrameSize:=SizeOf(TSquishMessageFrame);
   end;

  DataStream^.Write(BaseHeader, SizeOf(BaseHeader));

  if DataStream^.Status <> stOk then
   begin
    rc:='unable to write header of ' + Path^ + 'sqd: rc=' + Long2Str(DataStream^.Status);

    Done;

    Fail;
   end;

  DataStream^.Flush;
  IndexStream^.Flush;
 end;

destructor TSquishMessageBase.Done;
 begin
  DisposeStr(Path);

  if IndexStream <> nil then
   Dispose(IndexStream, Done);

  if DataStream <> nil then
   Dispose(DataStream, Done);

  inherited Done;
 end;

{ public methods }

function TSquishMessageBase.Add(const Message: Pointer; var rc: String; const Flags: Longint): Boolean;
 var
  Frame, FrameTemp: TSquishMessageFrame;
  Header: TSquishMessageHeader;
  Index: TSquishMessageIndex;
  ControlInformation: PRamStream;
  FramePos, Anchor: Longint;
  Text: PStream;
 begin
  Add:=False;

  if Lock then
   begin
    if ReadBaseHeader(rc) then
     begin
      Text:=New(PStreamThunk, Init(jt.msg.GetText(Message), jt.StreamLinker, False));

      MessageToSquish(Message, Header, Anchor, ControlInformation, True);

      FillChar(Frame, SizeOf(Frame), 0);

      Frame.id:=SquishFrameID;
      Frame.ControlLength:=ControlInformation^.GetSize;
      Frame.MsgLength:=SizeOf(Header) + ControlInformation^.GetSize + Text^.GetSize - Anchor;
      Frame.FrameLength:=Frame.MsgLength;
      Frame.FrameType:=squishFrameMessage;

      rc:='';

      if SearchForFreeFrame(Frame.FrameLength, FrameTemp, FramePos, rc) then
       begin
        if FramePos <> 0 then
         if UnlinkFrameFromChain(FrameTemp, FramePos, BaseHeader.FirstFreeFrame, BaseHeader.LastFreeFrame, rc) then
          Frame.FrameLength:=FrameTemp.FrameLength
         else
        else
         begin
          if BaseHeader.EndOfFile > DataStream^.GetSize then
           BaseHeader.EndOfFile:=DataStream^.GetSize;

          if BaseHeader.EndOfFile < BaseHeader.Size then
           BaseHeader.EndOfFile:=BaseHeader.Size;

          FramePos:=BaseHeader.EndOfFile;

          DataStream^.Seek(FramePos + SizeOf(TSquishMessageFrame) + Frame.FrameLength);

          DataStream^.Truncate;

          if DataStream^.Status <> stOk then
           begin
            rc:='unable to allocate space for new frame, rc=' + Long2Str(DataStream^.Status);

            Dispose(Text, Done);

            DataStream^.Reset;

            Exit;
           end;

          Inc(BaseHeader.EndOfFile, SizeOf(TSquishMessageFrame) + Frame.FrameLength);
         end;

        if rc = '' then
         LinkFrameToEndOfChain(Frame, FramePos, BaseHeader.FirstMessageFrame, BaseHeader.LastMessageFrame, rc);

        if rc = '' then
         begin
          DataStream^.Seek(FramePos);

          DataStream^.Write(Frame, SizeOf(Frame));

          DataStream^.Write(Header, SizeOf(Header));

          ControlInformation^.Seek(0);

          DataStream^.CopyFrom(ControlInformation^, ControlInformation^.GetSize);

          Text^.Seek(Anchor);

          DataStream^.CopyFrom(Text^, Text^.GetSize - Anchor);

          Inc(BaseHeader.NumMsg);
          Inc(BaseHeader.HighMsg);

          if WriteBaseHeader(rc) then
           begin
            Index.Offset:=FramePos;
            Index.UID:=Header.UID;

            if jt.msg.GetToName(Message) = nil then
             Index.Hash:=0
            else
             Index.Hash:=MakeSquishHash(jt.msg.GetToName(Message)^);

            IndexStream^.Seek(IndexStream^.GetSize);

            IndexStream^.Write(Index, SizeOf(Index));

            if IndexStream^.Status <> stOk then
             IndexStream^.Reset;
           end;
         end;
       end;

      Add:=rc = '';

      Dispose(ControlInformation, Done);

      Dispose(Text, Done);
     end;

    Unlock;
   end
  else
   rc:='unable to lock base';

  if FlushAfterAdd then
   begin
    DataStream^.Flush;

    IndexStream^.Flush;
   end;
 end;


function TSquishMessageBase.Count: Longint;
 begin
  Count:=IndexStream^.GetSize div SizeOf(TSquishMessageIndex);
 end;

function TSquishMessageBase.Load(const No: Longint; var Message: Pointer; var rc: String; const Flags: Longint): Boolean;
 var
  Index: TSquishMessageIndex;
  Frame: TSquishMessageFrame;
  Header: TSquishMessageHeader;
  ControlInformation, Body: PRamStream;
  Text: PStream;
  Ch: Char;
 begin
  Load:=False;

  IndexStream^.Seek(No * SizeOf(Index));

  IndexStream^.Read(Index, SizeOf(Index));

  if IndexStream^.Status <> stOk then
   begin
    rc:='unable to read index entry #' + Long2Str(No) + ' at ' + HexL(No * SizeOf(Index)) +
     ', rc=' + Long2Str(IndexStream^.Status);

    IndexStream^.Reset;

    Exit;
   end;

  DataStream^.Seek(Index.Offset);

  DataStream^.Read(Frame, SizeOf(Frame));

  if DataStream^.Status <> stOk then
   begin
    rc:='unable to load frame header at ' + HexL(Index.Offset) + ', rc=' + Long2Str(DataStream^.Status);

    DataStream^.Reset;

    Exit;
   end;

  if Frame.id <> squishFrameID then
   begin
    rc:='wrong frame id at ' + HexL(Index.Offset);

    Exit;
   end;

  if Frame.FrameType = squishFrameUpdate then
   begin
    rc:='frame ' + HexL(Index.Offset) + ' is being processed by another process';

    Exit;
   end;

  if Frame.FrameType <> squishFrameMessage then
   begin
    rc:='frame ' + HexL(Index.Offset) + ' type is unknown';

    Exit;
   end;

  DataStream^.Read(Header, SizeOf(Header));

  if DataStream^.Status <> stOk then
   begin
    rc:='unable to read message header of frame at ' + HexL(Index.Offset) + ', rc=' + Long2Str(DataStream^.Status);

    DataStream^.Reset;

    Exit;
   end;

  if Header.Attributes and $40000000 <> 0 then
   begin
    rc:='message is locked';

    Exit;
   end;

  if Flags and mfNoText = 0 then
   begin
    ControlInformation:=New(PRamStream, Init);

    ControlInformation^.CopyFrom(DataStream^, Frame.ControlLength);

    if DataStream^.Status <> stOk then
     begin
      rc:='unable to read control information of frame at ' + HexL(Index.Offset) + ', rc=' + Long2Str(DataStream^.Status);

      DataStream^.Reset;

      Exit;
     end;

    Message:=jt.msg.Create;

    Text:=New(PStreamThunk, Init(jt.msg.GetText(Message), jt.StreamLinker, False));

    jt.msg.SetGlobal(Message, Index.UID);

    SquishToMessage(Header, ControlInformation, Message);


    Text^.CopyFrom(DataStream^, Frame.MsgLength - Frame.ControlLength - SizeOf(Header));

    if DataStream^.Status <> stOk then
     begin
      rc:='unable to read body of frame at ' + HexL(Index.Offset) + ', rc=' + Long2Str(DataStream^.Status);

      Dispose(Text, Done);

      DataStream^.Reset;

      jt.msg.Destroy(Message);

      Exit;
     end;

    repeat
     Text^.Seek(Text^.GetSize - 1);

     Text^.Read(Ch, SizeOf(Ch));

     if Ch = #0 then
      begin
       Text^.Seek(Text^.GetSize - 1);

       Text^.Truncate;
      end;
    until Ch <> #0;

    Dispose(Text, Done);
   end
  else
   begin
    Message:=jt.msg.Create;

    jt.msg.SetGlobal(Message, Index.UID);

    ControlInformation:=New(PRamStream, Init);

    SquishToMessage(Header, ControlInformation, Message);
   end;

  Load:=True;
 end;

function TSquishMessageBase.Store(const No: Longint; const Message: Pointer; var rc: String; const Flags: Longint): Boolean;
 var
  Index: TSquishMessageIndex;
  Frame: TSquishMessageFrame;
  Header: TSquishMessageHeader;
  Anchor, RequiredFrameLength, PrevFrame, NextFrame: Longint;
  ControlInformation: PRamStream;
  FrameMode: (fmThis, fmFree, fmNew);
  Text: PStream;
 function FindFramePos: Boolean;
  function Link: Boolean;
   begin
    Frame.PrevFrame:=PrevFrame;
    Frame.NextFrame:=NextFrame;

    if LinkFrameIntoChain(Frame, Index.Offset, BaseHeader.FirstMessageFrame, BaseHeader.LastMessageFrame, rc) then
     begin
      DataStream^.Seek(Index.Offset);

      Link:=True;
     end
    else
     Link:=False;
   end;
  begin
   (*
    read THIS frame

    if THIS frame is NOT enough then
     {
      unlink THIS frame from messages chain

      link THIS frame to end of free chain

      update THIS frame
     }
    else
     job done (using THIS frame, FrameMode = fmThis)

    searching for free frame

    if FOUND enough free frame then
     {
      unlink FOUND free frame from free chain

      link FOUND free frame INTO messages chain instead of THIS frame

      job done (using FOUND free frame, FrameMode = fmFree)
     }

    creating new frame

    allocating space

    link NEW frame INTO messages chain instead of THIS frame

    job done (using NEW frame, FrameMode = fmNew)
   *)

   FindFramePos:=False;

   { check this frame for enough space and unlink if any }

   DataStream^.Seek(Index.Offset);

   DataStream^.Read(Frame, SizeOf(Frame));

   if DataStream^.Status <> stOk then
    begin
     rc:='unable to read frame at ' + HexL(Index.Offset) + ', rc=' + Long2Str(DataStream^.Status);

     DataStream^.Reset;

     Exit;
    end;

   if Frame.id <> squishFrameID then
    begin
     rc:='wrong id of frame at ' + HexL(Index.Offset);

     Exit;
    end;

   if Frame.FrameType <> squishFrameMessage then
    begin
     rc:='wrong type of frame at ' + HexL(Index.Offset);

     Exit;
    end;

   { save previous and next frame pointers }

   PrevFrame:=Frame.PrevFrame;
   NextFrame:=Frame.NextFrame;

   if Frame.FrameLength < RequiredFrameLength then
    if UnlinkFrameFromChain(Frame, Index.Offset, BaseHeader.FirstMessageFrame, BaseHeader.LastMessageFrame, rc) and
       LinkFrameToEndOfChain(Frame, Index.Offset, BaseHeader.FirstFreeFrame, BaseHeader.LastFreeFrame, rc) then
     begin
      Frame.FrameType:=squishFrameFree;

      DataStream^.Seek(Index.Offset);

      DataStream^.Write(Frame, SizeOf(Frame));

      if DataStream^.Status <> stOk then
       begin
        rc:='unable to write frame at ' + HexL(Index.Offset) + ', rc=' + Long2Str(DataStream^.Status);

        DataStream^.Reset;

        Exit;
       end;
     end
    else
     Exit
   else
    begin
     FrameMode:=fmThis;

     FindFramePos:=True;

     Exit;
    end;

   if not SearchForFreeFrame(RequiredFrameLength, Frame, Index.Offset, rc) then
    Exit;

   if Index.Offset <> 0 then { found a free frame }
    if UnlinkFrameFromChain(Frame, Index.Offset, BaseHeader.FirstFreeFrame, BaseHeader.LastFreeFrame, rc) then
     begin
      if not Link then
       Exit;

      FrameMode:=fmFree;

      FindFramePos:=True;

      Exit;
     end
    else
     Exit;

   if BaseHeader.EndOfFile > DataStream^.GetSize then
    BaseHeader.EndOfFile:=DataStream^.GetSize;

   if BaseHeader.EndOfFile < BaseHeader.Size then
    BaseHeader.EndOfFile:=BaseHeader.Size;

   Index.Offset:=BaseHeader.EndOfFile;

   DataStream^.Seek(Index.Offset + SizeOf(TSquishMessageFrame) + RequiredFrameLength);

   DataStream^.Truncate;

   if DataStream^.Status <> stOk then
    begin
     rc:='unable to allocate space for new frame, rc=' + Long2Str(DataStream^.Status);

     DataStream^.Reset;
    end;

   Inc(BaseHeader.EndOfFile, SizeOf(TSquishMessageFrame) + RequiredFrameLength);

   Frame.FrameLength:=RequiredFrameLength;

   if not Link then
    Exit;

   FrameMode:=fmNew;

   FindFramePos:=True;
  end;
 begin
  if No < 0 then
   begin
    Store:=Add(Message, rc, Flags);

    Exit;
   end;

  Store:=False;

  IndexStream^.Seek(No * SizeOf(Index));

  IndexStream^.Read(Index, SizeOf(Index));

  if IndexStream^.Status <> stOk then
   begin
    rc:='unable to read index entry #' + Long2Str(No) + ' at ' + HexL(No * SizeOf(Index)) +
     ', rc=' + Long2Str(IndexStream^.Status);

    IndexStream^.Reset;

    Exit;
   end;

  if Lock then
   begin
    rc:='';

    if ReadBaseHeader(rc) then
     begin
      Text:=New(PStreamThunk, Init(jt.msg.GetText(Message), jt.StreamLinker, False));

      MessageToSquish(Message, Header, Anchor, ControlInformation, False);

      RequiredFrameLength:=SizeOf(Header) + ControlInformation^.GetSize + Text^.GetSize - Anchor;

      if FindFramePos then
       begin
        Frame.id:=squishFrameID;
        Frame.MsgLength:=SizeOf(Header) + ControlInformation^.GetSize + Text^.GetSize - Anchor;
        Frame.ControlLength:=ControlInformation^.GetSize;
        Frame.FrameType:=squishFrameMessage;

        DataStream^.Seek(Index.Offset);

        DataStream^.Write(Frame, SizeOf(Frame));

        DataStream^.Write(Header, SizeOf(Header));

        if (FrameMode <> fmThis) or (Flags and mfNoText = 0) then
         begin
          ControlInformation^.Seek(0);

          DataStream^.CopyFrom(ControlInformation^, ControlInformation^.GetSize);

          Text^.Seek(Anchor);

          DataStream^.CopyFrom(Text^, Text^.GetSize - Anchor);

          if WriteBaseHeader(rc) then
           begin
            if jt.msg.GetToName(Message) = nil then
             Index.Hash:=0
            else
             Index.Hash:=MakeSquishHash(jt.msg.GetToName(Message)^);

            IndexStream^.Seek(No * SizeOf(Index));

            IndexStream^.Write(Index, SizeOf(Index));

            if IndexStream^.Status <> stOk then
             begin
              rc:='unable to update index entry #' + Long2Str(No) + ' at ' + HexL(No * SizeOf(Index)) +
               ', rc=' + Long2Str(IndexStream^.Status);

              IndexStream^.Reset;
             end;
           end;
         end;
       end;

      Dispose(ControlInformation, Done);

      Dispose(Text, Done);
     end;

    Unlock;

    Store:=rc = '';
   end
  else
   rc:='unable to lock base';

  if FlushAfterStore then
   begin
    DataStream^.Flush;

    IndexStream^.Flush;
   end;
 end;

function TSquishMessageBase.Kill(const No: Longint; var rc: String): Boolean;
 function ShiftIndex: Boolean;
  var
   Indexes: array[1..1000] of TSquishMessageIndex;
   K, ToDo, Max, Up: Longint;
  begin
   IndexStream^.Seek(0);

   K:=No + 1;

   Up:=IndexStream^.GetSize div SizeOf(TSquishMessageIndex);

   while K < Up do
    begin
     ToDo:=SizeOf(Indexes);

     Max:=(IndexStream^.GetSize - K * SizeOf(TSquishMessageIndex)) div SizeOf(TSquishMessageIndex) *
      SizeOf(TSquishMessageIndex);

     if ToDo > Max then
      ToDo:=Max;

     IndexStream^.Seek(K * SizeOf(TSquishMessageIndex));

     IndexStream^.Read(Indexes, ToDo);

     if IndexStream^.Status <> stOk then
      begin
       rc:='unable to read indexes from entry #' + Long2Str(K) + ', rc=' + Long2Str(IndexStream^.Status);

       IndexStream^.Reset;

       ShiftIndex:=False;

       Exit;
      end;

     IndexStream^.Seek((K - 1) * SizeOf(TSquishMessageIndex));

     IndexStream^.Write(Indexes, ToDo);

     if IndexStream^.Status <> stOk then
      begin
       rc:='unable to write indexes at entry #' + Long2Str(K - 1) + ', rc=' + Long2Str(IndexStream^.Status);

       IndexStream^.Reset;

       ShiftIndex:=False;

       Exit;
      end;

     Inc(K, ToDo div SizeOf(TSquishMessageIndex));
    end;

   IndexStream^.Truncate;

   if IndexStream^.Status <> stOk then
    begin
     rc:='unable to truncate index, rc=' + Long2Str(IndexStream^.Status);

     IndexStream^.Reset;

     ShiftIndex:=False;
    end
   else
    ShiftIndex:=True;
  end;
 var
  Index: TSquishMessageIndex;
  Frame: TSquishMessageFrame;
 begin
  Kill:=False;

  if Lock then
   begin
    rc:='';

    if ReadBaseHeader(rc) then
     begin
      IndexStream^.Seek(No * SizeOf(Index));

      IndexStream^.Read(Index, SizeOf(Index));

      if IndexStream^.Status <> stOk then
       begin
        rc:='unable to read index entry #' + Long2Str(No) + ' at ' + HexL(No * SizeOf(Index)) +
         ', rc=' + Long2Str(IndexStream^.Status);

        IndexStream^.Reset;
       end
      else
       begin
        DataStream^.Seek(Index.Offset);

        DataStream^.Read(Frame, SizeOf(Frame));

        if DataStream^.Status <> stOk then
         begin
          rc:='unable to load frame header at ' + HexL(Index.Offset) + ', rc=' + Long2Str(DataStream^.Status);

          DataStream^.Reset;
         end
        else
         if Frame.id <> squishFrameID then
          rc:='wrong frame id at ' + HexL(Index.Offset)
         else
          if Frame.FrameType = squishFrameUpdate then
           rc:='frame ' + HexL(Index.Offset) + ' is being processed by another process'
          else
           if Frame.FrameType <> squishFrameMessage then
            rc:='frame ' + HexL(Index.Offset) + ' type is unknown'
           else
            if ShiftIndex and
               UnlinkFrameFromChain(Frame, Index.Offset, BaseHeader.FirstMessageFrame, BaseHeader.LastMessageFrame, rc) and
               LinkFrameToEndOfChain(Frame, Index.Offset, BaseHeader.FirstFreeFrame, BaseHeader.LastFreeFrame, rc) then
             begin
              Frame.FrameType:=squishFrameFree;

              DataStream^.Seek(Index.Offset);

              DataStream^.Write(Frame, SizeOf(Frame));

              Dec(BaseHeader.NumMsg);
              Dec(BaseHeader.HighMsg);

              WriteBaseHeader(rc);
             end;
       end;
     end;

    Unlock;

    Kill:=rc = '';
   end
  else
   rc:='unable to lock base';
 end;

function TSquishMessageBase.NoToGlobal(const No: Longint): Longint;
 var
  Index: TSquishMessageIndex;
 begin
  IndexStream^.Seek(No * SizeOf(Index));

  IndexStream^.Read(Index, SizeOf(Index));

  if IndexStream^.Status <> stOk then
   begin
    NoToGlobal:=-1;

    IndexStream^.Reset;
   end
  else
   NoToGlobal:=Index.UID;
 end;

function TSquishMessageBase.GlobalToNo(const Global: Longint): Longint;
 var
  Countdown, Countup: Longint;
  Index: TSquishMessageIndex;
 begin
  Countdown:=IndexStream^.GetSize div SizeOf(Index);
  Countup:=0;

  IndexStream^.Seek(0);

  while Countdown <> 0 do
   begin
    IndexStream^.Read(Index, SizeOf(Index));

    if IndexStream^.Status <> stOk then
     begin
      GlobalToNo:=-1;

      IndexStream^.Reset;

      Exit;
     end;

    if Index.UID = Global then
     begin
      GlobalToNo:=Countup;

      Exit;
     end;

    Dec(Countdown);

    Inc(Countup);
   end;

  GlobalToNo:=-1;
 end;

function TSquishMessageBase.LockBase: Boolean;
 begin
  LockBase:=DataStream^.Lock(0, 1) = 0;
 end;

procedure TSquishMessageBase.UnlockBase;
 begin
  DataStream^.Unlock(0, 1);
 end;

{ private methods }

procedure TSquishMessageBase.MessageToSquish(const Message: Pointer; var Header: TSquishMessageHeader;
  var ControlInformationSize: Longint; var ControlInformation: PRamStream; const Adding: Boolean);
 procedure PStringToPChar(const Source: PString; var Target: array of Char);
  begin
   if Source = nil then
    Target[Low(Target)]:=#0
   else
    StrPCopy(@Target, Copy(Source^, 1, High(Target) - Low(Target)));
  end;
 var
  Parser: Pointer;
  Anchor: Longint;
  Line: PChar;
  DT: TDateTime;
  Text: PStream;
 begin
  { prepare header }

  FillChar(Header, SizeOf(Header), 0);

  Header.Attributes:=JTMHLToSquishAttributes(jt.msg.GetAttributes(Message));

  PStringToPChar(jt.msg.GetFromName(Message), Header.FromName);
  PStringToPChar(jt.msg.GetToName(Message), Header.ToName);
  PStringToPChar(jt.msg.GetSubject(Message), Header.Subject);

  Parser:=jt.msg.CreateParser(Message, Netmail);

  jt.msg.p.GetFromAndToAddress(Parser, Header.FromAddress, Header.ToAddress);

  jt.msg.GetArrived(Message, DT);
  DateTimeToDosDateTime(DT, Header.DateArrived);
  SquishSwap(Header.DateArrived);

  jt.msg.GetWritten(Message, DT);
  DateTimeToDosDateTime(DT, Header.DateWritten);
  SquishSwap(Header.DateWritten);

  StrPCopy(Header.Date, lz(DT.Day) + ' ' +
                        MonthNumberToMonthString(DT.Month) + ' ' +
                        lz(Y2ToDouble(DT.Year)) + '  ' +
                        lz(DT.Hour) + ':' +
                        lz(DT.Min) + ':' +
                        lz(DT.Sec));

  if Adding then
   begin
    Header.UID:=BaseHeader.UID;

    Inc(BaseHeader.UID);
   end
  else
   begin
    Header.UID:=Longint(jt.msg.GetGlobal(Message));

    if Header.UID = -1 then
     begin
      Header.UID:=BaseHeader.UID;

      Inc(BaseHeader.UID);
     end;
   end;

  { prepare control information }

  ControlInformation:=New(PRamStream, Init);

  Text:=New(PStreamThunk, Init(jt.msg.GetText(Message), jt.StreamLinker, False));

  Text^.Seek(0);

  ControlInformationSize:=0;
  Anchor:=0;

  while Anchor <> Text^.GetSize do
   begin
    Line:=jt.msg.p.GetStringPCharEx(Parser);

    if Line[0] <> #1 then
     begin
      ControlInformationSize:=Anchor;

      jt.StrDispose(Line);

      Break;
     end
    else
     begin
      ControlInformation^.Write(Line[0], StrLen(Line));

      jt.StrDispose(Line);
     end;

    Anchor:=Text^.GetPos;
   end;

  Dispose(Text, Done);

  jt.msg.p.Destroy(Parser);
 end;

procedure TSquishMessageBase.SquishToMessage(var Header: TSquishMessageHeader; const ControlInformation: PRamStream;
  const Message: Pointer);
 procedure PCharToString(var Source: array of Char; var Destination: String);
  var
   K: Integer;
  begin
   for K:=High(Source) downto Low(Source) do
    if Source[K] = #0 then
     begin
      Destination:=StrPas(@Source);

      Exit;
     end;

   SetLength(Destination, SizeOf(Source));

   Move(Source, Destination[1], SizeOf(Source));
  end;
 var
  Line, ALine, Divider: PChar;
  Parser: Pointer;
  DT: TDateTime;
  S: String;
 begin
  jt.msg.SetAttributes(Message, SquishToJTMHLAttributes(Header.Attributes));

  PCharToString(Header.FromName, S);
  jt.msg.SetFromName(Message, S);

  PCharToString(Header.ToName, S);
  jt.msg.SetToName(Message, S);

  PCharToString(Header.Subject, S);
  jt.msg.SetSubject(Message, S);

  SquishSwap(Header.DateWritten);
  DosDateTimeToDateTime(Header.DateWritten, DT);
  jt.msg.SetWritten(Message, DT);

  SquishSwap(Header.DateArrived);
  DosDateTimeToDateTime(Header.DateArrived, DT);
  jt.msg.SetArrived(Message, DT);

  { get control information }

  Parser:=jt.msg.p.CreateEx(ControlInformation, False, StreamLinker, False);

  jt.msg.p.SetTextPos(Parser, 0);

  Line:=jt.msg.p.GetStringPCharEx(Parser);

  ALine:=Line;

  jt.msg.p.Destroy(Parser);

  { convert control information }

  Parser:=jt.msg.CreateParser(Message, Netmail);

  while Line[0] = #1 do
   begin
    Divider:=StrScan(@Line[1], #1);

    if Divider = nil then
     begin
      jt.msg.p.PutStringPChar(Parser, Line);

      Break;
     end;

    Divider[0]:=#0;

    jt.msg.p.PutStringPChar(Parser, Line);

    Divider[0]:=#1;

    Line:=Divider;
   end;

  jt.msg.p.Destroy(Parser);

  jt.StrDispose(ALine);

  Dispose(ControlInformation, Done);
 end;


function TSquishMessageBase.JTMHLToSquishAttributes(const JTMHLAttributes: Longint): Longint;
 var
  SquishAttributes: Longint;
  K: Byte;
 begin
  SquishAttributes:=squishaUID;

  for K:=Low(AttributesMap) to High(AttributesMap) do
   if JTMHLAttributes and AttributesMap[K, amtJTMHL] <> 0 then
    SquishAttributes:=SquishAttributes or AttributesMap[K, amtSquish];

  JTMHLToSquishAttributes:=SquishAttributes;
 end;

function TSquishMessageBase.SquishToJTMHLAttributes(const SquishAttributes: Longint): Longint;
 var
  JTMHLAttributes: Longint;
  K: Byte;
 begin
  JTMHLAttributes:=maNone;

  for K:=Low(AttributesMap) to High(AttributesMap) do
   if SquishAttributes and AttributesMap[K, amtSquish] <> 0 then
    JTMHLAttributes:=JTMHLAttributes or AttributesMap[K, amtJTMHL];

  SquishToJTMHLAttributes:=JTMHLAttributes;
 end;


function TSquishMessageBase.ReadBaseHeader(var rc: String): Boolean;
 begin
  DataStream^.Seek(0);

  DataStream^.Read(BaseHeader, SizeOf(BaseHeader));

  if DataStream^.Status = stOk then
   ReadBaseHeader:=True
  else
   begin
    ReadBaseHeader:=False;

    rc:='unable to read base header, rc=' + Long2Str(DataStream^.Status);

    DataStream^.Reset;
   end;
 end;

function TSquishMessageBase.WriteBaseHeader(var rc: String): Boolean;
 begin
  DataStream^.Seek(0);

  DataStream^.Write(BaseHeader, SizeOf(BaseHeader));

  if DataStream^.Status = stOk then
   WriteBaseHeader:=True
  else
   begin
    WriteBaseHeader:=False;

    rc:='unable to write base header, rc=' + Long2Str(DataStream^.Status);

    DataStream^.Reset;
   end;
 end;


function TSquishMessageBase.SearchForFreeFrame(const FrameLength: Longint; var Frame: TSquishMessageFrame;
  var FramePos: Longint; var rc: String): Boolean;
 begin
  SearchForFreeFrame:=True;

  FramePos:=BaseHeader.FirstFreeFrame;

  while FramePos <> 0 do
   begin
    DataStream^.Seek(FramePos);

    DataStream^.Read(Frame, SizeOf(Frame));

    if DataStream^.Status <> stOk then
     begin
      DataStream^.Reset;

      FramePos:=0;

      Exit;
     end;

    if Frame.id <> squishFrameID then
     begin
      SearchForFreeFrame:=False;

      rc:='unknown frame id at ' + HexL(FramePos);

      Exit;
     end;

    if Frame.FrameType <> squishFrameFree then
     begin
      SearchForFreeFrame:=False;

      rc:='non-free frame in free chain at ' + HexL(FramePos);

      Exit;
     end;

    if Frame.FrameLength >= FrameLength then
     Exit;

    FramePos:=Frame.NextFrame;
   end;
 end;

procedure TSquishMessageBase.SquishSwap(var Number: Longint);
 begin
  Number:=(Number shr 16) + ((Number and $FFFF) shl 16);
 end;


function TSquishMessageBase.UnlinkFrameFromChain(var Frame: TSquishMessageFrame;
  var FramePos, FirstFrame, LastFrame: Longint; var rc: String): Boolean;
 function Validate(const Action: String; const Offset: Longint): Boolean;
  begin
   if DataStream^.Status = stOk then
    Validate:=True
   else
    begin
     rc:='unable to ' + Action + ' frame ' + HexL(Offset) + ', rc=' + Long2Str(DataStream^.Status);

     DataStream^.Reset;

     Validate:=False;

     UnlinkFrameFromChain:=False;
    end;
  end;
 var
  FrameTemp: TSquishMessageFrame;
 begin
  if Frame.PrevFrame <> 0 then
   begin
    { read previous frame header }

    DataStream^.Seek(Frame.PrevFrame);

    DataStream^.Read(FrameTemp, SizeOf(FrameTemp));

    if not Validate('read', Frame.PrevFrame) then Exit;

    { link previous to next }

    FrameTemp.NextFrame:=Frame.NextFrame;

    { write previous frame header }

    DataStream^.Seek(Frame.PrevFrame);

    DataStream^.Write(FrameTemp, SizeOf(FrameTemp));

    if not Validate('write', Frame.PrevFrame) then Exit;

    { check for first message in the chain }

    if FirstFrame = FramePos then
     FirstFrame:=Frame.NextFrame;
   end
  else
   FirstFrame:=Frame.NextFrame; { make next frame first }

  if Frame.NextFrame <> 0 then
   begin
    { read next frame header }

    DataStream^.Seek(Frame.NextFrame);

    DataStream^.Read(FrameTemp, SizeOf(FrameTemp));

    if not Validate('read', Frame.NextFrame) then Exit;

    { link next to previous }

    FrameTemp.PrevFrame:=Frame.PrevFrame;

    { write next frame header }

    DataStream^.Seek(Frame.NextFrame);

    DataStream^.Write(FrameTemp, SizeOf(FrameTemp));

    if not Validate('write', Frame.NextFrame) then Exit;

    { check for last message in the chain }

    if LastFrame = FramePos then
     LastFrame:=Frame.PrevFrame;
   end
  else
   LastFrame:=Frame.PrevFrame; { make prev frame last }

  UnlinkFrameFromChain:=True;
 end;

function TSquishMessageBase.LinkFrameToEndOfChain(var Frame: TSquishMessageFrame;
  var FramePos, FirstFrame, LastFrame: Longint; var rc: String): Boolean;
 function Validate(const Action: String; const Offset: Longint): Boolean;
  begin
   if DataStream^.Status = stOk then
    Validate:=True
   else
    begin
     rc:='unable to ' + Action + ' frame ' + HexL(Offset) + ', rc=' + Long2Str(DataStream^.Status);

     DataStream^.Reset;

     Validate:=False;

     LinkFrameToEndOfChain:=False;
    end;
  end;
 var
  FrameTemp: TSquishMessageFrame;
 begin
  if LastFrame = 0 then
   begin
    FirstFrame:=FramePos;
    LastFrame:=FramePos;

    Frame.PrevFrame:=0;
    Frame.NextFrame:=0;
   end
  else
   begin
    DataStream^.Seek(LastFrame);

    DataStream^.Read(FrameTemp, SizeOf(FrameTemp));

    if not Validate('read', LastFrame) then Exit;

    FrameTemp.NextFrame:=FramePos;

    DataStream^.Seek(LastFrame);

    DataStream^.Write(FrameTemp, SizeOf(FrameTemp));

    if not Validate('write', LastFrame) then Exit;

    Frame.PrevFrame:=LastFrame;
    Frame.NextFrame:=0;

    LastFrame:=FramePos;
   end;

  LinkFrameToEndOfChain:=True;
 end;

function TSquishMessageBase.LinkFrameIntoChain(var Frame: TSquishMessageFrame; var FramePos, FirstFrame, LastFrame: Longint;
  var rc: String): Boolean;
 function Validate(const Action: String; const Offset: Longint): Boolean;
  begin
   if DataStream^.Status = stOk then
    Validate:=True
   else
    begin
     rc:='unable to ' + Action + ' frame ' + HexL(Offset) + ', rc=' + Long2Str(DataStream^.Status);

     DataStream^.Reset;

     Validate:=False;

     LinkFrameIntoChain:=False;
    end;
  end;
 var
  FrameTemp: TSquishMessageFrame;
 begin
  if Frame.PrevFrame <> 0 then
   begin
    { read previous frame header }

    DataStream^.Seek(Frame.PrevFrame);

    DataStream^.Read(FrameTemp, SizeOf(FrameTemp));

    if not Validate('read', Frame.PrevFrame) then Exit;

    { link previous to next }

    FrameTemp.NextFrame:=FramePos;

    { write previous frame header }

    DataStream^.Seek(Frame.PrevFrame);

    DataStream^.Write(FrameTemp, SizeOf(FrameTemp));

    if not Validate('write', Frame.PrevFrame) then Exit;
   end
  else
   FirstFrame:=FramePos; { make next frame first }

  if Frame.NextFrame <> 0 then
   begin
    { read next frame header }

    DataStream^.Seek(Frame.NextFrame);

    DataStream^.Read(FrameTemp, SizeOf(FrameTemp));

    if not Validate('read', Frame.NextFrame) then Exit;

    { link next to previous }

    FrameTemp.PrevFrame:=FramePos;

    { write next frame header }

    DataStream^.Seek(Frame.NextFrame);

    DataStream^.Write(FrameTemp, SizeOf(FrameTemp));

    if not Validate('write', Frame.NextFrame) then Exit;
   end
  else
   LastFrame:=FramePos; { make prev frame last }

  LinkFrameIntoChain:=True;
 end;

{ public methods }

procedure TSquishMessageBase.LoadLinkInfo(var Block: TLoadLinkMessageBaseBlock);
 var
  Errors: Longint;
 procedure Error(const Message: String);
  begin
   jt.LogWrite('?', Message + ' (' + Path^ + ')');

   Inc(Errors);
  end;
 var
  REPLYCRC, MSGIDCRC: Longint;
 procedure Check(const Line: PChar);
  begin
   if StrLComp(Line, #1'MSGID: ', 8) = 0 then
    MSGIDCRC:=PCharCRC32(@Line[8], -1)
   else
    if StrLComp(Line, #1'REPLY: ', 8) = 0 then
     REPLYCRC:=PCharCRC32(@Line[8], -1);
  end;
 var
  No, BufferSize: Longint;
  SearchIndex: xInteger;
  Index: TSquishMessageIndex;
  Frame: TSquishMessageFrame;
  Header: TSquishMessageHeader;
  Buffer, Line, Divider: PChar;
  Additional: PAdditionalBlock;
  Unused: PLinkerCacheItems;
  Cache: PLinkerCache;
  Item: PLinkerCacheItem;
 procedure AddUnused(const Item: PLinkerCacheItem); far;
  begin
   if not Item^.Used then
    Unused^.Insert(Item);
  end;
 procedure RemoveFromCache(const Item: PLinkerCacheItem); far;
  begin
   Cache^.Free(Item);
  end;
 begin
  Block.Supported:=True;
  Block.Did:=True;
  Block.Dispose:=AdditionalDisposer;
  Block.Flags:=llmbbfBranchLimit;
  Block.BranchLimit:=9;

  Cache:=New(PLinkerCache, Init(Block.Area));

  Errors:=0;

  IndexStream^.Seek(0);

  for No:=0 to Count - 1 do
   begin
    IndexStream^.Read(Index, SizeOf(Index));

    if IndexStream^.Status = stOk then
     begin
      if Cache^.Search(Pointer(Index.UID), SearchIndex) then
       begin
        Item:=Cache^.At(SearchIndex);

        Item^.Used:=True;

        New(Additional);

        Additional^:=Item^.Additional;

        if not Block.Add(Block.Param, No, Index.UID, Item^.MSGIDCRC, Item^.REPLYCRC, Additional) then
         begin
          Block.Did:=False;

          Break;
         end;
       end
      else
       begin
        DataStream^.Seek(Index.Offset);

        DataStream^.Read(Frame, SizeOf(Frame));

        if DataStream^.Status = stOk then
         if Frame.id <> squishFrameID then Error('wrong frame id at ' + HexL(Index.Offset)) else
         if Frame.FrameType = squishFrameUpdate then Error('frame ' + HexL(Index.Offset) +
           ' is being processed by another process') else
         if Frame.FrameType <> squishFrameMessage then Error('frame ' + HexL(Index.Offset) + ' type is unknown') else
          begin
           BufferSize:=Frame.ControlLength;

           MSGIDCRC:=-1;
           REPLYCRC:=-1;

           if BufferSize > 3 then
            begin
             {$IFDEF DPMI}
             if BufferSize > 65519 then
              BufferSize:=65519;
             {$ENDIF}

             GetMem(Buffer, BufferSize + 1);

             DataStream^.Read(Header, SizeOf(Header));

             DataStream^.Read(Buffer[0], BufferSize);

             Buffer[BufferSize]:=#0;

             Line:=Buffer;

             while (Line[0] = #1) and ((MSGIDCRC = -1) or (REPLYCRC = -1)) do
              begin
               Divider:=StrScan(@Line[1], #1);

               if Divider = nil then
                begin
                 Check(Line);

                 Break;
                end;

               Divider[0]:=#0;

               Check(Line);

               Divider[0]:=#1;

               Line:=Divider;
              end;

             FreeMem(Buffer, BufferSize + 1);

             New(Additional);

             Additional^.ReplyTo:=Header.ReplyTo;
             Additional^.ReplyInfo:=Header.ReplyInfo;

             New(Item);

             Item^.No:=No;
             Item^.Global:=Index.UID;
             Item^.MSGIDCRC:=MSGIDCRC;
             Item^.REPLYCRC:=REPLYCRC;
             Item^.Additional:=Additional^;
             Item^.Used:=True;

             Cache^.AtInsert(SearchIndex, Item);

             Cache^.Changed:=True;

             if not Block.Add(Block.Param, No, Index.UID, MSGIDCRC, REPLYCRC, Additional) then
              begin
               Block.Did:=False;

               Break;
              end;

             Errors:=0;
            end;
          end
        else
         begin
          Error('unable to load frame header at ' + HexL(Index.Offset) + ', rc=' + Long2Str(DataStream^.Status));

          DataStream^.Reset;
         end;
       end;
     end
    else
     begin
      Error('unable to read index entry #' + Long2Str(No) + ' at ' + HexL(No * SizeOf(Index)) +
        ', rc=' + Long2Str(IndexStream^.Status));

      IndexStream^.Reset;
     end;

    if Errors > 10 then
     begin
      Block.Did:=False;

      Break;
     end;
   end;

  if Block.Did then
   begin
    Unused:=New(PLinkerCacheItems, Init(Cache^.Count, $1000));

    Cache^.ForEach(@AddUnused);

    if Unused^.Count <> 0 then
     begin
      Unused^.ForEach(@RemoveFromCache);

      Unused^.DeleteAll;

      Cache^.Changed:=True;
     end;

    Dispose(Unused, Done);
   end;

  Dispose(Cache, Done);
 end;

procedure TSquishMessageBase.StoreLinkInfo(var Block: TStoreLinkMessageBaseBlock);
 var
  K, L, Errors: Longint;
  Message, M: PLinkerMessage;
  NewAdditional: TAdditionalBlock;
  Update: Boolean;
  Index: TSquishMessageIndex;
  Frame: TSquishMessageFrame;
 procedure Error(const Message: String);
  begin
   jt.LogWrite('?', Message + ' (' + Path^ + ')');

   Inc(Errors);
  end;
 begin
  Block.Supported:=True;
  Block.Did:=True;

  Errors:=0;

  for K:=0 to Block.Count - 1 do
   begin
    if not Block.Callback(Block.Param) then
     begin
      Block.Did:=False;

      Break;
     end;

    Message:=Block.Items^[K];

    if Message^.Owner^.Global = -1 then
     NewAdditional.ReplyTo:=0
    else
     NewAdditional.ReplyTo:=Message^.Owner^.Global;

    Update:=NewAdditional.ReplyTo <> PAdditionalBlock(Message^.Additional)^.ReplyTo;

    FillChar(NewAdditional.ReplyInfo, SizeOf(NewAdditional.ReplyInfo), 0);

    M:=Message^.ChildFirst;

    for L:=1 to 9 do
     if M = nil then
      begin
       NewAdditional.ReplyInfo[L]:=0;

       Update:=Update or (0 <> PAdditionalBlock(Message^.Additional)^.ReplyInfo[L]);
      end
     else
      begin
       NewAdditional.ReplyInfo[L]:=M^.Global;

       Update:=Update or (M^.Global <> PAdditionalBlock(Message^.Additional)^.ReplyInfo[L]);

       M:=M^.Next;
      end;

    if Update then
     begin
      IndexStream^.Seek(Message^.No * SizeOf(TSquishMessageIndex));

      IndexStream^.Read(Index, SizeOf(Index));

      if IndexStream^.Status = stOk then
       begin
        DataStream^.Seek(Index.Offset);

        DataStream^.Read(Frame, SizeOf(Frame));

        if DataStream^.Status = stOk then
         if Frame.id <> squishFrameID then Error('wrong frame id at ' + HexL(Index.Offset)) else
         if Frame.FrameType = squishFrameUpdate then Error('frame ' + HexL(Index.Offset) +
           ' is being processed by another process') else
         if Frame.FrameType <> squishFrameMessage then Error('frame ' + HexL(Index.Offset) + ' type is unknown') else
          begin
           DataStream^.Seek(Index.Offset + SizeOf(Frame) + 174);

           DataStream^.Write(NewAdditional, SizeOf(NewAdditional));
{
           jt.LogWrite('~', Long2Str(Message^.Global) + ': ' + Long2Str(NewAdditional.ReplyTo) + '; ' +
                                                               Long2Str(NewAdditional.ReplyInfo[1]) + ',' +
                                                               Long2Str(NewAdditional.ReplyInfo[2]) + ',' +
                                                               Long2Str(NewAdditional.ReplyInfo[3]) + ',' +
                                                               Long2Str(NewAdditional.ReplyInfo[4]) + ',' +
                                                               Long2Str(NewAdditional.ReplyInfo[5]) + ',' +
                                                               Long2Str(NewAdditional.ReplyInfo[6]) + ',' +
                                                               Long2Str(NewAdditional.ReplyInfo[7]) + ',' +
                                                               Long2Str(NewAdditional.ReplyInfo[8]) + ',' +
                                                               Long2Str(NewAdditional.ReplyInfo[9]));
}
           if DataStream^.Status <> stOk then
            begin
             Error('unable to set replyinfo at ' + HexL(Index.Offset) + ', rc=' + Long2Str(DataStream^.Status));

             DataStream^.Reset;
            end
           else
            Errors:=0;
          end
        else
         begin
          Error('unable to load frame header at ' + HexL(Index.Offset) + ', rc=' + Long2Str(DataStream^.Status));

          DataStream^.Reset;
         end;
       end
      else
       begin
        Error('unable to read index entry #' + Long2Str(Message^.No) + ' at ' + HexL(Message^.No * SizeOf(Index)) +
         ', rc=' + Long2Str(IndexStream^.Status));

        IndexStream^.Reset;
       end;
     end;

    if Errors > 10 then
     begin
      Block.Did:=False;

      Break;
     end;
   end;
 end;

function TSquishMessageBase.ExtendedOpened(const Operation: Longint; var Data): Boolean;
 var
  HighWater: Longint absolute Data;
  Attributes: Longint absolute Data;
  LastRead: Longint absolute Data;
  rc: String;
 begin
  if Operation = eooGetHighWater then
   begin
    HighWater:=BaseHeader.HighWater;

    ExtendedOpened:=True;
   end else
  if Operation = eooSetHighWater then
   if Lock then
    begin
     if ReadBaseHeader(rc) then
      begin
       BaseHeader.Highwater:=HighWater;

       ExtendedOpened:=WriteBaseHeader(rc);
      end
     else
      ExtendedOpened:=False;

     Unlock;
    end
   else
    ExtendedOpened:=False else
  if Operation = eooNotScannableAttributes then ExtendedOpened:=Attributes and maScanned <> 0 else
  if Operation = eooSetScannedAttributes then
   begin
    Attributes:=Attributes or maScanned;

    ExtendedOpened:=True;
   end else
  if Operation = eooGetLastRead then ExtendedOpened:=GetLastRead(LastRead) else
  if Operation = eooSetLastRead then ExtendedOpened:=SetLastRead(LastRead) else
   ExtendedOpened:=inherited ExtendedOpened(Operation, Data);
 end;

function TSquishMessageBase.GetLastRead(var LastRead: Longint): Boolean;
 var
  Stream: PStream;
 begin
  Stream:=New(PDosStreamEx, Init(Path^ + 'sql', stOpenRead));

  if Stream^.Status = stOk then
   begin
    Stream^.Seek(DefaultLastReadUserNumber * SizeOf(LastRead));

    Stream^.Read(LastRead, SizeOf(LastRead));

    GetLastRead:=Stream^.Status = stOk;
   end
  else
   GetLastRead:=False;

  Dispose(Stream, Done);
 end;

function TSquishMessageBase.SetLastRead(var LastRead: Longint): Boolean;
 var
  Stream: PStream;
 begin
  Stream:=New(PDosStreamEx, Init(Path^ + 'sql', stOpen or stDenyAll));

  if Stream^.Status <> stOk then
   begin
    Dispose(Stream, Done);

    Stream:=New(PDosStreamEx, Init(Path^ + 'sql', stCreate or stDenyAll));
   end;

  if Stream^.Status = stOk then
   begin
    Stream^.Seek(DefaultLastReadUserNumber * SizeOf(LastRead));

    Stream^.Write(LastRead, SizeOf(LastRead));

    SetLastRead:=Stream^.Status = stOk;
   end
  else
   SetLastRead:=False;

  Dispose(Stream, Done);
 end;

end.