unit mbsqPack;

{
 28 frameheader (id, next, prev, framelen, msglen, ctrllen, frametype, rsvd)
238 msgheader (attr, from, to, subj, orig, dest, writ, arrvd, etc)
    control information (frameheader.ctrleln)
    message text (frameheader.msglen - 238 - frameheader.ctrlen)
}

interface
uses
     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     Objects,
     Strings,

     RStream,

     mbsqCmn,

     jtMHLcmn,
     jtMHLstm,

     Wizard;

procedure PackSquishBase(Path: String; var Block: TPackMessageBaseBlock);

implementation
uses
     mbSquish;

type
 PSquishIndex = ^TSquishIndex;
 TSquishIndex = object(TSortedCollection)
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  function KeyOf(Item: Pointer): Pointer; virtual;
  procedure FreeItem(Item: Pointer); virtual;
 end;

const
 Debugger               : TDebugger     = (Enabled: False; Prefix: 'mbsqPack');

{ TSquishIndex }

function TSquishIndex.Compare(Key1, Key2: Pointer): xInteger;
 begin
  Compare:=CompareLongint(Longint(Key1), Longint(Key2));

{
  if Longint(Key1) < Longint(Key2) then Compare:=-1 else
  if Longint(Key1) > Longint(Key2) then Compare:=1 else
   Compare:=0;
}
 end;

function TSquishIndex.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=Pointer(PSquishMessageIndex(Item)^.UID);
 end;

procedure TSquishIndex.FreeItem(Item: Pointer);
 begin
  Dispose(PSquishMessageIndex(Item));
 end;

{ PackSquishBase }

procedure PackSquishBase(Path: String; var Block: TPackMessageBaseBlock);
 var
  SourceDataStream, SourceIndexStream,
  TargetDataStream, TargetIndexStream: PDosStreamEx;
 function CheckStream(const Stream: PStream; const Action, Extension: String): Boolean;
  begin
   if Stream^.Status = stOk then
    CheckStream:=True
   else
    begin
     jt.LogWrite('?', 'Failed to ' + Action + ' ' + Path + Extension + ', rc=' + Long2Str(Stream^.Status) +
      ' (while packing ' + jt.area.GetName(Block.Area)^ + ')');

     CheckStream:=False;
    end;
  end;
 function CheckStreamStatus(const Stream: PStream; const Operation, Extension: String): Boolean;
  begin
   if Stream^.Status = stOk then
    CheckStreamStatus:=True
   else
    begin
     CheckStreamStatus:=False;

     jt.LogWrite('?', 'Failed to ' + Operation + ' of ' + Path + Extension + ': rc=' + Long2Str(Stream^.Status));
    end;
  end;
 procedure FailedToRename(const Original, Target: String; const rc: Longint);
  begin
   jt.LogWrite('?', 'Failed to rename ' + Path + Original + ' to ' + Path + Target +
    ', rc=' + Long2Str(rc) + ' (while packing ' + jt.area.GetName(Block.Area)^ + ')');

   Block.Did:=False;
  end;
 var
  BaseHeader: TSquishBaseHeader;
  MessageIndex: PSquishMessageIndex;
  MessageFrame: TSquishMessageFrame;
  MessageHeader: TSquishMessageHeader;
  Index: PSquishIndex;
  Pos, TargetPos, PrevPos, rc: Longint;
  Body: PRamStream;
  Ch: Char;
 function WriteIndex(const Index: PSquishMessageIndex): Boolean; far;
  begin
   TargetIndexStream^.Write(Index^, SizeOf(TSquishMessageIndex));

   WriteIndex:=(TargetIndexStream^.Status <> stOk) or Block.Callback(Block.Area, Block.Param, paDoing);
  end;
 begin
  Path:=AddDot(Path);

  Block.Supported:=True;

  if not smbIsBaseExists(Path) then
   begin
    Block.Did:=True;

    Block.BeforePack:=0;
    Block.AfterPack:=0;

    Debugger.Log(Path + ' is not exist!');

    Exit;
   end;

  Block.Did:=False;

  Debugger.Log(jt.area.GetName(Block.Area)^ + ': ' + Path);

  SourceDataStream:=New(PDosStreamEx, Init(Path + 'sqd', stOpen or stDenyAll));
  SourceIndexStream:=New(PDosStreamEx, Init(Path + 'sqi', stOpen or stDenyAll));

  if (SourceDataStream^.Status = stOk) and (SourceIndexStream^.Status = -2) then
   begin
    Debugger.Log('p  , p騪 ᤮: rc=' + Long2Str(SourceIndexStream^.Status));

    Dispose(SourceIndexStream, Done);

    SourceIndexStream:=New(PDosStreamEx, Init(Path + 'sqi', stCreate or stDenyAll));
   end;

  TargetDataStream:=New(PDosStreamEx, Init(Path + 's$d', stCreate or stDenyAll));
  TargetIndexStream:=New(PDosStreamEx, Init(Path + 's$i', stCreate or stDenyAll));

  Body:=New(PRamStream, Init);

  Index:=nil;

  repeat
   if not CheckStream(SourceDataStream, 'open', 'sqd') or
      not CheckStream(SourceIndexStream, 'open', 'sqi') or
      not CheckStream(TargetDataStream, 'create', 's$d') or
      not CheckStream(TargetIndexStream, 'create', 's$i') then Break;

   SourceDataStream^.Read(BaseHeader, SizeOf(BaseHeader));

   if not CheckStreamStatus(SourceDataStream, 'read base header', 'sqd') then
    Break;

   Block.BeforePack:=SourceIndexStream^.GetSize + SourceDataStream^.GetSize;
   Block.AfterPack:=Block.BeforePack;

   if (BaseHeader.FirstFreeFrame = 0) and (BaseHeader.NumMsg = SourceIndexStream^.GetSize div SizeOf(TSquishMessageIndex)) then
    begin
     Debugger.Log('no need to pack (fff=' + HexL(BaseHeader.FirstFreeFrame) + ', bh.nm=' + HexL(BaseHeader.NumMsg) + ', is=' +
      HexL(SourceIndexStream^.GetSize div SizeOf(TSquishMessageIndex)) + ')');

     Break;
    end;

   Index:=New(PSquishIndex, Init(BaseHeader.NumMsg, $100));

   Index^.Duplicates:=True;

   Pos:=BaseHeader.FirstMessageFrame;

   BaseHeader.NumMsg:=0;
   BaseHeader.HighMsg:=0;
   BaseHeader.FirstMessageFrame:=0;
   BaseHeader.LastMessageFrame:=0;
   BaseHeader.FirstFreeFrame:=0;
   BaseHeader.LastFreeFrame:=0;

   TargetDataStream^.Write(BaseHeader, SizeOf(BaseHeader));

   PrevPos:=0;

   while Pos <> 0 do
    begin
     SourceDataStream^.Seek(Pos);

     SourceDataStream^.Read(MessageFrame, SizeOf(MessageFrame));

     if not CheckStreamStatus(SourceDataStream, 'read msg frame at ' + HexL(Pos), 'sqd') then
      begin
       Pos:=-1;

       Break;
      end;

     if MessageFrame.id <> squishFrameID then
      begin
       jt.LogWrite('?', 'Strange frame id got at ' + HexL(Pos) + ' of ' + Path + 'sqd (got ' +
         HexL(MessageFrame.id) + ', expected ' + HexL(squishFrameID) + ')');

       Pos:=-1;

       Break;
      end;

     if MessageFrame.FrameType <> squishFrameMessage then
      begin
       jt.LogWrite('?', 'Strange frame type got at ' + HexL(Pos) + ' of ' + Path + 'sqd (got ' +
         HexW(MessageFrame.FrameType) + ', expected ' + HexW(squishFrameMessage) + ')');

       Pos:=-1;

       Break;
      end;

     SourceDataStream^.Read(MessageHeader, SizeOf(MessageHeader));

     if not CheckStreamStatus(SourceDataStream, 'read msg header at ' + HexL(Pos), 'sqd') then
      begin
       Pos:=-1;

       Break;
      end;
{
     MessageHeader.ReplyTo:=0;

     FillChar(MessageHeader.ReplyInfo, SizeOf(MessageHeader.ReplyInfo), 0);
}
     TargetPos:=TargetDataStream^.GetSize;

     { // prepare new index item }

     Inc(BaseHeader.NumMsg);
     Inc(BaseHeader.HighMsg);

     New(MessageIndex);

     MessageIndex^.Offset:=TargetPos;
     MessageIndex^.UID:=MessageHeader.UID;
     MessageIndex^.Hash:=MakeSquishHash(StrPas(MessageHeader.ToName));

     Index^.Insert(MessageIndex);

     Body^.Seek(0);
     Body^.CopyFrom(SourceDataStream^, MessageFrame.MsgLength - SizeOf(MessageHeader));
     Body^.Truncate;

     Body^.Seek(Body^.GetSize - 1);
     Body^.Read(Ch, SizeOf(Ch));

     if Ch <> #0 then
      begin
       Ch:=#0;

       Body^.Write(Ch, SizeOf(Ch));

       Inc(MessageFrame.MsgLength);
      end;

     MessageFrame.FrameLength:=MessageFrame.MsgLength;

     { // prepare new frame header }

     if BaseHeader.FirstMessageFrame = 0 then
      BaseHeader.FirstMessageFrame:=TargetPos;

     MessageFrame.PrevFrame:=PrevPos;

     PrevPos:=TargetPos;

     Pos:=MessageFrame.NextFrame;

     if Pos = 0 then
      BaseHeader.LastMessageFrame:=TargetPos
     else
      MessageFrame.NextFrame:=TargetPos + SizeOf(MessageFrame) + MessageFrame.MsgLength;

     Body^.Seek(0);

     TargetDataStream^.Write(MessageFrame, SizeOf(MessageFrame));
     TargetDataStream^.Write(MessageHeader, SizeOf(MessageHeader));
     TargetDataStream^.CopyFrom(Body^, Body^.GetSize);

     if not CheckStreamStatus(TargetDataStream, 'add msg', 's$d') then
      begin
       Pos:=-1;

       Break;
      end;

     if Block.Callback(Block.Area, Block.Param, paDoing) then
      begin
       Pos:=-1;

       Break;
      end;
    end;

   if Pos = -1 then
    Break;

   BaseHeader.EndOfFile:=SourceDataStream^.GetSize;

   TargetDataStream^.Seek(0);
   TargetDataStream^.Write(BaseHeader, SizeOf(BaseHeader));

   if not CheckStreamStatus(TargetDataStream, 'write final base header', 's$d') then
    Break;

   Index^.FirstThat(@WriteIndex);

   if not CheckStreamStatus(TargetIndexStream, 'write final index', 's$i') then
    Break;

   Block.BeforePack:=SourceIndexStream^.GetSize + SourceDataStream^.GetSize;
   Block.AfterPack:=TargetIndexStream^.GetSize + TargetDataStream^.GetSize;

   Block.Did:=True;
  until True;

  if Index <> nil then
   Dispose(Index, Done);

  Dispose(TargetIndexStream, Done);
  Dispose(TargetDataStream, Done);

  Dispose(SourceIndexStream, Done);
  Dispose(SourceDataStream, Done);

  Dispose(Body, Done);

  if Block.Did then
   begin
    EraseFile(Path + 's!d');
    EraseFile(Path + 's!i');

    if not RenameFileEx(Path + 'sqd', Path + 's!d', rc) then FailedToRename('sqd', 's!d', rc) else
    if not RenameFileEx(Path + 'sqi', Path + 's!i', rc) then FailedToRename('sqi', 's!i', rc) else
    if not RenameFileEx(Path + 's$d', Path + 'sqd', rc) then FailedToRename('s$d', 'sqd', rc) else
    if not RenameFileEx(Path + 's$i', Path + 'sqi', rc) then FailedToRename('s$i', 'sqi', rc) else
     begin
      EraseFile(Path + 's!d');
      EraseFile(Path + 's!i');
     end;
   end
  else
   begin
    EraseFile(Path + 's$d');
    EraseFile(Path + 's$i');
   end;
 end;

end.
