unit MParser;

interface
uses
     Objects,

     PlugAPI,
     PlugSIDE,

     jtMHLcmn,

     Wizard;

const
 mpsOpeningBase                         = $1000;
 mpsScanningBase                        = $1001;
 mpsClosingBase                         = $1002;

 mpeFailedToOpenBase                    = $1000;
 mpeFailedToLock                        = $1001;
 mpeFailedToKillMessage                 = $1002;
 mpeFailedToLoadMessage                 = $1003;
 mpeFailedToStoreMessage                = $1004;

type
 TMParserToDo = (tdNothing, tdKill, tdStore);

 PMParser = ^TMParser;
 TMParser = object(TObject)
  Area, mb: Pointer;
  AreaName: String;
  Locked: Boolean;
  No: Longint;
  ReplyToName: String;
  ReplyToAddress: TAddress;
  ToDo: TMParserToDo;

  constructor Init(const AArea: Pointer);
  procedure Scan; virtual;
  destructor Done; virtual;
 public
  function Lock: Boolean; virtual;
  procedure Status(const AStatus: Word; const Param: Pointer); virtual;
  procedure Error(const AError: Word; const Param: Pointer); virtual;
 public
  function Applicable(const msg: Pointer): Boolean; virtual;
  procedure Process(const msg: Pointer); virtual;
  procedure ProcessMessage(const msg, msgp: Pointer); virtual;
  procedure ProcessLine(const msg, msgp: Pointer; var Line: String; var Finish: Boolean); virtual;
 public
  function CreateReply(const FromName: String; const ReplyToArea: Pointer): Pointer; virtual;
  procedure CheckValidReplyToName(var ToName: String); virtual;
  procedure AfterCreateReply(const msg: Pointer); virtual;
  procedure PostReply(const msg: Pointer); virtual;
  procedure FixEmptyReply(const msg: Pointer); virtual;
  function IsEmptyReply(const msg: Pointer): Boolean; virtual;
 end;

implementation

{ TMParser }

constructor TMParser.Init(const AArea: Pointer);
 var
  rc: String;
 begin
  inherited Init;

  Area:=AArea;
  AreaName:=jt.area.GetName(Area)^;

  Status(mpsOpeningBase, nil);

  if not jt.area.Lock(Area, mb, rc) then
   begin
    Error(mpeFailedToOpenBase, @rc);

    Fail;
   end;

  No:=0;

  Locked:=False;
 end;

procedure TMParser.Scan;
 var
  rc: String;
  msg: Pointer;
 begin
  Status(mpsScanningBase, nil);

  while No < jt.mb.Count(mb) do
   if jt.mb.Load(mb, No, msg, rc, mfNone) then
    begin
     ToDo:=tdNothing;

     if Applicable(msg) then
      Process(msg);

     case ToDo of
      tdNothing: Inc(No);
      tdKill:
       if not jt.mb.Kill(mb, No, rc) then
        begin
         Error(mpeFailedToKillMessage, @rc);

         Inc(No);
        end;
      tdStore:
       begin
        if not jt.mb.Store(mb, No, msg, rc, mfNone) then
         Error(mpeFailedToStoreMessage, @rc);

        Inc(No);
       end;
     end;

     jt.msg.Destroy(msg);
    end
   else
    begin
     Error(mpeFailedToLoadMessage, @rc);

     Inc(No);
    end;
 end;

destructor TMParser.Done;
 begin
  Status(mpsClosingBase, nil);

  if Locked then
   jt.mb.Unlock(mb);

  jt.area.Unlock(Area, mb);

  inherited Done;
 end;

{ public methods }

function TMParser.Lock: Boolean;
 begin
  if Locked then
   Lock:=True
  else
   begin
    Locked:=jt.mb.Lock(mb);

    if not Locked then
     Error(mpeFailedToLock, nil);

    Lock:=Locked;
   end;
 end;

procedure TMParser.Status(const AStatus: Word; const Param: Pointer);
 begin
  Abstract;
 end;

procedure TMParser.Error(const AError: Word; const Param: Pointer);
 begin
  Abstract;
 end;

{ public methods }

function TMParser.Applicable(const msg: Pointer): Boolean;
 begin
  Abstract;
 end;

procedure TMParser.Process(const msg: Pointer);
 var
  Ok: Boolean;
  msgp: Pointer;
  ReplyTo, ReplyAddr: String;
 begin
  msgp:=jt.msg.CreateParser(msg, jt.mb.Netmail(mb));

  { // get REPLYTO/REPLYADDR kludges and parse them, prepare reply information }

  if jt.msg.p.GetKludge(msgp, #1'RELPYTO:', ReplyTo) and jt.msg.p.GetKludge(msgp, #1'RELPYADDR:', ReplyAddr) then
   begin
    Delete(ReplyTo, 1, 9);
    Delete(ReplyAddr, 1, 11);

    TrimEx(ReplyTo);
    TrimEx(ReplyAddr);

    ReplyToName:=ReplyAddr;

    ClearAddress(ReplyToAddress);

    Ok:=StrToAddress(ReplyTo, ReplyToAddress) and (ReplyToAddress.Zone <> 0) and (ReplyToName <> '');
   end
  else
   Ok:=False;

  if not Ok then
   begin
    ClearAddress(ReplyToAddress);

    jt.msg.p.GetFromAddress(msgp, ReplyToAddress);

    GetPStringEx(jt.msg.GetFromName(msg), ReplyToName);
   end;

  ProcessMessage(msg, msgp);

  jt.msg.DestroyParser(msgp);
 end;

procedure TMParser.ProcessMessage(const msg, msgp: Pointer);
 var
  S: String;
  Finish: Boolean;
 begin
  jt.msg.p.SetTextPos(msgp, 0);

  Finish:=False;

  while not jt.msg.p.EndOfMessage(msgp) and not Finish do
   begin
    jt.msg.p.GetString(msgp, S);

    if (S = '') or (S[1] = #1) then
     Continue;

    ProcessLine(msg, msgp, S, Finish)
   end;
 end;

procedure TMParser.ProcessLine(const msg, msgp: Pointer; var Line: String; var Finish: Boolean);
 begin
  Abstract;
 end;

{ public methods }

function TMParser.CreateReply(const FromName: String; const ReplyToArea: Pointer): Pointer;
 var
  msg, msgp: Pointer;
  AreaFormat: TAreaFormat;
  Address: TAddress;
  S: String;
 begin
  jt.area.GetFormat(ReplyToArea, AreaFormat);

  msg:=jt.msg.Create;

  msgp:=jt.msg.CreateParser(msg, AreaFormat.Style = asNetmail);

  jt.GetNetmailAka(ReplyToAddress, Address);

  jt.msg.p.SetFromAndToAddress(msgp, Address, ReplyToAddress, True);

  jt.msg.SetFromName(msg, FromName);

  S:=ReplyToName;

  CheckValidReplyToName(S);

  if Length(S) > 35 then
   begin
    jt.msg.SetToName(msg, 'UUCP');

    jt.msg.Add(msg, 'To: ' + S);
    jt.msg.Add(msg, '');
   end
  else
   jt.msg.SetToName(msg, S);

  jt.msg.DestroyParser(msgp);

  jt.msg.SetArea(msg, jt.area.GetName(ReplyToArea)^);

  AfterCreateReply(msg);

  CreateReply:=msg;
 end;

procedure TMParser.CheckValidReplyToName(var ToName: String);
 begin
 end;

procedure TMParser.AfterCreateReply(const msg: Pointer);
 begin
 end;

procedure TMParser.PostReply(const msg: Pointer);
 var
  st: Pointer;
 begin
  st:=jt.msg.st(msg);

  jt.st.Add(st, 'Source', Pointer(tsLocalInbound), nil);

  jt.tossscan.TossMessage(msg);
 end;

procedure TMParser.FixEmptyReply(const msg: Pointer);
 begin
  jt.st.Add(jt.msg.st(msg), 'EmptySize', Pointer(jt.msg.Size(msg)), nil);
 end;

function TMParser.IsEmptyReply(const msg: Pointer): Boolean;
 var
  Size: Longint;
 begin
  Size:=Longint(jt.st.Get(jt.msg.st(msg), 'EmptySize'));

  IsEmptyReply:=jt.msg.Size(msg) <= Size;
 end;

end.