unit MsgBox;

{$O+,F+,X+,I-,S-}

interface
uses
     Objects,

     Dialogs;

const
 mfWarning      = $0000;
 mfError        = $0001;
 mfInformation  = $0002;
 mfConfirmation = $0003;

 mfInsertInApp  = $0080;

 mfYesButton    = $0100;
 mfNoButton     = $0200;
 mfOKButton     = $0400;
 mfCancelButton = $0800;

 mfDefault2     = $1000;

 mfYesNoCancel  = mfYesButton + mfNoButton + mfCancelButton;
 mfOKCancel     = mfOKButton + mfCancelButton;

type
 TMessageBoxDialogCreator = function(var Bounds: TRect; const Title: String): PDialog;

function MessageBox(const Msg: String; Params: Pointer; AOptions: Word): Word;
function MessageBoxRect(var R: TRect; const Msg: String; Params: Pointer; AOptions: Word): Word;

procedure BeforehandFormat(const S: String; const SizeX: Longint; var Width, Height: Integer);

function DefaultMessageBoxDialogCreator(var Bounds: TRect; const Title: String): PDialog;

const
 MessageBoxDialogCreator: TMessageBoxDialogCreator = DefaultMessageBoxDialogCreator;

implementation
uses
     Drivers,
     Views,
     App;

function MessageBox(const Msg: String; Params: Pointer; AOptions: Word): Word;
 var
  R: TRect;
  Width, Height: Integer;
 begin
  Width:=Length(Msg);

  if Width < 50 then Width:=50;
  if Width > 76 then Width:=76;

  BeforehandFormat(Msg, Width - 4, Width, Height);

  Inc(Width, 4);

  if Width < 50 then Width:=50;
  if Width > 76 then Width:=76;

  BeforehandFormat(Msg, Width - 4, Width, Height);

  if Width < 50 then Width:=50;
  if Width > 70 then Width:=70;

  Inc(Width, 8);

  R.Assign(0, 0, Width, Height + 6);

  if (AOptions and mfInsertInApp = 0) or (Desktop = nil) then
   R.Move((Desktop^.Size.X - R.B.X) div 2, (Desktop^.Size.Y - R.B.Y) div 2)
  else
   R.Move((Application^.Size.X - R.B.X) div 2, (Application^.Size.Y - R.B.Y) div 2);

  MessageBox:=MessageBoxRect(R, Msg, Params, AOptions);
 end;

function MessageBoxRect(var R: TRect; const Msg: String; Params: Pointer; AOptions: Word): Word;
 const
  ButtonName: array[0..3] of string[6] =
    ('~Y~es',
     '~N~o',
     '~O~K',
     'Cancel');
  Commands: array[0..3] of word =
    (cmYes,
     cmNo,
     cmOK,
     cmCancel);
  Titles: array[0..3] of string[13] =
    ('Warning',
     'Error',
     'Information',
     'Confirmation');
 var
  ButtonList: array[0..4] of PView;
  I, X, ButtonCount: Integer;
  Dialog: PDialog;
  Control: PView;
  S: String;
 begin
  Dialog:=MessageBoxDialogCreator(R, Titles[AOptions and $3]);

  with Dialog^ do
   begin
    R.Assign(4, 2, Size.X - 4, Size.Y - 4);

    FormatStr(S, Msg, Params^);

    Control:=New(PStaticText, Init(R, S));
{
    Control^.Options:=Control^.Options or ofFramed;
}
    Insert(Control);

    X:=-2;

    ButtonCount:=0;

    for I:= 0 to 3 do
     if AOptions and ($0100 shl I) <> 0 then
      begin
       R.Assign(0, 0, 10, 2);

       Control:=New(PButton, Init(R, ButtonName[I], Commands[i], bfNormal));

       Inc(X, Control^.Size.X + 2);

       ButtonList[ButtonCount]:=Control;

       Inc(ButtonCount);
      end;

    X:=(Size.X - X) shr 1;

    for I:=0 to ButtonCount - 1 do
     begin
      Control:=ButtonList[I];

      Insert(Control);

      Control^.MoveTo(X, Size.Y - 3);

      Inc(X, Control^.Size.X + 2);
     end;

    SelectNext(False);

    if AOptions and mfDefault2 <> 0 then
     SelectNext(False);
   end;

  if (AOptions and mfInsertInApp = 0) or (Desktop = nil) then
   MessageBoxRect:=Desktop^.ExecView(Dialog)
  else
   MessageBoxRect:=Application^.ExecView(Dialog);

  Dispose(Dialog, Done);
 end;

{ BeforehandFormat // dnw32 }

procedure BeforehandFormat(const S: String; const SizeX: Longint; var Width, Height: Integer);
 var
  Center: Boolean;
  I, J, L, P, LastWidth: Longint;
 begin
  L:=Length(S);

  P:=1;
  Height:=0;
  Width:=0;
  LastWidth:=1;

  Center:=False;

  while True do
   begin
    if P <= L then
     begin
      if S[P] = #3 then
       begin
        Center:=True;

        Inc(P);
       end;

      I:=P;

      repeat
       J:=P;

       while (P <= L) and (S[P] = ' ') do Inc(P);
       while (P <= L) and (S[P] <> ' ') and (S[P] <> #13) do Inc(P);
      until (P > L) or (P >= I + SizeX) or (S[P] = #13);

      if P > I + SizeX then
       if J > I then
        P:=J
       else
        P:=I + SizeX;

      if Center then
       J:=(SizeX - P + I) div 2
      else
       J:=0;

      LastWidth:=P - I;

      if LastWidth > Width then
       Width:=LastWidth;

      while (P <= L) and (S[P] = ' ') do Inc(P);

      if (P <= L) and (S[P] = #13) then
       begin
        Center:=False;

        Inc(P);

        if (P <= L) and (S[P] = #10) then Inc(P);
       end;
     end
    else
     Break;

    Inc(Height);
   end;

  if LastWidth = 0 then
   Dec(Height);
 end;

{ DefaultMessageBoxDialogCreator }

function DefaultMessageBoxDialogCreator(var Bounds: TRect; const Title: String): PDialog;
 begin
  DefaultMessageBoxDialogCreator:=New(PDialog, Init(Bounds, Title));
 end;

end.
