library Onfire;

{
 dedicated to Dmitry Demidoff and Johnny Markewitch
}

uses
     PlugAPI,
     PlugSIDE,

     Objects,

     TossAPI,
     PrioAPI,

     Thunks,

     jtMHLcmn,

     Wizard;

const
 OnfirePlugin           : Pointer       = nil;
 OnfireRestriction      : Pointer       = nil;
 OnfireArea             : Pointer       = nil;

 OnfireCriterion        : (ocWritten, ocArrived) = ocArrived;

 OnfireTossFire                         = $1000;

 OnfireSkipFire                         = 'OnfireSkipFire';

 AreaKludge                             = #1'AREA:';
 AreaKludge2                            =   'AREA:';
 FromKludge                             = #1'FROM:';
 FromKludge2                            =   'FROM:';

{ TossMessage }

procedure TossMessage(var tmb: TTossMessageBlock);
 var
  st, msg: Pointer;
  Address: TAddress;
  Source, Destination: PStream;
 begin
  st:=jt.msg.st(tmb.msg);

  if (tmb.Area = nil) or (tmb.Link = nil) or LongBool(jt.st.Get(st, OnfireSkipFire)) then
   Exit;

  jt.link.GetAddress(tmb.Link, Address);

  if not jt.rests.Check(OnfireRestriction, @Address, jt.area.GetName(tmb.Area)) then
   Exit;

  msg:=jt.msg.Clone(tmb.msg, mcfCloneHeader);

  jt.msg.SetArea(msg, jt.area.GetName(OnfireArea)^);

  jt.msg.Add(msg, AreaKludge + ' ' + jt.area.GetName(tmb.Area)^);
  jt.msg.Add(msg, FromKludge + ' ' + AddressToStrEx(PAddress(jt.st.Get(st, 'Packet.From'))^));

  Source:=New(PStreamThunk, Init(jt.msg.GetText(tmb.msg), jt.StreamLinker, False));
  Destination:=New(PStreamThunk, Init(jt.msg.GetText(msg), jt.StreamLinker, False));

  Source^.Seek(0);
  Destination^.CopyFrom(Source^, Source^.GetSize);

  Dispose(Destination, Done);
  Dispose(Source, Done);

  st:=jt.msg.st(msg);

  jt.st.Add(st, 'Source', Pointer(tsBaseInbound), nil);
  jt.st.Add(st, 'TossStatusPrefix', jt.area.GetName(tmb.Area), nil);

  jt.tossscan.TossMessage(msg);

  jt.LogWrite('O', 'Onfire: ' + AddressToStrEx(Address) + ' -> ' + jt.area.GetName(tmb.Area)^);

  tmb.Area:=nil;
 end;

{ TossFire }

procedure TossFire(const Days: Longint);
 var
  DT: TDateTime;
  Locked: Boolean;
  FromAddress, ToAddress: TAddress;
  mb, msg, msgp, st, Link: Pointer;
  No, ThatDay, ThisDay, Unfired, Keeping: Longint;
  rc, Area, AreaToKill, From, FromToKill, Password: String;
 const
  First: Boolean = True;
 procedure RemoveKludgePrefix(var ToKill, S: String);
  var
   K: Byte;
  begin
   K:=Pos(':', S);

   if K <> 0 then
    begin
     ToKill:=S;

     Delete(S, 1, K);

     TrimEx(S);
    end;
  end;
 begin
  if OnfireArea = nil then
   Exit;

  if not jt.area.Lock(OnfireArea, mb, rc) then
   begin
    jt.LogWrite('?', 'Unable to open ' + jt.area.GetName(OnfireArea)^ + ': ' + rc);

    Exit;
   end;

  jt.SetStatus('unfiring ' + jt.area.GetName(OnfireArea)^ + '...');

  No:=0;

  Locked:=False;
  Unfired:=0;
  Keeping:=0;

  if Days <> -1 then
   begin
    GetCurrentDateTime(DT);

    ThisDay:=GregorianToJulian(DT);
   end;

  while No <= jt.mb.Count(mb) - 1 do
   begin
    if jt.mb.Load(mb, No, msg, rc, mfNone) then
     begin
      if Days <> -1 then
       begin
        case OnfireCriterion of
         ocWritten: jt.msg.GetWritten(msg, DT);
         ocArrived: jt.msg.GetArrived(msg, DT);
        end;

        ThatDay:=GregorianToJulian(DT);
       end;

      if (jt.msg.GetAttributes(msg) and maRead <> 0) or ((Days <> -1) and (ThisDay - ThatDay <= Days)) then
       begin
        msgp:=jt.msg.CreateParser(msg, False);

        if (jt.msg.p.GetKludge(msgp, AreaKludge, Area) or jt.msg.p.GetKludge(msgp, AreaKludge2, Area)) and
           (jt.msg.p.GetKludge(msgp, FromKludge, From) or jt.msg.p.GetKludge(msgp, FromKludge2, From)) then
         begin
          if not Locked then
           if not jt.mb.Lock(mb) then
            begin
             jt.LogWrite('?', 'Unable to lock ' + jt.area.GetName(OnfireArea)^);

             Break;
            end
           else
            begin
             Locked:=True;

             PerformPriorityAction('unfiring');
            end;

          RemoveKludgePrefix(AreaToKill, Area);
          RemoveKludgePrefix(FromToKill, From);

          ClearAddress(FromAddress);

          if (not StrToAddress(From, FromAddress)) or (Area = '') then
           begin
            if Area = '' then
             jt.LogWrite('?', 'Invalid @AREA kludge in msg ' + Long2Str(No) + ' of ' + jt.area.GetName(OnfireArea)^)
            else
             jt.LogWrite('?', 'Invalid @FROM kludge in msg ' + Long2Str(No) + ' of ' + jt.area.GetName(OnfireArea)^);

            jt.msg.DestroyParser(msgp);

            jt.msg.Destroy(msg);

            Inc(No);
           end
          else
           begin
            Link:=jt.link.Seek(FromAddress);

            if Link = nil then
             begin
              jt.LogWrite('?', 'Unknown link ' + AddressToStrEx(FromAddress) + ' in msg ' +
               Long2Str(No) + ' of ' + jt.area.GetName(OnfireArea)^);

              jt.msg.DestroyParser(msgp);

              jt.msg.Destroy(msg);

              Inc(No);
             end
            else
             begin
              jt.msg.p.DeleteKludge(msgp, AreaToKill);
              jt.msg.p.DeleteKludge(msgp, FromToKill);

              jt.msg.DestroyParser(msgp);

              st:=jt.msg.st(msg);

              jt.link.GetAka(Link, ToAddress);
              jt.link.GetInPassword(Link, Password);

              jt.st.Add(st, 'Packet.From', jt.NewAddress(FromAddress), jt.svc.stAddress);
              jt.st.Add(st, 'Packet.To', jt.NewAddress(ToAddress), jt.svc.stAddress);
              jt.st.Add(st, 'Packet.Password', jt.NewStr(Password), jt.svc.stString);
              jt.st.Add(st, 'Source', Pointer(tsInbound), nil);
              jt.st.Add(st, OnfireSkipFire, Pointer(1), nil);

              jt.msg.SetArea(msg, Area);

              if jt.mb.Kill(mb, No, rc) then
               begin
                jt.tossscan.TossMessage(msg);

                Inc(Unfired);
               end
              else
               begin
                jt.LogWrite('?', 'Unable to kill msg ' + Long2Str(No) + ' in ' + jt.area.GetName(OnfireArea)^ + ': ' + rc);

                jt.msg.Destroy(msg);

                Inc(No);
               end;
             end;
           end;
         end
        else
         begin
          jt.msg.DestroyParser(msgp);

          jt.msg.Destroy(msg);

          Inc(No);
         end;
       end
      else
       begin
        Inc(No);

        Inc(Keeping);

        jt.msg.Destroy(msg);
       end;
     end
    else
     begin
      jt.LogWrite('?', 'Unable to open msg ' + Long2Str(No) + ' in ' + jt.area.GetName(OnfireArea)^ + ': ' + rc);

      Inc(No);
     end;
   end;

  if Locked then
   jt.mb.Unlock(mb);

  jt.area.Unlock(OnfireArea, mb);

  if Unfired <> 0 then
   begin
    jt.list.Add(jt.list.ImportList, jt.area.GetName(OnfireArea)^);

    if Days = -1 then
     jt.LogWrite('O', Long2Str(Unfired) + ' message' +
                            _s(Unfired) + ' unfired' +
                           _nz(Keeping, ', ' + Long2Str(Keeping) + ' still keeping'))
    else
     jt.LogWrite('O', Long2Str(Unfired) + ' message' +
                            _s(Unfired) + ' unfired' +
                           _nz(Keeping, ', ' + Long2Str(Keeping) + ' still keeping') +
                           ' (' + Long2Str(Days) + ' day' + _s(Days) + ')');
   end
  else
   jt.ClearStatus;
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, 'Onfire', 1);

  OnfirePlugin:=Plugin;

  PluginSetup:=True;

  OnfireRestriction:=jt.rests.Add('Onfire', rsMinus);

  RegisterPriorityAction('unfiring');
 end;

{ PluginStartup }

procedure PluginStartup; export;
 begin
  if (OnfireArea <> nil) and (jt.rests.GetUsed(OnfireRestriction) > 0) then
   RegisterTosserCallback(OnfirePlugin, taMessageTossFilter);
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
  DisposeTosserCallbacks;
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 begin
  if Command = tcMessageTossFilter then TossMessage(PTossMessageBlock(Param)^) else
  if Command = OnfireTossFire then TossFire(Param);
 end;

{ PluginHandleCmdlineSwitch }

function PluginHandleCmdlineSwitch(Switch: String): Boolean; export;
 var
  Number: Longint;
 begin
  StUpcaseEx(Switch);

  PluginHandleCmdlineSwitch:=True;

  if Switch = 'TOSSFIRE' then jt.EnqueueCommand(OnfirePlugin, OnfireTossFire, -1, cwBeforeToss) else
  if Copy(Switch, 1, 9) = 'TOSSFIRE:' then
   begin
    Delete(Switch, 1, 9);

    if not Str2Longint(Switch, Number) then
     jt.linker.Error(jt.linker.Stub, 'failed to parse number "' + Switch + '"');

    if Number < 0 then
     jt.linker.Error(jt.linker.Stub, 'invalid number "' + Switch + '"');

    jt.EnqueueCommand(OnfirePlugin, OnfireTossFire, Number, cwBeforeToss);
   end
  else
   PluginHandleCmdlineSwitch:=False;
 end;

{ PluginHandleConfigSwitch }

procedure PluginHandleConfigSwitch(const Keyword: String; Value: String; const Linker: Pointer); export;
 var
  S: String;
 begin
  if Keyword = 'AREA' then
   begin
    S:=jt.linker.ParseStringToken(Linker, Value);

    OnfireArea:=jt.area.Seek(S);

    if OnfireArea = nil then
     jt.linker.Error(Linker, 'unknown area "' + S + '"')
    else
     jt.area.Used(OnfireArea);
   end else
  if Keyword = 'TOSSFIREDATECRITERION' then
   begin
    S:=jt.linker.ParseWord(Linker, Value);

    StUpcaseEx(S);

    if S = 'WRITTEN' then OnfireCriterion:=ocWritten else
    if S = 'ARRIVED' then OnfireCriterion:=ocArrived else
     jt.linker.Error(Linker, 'WRITTEN or ARRIVED expected');
   end
  else
   jt.linker.Error(Linker, 'syntax error');
 end;

{ PluginShowHelp }

procedure PluginShowHelp; export;
 begin
  jt.PrintStr(' TOSSFIRE         scan Onfire area for read messages and admit them'#13#10);
  jt.PrintStr(' TOSSFIRE:X       scan Onfire area for read messages / messages older X days'#13#10);
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleCmdlineSwitch,
 PluginHandleConfigSwitch,
 PluginShowHelp;

begin
end.
