unit OutbCche;

interface
uses
     Objects,

     jtMHLcmn,

     RStream,

     Thunks,

     Wizard,

     PlugAPI,
     PlugSIDE;

type
 PBundleNameCacheLink = ^TBundleNameCacheLink;
 TBundleNameCacheLink = object(TStringCollection)
  Address: TAddress;

  constructor Init(const AAddress: TAddress);
  constructor Load(var S: TStream);
  procedure Store(var S: TStream); virtual;
 end;

 PBundleNameCache = ^TBundleNameCache;
 TBundleNameCache = object(TSortedCollection)
  Changed: Boolean;

  constructor Init;
  constructor Load(var S: TStream);
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  function KeyOf(Item: Pointer): Pointer; virtual;
  function Was(const Address: TAddress; FName: String): Boolean;
 end;

const
 BundleNameCache        : PBundleNameCache      = nil;

 idBundleNameCache      : Longint               = $91782600 or OSuid;

 rfBundleNameCache                              = 'Outbound.BundleNameCache';

 RBundleNameCacheLink: TStreamRec = (
  ObjType: $1000;
  VmtLink: Ofs(TypeOf(TBundleNameCacheLink)^);
  Load: @TBundleNameCacheLink.Load;
  Store: @TBundleNameCacheLink.Store);

 RBundleNameCache: TStreamRec = (
  ObjType: $1001;
  VmtLink: Ofs(TypeOf(TBundleNameCache)^);
  Load: @TBundleNameCache.Load;
  Store: @TBundleNameCache.Store);

procedure InitBundleNameCache;
procedure DoneBundleNameCache;

implementation

{ TBundleNameCacheLink }

constructor TBundleNameCacheLink.Init(const AAddress: TAddress);
 begin
  inherited Init($0, $10);

  Address:=AAddress;
 end;

constructor TBundleNameCacheLink.Load(var S: TStream);
 begin
  S.Read(Address, SizeOf(Address));

  inherited Load(S);
 end;

procedure TBundleNameCacheLink.Store(var S: TStream);
 begin
  S.Write(Address, SizeOf(Address));

  inherited Store(S);
 end;

{ TBundleNameCache }

constructor TBundleNameCache.Init;
 begin
  inherited Init($0, $10);

  Changed:=True;
 end;

constructor TBundleNameCache.Load(var S: TStream);
 begin
  Changed:=False;

  inherited Load(S);
 end;

function TBundleNameCache.Compare(Key1, Key2: Pointer): xInteger;
 begin
  Compare:=AddressCompare(PAddress(Key1)^, PAddress(Key2)^);
 end;

function TBundleNameCache.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=@PBundleNameCacheLink(Item)^.Address;
 end;

function TBundleNameCache.Was(const Address: TAddress; FName: String): Boolean;
 var
  Index: xInteger;
  Link: PBundleNameCacheLink;
 begin
  FName:=JustFileName(FName);

  StUpcaseEx(FName);

  if Search(@Address, Index) then
   begin
    Link:=At(Index);

    if Link^.Search(@FName, Index) then
     Was:=True
    else
     begin
      Link^.Insert(NewStr(FName));

      Changed:=True;

      Was:=False;
     end;
   end
  else
   begin
    Link:=New(PBundleNameCacheLink, Init(Address));

    Insert(Link);

    Link^.Insert(NewStr(FName));

    Changed:=True;

    Was:=False;
   end;
 end;

{ InitBundleNameCache }

procedure InitBundleNameCache;
 var
  Stream: PStream;
  id, Day: Longint;
  DT: TDateTime;
 begin
  RegisterType(RBundleNameCacheLink);
  RegisterType(RBundleNameCache);

  Stream:=New(PRamStream, Init);

  jt.rf.Get(jt.rf.common, rfBundleNameCache, Stream, StreamLinker);

  if Stream^.Status = stOk then
   begin
    Stream^.Seek(0);

    Stream^.Read(id, SizeOf(id));

    if id = idBundleNameCache then
     begin
      Stream^.Read(Day, SizeOf(Day));

      GetCurrentDateTime(DT);

      if Day = GregorianToJulian(DT) then
       begin
        BundleNameCache:=PBundleNameCache(Stream^.Get);

        if BundleNameCache = nil then
         BundleNameCache:=New(PBundleNameCache, Init);
       end
      else
       BundleNameCache:=New(PBundleNameCache, Init);
     end
    else
     BundleNameCache:=New(PBundleNameCache, Init);
   end
  else
   BundleNameCache:=New(PBundleNameCache, Init);

  Dispose(Stream, Done);
 end;

{ DoneBundleNameCache }

procedure DoneBundleNameCache;
 var
  Day: Longint;
  DT: TDateTime;
  Stream: PStream;
 begin
  if BundleNameCache^.Changed then
   begin
    Stream:=New(PRamStream, Init);

    Stream^.Write(idBundleNameCache, SizeOf(idBundleNameCache));

    GetCurrentDateTime(DT);

    Day:=GregorianToJulian(DT);

    Stream^.Write(Day, SizeOf(Day));

    Stream^.Put(BundleNameCache);

    jt.rf.Add(jt.rf.common, rfBundleNameCache, Stream, StreamLinker);

    Dispose(Stream, Done);
   end;

  Dispose(BundleNameCache, Done);
 end;

end.
