unit OutbCmn;

interface
uses
     Objects,

     PlugAPI,
     PlugSIDE,

     ccAPI,

     Wizard;

{$IFDEF VIRTUALPASCAL}
 {$DEFINE LFN}
{$ENDIF}

const
 obName                         = 'Outbounds';
 obVersion                      = 4;
 obPlugin       : Pointer       = nil;

 obLetterError                  = '!';
 obLetterQueue                  = 'Q';

 obomlFormat                    = 'ob.Format';
 obomlOutprocessor              = 'ob.Outprocessor';
 obomlQueue                     = 'ob.Queue';
 obomlKill                      = 'ob.Kill';
 obomlDirectory                 = 'ob.Directory';
 obomlFlavour                   = 'ob.Flavour';
 obomlPackEchomail              = 'ob.PackEchomail';
 obomlAlwaysNewBundle           = 'ob.AlwaysNewBundle';

 obLinkQueue                    = 'ob.LinkQueue';

 osfNone                        = $0000;
 osfQueue                       = $0001;
 osfKill                        = $0002;
 osfPackEchomail                = $0004;
 osfNetmail                     = $0008;
 osfAlwaysNewBundle             = $0010;

 obFlushQueue                   = $1000;

 PacketsQueue         : String  = '';

 RootBSO              : String  = '';
 RootTBOX             : String  = '';
 RootLBSO             : String  = '';
 RootTLBOX            : String  = '';
 RootLBOX             : String  = '';
 RootASO              : String  = '';

 HoldPack             : Boolean = False;

 FlushQueueAfterToss  : Boolean = False;

type
 TobMode    = (omBSO,
               omTBOX,
               omBOX
               {$IFDEF LFN},
               omLBSO,
               omTLBOX,
               omLBOX,
               omASO
               {$ENDIF});

 TobFlavour = (ofUnknown, ofHold, ofNormal, ofDirect, ofCrash, ofImmediate);
 TobFlavours = set of TobFlavour;

 TobFileMode = (ofmNormal, ofmKill, ofmTruncate);

 PobDesc = ^TobDesc;
 TobDesc = record
  Name: String[10];
  Root: PString;
  Refs: Integer;
  om: Pointer;
  Kill: Boolean;
  Flavours: TobFlavours;
 end;

const
 AllFlavours = [ofHold, ofNormal, ofDirect, ofCrash, ofImmediate];

 obfName: array[TobFlavour] of String[9] =
  ('Unknown',
   'Hold',
   'Normal',
   'Direct',
   'Crash',
   'Immediate');

 ob: array[TobMode] of TobDesc = (
  (Name: 'BSO';   Root: @RootBSO;   Refs: 0; om: nil; Kill: False; Flavours: AllFlavours),
  (Name: 'TBOX';  Root: @RootTBOX;  Refs: 0; om: nil; Kill: True;  Flavours: [ofHold, ofNormal]),
  (Name: 'BOX';   Root: nil;        Refs: 0; om: nil; Kill: True;  Flavours: [ofNormal])
  {$IFDEF LFN},
  (Name: 'LBSO';  Root: @RootLBSO;  Refs: 0; om: nil; Kill: False; Flavours: AllFlavours),
  (Name: 'TLBOX'; Root: @RootTLBOX; Refs: 0; om: nil; Kill: True;  Flavours: [ofHold, ofNormal]),
  (Name: 'LBOX';  Root: @RootLBOX;  Refs: 0; om: nil; Kill: True;  Flavours: [ofNormal]),
  (Name: 'ASO';   Root: @RootASO;   Refs: 0; om: nil; Kill: False; Flavours: AllFlavours)
  {$ENDIF});

 ocNone                 = $00000000;
 ocSetFiles             = $00000001;

var
 ccoaQueueBeforePack    : PccHook;
 ccoaQueuePack          : PccHook;

function DistinguishMode(const om: Pointer): TobMode;

function IsArcmail(FName: String): Boolean;

{ outbound mode session thunks }

procedure omsInit(var Block: TomsInitBlock);
procedure omsDone(var Block: TomsDoneBlock);
procedure omsGetBusy(var Block: TomsGetBusyBlock);
procedure omsSetBusy(var Block: TomsSetBusyBlock);
procedure omsExportMessage(var Block: TomsExportMessageBlock);

implementation
uses
     OutbSess;

{ DistinguishMode }

function DistinguishMode(const om: Pointer): TobMode;
 var
  K: TobMode;
 begin
  for K:=Low(TobMode) to High(TobMode) do
   if ob[K].om = om then
    begin
     DistinguishMode:=K;

     Exit;
    end;
 end;

{ IsArcmail }

function IsArcmail(FName: String): Boolean;
 begin
  FName:=Copy(JustExtension(FName), 1, 2);

  StUpcaseEx(FName);

  IsArcmail:=(FName = 'SU') or
             (FName = 'MO') or
             (FName = 'TU') or
             (FName = 'WE') or
             (FName = 'TH') or
             (FName = 'FR') or
             (FName = 'SA');
 end;

{ outbound mode session thunks }

procedure omsInit(var Block: TomsInitBlock);
 begin
  Block.Session:=New(PSession, Init(Block.oml, Block.FromAddress, Block.ToAddress, Block.Password));

  Block.Done:=Block.Session <> nil;
 end;

procedure omsDone(var Block: TomsDoneBlock);
 begin
  Dispose(PSession(Block.Session), Done);
 end;

procedure omsGetBusy(var Block: TomsGetBusyBlock);
 begin
  Block.Busy:=PSession(Block.Session)^.GetBusy;
 end;

procedure omsSetBusy(var Block: TomsSetBusyBlock);
 begin
  Block.Done:=PSession(Block.Session)^.SetBusy(Block.Busy);
 end;

procedure omsExportMessage(var Block: TomsExportMessageBlock);
 begin
  Block.Done:=PSession(Block.Session)^.ExportMessage(Block.msg);
 end;

end.