unit OutbOB;

interface
uses
     Objects,
     Dos,

     OutbCmn,
     OutbSess,

     PlugAPI,
     PlugSIDE,

     Wizard;

{$IFDEF VIRTUALPASCAL}
 {$DEFINE LFN}
{$ENDIF}

type
 POutboundFile = ^TOutboundFile;
 TOutboundFile = object(TObject)
  constructor Init(const AFlavour: TobFlavour; const AFileMode: TobFileMode; const AFName: String);
  destructor Done; virtual;
 public
  Flavour: TobFlavour;
  FileMode: TobFileMode;
  FName: PString;
 end;

 POutboundFiles = ^TOutboundFiles;
 TOutboundFiles = object(TCollection)
 end;

 POutbound = ^TOutbound;
 TOutbound = object(TObject)
  constructor Init(const ASession: PSession);
  destructor Done; virtual;
 public
  function InitRoot(var ARoot: String): Boolean; virtual;
  function InitPath(var APath: String): Boolean; virtual;
  function GetBusy: Boolean; virtual;
  function SetBusy(const Busy: Boolean): Boolean; virtual;
  function GetFiles: POutboundFiles; virtual;
  function SetFiles(const Files: POutboundFiles; const Fresh: Boolean): Boolean; virtual;
  function Capabilities: Longint; virtual;
  function IsValidFlavour(const Flavour: TobFlavour): Boolean; virtual;
  function NetmailPacketName: String; virtual;
  function PurePath: String; virtual;
 public
  Session: PSession;
  Domain, DomainOutbound, Root, Path: PString;
  MainZone: Longint;
  Link: Pointer;
 end;

 PBSO = ^TBSO;
 TBSO = object(TOutbound)
  function InitRoot(var ARoot: String): Boolean; virtual;
  function InitPath(var APath: String): Boolean; virtual;
  function GetBusy: Boolean; virtual;
  function SetBusy(const Busy: Boolean): Boolean; virtual;
  function GetFiles: POutboundFiles; virtual;
  function SetFiles(const Files: POutboundFiles; const Fresh: Boolean): Boolean; virtual;
  function Capabilities: Longint; virtual;
  function IsValidFlavour(const Flavour: TobFlavour): Boolean; virtual;
  function NetmailPacketName: String; virtual;
 public
  function FlavourExtension(const Flavour: TobFlavour): String; virtual;
  function NetmailExtension(const Flavour: TobFlavour): String; virtual;
 end;

 PBoxOutbound = ^TBoxOutbound;
 TBoxOutbound = object(TOutbound)
  function GetBusy: Boolean; virtual;
  function SetBusy(const Busy: Boolean): Boolean; virtual;
  function GetFiles: POutboundFiles; virtual;
  function Capabilities: Longint; virtual;
 end;

 PTBOX = ^TTBOX;
 TTBOX = object(TBoxOutbound)
  function InitRoot(var ARoot: String): Boolean; virtual;
  function IsValidFlavour(const Flavour: TobFlavour): Boolean; virtual;
 end;

 {$IFDEF LFN}
 PLBSO = ^TLBSO;
 TLBSO = object(TOutbound)
  function InitRoot(var ARoot: String): Boolean; virtual;
  function InitPath(var APath: String): Boolean; virtual;
  function GetBusy: Boolean; virtual;
  function SetBusy(const Busy: Boolean): Boolean; virtual;
  function IsValidFlavour(const Flavour: TobFlavour): Boolean; virtual;
 public
  function FlavourExtension(const Flavour: TobFlavour): String; virtual;
  function NetmailExtension(const Flavour: TobFlavour): String; virtual;
 end;
 {$ENDIF}

 {$IFDEF LFN}
 PTLBOX = ^TTLBOX;
 TTLBOX = object(TBoxOutbound)
  function InitRoot(var ARoot: String): Boolean; virtual;
  function IsValidFlavour(const Flavour: TobFlavour): Boolean; virtual;
 end;
 {$ENDIF}

 {$IFDEF LFN}
 PLBOX = ^TLBOX;
 TLBOX = object(TBoxOutbound)
  function InitRoot(var ARoot: String): Boolean; virtual;
  function IsValidFlavour(const Flavour: TobFlavour): Boolean; virtual;
 end;
 {$ENDIF}

 {$IFDEF LFN}
 PASO = ^TASO;
 TASO = object(TBSO)
  function InitRoot(var ARoot: String): Boolean; virtual;
  function InitPath(var APath: String): Boolean; virtual;
  function IsValidFlavour(const Flavour: TobFlavour): Boolean; virtual;
 end;
 {$ENDIF}

 PBOX = ^TBOX;
 TBOX = object(TBoxOutbound)
  function InitRoot(var ARoot: String): Boolean; virtual;
  function IsValidFlavour(const Flavour: TobFlavour): Boolean; virtual;
 end;

 POutboundErrors = ^TOutboundErrors;
 TOutboundErrors = object(TStrings)
  procedure Add(const S: String);
 end;

const
 obErrors       : POutboundErrors       = nil;

implementation

{ Duotricenary }

const
 DuotricTable: String = '0123456789ABCDEFGHIJKLMNOPQRSTUV';

function Duotricenary(I, Digits: Word): String;
 var
  S: String;
  B, J: Integer;
 begin
  S[0]:=Chr(Digits);

  for J:=1 to Digits do
   begin
    B:=I and 31;

    S[Digits - J + 1]:=DuotricTable[B + 1];

    I:=I shr 5;
   end;

  Duotricenary:=S;
 end;

{ MapBrakeFlavour }

function MapBrakeFlavour(const Flavour: TobFlavour): String;
 begin
  case Flavour of
   ofImmediate: MapBrakeFlavour:='Immediate';
   ofCrash: MapBrakeFlavour:='Crash';
   ofDirect: MapBrakeFlavour:='Direct';
   ofNormal: MapBrakeFlavour:='Normal';
   ofHold: MapBrakeFlavour:='Hold';
  else
   MapBrakeFlavour:='Normal';
  end;
 end;

{ TOutboundFile }

constructor TOutboundFile.Init(const AFlavour: TobFlavour; const AFileMode: TobFileMode; const AFName: String);
 begin
  inherited Init;

  Flavour:=AFlavour;
  FileMode:=AFileMode;
  FName:=NewStr(AFName);
 end;

destructor TOutboundFile.Done;
 begin
  DisposeStr(FName);

  inherited Done;
 end;

{ TOutbound }

constructor TOutbound.Init(const ASession: PSession);
 var
  ADomain, ADomainOutbound, ARoot, APath: String;
 begin
  inherited Init;

  Session:=ASession;

  jt.GetDomainInfo(Session^.ToAddress, ADomain, ADomainOutbound, MainZone);

  Domain:=NewStr(ADomain);
  DomainOutbound:=NewStr(ADomainOutbound);

  Link:=jt.link.Seek(Session^.ToAddress);

  if not InitRoot(ARoot) or (ARoot = '') then
   begin
    Done;

    Fail;
   end;

  Root:=NewStr(ARoot);

  if not InitPath(APath) then
   begin
    Done;

    Fail;
   end;

  Path:=NewStr(APath);
 end;

destructor TOutbound.Done;
 begin
  DisposeStr(DomainOutbound);
  DisposeStr(Domain);
  DisposeStr(Root);
  DisposeStr(Path);

  inherited Done;
 end;

{ public methods }

function TOutbound.InitRoot(var ARoot: String): Boolean;
 begin
  if ob[Session^.Mode].Root^ <> '' then
   begin
    ARoot:=AddBackSlash(ob[Session^.Mode].Root^);

    InitRoot:=True;
   end
  else
   InitRoot:=False;
 end;

function TOutbound.InitPath(var APath: String): Boolean;
 begin
  APath:=Root^;

  InitPath:=True;
 end;

function TOutbound.GetBusy: Boolean;
 begin
 end;

function TOutbound.SetBusy(const Busy: Boolean): Boolean;
 begin
 end;

function TOutbound.GetFiles: POutboundFiles;
 begin
  GetFiles:=nil;
 end;

function TOutbound.SetFiles(const Files: POutboundFiles; const Fresh: Boolean): Boolean;
 begin
  SetFiles:=False;
 end;

function TOutbound.Capabilities: Longint;
 begin
  Capabilities:=ocNone;
 end;

function TOutbound.IsValidFlavour(const Flavour: TobFlavour): Boolean;
 begin
  IsValidFlavour:=True;
 end;

function TOutbound.NetmailPacketName: String;
 begin
  NetmailPacketName:='';
 end;

function TOutbound.PurePath: String;
 begin
  PurePath:=AddBackSlash(JustPathName(Path^));
 end;

{ TBSO }

function TBSO.InitRoot(var ARoot: String): Boolean;
 var
  Extension: String;
 begin
  if inherited InitRoot(ARoot) then
   begin
    if Session^.ToAddress.Zone = MainZone then
     Extension:=''
    else
     Extension:='.' + HexT(Session^.ToAddress.Zone);

    if DomainOutbound = nil then
     ARoot:=ARoot + 'OUTBOUND' + Extension + '\'
    else
     ARoot:=ARoot + DomainOutbound^ + Extension + '\';

    InitRoot:=jt.CreateDirectory(ARoot);
   end
  else
   InitRoot:=False;
 end;

function TBSO.InitPath(var APath: String): Boolean;
 begin
  APath:=Root^ + HexW(Session^.ToAddress.Net) + HexW(Session^.ToAddress.Node);

  if Session^.ToAddress.Point <> 0 then
   begin
    APath:=APath + '.PNT\' + HexL(Session^.ToAddress.Point);

    if not jt.CreateDirectory(JustPathName(APath)) then
     begin
      InitPath:=False;

      Exit;
     end;
   end;

  APath:=APath + '.';

  InitPath:=True;
 end;

function TBSO.GetBusy: Boolean;
 begin
  GetBusy:=ExistFile(Path^ + 'bsy') or ExistFile(Path^ + 'csy');
 end;

function TBSO.SetBusy(const Busy: Boolean): Boolean;
 begin
  if Busy then
   if TouchFile(Path^ + 'bsy') then
    SetBusy:=True
   else
    begin
     SetBusy:=False;

     jt.Report('Failed to touch ' + Path^ + 'bsy', True);
    end
  else
   if EraseFile(Path^ + 'bsy') then
    SetBusy:=True
   else
    begin
     SetBusy:=False;

     jt.Report('Failed to erase ' + Path^ + 'bsy', True);
    end;
 end;

function TBSO.GetFiles: POutboundFiles;
 var
  Files: POutboundFiles;
 procedure GetList(const Extension: String; const Flavour: TobFlavour);
  var
   F: Text;
   S: String;
   Mode: TobFileMode;
  begin
   if Extension = '' then
    Exit;

   if IOResult <> 0 then;

   Assign(F, Path^ + Extension);
   Reset(F);

   if IOResult = 0 then
    begin
     while not Eof(F) do
      begin
       ReadLn(F, S);

       if S = '' then
        Continue;

       case S[1] of
        '^': Mode:=ofmKill;
        '#': Mode:=ofmTruncate;
        '~': Continue;
       else
        Mode:=ofmNormal;
       end;

       if Mode <> ofmNormal then
        Delete(S, 1, 1);

       if S <> '' then
        Files^.Insert(New(POutboundFile, Init(Flavour, Mode, S)));
      end;

     Close(F);

     if IOResult <> 0 then;
    end;
  end;
 var
  Flavour: TobFlavour;
 begin
  Files:=New(POutboundFiles, Init($10, $10));

  for Flavour:=Low(Flavour) to High(Flavour) do
   GetList(FlavourExtension(Flavour), Flavour);

  GetFiles:=Files;
 end;

function TBSO.SetFiles(const Files: POutboundFiles; const Fresh: Boolean): Boolean;
 function SetList(const Extension: String; const Flavour: TobFlavour): Boolean;
  var
   Found: Boolean;
   Target: Text;
  procedure Perform(const F: POutboundFile); far;
   begin
    if (F^.FName <> nil) and (F^.Flavour = Flavour) then
     begin
      case F^.FileMode of
       ofmNormal: WriteLn(Target, F^.FName^);
       ofmKill: WriteLn(Target, '^', F^.FName^);
       ofmTruncate: WriteLn(Target, '#', F^.FName^);
      end;

      Found:=True;
     end;
   end;
  var
   Time: Longint;
  begin
   if Extension = '' then
    begin
     SetList:=True;

     Exit;
    end;

   if IOResult <> 0 then;

   if Fresh then
    Time:=GetFileDate(Path^ + Extension);

   Assign(Target, Path^ + Extension);
   Rewrite(Target);

   if InOutRes = 0 then
    begin
     Found:=False;

     Files^.ForEach(@Perform);

     if Fresh then
      SetFTime(Target, Time);

     Close(Target);

     if not Found then
      begin
       if IOResult <> 0 then;

       Erase(Target);

       if InOutRes <> 0 then
        begin
         SetFiles:=False;

         obErrors^.Add('unable to erase ' + Path^ + Extension + ', rc=' + Long2Str(IOResult));
        end;
      end;
    end
   else
    begin
     SetFiles:=False;

     obErrors^.Add('unable to create ' + Path^ + Extension + ', rc=' + Long2Str(IOResult));
    end;
  end;
 var
  Flavour: TobFlavour;
 begin
  SetFiles:=True;

  for Flavour:=Low(Flavour) to High(Flavour) do
   SetList(FlavourExtension(Flavour), Flavour);
 end;

function TBSO.Capabilities: Longint;
 begin
  Capabilities:=ocSetFiles;
 end;

function TBSO.IsValidFlavour(const Flavour: TobFlavour): Boolean;
 begin
  IsValidFlavour:=Flavour in ob[omBSO].Flavours;
 end;

function TBSO.NetmailPacketName: String;
 begin
  NetmailPacketName:=Path^ + NetmailExtension(Session^.Flavour);
 end;

{ public methods }

function TBSO.FlavourExtension(const Flavour: TobFlavour): String;
 begin
  case Flavour of
   ofHold: FlavourExtension:='hlo';
   ofNormal, ofUnknown: FlavourExtension:='flo';
   ofDirect: FlavourExtension:='dlo';
   ofCrash: FlavourExtension:='clo';
   ofImmediate: FlavourExtension:='ilo';
  else
   FlavourExtension:='';
  end;
 end;

function TBSO.NetmailExtension(const Flavour: TobFlavour): String;
 begin
  case Flavour of
   ofHold: NetmailExtension:='hut';
   ofNormal, ofUnknown: NetmailExtension:='out';
   ofDirect: NetmailExtension:='dut';
   ofCrash: NetmailExtension:='cut';
   ofImmediate: NetmailExtension:='iut';
  else
   NetmailExtension:='';
  end;
 end;

{ TBoxOutbound }

function TBoxOutbound.GetBusy: Boolean;
 begin
  GetBusy:=False;
 end;

function TBoxOutbound.SetBusy(const Busy: Boolean): Boolean;
 begin
  SetBusy:=True;
 end;

function TBoxOutbound.GetFiles: POutboundFiles;
 var
  Files: POutboundFiles;
  SR: SearchRec;
 begin
  Files:=New(POutboundFiles, Init($10, $10));

  FindFirst(Root^ + '*.*', AnyFile, SR);

  while DosError = 0 do
   begin
    if SR.Attr and (Directory or VolumeID) = 0 then
     Files^.Insert(New(POutboundFile, Init(Session^.Flavour, ofmKill, Root^ + SR.Name)));

    FindNext(SR);
   end;

  FindClose(SR);

  GetFiles:=Files;
 end;

function TBoxOutbound.Capabilities: Longint;
 begin
  Capabilities:=ocNone;
 end;

{ TTBOX }

function TTBOX.InitRoot(var ARoot: String): Boolean;
 begin
  if inherited InitRoot(ARoot) then
   begin
    ARoot:=ARoot + Duotricenary(Session^.ToAddress.Zone, 2) +
                   Duotricenary(Session^.ToAddress.Net, 3) +
                   Duotricenary(Session^.ToAddress.Node, 3) + '.' +
                   Duotricenary(Session^.ToAddress.Point, 2);

    if Session^.Flavour = ofHold then
     ARoot:=ARoot + 'H\'
    else
     ARoot:=ARoot + '\';

    InitRoot:=jt.CreateDirectory(ARoot);
   end
  else
   InitRoot:=False;
 end;

function TTBOX.IsValidFlavour(const Flavour: TobFlavour): Boolean;
 begin
  IsValidFlavour:=(Flavour = ofHold) or (Flavour = ofNormal);
 end;

{ TLBSO }

{$IFDEF LFN}
function TLBSO.InitRoot(var ARoot: String): Boolean;
 begin
  InitRoot:=TOutbound.InitRoot(ARoot) and jt.CreateDirectory(ARoot);
 end;

function TLBSO.InitPath(var APath: String): Boolean;
 begin
  if Domain = nil then
   APath:=Root^ + 'fidonet'
  else
   APath:=Root^ + Domain^;

  APath:=APath + '.' + Long2Str(Session^.ToAddress.Zone) + '.' +
                       Long2Str(Session^.ToAddress.Net) + '.' +
                       Long2Str(Session^.ToAddress.Node) + '.' +
                       Long2Str(Session^.ToAddress.Point) + '.';

  InitPath:=True;
 end;

function TLBSO.GetBusy: Boolean;
 begin
  GetBusy:=ExistFile(Path^ + 'Busy');
 end;

function TLBSO.SetBusy(const Busy: Boolean): Boolean;
 begin
  if Busy then
   if TouchFile(Path^ + 'Busy') then
    SetBusy:=True
   else
    begin
     jt.Report('Failed to touch ' + Path^ + 'Busy', True);

     SetBusy:=False;
    end
  else
   if EraseFile(Path^ + 'Busy') then
    begin
     jt.Report('Failed to touch ' + Path^ + 'Busy', True);

     SetBusy:=False;
    end;
 end;

function TLBSO.IsValidFlavour(const Flavour: TobFlavour): Boolean;
 begin
  IsValidFlavour:=Flavour in ob[omLBSO].Flavours;
 end;

{ public methods }

function TLBSO.FlavourExtension(const Flavour: TobFlavour): String;
 begin
  case Flavour of
   ofHold,
   ofNormal,
   ofDirect,
   ofCrash,
   ofImmediate: FlavourExtension:=MapBrakeFlavour(Flavour) + '.List';
  else
   FlavourExtension:='';
  end;
 end;

function TLBSO.NetmailExtension(const Flavour: TobFlavour): String;
 begin
  case Flavour of
   ofHold,
   ofNormal,
   ofDirect,
   ofCrash,
   ofImmediate: NetmailExtension:=MapBrakeFlavour(Flavour) + '.Mail';
  else
   NetmailExtension:='';
  end;
 end;
{$ENDIF}

{ TTLBOX }

{$IFDEF LFN}
function TTLBOX.InitRoot(var ARoot: String): Boolean;
 begin
  if inherited InitRoot(ARoot) then
   begin
    ARoot:=ARoot + Long2Str(Session^.ToAddress.Zone) + '.' +
                   Long2Str(Session^.ToAddress.Net) + '.' +
                   Long2Str(Session^.ToAddress.Node) + '.' +
                   Long2Str(Session^.ToAddress.Point);

    if Session^.Flavour = ofHold then
     ARoot:=ARoot + '.H\'
    else
     ARoot:=ARoot + '\';

    InitRoot:=jt.CreateDirectory(ARoot);
   end
  else
   InitRoot:=False;
 end;

function TTLBOX.IsValidFlavour(const Flavour: TobFlavour): Boolean;
 begin
  IsValidFlavour:=Flavour in ob[omTLBOX].Flavours;
 end;
{$ENDIF}

{ TLBOX }

{$IFDEF LFN}
function TLBOX.InitRoot(var ARoot: String): Boolean;
 begin
  if inherited InitRoot(ARoot) then
   begin
    if Domain = nil then
     ARoot:=ARoot + 'fidonet.'
    else
     ARoot:=ARoot + Domain^ + '.';

    ARoot:=ARoot + Long2Str(Session^.ToAddress.Zone) + '.' +
                   Long2Str(Session^.ToAddress.Net) + '.' +
                   Long2Str(Session^.ToAddress.Node) + '.' +
                   Long2Str(Session^.ToAddress.Point) + '.' +
                   MapBrakeFlavour(Session^.Flavour) + '\';

    InitRoot:=jt.CreateDirectory(ARoot);
   end
  else
   InitRoot:=False;
 end;

function TLBOX.IsValidFlavour(const Flavour: TobFlavour): Boolean;
 begin
  IsValidFlavour:=Flavour in ob[omTLBOX].Flavours;
 end;
{$ENDIF}

{ TASO }

{$IFDEF LFN}
function TASO.InitRoot(var ARoot: String): Boolean;
 begin
  InitRoot:=TOutbound.InitRoot(ARoot) and jt.CreateDirectory(ARoot);
 end;

function TASO.InitPath(var APath: String): Boolean;
 begin
  APath:=Root^ + Long2Str(Session^.ToAddress.Zone) + '.' +
                 Long2Str(Session^.ToAddress.Net) + '.' +
                 Long2Str(Session^.ToAddress.Node) + '.' +
                 Long2Str(Session^.ToAddress.Point);

  InitPath:=True;
 end;

function TASO.IsValidFlavour(const Flavour: TobFlavour): Boolean;
 begin
  IsValidFlavour:=Flavour in ob[omASO].Flavours;
 end;
{$ENDIF}

{ TBOX }

function TBOX.InitRoot(var ARoot: String): Boolean;
 begin
  ARoot:=AddBackSlash(PString(jt.st.Get(jt.om.l.st(Session^.oml), obomlDirectory))^);

  InitRoot:=jt.CreateDirectory(ARoot);
 end;

function TBOX.IsValidFlavour(const Flavour: TobFlavour): Boolean;
 begin
  IsValidFlavour:=Flavour in ob[omBOX].Flavours;
 end;

{ TOutboundErrors }

procedure TOutboundErrors.Add(const S: String);
 begin
  if S <> '' then
   Insert(NewStr(S));
 end;

end.