unit OutbQue;

interface
uses
     Objects,
     Dos,

     ccAPI,

     OutbCmn,
     OutbSess,
     OutbOB,

     Wizard,

     jtMHLcmn,

     PlugAPI,
     PlugSIDE,
     PlugUTIL;

procedure FlushQueue;

function stLinkQueue(const Action: TStorageAction; const Data: Pointer): Pointer; export;

implementation

type
 PLinkQueueItem = ^TLinkQueueItem;
 TLinkQueueItem = object(TObject)
  DateTime, Size, Flags: Longint;
  FName: PString;

  constructor Init(const ADateTime, ASize, AFlags: Longint; const AFName: String);
  destructor Done; virtual;
 end;

 PLinkQueueOML = ^TLinkQueueOML;
 TLinkQueueOML = object(TSortedCollection)
  oml: Pointer;

  constructor Init(const Aoml: Pointer);
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  function KeyOf(Key: Pointer): Pointer; virtual;
 end;

 PLinkQueue = ^TLinkQueue;
 TLinkQueue = object(TSortedCollection)
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  function KeyOf(Key: Pointer): Pointer; virtual;
  function Seek(const oml: Pointer): PLinkQueueOML;
 end;

{ TLinkQueueItem }

constructor TLinkQueueItem.Init(const ADateTime, ASize, AFlags: Longint; const AFName: String);
 begin
  inherited Init;

  DateTime:=ADateTime;
  Size:=ASize;
  Flags:=AFlags;
  FName:=NewStr(AFName);
 end;

destructor TLinkQueueItem.Done;
 begin
  DisposeStr(FName);

  inherited Done;
 end;

{ TLinkQueueOML }

constructor TLinkQueueOML.Init(const Aoml: Pointer);
 begin
  inherited Init($10, $10);

  oml:=Aoml;

  Duplicates:=True;
 end;

function TLinkQueueOML.Compare(Key1, Key2: Pointer): xInteger;
 begin
  Compare:=CompareLongint(Longint(Key1), Longint(Key2));

{
  if Longint(Key1) < Longint(Key2) then Compare:=-1 else
  if Longint(Key1) > Longint(Key2) then Compare:=1 else
   Compare:=0;
}
 end;

function TLinkQueueOML.KeyOf(Key: Pointer): Pointer;
 begin
  KeyOf:=Pointer(PLinkQueueItem(Key)^.DateTime);
 end;

{ TLinkQueue }

function TLinkQueue.Compare(Key1, Key2: Pointer): xInteger;
 begin
  Compare:=CompareLongint(Longint(Key1), Longint(Key2));

{
  if Longint(Key1) < Longint(Key2) then Compare:=-1 else
  if Longint(Key1) > Longint(Key2) then Compare:=1 else
   Compare:=0;
}
 end;

function TLinkQueue.KeyOf(Key: Pointer): Pointer;
 begin
  KeyOf:=PLinkQueueOML(Key)^.oml;
 end;

function TLinkQueue.Seek(const oml: Pointer): PLinkQueueOML;
 var
  Index: xInteger;
  lqoml: PLinkQueueOML;
 begin
  if Search(oml, Index) then
   Seek:=At(Index)
  else
   begin
    lqoml:=New(PLinkQueueOML, Init(oml));

    AtInsert(Index, lqoml);

    Seek:=lqoml;
   end;
 end;

{ stLinkQueue }

function stLinkQueue(const Action: TStorageAction; const Data: Pointer): Pointer;
 var
  lq: PLinkQueue absolute Data;
 begin
  case Action of
   saDestroy: Dispose(lq, Done);
   saClone: stLinkQueue:=nil;
   saInfo: stLinkQueue:=NewStr('linkqueue');
  end;
 end;

{ PerformPack }

procedure PerformPack;
 var
  FromAddress, ToAddress: TAddress;
  Password: String;
 function PackOML(const lqoml: PLinkQueueOML): Boolean; far;
  var
   CurrentSize, TotalSize: Longint;
   Session: PSession;
  procedure CountTotalSize(const lqi: PLinkQueueItem); far;
   begin
    Inc(TotalSize, lqi^.Size);
   end;
  function PackItem(const lqi: PLinkQueueItem): Boolean; far;
   var
    Percent: String;
   begin
    if jt.Cancelled then
     PackItem:=True
    else
     begin
      PackItem:=False;

      Percent:=jt.Percent(CurrentSize, TotalSize);

      jt.SetTitle(AddressToStrEx(ToAddress) + ' ' + Percent + ' (packing)');

      jt.SetStatus('packing for ' + AddressToStrEx(ToAddress) + ' [' + Percent + ']');

      Session^.Outprocess(lqi^.FName^, lqi^.Flags and osfPackEchomail <> 0);

      Inc(CurrentSize, lqi^.Size);
     end;
   end;
  begin
   PackOML:=False;

   TotalSize:=0;
   CurrentSize:=0;

   lqoml^.ForEach(@CountTotalSize);

   Session:=New(PSession, Init(lqoml^.oml, FromAddress, ToAddress, Password));

   if Session^.GetBusy then
    if not LongBool(jt.st.Get(jt.st.Globalst, 'OutboundQueueNotifiedBusy.' + AddressToStrEx(ToAddress))) then
     begin
      jt.LogWrite(obLetterQueue, AddressToStrEx(ToAddress) + ' busy, ' + Long2Str(lqoml^.Count) + ' packet' +
       _s(lqoml^.Count) + ' ' + Size2KB(TotalSize) + ' holded');

      jt.st.Add(jt.st.Globalst, 'OutboundQueueNotifiedBusy.' + AddressToStrEx(ToAddress), Pointer(1), nil);
     end
    else
   else
    if Session^.SetBusy(True) then
     begin
      PackOML:=lqoml^.FirstThat(@PackItem) <> nil;

      Session^.SetBusy(False);
     end;

   Dispose(Session, Done);
  end;
 var
  Link: Pointer;
  lq: PLinkQueue;
  K: Longint;
 begin
  for K:=0 to jt.link.Count - 1 do
   begin
    Link:=jt.link.At(K);

    lq:=jt.st.Get(jt.link.st(Link), obLinkQueue);

    if (lq = nil) or (lq^.Count = 0) then
     Continue;

    jt.link.GetAka(Link, FromAddress);
    jt.link.GetAddress(Link, ToAddress);
    jt.link.GetOutPassword(Link, Password);

    lq^.FirstThat(@PackOML);

    jt.st.Remove(jt.link.st(Link), obLinkQueue);
   end;

  jt.ClearStatus;
 end;

{ FlushQueue }

procedure FlushQueue;
 var
  FormatReader: PFormatReader;
  Reader, Link: Pointer;
  SR: SearchRec;
  FromAddress, ToAddress: TAddress;
  rc, Password: String;
  Flags: Longint;
  Kill: Boolean;
 procedure RenameToBad(const Reason: String);
  var
   Target: String;
   rc: Longint;
  begin
   if Reader <> nil then
    begin
     FormatReader^.Close(Reader);

     Reader:=nil;
    end;

   Target:=PacketsQueue + ForceExtension(SR.Name, 'bad');

   if not DistortFileExtension(Target) then
    jt.Report('Unable to distort filename ' + Target + ' (all namespace exhaused?)', True)
   else
    begin
     jt.MoveFile(PacketsQueue + SR.Name, Target, rc);

     if rc <> 0 then
      jt.Report('Unable to rename ' + PacketsQueue + SR.Name + ' to ' + Target + ', rc=' + Long2Str(rc), True)
     else
      jt.LogWrite(obLetterQueue, PacketsQueue + SR.Name + ' >>> ' + Target + ' (' + Reason + ')');
    end;
  end;
 procedure DoNetmail(const oml: Pointer);
  var
   Session: PSession;
   Target, rc, Reason: String;
   Appender, msg: Pointer;
   Bad: Boolean;
  begin
   Session:=New(PSession, Init(oml, FromAddress, ToAddress, Password));

   if Session <> nil then
    begin
     if not Session^.GetBusy and Session^.SetBusy(True) then
      begin
       Target:=POutbound(Session^.ob)^.NetmailPacketName;

       if Target = '' then
        begin
         Target:=POutbound(Session^.ob)^.Root^ + SR.Name;

         while ExistFile(Target) do
          Target:=POutbound(Session^.ob)^.Root^ + GenerateMSGID + '.' + Session^.Writer^.Extension^;
        end;

       if ExistFile(Target) then
        Appender:=Session^.Writer^.Append(Target, rc)
       else
        Appender:=Session^.Writer^.Create(Target, rc, FromAddress, ToAddress, Password, Session^.Flags or osfNetmail);

       if Appender = nil then
        jt.Report('Unable to append netmail packet - failed to append/create ' + Target + ' (' +
         Session^.Writer^.Name^ + '): ' + rc, True)
       else
        begin
         Bad:=False;

         repeat
          if not FormatReader^.Read(Reader, msg, rc) then
           begin
            jt.Report('Unable to append netmail packet - failed to read ' + PacketsQueue + SR.Name + ' (' +
             FormatReader^.Name^ + '): ' + rc, True);

            Bad:=True;

            Reason:='unable to read ' + PacketsQueue + SR.Name;

            Break;
           end;

          if msg = nil then
           begin
            Kill:=True;

            Break;
           end;

          if not Session^.Writer^.Write(Appender, msg, rc) then
           begin
            jt.Report('Unable to append netmail packet - failed to write ' + Target + ' (' +
             Session^.Writer^.Name^ + '): ' + rc, True);

            jt.msg.Destroy(msg);

            Bad:=True;

            Reason:='unable to write ' + Target;

            Break;
           end;

          jt.msg.Destroy(msg);
         until False;

         Session^.Writer^.Close(Appender);

         if Bad then
          RenameToBad(Reason);
        end;

       POutbound(Session^.ob)^.SetBusy(False);
      end;

     Dispose(Session, Done);
    end;
  end;
 const
  First: Boolean = True;
 var
  F: Text;
  lq: PLinkQueue;
  lqoml: PLinkQueueOML;
  oml: Pointer;
 begin
  if HoldPack then
   Exit;

  jt.PushTitle;

  jt.SetTitle('checking pq');

  FindFirst(PacketsQueue + '*.*', AnyFile, SR);

  while DosError = 0 do
   begin
    if (SR.Attr and (Dos.Directory or Dos.VolumeID) = 0) and not FileLocked(PacketsQueue + SR.Name) and
       not obSessions^.Using(PacketsQueue + SR.Name) then
     begin
      if First then
       begin
        if ccAvailable then
         ccoaQueueBeforePack^.Execute;

        First:=False;
       end;

      jt.SetStatus('checking pq ' + SR.Name + '...');

      FormatReader:=IdentifyReaderNoExtCare(PacketsQueue + SR.Name);
      Reader:=nil;

      if FormatReader = nil then
       RenameToBad('unknown format')
      else
       if FormatReader^.IsOur(PacketsQueue + SR.Name) then
        begin
         Reader:=FormatReader^.Open(PacketsQueue + SR.Name, rc, FromAddress, ToAddress, Password, Flags);

         if Reader = nil then
          RenameToBad('open failed: ' + rc)
         else
          begin
           if not jt.core.IsOurAddress(FromAddress) then
            RenameToBad('packet not from us')
           else
            begin
             Link:=jt.link.Seek(ToAddress);

             if Link = nil then
              RenameToBad('packet to unknown link')
             else
              begin
               Kill:=False;

               oml:=jt.om.l.SeekByHash(Flags shr 16);

               if oml = nil then
                oml:=jt.link.oml(Link);

               if Flags and osfNetmail <> 0 then
                DoNetmail(oml)
               else
                begin
                 lq:=jt.st.Get(jt.link.st(Link), obLinkQueue);

                 if lq = nil then
                  begin
                   lq:=New(PLinkQueue, Init($10, $10));

                   jt.st.Add(jt.link.st(Link), obLinkQueue, lq, stLinkQueue);
                  end;

                 lqoml:=lq^.Seek(oml);

                 lqoml^.Insert(New(PLinkQueueItem, Init(SR.Time, SR.Size, Flags, PacketsQueue + SR.Name)));
                end;

               if Reader <> nil then
                begin
                 FormatReader^.Close(Reader);

                 if Kill then
                  begin
                   if IOResult <> 0 then;

                   Assign(F, PacketsQueue + SR.Name);

                   Erase(F);

                   if InOutRes <> 0 then
                    jt.Report('Unable to erase ' + PacketsQueue + SR.Name + ', rc=' + Long2Str(IOResult), True);
                  end;
                end;
              end;
            end;
          end;
        end;
     end;

    FindNext(SR);
   end;

  FindClose(SR);

  jt.ClearStatus;

  if not jt.Cancelled then
   PerformPack;

  jt.PopTitle;
 end;

end.
