library Packer;

uses
     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     Objects,
     Strings,

     jtMHLcmn,

     PackPack,

     PrioAPI,

     Wizard;

const
 PackerName                             = 'Packer';
 PackerVersion                          = 1;

 PackerPlugin           : Pointer       = nil;

 PackFirst              : Pointer       = nil;
 PackLast               : Pointer       = nil;
 PackDisable            : Pointer       = nil;
 PackNetmail            : Pointer       = nil;

 PackAreas              : PAreaList     = nil;

 PackPerform                            = $1000;

 PackHaveSense          : Boolean       = False;

 pfNone                                 = $0000;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 var
  ProhibitFormats: PNumbers;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, PackerName, PackerVersion);

  PackerPlugin:=Plugin;

  PackFirst:=jt.rests.Add('PackFirst', rsMinus);
  PackLast:=jt.rests.Add('PackLast', rsMinus);
  PackDisable:=jt.rests.Add('PackDisable', rsMinus);
  PackNetmail:=jt.rests.Add('PackNetmail', rsMinus);

  ProhibitFormats:=New(PNumbers, Init($2, $4));
  ProhibitFormats^.Add(afPassthrough);

  PackAreas:=New(PAreaList, Init(PackFirst, PackLast, PackDisable, ProhibitFormats, 0));

  PluginSetup:=True;

  RegisterPriorityAction('PACKING');
 end;

{ PluginStartup }

procedure PluginStartup; export;
 begin
  if PackHaveSense then
   jt.EnqueueCommand(PackerPlugin, PackPerform, 0, cwPurge);
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
  Dispose(PackAreas, Done);
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 const
  FirstTime: Boolean = True;
 function PerformPack(const Area: PAreaListItem): Boolean; far;
  var
   Format: TAreaFormat;
   BeforePack, AfterPack: Longint;
  begin
   if FirstTime then
    begin
     jt.LogWrite('P', 'Packing areas');

     FirstTime:=False;
    end;

   { // pack netmail }

   jt.area.GetFormat(Area^.Area, Format);

   if (Format.Style = asNetmail) and not jt.rests.Check(PackNetmail, nil, jt.area.GetName(Area^.Area)) then
    begin
     PerformPack:=jt.Cancelled;

     Exit;
    end;

   if not jt.Cancelled then
    if PackArea(Area^.Area, BeforePack, AfterPack, PackAreas^.IndexOf(Area), PackAreas^.Count) then
     begin
      if BeforePack <> AfterPack then
       jt.LogWrite('P', jt.area.GetName(Area^.Area)^ + ': ' + jt.Percent(BeforePack - AfterPack, BeforePack) +
        ' packed (' + Size2KB(BeforePack) + ' -> ' + Size2KB(AfterPack) + ')');

      PerformPack:=jt.Cancelled;
     end
    else
     PerformPack:=jt.Cancelled
   else
    PerformPack:=True;
  end;
 begin
  if Command = PackPerform then
   begin
    if PackAreas^.Full then
     PackAreas^.AddList(jt.list.ImportList, PackAreas^.FullFlags);

    if PackAreas^.HaveSense then
     PackAreas^.Prepare;

    if PackAreas^.Count = 0 then
     jt.LogWrite('P', #0'Nothing to pack.')
    else
     begin
      jt.PushTitle;

      PackAreas^.FirstThat(@PerformPack);

      jt.PopTitle;
     end;
   end;
 end;

{ PluginHandleCmdlineSwitch }

function PluginHandleCmdlineSwitch(Switch: String): Boolean; export;
 procedure Full(const Flags: Longint);
  begin
   PackAreas^.AddFull(Flags, jt.linker.Stub);

   PackHaveSense:=True;
  end;
 procedure Mask(const Skip: Byte; const Flags: Longint);
  begin
   PackAreas^.AddMask(Copy(Switch, Skip + 1, 255), Flags, jt.linker.Stub);

   PackHaveSense:=True;
  end;
 begin
  StUpcaseEx(Switch);

  PluginHandleCmdlineSwitch:=True;

  if Switch = 'PACK' then Full(pfNone) else
  if Switch = 'PACKFORCE' then Full(alifIgnoreDisable) else
  if Copy(Switch, 1, 5) = 'PACK:' then Mask(5, pfNone) else
  if Copy(Switch, 1, 10) = 'PACKFORCE:' then Mask(10, alifIgnoreDisable) else
   PluginHandleCmdlineSwitch:=False;
 end;

{ PluginShowHelp }

procedure PluginShowHelp; export;
 begin
  jt.PrintStr(' PACK[:re]               pack importlist or regexp if not omitted'#13#10);
  jt.PrintStr(' PACKFORCE[:re]          pack, PurgeDisable have no influence'#13#10);
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleCmdlineSwitch,
 PluginShowHelp;

begin
 Assign(Output, '');
 Rewrite(Output);
end.
