unit PlugUTIL;

interface
uses
     PlugSIDE,
     PlugAPI,

     jtMHLcmn,

     Objects,

     Wizard;

type
 PInbound = ^TInbound;
 TInbound = record
  Source: Longint;
  Directory: PString;
 end;

 PInbounds = ^TInbounds;
 TInbounds = object(TCollection)
  constructor Init;
  procedure FreeItem(Item: Pointer); virtual;
 private
  procedure Add(const Source: Longint; const Directory: PString);
 end;

 POpenedSession = ^TOpenedSession;
 TOpenedSession = object(TObject)
  FromAddress, ToAddress: TAddress;
  Password: PString;
  oms: Pointer;

  constructor Init(const AFromAddress, AToAddress: TAddress; const APassword: String; const Aoms: Pointer);
  destructor Done; virtual;
 end;

 POpenedSessions = ^TOpenedSessions;
 TOpenedSessions = object(TCollection)
  function Seek(const AFromAddress, AToAddress: TAddress; const APassword: String): POpenedSession;
 end;

 TDebugger = object
  Enabled: Boolean;
  Prefix: String[20];

  procedure Log(const Message: String);
 end;

 TRoller = object
  Pos: Byte;

  function GetRoller: Char;
 end;

 PNumbers = ^TNumbers;
 TNumbers = object(TSortedCollection)
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  procedure FreeItem(Item: Pointer); virtual;
 public
  SubmitAll: Boolean;

  procedure Add(const Number: Longint);
  function IsIn(const Number: Longint): Boolean;
  procedure Intersect(const Another: PNumbers);
 end;

 PAreaListMask = ^TAreaListMask;
 TAreaListMask = object(TObject)
  re: Pointer;
  Flags: Longint;

  constructor Init(const Are: String; const AFlags: Longint; const Linker: Pointer);
  destructor Done; virtual;
 end;

 PAreaListItem = ^TAreaListItem;
 TAreaListItem = object(TObject)
  Area: Pointer;
  Order: Byte;
  Flags: Longint;

  constructor Init(const AArea: Pointer; const AOrder: Byte; const AFlags: Longint);
 end;

 PAreaList = ^TAreaList;
 TAreaList = object(TSortedCollection)
  Masks: PCollection;
  First, Last, Disable: Pointer;
  Added, Full: Boolean;
  ProhibitFormats: PNumbers;
  ProhibitStyles: Longint;
  FullFlags: Longint;

  constructor Init(const AFirst, ALast, ADisable: Pointer; const AProhibitFormats: PNumbers; const AProhibitStyles: Longint);
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  destructor Done; virtual;
 public
  procedure AddMask(const Mask: String; const Flags: Longint; const Linker: Pointer);
  procedure AddFull(const Flags: Longint; const Linker: Pointer);
  procedure AddList(const List: Pointer; const Flags: Longint);
  function HaveSense: Boolean;
  procedure Prepare;
  procedure SetAdditionalFlags(const Area: Pointer; const AreaName: String; var Flags: Longint); virtual;
 end;

 PRegExpList = ^TRegExpList;
 TRegExpList = object(TCollection)
  procedure Add(const S: String; const Linker: Pointer);
  function Match(const S: String; const CaseSensitive: Boolean): Boolean;
  procedure FreeItem(Item: Pointer); virtual;
 end;

const
 alioFirst              = $01;
 alioNormal             = $02;
 alioLast               = $03;

 alifIgnoreDisable      = $10000000;

 Rollers                : array[1..4] of Char = '|/-\';

function IdentifyReader(const FName: String): PFormatReader; {*}
function IdentifyReaderNoExtCare(const FName: String): PFormatReader; {*}
function AddressOfLink(const Link: Pointer): String; {*}

implementation

{ TInbounds }

constructor TInbounds.Init;
 var
  K: Longint;
 begin
  inherited Init($10, $10);

  for K:=0 to Longint(jt.core.GetInbound(-1)) - 1 do
   Add(tsInbound, jt.core.GetInbound(K));

  Add(tsLocalInbound, jt.core.GetLocalInbound);
  Add(tsTempInbound, jt.core.GetTempInbound);
  Add(tsBaseInbound, jt.core.GetBaseInbound);
  Add(tsBadInbound, jt.core.GetBadInbound);
 end;

procedure TInbounds.FreeItem(Item: Pointer);
 begin
  if Item <> nil then
   Dispose(PInbound(Item));
 end;

{ private methods }

procedure TInbounds.Add(const Source: Longint; const Directory: PString);
 var
  Inbound: PInbound;
 begin
  New(Inbound);

  Inbound^.Source:=Source;
  Inbound^.Directory:=Directory;

  Insert(Inbound);
 end;

{ TOpenedSession }

constructor TOpenedSession.Init(const AFromAddress, AToAddress: TAddress; const APassword: String; const Aoms: Pointer);
 begin
  inherited Init;

  FromAddress:=AFromAddress;
  ToAddress:=AToAddress;
  Password:=NewStr(APassword);

  oms:=Aoms;
 end;

destructor TOpenedSession.Done;
 begin
  DisposeStr(Password);

  jt.om.s.Destroy(oms);

  inherited Done;
 end;

{ TOpenedSessions }

function TOpenedSessions.Seek(const AFromAddress, AToAddress: TAddress; const APassword: String): POpenedSession;
 function Lookup(const Session: POpenedSession): Boolean; far;
  begin
   Lookup:=AddressEqu(Session^.FromAddress, AFromAddress) and
           AddressEqu(Session^.ToAddress, AToAddress) and
           (APassword = GetPString(Session^.Password));
  end;
 begin
  Seek:=FirstThat(@Lookup);
 end;

{ IdentifyReader }

function IdentifyReader(const FName: String): PFormatReader;
 var
  Reader: PFormatReader;
  K: Longint;
 begin
  for K:=0 to jt.fmt.r.Count - 1 do
   begin
    Reader:=jt.fmt.r.At(K);

    if Reader^.IsOur(FName) and Reader^.Ident(FName) then
     begin
      IdentifyReader:=Reader;

      Exit;
     end;
   end;

  IdentifyReader:=nil;
 end;

{ IdentifyReaderNoExtCare }

function IdentifyReaderNoExtCare(const FName: String): PFormatReader;
 var
  Reader: PFormatReader;
  K: Longint;
 begin
  for K:=0 to jt.fmt.r.Count - 1 do
   begin
    Reader:=jt.fmt.r.At(K);

    if Reader^.Ident(FName) then
     begin
      IdentifyReaderNoExtCare:=Reader;

      Exit;
     end;
   end;

  IdentifyReaderNoExtCare:=nil;
 end;

{ AddressOfLink }

function AddressOfLink(const Link: Pointer): String;
 var
  Address: TAddress;
 begin
  if Link = nil then
   AddressOfLink:='?'
  else
   begin
    jt.link.GetAddress(Link, Address);

    AddressOfLink:=AddressToStrEx(Address);
   end;
 end;

{ TDebugger }

procedure TDebugger.Log(const Message: String);
 begin
  if Enabled and (@jt.LogWrite <> nil) then
   jt.LogWrite('~', Prefix + ': ' + Message);
 end;

{ TRoller }

function TRoller.GetRoller: Char;
 begin
  if Pos > High(Rollers) then
   Pos:=Low(Rollers)
  else
   if Pos < Low(Rollers) then
    Pos:=Low(Rollers);

  GetRoller:=Rollers[Pos];

  Inc(Pos);
 end;

{ TNumbers }

function TNumbers.Compare(Key1, Key2: Pointer): xInteger;
 var
  K1: Longint absolute Key1;
  K2: Longint absolute Key2;
 begin
  Compare:=CompareLongint(K1, K2);
 
{
  if K1 < K2 then Compare:=-1 else
  if K1 > K2 then Compare:=1 else
   Compare:=0;
}
 end;

procedure TNumbers.FreeItem(Item: Pointer);
 begin
 end;

{ public methods }

procedure TNumbers.Add(const Number: Longint);
 var
  Index: xInteger;
 begin
  if not Search(Pointer(Number), Index) then
   AtInsert(Index, Pointer(Number));
 end;

function TNumbers.IsIn(const Number: Longint): Boolean;
 var
  Index: xInteger;
 begin
  IsIn:=SubmitAll or Search(Pointer(Number), Index);
 end;

procedure TNumbers.Intersect(const Another: PNumbers);
 var
  K: xInteger;
 begin
  K:=0;

  while K < Count do
   if Another^.IsIn(Longint(At(K))) then
    Inc(K)
   else
    AtDelete(K);
 end;

{ TAreaListMask }

constructor TAreaListMask.Init(const Are: String; const AFlags: Longint; const Linker: Pointer);
 begin
  inherited Init;

  if Are = '' then
   jt.CompileRegExp('ANY', re, Linker, True)
  else
   jt.CompileRegExp(Are, re, Linker, False);

  if re = nil then
   Fail;

  Flags:=AFlags;
 end;

destructor TAreaListMask.Done;
 begin
  jt.DestroyRegExp(re);

  inherited Done;
 end;

{ TAreaListItem }

constructor TAreaListItem.Init(const AArea: Pointer; const AOrder: Byte; const AFlags: Longint);
 begin
  inherited Init;

  Area:=AArea;
  Order:=AOrder;
  Flags:=AFlags;
 end;

{ TAreaList }

constructor TAreaList.Init(const AFirst, ALast, ADisable: Pointer; const AProhibitFormats: PNumbers;
 const AProhibitStyles: Longint);
 begin
  inherited Init($10, $100);

  Masks:=New(PCollection, Init($2, $4));

  First:=AFirst;
  Last:=ALast;
  Disable:=ADisable;

  ProhibitFormats:=AProhibitFormats;
  ProhibitStyles:=AProhibitStyles;

  Added:=False;
  Full:=False;
  FullFlags:=0;
 end;

function TAreaList.Compare(Key1, Key2: Pointer): xInteger;
 var
  K1: PAreaListItem absolute Key1;
  K2: PAreaListItem absolute Key2;
  S1: PString;
  S2: PString;
 begin
  if K1^.Order < K2^.Order then Compare:=-1 else
  if K1^.Order > K2^.Order then Compare:=1 else
   begin
    S1:=jt.area.GetName(K1^.Area);
    S2:=jt.area.GetName(K2^.Area);

    if S1^ < S2^ then Compare:=-1 else
    if S1^ > S2^ then Compare:=1 else
     Compare:=0;
   end;
 end;

destructor TAreaList.Done;
 begin
  Dispose(Masks, Done);
  Dispose(ProhibitFormats, Done);

  inherited Done;
 end;

{ public methods }

procedure TAreaList.AddMask(const Mask: String; const Flags: Longint; const Linker: Pointer);
 begin
  Masks^.Insert(New(PAreaListMask, Init(Mask, Flags, Linker)));

  Added:=True;
 end;

procedure TAreaList.AddFull(const Flags: Longint; const Linker: Pointer);
 begin
  Full:=True;

  Added:=True;

  FullFlags:=Flags;
 end;

procedure TAreaList.AddList(const List: Pointer; const Flags: Longint);
 var
  K: Longint;
  S: String;
  Mask: PAreaListMask;
 begin
  for K:=0 to jt.list.Count(List) - 1 do
   begin
    S:=jt.list.At(List, K);

    if S <> '' then
     begin
      Mask:=New(PAreaListMask, Init(jt.re.PreprocessWildcard(S), Flags, nil));

      if Mask <> nil then
       begin
        Masks^.Insert(Mask);

        Added:=True;
       end;
     end;
   end;
 end;

function TAreaList.HaveSense: Boolean;
 begin
  HaveSense:=Added;
 end;

procedure TAreaList.Prepare;
 var
  AreaName: String;
 function CheckMask(const Mask: PAreaListMask): Boolean; far;
  begin
   CheckMask:=jt.re.MatchStr(Mask^.re, AreaName, False);
  end;
 var
  K, Flags: Longint;
  Area: Pointer;
  Format: TAreaFormat;
  Mask: PAreaListMask;
  Order: Byte;
 begin
  for K:=0 to jt.area.Count - 1 do
   begin
    Area:=jt.area.At(K);

    jt.area.GetFormat(Area, Format);

    GetPStringEx(jt.area.GetName(Area), AreaName);

    if (not ProhibitFormats^.IsIn(Format.Format)) and
       (Format.Style and ProhibitStyles = 0) then
     begin
      Mask:=Masks^.FirstThat(@CheckMask);

      if (Mask <> nil) and ((Mask^.Flags and alifIgnoreDisable <> 0) or (Disable = nil) or
        not jt.rests.Check(Disable, nil, @AreaName)) then
       begin
        if (First <> nil) and jt.rests.Check(First, nil, @AreaName) then
         Order:=alioFirst
        else
         if (Last <> nil) and jt.rests.Check(Last, nil, @AreaName) then
          Order:=alioLast
         else
          Order:=alioNormal;

        Flags:=Mask^.Flags;

        SetAdditionalFlags(Area, AreaName, Flags);

        Insert(New(PAreaListItem, Init(Area, Order, Flags)));
       end;
     end;
   end;

  Masks^.FreeAll;
 end;

procedure TAreaList.SetAdditionalFlags(const Area: Pointer; const AreaName: String; var Flags: Longint);
 begin
 end;

{ TRegExpList }

procedure TRegExpList.Add(const S: String; const Linker: Pointer);
 var
  re: Pointer;
 begin
  jt.CompileRegExp(S, re, Linker, False);

  Insert(re);
 end;

function TRegExpList.Match(const S: String; const CaseSensitive: Boolean): Boolean;
 function Perform(const re: Pointer): Boolean; far;
  begin
   Perform:=jt.re.MatchStr(re, S, CaseSensitive);
  end;
 begin
  Match:=FirstThat(@Perform) <> nil;
 end;

procedure TRegExpList.FreeItem(Item: Pointer);
 begin
  jt.DestroyRegExp(Item);
 end;

end.