library Poster;

uses
     PlugAPI,
     PlugSIDE,

     Objects,
     Strings,

     jtMHLcmn,

     Wizard;

type
 PPosterAttribute = ^TPosterAttribute;
 TPosterAttribute = packed record
  Attribute: Longint;
  Name: PChar;
 end;

 PPost = ^TPost;
 TPost = object(TObject)
  AreaName, FromName, ToName, Subject, FName: PString;
  FromAddress, ToAddress: TAddress;
  Flags, Attributes: Longint;

  constructor Init(Value: String; const Linker: Pointer);
  procedure Perform;
  destructor Done; virtual;
 end;

const
 Posts          : PCollection           = nil;

 PosterPlugin   : Pointer               = nil;

 PosterPerform                          = $1000;

 pfNone                                 = $00000000;
 pfKillFile                             = $00000001;

 PosterAttributes: array[1..17] of TPosterAttribute = (
  (Attribute: maPrivate;  Name: 'PVT'),
  (Attribute: maCrash;    Name: 'CRA'),
  (Attribute: maReceived; Name: 'RCV'),
  (Attribute: maSent;     Name: 'SNT'),
  (Attribute: maAttach;   Name: 'ATT'),
  (Attribute: maTransit;  Name: 'TRS'),
  (Attribute: maOrphan;   Name: 'ORP'),
  (Attribute: maKill;     Name: 'K/S'),
  (Attribute: maLocal;    Name: 'LOC'),
  (Attribute: maHold;     Name: 'HLD'),
  (Attribute: maFRq;      Name: 'FRQ'),
  (Attribute: maRRq;      Name: 'RRQ'),
  (Attribute: maRRc;      Name: 'RRC'),
  (Attribute: maARq;      Name: 'ARQ'),
  (Attribute: maURq;      Name: 'URQ'),
  (Attribute: maScanned;  Name: 'SCN'),
  (Attribute: maRead;     Name: 'RDD'));

constructor TPost.Init(Value: String; const Linker: Pointer);
 var
  S: String;
  K: Longint;
 begin
  inherited Init;

  AreaName:=NewStr(jt.GetCommaToken(Value));

  if AreaName = nil then
   jt.linker.Error(Linker, 'POST: area name must be specified');

  FromName:=NewStr(jt.GetCommaToken(Value));

  S:=jt.GetCommaToken(Value);

  if (S <> '') and not StrToAddress(S, FromAddress) then
    jt.linker.Error(Linker, 'POST: invalid address "' + S + '"');

  ToName:=NewStr(jt.GetCommaToken(Value));

  if ToName = nil then
   ToName:=NewStr('All');

  S:=jt.GetCommaToken(Value);

  if (S <> '') and not StrToAddress(S, ToAddress) then
    jt.linker.Error(Linker, 'POST: invalid address "' + S + '"');

  Subject:=NewStr(jt.GetCommaToken(Value));

  FName:=NewStr(jt.GetCommaToken(Value));

  if FName = nil then
   jt.linker.Error(Linker, 'POST: filename must be specified');

  Flags:=pfNone;
  Attributes:=maNone;

  while Value <> '' do
   begin
    S:=jt.GetCommaToken(Value);

    StUpcaseEx(S);

    if S = '' then
     Continue;

    if S = 'KILL' then Flags:=Flags or pfKillFile else
     begin
      for K:=Low(PosterAttributes) to High(PosterAttributes) do
       if S = StrPas(PosterAttributes[K].Name) then
        begin
         Attributes:=Attributes or PosterAttributes[K].Attribute;

         S:='';

         Break;
        end;

      if S <> '' then
       jt.linker.Error(Linker, 'POST: invalid attribute "' + S + '"');
     end;
   end;
 end;

procedure TPost.Perform;
 function FixString(const Source: PString): String;
  var
   Target: String;
   K: Byte;
  begin
   if Source = nil then
    FixString:=''
   else
    begin
     Target:=Source^;

     for K:=1 to Length(Target) do
      if Target[K] = '_' then
       Target[K]:=' ';

     FixString:=Target;
    end;
  end;
 var
  F: Text;
  S: String;
  Spaces: Longint;
  Format: TAreaFormat;
  msg, msgp, st, Area: Pointer;
 begin
  Area:=jt.area.Seek(AreaName^);

  if Area = nil then
   begin
    jt.LogWrite('?', 'Unknown area "' + AreaName^ + '"');

    Exit;
   end;

  Assign(F, FName^);
  Reset(F);

  if InOutRes <> 0 then
   begin
    jt.Report('Unable to open ' + FName^ + ', rc=#'+ Long2Str(IOResult), True);

    Exit;
   end;

  jt.area.GetFormat(Area, Format);

  msg:=jt.msg.Create;

  jt.msg.SetArea(msg, jt.area.GetName(Area)^);

  if FromName = nil then
   begin
    jt.core.GetMainSysop(S);

    FromName:=NewStr(S);
   end;

  if Format.Style = asNetmail then
   begin
    if IsCleanAddress(ToAddress) then
     jt.core.GetMainAddress(ToAddress);

    if IsCleanAddress(FromAddress) then
     jt.GetNetmailAka(ToAddress, FromAddress);
   end
  else
   if IsCleanAddress(FromAddress) then
    jt.area.GetAka(Area, FromAddress);

  jt.msg.SetFromName(msg, FixString(FromName));
  jt.msg.SetToName(msg, FixString(ToName));
  jt.msg.SetSubject(msg, FixString(Subject));

  jt.msg.SetAttributes(msg, Attributes);

  msgp:=jt.msg.CreateParser(msg, Format.Style = asNetmail);

  if Format.Style = asNetmail then
   jt.msg.p.SetFromAndToAddress(msgp, FromAddress, ToAddress, True)
  else
   jt.msg.p.SetFromAddress(msgp, FromAddress, True);

  jt.msg.p.SetKludge(msgp, #1'PID:', #1'PID: ' + jt.core.GetPID);

  Spaces:=0;

  while not Eof(F) do
   begin
    ReadLn(F, S);

    while (Length(S) > 0) and (S[Length(S)] in [#32, #9]) do
     SetLength(S, Length(S) - 1);

    if S = '' then
     Inc(Spaces)
    else
     begin
      while Spaces > 0 do
       begin
        jt.msg.Add(msg, '');

        Dec(Spaces);
       end;

      jt.msg.Add(msg, S);
     end;
   end;

  Close(F);

  jt.msg.Add(msg, '');

  jt.msg.p.PutTearline(msgp);

  jt.msg.p.PutOrigin(msgp, FromAddress);

  jt.msg.DestroyParser(msgp);

  st:=jt.msg.st(msg);

  jt.st.Add(st, 'Source', Pointer(tsLocalInbound), nil);

  if Flags and pfKillFile <> 0 then
   begin
    Erase(F);

    if InOutRes <> 0 then
     jt.LogWrite('?', 'Failed to kill ' + FName^ + ', rc=#' + Long2Str(IOResult));

    jt.LogWrite('P', FName^ + ' posted to ' + jt.area.GetName(Area)^ + ' (kill)');
   end
  else
   jt.LogWrite('P', FName^ + ' posted to ' + jt.area.GetName(Area)^);

  jt.tossscan.TossMessage(msg);
 end;

destructor TPost.Done;
 begin
  DisposeStr(AreaName);
  DisposeStr(FromName);
  DisposeStr(ToName);
  DisposeStr(Subject);
  DisposeStr(FName);

  inherited Done;
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, 'Poster', 1);

  PosterPlugin:=Plugin;

  PluginSetup:=True;

  Posts:=New(PCollection, Init($10, $10));
 end;

{ PluginStartup }

procedure PluginStartup; export;
 begin
  if Posts^.Count > 0 then
   jt.EnqueueCommand(PosterPlugin, PosterPerform, 0, cwLatest);
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
  Dispose(Posts, Done);
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 function Perform(const Post: PPost): Boolean; far;
  begin
   Post^.Perform;

   Perform:=jt.Cancelled;
  end;
 begin
  if Command = PosterPerform then Posts^.FirstThat(@Perform);
 end;

{ PluginHandleCmdlineSwitch }

function PluginHandleCmdlineSwitch(Switch: String): Boolean; export;
 begin
  if StUpcase(Copy(Switch, 1, 5)) = 'POST:' then
   begin
    Posts^.Insert(New(PPost, Init(Copy(Switch, 6, 255), jt.linker.Stub)));

    PluginHandleCmdlineSwitch:=True;
   end
  else
   PluginHandleCmdlineSwitch:=False;
 end;

{ PluginShowHelp }

procedure PluginShowHelp; export;
 begin
  jt.PrintStr(' POST:area,[fromname],[fromaddr],[toname],[toaddr],[subj],fname[,kill[,attrs]]'#13#10);
  jt.PrintStr(#13#10);
  jt.PrintStr(' attrs: pvt,cra,rcv,snt,att,trs,orp,k/s,loc,hld,frq,rrq,rrc,arq,urq,scn,rdd'#13#10);
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleCmdlineSwitch,
 PluginShowHelp;

begin
 Assign(Output, '');
 Rewrite(Output);
end.