library Priority;

uses
{$IFDEF WIN32}
     Windows,
{$ENDIF}

{$IFDEF OS2}
     OS2Def,
     OS2Base,
{$ENDIF}

     PlugAPI,
     PlugSIDE,

     Objects,
     Strings,

     PrioAPI,

     Wizard;

const
 PriorityName                           = 'Priority';
 PriorityVersion                        = 1;

 PriorityLetter                         = 'P';

 PriorityPlugin         : Pointer       = nil;

 PriorityPerformStartupActions          = $2000;
 PriorityPerformExportingActions        = $2001;
 PriorityPerformShutdownActions         = $2002;

 PriorityPerformInfo                    = $2100;

 PriorityReport         : (prNo, prScreen, prYes) = prYes;

type
 TPriorityPerformer = procedure;

 PPriority = ^TPriority;
 TPriority = record
  Name: PChar;
  Performer: TPriorityPerformer;
 end;

 PPriorityAction = ^TPriorityAction;
 TPriorityAction = object(TObject)
  Name: PString;
  Priority: PPriority;

  constructor Init(const AName: String);
  destructor Done; virtual;
 end;

 PPriorityActions = ^TPriorityActions;
 TPriorityActions = object(TStringCollection)
  procedure FreeItem(Item: Pointer); virtual;
  function KeyOf(Item: Pointer): Pointer; virtual;
  function Seek(Name: String): PPriorityAction; virtual;
 end;

const
 PriorityGeneral        : PPriority             = nil;
 PriorityPrevious       : PPriority             = nil;

 PriorityActions        : PPriorityActions      = nil;

{ TPriorityAction }

constructor TPriorityAction.Init(const AName: String);
 begin
  inherited Init;

  Name:=NewStr(AName);
  Priority:=nil;
 end;

destructor TPriorityAction.Done;
 begin
  DisposeStr(Name);

  inherited Done;
 end;

{ TPriorityActions }

procedure TPriorityActions.FreeItem(Item: Pointer);
 begin
  TCollection.FreeItem(Item);
 end;

function TPriorityActions.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=PPriorityAction(Item)^.Name;
 end;

function TPriorityActions.Seek(Name: String): PPriorityAction;
 var
  Index: xInteger;
 begin
  StUpcaseEx(Name);

  if Search(@Name, Index) then
   Seek:=At(Index)
  else
   Seek:=nil;
 end;

{$IFDEF WIN32} // Win32 priority performers

procedure SetIdlePriority; // IDLE
 begin
  SetPriorityClass(GetCurrentProcess, IDLE_PRIORITY_CLASS);
 end;

procedure SetNormalPriority; // NORMAL
 begin
  SetPriorityClass(GetCurrentProcess, NORMAL_PRIORITY_CLASS);
 end;

procedure SetHighPriority; // HIGH
 begin
  SetPriorityClass(GetCurrentProcess, HIGH_PRIORITY_CLASS);
 end;

procedure SetRealtimePriority; // REALTIME
 begin
  SetPriorityClass(GetCurrentProcess, REALTIME_PRIORITY_CLASS);
 end;
{$ENDIF}

{$IFDEF OS2} // OS/2 priority performers

procedure SetIdlePriority; // IDLE
 begin
  DosSetPriority(prtys_Process, prtyc_IdleTime, 0, 0);
 end;

procedure SetRegularPriority; // REGULAR
 begin
  DosSetPriority(prtys_Process, prtyc_Regular, 0, 0);
 end;

procedure SetTimeCriticalPriority; // TIMECRITICAL
 begin
  DosSetPriority(prtys_Process, prtyc_TimeCritical, 0, 0);
 end;

procedure SetForegroundServerPriority; // FOREGROUNDSERVER
 begin
  DosSetPriority(prtys_Process, prtyc_ForegroundServer, 0, 0);
 end;
{$ENDIF}

const
{$IFDEF WIN32}
 Priorities: array[0..3] of TPriority = (
              (Name: 'IDLE'; Performer: SetIdlePriority),
              (Name: 'NORMAL'; Performer: SetNormalPriority),
              (Name: 'HIGH'; Performer: SetHighPriority),
              (Name: 'REALTIME'; Performer: SetRealtimePriority));
{$ENDIF}

{$IFDEF OS2}
 Priorities: array[0..3] of TPriority = (
              (Name: 'IDLE'; Performer: SetIdlePriority),
              (Name: 'REGULAR'; Performer: SetRegularPriority),
              (Name: 'TIMECRITICAL'; Performer: SetTimeCriticalPriority),
              (Name: 'FOREGROUNDSERVER'; Performer: SetForegroundServerPriority));
{$ENDIF}

{ SeekPriority }

function SeekPriority(Name: String): PPriority;
 var
  K: Longint;
 begin
  StUpcaseEx(Name);

  for K:=Low(Priorities) to High(Priorities) do
   if StrPas(Priorities[K].Name) = Name then
    begin
     SeekPriority:=@Priorities[K];

     Exit;
    end;

  SeekPriority:=nil;
 end;

{ PerformPriority }

procedure PerformPriority(const Priority: PPriority; const Reason: String);
 var
  Prefix: String;
 begin
  if (Priority <> nil) and (Priority <> PriorityPrevious) then
   begin
    Priority^.Performer;

    if PriorityReport <> prNo then
     begin
      if PriorityReport = prYes then
       Prefix:=''
      else
       Prefix:=#0;

      jt.LogWrite('P', Prefix + 'Setting ' + StrPas(Priority^.Name) + ' priority (' + Reason + ')');
     end;

    PriorityPrevious:=Priority;
   end;
 end;

{ RegisterAction }

procedure RegisterAction(Name: String);
 var
  Index: xInteger;
 begin
  StUpcaseEx(Name);

  if not PriorityActions^.Search(@Name, Index) then
   if Name <> '' then
    PriorityActions^.AtInsert(Index, New(PPriorityAction, Init(Name)));
 end;

{ PerformActions }

procedure PerformAction(const Name: String);
 var
  NameU: String;
  Index: xInteger;
  PriorityAction: PPriorityAction;
 begin
  NameU:=StUpcase(Name);

  if (PriorityGeneral = nil) and PriorityActions^.Search(@NameU, Index) then
   begin
    PriorityAction:=PriorityActions^.At(Index);

    if PriorityAction <> nil then
     PerformPriority(PriorityAction^.Priority, Name);
   end;
 end;

{ PriorityInfo }

procedure PriorityInfo;
 procedure DumpPriorityAction(const Action: PPriorityAction); far;
  begin
   jt.LogWrite('P', '   ' + Action^.Name^);
  end;
 var
  K: Longint;
 begin
  jt.LogWrite('P', 'Here follows some priority information:');
  jt.LogWrite('P', ' - valid priorities are:');

  for K:=Low(Priorities) to High(Priorities) do
   jt.LogWrite('P', '   ' + StrPas(Priorities[K].Name));

  jt.LogWrite('P', ' - valid priority actions are:');

  PriorityActions^.ForEach(@DumpPriorityAction);
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, PriorityName, PriorityVersion);

  PriorityPlugin:=Plugin;

  PluginSetup:=True;

  PriorityActions:=New(PPriorityActions, Init($10, $10));

  PriorityActions^.Insert(New(PPriorityAction, Init('STARTUP'))); // cwBeforeStartup
  PriorityActions^.Insert(New(PPriorityAction, Init('EXPORTING'))); // cwBeforeExport
  PriorityActions^.Insert(New(PPriorityAction, Init('SHUTDOWN'))); // cwBeforeShutdown

  jt.EnqueueCommand(Plugin, PriorityPerformStartupActions, 0, cwBeforeStartup);
  jt.EnqueueCommand(Plugin, PriorityPerformExportingActions, 0, cwBeforeExport);
  jt.EnqueueCommand(Plugin, PriorityPerformShutdownActions, 0, cwBeforeShutdown);
 end;

{ PluginStartup }

procedure PluginStartup; export;
 begin
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
  Dispose(PriorityActions, Done);
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 begin
  if Command = PriorityRegisterAction then RegisterAction(PString(Param)^) else
  if Command = PriorityPerformAction then PerformAction(PString(Param)^) else
  if Command = PriorityPerformStartupActions then PerformAction('startup') else
  if Command = PriorityPerformExportingActions then PerformAction('exporting') else
  if Command = PriorityPerformShutdownActions then PerformAction('shutdown') else
  if Command = PriorityPerformInfo then PriorityInfo;
 end;

{ PluginHandleConfigSwitch }

procedure PluginHandleConfigSwitch(const Keyword: String; Value: String; const Linker: Pointer); export;
 var
  PriorityAction: PPriorityAction;
  Index: xInteger;
  S: String;
 begin
  if Keyword = 'PRIORITY' then
   begin
    S:=jt.linker.ParseStringToken(Linker, Value);

    PriorityAction:=PriorityActions^.Seek(S);

    if PriorityAction = nil then
     jt.linker.Error(Linker, 'unregistered priority action "' + S + '"');

    if PriorityAction^.Priority <> nil then
     jt.linker.Error(Linker, 'redefinition of priority action "' + S + '"');

    S:=jt.linker.ParseStringToken(Linker, Value);

    PriorityAction^.Priority:=SeekPriority(S);

    if PriorityAction^.Priority = nil then
     jt.linker.Error(Linker, 'unknown priority "' + S + '"');
   end else
  if Keyword = 'PRIORITYREPORT' then
   begin
    Value:=jt.linker.ParseWord(Linker, Value);

    StUpcaseEx(Value);
    TrimEx(Value);

    if Value = 'YES' then PriorityReport:=prYes else
    if Value = 'SCREEN' then PriorityReport:=prScreen else
    if Value = 'NO' then PriorityReport:=prNo else
     jt.linker.Error(Linker, 'YES, SCREEN or NO expected');
   end
  else
   jt.linker.Error(Linker, 'syntax error');
 end;

{ PluginHandleCmdlineSwitch }

function PluginHandleCmdlineSwitch(Switch: String): Boolean; export;
 begin
  StUpcaseEx(Switch);

  PluginHandleCmdlineSwitch:=True;

  if Copy(Switch, 1, 5) = 'PRIO:' then
   begin
    PriorityGeneral:=SeekPriority(Copy(Switch, 6, 255));

    if PriorityGeneral = nil then
     jt.linker.Error(jt.linker.Stub, 'unknown priority "' + Copy(Switch, 6, 255) + '"');

    PerformPriority(PriorityGeneral, 'general');
   end else
  if Switch = 'PRIOINFO' then jt.EnqueueCommand(PriorityPlugin, PriorityPerformInfo, 0, cwEarliest) else
   PluginHandleCmdlineSwitch:=False;
 end;

{ PluginShowHelp }

procedure PluginShowHelp; export;
 begin
  jt.PrintStr(' PRIO:priority    use this priority (overrides config)'#13#10);
  jt.PrintStr(' PRIOINFO         print valid priority actions & priorities'#13#10);
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleConfigSwitch,
 PluginHandleCmdlineSwitch,
 PluginShowHelp;

begin
 Assign(Output, '');
 Rewrite(Output);
end.
