library Purger;

uses
     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     Objects,
     Strings,

     jtMHLcmn,

     PurgPurg,

     PrioAPI,

     Wizard;

const
 PurgerName                             = 'Purger';
 PurgerVersion                          = 2;

 PurgerPlugin           : Pointer       = nil;

 PurgeFirst             : Pointer       = nil;
 PurgeLast              : Pointer       = nil;
 PurgeDisable           : Pointer       = nil;
 PurgeDays              : Pointer       = nil;
 PurgeMessages          : Pointer       = nil;
 PurgeAddToImportList   : Pointer       = nil;
 PurgeUseArrived        : Pointer       = nil;
 PurgeNetmail           : Pointer       = nil;
 PurgeRead              : Pointer       = nil;

 PurgeAreas             : PAreaList     = nil;

 PurgePerform                           = $1000;

 PurgeHaveSense         : Boolean       = False;

 pfNone                                 = $0000;

{ PurgeDaysValueParser }

procedure PurgeDaysValueParser(const Keyword, Linker, st: Pointer; Value: String); export;
 var
  Days: Longint;
 begin
  Days:=jt.linker.ParseNumber(Linker, Value);

  if (Days < 3) and (Days <> 0) then
   jt.linker.Error(Linker, 'Too low PurgeDays value (must be zero or greater than 2)');

  jt.st.Add(st, 'Days', Pointer(Days), nil);
 end;

{ PurgeMessagesValueParser }

procedure PurgeMessagesValueParser(const Keyword, Linker, st: Pointer; Value: String); export;
 var
  Messages: Longint;
 begin
  Messages:=jt.linker.ParseNumber(Linker, Value);

  if (Messages < 16) and (Messages <> 0) then
   jt.linker.Error(Linker, 'Too low PurgeMessages value (must be zero or greater than 15)');

  jt.st.Add(st, 'Messages', Pointer(Messages), nil);
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 var
  ProhibitFormats: PNumbers;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, PurgerName, PurgerVersion);

  PurgerPlugin:=Plugin;

  PurgeFirst:=jt.rests.Add('PurgeFirst', rsMinus);
  PurgeLast:=jt.rests.Add('PurgeLast', rsMinus);
  PurgeDisable:=jt.rests.Add('PurgeDisable', rsMinus);
  PurgeDays:=jt.rests.AddValueParser('PurgeDays', rsMinus or rsValue or rsStorage, PurgeDaysValueParser);
  PurgeMessages:=jt.rests.AddValueParser('PurgeMessages', rsMinus or rsValue or rsStorage, PurgeMessagesValueParser);
  PurgeAddToImportList:=jt.rests.Add('PurgeAddToImportList', rsMinus);
  PurgeUseArrived:=jt.rests.Add('PurgeUseArrived', rsMinus);
  PurgeNetmail:=jt.rests.Add('PurgeNetmail', rsMinus);
  PurgeRead:=jt.rests.Add('PurgeRead', rsMinus);

  ProhibitFormats:=New(PNumbers, Init($2, $4));
  ProhibitFormats^.Add(afPassthrough);

  PurgeAreas:=New(PAreaList, Init(PurgeFirst, PurgeLast, PurgeDisable, ProhibitFormats, 0));

  PluginSetup:=True;

  RegisterPriorityAction('PURGING');
 end;

{ PluginStartup }

procedure PluginStartup; export;
 begin
  if PurgeHaveSense then
   jt.EnqueueCommand(PurgerPlugin, PurgePerform, 0, cwPurge);
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
  Dispose(PurgeAreas, Done);
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 const
  FirstTime: Boolean = True;
 function PerformPurge(const Area: PAreaListItem): Boolean; far;
  var
   Days, Messages, Purged: Longint;
   Purger: PPurger;
   Format: TAreaFormat;
   UseArrived, Read: Boolean;
   st: Pointer;
   rc: String;
  begin
   if FirstTime then
    begin
     jt.LogWrite('P', 'Purging areas');

     FirstTime:=False;
    end;

   { // purge netmail }

   jt.area.GetFormat(Area^.Area, Format);

   if (Format.Style = asNetmail) and not jt.rests.Check(PurgeNetmail, nil, jt.area.GetName(Area^.Area)) then
    begin
     PerformPurge:=jt.Cancelled;

     Exit;
    end;

   { // purge days }

   jt.rests.Getst(PurgeDays, nil, jt.area.GetName(Area^.Area), st);

   if st = nil then
    Days:=0
   else
    Days:=Longint(jt.st.Get(st, 'Days'));

   { // purge messages }

   jt.rests.Getst(PurgeMessages, nil, jt.area.GetName(Area^.Area), st);

   if st = nil then
    Messages:=0
   else
    Messages:=Longint(jt.st.Get(st, 'Messages'));

   { // purge use arrived }

   UseArrived:=jt.rests.Check(PurgeUseArrived, nil, jt.area.GetName(Area^.Area));

   { // purge read }

   Read:=jt.rests.Check(PurgeRead, nil, jt.area.GetName(Area^.Area));

   { // perform purge }

   if (Messages + Days > 0) or Read then
    begin
     Purger:=New(PPurger, Init(Area^.Area, Days, Messages, UseArrived, Read, PurgeAreas^.IndexOf(Area),
      PurgeAreas^.Count));

     if Purger = nil then
      begin
       PerformPurge:=jt.Cancelled;

       Exit;
      end;

     PerformPriorityAction('purging');

     Purger^.Purge;

     Purged:=Purger^.Purged;

     Dispose(Purger, Done);
    end
   else
    Purged:=0;

   if Purged = 0 then
    rc:=''
   else
    rc:=Long2Str(Purged) + ' msg' + _s(Purged) + ' purged';

   if rc <> '' then
    jt.LogWrite('P', jt.area.GetName(Area^.Area)^ + ': ' + rc);

   if (Purged <> 0) and jt.rests.Check(PurgeAddToImportList, nil, jt.area.GetName(Area^.Area)) then
    jt.list.Add(jt.list.ImportList, jt.area.GetName(Area^.Area)^);

   PerformPurge:=jt.Cancelled;
  end;
 begin
  if Command = PurgePerform then
   begin
    if PurgeAreas^.Full then
     PurgeAreas^.AddList(jt.list.ImportList, PurgeAreas^.FullFlags);

    if PurgeAreas^.HaveSense then
     PurgeAreas^.Prepare;

    if PurgeAreas^.Count = 0 then
     jt.LogWrite('P', #0'Nothing to purge.')
    else
     begin
      jt.PushTitle;

      PurgeAreas^.FirstThat(@PerformPurge);

      jt.PopTitle;
     end;
   end;
 end;

{ PluginHandleCmdlineSwitch }

function PluginHandleCmdlineSwitch(Switch: String): Boolean; export;
 procedure Full(const Flags: Longint);
  begin
   PurgeAreas^.AddFull(Flags, jt.linker.Stub);

   PurgeHaveSense:=True;
  end;
 procedure Mask(const Skip: Byte; const Flags: Longint);
  begin
   PurgeAreas^.AddMask(Copy(Switch, Skip + 1, 255), Flags, jt.linker.Stub);

   PurgeHaveSense:=True;
  end;
 begin
  StUpcaseEx(Switch);

  PluginHandleCmdlineSwitch:=True;

  if Switch = 'PURGE' then Full(pfNone) else
  if Switch = 'PURGEFORCE' then Full(alifIgnoreDisable) else
  if Copy(Switch, 1, 6) = 'PURGE:' then Mask(6, pfNone) else
  if Copy(Switch, 1, 11) = 'PURGEFORCE:' then Mask(11, alifIgnoreDisable) else
   PluginHandleCmdlineSwitch:=False;
 end;

{ PluginShowHelp }

procedure PluginShowHelp; export;
 begin
  jt.PrintStr(' PURGE[:re]              purge importlist or regexp if not omitted'#13#10);
  jt.PrintStr(' PURGEFORCE[:re]         purge, PurgeDisable have no influence'#13#10);
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleCmdlineSwitch,
 PluginShowHelp;

begin
 Assign(Output, '');
 Rewrite(Output);
end.
