unit PurgPurg;

interface
uses
     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     Objects,

     jtMHLcmn,

     Wizard;

type
 TPurgerStatus = (psOpening, psPurging, psClosing);

const
 CriticalErrorsCount            = 10;

type
 PPurger = ^TPurger;
 TPurger = object(TObject)
  Area, mb: Pointer;
  Days, Messages: Longint;
  Current, Total, Purged, Errors, No, TotalNo: Longint;
  UseArrived, PurgeRead: Boolean;
  AreaName: String;

  constructor Init(const AArea: Pointer; const ADays, AMessages: Longint; const AUseArrived, APurgeRead: Boolean;
   const ACurrent, ATotal: Longint);
  procedure Status(const PurgerStatus: TPurgerStatus);
  function Purge: Boolean;
  destructor Done; virtual;
 end;

var
 Roller: TRoller;

implementation

{ TPurger }

constructor TPurger.Init(const AArea: Pointer; const ADays, AMessages: Longint; const AUseArrived, APurgeRead: Boolean;
  const ACurrent, ATotal: Longint);
 var
  rc: String;
 begin
  inherited Init;

  Area:=AArea;

  Days:=ADays;
  Messages:=AMessages;

  UseArrived:=AUseArrived;
  PurgeRead:=APurgeRead;

  Current:=ACurrent;
  Total:=ATotal;

  Purged:=0;
  Errors:=0;

  GetPStringEx(jt.area.GetName(Area), AreaName);

  Status(psOpening);

  if not jt.area.Lock(Area, mb, rc) then
   begin
    jt.LogWrite('?', 'Unable to open ' + AreaName + ': ' + rc);

    Fail;
   end;

  if not jt.mb.Lock(mb) then
   begin
    jt.LogWrite('?', 'Unable to lock ' + AreaName + ' for purging');

    jt.area.Unlock(Area, mb);

    Fail;
   end;
 end;

procedure TPurger.Status(const PurgerStatus: TPurgerStatus);
 var
  Percent: String;
 begin
  Percent:=jt.Percent(Current, Total);

  case PurgerStatus of
   psOpening: jt.SetTitle(AreaName + ' ' + Percent + ' (opening)');
   psPurging: jt.SetTitle(AreaName + ' ' + Percent + ' (purging)');
   psClosing: jt.SetTitle(AreaName + ' ' + Percent + ' (closing)');
  end;

  case PurgerStatus of
   psOpening: jt.SetStatus(AreaName + ' ' + Percent);
   psPurging: jt.SetStatus(AreaName + ' ' + Percent + ' ' + Roller.GetRoller + ' purging ' + jt.Percent(No, TotalNo));
  end;
 end;

function TPurger.Purge: Boolean;
 const
  Timeout = 100;
 var
  Timer, Anchor, Countdown: Longint;
  DateTime: TDateTime;
  msg: Pointer;
  rc: String;
 begin
  Status(psPurging);

  { // first step: purge by days / purge read }

  if (Days <> 0) or PurgeRead then
   begin
    No:=0;

    Timer:=Clock;

    GetCurrentDateTime(DateTime);

    Anchor:=GregorianToJulian(DateTime) - Days;

    TotalNo:=jt.mb.Count(mb);

    while (No < TotalNo) and (Errors < CriticalErrorsCount) do
     begin
      if Clock - Timer > Timeout then
       begin
        Status(psPurging);

        Timer:=Clock;

        if jt.Cancelled then
         Break;
       end;

      if jt.mb.Load(mb, No, msg, rc, mfNoText) then
       if PurgeRead and (jt.msg.GetAttributes(msg) and maRead <> 0) then
        begin
         if jt.mb.Kill(mb, No, rc) then
          begin
           Inc(Purged);

           Errors:=0;
          end
         else
          begin
           jt.LogWrite('?', 'Failed to kill msg ' + Long2Str(No) + ' in ' + AreaName + ': ' + rc);

           Inc(Errors);

           Inc(No);
          end;

         jt.msg.Destroy(msg);
        end
       else
        begin
         if UseArrived then
          jt.msg.GetArrived(msg, DateTime);

         if not UseArrived or not IsValidDateTime(DateTime) then
          jt.msg.GetWritten(msg, DateTime);

         if IsValidDateTime(DateTime) and (GregorianToJulian(DateTime) < Anchor) then
          if jt.mb.Kill(mb, No, rc) then
           begin
            Inc(Purged);

            Errors:=0;
           end
          else
           begin
            jt.LogWrite('?', 'Failed to kill msg ' + Long2Str(No) + ' in ' + AreaName + ': ' + rc);

            Inc(Errors);

            Inc(No);
           end
         else
          Inc(No);

         jt.msg.Destroy(msg);
        end
      else
       begin
        jt.LogWrite('?', 'Failed to open msg ' + Long2Str(No) + ' in ' + AreaName + ': ' + rc);

        Inc(Errors);

        Inc(No);
       end;

      TotalNo:=jt.mb.Count(mb);
     end;
   end;

  { // second step: purge by messages }

  if (Messages <> 0) and (Errors = 0) and not jt.Cancelled then
   begin
    Countdown:=jt.mb.Count(mb) - Messages;

    No:=0;

    while (Countdown > 0) and (No < jt.mb.Count(mb)) do
     begin
      if Clock - Timer > Timeout then
       begin
        Status(psPurging);

        Timer:=Clock;

        if jt.Cancelled then
         Break;
       end;

      if jt.mb.Kill(mb, No, rc) then
       begin
        Inc(Purged);

        Dec(Countdown);

        Errors:=0;
       end
      else
       begin
        jt.LogWrite('?', 'Failed to kill msg ' + Long2Str(No) + ' in ' + AreaName + ': ' + rc);

        Inc(Errors);

        Inc(No);
       end;
     end;
   end;

  if Errors >= CriticalErrorsCount then
   begin
    jt.Report(AreaName + ': too many serial errors encountered while purging', True);

    Purge:=False;
   end
  else
   Purge:=True;
 end;

destructor TPurger.Done;
 begin
  Status(psClosing);

  jt.mb.Unlock(mb);

  jt.area.Unlock(Area, mb);

  inherited Done;
 end;

end.