library RLOP;

uses
     Objects,

     PlugAPI,
     PlugSIDE,

     jtMHLcmn,

     Wizard,

     rlopAPI;

const
 rlopPlugin                     : Pointer       = nil;

 rlopFlush                                      = $1000;

 DefaultRobotName               : String        = 'Areafix';
 DefaultSubscribePrefix         : String        = '+';
 DefaultUnsubscribePrefix       : String        = '-';
 DefaultTearline                : Boolean       = True;
 DefaultOrigin                  : Boolean       = True;

 Yes                            : Boolean       = True;
 No                             : Boolean       = False;

 BooleanSwitch                  : array[Boolean] of PBoolean = (@No, @Yes);

type
 TOperation = (loSubscribe, loUnsubscribe);

 PLinkOperation = ^TLinkOperation;
 TLinkOperation = object(TObject)
  Operation: TOperation;
  Name: PString;

  constructor Init(const AOperation: TOperation; const AName: String);
  destructor Done; virtual;
 end;

 PLinkOperations = ^TLinkOperations;
 TLinkOperations = object(TCollection)
  Link: Pointer;
  Configured: Boolean;
  RobotName: PString;
  RobotPassword: PString;
  RobotSubscribePrefix: PString;
  RobotUnsubscribePrefix: PString;
  RobotTearline: PBoolean;
  RobotOrigin: PBoolean;

  constructor Init(const ALink: Pointer);
  destructor Done; virtual;
 public
  function Valid(var rc: String): Boolean;
  procedure Flush;
  function GetRobotName: String;
  function GetRobotPassword: String;
  function GetSubscribePrefix: String;
  function GetUnsubscribePrefix: String;
  function GetTearline: Boolean;
  function GetOrigin: Boolean;
  procedure Subscribe(const Name: String);
  procedure Unsubscribe(const Name: String);
 end;

{ TLinkOperation }

constructor TLinkOperation.Init(const AOperation: TOperation; const AName: String);
 begin
  inherited Init;

  Operation:=AOperation;

  Name:=NewStr(AName);
 end;

destructor TLinkOperation.Done;
 begin
  DisposeStr(Name);

  inherited Done;
 end;

{ TLinkOperations }

constructor TLinkOperations.Init(const ALink: Pointer);
 begin
  inherited Init($10, $10);

  Link:=ALink;
 end;

destructor TLinkOperations.Done;
 begin
  DisposeStr(RobotUnsubscribePrefix);
  DisposeStr(RobotSubscribePrefix);
  DisposeStr(RobotPassword);
  DisposeStr(RobotName);

  inherited Done;
 end;

{ public methods }

function TLinkOperations.Valid(var rc: String): Boolean;
 var
  Address: TAddress;
  Suffix: String;
 begin
  Valid:=False;

  jt.link.GetAddress(Link, Address);

  Suffix:=AddressToStrEx(Address);

  if RobotPassword = nil then rc:='no RobotPassword specified for ' + Suffix else
   Valid:=True;
 end;

procedure TLinkOperations.Flush;
 var
  msg, msgp, st: Pointer;
  FromAddress, ToAddress: TAddress;
  S: String;
 procedure Perform(const Operation: PLinkOperation); far;
  begin
   if Operation^.Name <> nil then
    case Operation^.Operation of
     loSubscribe: jt.msg.Add(msg, GetSubscribePrefix + Operation^.Name^);
     loUnsubscribe: jt.msg.Add(msg, GetUnsubscribePrefix + Operation^.Name^);
    end;
  end;
 begin
  jt.link.GetAddress(Link, ToAddress);

  if Count > 0 then
   if RobotPassword = nil then
    begin
     jt.LogWrite('?', Long2Str(Count) + ' RLOP request' + _s(Count) + ' lost for ' + AddressToStrEx(ToAddress) +
      ' (no RobotPassword defined)');

     FreeAll;
    end
   else
    begin
     msg:=jt.msg.Create;

     st:=jt.msg.st(msg);

     jt.st.Add(st, 'Source', Pointer(tsLocalInbound), nil);

     jt.msg.SetArea(msg, jt.area.GetName(jt.area.SeekNetmail)^);

     jt.core.GetMainSysop(S);

     jt.msg.SetFromName(msg, S);
     jt.msg.SetToName(msg, GetRobotName);
     jt.msg.SetSubject(msg, GetRobotPassword);

     jt.msg.SetAttributes(msg, jt.msg.GetAttributes(msg) or maLocal and not maSent);

     msgp:=jt.msg.CreateParser(msg, True);

     jt.link.GetAka(Link, FromAddress);

     jt.msg.p.SetFromAndToAddress(msgp, FromAddress, ToAddress, True);

     jt.msg.p.AddKludge(msgp, #1'PID: ' + jt.core.GetPID);

     ForEach(@Perform);

     jt.msg.Add(msg, '');

     if GetTearline then
      jt.msg.p.PutTearline(msgp);

     if GetOrigin then
      jt.msg.p.PutOrigin(msgp, FromAddress);

     jt.msg.DestroyParser(msgp);

     jt.tossscan.TossMessage(msg);

     jt.LogWrite('R', 'Wrote message to ' + GetRobotName + ' of ' + AddressToStrEx(ToAddress) + ' (' + Long2Str(Count) +
      ' action' + _s(Count) + ')');

     FreeAll;
    end;
 end;

function TLinkOperations.GetRobotName: String;
 begin
  if RobotName = nil then
   GetRobotName:=DefaultRobotName
  else
   GetRobotName:=RobotName^;
 end;

function TLinkOperations.GetRobotPassword: String;
 begin
  if RobotPassword = nil then
   GetRobotPassword:=''
  else
   GetRobotPassword:=RobotPassword^;
 end;

function TLinkOperations.GetSubscribePrefix: String;
 begin
  if RobotSubscribePrefix = nil then
   GetSubscribePrefix:=DefaultSubscribePrefix
  else
   GetSubscribePrefix:=RobotSubscribePrefix^;
 end;

function TLinkOperations.GetUnsubscribePrefix: String;
 begin
  if RobotUnsubscribePrefix = nil then
   GetUnsubscribePrefix:=DefaultUnsubscribePrefix
  else
   GetUnsubscribePrefix:=RobotUnsubscribePrefix^;
 end;

function TLinkOperations.GetTearline: Boolean;
 begin
  if RobotTearline = nil then
   GetTearline:=DefaultTearline
  else
   GetTearline:=RobotTearline^;
 end;

function TLinkOperations.GetOrigin: Boolean;
 begin
  if RobotOrigin = nil then
   GetOrigin:=DefaultOrigin
  else
   GetOrigin:=RobotOrigin^;
 end;

procedure TLinkOperations.Subscribe(const Name: String);
 begin
  Insert(New(PLinkOperation, Init(loSubscribe, Name)));
 end;

procedure TLinkOperations.Unsubscribe(const Name: String);
 begin
  Insert(New(PLinkOperation, Init(loUnsubscribe, Name)));
 end;

{ stLinkOperations }

function stLinkOperations(const Action: TStorageAction; const Data: Pointer): Pointer; export;
 var
  Operations: PLinkOperations absolute Data;
 begin
  case Action of
   saDestroy: Dispose(Operations, Done);
   saClone: stLinkOperations:=nil;
   saInfo: stLinkOperations:=jt.NewStr('rlop/link operations');
  end;
 end;

{ GetLinkOperations }

function GetLinkOperations(const Link: Pointer): PLinkOperations;
 var
  LinkOperations: PLinkOperations;
 begin
  LinkOperations:=jt.st.Get(jt.link.st(Link), 'rlop.LinkOperations');

  if LinkOperations = nil then
   begin
    LinkOperations:=New(PLinkOperations, Init(Link));

    jt.st.Add(jt.link.st(Link), 'rlop.LinkOperations', LinkOperations, stLinkOperations);
   end;

  GetLinkOperations:=LinkOperations;
 end;

{ IsLinkValid }

function IsLinkValid(const Link: Pointer; var rc: String): Boolean; export;
 begin
  IsLinkValid:=GetLinkOperations(Link)^.Valid(rc);
 end;

{ SendAreaSubscribe }

procedure SendAreaSubscribe(const Link: Pointer; const Area: String); export;
 begin
  GetLinkOperations(Link)^.Subscribe(Area);
 end;

{ SendAreaUnsubscribe }

procedure SendAreaUnsubscribe(const Link: Pointer; const Area: String); export;
 begin
  GetLinkOperations(Link)^.Unsubscribe(Area);
 end;

{ CheckLinkValidity }

procedure CheckLinkValidity(var Stuff);
 var
  Data: packed record
   Link, Linker: Pointer;
  end absolute Stuff;
  LinkOperations: PLinkOperations;
  rc: String;
 begin
  LinkOperations:=GetLinkOperations(Data.Link);

  if LinkOperations^.Configured then
   if not LinkOperations^.Valid(rc) then
    jt.linker.Error(Data.Linker, rc);
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, 'RemoteLinkOperations', 2);

  rlopPlugin:=Plugin;

  jt.EnqueueCommand(Plugin, rlopFlush, 0, cwBeforeFlush);

  jt.linker.RegisterLinkSectionCallback(Plugin);

  PluginSetup:=True;
 end;

{ PluginStartup }

procedure PluginStartup; export;
 begin
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 var
  LinkOperations: PLinkOperations;
  Link: Pointer;
  K: Longint;
  IsLinkValidBlock: PrlopIsLinkValidBlock absolute Param;
  SendAreaSubscribeBlock: PrlopSendAreaSubscribeBlock absolute Param;
  SendAreaUnsubscribeBlock: PrlopSendAreaUnsubscribeBlock absolute Param;
 begin
  if Command = jtpcLinkSectionEnd then CheckLinkValidity(Pointer(Param)^) else
  if Command = rlopFlush then
   for K:=0 to jt.link.Count - 1 do
    begin
     Link:=jt.link.At(K);

     LinkOperations:=jt.st.Get(jt.link.st(Link), 'rlop.LinkOperations');

     if LinkOperations <> nil then
      LinkOperations^.Flush;
    end else
  if Command = rlopcIsLinkValid then with IsLinkValidBlock^ do Valid:=IsLinkValid(Link, rc) else
  if Command = rlopcSendAreaSubscribe then with SendAreaSubscribeBlock^ do SendAreaSubscribe(Link, Area^) else
  if Command = rlopcSendAreaUnsubscribe then with SendAreaUnsubscribeBlock^ do SendAreaUnsubscribe(Link, Area^);
 end;

{ PluginHandleConfigSwitch }

procedure PluginHandleConfigSwitch(const Keyword: String; Value: String; const Linker: Pointer); export;
 begin
  if Keyword = 'DEFAULTROBOTNAME' then
   begin
    DefaultRobotName:=jt.linker.ParseStringToken(Linker, Value);

    if DefaultRobotName = '' then
     jt.linker.Error(Linker, 'DefaultRobotName cannot be empty');
   end else
  if Keyword = 'DEFAULTSUBSCRIBEPREFIX' then DefaultSubscribePrefix:=jt.linker.ParseStringToken(Linker, Value) else
  if Keyword = 'DEFAULTUNSUBSCRIBEPREFIX' then DefaultUnsubscribePrefix:=jt.linker.ParseStringToken(Linker, Value) else
  if Keyword = 'DEFAULTTEARLINE' then DefaultTearline:=jt.linker.ParseBoolean(Linker, Value) else
  if Keyword = 'DEFAULTORIGIN' then DefaultOrigin:=jt.linker.ParseBoolean(Linker, Value) else
   jt.linker.Error(Linker, 'syntax error');
 end;

{ PluginHandleLinkSwitch }

function PluginHandleLinkSwitch(const Link: Pointer; const Keyword: String; Value: String; const Linker: Pointer): Boolean;
 export;
 var
  LinkOperations: PLinkOperations;
 procedure SetString(var AValue: PString; const Name: String);
  begin
   if AValue <> nil then
    jt.linker.Error(Linker, 'redefinition of ' + Name);

   AValue:=NewStr(jt.linker.ParseStringToken(Linker, Value));

   LinkOperations^.Configured:=True;
  end;
 begin
  if Copy(Keyword, 1, 5) <> 'ROBOT' then
   begin
    PluginHandleLinkSwitch:=False;

    Exit;
   end;

  PluginHandleLinkSwitch:=True;

  LinkOperations:=GetLinkOperations(Link);

  if Keyword = 'ROBOTNAME' then SetString(LinkOperations^.RobotName, 'RobotName') else
  if Keyword = 'ROBOTPASSWORD' then SetString(LinkOperations^.RobotPassword, 'RobotPassword') else
  if Keyword = 'ROBOTSUBSCRIBEPREFIX' then SetString(LinkOperations^.RobotSubscribePrefix, 'RobotSubscribePrefix') else
  if Keyword = 'ROBOTUNSUBSCRIBEPREFIX' then SetString(LinkOperations^.RobotUnsubscribePrefix, 'RobotUnsubscribePrefix') else
  if Keyword = 'ROBOTTEARLINE' then LinkOperations^.RobotTearline:=BooleanSwitch[jt.linker.ParseBoolean(Linker, Value)] else
  if Keyword = 'ROBOTORIGIN' then LinkOperations^.RobotOrigin:=BooleanSwitch[jt.linker.ParseBoolean(Linker, Value)] else
   PluginHandleLinkSwitch:=False;
 end;

{ PluginExportLinkSwitches }

procedure PluginExportLinkSwitches(const Link, Linker: Pointer); export;
 procedure ExportString(const Value: PString; const Name: String);
  begin
   if Value <> nil then
    jt.linker.ExportLine(Linker, ' ' + Name + ' ' + jt.linker.UnparseString(Linker, Value^));
  end;
 var
  LinkOperations: PLinkOperations;
 begin
  LinkOperations:=GetLinkOperations(Link);

  ExportString(LinkOperations^.RobotName, 'RobotName');
  ExportString(LinkOperations^.RobotPassword, 'RobotPassword');
  ExportString(LinkOperations^.RobotSubscribePrefix, 'RobotSubscribePrefix');
  ExportString(LinkOperations^.RobotUnsubscribePrefix, 'RobotUnsubscribePrefix');

  if (LinkOperations^.RobotTearline <> nil) and LinkOperations^.RobotTearline^ then
   jt.linker.ExportLine(Linker, ' RobotTearline ' + jt.linker.UnparseBoolean(Linker, True));

  if (LinkOperations^.RobotOrigin <> nil) and LinkOperations^.RobotOrigin^ then
   jt.linker.ExportLine(Linker, ' RobotOrigin ' + jt.linker.UnparseBoolean(Linker, False));
 end;

{ PluginExportLinksHeader }

procedure PluginExportLinksHeader(const Linker: Pointer); export;
 begin
  jt.linker.ExportLine(Linker, '; RemoteLinkOperations switches:');
  jt.linker.ExportLine(Linker, ';   RobotName "remote robot name"');
  jt.linker.ExportLine(Linker, ';   RobotSubscribePrefix "subscribe prefix"');
  jt.linker.ExportLine(Linker, ';   RobotUnsubscribePrefix "unsubscribe prefix"');
  jt.linker.ExportLine(Linker, ';   RobotTearline Yes|No');
  jt.linker.ExportLine(Linker, ';   RobotOrigin Yes|No');
  jt.linker.ExportLine(Linker, ';');
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleConfigSwitch,
 PluginHandleLinkSwitch,
 PluginExportLinkSwitches,
 PluginExportLinksHeader;

begin
 Assign(Output, '');
 Rewrite(Output);
end.