unit rlopAPI;

interface
uses
     PlugAPI,
     PlugSIDE,

     Objects;

const
 rlopcIsLinkValid                                       = $2000;
 rlopcSendAreaSubscribe                                 = $2001;
 rlopcSendAreaUnsubscribe                               = $2002;

type
 PrlopIsLinkValidBlock = ^TrlopIsLinkValidBlock;
 TrlopIsLinkValidBlock = packed record
  Link: Pointer;
  rc: String;
  Valid: Boolean;
 end;

 PrlopSendAreaSubscribeBlock = ^TrlopSendAreaSubscribeBlock;
 TrlopSendAreaSubscribeBlock = packed record
  Link: Pointer;
  Area: PString;
 end;

 PrlopSendAreaUnsubscribeBlock = ^TrlopSendAreaUnsubscribeBlock;
 TrlopSendAreaUnsubscribeBlock = packed record
  Link: Pointer;
  Area: PString;
 end;

function rlopAvailable: Boolean;
function rlopIsLinkValid(const Link: Pointer; var rc: String): Boolean;
procedure rlopSendAreaSubscribe(const Link: Pointer; const Area: String);
procedure rlopSendAreaUnsubscribe(const Link: Pointer; const Area: String);

implementation

const
 rlopPluginHandleCommand: TPluginHandleCommand = nil;

{ FixPlugin }

function FixPlugin: Boolean;
 var
  Plugin: Pointer;
 begin
  if @rlopPluginHandleCommand <> nil then
   FixPlugin:=True
  else
   begin
    Plugin:=jt.plugs.Seek('RemoteLinkOperations');

    if Plugin = nil then
     FixPlugin:=False
    else
     begin
      jt.plugs.GetPluginHandleCommand(Plugin, rlopPluginHandleCommand);

      FixPlugin:=@rlopPluginHandleCommand <> nil;
     end;
   end;
 end;

{ rlopAvailable }

function rlopAvailable: Boolean;
 begin
  rlopAvailable:=jt.plugs.Seek('RemoteLinkOperations') <> nil;
 end;

{ rlopIsLinkValid }

function rlopIsLinkValid(const Link: Pointer; var rc: String): Boolean;
 var
  Block: TrlopIsLinkValidBlock;
 begin
  if FixPlugin then
   begin
    Block.Link:=Link;

    rlopPluginHandleCommand(rlopcIsLinkValid, Longint(@Block));

    rc:=Block.rc;

    rlopIsLinkValid:=Block.Valid;
   end
  else
   rlopIsLinkValid:=False;
 end;

{ rlopSendAreaSubscribe }

procedure rlopSendAreaSubscribe(const Link: Pointer; const Area: String);
 var
  Block: TrlopSendAreaSubscribeBlock;
 begin
  if FixPlugin then
   begin
    Block.Link:=Link;
    Block.Area:=@Area;

    rlopPluginHandleCommand(rlopcSendAreaSubscribe, Longint(@Block));
   end;
 end;

{ rlopSendAreaUnsubscribe }

procedure rlopSendAreaUnsubscribe(const Link: Pointer; const Area: String);
 var
  Block: TrlopSendAreaUnsubscribeBlock;
 begin
  if FixPlugin then
   begin
    Block.Link:=Link;
    Block.Area:=@Area;

    rlopPluginHandleCommand(rlopcSendAreaUnsubscribe, Longint(@Block));
   end;
 end;

end.