library Readonly;

uses
     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     Objects,

     afxAPI,

     jtMHLcmn,

     Wizard;

const
 ReadonlyPlugin                 : Pointer       = nil;

 ReadonlyRestriction            : Pointer       = nil;
 ReadonlyAutoRestriction        : Pointer       = nil;

type
 TReadonlyMode = (rmAuto, rmManual);

 PReadonly = ^TReadonly;
 TReadonly = object(TObject)
  Area: Pointer;
  Till: TDateTime;
  Mode: TReadonlyMode;

  constructor Init(const AArea: Pointer; const ATill: TDateTime; const AMode: TReadonlyMode);
  constructor InitDays(const AArea: Pointer; const Days: Longint; const AMode: TReadonlyMode);
  procedure Export(const Linker: Pointer); virtual;
 end;

 PReadonlies = ^TReadonlies;
 TReadonlies = object(TStringCollection)
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  function KeyOf(Item: Pointer): Pointer; virtual;
  procedure FreeItem(Item: Pointer); virtual;
 public
  procedure Export(const Linker: Pointer);
 end;

{ TReadonly }

constructor TReadonly.Init(const AArea: Pointer; const ATill: TDateTime; const AMode: TReadonlyMode);
 begin
  inherited Init;

  Area:=AArea;
  Till:=ATill;
  Mode:=AMode;
 end;

constructor TReadonly.InitDays(const AArea: Pointer; const Days: Longint; const AMode: TReadonlyMode);
 var
  DT: TDateTime;
 begin
  GetCurrentDateTime(DT);

  JulianToGregorian(GregorianToJulian(DT) + Days, DT.Year, DT.Month, DT.Day);

  Init(AArea, DT, AMode);
 end;

procedure TReadonly.Export(const Linker: Pointer);
 begin
  case Mode of
   rmAuto:
    jt.linker.ExportLine(Linker, ' ReadonlyAuto ' + jt.linker.UnparseString(Linker, jt.area.GetName(Area)^) + ' ' +
     lz(Till.Day) + '/' + lz(Till.Month) + '/' + Long2Str(Till.Year));
   rmManual:
    jt.linker.ExportLine(Linker, ' ReadonlyManual ' + jt.linker.UnparseString(Linker, jt.area.GetName(Area)^) + ' ' +
     lz(Till.Day) + '/' + lz(Till.Month) + '/' + Long2Str(Till.Year));
  end;
 end;

{ TReadonlies }

function TReadonlies.Compare(Key1, Key2: Pointer): xInteger;
 begin
  Compare:=inherited Compare(jt.area.GetName(Key1), jt.area.GetName(Key2));
 end;

function TReadonlies.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=PReadonly(Item)^.Area;
 end;

procedure TReadonlies.FreeItem(Item: Pointer);
 begin
  TCollection.FreeItem(Item);
 end;

{ public methods }

procedure TReadonlies.Export(const Linker: Pointer);
 procedure Perform(const Readonly: PReadonly); far;
  begin
   Readonly^.Export(Linker);
  end;
 begin
  ForEach(@Perform);
 end;

{ stReadonlies }

function stReadonlies(const Action: TStorageAction; const Data: Pointer): Pointer; export;
 var
  Readonlies: PReadonlies absolute Data;
 begin
  case Action of
   saDestroy: Dispose(Readonlies, Done);
   saClone: stReadonlies:=nil;
   saInfo: stReadonlies:=jt.NewStr('readonlies');
  end;
 end;

{ GetReadonlies }

function GetReadonlies(const Link: Pointer): PReadonlies;
 var
  Readonlies: PReadonlies;
  st: Pointer;
 begin
  st:=jt.link.st(Link);

  Readonlies:=jt.st.Get(st, 'Readonlies');

  if Readonlies = nil then
   begin
    Readonlies:=New(PReadonlies, Init($10, $10));

    jt.st.Add(st, 'Readonlies', Readonlies, stReadonlies);
   end;

  GetReadonlies:=Readonlies;
 end;

{ CheckSubscribed }

procedure CheckSubscribed(const Link, Area: Pointer);
 var
  Readonlies: PReadonlies;
  Address: TAddress;
  Index: xInteger;
  Days: Longint;
  st: Pointer;
 begin
  jt.link.GetAddress(Link, Address);

  jt.rests.Getst(ReadonlyAutoRestriction, @Address, jt.area.GetName(Area), st);

  if st <> nil then
   begin
    Days:=Longint(jt.st.Get(st, 'Days'));

    if Days <> 0 then
     begin
      Readonlies:=GetReadonlies(Link);

      if not Readonlies^.Search(Area, Index) then
       begin
        Readonlies^.AtInsert(Index, New(PReadonly, InitDays(Area, Days, rmAuto)));

        jt.Report('AutoReadonly: ' + AddressToStrEx(Address) + ' for ' + jt.area.GetName(Area)^ + ', ' +
         Long2Str(Days) + ' day' + _s(Days), True);

        jt.link.SetChanged(lcfChanged, 'autoreadonly');
       end;
     end;
   end;
 end;

{ CheckUnsubscribed }

procedure CheckUnsubscribed(const Link, Area: Pointer);
 var
  Readonlies: PReadonlies;
  Readonly: PReadonly;
  Index: xInteger;
 begin
  Readonlies:=GetReadonlies(Link);

  if Readonlies^.Search(Area, Index) then
   begin
    Readonly:=Readonlies^.At(Index);

    if Readonly^.Mode = rmAuto then
     begin
      Readonlies^.AtFree(Index);

      jt.Report('AutoReadonly: dropped ' + AddressOfLink(Link) + ' for ' + jt.area.GetName(Area)^, True);

      jt.link.SetChanged(lcfChanged, 'autoreadonly dropped');
     end;
   end;
 end;

{ AreaCreated }

procedure AreaCreated(const Area: Pointer);
 var
  K: Longint;
  LinkArea: Pointer;
 begin
  for K:=0 to jt.link.Count - 1 do
   begin
    LinkArea:=jt.link.SeekArea(jt.link.At(K), Area);

    if LinkArea <> nil then
     CheckSubscribed(jt.link.At(K), Area);
   end;
 end;

{ AreaDestroyed }

procedure AreaDestroyed(const Area: Pointer);
 var
  K: Longint;
 begin
  for K:=0 to jt.link.Count - 1 do
   CheckUnsubscribed(jt.link.At(K), Area);
 end;

{ AreafixSubscribed }

procedure AreafixSubscribed(var Block: TAreafixSubscribeBlock);
 var
  Info: TAreafixGetRequestInfoBlock;
 begin
  afxGetRequestInfo(Info);

  CheckSubscribed(Info.Link, Block.Area);
 end;

{ AreafixUnsubscribed }

procedure AreafixUnsubscribed(var Block: TAreafixUnsubscribeBlock);
 var
  Info: TAreafixGetRequestInfoBlock;
 begin
  afxGetRequestInfo(Info);

  CheckUnsubscribed(Info.Link, Block.Area);
 end;

{ IsWriteable }

procedure IsWriteable(var Block: TLinkAreaCallbackBlock);
 var
  Address, Uplink: TAddress;
  Readonlies: PReadonlies;
  Index: xInteger;
 begin
  if Block.Value then
   begin
    jt.link.GetAddress(Block.Link, Address);

    jt.area.GetUplink(Block.Area, Uplink);

    if not AddressEqu(Address, Uplink) then
     if jt.rests.Check(ReadonlyRestriction, @Address, jt.area.GetName(Block.Area)) then
      Block.Value:=False;

    if Block.Value then
     begin
      Readonlies:=GetReadonlies(Block.Link);

      if Readonlies^.Search(Block.Area, Index) then
       Block.Value:=False;
     end;
   end;
 end;

{ ReadonlyAutoValueParser }

procedure ReadonlyAutoValueParser(const Keyword, Linker, st: Pointer; Value: String); export;
 var
  Days: Longint;
 begin
  Days:=jt.linker.ParseNumber(Linker, Value);

  if Days < 0 then
   Days:=0;

  jt.st.Add(st, 'Days', Pointer(Days), nil);
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, 'Readonly', 1);

  ReadonlyPlugin:=Plugin;

  PluginSetup:=True;

  ReadonlyRestriction:=jt.rests.Add('Readonly', rsMinus);
  ReadonlyAutoRestriction:=jt.rests.AddValueParser('ReadonlyAuto', rsValue or rsStorage, ReadonlyAutoValueParser);

  jt.link.area.RegisterIsWriteableCallback(Plugin);
 end;

{ PluginStartup }

procedure PluginStartup; export;
 var
  K, L, This, That: Longint;
  Readonlies: PReadonlies;
  Readonly: PReadonly;
  Link: Pointer;
  DT: TDateTime;
 begin
  RegisterAreafixCallback(ReadonlyPlugin, aaSubscribeSubscribed);
  RegisterAreafixCallback(ReadonlyPlugin, aaUnsubscribeUnsubscribed);

  jt.area.RegisterNotifyAreaCreated(ReadonlyPlugin);

  GetCurrentDateTime(DT);

  DateTimeToUnixDateTime(DT, This);

  for L:=0 to jt.link.Count - 1 do
   begin
    Link:=jt.link.At(L);

    Readonlies:=GetReadonlies(Link);

    K:=0;

    while K < Readonlies^.Count do
     begin
      Readonly:=Readonlies^.At(K);

      DateTimeToUnixDateTime(Readonly^.Till, That);

      if This > That then
       begin
        jt.Report('AutoReadonly: autodropped ' + AddressOfLink(Link) + ' for ' + jt.area.GetName(Readonly^.Area)^, True);

        Readonlies^.AtFree(K);

        jt.link.SetChanged(lcfChanged, 'readonlies purge');
       end
      else
       Inc(K);
     end;
   end;
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
  DisposeAreafixCallbacks;
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 begin
  if Command = jtpcIsWriteable then IsWriteable(PLinkAreaCallbackBlock(Param)^) else
  if Command = jtpcanCreated then AreaCreated(Pointer(Param)) else
  if Command = jtpcanDestroyed then AreaDestroyed(Pointer(Param)) else
  if Command = acSubscribeSubscribed then AreafixSubscribed(PAreafixSubscribeBlock(Param)^) else
  if Command = acUnsubscribeUnsubscribed then AreafixUnsubscribed(PAreafixUnsubscribeBlock(Param)^);
 end;

{ PluginHandleConfigSwitch }

procedure PluginHandleConfigSwitch(const Keyword: String; Value: String; const Linker: Pointer); export;
 begin
 end;

{ PluginHandleLinkSwitch }

function PluginHandleLinkSwitch(const Link: Pointer; const Keyword: String; Value: String; const Linker: Pointer): Boolean;
 export;
 procedure ParseReadonly(const Mode: TReadonlyMode);
  var
   Readonlies: PReadonlies;
   Name, S: String;
   Index: xInteger;
   Area: Pointer;
   DT: TDateTime;
  begin
   Name:=jt.linker.ParseStringToken(Linker, Value);

   Area:=jt.area.Seek(Name);

   if Area = nil then
    jt.linker.Error(Linker, 'undefined area "' + Name + '"');

   Readonlies:=GetReadonlies(Link);

   if Readonlies^.Search(Area, Index) then
    jt.linker.Error(Linker, 'readonly redefinition for "' + Name + '"');

   S:=jt.linker.ParseWord(Linker, Value);

   FillChar(DT, SizeOf(DT), 0);

   Str2Word(ExtractWord(1, S, ['/']), DT.Day);
   Str2Word(ExtractWord(2, S, ['/']), DT.Month);
   Str2Word(ExtractWord(3, S, ['/']), DT.Year);

   if not IsValidDateTime(DT) then
    jt.linker.Error(Linker, 'invalid datetime');

   Readonlies^.AtInsert(Index, New(PReadonly, Init(Area, DT, Mode)));
  end;
 begin
  PluginHandleLinkSwitch:=True;

  if Keyword = 'READONLYAUTO' then ParseReadonly(rmAuto) else
  if Keyword = 'READONLYMANUAL' then ParseReadonly(rmManual) else
   PluginHandleLinkSwitch:=False;
 end;

{ PluginExportLinkSwitches }

procedure PluginExportLinkSwitches(const Link, Linker: Pointer); export;
 begin
  GetReadonlies(Link)^.Export(Linker);
 end;

{ PluginExportLinksHeader }

procedure PluginExportLinksHeader(const Linker: Pointer); export;
 begin
  jt.linker.ExportLine(Linker, '; Readonly switches:');
  jt.linker.ExportLine(Linker, ';   ReadonlyAuto "area" dd/mm/yyyy');
  jt.linker.ExportLine(Linker, ';   ReadonlyManual "area" dd/mm/yyyy');
  jt.linker.ExportLine(Linker, ';');
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleConfigSwitch,
 PluginHandleLinkSwitch,
 PluginExportLinkSwitches,
 PluginExportLinksHeader;

begin
 Assign(Output, '');
 Rewrite(Output);
end.