unit RStream;

interface
uses
     Objects,

     Wizard;

const
 BlockSize      = 1024 * 8;

type
 PMemoryBlock = ^TMemoryBlock;
 TMemoryBlock = record
  Size: Longint;
  Block: Pointer;
  Next: PMemoryBlock;
 end;

 PRamBuffer = ^TRamBuffer;
 TRamBuffer = array[0..65521] of byte;

 PRamBlock = ^TRamBlock;
 TRamBlock = record
  Buffer: PRamBuffer;
  Size: Longint;
  Next: PRamBlock;
 end;

 PRamStream = ^TRamStream;
 TRamStream = object(TStream)
  constructor Init;
  destructor Done; virtual;
  function GetPos: Longint; virtual;
  function GetSize: Longint; virtual;
  procedure Read(var Buf; Count: xWord); virtual;
  procedure Seek(Pos: Longint); virtual;
  procedure Truncate; virtual;
  procedure Write(var Buf; Count: xWord); virtual;
  procedure CopyFrom(var S: TStream; Count: Longint); virtual;
  procedure Flush; virtual;
 public
  First: PRamBlock;
  CurPos, CurSize, VirtualSize: Longint;

  procedure SetSize(NewSize: Longint);
  procedure GetPosEx(Position: Longint; var Block: PRamBlock; var Offset, Remain: Longint);
 end;

implementation

{ TRamStream }

constructor TRamStream.Init;
 begin
  inherited Init;

  New(First);

  First^.Size:=8192;
  First^.Next:=nil;

  GetMem(First^.Buffer, First^.Size);

  CurPos:=0;
  CurSize:=First^.Size;

  VirtualSize:=0;
 end;

destructor TRamStream.Done;
 var
  Block, SavedBlock: PRamBlock;
 begin
  Block:=First;

  while Block <> Nil do
   begin
    FreeMem(Block^.Buffer, Block^.Size);

    SavedBlock:=Block;

    Block:=Block^.Next;

    Dispose(SavedBlock);
   end;

  inherited Done;
 end;

function TRamStream.GetPos: Longint;
 begin
  GetPos:=CurPos;
 end;

function TRamStream.GetSize: Longint;
 begin
  GetSize:=VirtualSize;
 end;

procedure TRamStream.Read(var Buf; Count: xWord);
 var
  Buffer: TRamBuffer absolute Buf;
  Block: PRamBlock;
  K, Offset, Remain: Longint;
 begin
  K:=0;

  while K < Count do
   begin
    GetPosEx(CurPos, Block, Offset, Remain);

    if Block = nil then
     Break;

    if Remain > Count - K then
     Remain:=Count - K;

    Move(Block^.Buffer^[Offset], Buffer[K], Remain);

    Inc(CurPos, Remain);

    Inc(K, Remain);
   end;
 end;

procedure TRamStream.Seek(Pos: Longint);
 begin
  CurPos:=Pos;
 end;

procedure TRamStream.Truncate;
 begin
  VirtualSize:=CurPos;

  SetSize(CurPos);
 end;

procedure TRamStream.Write(var Buf; Count: xWord);
 var
  Buffer: TRamBuffer absolute Buf;
  Block: PRamBlock;
  K, Offset, Remain: Longint;
 begin
  if CurPos + Count > CurSize then
   SetSize(CurPos + Count);

  if CurPos + Count > VirtualSize then
   VirtualSize:=CurPos + Count;

  K:=0;

  while K < Count do
   begin
    GetPosEx(CurPos, Block, Offset, Remain);

    if Block = Nil then
     Break;

    if Remain > Count - K then
     Remain:=Count - K;

    Move(Buffer[K], Block^.Buffer^[Offset], Remain);

    Inc(CurPos, Remain);

    Inc(K, Remain);
   end;
 end;

procedure TRamStream.CopyFrom(var S: TStream; Count: Longint);
 begin
  if CurPos + Count > CurSize then
   SetSize(CurPos + Count);

  if CurPos + Count > VirtualSize then
   VirtualSize:=CurPos + Count;

  inherited CopyFrom(S, Count);
 end;

procedure TRamStream.Flush;
 begin
 end;

procedure TRamStream.SetSize(NewSize: Longint);
 var
  NextBlockSize: Longint;
  Block, AnotherBlock, Last: PRamBlock;
  Temp: Pointer;
 begin
  NewSize:=(NewSize div BlockSize + 1) * BlockSize;

  if NewSize > CurSize then
   begin
    Last:=First;

    while Last^.Next <> Nil do
     Last:=Last^.Next;

    repeat
     NextBlockSize:=NewSize - CurSize;

     if NextBlockSize = 0 then
      Break;

     if NextBlockSize > BlockSize then
      NextBlockSize:=BlockSize;

     Block:=Last;

     if Block^.Size < NextBlockSize then
      begin
       Dec(CurSize, Block^.Size);
       Inc(CurSize, NextBlockSize);

       GetMem(Temp, Block^.Size);
       Move(Block^.Buffer^, Temp^, Block^.Size);
       FreeMem(Block^.Buffer, Block^.Size);

       GetMem(Block^.Buffer, NextBlockSize);
       Move(Temp^, Block^.Buffer^, Block^.Size);
       FreeMem(Temp, Block^.Size);

       Block^.Size:=NextBlockSize;
      end
     else
      begin
       New(AnotherBlock);

       AnotherBlock^.Size:=NextBlockSize;
       AnotherBlock^.Next:=Nil;

       GetMem(AnotherBlock^.Buffer, NextBlockSize);

       Block^.Next:=AnotherBlock;

       Inc(CurSize, AnotherBlock^.Size);

       Last:=AnotherBlock;
      end;
    until False;
   end else
  if NewSize < CurSize then
   repeat
    NextBlockSize:=CurSize - NewSize;

    if NextBlockSize = 0 then
     Break;

    Block:=First;

    while Block^.Next <> Nil do
     begin
      AnotherBlock:=Block;

      Block:=Block^.Next;
     end;

    if NextBlockSize > Block^.Size then
     begin
      AnotherBlock^.Next:=Nil;

      FreeMem(Block^.Buffer, Block^.Size);

      Dec(CurSize, Block^.Size);

      Dispose(Block);

      Last:=AnotherBlock;
     end
    else
     begin
      NextBlockSize:=Block^.Size - NextBlockSize;

      Dec(CurSize, Block^.Size);
      Inc(CurSize, NextBlockSize);

      GetMem(Temp, NextBlockSize);
      Move(Block^.Buffer^, Temp^, NextBlockSize);
      FreeMem(Block^.Buffer, Block^.Size);

      GetMem(Block^.Buffer, NextBlockSize);
      Move(Temp^, Block^.Buffer^, NextBlockSize);
      FreeMem(Temp, NextBlockSize);

      Block^.Size:=NextBlockSize;
     end;
   until False;

  CurSize:=NewSize;
 end;

procedure TRamStream.GetPosEx(Position: Longint; var Block: PRamBlock; var Offset, Remain: Longint);
 begin
  Block:=First;

  repeat
   if Position < Block^.Size then
    begin
     Offset:=Position;

     Remain:=Block^.Size - Position;

     Exit;
    end;

   Dec(Position, Block^.Size);

   Block:=Block^.Next;
  until Block = Nil;

  Offset:=-1;
  Remain:=-1;
 end;

end.
