library Scanner;

uses
     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     Objects,
     Strings,

     ScanAPI,
     ScanScan,
     ScanCmn,

     PrioAPI,

     Wizard;

const
 ScannerName                            = 'Scanner';
 ScannerVersion                         = 3;

 ScannerPlugin          : Pointer       = nil;

 ScanFirst              : Pointer       = nil;
 ScanLast               : Pointer       = nil;
 ScanDisable            : Pointer       = nil;
 ScanKillSent           : Pointer       = nil;
 ScanNoLock             : Pointer       = nil;
 ScanKillInExportList   : Pointer       = nil;
 ScanAddToImportList    : Pointer       = nil;
 ScanReport             : Pointer       = nil;

 ScanPerform            : Longint       = $00001000;

 ScanFirstTime          : Boolean       = True;

 sfNone                                 = $00000000;
 sfFull                                 = $00000001;
 sfKillSent                             = $00000002;
 sfNoLock                               = $00000004;
 sfKillInExportList                     = $00000008;
 sfAddToImportList                      = $00000010;

type
 PScanAreaList = ^TScanAreaList;
 TScanAreaList = object(TAreaList)
  procedure SetAdditionalFlags(const Area: Pointer; const AreaName: String; var Flags: Longint); virtual;
 end;

{ TScanAreaList }

procedure TScanAreaList.SetAdditionalFlags(const Area: Pointer; const AreaName: String; var Flags: Longint);
 begin
  if jt.rests.Check(ScanKillSent, nil, @AreaName) then
   Flags:=Flags or sfKillSent;

  if jt.rests.Check(ScanNoLock, nil, @AreaName) then
   Flags:=Flags or sfNoLock;

  if jt.rests.Check(ScanKillInExportList, nil, @AreaName) then
   Flags:=Flags or sfKillInExportList;

  if jt.rests.Check(ScanAddToImportList, nil, @AreaName) then
   Flags:=Flags or sfAddToImportList;
 end;

const
 ScanAreas              : PScanAreaList = nil;

{ RegisterCallback }

procedure RegisterCallback(const Block: TRegisterScannerCallbackBlock);
 begin
  Callbacks[Block.Action]^.Add(Block.Plugin);
 end;

{ ScanArea }

procedure ScanArea(const Area: Pointer; const Full, KillSent, Lock, KillInExportList, AddToImportList: Boolean;
  const ReportMode: TReportMode; const Current, Total: Longint);
 var
  Scanner: PScanner;
  Format: TAreaFormat;
 begin
  jt.area.GetFormat(Area, Format);

  Scanner:=New(PScanner, Init(Area, Full, KillSent, Lock, KillInExportList, AddToImportList, ReportMode, Current, Total));

  if Scanner <> nil then
   begin
    if ScanFirstTime then
     begin
      jt.LogWrite('S', 'Scanning areas');

      ScanFirstTime:=False;

      PerformPriorityAction('scanning');
     end;

    Scanner^.Scan;

    Dispose(Scanner, Done);
   end;
 end;

{ ScanReportValueParser }

procedure ScanReportValueParser(const Keyword, Linker, st: Pointer; Value: String); export;
 var
  ReportMode: TReportMode;
 begin
  StUpcaseEx(Value);

  if Value = 'NONE' then ReportMode:=rmNone else
  if Value = 'SHORT' then ReportMode:=rmShort else
  if Value = 'DETAIL' then ReportMode:=rmDetail else
   jt.linker.Error(Linker, 'Invalid ScanReport restriction value (NONE, SHOT or DETAIL expected)');

  jt.st.Add(st, 'ReportMode', Pointer(ReportMode), nil);
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 var
  ProhibitFormats: PNumbers;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, ScannerName, ScannerVersion);

  ScannerPlugin:=Plugin;

  ScanFirst:=jt.rests.Add('ScanFirst', rsMinus);
  ScanLast:=jt.rests.Add('ScanLast', rsMinus);
  ScanDisable:=jt.rests.Add('ScanDisable', rsMinus);
  ScanKillSent:=jt.rests.Add('ScanKillSent', rsMinus);
  ScanNoLock:=jt.rests.Add('ScanNoLock', rsMinus);
  ScanKillInExportList:=jt.rests.Add('ScanKillInExportList', rsMinus);
  ScanAddToImportList:=jt.rests.Add('ScanAddToImportList', rsMinus);
  ScanReport:=jt.rests.AddValueParser('ScanReport', rsMinus or rsValue or rsStorage, ScanReportValueParser);

  ProhibitFormats:=New(PNumbers, Init($2, $2));
  ProhibitFormats^.Add(afPassthrough);

  ScanAreas:=New(PScanAreaList, Init(ScanFirst, ScanLast, ScanDisable, ProhibitFormats, asNetmail or asLocal));

  PluginSetup:=True;

  RegisterPriorityAction('scanning');
 end;

{ PluginStartup }

procedure PluginStartup; export;
 var
  Action: TScannerAction;
 begin
  if ScanAreas^.Full then
   ScanAreas^.AddList(jt.list.ExportList, ScanAreas^.FullFlags);

  if ScanAreas^.HaveSense then
   begin
    ScanAreas^.Prepare;

    jt.EnqueueCommand(ScannerPlugin, ScanPerform, 0, cwScan);
   end;

  for Action:=Low(Action) to High(Action) do
   Callbacks[Action]:=New(PCallbacks, Init(StrPas(ScannerActionNames[Action])));
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 var
  Action: TScannerAction;
 begin
  for Action:=Low(Action) to High(Action) do
   Dispose(Callbacks[Action], Done);

  Dispose(ScanAreas, Done);
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 function PerformScan(const Area: PAreaListItem): Boolean; far;
  var
   ReportMode: TReportMode;
   st: Pointer;
  begin
   if jt.Cancelled then
    PerformScan:=True
   else
    begin
     jt.rests.Getst(ScanReport, nil, jt.area.GetName(Area^.Area), st);

     if st = nil then
      ReportMode:=rmShort
     else
      ReportMode:=TReportMode(jt.st.Get(st, 'ReportMode'));

     ScanArea(Area^.Area, Area^.Flags and sfFull <> 0,
                          Area^.Flags and sfKillSent <> 0,
                          Area^.Flags and sfNoLock = 0,
                          Area^.Flags and sfKillInExportList <> 0,
                          Area^.Flags and sfAddToImportList <> 0,
                          ReportMode,
                          ScanAreas^.IndexOf(Area), ScanAreas^.Count);

     PerformScan:=jt.Cancelled;
    end;
  end;
 begin
  if Command = scRegisterCallback then RegisterCallback(PRegisterScannerCallbackBlock(Param)^) else
  if Command = ScanPerform then
   if ScanAreas^.Count = 0 then
    jt.LogWrite('S', #0'Nothing to scan.')
   else
    begin
     jt.PushTitle;

     ScanAreas^.FirstThat(@PerformScan);

     jt.PopTitle;
    end;
 end;

{ PluginHandleCmdlineSwitch }

function PluginHandleCmdlineSwitch(Switch: String): Boolean; export;
 procedure Full(const Flags: Longint);
  begin
   ScanAreas^.AddFull(Flags, jt.linker.Stub);
  end;
 procedure Mask(const Skip: Byte; const Flags: Longint);
  begin
   ScanAreas^.AddMask(Copy(Switch, Skip + 1, 255), Flags, jt.linker.Stub);
  end;
 begin
  StUpcaseEx(Switch);

  PluginHandleCmdlineSwitch:=True;

  if Switch = 'SCAN' then Full(sfNone) else
  if Switch = 'SCANFORCE' then Full(alifIgnoreDisable) else
  if Switch = 'SCANFULL' then Full(sfFull) else
  if (Switch = 'SCANFORCEFULL') or (Switch = 'SCANFULLFORCE') then Full(alifIgnoreDisable or sfFull) else
  if Copy(Switch, 1, 5) = 'SCAN:' then Mask(5, sfNone) else
  if Copy(Switch, 1, 10) = 'SCANFORCE:' then Mask(10, alifIgnoreDisable) else
  if Copy(Switch, 1, 9) = 'SCANFULL:' then Mask(9, sfFull) else
  if (Copy(Switch, 1, 14) = 'SCANFORCEFULL:') or (Copy(Switch, 1, 14) = 'SCANFULLFORCE:') then
   Mask(14, alifIgnoreDisable or sfFull) else
  if Switch = 'DEBUGSCANHIGHWATER' then HighwaterDebugger.Enabled:=True else
   PluginHandleCmdlineSwitch:=False;
 end;

{ PluginHandleAreaSwitch }

function PluginHandleAreaSwitch(const Area: Pointer; const Keyword: String; Value: String;
  const Linker: Pointer): Boolean; export;
 begin
  if Keyword = 'LASTSCANNED' then
   begin
    jt.st.Add(jt.area.st(Area), 'LastScanned', Pointer(jt.linker.ParseNumber(Linker, Value)), nil);

    PluginHandleAreaSwitch:=True;
   end
  else
   PluginHandleAreaSwitch:=False;
 end;

{ PluginExportAreaSwitches }

procedure PluginExportAreaSwitches(const Area, Linker: Pointer); export;
 var
  LastScanned: Longint;
 begin
  LastScanned:=Longint(jt.st.Get(jt.area.st(Area), 'LastScanned'));

  if (LastScanned <> 0) and (LastScanned <> -1) then
   jt.linker.ExportLine(Linker, ' LastScanned ' + jt.linker.UnparseNumber(Linker, LastScanned));
 end;

{ PluginShowHelp }

procedure PluginShowHelp; export;
 begin
  jt.PrintStr(' SCAN[:re]               quick scan exportlist or regexp if not omitted'#13#10);
  jt.PrintStr(' SCANFORCE[:re]          quick scan, ScanDisable have no influence'#13#10);
  jt.PrintStr(' SCANFULL[:re]           full scan exportlist or regexp if not omitted'#13#10);
  jt.PrintStr(' SCANFORCEFULL[:re]      full scan, ScanDisable have no influence'#13#10);
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleCmdlineSwitch,
 PluginHandleAreaSwitch,
 PluginExportAreaSwitches,
 PluginShowHelp;

begin
 Assign(Output, '');
 Rewrite(Output);
end.
