unit ScrState;

{
  ScreenState saver/restorer
  version 1.3

  (c) by sk // [http://sk.elk.ru] [2:6033/27], 1998-2001.

  Compilers supported:
   - Borland Pascal 7.0 [DOS, DPMI targets]
   - Virtual Pascal 2.1 [OS/2, WIN32 targets]
}

{$IFDEF DPMI}
 {$DEFINE DOS}
{$ENDIF}
{$IFDEF MSDOS}
 {$DEFINE DOS}
{$ENDIF}

{$IFDEF DELPHI}
This unit cannot be compiled using Delphi!
{$ENDIF}

interface
uses
{$IFDEF WIN32}
     Windows,
     vpSysLow,
{$ENDIF}
{$IFDEF OS2}
     OS2Base,
{$ENDIF}
     Objects;

type
 PScreenState = ^TScreenState;
 TScreenState = record
  Cursor: TPoint;
  BufferSize: Word;
  Buffer: Pointer;
{$IFDEF WIN32}
  UpLeft, Size: TCoord;
  ReadFrom: TSmallRect;
{$ENDIF}
 end;

procedure SaveScreenState(var State: PScreenState);
procedure LoadScreenState(var State: PScreenState);

implementation

{// OS/2 target }

{$IFDEF OS2}
procedure SaveScreenState(var State: PScreenState);
 var
  P: Pointer;
  X, Y: SmallWord;
 begin
  New(State);

  VioGetBuf(P, State^.BufferSize, 0);

  GetMem(State^.Buffer, State^.BufferSize);

  SelToFlat(P);

  Move(P^, State^.Buffer^, State^.BufferSize);

  VioGetCurPos(Y, X, 0);

  State^.Cursor.X:=X;
  State^.Cursor.Y:=Y;
 end;

procedure LoadScreenState(var State: PScreenState);
 var
  P: Pointer;
 begin
  if State = nil then
   Exit;

  VioGetBuf(P, State^.BufferSize, 0);

  SelToFlat(P);

  Move(State^.Buffer^, P^, State^.BufferSize);

  VioShowBuf(0, State^.BufferSize, 0);

  VioSetCurPos(State^.Cursor.Y, State^.Cursor.X, 0);

  FreeMem(State^.Buffer, State^.BufferSize);

  Dispose(State);

  State:=nil;
 end;
{$ENDIF}

{// DOS target }

{$IFDEF DOS}
procedure SaveScreenState(var State: PScreenState);
 var
  X, Y: Byte;
 begin
  New(State);

  State^.BufferSize:=16384;

  GetMem(State^.Buffer, State^.BufferSize);

  Move(Ptr(SegB800, 0)^, State^.Buffer^, State^.BufferSize);

  asm
   push ds
   push bp

   mov  ah, 03h
   xor  bh, bh
   int  10h

   pop  bp
   pop  ds

   mov  X, dl
   mov  Y, dh
  end;

  State^.Cursor.X:=X;
  State^.Cursor.Y:=Y;
 end;

procedure LoadScreenState;
 var
  X, Y: Byte;
 begin
  if State = nil then
   Exit;

  X:=State^.Cursor.X;
  Y:=State^.Cursor.Y;

  asm
   push ds
   push bp

   mov  ah, 02h
   xor  bh, bh
   mov  dl, X
   mov  dh, Y

   int  10h

   pop  bp
   pop  ds
  end;

  Move(State^.Buffer^, Ptr(SegB800, 0)^, State^.BufferSize);

  FreeMem(State^.Buffer, State^.BufferSize);

  Dispose(State);

  State:=nil;
 end;
{$ENDIF}

{// Win32 target }

{$IFDEF WIN32}
type
 PWin32Cell = ^TWin32Cell;
 TWin32Cell = record
  Ch: SmallWord;
  Attr: SmallWord;
 end;

 PStandardCell = ^TStandardCell;
 TStandardCell = packed record
  Ch: Char;
  Attr: Byte;
 end;

procedure SaveScreenState(var State: PScreenState);
 var
  Buffer, PSource: PWin32Cell;
  BufferSize, X, Y: Longint;
  PDestination: PStandardCell;
  Info: TConsoleScreenBufferInfo;
 begin
  New(State);

  { cursor }

  GetConsoleScreenBufferInfo(SysFileStdOut, Info);

  State.Cursor.X:=Info.dwCursorPosition.X;
  State.Cursor.Y:=Info.dwCursorPosition.Y;

  State.BufferSize:=(Info.dwSize.Y + 1) * (Info.dwSize.X + 1) * SizeOf(TStandardCell);

  BufferSize:=(Info.dwSize.Y + 1) * (Info.dwSize.X + 1) * SizeOf(TWin32Cell);

  GetMem(Buffer, BufferSize);
  GetMem(State.Buffer, State.BufferSize);

  State.Size.X:=Info.dwSize.X;
  State.Size.Y:=Info.dwSize.Y;

  State.UpLeft.X:=0;
  State.UpLeft.Y:=0;

  State.ReadFrom.Left:=0;
  State.ReadFrom.Top:=0;

  State.ReadFrom.Right:=Info.dwSize.X;
  State.ReadFrom.Bottom:=Info.dwSize.Y;

  ReadConsoleOutput(SysFileStdOut, Buffer, State.Size, State.UpLeft, State.ReadFrom);

  PSource:=Buffer;
  PDestination:=State.Buffer;

  for Y:=0 to Info.dwSize.Y - 1 do
   for X:=0 to info.dwSize.X - 1 do
    begin
     PDestination^.Ch:=Chr(PSource^.Ch);
     PDestination^.Attr:=Byte(PSource^.Attr);

     Inc(PSource);
     Inc(PDestination);
    end;

  FreeMem(Buffer, BufferSize);
 end;

procedure LoadScreenState(var State: PScreenState);
 var
  Buffer: Pointer;
  BufferSize, X, Y: Longint;
  PSource: PStandardCell;
  PDestination: PWin32Cell;
  Coord: TCoord;
 begin
  if State = nil then
   Exit;

  BufferSize:=State.BufferSize div SizeOf(TStandardCell) * SizeOf(TWin32Cell);

  GetMem(Buffer, BufferSize);

  PSource:=State.Buffer;
  PDestination:=Buffer;

  for Y:=0 to State.Size.Y - 1 do
   for X:=0 to State.Size.X - 1 do
    begin
     PDestination^.Ch:=Ord(PSource^.Ch);
     PDestination^.Attr:=PSource^.Attr;

     Inc(PSource);
     Inc(PDestination);
    end;

  WriteConsoleOutput(SysFileStdOut, Buffer, State.Size, State.UpLeft, State.ReadFrom);

  {$IFDEF VIRTUALPASCAL}
  SysTVSetCurPos(State.Cursor.X, State.Cursor.Y);
  {$ELSE}
  Coord.X:=State.Cursor.X;
  Coord.Y:=State.Cursor.Y;

  SetConsoleCursorPosition(SysFileStdOut, Coord);
  {$ENDIF}

  FreeMem(Buffer, BufferSize);
  FreeMem(State^.Buffer, State^.BufferSize);

  Dispose(State);

  State:=nil;
 end;
{$ENDIF}

end.