library Splitter;

uses
     Objects,

     PlugAPI,
     PlugSIDE,

     RStream,

     Thunks,

     ccAPI,

     jtmhlCmn,

     Wizard;

const
 SplitterPlugin         : Pointer = nil;

type
 PSections = ^TSections;
 
 PSection = ^TSection;
 TSection = object(TObject)
  Start, Len: Longint;

  constructor Init(const AStart, ALen: Longint);
  procedure PatchSubject(const Sections: PSections; const msg: Pointer);
 end;

 TSections = object(TCollection)
  Prefix, Suffix, Text: PStream;
  msg: Pointer;

  constructor Init(const Amsg: Pointer; const SectionSize: Longint);
  destructor Done; virtual;
 public
  procedure TossSection(const Section: PSection; const ChangeMSGID: Boolean);
  procedure Crop;
 end;

 PTossAndSplit = ^TTossAndSplit;
 TTossAndSplit = object(TccAction)
  msg_, Size_, ChangeMSGID_: Pointer;

  constructor Init(const Tokenizer, Actions: Pointer; var Atom: Pointer);
  function Execute(const Data: Pointer): TExecuteResult; virtual;
  destructor Done; virtual;
 end;

{ TSection }

constructor TSection.Init(const AStart, ALen: Longint);
 begin
  inherited Init;

  Start:=AStart;
  Len:=ALen;
 end;

procedure TSection.PatchSubject(const Sections: PSections; const msg: Pointer);
 var
  Subject: PString;
  Suffix: String;
 begin
  Subject:=jt.msg.GetSubject(msg);

  Suffix:=Long2Str(Sections^.IndexOf(@Self) + 1) + '/' + Long2Str(Sections^.Count);

  if Subject = nil then
   jt.msg.SetSubject(msg, Suffix)
  else
   jt.msg.SetSubject(msg, Subject^ + ' ' + Suffix);
 end;

{ TSections }

constructor TSections.Init(const Amsg: Pointer; const SectionSize: Longint);
 var
  Anchor, Anchor2, LowBound, HighBound: Longint;
  msgp: Pointer;
  S: String;
 begin
  inherited Init($10, $10);

  msg:=Amsg;

  Prefix:=New(PRamStream, Init);

  Suffix:=New(PRamStream, Init);

  Text:=New(PStreamThunk, Init(jt.msg.GetText(msg), jt.StreamLinker, False));

  msgp:=jt.msg.CreateParser(msg, False);

  { put kludges in Prefix }

  Anchor:=jt.msg.p.AfterLastKludge(msgp);

  Text^.Seek(0);

  Prefix^.CopyFrom(Text^, Anchor);

  { put tearline & origin in Suffix }

  Anchor:=jt.msg.p.GetTextSize(msgp) - 128 * 2;

  if Anchor < 0 then
   Anchor:=0;

  jt.msg.p.SetTextPos(msgp, Anchor);

  Anchor:=-1;

  while not jt.msg.p.EndOfMessage(msgp) do
   begin
    Anchor2:=jt.msg.p.GetTextPos(msgp);

    jt.msg.p.GetString(msgp, S);

    if (S = '---') or (Copy(S, 1, 4) = '--- ') or ((Copy(S, 1, 10) = ' * Origin:') and (Anchor = -1)) then
     Anchor:=Anchor2;
   end;

  if Anchor <> -1 then
   begin
    Text^.Seek(Anchor);

    Suffix^.CopyFrom(Text^, Text^.GetSize - Anchor);
   end;

  { define working bounds }

  LowBound:=Prefix^.GetSize;
  HighBound:=Text^.GetSize - Suffix^.GetSize;

  if (HighBound > LowBound) and (HighBound - LowBound > SectionSize) then
   begin
    Anchor:=LowBound;

    repeat
     Anchor2:=Anchor + SectionSize;

     if Anchor2 > HighBound then
      Anchor2:=HighBound;

     if Anchor >= Anchor2 then
      Break;

     if Anchor2 <> HighBound then
      begin
       jt.msg.p.SetTextPos(msgp, Anchor2);

       jt.msg.p.GetString(msgp, S);

       Anchor2:=jt.msg.p.GetTextPos(msgp);
      end;

     Insert(New(PSection, Init(Anchor, Anchor2 - Anchor)));

     Anchor:=Anchor2;
    until False;
   end;

  jt.msg.DestroyParser(msgp);
 end;

destructor TSections.Done;
 begin
  Dispose(Text, Done);

  Dispose(Suffix, Done);

  Dispose(Prefix, Done);

  inherited Done;
 end;

{ public methods }

procedure TSections.TossSection(const Section: PSection; const ChangeMSGID: Boolean);
 var
  smsg, smsgp: Pointer;
  smsgText: PStream;
  S: String;
 begin
  smsg:=jt.msg.Clone(msg, mcfCloneHeader);

  Section^.PatchSubject(@Self, smsg);

  smsgText:=New(PStreamThunk, Init(jt.msg.GetText(smsg), jt.StreamLinker, False));

  Prefix^.Seek(0);

  smsgText^.CopyFrom(Prefix^, Prefix^.GetSize);

  if ChangeMSGID then
   begin
    smsgp:=jt.msg.CreateParser(smsg, False);

    if jt.msg.p.GetKludge(smsgp, #1'MSGID', S) then
     jt.msg.p.SetKludge(smsgp, #1'MSGID', #1'MSGID: ' + ExtractWord(2, S, [' ']) + ' ' + GenerateMSGID);

    jt.msg.DestroyParser(smsgp);
   end;

  Text^.Seek(Section^.Start);

  smsgText^.CopyFrom(Text^, Section^.Len);

  Suffix^.Seek(0);

  smsgText^.CopyFrom(Suffix^, Suffix^.GetSize);

  Dispose(smsgText, Done);

  jt.st.Add(jt.msg.st(smsg), 'Splitted', Pointer(1), nil);

  jt.tossscan.TossMessage(smsg);
 end;

procedure TSections.Crop;
 var
  First: PSection;
 begin
  First:=At(0);

  First^.PatchSubject(@Self, msg);

  Text^.Seek(Prefix^.GetSize + First^.Len);

  Suffix^.Seek(0);

  Text^.CopyFrom(Suffix^, Suffix^.GetSize);

  Text^.Truncate;
 end;

{ TTossAndSplit }

constructor TTossAndSplit.Init(const Tokenizer, Actions: Pointer; var Atom: Pointer);
 begin
  inherited Init('TOSSANDSPLIT', Actions, Atom);

  cc.TokenizerNext(Tokenizer, [tmOpenBracket]);

  msg_:=cc.ParseNumberExpression(Tokenizer, Actions, True, []);

  cc.TokenizerNext(Tokenizer, [tmComma]);

  Size_:=cc.ParseNumberExpression(Tokenizer, Actions, True, []);

  cc.TokenizerNext(Tokenizer, [tmComma]);

  ChangeMSGID_:=cc.ParseBooleanExpression(Tokenizer, Actions, True);

  cc.TokenizerNext(Tokenizer, [tmCloseBracket]);
 end;

function TTossAndSplit.Execute(const Data: Pointer): TExecuteResult;
 var
  Sections: PSections;
  ChangeMSGID, AddKludge: Boolean;
  Size, K: Longint;
  msg: Pointer;
  S: PStream;
 procedure Dump(const Section: PSection); far;
  begin
   jt.LogWrite('S', 'Section at ' + Long2Str(Section^.Start) + ', len ' + Long2Str(Section^.Len));
  end;
 begin
  Execute:=erOk;

  cc.ExpressionGet(msg_, Data, msg);

  if LongBool(jt.st.Get(jt.msg.st(msg), 'Splitted')) then
   Exit;

  cc.ExpressionGet(Size_, Data, Size);
  cc.ExpressionGet(ChangeMSGID_, Data, ChangeMSGID);

  Sections:=New(PSections, Init(msg, Size));

  if Sections^.Count > 1 then
   begin
    for K:=1 to Sections^.Count - 1 do
     Sections^.TossSection(Sections^.At(K), ChangeMSGID);

    Sections^.Crop;
   end;

  Dispose(Sections, Done);
{
  jt.LogWrite('~', 'Toss&Split: ' + HexPtr(msg) + '/' + Long2Str(Size) + '/' + Long2Str(Byte(ChangeMSGID)));
}
 end;

destructor TTossAndSplit.Done;
 begin
  cc.DisposeObject(ChangeMSGID_);
  cc.DisposeObject(msg_);
  cc.DisposeObject(Size_);

  inherited Done;
 end;

{ TossAndSplitCreator }

function TossAndSplitCreator(const ATokenizer, AActions: Pointer): Pointer; export;
 var
  Atom: Pointer;
  TossAndSplit: PTossAndSplit;
 begin
  New(TossAndSplit, Init(ATokenizer, AActions, Atom));

  TossAndSplitCreator:=Atom;
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, 'Splitter', 1);

  SplitterPlugin:=Plugin;

  PluginSetup:=True;

  InitCCAPI;

  if not ccAvailable then
   jt.linker.Error(jt.linker.Stub, 'splitter.dll requires cc.dll loaded');

  cc.AddAction(nil, 'TOSSANDSPLIT', TossAndSplitCreator);
 end;

exports
 PluginSetup;

begin
 Assign(Output, '');
 Rewrite(Output);
end.