unit stpl;

interface
uses
     Objects,

     PlugSIDE,

     Wizard;

type
 PSimpleTemplatePending = ^TSimpleTemplatePending;
 TSimpleTemplatePending = object(TObject)
  Command: PString;
  Unlock: Boolean;

  constructor Init(const ACommand: String; const AUnlock: Boolean);
  destructor Done; virtual;
 end;

 PSimpleTemplatePendings = ^TSimpleTemplatePendings;
 TSimpleTemplatePendings = object(TCollection)
 end;

 PSimpleTemplate = ^TSimpleTemplate;
 TSimpleTemplate = object(TObject)
  Source, Target: PStrings;
  Pendings: PSimpleTemplatePendings;
  Stop: Boolean;
  CurrentLine: Longint;

  constructor Init(const ASource, ATarget: PStrings);
  procedure Process; virtual;
  procedure ProcessLine(const Line: String); virtual;
  procedure ProcessCommand(const Command, Value: String); virtual;
  procedure AddPendingCommand(const Command: String; const ALock, AUnlock: Boolean); virtual;
  procedure Lock; virtual;
  procedure Unlock; virtual;
  destructor Done; virtual;
 public
  procedure Error(const Message: String); virtual;
  function IsOperableCommand(const Command: String): Boolean; virtual;
  function IsPendingCommand(const Command: String): Boolean; virtual;
  procedure PerformOperableCommand(const Command: String; Value: String); virtual;
  procedure PerformPendingCommand(const Pending: PSimpleTemplatePending; const Command: String; Value: String); virtual;
 private
  LockCount: Longint;
 end;

implementation

{ TSimpleTemplatePending }

constructor TSimpleTemplatePending.Init(const ACommand: String; const AUnlock: Boolean);
 begin
  inherited Init;

  Command:=NewStr(ACommand);
  Unlock:=AUnlock;
 end;

destructor TSimpleTemplatePending.Done;
 begin
  DisposeStr(Command);

  inherited Done;
 end;

{ TSimpleTemplate }

constructor TSimpleTemplate.Init(const ASource, ATarget: PStrings);
 begin
  inherited Init;

  Source:=ASource;
  Target:=ATarget;

  Pendings:=New(PSimpleTemplatePendings, Init($10, $10));
 end;

procedure TSimpleTemplate.Process;
 function Perform(const Line: PString): Boolean; far;
  var
   K: Byte;
  begin
   Inc(CurrentLine);

   if Line = nil then
    ProcessLine('')
   else
    if Line^[1] <> ';' then
     if Line^[1] = '$' then
      begin
       K:=Pos(' ', Line^);

       if K = 0 then
        ProcessCommand(StUpcase(Copy(Line^, 2, 255)), '')
       else
        ProcessCommand(StUpcase(Copy(Line^, 2, K - 2)), Copy(Line^, K + 1, 255));
      end
     else
      ProcessLine(Line^);

   Perform:=Stop;
  end;
 begin
  CurrentLine:=0;

  if Source^.FirstThat(@Perform) = nil then
   if Pendings^.Count <> 0 then
    Error('unresolved pendings');
 end;

procedure TSimpleTemplate.ProcessLine(const Line: String);
 begin
  if LockCount = 0 then
   Target^.Insert(NewStr(Line));
 end;

procedure TSimpleTemplate.ProcessCommand(const Command, Value: String);
 var
  Pending: PSimpleTemplatePending;
 begin
  if IsOperableCommand(Command) then PerformOperableCommand(Command, Value) else
  if IsPendingCommand(Command) then
   begin
    Pending:=Pendings^.At(Pendings^.Count - 1);

    if Pending = nil then
     Error('pending command got but not expected')
    else
     if Pending^.Command^ = Command then
      begin
       PerformPendingCommand(Pending, Command, Value);

       Pendings^.AtFree(Pendings^.Count - 1);
      end
     else
      Error('expected ' + Pending^.Command^ + ' pending command, got ' + Command);
   end
  else
   Error('unknown command "' + Command + '"');
 end;

procedure TSimpleTemplate.AddPendingCommand(const Command: String; const ALock, AUnlock: Boolean);
 begin
  Pendings^.Insert(New(PSimpleTemplatePending, Init(Command, AUnlock)));

  if ALock then
   Lock;
 end;

procedure TSimpleTemplate.Lock;
 begin
  Inc(LockCount);
 end;

procedure TSimpleTemplate.Unlock;
 begin
  Dec(LockCount);
 end;

destructor TSimpleTemplate.Done;
 begin
  Dispose(Pendings, Done);

  inherited Done;
 end;

{ public methods }

procedure TSimpleTemplate.Error(const Message: String);
 begin
  Stop:=True;
 end;

function TSimpleTemplate.IsOperableCommand(const Command: String): Boolean;
 begin
  Abstract;
 end;

function TSimpleTemplate.IsPendingCommand(const Command: String): Boolean;
 begin
  Abstract;
 end;

procedure TSimpleTemplate.PerformOperableCommand(const Command: String; Value: String);
 begin
  Abstract;
 end;

procedure TSimpleTemplate.PerformPendingCommand(const Pending: PSimpleTemplatePending; const Command: String; Value: String);
 begin
  if Pending^.Unlock then
   Unlock;
 end;

end.