library TechInfo;

uses
     PlugAPI,
     PlugSIDE,

     TossAPI,

     Dos,

     Objects,

     jtMHLcmn,

     Wizard;

const
 tiName                                 = 'TechInfo';
 tiVersion                              = 1;

 tiPlugin               : Pointer       = nil;

 tiAddSeenBy            : Pointer       = nil; { Toss }
 tiAddPath              : Pointer       = nil;

 tiKillSeenBysInBase    : Pointer       = nil; { TossBase }
 tiKillPathInBase       : Pointer       = nil;

{ stAddresses }

function stAddresses(const Action: TStorageAction; const Data: Pointer): Pointer; export;
 var
  Addresses: PAddresses absolute Data;
 begin
  case Action of
   saDestroy: Dispose(Addresses, Done);
   saClone: stAddresses:=nil;
   saInfo: stAddresses:=jt.NewStr('addresses');
  end;
 end;

{ AddressesValueParser }

procedure AddressesValueParser(const Keyword, Linker, st: Pointer; Value: String); export;
 var
  Addresses: PAddresses;
  Address: PAddress;
  S: String;
  K: Integer;
 begin
  Addresses:=jt.st.Get(st, 'Addresses');

  if Addresses = nil then
   begin
    Addresses:=New(PAddresses, Init($1, $4));

    jt.st.Add(st, 'Addresses', Addresses, stAddresses);
   end;

  for K:=1 to WordCount(Value, [',']) do
   begin
    S:=ExtractWord(K, Value, [',']);

    New(Address);

    jt.core.GetMainAddress(Address^);

    if not StrToAddress(S, Address^) then
     jt.linker.Error(Linker, 'Failed to parse address "' + S + '"');

    Addresses^.Insert(Address);
   end;
 end;

{ AddSeenbyCallback }

function AddSeenbyCallback(const st, Param: Pointer; const Value: String): Boolean; export;
 var
  msg: Pointer absolute Param;
  seenby: Pointer;
 procedure AddSeenby(const Address: PAddress); far;
  var
   TechInfo: PTechInfo;
   Index: xInteger;
  begin
   TechInfo:=jt.msg.ti.CreateEx(Address^.Net, Address^.Node);

   if jt.msg.ti.seenby.Search(seenby, TechInfo^, Index) <> nil then
    jt.msg.ti.Destroy(TechInfo)
   else
    jt.msg.ti.seenby.Insert(seenby, TechInfo);
  end;
 var
  Addresses: PAddresses;
 begin
  Addresses:=jt.st.Get(st, 'Addresses');

  if Addresses <> nil then
   begin
    seenby:=jt.st.Get(jt.msg.st(msg), 'Seenby');

    if seenby <> nil then
     Addresses^.ForEach(@AddSeenby);
   end;

  AddSeenbyCallback:=False;
 end;

{ AddPathCallback }

function AddPathCallback(const st, Param: Pointer; const Value: String): Boolean; export;
 var
  msg: Pointer absolute Param;
  path: Pointer;
 procedure AddPath(const Address: PAddress); far;
  begin
   jt.msg.ti.path.Insert(path, jt.msg.ti.path.Count(path), jt.msg.ti.CreateEx(Address^.Net, Address^.Node));
  end;
 var
  Addresses: PAddresses;
 begin
  Addresses:=jt.st.Get(st, 'Addresses');

  if Addresses <> nil then
   begin
    path:=jt.st.Get(jt.msg.st(msg), 'Path');

    if path <> nil then
     Addresses^.ForEach(@AddPath);
   end;

  AddPathCallback:=False;
 end;

{ MessageToss }

procedure MessageToss(var Block: TTossMessageBlock);
 begin
  jt.rests.Callback(tiAddSeenby, nil, jt.area.GetName(Block.Area), AddSeenbyCallback, Block.msg);
  jt.rests.Callback(tiAddPath, nil, jt.area.GetName(Block.Area), AddPathCallback, Block.msg);
 end;

{ MessageTossBase }

procedure MessageTossBase(var Block: TTossMessageBlock);
 begin
  if jt.rests.Check(tiKillSeenbysInBase, nil, jt.area.GetName(Block.Area)) then
   jt.st.Remove(jt.msg.st(Block.msg), 'Seenby');

  if jt.rests.Check(tiKillPathInBase, nil, jt.area.GetName(Block.Area)) then
   jt.st.Remove(jt.msg.st(Block.msg), 'Path');
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, tiName, tiVersion);

  tiPlugin:=Plugin;

  PluginSetup:=True;

  tiAddSeenby:=jt.rests.AddValueParser('AddSeenby', rsValue or rsMinus or rsStorage, AddressesValueParser);
  tiAddPath:=jt.rests.AddValueParser('AddPath', rsValue or rsMinus or rsStorage, AddressesValueParser);

  tiKillSeenbysInBase:=jt.rests.Add('KillSeenbysInBase', rsMinus);
  tiKillPathInBase:=jt.rests.Add('KillPathInBase', rsMinus);
 end;

{ PluginStartup }

procedure PluginStartup; export;
 begin
  if jt.rests.GetUsed(tiAddSeenby) + jt.rests.GetUsed(tiAddPath) > 0 then
   RegisterTosserCallback(tiPlugin, taMessageTossLast);

  if jt.rests.GetUsed(tiKillSeenbysInBase) + jt.rests.GetUsed(tiKillPathInBase) > 0 then
   RegisterTosserCallback(tiPlugin, taMessageTossBaseLast);
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
  DisposeTosserCallbacks;
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 begin
  if Command = tcMessageTossBaseLast then MessageTossBase(PTossMessageBlock(Param)^) else
  if Command = tcMessageTossLast then MessageToss(PTossMessageBlock(Param)^);
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand;

begin
 Assign(Output, '');
 Rewrite(Output);
end.