{&StdCall-}
unit Thunks;

interface
uses
     Objects,

     Wizard;

type
 PStreamLinker = ^TStreamLinker;
 TStreamLinker = packed record
  Done: procedure(const Stream: Pointer);
  Flush: function(const Stream: Pointer): xInteger;
  GetPos: function(const Stream: Pointer): Longint;
  GetSize: function(const Stream: Pointer): Longint;
  Read: function(const Stream: Pointer; var Buf; Count: xWord): xInteger;
  Reset: function(const Stream: Pointer): xInteger;
  Seek: function(const Stream: Pointer; const Pos: Longint): xInteger;
  Truncate: function(const Stream: Pointer): xInteger;
  Write: function(const Stream: Pointer; var Buf; Count: xWord): xInteger;
  Status: function(const Stream: Pointer): xInteger;
 end;

 PStreamThunk = ^TStreamThunk;
 TStreamThunk = object(TStream)
  Stream: Pointer;
  Linker: PStreamLinker;
  Destroy: Boolean;

  constructor Init(const AStream: Pointer; const ALinker: PStreamLinker; const ADestroy: Boolean);
  procedure Flush; virtual;
  function GetPos: Longint; virtual;
  function GetSize: Longint; virtual;
  procedure Read(var Buf; Count: xWord); virtual;
  procedure Reset; virtual;
  procedure Seek(Pos: Longint); virtual;
  procedure Truncate; virtual;
  procedure Write(var Buf; Count: xWord); virtual;
  destructor Done; virtual;
 end;

procedure slDone(const Stream: Pointer); export;
function slFlush(const Stream: Pointer): xInteger; export;
function slGetPos(const Stream: Pointer): Longint; export;
function slGetSize(const Stream: Pointer): Longint; export;
function slRead(const Stream: Pointer; var Buf; Count: xWord): xInteger; export;
function slReset(const Stream: Pointer): xInteger; export;
function slSeek(const Stream: Pointer; const Pos: Longint): xInteger; export;
function slTruncate(const Stream: Pointer): xInteger; export;
function slWrite(const Stream: Pointer; var Buf; Count: xWord): xInteger; export;
function slStatus(const Stream: Pointer): xInteger; export;

const
 StreamLinkerRecord: TStreamLinker = (
  Done: slDone;
  Flush: slFlush;
  GetPos: slGetPos;
  GetSize: slGetSize;
  Read: slRead;
  Reset: slReset;
  Seek: slSeek;
  Truncate: slTruncate;
  Write: slWrite;
  Status: slStatus);

 StreamLinker: PStreamLinker = @StreamLinkerRecord;

implementation

{ TStreamThunk }

constructor TStreamThunk.Init(const AStream: Pointer; const ALinker: PStreamLinker; const ADestroy: Boolean);
 begin
  inherited Init;

  Stream:=AStream;
  Linker:=ALinker;
  Destroy:=ADestroy;

  Status:=Linker^.Status(Stream);
 end;

procedure TStreamThunk.Flush;
 begin
  Status:=Linker^.Flush(Stream);
 end;

function TStreamThunk.GetPos: Longint;
 begin
  GetPos:=Linker^.GetPos(Stream);
 end;

function TStreamThunk.GetSize: Longint;
 begin
  GetSize:=Linker^.GetSize(Stream);
 end;

procedure TStreamThunk.Read(var Buf; Count: xWord);
 begin
  Status:=Linker^.Read(Stream, Buf, Count);
 end;

procedure TStreamThunk.Reset;
 begin
  Status:=Linker^.Reset(Stream);
 end;

procedure TStreamThunk.Seek(Pos: Longint);
 begin
  Status:=Linker^.Seek(Stream, Pos);
 end;

procedure TStreamThunk.Truncate;
 begin
  Status:=Linker^.Truncate(Stream);
 end;

procedure TStreamThunk.Write(var Buf; Count: xWord);
 begin
  Status:=Linker^.Write(Stream, Buf, Count);
 end;

destructor TStreamThunk.Done;
 begin
  if Destroy then
   Linker^.Done(Stream);

  inherited Done;
 end;

{ stream linker thunk }

procedure slDone(const Stream: Pointer);
 begin
  Dispose(PStream(Stream), Done);
 end;

function slFlush(const Stream: Pointer): xInteger;
 begin
  PStream(Stream)^.Flush;

  slFlush:=PStream(Stream)^.Status;
 end;

function slGetPos(const Stream: Pointer): Longint;
 begin
  slGetPos:=PStream(Stream)^.GetPos;
 end;

function slGetSize(const Stream: Pointer): Longint;
 begin
  slGetSize:=PStream(Stream)^.GetSize;
 end;

function slRead(const Stream: Pointer; var Buf; Count: xWord): xInteger;
 begin
  PStream(Stream)^.Read(Buf, Count);

  slRead:=PStream(Stream)^.Status;
 end;

function slReset(const Stream: Pointer): xInteger;
 begin
  PStream(Stream)^.Reset;

  slReset:=PStream(Stream)^.Status;
 end;

function slSeek(const Stream: Pointer; const Pos: Longint): xInteger;
 begin
  PStream(Stream)^.Seek(Pos);

  slSeek:=PStream(Stream)^.Status;
 end;

function slTruncate(const Stream: Pointer): xInteger;
 begin
  PStream(Stream)^.Truncate;

  slTruncate:=PStream(Stream)^.Status;
 end;

function slWrite(const Stream: Pointer; var Buf; Count: xWord): xInteger;
 begin
  PStream(Stream)^.Write(Buf, Count);

  slWrite:=PStream(Stream)^.Status;
 end;

function slStatus(const Stream: Pointer): xInteger;
 begin
  slStatus:=PStream(Stream)^.Status;
 end;

end.
