unit TossCmn;

interface
uses
     Objects,

     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     TossAPI,
     PrioAPI,
     ccAPI,

     Wizard,

     jtMHLcmn;

const
 TosserName                                     = 'Tosser';
 TosserVersion                                  = 2;
 TosserPlugin                   : Pointer       = nil;

 tfNone                                         = $00000000;
 tfToss                                         = $00000001; { toss general inbounds }
 tfTossBadInbound                               = $00000002; { toss badinbound }
 tfTossBadmail                                  = $00000004; { toss badmail }

 tfTossingStarted                               = $10000000;

 TosserFlags                    : Longint       = tfNone;

 tcToss                                         = $00001000;
 tcFlush                                        = $00001001;
 tcRegisterCC                                   = $00001002;

 NetmailRecycler                : Pointer       = nil;
 BadRecycler                    : Pointer       = nil;
 BaseRecycler                   : Pointer       = nil;

 NetmailRecyclerSessions        : POpenedSessions       = nil;
 BadRecyclerSessions            : POpenedSessions       = nil;
 BaseRecyclerSessions           : POpenedSessions       = nil;

 SessionHosts                   : PStringCollection     = nil;

 TosserQueue                    : PCollection   = nil;

 TossMessageBusy                : Boolean       = False;

 restAlreadyInSeenbyReport      : Pointer       = nil;
 restProhibitNetmailToss        : Pointer       = nil;

 DefaultEchomailSessionHost     : String[22]    = 'Tosser.EchomailSession';

var
 Callbacks               : array[TTosserAction] of PCallbacks;

 cctaBadMessageToss      : PccHook;
 cctaTossBefore          : PccHook;
 cctaTossPacket          : PccHook;
 cctaTossAfterPreprocess : PccHook;

type
 PTossQueueItem = ^TTossQueueItem;
 TTossQueueItem = object(TObject)
  Reader: PFormatReader;
  Inbound: PInbound;
  DateTime: Longint;
  FName: PString;

  constructor Init(const AReader: PFormatReader; const AInbound: PInbound; const ADateTime: Longint;
   const AFName: String);
  destructor Done; virtual;
 end;

 PTossQueue = ^TTossQueue;
 TTossQueue = object(TSortedCollection)
  constructor Init;
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  function KeyOf(Item: Pointer): Pointer; virtual;
 end;

procedure Recycle(const oml: Pointer; const os: POpenedSessions; const msg: Pointer);
procedure RecycleLink(const oml, Link, msg: Pointer; const Host: String; const FromAddress, ToAddress: TAddress;
  const Password: String);

procedure TossingStatus(const Area: String);

procedure CheckTossingStarted;

implementation

{ TTossQueueItem }

constructor TTossQueueItem.Init(const AReader: PFormatReader; const AInbound: PInbound; const ADateTime: Longint;
  const AFName: String);
 begin
  inherited Init;

  Reader:=AReader;
  Inbound:=AInbound;
  DateTime:=ADateTime;
  FName:=NewStr(AFName);
 end;

destructor TTossQueueItem.Done;
 begin
  DisposeStr(FName);

  inherited Done;
 end;

{ TTossQueue }

constructor TTossQueue.Init;
 begin
  inherited Init($100, $10);

  Duplicates:=True;
 end;

function TTossQueue.Compare(Key1, Key2: Pointer): xInteger;
 begin
  Compare:=CompareLongint(Longint(Key1), Longint(Key2));

{
  if Longint(Key1) < Longint(Key2) then Compare:=-1 else
  if Longint(Key1) > Longint(Key2) then Compare:=1 else
   Compare:=0;
}
 end;

function TTossQueue.KeyOf(Item: Pointer): Pointer;
 begin
  KeyOf:=Pointer(PTossQueueItem(Item)^.DateTime);
 end;

{ Recycle }

procedure Recycle(const oml: Pointer; const os: POpenedSessions; const msg: Pointer);
 var
  FromAddress, ToAddress: TAddress;
  Password: String;
  st, oms: Pointer;
  s: POpenedSession;
 begin
  st:=jt.msg.st(msg);

  FromAddress:=PAddress(jt.st.Get(st, 'Packet.From'))^;
  ToAddress:=PAddress(jt.st.Get(st, 'Packet.To'))^;
  GetPStringEx(jt.st.Get(st, 'Packet.Password'), Password);

  s:=os^.Seek(FromAddress, ToAddress, Password);

  if s = nil then
   begin
    oms:=jt.om.s.Create(oml, FromAddress, ToAddress, Password);

    if oms = nil then
     begin
      jt.Report('Unable to create outbound mode session from ' + AddressToStrEx(FromAddress) + ' to ' +
                                                                 AddressToStrEx(ToAddress) + ' (' +
                                                                 Password + ')', True);

      Exit;
     end;

    s:=New(POpenedSession, Init(FromAddress, ToAddress, Password, oms));

    os^.Insert(s);
   end
  else
   oms:=s^.oms;

  if not jt.om.s.ExportMessage(oms, msg) then
   jt.Report('Unable to export message from ' + AddressToStrEx(FromAddress) + ' to ' +
                                                AddressToStrEx(ToAddress) + ' (' +
                                                Password + ')', True);
 end;

{ RecycleLink }

procedure RecycleLink(const oml, Link, msg: Pointer; const Host: String; const FromAddress, ToAddress: TAddress;
  const Password: String);
 var
  oms, st: Pointer;
 begin
  st:=jt.link.st(Link);

  oms:=jt.st.Get(st, Host);

  if oms = nil then
   begin
    oms:=jt.om.s.Create(oml, FromAddress, ToAddress, Password);

    if oms = nil then
     begin
      jt.Report('Unable to create outbound mode session from ' + AddressToStrEx(FromAddress) + ' to ' +
                                                                 AddressToStrEx(ToAddress) + ' (' +
                                                                 Password + ')', True);

      Exit;
     end;

    jt.st.Add(st, Host, oms, jt.svc.stoms);
   end;

  if not jt.om.s.ExportMessage(oms, msg) then
   jt.Report('Unable to export message from ' + AddressToStrEx(FromAddress) + ' to' +
                                                AddressToStrEx(ToAddress) + ' (' +
                                                Password + ')', True);
 end;

{ TossingStatus }

procedure TossingStatus(const Area: String);
 begin
  jt.SetTitle(Area + ' (tossing)');

  jt.SetStatus(Area);
 end;

{ CheckTossingStarted }

procedure CheckTossingStarted;
 begin
  if TosserFlags and tfTossingStarted = 0 then
   begin
    PerformPriorityAction('tossing');

    TosserFlags:=TosserFlags or tfTossingStarted;

    if ccAvailable then
     cctaTossBefore^.Execute;
   end;
 end;

end.