library Tosser;

{$IFDEF DEVELOPING}
 {$DEFINE NOKILLER}
{$ENDIF}

{
  Tosser behaviour:

  tossing BaseInbound           - always
          Inbounds, TempInbound - if TOSS specified
          BadInbound            - if TOSSBAD specified
          Badmail               - if TOSSBADMAIL specified
}

uses
     PlugSIDE,
     PlugAPI,
     PlugUTIL,

     Objects,
     Strings,

     Dos,

     jtMHLcmn,

     Wizard,

     TossCmn,
     TossNet,
     TossEcho,
     TossBad,
     TossTL,
     TossAPI,

     PrioAPI,
     ccAPI;

var
 ccHook: array[TTosserAction] of PccHook;

{ TossMessageIndirect }

procedure TossMessageIndirect(const msg: Pointer);
 begin
  if jt.msg.GetArea(msg) = nil then
   TossNetmail(msg)
  else
   TossEchomail(msg);

  jt.msg.Destroy(msg);
 end;

{ TossMessage }

procedure TossMessage(msg: Pointer); export;
 begin
  if TossMessageBusy then
   begin
    TosserQueue^.Insert(msg);

    Exit;
   end;

  TossMessageBusy:=True;

  TossMessageIndirect(msg);

  while TosserQueue^.Count > 0 do
   begin
    msg:=TosserQueue^.At(0);

    TosserQueue^.AtDelete(0);

    TossMessageIndirect(msg);
   end;

  TossMessageBusy:=False;
 end;

{ DistortFile }

function DistortFile(const FName, Extension: String; var TargetFName: String): Boolean;
 var
  rc: Longint;
 begin
  TargetFName:=ForceExtension(FName, Extension);

  if not DistortFileName(TargetFName, True) then
   begin
    TargetFName:=AddBackSlash(JustPathName(TargetFName)) + '00000000.000';

    if not DistortFileName(TargetFName, True) then
     begin
      jt.Report('Unable to distort fname ' + FName + ' (namespace exhausted?)', True);

      DistortFile:=False;

      Exit;
     end;
   end;

  jt.MoveFile(FName, TargetFName, rc);

  if rc <> 0 then
   begin
    jt.Report('Unable to move ' + FName + ' to ' + TargetFName + ', rc=' + Long2Str(rc), True);

    DistortFile:=False;
   end
  else
   DistortFile:=True;
 end;

{ Toss }

procedure Toss;
 var
  Inbounds: PInbounds;
  Queue: PTossQueue;
 function PreprocessInbound(const Inbound: PInbound): Boolean; far;
  var
   Preprocessor: PFormatPreprocessor;
   SR: SearchRec;
   K: Longint;
   S: String;
  begin
   if jt.Cancelled then
    begin
     PreprocessInbound:=True;

     Exit;
    end;

   PreprocessInbound:=False;

   if (Inbound^.Source <> tsInbound) or (TosserFlags and tfToss = 0) then
    Exit;

   FindFirst(Inbound^.Directory^ + '*.*', AnyFile, SR);

   while DosError = 0 do
    begin
     if SR.Attr and (Directory or VolumeID or Hidden) = 0 then
      for K:=0 to jt.fmt.pp.Count - 1 do
       begin
        Preprocessor:=jt.fmt.pp.At(K);

        if Preprocessor^.IsOur(Inbound^.Directory^ + SR.Name) then
         begin
          CheckTossingStarted;

          if Preprocessor^.Preprocess(Inbound^.Directory^ + SR.Name) then
           Break
          else
           if DistortFile(Inbound^.Directory^ + SR.Name, 'bad', S) then
            begin
             jt.Report('Don''t know how to preprocess file ' + Inbound^.Directory^ + SR.Name + ', renamed to ' + S, True);

             Break;
            end;
         end;
       end;

     FindNext(SR);
    end;

   FindClose(SR);
  end;
 function TossInbound(const Inbound: PInbound): Boolean; far;
  var
   Reader: PFormatReader;
   S, rc: String;
   SR: SearchRec;
   K, Flag: Longint;
  begin
   if jt.Cancelled then
    begin
     TossInbound:=True;

     Exit;
    end;

   TossInbound:=False;

   case Inbound^.Source of
    tsInbound      : if TosserFlags and tfToss = 0 then Exit;
    tsTempInbound  : if TosserFlags and tfToss = 0 then Exit;
    tsBadInbound   : if TosserFlags and tfTossBadInbound = 0 then Exit;
    tsBaseInbound  :;
    tsLocalInbound : if TosserFlags and tfToss = 0 then Exit;
   else
    Exit;
   end;

   FindFirst(Inbound^.Directory^ + '*.*', AnyFile, SR);

   while DosError = 0 do
    begin
     if SR.Attr and (Directory or VolumeID or Hidden) = 0 then
      begin
       for K:=0 to jt.fmt.r.Count - 1 do
        begin
         Reader:=jt.fmt.r.At(K);

         if Reader^.IsOur(Inbound^.Directory^ + SR.Name) then
          if Reader^.Ident(Inbound^.Directory^ + SR.Name) then
           begin
            Queue^.Insert(New(PTossQueueItem, Init(Reader, Inbound, SR.Time, SR.Name)));

            Break;
           end
          else
           if DistortFile(Inbound^.Directory^ + SR.Name, 'bad', S) then
            begin
             jt.Report('Don''t know how to toss file ' + Inbound^.Directory^ + SR.Name + ', renamed to ' + S, True);

             Break;
            end;
        end;
      end;

     FindNext(SR);
    end;

   FindClose(SR);
  end;
 function TossFile(const FormatReader: PFormatReader; const Inbound: PInbound; const FName: String; var rc: String): Boolean;
  var
   FromAddress, ToAddress: TAddress;
   Reader, msg, st: Pointer;
   Password: String;
   Flags: Longint;
  begin
   if ccAvailable then
    begin
     with cctaTossPacket^ do
      begin
       AddConstant('FNAME', etString)^.SetString(Inbound^.Directory^ + FName);
       AddConstant('INBOUND', etNumber)^.SetNumber(Inbound^.Source);
       AddVariable('PERFORM', etBoolean)^.SetBoolean(True);

       Execute;

       if not AddVariable('PERFORM', etBoolean)^.GetBoolean then
        begin
         TossFile:=True;

         Exit;
        end;
      end;
    end;

   TossFile:=False;

   Reader:=FormatReader^.Open(Inbound^.Directory^ + FName, rc, FromAddress, ToAddress, Password, Flags);

   if Reader = nil then
    begin
     rc:='unable to open: ' + rc;

     Exit;
    end;

   jt.LogWrite('<', FName + ': ' + AddressToStrEx(FromAddress) + '  ' +
                                   AddressToStrEx(ToAddress) + ', ' +
                                   Size2KB(FormatReader^.GetSize(reader)) + ' (' +
                                   GetTossSourceName(Inbound^.Source) + ')');

   repeat
    if not FormatReader^.Read(Reader, msg, rc) then
     begin
      rc:='unable to read: ' + rc;

      Break;
     end;

    if msg = nil then
     begin
      TossFile:=True;

      Break;
     end;

    st:=jt.msg.st(msg);

    jt.st.Add(st, 'Packet.From', @FromAddress, nil);
    jt.st.Add(st, 'Packet.To', @ToAddress, nil);
    jt.st.Add(st, 'Packet.Password', @Password, nil);
    jt.st.Add(st, 'Source', Pointer(Inbound^.Source), nil);

    jt.tossscan.TossMessage(msg);
   until False;

   FormatReader^.Close(Reader);

   {$IFNDEF NOKILLER}
   if not EraseFile(Inbound^.Directory^ + FName) then
    jt.Report('Unable to erase ' + Inbound^.Directory^ + FName, True);
   {$ENDIF}
  end;
 function PerformToss(const Item: PTossQueueItem): Boolean; far;
  var
   rc, S: String;
  begin
   CheckTossingStarted;

   if not TossFile(Item^.Reader, Item^.Inbound, Item^.FName^, rc) then
    if DistortFile(Item^.Inbound^.Directory^ + Item^.FName^, 'bad', S) then
     jt.Report('Unable to toss ' + Item^.Inbound^.Directory^ + Item^.FName^ + ' (renamed to ' + S + '): ' + rc, True)
    else
     jt.Report('Unable to toss ' + Item^.Inbound^.Directory^ + Item^.FName^ + ': ' + rc, True);

   PerformToss:=jt.Cancelled;
  end;
 begin
  jt.PushTitle;

  Inbounds:=New(PInbounds, Init);
  Queue:=New(PTossQueue, Init);

  Inbounds^.FirstThat(@PreprocessInbound);

  if ccAvailable then
   begin
    cctaTossAfterPreprocess^.AddVariable('TOSSINGSTARTED', etBoolean)^.SetBoolean(TosserFlags and tfTossingStarted <> 0);

    cctaTossAfterPreprocess^.Execute;
   end;

  if TosserFlags and tfTossBadmail <> 0 then
   TossBadmail;

  if not jt.Cancelled and (Inbounds^.FirstThat(@TossInbound) = nil) then
   Queue^.FirstThat(@PerformToss);

  Dispose(Queue, Done);
  Dispose(Inbounds, Done);

  jt.PopTitle;
 end;

{ Flush }

procedure Flush;
 var
  First: Boolean;
 procedure FlushSessions(const Sessions: POpenedSessions);
  begin
   if Sessions^.Count > 0 then
    begin
     if First then
      begin
       PerformPriorityAction('flushqueue');

       First:=False;
      end;

     Sessions^.FreeAll;
    end;
  end;
 procedure FlushLinkSessions(const Link: Pointer);
  var
   st: Pointer;
  procedure FlushLinkSession(const Host: PString); far;
   begin
    if jt.st.Get(st, Host^) <> nil then
     begin
      if First then
       begin
        PerformPriorityAction('flushqueue');

        First:=False;
       end;

      jt.st.Remove(st, Host^);
     end;
   end;
  begin
   st:=jt.link.st(Link);

   SessionHosts^.ForEach(@FlushLinkSession);
  end;
 var
  K: Longint;
 begin
  First:=True;

  FlushSessions(NetmailRecyclerSessions);
  FlushSessions(BadRecyclerSessions);
  FlushSessions(BaseRecyclerSessions);

  for K:=0 to jt.link.Count - 1 do
   FlushLinkSessions(jt.link.At(K));
 end;

{ RegisterCallback }

procedure RegisterCallback(const Block: TRegisterTosserCallbackBlock);
 begin
  Callbacks[Block.Action]^.Add(Block.Plugin);
 end;

{ RegisterSessionHost }

procedure RegisterSessionHost(const Host: PString);
 var
  Index: xInteger;
 begin
  if (Host <> nil) and not SessionHosts^.Search(Host, Index) then
   SessionHosts^.AtInsert(Index, NewStr(Host^));

  jt.DisposeStr(Host);
 end;

{ ValidateConfig }

procedure ValidateConfig(const Linker: Pointer);
 begin
  if NetmailRecycler = nil then
   jt.linker.Error(Linker, 'NetmailRecycler was not defined');

  if BadRecycler = nil then
   jt.linker.Error(Linker, 'BadRecycler was not defined');

  if BaseRecycler = nil then
   jt.linker.Error(Linker, 'BaseRecycler was not defined');
 end;

{ RegisterCC }

procedure RegisterCC;
 var
  Action: TTosserAction;
 begin
  for Action:=Low(Action) to High(Action) do
   if ccHook[Action]^.HooksCount > 0 then
    Callbacks[Action]^.Add(TosserPlugin);
 end;

{ PerformHook }

procedure PerformHook(const Action: TTosserAction; var Stuff);
 var
  AutoCreateBlock: TTosserAutoCreateBlock absolute Stuff;
  TossMessageBlock: TTossMessageBlock absolute Stuff;
  TossMessageToLinkBlock: TTossMessageToLinkBlock absolute Stuff;
  TossMessageToLinkDistinguishHostOMLBlock: TTossMessageToLinkDistinguishHostOMLBlock absolute Stuff;
  NetmailMessageTossBlock: TNetmailMessageTossBlock absolute Stuff;
  varDone, varArea, varLink, varmsg, varHost, varoml: PccVariable;
  Hook: PccHook;
 begin
  Hook:=ccHook[Action];

  case Action of
   taAutoCreateFirst,
   taAutoCreate,
   taAutoCreateLast,
   taAreaCreatedFirst,
   taAreaCreated,
   taAreaCreatedLast:
    begin
     varDone:=Hook^.AddVariable('DONE', etBoolean);
     varArea:=Hook^.AddVariable('AREA', etNumber);
     varLink:=Hook^.AddVariable('LINK', etNumber);
     varmsg:=Hook^.AddVariable('MSG', etNumber);

     varDone^.SetBoolean(AutoCreateBlock.Done);
     varArea^.SetNumber(Longint(AutoCreateBlock.Area));
     varLink^.SetNumber(Longint(AutoCreateBlock.Link));
     varmsg^.SetNumber(Longint(AutoCreateBlock.msg));

     Hook^.Execute;

     AutoCreateBlock.Done:=varDone^.GetBoolean;
     AutoCreateBlock.Area:=Pointer(varArea^.GetNumber);
     AutoCreateBlock.Link:=Pointer(varLink^.GetNumber);
     AutoCreateBlock.msg:=Pointer(varmsg^.GetNumber);
    end;
   taMessageTossFirst,
   taMessageToss,
   taMessageTossLast,
   taMessageTossBaseFirst,
   taMessageTossBase,
   taMessageTossBaseLast,
   taMessageTossGeneral,
   taMessageTossFilter:
    begin
     varArea:=Hook^.AddVariable('AREA', etNumber);
     varLink:=Hook^.AddVariable('LINK', etNumber);
     varmsg:=Hook^.AddVariable('MSG', etNumber);

     varArea^.SetNumber(Longint(TossMessageBlock.Area));
     varLink^.SetNumber(Longint(TossMessageBlock.Link));
     varmsg^.SetNumber(Longint(TossMessageBlock.msg));

     Hook^.Execute;

     TossMessageBlock.Area:=Pointer(varArea^.GetNumber);
     TossMessageBlock.Link:=Pointer(varLink^.GetNumber);
     TossMessageBlock.msg:=Pointer(varmsg^.GetNumber);
    end;
   taMessageTossToLink,
   taMessageTossToLinkNoClone:
    begin
     varLink:=Hook^.AddVariable('LINK', etNumber);
     varmsg:=Hook^.AddVariable('MSG', etNumber);

     varLink^.SetNumber(Longint(TossMessageToLinkBlock.Link));
     varmsg^.SetNumber(Longint(TossMessageToLinkBlock.msg));

     Hook^.Execute;

     TossMessageToLinkBlock.Link:=Pointer(varLink^.GetNumber);
     TossMessageToLinkBlock.msg:=Pointer(varmsg^.GetNumber);
    end;
   taMessageTossToLinkDistinguishHostOML:
    begin
     varLink:=Hook^.AddVariable('LINK', etNumber);
     varmsg:=Hook^.AddVariable('MSG', etNumber);
     varHost:=Hook^.AddVariable('HOST', etNumber);
     varoml:=Hook^.AddVariable('OML', etNumber);

     varLink^.SetNumber(Longint(TossMessageToLinkDistinguishHostOMLBlock.Link));
     varmsg^.SetNumber(Longint(TossMessageToLinkDistinguishHostOMLBlock.msg));
     varHost^.SetNumber(Longint(TossMessageToLinkDistinguishHostOMLBlock.Host));
     varoml^.SetNumber(Longint(TossMessageToLinkDistinguishHostOMLBlock.oml));

     Hook^.Execute;

     TossMessageToLinkDistinguishHostOMLBlock.Link:=Pointer(varLink^.GetNumber);
     TossMessageToLinkDistinguishHostOMLBlock.msg:=Pointer(varmsg^.GetNumber);
     TossMessageToLinkDistinguishHostOMLBlock.Host:=Pointer(varHost^.GetNumber);
     TossMessageToLinkDistinguishHostOMLBlock.oml:=Pointer(varoml^.GetNumber);
    end;
   taNetmailMessageTossFirst,
   taNetmailMessageToss,
   taNetmailMessageTossLast:
    begin
     varDone:=Hook^.AddVariable('DONE', etBoolean);
     varmsg:=Hook^.AddVariable('MSG', etNumber);

     varDone^.SetBoolean(NetmailMessageTossBlock.Done);
     varmsg^.SetNumber(Longint(NetmailMessageTossBlock.msg));

     Hook^.Execute;

     NetmailMessageTossBlock.Done:=varDone^.GetBoolean;
     NetmailMessageTossBlock.msg:=Pointer(varmsg^.GetNumber);
    end;
  end;
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 var
  Action: TTosserAction;
  Hook: PccHook;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, TosserName, TosserVersion);

  TosserPlugin:=Plugin;

  PluginSetup:=True;

  restAlreadyInSeenbyReport:=jt.rests.Add('AlreadyInSeenbyReport', rsMinus);
  restProhibitNetmailToss:=jt.rests.Add('ProhibitNetmailToss', rsMinus);

  RegisterPriorityAction('tossing');
  RegisterPriorityAction('flushqueue');

  InitCCAPI;

  if ccAvailable then
   begin
    cctaTossBefore:=New(PccHook, Init('TATOSSBEFORE'));

    cctaTossAfterPreprocess:=New(PccHook, Init('TATOSSAFTERPREPROCESS'));

    cctaTossPacket:=New(PccHook, Init('TATOSSPACKET'));

    with cctaTossPacket^ do
     begin
      AddConstant('FNAME', etString);
      AddConstant('INBOUND', etNumber);
      AddVariable('PERFORM', etBoolean);
     end;

    with cctaTossAfterPreprocess^ do
     AddVariable('TOSSINGSTARTED', etBoolean);

    cctaBadMessageToss:=New(PccHook, Init('TABADMESSAGETOSS'));

    with cctaBadMessageToss^ do
     AddVariable('MSG', etNumber);
   end;

  for Action:=Low(Action) to High(Action) do
   begin
    Callbacks[Action]:=New(PCallbacks, Init(StrPas(TosserActionNames[Action])));

    if ccAvailable then
     begin
      Hook:=New(PccHook, Init(StUpcase(StrPas(TosserActionNames[Action]))));

      case Action of
       taAutoCreateFirst,
       taAutoCreate,
       taAutoCreateLast,
       taAreaCreatedFirst,
       taAreaCreated,
       taAreaCreatedLast:
        begin
         Hook^.AddVariable('DONE', etBoolean);
         Hook^.AddVariable('AREA', etNumber);
         Hook^.AddVariable('LINK', etNumber);
         Hook^.AddVariable('MSG', etNumber);
        end;
       taMessageTossFirst,
       taMessageToss,
       taMessageTossLast,
       taMessageTossBaseFirst,
       taMessageTossBase,
       taMessageTossBaseLast,
       taMessageTossGeneral,
       taMessageTossFilter:
        begin
         Hook^.AddVariable('AREA', etNumber);
         Hook^.AddVariable('LINK', etNumber);
         Hook^.AddVariable('MSG', etNumber);
        end;
       taMessageTossToLink,
       taMessageTossToLinkNoClone:
        begin
         Hook^.AddVariable('LINK', etNumber);
         Hook^.AddVariable('MSG', etNumber);
        end;
       taMessageTossToLinkDistinguishHostOML:
        begin
         Hook^.AddVariable('LINK', etNumber);
         Hook^.AddVariable('MSG', etNumber);
         Hook^.AddVariable('HOST', etNumber);
         Hook^.AddVariable('OML', etNumber);
        end;
       taNetmailMessageTossFirst,
       taNetmailMessageToss,
       taNetmailMessageTossLast:
        begin
         Hook^.AddVariable('DONE', etBoolean);
         Hook^.AddVariable('MSG', etNumber);
        end;
      end;

      ccHook[Action]:=Hook;
     end;
   end;
 end;

{ PluginStartup }

procedure PluginStartup; export;
 begin
  if ccAvailable then
   jt.EnqueueCommand(TosserPlugin, tcRegisterCC, 0, cwAfterStartup);

  jt.EnqueueCommand(TosserPlugin, tcToss, 0, cwToss);
  jt.EnqueueCommand(TosserPlugin, tcFlush, 0, cwAfterToss);
  jt.EnqueueCommand(TosserPlugin, tcFlush, 0, cwFlush);

  jt.tossscan.SetTossMessage(TossMessage);
  jt.tossscan.SetTossMessageToLinks(TossToLinks);
  jt.tossscan.SetTossToBadmail(TossToBadmail);

  TosserQueue:=New(PCollection, Init($10, $10));

  NetmailRecyclerSessions:=New(POpenedSessions, Init($0, $10));
  BadRecyclerSessions:=New(POpenedSessions, Init($0, $10));
  BaseRecyclerSessions:=New(POpenedSessions, Init($0, $10));
  SessionHosts:=New(PStringCollection, Init($10, $10));

  SessionHosts^.Insert(NewStr(DefaultEchomailSessionHost));
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 var
  Action: TTosserAction;
 begin
  for Action:=Low(Action) to High(Action) do
   begin
    Dispose(Callbacks[Action], Done);

    if ccAvailable then
     Dispose(ccHook[Action], Done);
   end;

  if ccAvailable then
   begin
    Dispose(cctaBadMessageToss, Done);
    Dispose(cctaTossBefore, Done);
    Dispose(cctaTossPacket, Done);
    Dispose(cctaTossAfterPreprocess, Done);
   end;

  Dispose(SessionHosts, Done);
  Dispose(NetmailRecyclerSessions, Done);
  Dispose(BadRecyclerSessions, Done);
  Dispose(BaseRecyclerSessions, Done);

  jt.om.l.Destroy(NetmailRecycler);
  jt.om.l.Destroy(BadRecycler);
  jt.om.l.Destroy(BaseRecycler);

  Dispose(TosserQueue, Done);
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 begin
  if Command = tcToss then Toss else
  if Command = tcFlush then Flush else
  if Command = tcRegisterCC then RegisterCC else
  if Command = tcRegisterCallback then RegisterCallback(PRegisterTosserCallbackBlock(Param)^) else
  if Command = tcRegisterSessionHost then RegisterSessionHost(PString(Param)) else
  if Command = jtpcPluginSectionEnd then ValidateConfig(Pointer(Param)) else

  if Command = tcAutoCreateFirst then PerformHook(taAutoCreateFirst, Pointer(Param)^) else
  if Command = tcAutoCreate then PerformHook(taAutoCreate, Pointer(Param)^) else
  if Command = tcAutoCreateLast then PerformHook(taAutoCreateLast, Pointer(Param)^) else
  if Command = tcAreaCreatedFirst then PerformHook(taAreaCreatedFirst, Pointer(Param)^) else
  if Command = tcAreaCreated then PerformHook(taAreaCreated, Pointer(Param)^) else
  if Command = tcAreaCreatedLast then PerformHook(taAreaCreatedLast, Pointer(Param)^) else
  if Command = tcMessageTossFirst then PerformHook(taMessageTossFirst, Pointer(Param)^) else
  if Command = tcMessageToss then PerformHook(taMessageToss, Pointer(Param)^) else
  if Command = tcMessageTossLast then PerformHook(taMessageTossLast, Pointer(Param)^) else
  if Command = tcMessageTossBaseFirst then PerformHook(taMessageTossBaseFirst, Pointer(Param)^) else
  if Command = tcMessageTossBase then PerformHook(taMessageTossBase, Pointer(Param)^) else
  if Command = tcMessageTossBaseLast then PerformHook(taMessageTossBaseLast, Pointer(Param)^) else
  if Command = tcMessageTossToLink then PerformHook(taMessageTossToLink, Pointer(Param)^) else
  if Command = tcMessageTossToLinkDistinguishHostOML then PerformHook(taMessageTossToLinkDistinguishHostOML,
    Pointer(Param)^) else
  if Command = tcNetmailMessageTossFirst then PerformHook(taNetmailMessageTossFirst, Pointer(Param)^) else
  if Command = tcNetmailMessageToss then PerformHook(taNetmailMessageToss, Pointer(Param)^) else
  if Command = tcNetmailMessageTossLast then PerformHook(taNetmailMessageTossLast, Pointer(Param)^) else
  if Command = tcMessageTossGeneral then PerformHook(taMessageTossGeneral, Pointer(Param)^) else
  if Command = tcMessageTossFilter then PerformHook(taMessageTossFilter, Pointer(Param)^) else
  if Command = tcMessageTossToLinkNoClone then PerformHook(taMessageTossToLinkNoClone, Pointer(Param)^) else
 end;

{ PluginHandleCmdlineSwitch }

function PluginHandleCmdlineSwitch(Switch: String): Boolean; export;
 begin
  StUpcaseEx(Switch);

  PluginHandleCmdlineSwitch:=True;

  if Switch = 'TOSS' then TosserFlags:=TosserFlags or tfToss else
  if Switch = 'TOSSBAD' then TosserFlags:=TosserFlags or tfTossBadInbound else
  if Switch = 'TOSSBADMAIL' then TosserFlags:=TosserFlags or tfTossBadmail else
   PluginHandleCmdlineSwitch:=False;
 end;

{ PluginHandleConfigSwitch }

procedure PluginHandleConfigSwitch(const Keyword: String; Value: String; const Linker: Pointer); export;
 procedure ParseOML(var oml: Pointer);
  var
   rc: String;
  begin
   if oml <> nil then
    jt.om.l.Destroy(oml);

   oml:=jt.om.l.Create(jt.linker.ParseStringToken(Linker, Value), rc);

   if oml = nil then
    jt.linker.Error(Linker, rc);
  end;
 begin
  if Keyword = 'NETMAILRECYCLER' then ParseOML(NetmailRecycler) else
  if Keyword = 'BADRECYCLER' then ParseOML(BadRecycler) else
  if Keyword = 'BASERECYCLER' then ParseOML(BaseRecycler) else
   jt.linker.Error(Linker, 'syntax error');
 end;

{ PluginShowHelp }

procedure PluginShowHelp; export;
 begin
  jt.PrintStr(' TOSS             toss general inbounds'#13#10);
  jt.PrintStr(' TOSSBAD          toss badinbound'#13#10);
  jt.PrintStr(' TOSSBADMAIL      toss badmail'#13#10);
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleCmdlineSwitch,
 PluginHandleConfigSwitch,
 PluginShowHelp;

begin
 Assign(Output, '');
 Rewrite(Output);
end.
