library TossMap;

uses
     PlugAPI,
     PlugSIDE,
     TossAPI,

     Objects,

     Wizard;

const
 TossMapName                            = 'TossMap';
 TossMapVersion                         = 1;

 TossMapPlugin          : Pointer       = nil;

 TossMapUsed            : Longint       = 0;

type
 PMapItem = ^TMapItem;
 TMapItem = object(TObject)
  SourceName, TargetName: PString;

  constructor Init(const ASourceName, ATargetName: String);
  destructor Done; virtual;
 end;

 PMap = ^TMap;
 TMap = object(TCollection)
  function MapOnToss(var Name: String): Boolean;
  function MapOnExport(var Name: String): Boolean;
 end;

{ TMapItem }

constructor TMapItem.Init(const ASourceName, ATargetName: String);
 begin
  inherited Init;

  SourceName:=NewStr(ASourceName);
  TargetName:=NewStr(ATargetName);
 end;

destructor TMapItem.Done;
 begin
  DisposeStr(TargetName);
  DisposeStr(SourceName);

  inherited Done;
 end;

{ TMap }

function TMap.MapOnToss(var Name: String): Boolean;
 function LookupMapItem(const Item: PMapItem): Boolean; far;
  begin
   LookupMapItem:=Item^.SourceName^ = Name;
  end;
 var
  MapItem: PMapItem;
 begin
  MapItem:=FirstThat(@LookupMapItem);

  if MapItem <> nil then
   begin
    Name:=MapItem^.TargetName^;

    MapOnToss:=True;
   end
  else
   MapOnToss:=False;
 end;

function TMap.MapOnExport(var Name: String): Boolean;
 function LookupMapItem(const Item: PMapItem): Boolean; far;
  begin
   LookupMapItem:=Item^.TargetName^ = Name;
  end;
 var
  MapItem: PMapItem;
 begin
  MapItem:=FirstThat(@LookupMapItem);

  if MapItem <> nil then
   begin
    Name:=MapItem^.SourceName^;

    MapOnExport:=True;
   end
  else
   MapOnExport:=False;
 end;

{ stMap }

function stMap(const Action: TStorageAction; const Data: Pointer): Pointer; export;
 var
  Map: PMap absolute Data;
 begin
  case Action of
   saDestroy: Dispose(Map, Done);
   saClone: stMap:=nil; { clone isn't supported ;) }
   saInfo: stMap:=NewStr('area mapper data');
  end;
 end;

{ TossMessageGeneral }

procedure TossMessageGeneral(var Block: TTossMessageBlock);
 var
  Name: String;
  Map: PMap;
 begin
  if Block.Link <> nil then
   begin
    Map:=jt.st.Get(jt.link.st(Block.Link), 'Tosser.TossMap.Map');

    if Map <> nil then
     begin
      GetPStringEx(jt.msg.GetArea(Block.msg), Name);

      Map^.MapOnToss(Name);

      jt.msg.SetArea(Block.msg, Name);
     end;
   end;
 end;

{ TossMessageToLink }

procedure TossMessageToLink(var Block: TTossMessageToLinkBlock);
 var
  Name: String;
  Map: PMap;
 begin
  if Block.Link <> nil then
   begin
    Map:=jt.st.Get(jt.link.st(Block.Link), 'Tosser.TossMap.Map');

    if Map <> nil then
     begin
      GetPStringEx(jt.msg.GetArea(Block.msg), Name);

      Map^.MapOnExport(Name);

      jt.msg.SetArea(Block.msg, Name);
     end;
   end;
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, TossMapName, TossMapVersion);

  TossMapPlugin:=Plugin;

  PluginSetup:=True;
 end;

{ PluginStartup }

procedure PluginStartup; export;
 begin
  if TossMapUsed > 0 then
   begin
    RegisterTosserCallback(TossMapPlugin, taMessageTossGeneral);
    RegisterTosserCallback(TossMapPlugin, taMessageTossToLink);
   end;
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
  DisposeTosserCallbacks;
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 begin
  if Command = tcMessageTossGeneral then TossMessageGeneral(PTossMessageBlock(Param)^) else
  if Command = tcMessageTossToLink then TossMessageToLink(PTossMessageToLinkBlock(Param)^);
 end;

{ PluginHandleLinkSwitch }

function PluginHandleLinkSwitch(const Link: Pointer; const Keyword: String; Value: String; const Linker: Pointer): Boolean;
 export;
 var
  st: Pointer;
 procedure ParseMapArea;
  var
   SourceName, TargetName: String;
   Map: PMap;
  begin
   SourceName:=jt.linker.ParseStringToken(Linker, Value);
   TargetName:=jt.linker.ParseStringToken(Linker, Value);

   if SourceName = '' then
    jt.linker.Error(Linker, 'TOSSMAP defined incorrectly (sourcename must not be empty)');

   if TargetName = '' then
    jt.linker.Error(Linker, 'TOSSMAP defined incorrectly (targetname must not be empty)');

   Map:=jt.st.Get(st, 'Tosser.TossMap.Map');

   if Map = nil then
    begin
     Map:=New(PMap, Init($10, $10));

     jt.st.Add(st, 'Tosser.TossMap.Map', Map, stMap);
    end;

   Map^.Insert(New(PMapItem, Init(SourceName, TargetName)));

   Inc(TossMapUsed);
  end;
 begin
  PluginHandleLinkSwitch:=True;

  st:=jt.link.st(Link);

  if Keyword = 'TOSSMAP' then ParseMapArea else
   PluginHandleLinkSwitch:=False;
 end;

{ PluginExportLinkSwitches }

procedure PluginExportLinkSwitches(const Link, Linker: Pointer); export;
 var
  st: Pointer;
  Map: PMap;
 procedure ExportMapItem(const MapItem: PMapItem); far;
  begin
   jt.linker.ExportLine(Linker, ' TossMap ' + jt.linker.UnparseString(Linker, MapItem^.SourceName^) + ' ' +
                                              jt.linker.UnparseString(Linker, MapItem^.TargetName^));
  end;
 begin
  st:=jt.link.st(Link);

  Map:=jt.st.Get(st, 'Tosser.TossMap.Map');

  if Map <> nil then
   Map^.ForEach(@ExportMapItem);
 end;

{ PluginExportLinksHeader }

procedure PluginExportLinksHeader(const Linker: Pointer); export;
 begin
  jt.linker.ExportLine(Linker, '; TossMap switch:');
  jt.linker.ExportLine(Linker, ';   TossMap "external_name" "internal_name"');
  jt.linker.ExportLine(Linker, ';');
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleLinkSwitch,
 PluginExportLinkSwitches,
 PluginExportLinksHeader;

begin
end.