library TossPrio;

uses
     PlugAPI,
     PlugSIDE,
     TossAPI,

     Objects,

     Wizard;

const
 TossPrioName                           = 'TossPrio(tm)';
 TossPrioVersion                        = 1;

 TossPrioPlugin         : Pointer       = nil;

type
 PTossPrio = ^TTossPrio;
 TTossPrio = object(TObject)
  reSource, omlSource, Host: PString;
  re, oml: Pointer;

  constructor Init(const AreSource, AomlSource: String; const Linker: Pointer);
  destructor Done; virtual;
 end;

 PTossPrios = ^TTossPrios;
 TTossPrios = object(TCollection)
 end;

{ stTossPrios }

function stTossPrios(const Action: TStorageAction; const Data: Pointer): Pointer; export;
 var
  TossPrios: PTossPrios absolute Data;
 begin
  case Action of
   saDestroy: Dispose(TossPrios, Done);
   saClone: stTossPrios:=nil;
   saInfo: stTossPrios:=jt.NewStr('tossprios');
  end;
 end;

{ TTossPrio }

constructor TTossPrio.Init(const AreSource, AomlSource: String; const Linker: Pointer);
 var
  rc: String;
 begin
  inherited Init;

  jt.CompileRegExp(AreSource, re, Linker, False);

  oml:=jt.om.l.Create(AomlSource, rc);

  if oml = nil then
   jt.linker.Error(Linker, rc);

  reSource:=NewStr(AreSource);
  omlSource:=NewStr(AomlSource);
  Host:=NewStr('Tosser.TossPrioSessionHost.' + HexPtr(@Self));

  RegisterSessionHost(Host^);
 end;

destructor TTossPrio.Done;
 begin
  DisposeStr(Host);
  DisposeStr(reSource);
  DisposeStr(omlSource);

  jt.om.l.Destroy(oml);

  jt.DestroyRegExp(re);

  inherited Done;
 end;

{ Distinguish }

procedure Distinguish(var Block: TTossMessageToLinkDistinguishHostOMLBlock);
 var
  Area: String;
 function Lookup(const prio: PTossPrio): Boolean; far;
  begin
   Lookup:=jt.re.MatchStr(prio^.re, Area, False);
  end;
 var
  prios: PTossPrios;
  prio: PTossPrio;
 begin
  prios:=jt.st.Get(jt.link.st(Block.Link), 'TossPrios');

  if prios <> nil then
   begin
    GetPStringEx(jt.msg.GetArea(Block.msg), Area);

    prio:=prios^.FirstThat(@Lookup);

    if prio <> nil then
     begin
      Block.Host:=prio^.Host;
      Block.oml:=prio^.oml;
     end;
   end;
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, TossPrioName, TossPrioVersion);

  TossPrioPlugin:=Plugin;

  PluginSetup:=True;
 end;

{ PluginStartup }

procedure PluginStartup; export;
 begin
  RegisterTosserCallback(TossPrioPlugin, taMessageTossToLinkDistinguishHostOML);
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
  DisposeTosserCallbacks;
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 begin
  if Command = tcMessageTossToLinkDistinguishHostOML then Distinguish(PTossMessageToLinkDistinguishHostOMLBlock(Param)^);
 end;

{ PluginHandleLinkSwitch }

function PluginHandleLinkSwitch(const Link: Pointer; const Keyword: String; Value: String; const Linker: Pointer): Boolean;
 export;
 var
  st: Pointer;
  prios: PTossPrios;
 begin
  if Keyword = 'TOSSPRIO' then
   begin
    PluginHandleLinkSwitch:=True;

    st:=jt.link.st(Link);

    prios:=jt.st.Get(st, 'TossPrios');

    if prios = nil then
     begin
      prios:=New(PTossPrios, Init($4, $10));

      jt.st.Add(st, 'TossPrios', prios, stTossPrios);
     end;

    prios^.Insert(New(PTossPrio, Init(jt.linker.ParseStringToken(Linker, Value),
     jt.linker.ParseStringToken(Linker, Value), Linker)));
   end
  else
   PluginHandleLinkSwitch:=False;
 end;

{ PluginExportLinkSwitches }

procedure PluginExportLinkSwitches(const Link, Linker: Pointer); export;
 procedure ExportPrio(const Prio: PTossPrio); far;
  begin
   jt.linker.ExportLine(Linker, ' TossPrio ' + jt.linker.UnparseString(Linker, GetPString(Prio^.reSource)) + ' ' +
                                               jt.linker.UnparseString(Linker, GetPString(Prio^.omlSource)));
  end;
 var
  st: Pointer;
  prios: PTossPrios;
 begin
  st:=jt.link.st(Link);

  prios:=jt.st.Get(st, 'TossPrios');

  if prios <> nil then
   prios^.ForEach(@ExportPrio);
 end;

{ PluginExportLinksHeader }

procedure PluginExportLinksHeader(const Linker: Pointer); export;
 begin
  jt.linker.ExportLine(Linker, '; TossPrio(tm) switch:');
  jt.linker.ExportLine(Linker, ';   TossPrio "area_regexp" "oml"');
  jt.linker.ExportLine(Linker, ';');
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleLinkSwitch,
 PluginExportLinkSwitches,
 PluginExportLinksHeader;

begin
end.