library WatchDog;

uses
     Objects,

     PlugAPI,
     PlugSIDE,
     PlugUTIL,

     afxAPI,
     rlopAPI,
     jtmPerf,

     Wizard;

type
 PWatchDogQueue = ^TWatchDogQueue;
 TWatchDogQueue = object(TSortedCollection)
  function Compare(Key1, Key2: Pointer): xInteger; virtual;
  procedure FreeItem(Item: Pointer); virtual;
 end;

function TWatchDogQueue.Compare(Key1, Key2: Pointer): xInteger;
 begin
  Compare:=CompareLongint(Longint(Key1), Longint(Key2));
 end;

procedure TWatchDogQueue.FreeItem(Item: Pointer);
 begin
 end;

const
 WatchDogPlugin                         : Pointer       = nil;
 WatchDogCheck                                          = $1000;

 WatchDogQueue                  : PWatchDogQueue        = nil;
 WatchDogTrust                  : Pointer               = nil;
 WatchDogCheckAllAreas          : Boolean               = False;

{ CheckArea }

function CheckArea(const Area: Pointer; var Links: PAreaLinks): Boolean;
 begin
  jt.SetStatus('WatchDog: checking ' + jt.area.GetName(Area)^);

  if jt.area.Passthrough(Area) and jt.rests.Check(WatchDogTrust, nil, jt.area.GetName(Area)) then
   begin
    Links:=New(PAreaLinks, Init(Area, False));

    if Links^.Count <= 1 then
     CheckArea:=True
    else
     begin
      Dispose(Links, Done);

      CheckArea:=False;
     end;
   end
  else
   CheckArea:=False;
 end;

{ KillArea }

procedure KillArea(const Area: Pointer; var Links: PAreaLinks);
 var
  Name, rc: String;
 procedure Unsubscribe(const Link: PAreaLink); far;
  begin
   if rlopIsLinkValid(Link^.Link, rc) then
    begin
     rlopSendAreaUnsubscribe(Link^.Link, Name);

     jt.LogWrite('W', Name + ': sending unsubscribe to ' + AddressOfLink(Link^.Link));
    end;
  end;
 begin
  Name:=jt.area.GetName(Area)^;

  jt.SetStatus('WatchDog: killing ' + Name);

  if jt.area.Kill(Area, rc) then
   begin
    Links^.ForEach(@Unsubscribe);

    jt.LogWrite('W', Name + ': killed');

    jt.area.SetChanged(acfFull, 'watchdog/killed');
    jt.link.SetChanged(lcfFull, 'watchdog/killed');
   end
  else
   jt.LogWrite('?', Name + ': failed to kill - ' + rc);

  Dispose(Links, Done);
 end;

{ Unsubscribed }

procedure Unsubscribed(var Block: TAreafixUnsubscribeBlock);
 var
  Index: xInteger;
 begin
  if not WatchDogQueue^.Search(Block.Area, Index) then
   WatchDogQueue^.AtInsert(Index, Block.Area);
 end;

{ Check }

procedure Check;
 procedure Perform(const Area: Pointer);
  var
   Links: PAreaLinks;
  begin
   if CheckArea(Area, Links) then
    KillArea(Area, Links);
  end;
 var
  Index: xInteger;
  K: Longint;
 begin
  if WatchDogCheckAllAreas then
   begin
    jt.LogWrite('W', 'Checking passthrough areas');

    for K:=0 to jt.area.Count - 1 do
     if not WatchDogQueue^.Search(jt.area.At(K), Index) then
      WatchDogQueue^.AtInsert(Index, jt.area.At(K));
   end;

  WatchDogQueue^.ForEach(@Perform);

  jt.ClearStatus;
 end;

{ PluginSetup }

function PluginSetup(const JTAPI: Pointer; const PlugInfo: PPluginInformation; const Plugin: Pointer): Boolean; export;
 begin
  SetupAPI(JTAPI);

  SetupPlugInfo(PlugInfo, 'WatchDog', 1);

  WatchDogPlugin:=Plugin;

  WatchDogTrust:=jt.rests.Add('WatchDogTrust', rsMinus);

  PluginSetup:=True;
 end;

{ PluginStartup }

procedure PluginStartup; export;
 begin
  RegisterAreafixCallback(WatchDogPlugin, aaUnsubscribeUnsubscribed);

  jt.EnqueueCommand(WatchDogPlugin, WatchDogCheck, 0, cwLatest);

  WatchDogQueue:=New(PWatchDogQueue, Init(jt.area.Count or $100, $100));
 end;

{ PluginShutdown }

procedure PluginShutdown; export;
 begin
  Dispose(WatchDogQueue, Done);

  DisposeAreafixCallbacks;
 end;

{ PluginHandleCommand }

procedure PluginHandleCommand(const Command, Param: Longint); export;
 begin
  if Command = acUnsubscribeUnsubscribed then Unsubscribed(PAreafixUnsubscribeBlock(Param)^) else
  if Command = WatchDogCheck then Check;
 end;

{ PluginHandleConfigSwitch }

procedure PluginHandleConfigSwitch(const Keyword: String; Value: String; const Linker: Pointer); export;
 begin
  jt.linker.Error(Linker, 'syntax error');
 end;

{ PluginHandleCmdlineSwitch }

function PluginHandleCmdlineSwitch(Switch: String): Boolean; export;
 begin
  StUpcaseEx(Switch);

  PluginHandleCmdlineSwitch:=True;

  if Switch = 'CHECKPTAREAS' then WatchDogCheckAllAreas:=True else
   PluginHandleCmdlineSwitch:=False;
 end;

{ PluginShowHelp }

procedure PluginShowHelp; export;
 begin
  jt.PrintStr(' CHECKPTAREAS            check all PT areas and (maybe) kill''em'#13#10);
 end;

exports
 PluginSetup,
 PluginStartup,
 PluginShutdown,
 PluginHandleCommand,
 PluginHandleConfigSwitch,
 PluginHandleCmdlineSwitch,
 PluginShowHelp;

begin
 Assign(Output, '');
 Rewrite(Output);
end.